/*
 *   $RCSfile: BF_FlushChunk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "list.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "lgobject.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "volume.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"


/*============================================================================*/
/* 
 * BF_FlushChunk:
 * 	Routine to flush a dirty chunk.
 *
 * IMPORTS:
 *	io_writepage()
 *	BF_SetFlock()
 *	BF_ReleaseFlock()
 *	BF_RemoveChunkDependents()
 *
 * RETURNS:
 *	esmNOERROR.
 *
 * ERROR RETURNS:
 *	none.
 *
 * SIDE EFFECTS:
 *	The chunk is flushed.
 */
/*============================================================================*/


 void
BF_FlushChunk (

	register CHUNKDESC	*chunkDesc,
	BOOL				force 
)
{

    register int		offset;		/* offset at which we're flushing	*/
    register CHUNKPAGE  *chunkPage;	/* pointer to page descriptor		*/
	register CHUNKPAGE	*endPage;	/* last page marker					*/
	register int		pageCount;	/* number of pages to look at		*/


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	check the magic number
	 */
	CHECK_CHUNKDESC_MAGIC(chunkDesc);

	/*
	 *	Setup initial loop conditions
	 */
    offset = 0;
    chunkPage = chunkDesc->chunkPage;
	pageCount = chunkDesc->pageCount;

	/*
	 *	Check all component pages to see if they need to be flushed
	 */
    for (endPage = &(chunkPage[pageCount]); chunkPage < endPage; chunkPage++) {

		TRPRINT(TR_BF, TR_LEVEL_2, ("processing page:%d", chunkPage->pid.page));

		/*
		 *	check the chunkpage magic
		 */
		CHECK_PAGEHASH_MAGIC(chunkPage->pageHash);

		/*
		 *	check to see if the page is dirty
		 */
		if (PAGE_IS_DIRTY(chunkPage->pageHash))	{

			/*
		 	 *	Only flush dirty pages that are not in other chunks
			 */
			if (force || (chunkPage->pageHash->memberCount == 1))	{

				TRPRINT(TR_BF, TR_LEVEL_2, ("flushing page:%d from offset:%d location:%x",
						chunkPage->pid.page, offset, chunkDesc->bufFrame + offset));

				/*
				 *	Flush the page to disk
				 *  This is like bf_ForcePage() except that it
				 *  uses an offset from the bufFrame, and uses the
				 *  bufFrame from chunkDesc, not from the pageHash.
				 */
				if (io_WritePage( &(chunkPage->pageHash->hashList.hashPid), 
						  chunkDesc->bufFrame + offset,
						  chunkPage->pageHash))	{
					
					SM_ERROR(TYPE_FATAL, esmINTERNAL);
				}

				/*
				 *	mark the page as clean
				 */
				chunkPage->flags &= ~PAGE_DIRTY;
				CHECK_PAGEHASH_MAGIC(chunkPage->pageHash);
				bf_CleanPage(chunkPage->pageHash);
			}
		}

		/*
		 *	Increment offset of the next page
		 */
		offset += chunkPage->size;
    }

    /*
     *	Mark the chunk as clean
     */
   	CLEAN_CHUNK(chunkDesc);
}
