/*
 *   $RCSfile: BF_FreeChunk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


 void
BF_FreeChunk (

	BUFGROUP			*bufGroup,
	register CHUNKDESC	*chunkDesc 
)
{

	register int		size;


	TRPRINT(TR_BF, TR_LEVEL_1, ("index:%d count:%d chunkDesc:%x",
					chunkDesc->bufIndex, chunkDesc->bufCount, chunkDesc));

	CHECK_CHUNKDESC_MAGIC(chunkDesc);

	/*
	 *	check to see that the specified group matches the
	 *	group contained in the descriptor
	 */
	SM_ASSERT(LEVEL_3, chunkDesc->bufGroup == bufGroup);

	/*
	 *	Get the size of the chunk in buffers
	 */
	size = chunkDesc->bufCount;

	/*
	 *	decrement the charges for the group
	 */
	bufGroup->unfixedCharges -= size;
	TRPRINT(TR_BF, TR_LEVEL_2, ("fixed:%d unfixed:%d",
			bufGroup->fixedCharges, bufGroup->unfixedCharges));

	/*
	 *	check to see that the unfixed is reasonable
	 */
	SM_ASSERT(LEVEL_3, bufGroup->unfixedCharges >= 0);

	/*
	 *	put the chunk on the free list
	 */
	BF_FreeBlock(chunkDesc);

	/*
	 *	mark the chunk descriptor as free
	 */
	chunkDesc->flags &= ~CHUNK_USED;

	BF_AuditGroup(bufGroup);
}
