/*
 *   $RCSfile: BF_FreePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "load.h"
#include "scan.h"
#include "sm_params.h"
#include "btree.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"
#ifdef notdef
#include "sm_extfuncs.h"
#endif notdef


 void
BF_FreePage (

	BUFGROUP	*bufGroup,
	GROUPLINK	*groupLink,
	BOOL		discardPage
)
{

	register PAGEHASH	*pageHash;


	TRPRINT(TR_BF, TR_LEVEL_1, ("freeing page:%d",
			groupLink->pageHash->hashList.hashPid.page));

	/*
	 *	check magic numbers
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);
	CHECK_BUFGROUP_MAGIC(bufGroup);

	/*
	 *	Pull out the hash pointer
	 */
	pageHash = groupLink->pageHash;
	CHECK_PAGEHASH_MAGIC(pageHash);
	    
	/*
	 * Update the group's unfixed size and
	 * reset the group links.
	 */
	bufGroup->unfixedCharges -= BLOCKCOUNT(pageHash->page2size);

	/*
	 *	check to see that the fixed charges are reasonable
	 */
	SM_ASSERT(LEVEL_3, bufGroup->unfixedCharges >= 0);

    /*
     *  check to see if the page is in a force group
	 *	then write the page before placing it on the free list 
     */
    if ( (bufGroup->flags & FORCE_GROUP) &&
         PAGE_IS_DIRTY(pageHash) && !discardPage)  {

        TRPRINT(TR_BF, TR_LEVEL_2, ("force buffer"));

		bf_ForcePage(pageHash, NOFLAGS);
    }
	
	/*
	 *	Check to see if this is the only group that has the page
	 *	as a member
	 */
	if (pageHash->memberCount == 1)	{

		TRPRINT(TR_BF, TR_LEVEL_3, ("putting on dirty list"));

		/*
		 *	If the page is a log page, then we can reuse it
		 *	immediately.  So, put it on the head of the queue
		 *	Otherwise, put it on the end of the LRU queue
		 */
		if (bufGroup->flags & LOG_GROUP) {
			listPush( &(FreeBufTable[OFFSET(groupLink->pageHash->page2size)]), 
				&(groupLink->groupList) );
		} else {
			listEnq( &(FreeBufTable[OFFSET(groupLink->pageHash->page2size)]), 
				&(groupLink->groupList) );
		}

		/*
		 *	set the buffer group pointer to null
		 */
		groupLink->bufGroup = NULL;

		/*
		 *	Set the group count to zero
		 */
		pageHash->memberCount = 0;
		TRPRINT(TR_BF, TR_LEVEL_3, ("freeing index:%d size:%d",
						groupLink->pageHash->pageInfo->bufIndex,
						groupLink->pageHash->pageInfo->blockCount));

	} else {

		TRPRINT(TR_BF, TR_LEVEL_3, ("member of %d other groups", pageHash->memberCount));

		/*
		 *	put the link on the free pool
		 */
		listMoveEnq( &(GroupLinkPool), &(groupLink->hashList) );

		/*
		 *	Decrement the number of groups it is in
		 */
		pageHash->memberCount--;
	}

#ifdef DEBUG
#ifdef notdef
	/* TODO: PUT THIS BACK IN */
	{
	/* 
	 * Page *could* be hosed from backing out of an error
	 * in a what-should-be-atomic operation, and the operation
	 * caused the tx to be aborted.  We need to allow the abort
	 * to discard all the pages without checking them
	 */
#include "trans.h"
#include "trans_globals.h"

		if( (TransRec.clientTransState != T_ABORT)  && 
			(pageHash->flags & H_SLOTTED))	{
			sm_AuditSlotPage((SLOTTEDPAGE *) pageHash->pageInfo->bufFrame);
		}
	}

#endif
#endif notdef
}
