/*
 *   $RCSfile: BF_GetLarger.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 char 
*BF_GetLarger (

	PAGE2SIZE			block2size,		/* pow-of-2 sizeof chunk */
	int					needType 
)
{

	register GROUPLINK	*groupLink = NULL;
	register GROUPLINK	*newLink;
	register PAGEHASH	*pageHash;
	register PAGEINFO	*pageInfo;
	register BUFINFO	*bufInfo;
	register CHUNKDESC	*chunkDesc;
	register int		size;
	register PAGE2SIZE	bsize;
	register int		start;


	TRPRINT(TR_BF, TR_LEVEL_1, ("looking for block2size:%d", block2size));

	SM_ASSERT(LEVEL_3, block2size >= MIN_PAGE2SIZE);

	/*
	 *	loop through the larger size queues
	 *	should convert this to pointer operations
	 */
	for (size = block2size + 1; size <= MaxPage2Size; size++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("checking size:%d", size));

		/*
		 *	attempt to get a buffer off the list
		 */
		if (groupLink = (GROUPLINK *) listDeq( &FreeBufTable[OFFSET(size)] ))	{

			TRPRINT(TR_BF, TR_LEVEL_2, ("got buffer of size:%d", size));
			break;
		}
	}

	/*
	 *	check to see if we got a buffer
	 */
	if (groupLink == NULL)	{

		/*
		 *	we didn't find a larger buffer
		 */
		return(NULL);
	}

	/*
	 *	check to see what type of buffer was gotten
	 */
	if (groupLink->type == TYPE_CHUNK)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("got chunk buffer"));

		/*
		 *	get a chunk pointer
		 */
		chunkDesc = (CHUNKDESC *) groupLink;
		CHECK_CHUNKDESC_MAGIC(chunkDesc);

		/*
		 *	invalidate the chunk
		 */
		if (CHUNK_IS_VALID(chunkDesc))	{

			BF_InvalidateChunk(chunkDesc);
		}

		/*
		 *	get the starting address to start splitting the buffers
		 */
		start = chunkDesc->bufIndex + BLOCKCOUNT(block2size);
		TRPRINT(TR_BF, TR_LEVEL_2, ("start:%d", start));

		/*
		 *	check to see what type of block is needed
		 */
		if (needType == TYPE_PAGE)	{

			TRPRINT(TR_BF, TR_LEVEL_2, ("need page buffer"));

			/*
			 *	get a set of page structures
			 */
			if ((groupLink = (GROUPLINK *) listDeq( &BufStructPool )) == NULL)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			/*
			 *	initialize the page info
			 */
			pageInfo = groupLink->pageHash->pageInfo;
			pageInfo->bufFrame = groupLink->bufFrame = chunkDesc->bufFrame;
			pageInfo->bufIndex = chunkDesc->bufIndex;
			pageInfo->blockCount = chunkDesc->bufCount;
			bufInfo = pageInfo->bufInfo = chunkDesc->bufInfo;

			/*
			 *	initialize the buf info structure
			 */
			BF_SetSlot(bufInfo, BLOCKCOUNT(block2size),
					   (char *) groupLink->pageHash, TYPE_PAGE);

			/*
			 *	hang the chunk descriptor on the free list
			 */
			SM_ASSERT(LEVEL_1, (chunkDesc->chunkPage == NULL));
			listEnq( &ChunkDescPool, &(chunkDesc->memberList) );
		}

	} else if (groupLink->type == TYPE_PAGE) {

		TRPRINT(TR_BF, TR_LEVEL_2, ("got page buffer"));

		/*
		 *	check the groupLink magic
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);

		/*
		 *	get the starting address to start splitting the buffers
		 */
		start = groupLink->pageHash->pageInfo->bufIndex + BLOCKCOUNT(block2size);
		TRPRINT(TR_BF, TR_LEVEL_2, ("start:%d", start));

		/*
		 *	Invalidate the page; it is on the free list and
		 *  may still be valid as well.  Remove it from the hash table.
		 *  (remove=TRUE); force it to the server if it is dirty
		 * 	(force==TRUE).
		 */
		bf_InvalidatePage(groupLink->pageHash, TRUE, TRUE);

		/*
		 *	check to see if we need a chunk buffer
		 */
		if (needType == TYPE_CHUNK)	{

			TRPRINT(TR_BF, TR_LEVEL_2, ("need chunk buffer"));

			/*
			 *	get a set of page structures
			 */
			if ((chunkDesc = (CHUNKDESC *) listDeq( &ChunkDescPool )) == NULL){
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}
			SM_ASSERT(LEVEL_1, (chunkDesc->chunkPage == NULL));

			/*
			 *	initialize the page info
			 */
			pageInfo			= groupLink->pageHash->pageInfo;
			chunkDesc->bufFrame = pageInfo->bufFrame;
			chunkDesc->bufIndex = pageInfo->bufIndex;
			chunkDesc->buf2size = block2size;
			bufInfo = chunkDesc->bufInfo = pageInfo->bufInfo;

			/*
			 *	initialize the buf info structure
			 */
			BF_SetSlot(bufInfo, BLOCKCOUNT(block2size), (char *) chunkDesc, TYPE_CHUNK);

			/*
			 *	hang the chunk descriptor on the free list
			 */
			listEnq( &BufStructPool, &(groupLink->groupList) );
		}

	} else {

		/*
		 *	this cannot happen
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	create buddy size pages at this address
	 */
	for (bsize = block2size; bsize < size; start += BLOCKCOUNT(bsize), bsize++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("creating block size:%d at:%d", bsize, start));

		/*
		 *	Get a group link structure
		 */
		if ((newLink = (GROUPLINK *) listDeq( &BufStructPool )) == NULL)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *	set the type to page
		 */
		pageHash = newLink->pageHash;
		CHECK_PAGEHASH_MAGIC(pageHash);
		pageHash->page2size = bsize;
		TRPRINT(TR_BF, TR_LEVEL_3, ("new page size:%d", pageHash->page2size));

		/*
		 *	Initialize the pointer to the actual buffer
		 */
		pageInfo = newLink->pageHash->pageInfo;
		pageInfo->bufFrame = &(BufferPool[start << MIN_PAGE2SIZE]);
		pageInfo->bufIndex = start;
		bufInfo = pageInfo->bufInfo = &(BufInfoTable[start]);
		pageInfo->blockCount = BLOCKCOUNT(pageHash->page2size);

		/*
		 *	Set the group link pointer to the buffer
		 */
		newLink->bufFrame = pageInfo->bufFrame;

		/*
		 *	put the buffer on the free list
		 */
		listEnq( &(FreeBufTable[OFFSET(bsize)]), &(newLink->groupList) );

		/*
		 *	clear the slot table
		 */
		BF_SetSlot(bufInfo, BLOCKCOUNT(pageHash->page2size), (char *) pageHash, TYPE_PAGE);
	}

	/*
	 *	check to see what type is gotten
	 */
	if (needType == TYPE_PAGE)	{

		/*
		 *	set the size of the page
		 */
		groupLink->pageHash->page2size = block2size;

		/*
		 *	return the found buffer
		 */
		return((char *) groupLink);

	} else {

		/*
		 *	set the size of the page
		 */
		chunkDesc->buf2size = block2size;
		chunkDesc->bufCount = BLOCKCOUNT(block2size);

		BF_AuditChunk(chunkDesc);
		/*
		 *	return the found buffer
		 */
		return((char *) chunkDesc);
	}
}
