/*
 *   $RCSfile: BF_InvalidateChunk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "ess.h"
#include "sysdefs.h"
#include "checking.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"

 void
_BF_InvalidateChunk (
	register CHUNKDESC	*chunkDesc,
	BOOL				discard /* true from abort, false from commit */
)
{

	register CHUNKPAGE	*chunkPage;
	register CHUNKPAGE	*endPage;
	register PAGEHASH	*pageHash;


	TRPRINT(TR_BF, TR_LEVEL_1, ("index:%d size:%d",
			chunkDesc->bufIndex, chunkDesc->bufCount));

	CHECK_CHUNKDESC_MAGIC(chunkDesc);

	SM_ASSERT(LEVEL_3, !(chunkDesc->flags & CHUNK_FIXED));

	if(!discard) {
		/* called from commit/prepare */
		BF_FlushChunk(chunkDesc, FALSE);
	}

	/*
	 *	setup end conditions of loop
	 */
	chunkPage = chunkDesc->chunkPage;
	endPage = &(chunkPage[chunkDesc->pageCount]);

	/*
	 *	process all pages in the chunk
	 */
	while (chunkPage < endPage)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("processing page:%d", chunkPage->pid.page));

		/*
		 *	remove the chunk from the list of chunks to which a page belongs
		 *  (The list this was on was some pageHash->memberList.)
		 */
		if(chunkPage -> flags & PAGE_VALID) {
			SM_ASSERT(LEVEL_1, LIST_MEMBER( &(chunkPage->hashList)));
			listRemove( &(chunkPage->hashList) );
			pageHash = chunkPage->pageHash;

			CHECK_PAGEHASH_MAGIC(pageHash);
			if(discard) {
				/* called from abort */
				bf_CleanPage(pageHash);
			}

			/*
			 *	check to see if the page is in any other chunk
			 */
			if (--(pageHash->memberCount) == 0)	{

				TRPRINT(TR_BF, TR_LEVEL_2, ("last reference to page"));
				SM_ASSERT(LEVEL_3, LIST_EMPTY( &(pageHash->memberList) ));
				SM_ASSERT(LEVEL_3, PAGE_IS_CLEAN(pageHash));
				BF_DiscardPageHash(pageHash);
			}
		}

		/*
		 *	Look at the next entry
		 */
		chunkPage++;
	}

	/*
	 *	mark the chunk as invalid
	 */
	chunkDesc->flags = CHUNK_INVALID;

	SM_FREE_PTR(chunkDesc->chunkPage);
	chunkDesc->pageCount = 0; /* since we just freed the chunkPages */
	BF_AuditChunk(chunkDesc);
	chunkDesc->chunkPage = NULL;
}

 void
BF_InvalidateChunk (
	register CHUNKDESC	*chunkDesc
)
{
	_BF_InvalidateChunk(chunkDesc, FALSE);
}

 void
BF_DiscardChunk(
	register CHUNKDESC	*chunkDesc
)
{
	_BF_InvalidateChunk(chunkDesc, TRUE);
}
