#ifndef __BF_PageHash_C__
#define __BF_PageHash_C__
/*
 *   $RCSfile: BF_PageHash.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

/*
 *****************************************************************
 *       c-callable wrapper funcs for page hash table and pool
 *****************************************************************
 */


#include "HashTable.h"
#include "Pool.h"
#include "bf.h"
#include "PAGEHASH.h" /* to be sure the funcs get defined */
#ifndef SERVER_MAKE
#include "serverinfo.h"
#include "msg_funcs.h"
#include "chunk.h"
#include "sm_globals.h"
#else SERVER_MAKE
#include "pool.h"
#include "thread.h"
#include "thread_globals.h"
#include "io_globals.h"
#endif SERVER_MAKE
#include "error.h"
#include "bf_extfuncs.h"
#include "bf_intfuncs.h"
#include "bf_globals.h"

#ifdef SERVER_MAKE

BOOL 
LRCCACHEENTRY:: Dump(FILE *f) {
	fprintf(f, 
		"LRCCACHEENTRY flags 0x%x\n", flags);
	fprintf(f, "\t lrc %d.%d fid %d.%d.%d, pid %d.%d\n",
		lrc.wrapCount, lrc.count,
		fid.pid.page, fid.pid.volid, fid.unique,
		hashList.hashPid.page, hashList.hashPid.volid
		);
	return FALSE; /* all funcs used in ForEach must be BOOL-valued */
}; 

PID *
LRCCACHEENTRY:: keylocation (int unique, PID **unused) {
	switch(unique) {
	case LRCCACHE:
		return (PID *)&(hashList.hashPid);
	default:
		TRPRINT(TR_BF, TR_LEVEL_1, ("bad unique value: %d\n", unique));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		/* to prevent the compiler from complaining 
		about no return value: */
		return (PID *)&(hashList.hashPid);
	}
};

LISTELEMENT *
LRCCACHEENTRY:: listlocation (int unique) { 
	switch(unique) {
	case LRCCACHE:
	case LRCCACHE_POOL:
		return &(hashList.hashList); 
	default:
		TRPRINT(TR_BF, TR_LEVEL_1, ("bad unique value: %d\n", unique));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		/* to prevent the compiler from 
			complaining about no return value: */
		return &(hashList.hashList); 
	}
};

void 
LRCCACHEENTRY:: ReInit() {
	this->flags = H_FREE;
	this->lrc = LrcNone;
	this->hashList.hashPid = PidNone;
	this->fid.pid = PidNone;
	this->fid.unique = 0;
};

void 
LRCCACHEENTRY:: Init() {
	initializeListElement( this->listlocation(LRCCACHE_POOL), this);
	this->ReInit();
};

HashTable<PID,LRCCACHEENTRY> 		*lrcCache = NULL;
Pool<LRCCACHEENTRY>   				*lrcPool = NULL;

#endif

HashTable<PID,PAGEHASH>   		*pageHashTable = NULL;
Pool<PAGEHASH>					*pageHashPool = NULL;


 int
BF_InitPageHash ()
{
	pageHashPool = new Pool<PAGEHASH> ( "PageHash", PAGEHASH_POOL,
		PageHashPoolSize, POOL_NOMORE_GETMORE );
	if(pageHashPool == NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return esmFAILURE;
	}
	/* pageHashPool->Stats(stderr); */

	pageHashTable = new HashTable<PID,PAGEHASH>( "PageHash", 
		PAGEHASH_TABLE, PageHashTableSize);
	if(pageHashTable == NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return esmFAILURE;
	}
	/* pageHashTable->Stats(stderr); */
	/* pageHashTable->Dump(stderr, __LINE__,__FILE__); */

#ifdef SERVER_MAKE
	lrcPool = new Pool<LRCCACHEENTRY>("LrcCacheEntry",  LRCCACHE_POOL,
		PageHashPoolSize, 0/* no flags */ );
	if(lrcPool == NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return esmFAILURE;
	}
	/* lrcPool->Stats(stderr); */

	lrcCache = new HashTable<PID,LRCCACHEENTRY>( 
		"LrcCache",  LRCCACHE, PageHashTableSize);
	if(lrcCache == NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return esmFAILURE;
	}
	/* lrcCache->Stats(stderr); */
	/* lrcCache->Dump(stderr, __LINE__,__FILE__); */
#endif SERVER_MAKE

	return esmNOERROR;
}

PAGEHASH *
BF_NewPageHash()
{
	PAGEHASH	*pageHash;

	pageHash  = pageHashPool->Get();
	return pageHash;
}

void
BF_DiscardPageHash( 
	PAGEHASH *pageHash
)
{
	pageHashTable->Remove(pageHash);
	pageHashPool->Put(pageHash);
}

#ifdef SERVER_MAKE
LRCCACHEENTRY *
bf_FindCacheEntry (
	PID		*pid 
)
{
    LRCCACHEENTRY				*lrcce;

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d.%d", pid->page, pid->volid));

    lrcce = lrcCache->Find(pid);
	if(lrcce == NULL) {
		TRPRINT(TR_BF, TR_LEVEL_2, ("page not found in cache"));
		return NULL;
	}

	/* better not be in the page hash table */
	SM_ASSERT(LEVEL_1, (pageHashTable->Find(pid) == NULL));

	return(lrcce);
}
#endif SERVER_MAKE

#ifdef SERVER_MAKE
 PAGEHASH 
*bf_FindPage (
	const 	PID     	*pid,
	FLAGS				flags 
)
{
    PAGEHASH					*pageHash;

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d.%d", pid->page, pid->volid));

again:
    pageHash = pageHashTable->Find(pid);

    if (pageHash != NULL)	{
		CHECK_PAGEHASH_MAGIC(pageHash);

		if (flags & BF_SEM)	{
			if (waitSemaphore( &(pageHash->semaphore) ) != esmNOERROR)	{

				if (Active->errno == esmPAGEGONE)	{
					SM_ERROR(TYPE_LOG, esmPAGEGONE);
					goto again;
				} else {
					return(NULL);
				}
			}
		}

		TRPRINT(TR_BF, TR_LEVEL_2, ("page is in cache"));
		Active->error = esmNOERROR;

		/* had better not be in the LRC cache */
		SM_ASSERT(LEVEL_1, (lrcCache->Find(pid) == NULL));

#if defined(SERVER_MAKE)&&defined(DEBUG)
		if(!formatvolRunning) {
			SM_ASSERT(LEVEL_1, PBUFFRAME(pageHash) != NULL);
		}
#endif  

	} else {

		TRPRINT(TR_BF, TR_LEVEL_2, ("page is not in cache"));
		Active->error = esmNOERROR;
		Active->errno = BF_NOT_PRESENT;
		pageHash = NULL;
	}
	return pageHash;
} /* server bf_FindPage */

#else

/* CLIENT SIDE */

 PAGEHASH *
bf_FindPage (
	const PID		*pid 
)
{
    PAGEHASH			*pageHash;

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d.%d", pid->page, pid->volid));

    pageHash = pageHashTable->Find(pid);
	if(pageHash == NULL) {
		TRPRINT(TR_BF, TR_LEVEL_2, ("pageHash not found"));
		return NULL;
	}

	CHECK_PAGEHASH_MAGIC(pageHash);

#ifdef DEBUG

	/*
	 *	check to make sure we haven't find found a chunk page
	 */
	if (PPAGE_TYPE(pageHash) !=  PAGE_LARGEDATA /* H_CHUNK */)	{

		/*
		 *	get the page info
		 */
		if (pageHash->pageInfo == NULL)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		TRPRINT(TR_BF, TR_LEVEL_2, ("bufIndex:%d bufCount:%d",
				pageHash->pageInfo->bufIndex, pageHash->pageInfo->blockCount));
	}
	TRPRINT(TR_BF, TR_LEVEL_2, ("page2size:%d", pageHash->page2size ));

	/* 
	 * some sanity checks: 
	 * Make sure there's a bufFrame.
	 */
	if(pageHash->pageInfo == NULL) {
		SM_ASSERT(LEVEL_1, (pageHash->flags == H_FREE) ||
				(PPAGE_TYPE(pageHash)==PAGE_LARGEDATA));
	} else {
		SM_ASSERT(LEVEL_1, PBUFFRAME(pageHash) != NULL);
	}
#endif DEBUG

	return pageHash;

} /* client bf_FindPage */
#endif SERVER_MAKE


 void
BF_InsertHashEntry (
	PAGEHASH			*pageHash,
	const	PID			*pid,
	PAGETYPE			pageType 
)
{
	TRPRINT(TR_BF, TR_LEVEL_1,  
		("BF_InsertHashEntry pid %d.%d",pid->page, pid->volid));

	CHECK_PAGEHASH_MAGIC(pageHash);

#ifdef DEBUG
	{
		char *bufFrame;
#ifndef SERVER_MAKE
		if( pageHash->pageInfo == NULL) {
			if ( pageHash->flags != H_FREE ) {

				TRPRINT(TR_SM | TR_BF, TR_LEVEL_1, 
					("MISSING pageInfo for pageHash 0x%x, pid %d.%d, pageType %x",
					pageHash, pid->page, pid->volid, pageType));
				SM_ASSERT(LEVEL_1, FALSE);
				SM_ERROR(TYPE_CRASH, esmINTERNAL);
			}
		} else 
#endif SERVER_MAKE
			bufFrame = PBUFFRAME(pageHash);
		
		if( (bufFrame == NULL) && 
#ifdef SERVER_MAKE
			!(pageHash->flags & H_TEMP) 
#else SERVER_MAKE
			(pageType != H_CHUNK)
#endif SERVER_MAKE
		) {
			TRPRINT(0x1, TR_LEVEL_1, 
				("MISSING bufFrame for pageHash 0x%x, pid %d.%d, pageType %x",
				pageHash, pid->page, pid->volid, pageType));
			SM_ASSERT(LEVEL_1, FALSE);
			SM_ERROR(TYPE_CRASH, esmINTERNAL);
		}
	}
#endif DEBUG

	/* the key value */
	pageHash->hashList.hashPid = *pid;

	pageHash->flags = 0; /* MUST come before setting pageType */
	PSET_PAGE_TYPE(pageHash,pageType);

#ifdef JUNK
	/* We do not set the page2size here, because the size is
	* sometimes set in the pagehash after BF_NewPageHash()
	* but before BF_InsertHashEntry, the reason being that
	* H_CHUNKs may be of just about any page2size
	* When chunks are eliminated, we can set the page2size here.
	*/
	switch(PPAGE_TYPE(pageHash)) {
		case  PAGE_GENERIC:
			pageHash->page2size = MIN_PAGE2SIZE;
			break;

		case  PAGE_SLOTTED:
			pageHash->page2size = SLOTTED_PAGE2SIZE;
			break;

		case  PAGE_LARGENODE:
			pageHash->page2size = LG_PAGE2SIZE;
			break;

		case  PAGE_LARGEDATA:
			pageHash->page2size = LGDATA_PAGE2SIZE;
			break;

		case  PAGE_LOG:
			pageHash->page2size = MIN_PAGE2SIZE;
			break;

		case  PAGE_INDEX:
			pageHash->page2size = BTREE_PAGE2SIZE;
			break;

		case  PAGE_INDEXDESC:
			pageHash->page2size = INDEXDESC_PAGE2SIZE;
			break;
	}
#endif JUNK

#ifndef SERVER_MAKE

	/* Initialize the recovery-related parts of the hash entry */

    pageHash->firstLSN.wrapCount = 0;
    pageHash->firstLSN.offset = 0;
    pageHash->firstLRC.wrapCount = 0;
    pageHash->firstLRC.count = 0;
    pageHash->readLock = pageHash->writeLock = NULL_TID;

	/* if this is not a log page, init the lrc in the page hash */
	if (PPAGE_TYPE(pageHash) == H_CHUNK) {
		SERVERINFO	*serverInfo;

		if ( (serverInfo = PPID2SERVERINFO(pid)) == NULL) {
			/* This should never happen */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
		/* Initialize the lrc in the hash entry just to be safe.  
		   Note that this is only used for large data pages */
		pageHash->lrc = *(serverInfo->logInfo.initLRC);
	} else { 
		pageHash->lrc = LrcNone;
	}


#else
	/*
	 * server: A page cannot be in both the page hash table
	 * and the lrc cache, so remove it from the lrc cache
	 * if it is there.
	 */

	{
		LRCCACHEENTRY 				*lrcce;
		
		lrcce = lrcCache->Find(pid);

		if(lrcce != NULL) {
			lrcCache->Remove( lrcce );
			lrcce->flags = H_FREE;
			lrcPool->Put( lrcce );
		}
	}
#endif SERVER_MAKE
	pageHashTable->Insert( pageHash );
	TRPRINT(TR_BF, TR_LEVEL_1,  
		("pid %d.%d inserted in %s",
		pageHash->hashList.hashPid.page, pageHash->hashList.hashPid.volid,
		pageHashTable->Name()));
}

#ifdef DEBUG

static BOOL
fatal(
	void *arg1,
	char *arg2
)
{
	TRPRINT(TR_BF, TR_LEVEL_1, 
	("Unexpected object 0x%x in the %s table", arg1, arg2));

	SM_ERROR(TYPE_FATAL, esmINTERNAL);
	/* NOTREACHED */
	return TRUE;
}

void
BF_AssertPageHashTableEmpty()
{
#ifdef __GNUC__
	PAGEHASH *x;
	pageHashTable->ForEach(2,(x->Dump),stderr);
	pageHashTable->ForEach(2,fatal,pageHashTable->Name());
#else __GNUC__
	pageHashTable->ForEach(2,((FOREACHFUNC)PAGEHASH::Dump),stderr);
	pageHashTable->ForEach(2,((FOREACHFUNC)fatal),pageHashTable->Name());
#endif __GNUC__
}

#endif DEBUG

static BOOL
discardPageHash( 
	PAGEHASH *pageHash
)
{
	BF_DiscardPageHash(pageHash);
	return FALSE;
}

void
BF_DeleteHashTable()
{
	if(pageHashTable == NULL) return;
#ifdef COMMENT
	fprintf(stderr, "BEGIN temporary debugging output (please ignore) .\n");
	pageHashTable->Dump(stderr, __LINE__,__FILE__);
#endif COMMENT
	bf_ForEachPageInHashTable( discardPageHash, NULL );
#ifdef COMMENT
	pageHashTable->Dump(stderr, __LINE__,__FILE__);
	fprintf(stderr, "END debugging output.\n");
#endif COMMENT
#ifdef DEBUG
	/* 
	 * Before we delete the hash table, remove everything in it,
	 * and put everything back into the pool. 
	 * Check to see that it worked.
	 */
	BF_AssertPageHashTableEmpty();
#endif DEBUG
	delete pageHashTable;
	delete pageHashPool; 
	pageHashTable = NULL;
	pageHashPool = NULL;


#ifdef SERVER_MAKE
	delete lrcCache;
	delete lrcPool;
	lrcCache = NULL;
	lrcPool = NULL;
#endif SERVER_MAKE
}

 void
BF_RemovePageFromHT (
	PAGEHASH	*pageHash 
)
{
	/* This func is NOT meant to put the pageHash back into the
	 * pool; it is called on pageHashes that are or will be 
	 * found on the  free buf list (and reached through their 
	 * corresponding groupLinks) 
	 */
	pageHashTable->Remove(pageHash);
}

#ifdef SERVER_MAKE

static void BF_StealLRCCache( PID *, int);

 void
BF_CacheLRCInfo (
	register PAGEHASH	*pageHash
)
{
	LRCCACHEENTRY			*lrcce;
	BOOL					firstTry = TRUE;

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d", pageHash->hashList.hashPid.page) );


#ifdef DEBUG
	{
		/* Sanity check: the page is not in the page hash table.
		 * (A page is removed from the lrc cache when it is inserted in the
		 * hash table, and put into the lrc cache after it is removed
		 * from the hash table.)
		 */
		PID	*pid;

		pid = pageHash->keylocation(PAGEHASH_TABLE, &pid);
		SM_ASSERT(LEVEL_1, lrcCache->Find(pid) == NULL );
	}
#endif DEBUG

again:
	lrcce = lrcPool->Get();

	if(lrcce == NULL) {
		if (firstTry) {
			/* steal a few from the lrcCache */
			BF_StealLRCCache( &pageHash->hashList.hashPid, 10 );
			firstTry = FALSE;
			goto again;
		} else {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	} else {
		/* copy key from pageHash to lrcce */
		PID *p1, *p2;

		p1 = lrcce->keylocation(LRCCACHE, &p1);
		p2 = pageHash->keylocation(PAGEHASH_TABLE,  &p2);

		*p1 = *p2;
	}
		
	{
		LRC						*lrcLoc;

		if(lrcLoc = bf_lrcLoc(pageHash))
			lrcce->lrc = *lrcLoc;
	}
	if(pageHash->pageType == PAGE_SLOTTED) {
		lrcce->fid = /* garbage for all but slotted pages */
		 ((SLOTTEDPAGE *) PBUFFRAME(pageHash))->header.fid;
	}
	lrcCache->Insert(lrcce);

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d.%d, lrc %d.%d", 
		lrcce->hashList.hashPid.page, lrcce->hashList.hashPid.volid,
		lrcce->lrc.wrapCount, lrcce->lrc.count));
} /* BF_CacheLRCInfo */

/* helper function for BF_FlushLRCCache and BF_StealLRCCache */
static  BOOL
discardIfMatch (
	LRCCACHEENTRY 	*lrcce, 
	VOLID 			volid,
	int				*wanted
)
{

	TRPRINT(TR_BF, TR_LEVEL_1,("discard if match %d", volid));
	if(lrcce->hashList.hashPid.volid == volid) {
		TRPRINT(TR_BF, TR_LEVEL_1,
				("Clobbering cached lrc for page %d, volid %d",
				lrcce->hashList.hashPid.page, lrcce->hashList.hashPid.volid));
		lrcCache->Remove(lrcce);
		lrcce->flags = H_FREE;
		lrcPool->Put( lrcce );
		/* this comparison must be ==, not <= 
		 * so that we'll never return TRUE when we try to 
		 * flush the entire cache.
		 */
		if( ((*wanted)--) == 0) 
			return TRUE; /* last one */
	}
	return FALSE;
}

 void
BF_FlushLRCCache( 
	VOLID volid
)
{
	int wanted = -1; /* do the whole thing */
	/* clear everything from the lrc cache  */
	/* uses helper the function above 		*/

	TRPRINT(TR_BF, TR_LEVEL_1, ("BF_FlushLRCCache for volid %d", volid));

	lrcCache->ForEach(3, ((FOREACHFUNC)discardIfMatch), volid, &wanted);
	TRPRINT(TR_BF, TR_LEVEL_1, ("end BF_FlushLRCCache for volid %d", volid));

	/* ignore "wanted" */
}

/* helper function for BF_StealLRCCache */
static  BOOL
discardFirst (
	LRCCACHEENTRY 	*lrcce, 
	PID 			*unused,
	int				*wanted
)
{
	TRPRINT(TR_BF, TR_LEVEL_1,
			("Clobbering cached lrc for page %d, volid %d",
			lrcce->hashList.hashPid.page, lrcce->hashList.hashPid.volid));
	lrcCache->Remove(lrcce);
	lrcce->flags = H_FREE;
	lrcPool->Put( lrcce );

	if( ((*wanted)--) <= 0)
			return TRUE; /* last one */
	return FALSE;
}

 void
BF_StealLRCCache( 
	PID 	*pid,
	int		numberWanted
)
{
	int wanted = numberWanted;

	/* steal some semi-arbitrary entries from the lrc cache */
	/* uses helper the function above 		*/

	lrcCache->ForEach(4, ((FOREACHFUNC)discardIfMatch), pid->volid, &wanted);
	if(wanted > 0) {
		lrcCache->ForEach(4, ((FOREACHFUNC)discardFirst), pid, &wanted);
	}
	if(wanted > 0) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
}

 void
BF_FreeHashEntry (

	register PAGEHASH	*pageHash,
	int					eno 
)
{
	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d.%d", 
		pageHash->hashList.hashPid.page,
		pageHash->hashList.hashPid.volid));

	/*
	 *	check the magic number
	 */
	CHECK_PAGEHASH_MAGIC(pageHash);

	/*
	 *	check to see if we need to notify any waiters by error
	 */
	if (eno != esmNOERROR)	{

		notify( &(pageHash->semaphore.waitList), esmFAILURE, eno );

	} else {

		notify( &(pageHash->semaphore.waitList), esmNOERROR, esmNOERROR );
	}

	BF_DiscardPageHash(pageHash);
}
#endif SERVER_MAKE

void
bf_ForEachPageInHashTable( void *func, void *arg )
{
	pageHashTable->ForEach(2, (FOREACHFUNC)func,arg);
}
#ifdef DEBUG
BEGIN_EXTERNC
    void BF_PrintHashTable();
END_EXTERNC
void
BF_PrintHashTable()
{
    pageHashTable->Dump(stderr, __LINE__,__FILE__);
}
#endif DEBUG


#endif __BF_PageHash_C__
