/*
 *   $RCSfile: BF_ReplacePolicy.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"

#ifdef DEBUG
 void
BF_ReplacePolicy (

	BUFGROUP	*bufGroup,
	LIST		*list,
	int			replacePolicy 	
)
{

	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	check the data structure magic number
	 */
	CHECK_BUFGROUP_MAGIC(bufGroup);

	/*
	 *	Check to see which policy is specified
	 */
	if (replacePolicy == BF_DEFAULT)	{

		/*
		 *	Set to the default for the buffer group
		 */
		replacePolicy = bufGroup->replacePolicy;
	}

	/*
	 *	take action based on the policy
	 */
	switch (replacePolicy)	{

		case BF_LRU:

			/*
			 *	Put on the end of the list
			 */
			listEnq( &(bufGroup->unfixedList), list );
			break;

		case BF_MRU:

			/*
			 *	Put on at the beginning of the list
			 */
			listPush( &(bufGroup->unfixedList), list );
			break;

		default:

			/*
			 *	This should not happen
			 */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}
}

#endif
