/*
 *   $RCSfile: bf_DeleteChunk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "util_funcs.h"
#include "bf_globals.h"


 void
bf_DeleteChunk (

	CHUNKDESC			*chunkDesc,
	int					offset,
	int					length 
)
{

	register CHUNKPAGE	*chunkPage;
	PAGEHASH			*pageHash;
	char				*deletePosition;


	TRPRINT(TR_BF, TR_LEVEL_1, ("offset:%d length:%d", offset, length));

	/*
	 *	Check the input paramters.
	 */
	SM_ASSERT(LEVEL_3, offset >= 0);

	/*
	 *	get a pointer to the ONLY chunkPage
	 */
	chunkPage = chunkDesc->chunkPage;

	/*
	 *	Invalidate any chunks (except the current one) which
	 *	hold this page.
	 */
	bf_HackChunkPage(&(chunkPage->pid));

	if ( (pageHash = bf_FindPage( &(chunkPage->pid) )) == NULL) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));

	/*
	 *	setup the new length
	 */
	chunkPage->size -= length;

	/*
	 *	dirty the page
	 */
	bf_DirtyPage(pageHash);

	/*
	 *	set the fields of the chunkpage
	 */
	chunkPage->flags     = PAGE_VALID|PAGE_DIRTY;
	SM_ASSERT(LEVEL_1, LIST_MEMBER(&(chunkPage->hashList)));

	/*
	 *	Set the dirty count for the chunk
	 */
	chunkDesc->dirtyPages = 1;

	/*
	 *	get a pointer to the insert position
	 */
	deletePosition = chunkDesc->bufFrame + offset;

	/*
	 *	update the chunk length
	 */
	chunkDesc->totalLength -= length;
	TRPRINT(TR_BF, TR_LEVEL_2, ("new chunk length:%d", chunkDesc->totalLength));

	/*
	 *	copy the data to overwrite the deleted portion
	 */
	bcopy(deletePosition + length, deletePosition, 	
						(int)(chunkDesc->totalLength - offset));
	TRPRINT(TR_BF, TR_LEVEL_2, ("filling hole in chunk source:%x dest:%x",
			deletePosition + length, deletePosition));

	/*
	 *	mark the chunk as dirty
	 */
	DIRTY_CHUNK(chunkDesc);

	BF_AuditChunk(chunkDesc);
	BF_AuditGroup(chunkDesc->bufGroup);
}
