/*
 *   $RCSfile: bf_DirtyPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trans.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


extern TRANSREC		TransRec;


 void
bf_DirtyPage (

	register PAGEHASH	*pageHash 
)
{

	TRPRINT(TR_BF, TR_LEVEL_1, ("dirtying page:%d", pageHash->hashList.hashPid.page));

	/*
	 *	check the magic number
	 */
	CHECK_PAGEHASH_MAGIC(pageHash);

	/*
	 *	Check to see if the page is on the dirty list
	 */
	if (PAGE_IS_CLEAN(pageHash))	{

		/*
		 *	check to make sure the element is not on dirty list
		 */
		SM_ASSERT(LEVEL_3, NOT_LIST_MEMBER( &(pageHash->dirtyList) ));

		/*
		 *	put on dirty list and mark dirty
		 */
		listEnq( &(TransRec.pageHashList), &(pageHash->dirtyList) );
		pageHash->flags |= H_DIRTY;

	} else {

		/*
		 *	check to make sure element is on dirty list
		 */
		SM_ASSERT(LEVEL_3, LIST_MEMBER( &(pageHash->dirtyList) ));
	}
}
