/*
 *   $RCSfile: bf_DismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "volume.h"
#include "load.h"
#include "scan.h"
#include "sm_params.h"
#include "btree.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "bf_globals.h"


 int 
bf_DismountVolume (

	register VOLID		volid 
)
{

    register PAGEHASH   *pageHash;
	register GROUPLINK	*groupLink;
	register BUFGROUP	*bufGroup;
	GROUPLINK			*tempLink;
	CHUNKDESC			*chunkDesc;
	int					i;


	TRPRINT(TR_BF, TR_LEVEL_1, ("dismounting:%d", volid));

	/*
	 *	get a pointer to the first in use buffer group
	 */
	bufGroup = (BUFGROUP *) FIRST_LIST_ELEMENT( &(UsedBufGroups) );

	/*
	 *	run through the open buffer group list
	 *	to see if there are any fixed pages
	 */
	while (bufGroup != NULL)	{

		/*
		 *	check the buffer group magic number
		 */
		CHECK_BUFGROUP_MAGIC(bufGroup);
		TRPRINT(TR_BF, TR_LEVEL_2, ("buffer group:%d", bufGroup->index));

		/*
		 *	remove a buffer from the list
		 */
		groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->fixedList) );

		/*
		 *	run down the list
		 */
		while (groupLink != NULL)	{

			/*
			 *	check to see if this is a page or chunk
			 */
			if (groupLink->type == TYPE_PAGE)	{

				/*
				 *	check the grouplink magic
				 */
				CHECK_GROUPLINK_MAGIC(groupLink);

				/*
				 *	check the pid to see if the volume matches
				 */
				if (groupLink->pageHash->hashList.hashPid.volid == volid)	{

					SM_ERROR(TYPE_LOG, esmFIXEDPAGES);
					return(esmFAILURE);
				}

				/*
				 *	get the pointer to the next entry
				 */
				groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
			
			} else if (groupLink->type == TYPE_CHUNK)	{

				/* get a chunk descriptor pointer */
				chunkDesc = (CHUNKDESC *) groupLink;

				/* check the chunkdescriptor magic */
				CHECK_CHUNKDESC_MAGIC(chunkDesc);

				/* Check to see if the chunk is on this volume */
				if (chunkDesc->chunkPage->pid.volid == volid)	{

					SM_ERROR(TYPE_LOG, esmFIXEDPAGES);
					return(esmFAILURE);
				}

				/* get the pointer to the next entry */
				groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );

			} else {

				/* this should not happen */
				SM_ERROR(TYPE_WARNING, esmINTERNAL);
				return(esmFAILURE);
			}
		}

		/*
		 *	get a pointer to the next buffer group
		 */
		bufGroup = (BUFGROUP *) NEXT_LIST_ELEMENT( &(bufGroup->groupList) );
	}

	/* get a pointer to the first in use buffer group */
	bufGroup = (BUFGROUP *) FIRST_LIST_ELEMENT( &(UsedBufGroups) );

	/*
	 *	Run through the open buffer group list to flush all the dirty pages.
	 *  The pages end up on the free list, and might have been
     *  forced to the server.  
	 */
	while (bufGroup != NULL)	{

		/* 	check the buffer group magic number */
		CHECK_BUFGROUP_MAGIC(bufGroup);
		TRPRINT(TR_BF, TR_LEVEL_2, ("buffer group:%d", bufGroup->index));

		/* remove a buffer from the list */
		groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->unfixedList) );

		/* run down the list */
		while (groupLink != NULL)	{

			/* check to see if this is a page or chunk */
			if (groupLink->type == TYPE_PAGE)	{

				/* check the grouplink magic */
				CHECK_GROUPLINK_MAGIC(groupLink);

				/* check the pid to see if the volume matches */
				if (groupLink->pageHash->hashList.hashPid.volid == volid)	{

					/* get the pointer to the next entry */
					tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );

					/* take the link off the list */
					listRemove( &(groupLink->groupList) );

					/* flush the page */
					BF_FreePage(bufGroup, groupLink, FALSE);

					/* setup the pointer to the next entry */
					groupLink = tempLink;

				} else {

					/* get the pointer to the next entry */
					groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
				}
			
			} else if (groupLink->type == TYPE_CHUNK)	{

				/* get a chunk descriptor pointer */
				chunkDesc = (CHUNKDESC *) groupLink;

				/* check the chunkdescriptor magic */
				CHECK_CHUNKDESC_MAGIC(chunkDesc);

				/* Check to see if the chunk is on this volume */
				if (chunkDesc->chunkPage->pid.volid == volid)	{

					/* save the pointer to the next link */
					tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );

					/* take the chunk off the list */
					listRemove( &(chunkDesc->memberList) );

					/* free the chunk */
					BF_FreeChunk(bufGroup, chunkDesc);

					/* put the link back */
					groupLink = tempLink;

				} else {

					/* get the pointer to the next entry */
					groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );
				}

			} else {

				/* this should not happen */
				SM_ERROR(TYPE_WARNING, esmINTERNAL);
				return(esmFAILURE);
			}
		}

		/* get a pointer to the next buffer group */
		bufGroup = (BUFGROUP *) NEXT_LIST_ELEMENT( &(bufGroup->groupList) );
	}

	/*
	 *	Loop through the free lists.  They can contain dirty pages
	 *  that need to be forced to the server.
	 */
	for (i = 0; i < NumBufferSizes; i++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("processing list:%d", i));
		
		if (LIST_NOT_EMPTY( &FreeBufTable[i] ))	{

			groupLink = (GROUPLINK *) 
				FIRST_LIST_ELEMENT( &(FreeBufTable[i]) );

			/* Flush all pages on the list */
			while (groupLink != NULL)	{

				/* check the type of the buffer */
				if (groupLink->type == TYPE_PAGE)	{

					/* Check the groupLink magic number */
					CHECK_GROUPLINK_MAGIC(groupLink);

					/* Get a pointer to the page hash structure */
					pageHash = groupLink->pageHash;

					if (pageHash->hashList.hashPid.volid == volid)	{
						/* 
						 * Force the page to the server, remove it from
						 * dirty lists, and discard it (remove from hash table).
						 * Also, invalidate its locks.
						 */
						bf_InvalidatePage(pageHash, TRUE, TRUE /* force */);

					}

					groupLink = (GROUPLINK *) 
						NEXT_LIST_ELEMENT( &(groupLink->groupList) );

				} else if (groupLink->type == TYPE_CHUNK)	{

					chunkDesc = (CHUNKDESC *) groupLink;
					CHECK_CHUNKDESC_MAGIC(chunkDesc);

					if (chunkDesc->flags & CHUNK_VALID)	{

						/* flush and invalidate the chunk */
						BF_InvalidateChunk(chunkDesc);
					}

					groupLink = (GROUPLINK *) 
						NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );
				
				} else {
					SM_ERROR(TYPE_FATAL, esmINTERNAL);
				}
			}
		}
	}

	/* dismount the volume */
	return(io_Dismount(volid));
}
