/*
 *   $RCSfile: bf_FakeReadPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:15 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "trans.h"
#include "bf_extfuncs.h"
#include "trans_globals.h"
#include "sm_state.h"


/*
 *	bf_FakeReadPage() fakes the read of a newly allocated page, and
 *	locks it exclusively
 */


 GROUPLINK 
*bf_FakeReadPage (

	BUFGROUP	*bufGroup,
	const PID	*pid,
	PAGETYPE	flags,
	PAGE2SIZE	page2size 
)
{

	GROUPLINK	*groupLink;
	PAGEHASH	*pageHash;


	TRPRINT(TR_BF, TR_LEVEL_2, ("getting buffer for page:%d page2size:%d",
					pid->page, page2size));

	/* 
	 *	See if page is alreay in the buffer pool.  If it is, it
	 *	must be removed. 
	 */
	if ( (pageHash = bf_FindPage(pid)) != NULL) {
		/* the page must be invalid and not dirty*/
		SM_ASSERT(LEVEL_3, (pageHash->flags & H_INVALID) && !(pageHash->flags & H_DIRTY));
		bf_InvalidatePage(pageHash, TRUE, FALSE);
	}

	if ( (groupLink = bf_GetPage(bufGroup, pid, flags, NULL, page2size)) == NULL) {
		return(NULL);
	}

	SET_PAGEHASHLOCK(groupLink->pageHash, EX);

	/* remember the page is newly allocated */
	groupLink->pageHash->flags |= H_NEW;

	{
		/* see if this is on a temp volume */
		VOLREC *volRec = io_FindVolId(pid->volid);

		SM_ASSERT(LEVEL_1, volRec!=NULL);

		/* We generate log records when we allocate pages (which
		 * is when we call this func) so our log level had better
		 * match that of the volume.
		 */
		if(volRec->properties & VOL_TEMP) {
			SM_ASSERT(LEVEL_1, SM_State.logLevel == LOG_NONE);
			 groupLink->pageHash->flags |= H_TEMP;
		} else {
			SM_ASSERT(LEVEL_1, SM_State.logLevel != LOG_NONE);
		}
	}

	return(groupLink);
}
