/*
 *   $RCSfile: bf_FlushDirty.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trans.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"

#ifdef COMMENT
/*
 * Not called anywhere.
 * If this does get used, must decide if you want
 * the pages to remain in the page hash.
 */
extern TRANSREC		TransRec;


 void
bf_FlushDirty ()

{
	register PAGEHASH	*pageHash;
	register GROUPLINK	*groupLink;


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	while there are dirty pages on the transaction list,	flush them
	 */
	while (LIST_NOT_EMPTY( &(TransRec.pageHashList) ))	{

		/*
		 *	get a pointer to the pageHash
		 */
		pageHash = (PAGEHASH *) FIRST_LIST_ELEMENT( &(TransRec.pageHashList) );
		CHECK_PAGEHASH_MAGIC(pageHash);
		TRPRINT(TR_BF, TR_LEVEL_2, ("pid:%d", pageHash->hashList.hashPid.page));

		/*
		 *	make sure the page is dirty
		 */
		SM_ASSERT(LEVEL_3, PAGE_IS_DIRTY(pageHash));

		/*
		 *	get a pointer to the groupLink
		 */
		groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(pageHash->memberList) );

		/*
		 *	switch on the type of the page
		 */
		switch (groupLink->type)	{

			case TYPE_PAGE:

				CHECK_GROUPLINK_MAGIC(groupLink);

				/*
				 *	check to see if the page is in a buffer group
				 */
				while (pageHash->memberCount > 0)	{

					groupLink = (GROUPLINK *) 
						NEXT_LIST_ELEMENT( &(pageHash->memberList) );

					CHECK_GROUPLINK_MAGIC(groupLink);

					/*
					 *	remove the link from the buffer group
					 */
					listRemove( &(groupLink->groupList) );

					/*
					 *	free the page
					 */
					BF_FreePage(groupLink->bufGroup, groupLink, FALSE);
				}

				/*
				 *	invalidate the page, but leave it in the
				 *  hash table for reclaims (remove==FALSE).
				 */
				bf_InvalidatePage(pageHash, FALSE, TRUE);
				break;

			case TYPE_CHUNK:

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
				break;

			default:

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
				break;
		}
	}
}
#endif COMMENT

 void
bf_RemoveFromBufGroups( 
	PAGEHASH *pageHash 
)
{
	register GROUPLINK	*groupLink;

	/*
	 *	check to see if the page is in a buffer group
	 */
	while (pageHash->memberCount > 0)	{

		groupLink = (GROUPLINK *) 
			NEXT_LIST_ELEMENT( &(pageHash->memberList) );

		CHECK_GROUPLINK_MAGIC(groupLink);

		/*
		 *	remove the link from the buffer group
		 */
		listRemove( &(groupLink->groupList) );

		/*
		 *	free the page
		 */
		BF_FreePage(groupLink->bufGroup, groupLink, FALSE);
	}
}
