/*
 *   $RCSfile: bf_ForcePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "lsn.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "volume.h"
#include "trace.h"
#include "error.h"
#include "trans.h"
#include "clog.h"
#include "loginfo.h"
#include "logrecs.h"
#include "serverinfo.h"
#include "clog_extfuncs.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "clog_globals.h"
#include "trans_globals.h"

#include "file.h"

 void
bf_ForcePage (
    register PAGEHASH   *pageHash,
    BFFLAGS             flags 
)
{

    TRPRINT(TR_BF, TR_LEVEL_1, 
		("pid:%d index:%d size:%d", pageHash->hashList.hashPid.page, 
		pageHash->pageInfo->bufIndex, BLOCKCOUNT(pageHash->page2size)));

    /* check the pagehash magic */
    CHECK_PAGEHASH_MAGIC(pageHash);

    /* check to see if the page is dirty */
    if (PAGE_IS_DIRTY(pageHash) || (flags & BF_FORCE))  {

		SM_ASSERT(LEVEL_1, (PPAGE_TYPE(pageHash) != H_LOG)); 
		SM_ASSERT(LEVEL_1, ((pageHash->flags & H_INVALID)==0));

        /*
         *  sanity check 
         */
        if (pageHash->forceMark == 0)   {
            SM_ASSERT(LEVEL_2, (pageHash->firstMark == 0));
		}

		/* 
		 * Force the page to the server.
		 * io_WritePage enforces WAL.
		 */
		if (io_WritePage( &(pageHash->hashList.hashPid),
				 pageHash->pageInfo->bufFrame, pageHash)) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		bf_CleanPage(pageHash);

		/* might already have been 0 */
		pageHash->forceMark = pageHash->firstMark = 0;
    }
}


 void
bf_ForceLogPage (
	SERVERINFO	*serverInfo,	
    PAGEHASH	*pageHash,
    BFFLAGS     flags 
)
{
	LOGINFO		*logInfo;

    TRPRINT(TR_BF, TR_LEVEL_1, ("pid:%d index:%d size:%d", pageHash->hashList.hashPid.page, pageHash->pageInfo->bufIndex, BLOCKCOUNT(pageHash->page2size)));

    /* check the pagehash magic */
    CHECK_PAGEHASH_MAGIC(pageHash);

	if (PPAGE_TYPE(pageHash) != H_LOG)    {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	logInfo = &(serverInfo->logInfo);

    /* check to see if the page is dirty */
    if (PAGE_IS_DIRTY(pageHash) || (flags & BF_FORCE))  {

		SM_ASSERT(LEVEL_3, pageHash->forceMark != 0);

		/* check to make sure WAL is correct */
		SM_ASSERT(LEVEL_3, logInfo->forceMark <= pageHash->forceMark);

		/* force it to disk */
		if (clog_WriteLogPage(serverInfo, pageHash)) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/* clean the page */
		bf_CleanPage(pageHash);

		/* update the log force indicator */
		logInfo->forceMark = pageHash->forceMark;
		TRPRINT(TR_BF, TR_LEVEL_2, ("forcing log page:%d", pageHash->forceMark));

		/* reset the force and first marks */
		pageHash->firstMark = pageHash->forceMark = 0;
    } else {
		if (pageHash == logInfo->tailBuffer) {
	         logInfo->tailBuffer = NULL;
	    }
	}
}
