/*
 *   $RCSfile: bf_GetPageBuf.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 GROUPLINK 
*BF_ConvertToPage (

	register CHUNKDESC	*chunkDesc 
)
{

	register PAGEHASH	*pageHash;
	register GROUPLINK	*groupLink;
	register PAGEINFO	*pageInfo;
	register BUFINFO	*bufInfo;


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	check to see if the chunk is valid
	 */
	if (chunkDesc->flags & CHUNK_VALID)	{

		/*
		 *	flush and invalidate the chunk
		 */
		BF_InvalidateChunk(chunkDesc);
	}

	/*
	 *  attempt to get page structures
	 */
	if ((groupLink = (GROUPLINK *) listDeq( &BufStructPool)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	get a pointer to the pageHash
	 */
	pageHash = groupLink->pageHash;
	CHECK_PAGEHASH_MAGIC(pageHash);

	/*
	 *	Initialize the pagehash structures
	 */
	pageHash->page2size = chunkDesc->buf2size;

	/*
	 *	initialize the pageInfo structure
	 */
	pageInfo			 = pageHash->pageInfo;
	pageInfo->bufIndex	 = chunkDesc->bufIndex;
	groupLink->bufFrame = pageInfo->bufFrame	 = chunkDesc->bufFrame;
	pageInfo->blockCount = BLOCKCOUNT(pageHash->page2size);
	bufInfo = pageInfo->bufInfo = chunkDesc->bufInfo;

	/*
	 *	mark the new bufInfo information
	 */
	BF_SetSlot(bufInfo, pageInfo->blockCount, (char *) pageHash, TYPE_PAGE);

	/*
	 *	put the chunk descriptor back into the chunk descriptor pool
	 */
	listEnq( &ChunkDescPool, &(chunkDesc->memberList) );

	/*
	 *	return the groupLink
	 */
	return(groupLink);
}
/*============================================================================*/
/* 
 * bf_GetPageBuf:
 * 	Routine to insert a page into the page hash,
 *  and accounted for it the given buffer group.
 *
 * RETURNS:
 *	esmNOERROR if all went ok and address of bucket via 
 *	hashPtr parameter.
 *
 * SIDE EFFECTS:
 *	The page is entered into the page hash and 
 *  accounted for in the given buffer group.
 *  If no buffer group is given, this effectively gets a
 *  temporary page, since the page doesn't belong to any buffer group.
 *
 * SPECIAL NOTE:
 *	No check is made to see whether the page is already in the hash.
 * 	This check should be made by the calling routine.
 *
 * CALLED FROM:
 *	bf_GetPage, bf_GetTempPage, fi_NeighborPage
 */
/*============================================================================*/


 GROUPLINK 
*bf_GetPageBuf (

	BUFGROUP			*bufGroup,
	PAGE2SIZE			page2size 
)
{

	register GROUPLINK	*groupLink;


	TRPRINT(TR_BF, TR_LEVEL_1, ("need page size:%d", BLOCKCOUNT(page2size)));

	/*
	 *	Attempt to get a buffer off of the free list
	 *	of the requested size
	 */
	if (groupLink = (GROUPLINK *) listDeq( &(FreeBufTable[OFFSET(page2size)]) ))	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("got right size buffer"));

		/*
		 *	check to what type of buffer this is
		 */
		if (groupLink->type == TYPE_PAGE)	{

			TRPRINT(TR_BF, TR_LEVEL_2, ("type page"));

			/*
			 *	check the magic number
			 */
			CHECK_GROUPLINK_MAGIC(groupLink);
			TRPRINT(TR_BF, TR_LEVEL_2, ("index:%d size:%d",
					groupLink->pageHash->pageInfo->bufIndex,
					groupLink->pageHash->pageInfo->blockCount));

			/*
			 *	check to see that the page is the right size
			 */
			SM_ASSERT(LEVEL_3, groupLink->pageHash->page2size == page2size);

			/*
			 *	Invalidate the page, remove it from the hash table.  
			 *  (remove == TRUE, forceToServer ==TRUE)
			 */
			bf_InvalidatePage(groupLink->pageHash, TRUE, TRUE);


		} else if (groupLink->type == TYPE_CHUNK)	{

			/*
			 *	convert the chunk buffer into a page
			 */
			if ((groupLink = BF_ConvertToPage((CHUNKDESC *) groupLink)) == NULL)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}
		
		} else {

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	} else if (groupLink = (GROUPLINK *) BF_GetLarger(page2size, TYPE_PAGE))	{

		/*
		 *	check the magic number
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);

	} else if (groupLink = (GROUPLINK *) BF_GetSmaller(page2size, TYPE_PAGE))	{

		/*
		 *	check the magic number
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);

	} else {

		SM_ERROR(TYPE_WARNING, esmINTERNAL);
		return(NULL);
	}

	/*
	 *	Set the Group count
	 */
	groupLink->pageHash->memberCount = 1;

	/*
	 *	Set the fixed count
	 */
	groupLink->fixedCount = 1;


	if(bufGroup != NULL) {
		/*
		 *	Hang the buffer off the fixed list
		 */
		listEnq( &(bufGroup->fixedList), &(groupLink->groupList) );

		groupLink->bufGroup = bufGroup;

		/*
		 *	Increment the fixed charges
		 */
		bufGroup->fixedCharges += BLOCKCOUNT(page2size);

		BF_AuditGroup(bufGroup);

	} else {
		TRPRINT(TR_BF, TR_LEVEL_2, 
		("index:%d size:%d", groupLink->pageHash->pageInfo->bufIndex,
			BLOCKCOUNT(groupLink->pageHash->page2size)));
	}

	return(groupLink);
}
