/*
 *   $RCSfile: bf_Group.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "chunk.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"

#ifdef DEBUG
void
BF_AuditChunkPage(CHUNKPAGE *chunkPage) 
{
	PAGEHASH *pageHash;

	pageHash = chunkPage->pageHash;

	if(chunkPage->flags & PAGE_VALID) {
		CHECK_PAGEHASH_MAGIC(pageHash);
		SM_ASSERT(LEVEL_1, pageHash->hashList.hashPid.page ==
			chunkPage->pid.page);
		SM_ASSERT(LEVEL_1, pageHash->flags & H_CHUNK);
		SM_ASSERT(LEVEL_1, (pageHash->memberCount > 0));
		SM_ASSERT(LEVEL_1, (pageHash->page2size == LGDATA_PAGE2SIZE));
		SM_ASSERT(LEVEL_1, LIST_MEMBER( &(chunkPage->hashList)));
	} else {
		SM_ASSERT(LEVEL_1, pageHash == NULL);
		SM_ASSERT(LEVEL_1, NOT_LIST_MEMBER( &(chunkPage->hashList)));
	}
}
void
_BF_AuditChunk( CHUNKDESC *chunkDesc, BOOL expectValid )
{
	int 		pageCount, i;
	CHUNKPAGE 	*chunkPage;
	PAGEHASH  	*pageHash;


	chunkPage = chunkDesc->chunkPage;
	pageCount = chunkDesc->pageCount;

	if(chunkPage != NULL) {
		SM_ASSERT(LEVEL_1, (pageCount+chunkDesc->descCount) > 0);
		for(i=pageCount; i>0; i--, chunkPage++) {
			BF_AuditChunkPage(chunkPage);
			SM_ASSERT(LEVEL_1, (chunkPage->chunkDesc == chunkDesc));

			if(expectValid) {
				SM_ASSERT(LEVEL_1, chunkPage->flags & PAGE_VALID);
			}
		}
	} else {
		SM_ASSERT(LEVEL_1, pageCount == 0);
	}
}
void
BF_AuditInvalidChunk( CHUNKDESC *chunkDesc)
{
	_BF_AuditChunk( chunkDesc, FALSE);
}
void
BF_AuditChunk( CHUNKDESC *chunkDesc)
{
	_BF_AuditChunk( chunkDesc, TRUE);
}

 void
BF_AuditGroup (
	register BUFGROUP	*bufGroup 
)
{

	register CHUNKDESC	*chunkDesc;
	register GROUPLINK	*groupLink;
	register LIST		*element;
	register int		charges;


	TRPRINT(TR_BF, TR_LEVEL_1, ("bufGroup:%x", bufGroup));

	CHECK_BUFGROUP_MAGIC(bufGroup);

    /*
     * Make sure the group is open.
     */
    if ((bufGroup->flags & G_OPEN) == 0) {

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

	/*
	 *	setup initial loop conditions
	 */
	element = (LIST *) bufGroup->unfixedList.succ;
	charges = 0;

    /*
	 *	Calculate the charges on the unfixed list
     */
    while (element != &(bufGroup->unfixedList))	{

		CHECK_LIST_MAGIC(element);

		/*
		 *	take action based on whether it is a
		 *	page or a chunk
		 */
		groupLink = (GROUPLINK *)(element->item);
		switch ( groupLink->type )	{

			case TYPE_PAGE:

				/*
				 *	Account for the page
				 */
				CHECK_GROUPLINK_MAGIC(groupLink);
				charges += BLOCKCOUNT(groupLink->pageHash->page2size);
				TRPRINT(TR_BF, TR_LEVEL_3, ("page unfixed:%d", 
						groupLink->pageHash->hashList.hashPid.page));
				break;

			case TYPE_CHUNK:

				/*
				 *	process the chunk
				 */
				chunkDesc = (CHUNKDESC *) element->item;
				CHECK_CHUNKDESC_MAGIC(chunkDesc);
				BF_AuditChunk(chunkDesc);
				charges += chunkDesc->bufCount;
				TRPRINT(TR_BF, TR_LEVEL_3,
						("chunk unfixed charge:%d", chunkDesc->bufCount));
				break;

			default:

				/*
				 *	this cannot happen
				 */
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
				break;
		}

		/*
		 *	Get a pointer to the next element on the list
		 */
		element = element->succ;
    }
	TRPRINT(TR_BF, TR_LEVEL_2, ("unfixed count:%d indicated:%d",
			charges, bufGroup->unfixedCharges));

	/*
	 *	check to see if the count matches
	 */
	if (charges != bufGroup->unfixedCharges)	{

		/*
		 *	print a message and die
		 */
		TRPRINT(TR_BF, TR_LEVEL_1, ("mismatch in count:%d indicated:%d",
						charges, bufGroup->unfixedCharges));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	setup initial loop conditions
	 */
	element = (LIST *) bufGroup->fixedList.succ;
	charges = 0;

    /*
	 *	Calculate the charges on the fixed list
     */
    while (element != &(bufGroup->fixedList))	{

		CHECK_LIST_MAGIC(element);

		/*
		 *	take action based on whether it is a
		 *	page or a chunk
		 */
		switch ( *((short *) element->item) )	{

			case TYPE_PAGE:

				/*
				 *	Account for the page
				 */
				CHECK_GROUPLINK_MAGIC(((GROUPLINK *) element->item));
				charges += BLOCKCOUNT(((GROUPLINK *)element->item)->pageHash->page2size);
				TRPRINT(TR_BF, TR_LEVEL_3, ("page fixed:%d fixed count:%d", 
						((GROUPLINK *)element->item)->pageHash->hashList.hashPid.page,
				((GROUPLINK *)element->item)->fixedCount));
				break;

			case TYPE_CHUNK:

				/*
				 *	process the chunk
				 */
				chunkDesc = (CHUNKDESC *) element->item;
				CHECK_CHUNKDESC_MAGIC(chunkDesc);
				charges += chunkDesc->bufCount;
				TRPRINT(TR_BF, TR_LEVEL_3, ("chunk fixed charge:%d",
						chunkDesc->bufCount));
				break;

			default:

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
				break;
		}

		/*
		 *	Get a pointer to the next element on the list
		 */
		element = element->succ;
    }
	TRPRINT(TR_BF, TR_LEVEL_2, ("fixed count:%d indicated:%d", charges, bufGroup->fixedCharges));

	/*
	 *	check to see if the count matches
	 */
	if (charges != bufGroup->fixedCharges)	{

		/*
		 *	print a message and die
		 */
		TRPRINT(TR_BF, TR_LEVEL_1, ("mismatch in count:%d indicated:%d",
						charges, bufGroup->unfixedCharges));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
}

#endif

 BOOL 
bf_FreeUnfixedGroup (
	BUFGROUP	*bufGroup,
	BOOL		discardPages
)
{
	GROUPLINK		*groupLink;
	register BOOL	freedAnything;

	TRPRINT(TR_BF, TR_LEVEL_1, ("flushing group:%d", bufGroup->index));

	CHECK_BUFGROUP_MAGIC(bufGroup);

	/*
	 * it is assumed that you unfixed everything before calling this
	 */
	/*
	 *	the above comment is not true when called from 
	 *	bf_ReadChunk.c , line 278
	 *	so we comment out the following assert 	
		SM_ASSERT(LEVEL_1, LIST_EMPTY(&(bufGroup->fixedList)));
	*/


    /*
     * Free up the regions on the unfixed list.
     */
	while (LIST_NOT_EMPTY( &(bufGroup->unfixedList) ))	{
		groupLink = (GROUPLINK *) listDeq( &(bufGroup->unfixedList) );
		if (groupLink->type == TYPE_PAGE)	{
			BF_FreePage(bufGroup, groupLink, discardPages);
		} else if (groupLink->type == TYPE_CHUNK)	{
			BF_FreeChunk(bufGroup, (CHUNKDESC *) groupLink);
		} else {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
		freedAnything = TRUE;
	}
	return freedAnything;
}

/*============================================================================*/
/* 
 * bf_CloseGroup:
 * 	This routine closes a group and returns it to the list of free groups.
 *
 * SIDE EFFECTS:
 *	The BufGroup structure is put on the list of free groups.
 *	The buffer regions in the group's unfixed list are freed.
 *	If the region being freed is a slotted page, then the page's
 *	list of user descriptors is freed.
 */
/*============================================================================*/


 int
bf_CloseGroup (
	BUFGROUP	*bufGroup
)
{
	return BF_CloseGroup(bufGroup, FALSE);
}

 int
bf_DiscardGroup (
	BUFGROUP	*bufGroup
)
{
	return BF_CloseGroup(bufGroup, TRUE);
}

 int
BF_CloseGroup (

	BUFGROUP	*bufGroup, 
	BOOL		discard
)
{
	
	TRACE(TR_BF, TR_LEVEL_1);

	CHECK_BUFGROUP_MAGIC(bufGroup);

    /*
     * Make sure there are no fixed regions in the group.
     */
    if (bufGroup->fixedCharges > 0) {

		TRPRINT(TR_BF, TR_LEVEL_2, ("fixedCharges:%d", bufGroup->fixedCharges));
		SM_ERROR(TYPE_WARNING, esmFIXEDPAGES);
		return(esmFAILURE);
    }

	(void) bf_FreeUnfixedGroup(bufGroup, discard);

	/*
	 *	add to the number of buffers available
	 */
	ReservedBufs -= bufGroup->maxCharges;

	/*
	 *	free the structure
	 */
	bufGroup->flags = G_FREE;
	listMoveEnq( &BufGroupPool, &(bufGroup->groupList) );

    return(esmNOERROR);
}

/*============================================================================*/
/* 
 *	bf_OpenGroup:
 *		This allocates and initializes a buffer group.
 *
 *	RETURNS:
 *		a pointer to the group allocated.
 *
 *	ERROR RETURNS:
 *		esmNOFREEGROUPS    - out of free buffer groups.
 *		esmNOUNRESERVED    - not enough unreserved buffer frames.
 *
 *	SIDE EFFECTS:
 *		A buffer group is pulled off the list of free groups
 *		and then initialized.
 *
 *	IMPORTS: none
 *
 *	BUGS:
 *
 *
 * The use of all the globals below should be clear except ReservedBufs.
 * ReservedBufs reflects the number of buffers that have been 
 * reserved by bf_OpenGroup() calls.
 * When a group is opened, a specified number of buffers is reserved 
 * for that group, so that when buffers are requested for the group,
 * there should be enough free buffers to satisfy the request.
 * If ReservedBufs is smaller than the number requested in an
 * bf_OpenGroup() call, then bf_OpenGroup() returns an error.
 */
/*============================================================================*/


 BUFGROUP
*bf_OpenGroup (

	int				maxCharges,
	int				replacePolicy,
	FLAGS			flags 
)
{

    register BUFGROUP    *bufGroup;


	TRPRINT(TR_BF, TR_LEVEL_1, ("maxCharges:%d replacePolicy:%d", maxCharges, replacePolicy));

	/*
	 *	check to see that the replacement policy is sane
	 */
	if ((replacePolicy != BF_LRU) && (replacePolicy != BF_MRU))	{

		SM_ERROR(TYPE_USER, esmBADREPLACEPOLICY);
		return(NULL);
	}

    /*
	 *	Check to see if there are sufficient buffers to fill the request
     */
    if (maxCharges + ReservedBufs > NumBuffers) {

		SM_ERROR(TYPE_WARNING, esmNOUNRESERVED);
		return(NULL);
    }

    if ((bufGroup = (BUFGROUP *) listDeq( &BufGroupPool )) == NULL)	{

		SM_ERROR(TYPE_WARNING, esmNOFREEGROUPS);
		return(NULL);
	}

    /*
     * Initialize all the fields in the allocated entry.
     */
    bufGroup->flags			 = G_OPEN;
    bufGroup->replacePolicy	 = replacePolicy;
    bufGroup->maxCharges	 = maxCharges;
    bufGroup->fixedCharges	 = 0;
    bufGroup->unfixedCharges = 0;

	if (flags & TRANS_GROUP)	{

		bufGroup->flags |= G_TRANS;
	}

	/*
	 *	put the group on the used list
	 */
	listEnq( &UsedBufGroups, &(bufGroup->groupList) );

	/*
	 *	increment the buffers in use
	 */
    ReservedBufs += maxCharges;

	/*
	 *	return a pointer to the allocated group
	 */
    return(bufGroup);
}

/*
 * bf_ShutGroup really should be called bf_UnfixAndCloseGroup()
 * since that is what it does.
 */
 void
bf_ShutGroup (

	BUFGROUP	*bufGroup 
)
{
	bf_UnfixGroup(bufGroup);
	
	if (bf_CloseGroup(bufGroup) != esmNOERROR) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
}

 void
bf_UnfixGroup (

	BUFGROUP	*bufGroup 
)
{
	register GROUPLINK	*groupLink;

	TRACE(TR_BF, TR_LEVEL_1);

	CHECK_BUFGROUP_MAGIC(bufGroup);

    /*
     * Unfix everything on the fixed list
     */
	while (LIST_NOT_EMPTY( &(bufGroup->fixedList) ))	{

		groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->fixedList) );

		if (groupLink->type == TYPE_PAGE)	{

			CHECK_GROUPLINK_MAGIC(groupLink);

			bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
		
		} else if (groupLink->type == TYPE_CHUNK)	{

			CHECK_CHUNKDESC_MAGIC((CHUNKDESC *) groupLink);

			bf_UnfixChunk((CHUNKDESC *) groupLink, BF_DEFAULT);

		} else {

			/*
			 *	this should not happen
			 */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	}
}

