/*
 *   $RCSfile: bf_HackChunkPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"

/*
 *	This function discards all the chunks that reference page pid
 *	(and have the matching rootPid -- no longer true), except the chunk 
 *  that is calling this routine (if "all" is tru).  This routine
 *  assumes that the current chunk is the last on the list.  
 */

 void
bf_HackChunkPage (

	PID		*pid,
	BOOL	all
)
{

	PAGEHASH	*pageHash;
	CHUNKDESC	*chunkDesc;
	int			i;
	int			skip = all ? 0 : 1;	

	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d", pid->page));

	/*
	 *	Check to see if the page is resident
	 */
	if ((pageHash = bf_FindPage(pid)) != NULL)	{
		SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));

		/*
		 *	check the pageHash magic
		 */
		CHECK_PAGEHASH_MAGIC(pageHash);
		TRPRINT(TR_BF, TR_LEVEL_2, ("%d chunks using page", pageHash->memberCount));

		/*
		 *	get rid of all the chunks that reference these pages
		 *	except the chunk that is calling this routine.  This
		 *	routine assumes that the current chunk is the last
		 *	on the list.
		 */
		for (i = pageHash->memberCount-skip; i > 0; i--)	{

			/*
			 *	extract the chunk descriptor pointer
			 */
			chunkDesc = ((CHUNKPAGE *) pageHash->memberList.succ->item)->chunkDesc;

			/*
			 *	Check the chunk descriptor magic
			 */
			CHECK_CHUNKDESC_MAGIC(chunkDesc);

			/*
			 *	if the chunk is fixed we cannot allow this to happen
			 */
			if (chunkDesc->flags & CHUNK_FIXED)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

#ifdef JUNK
			/*
			 *	Make sure the chunk belongs to the object with root
			 *	page == rootPid before deleting it.
			 */
			if ( PIDEQ(chunkDesc->rootPid, *rootPid) &&
				 (chunkDesc->rootSlot == rootSlot) ) {
#endif JUNK
				/*
				 *	check to see if the block was used
				 */
				if (chunkDesc->flags & CHUNK_USED)	{
					
					TRPRINT(TR_BF, TR_LEVEL_2, ("chunk is used"));

					/*
					 *	invalidate the referenced chunk
					 */
					BF_InvalidateChunk(chunkDesc);

					/*
					 *	take the chunk off the bufgroup
					 */
					listRemove( &(chunkDesc->memberList) );

					/*
					 *	decrement the number of buffers in the group
					 */
					chunkDesc->bufGroup->unfixedCharges -= chunkDesc->bufCount;

					/*
					 *	we must free the block
					 */
					BF_FreeBlock(chunkDesc);
				
				} else {

					TRPRINT(TR_BF, TR_LEVEL_2, ("chunk is valid"));

					/*
					 *	just invalidate the chunk
					 */
					BF_InvalidateChunk(chunkDesc);
				}
#ifdef JUNK
			}
#endif JUNK
		}
	}
}
