/*
 *   $RCSfile: bf_Init.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "sm_params.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "util_funcs.h"
#include "bf_globals.h"
#include "sm_globals.h"


 int
BF_InitSpace ()
{
	TRACE(TR_BF, TR_LEVEL_1);
	
	SM_ASSERT(LEVEL_3,	NumBuffers > 4);;
	SM_ASSERT(LEVEL_3,	NumBufGroups > 3);;
	SM_ASSERT(LEVEL_3,	NumUserDesc > 0);;

	NumGroupLink =		  NumBuffers * 2;
	TRPRINT(TR_BF, TR_LEVEL_2, ("NumGroupLink:%d", NumGroupLink));

	NumChunkDesc =		  NumBuffers;
	TRPRINT(TR_BF, TR_LEVEL_2, ("NumChunkDesc:%d", NumChunkDesc));

	PageHashPoolSize =		  (NumBuffers * 3) / 2;
	TRPRINT(TR_BF, TR_LEVEL_2, ("PageHashPoolSize:%d", PageHashPoolSize));

	PageHashTableSize =	  1 << (findLog2(NumBuffers) + 1);
	TRPRINT(TR_BF, TR_LEVEL_2, ("PageHashTableSize:%d", PageHashTableSize));

	ObjectHashTableSize = 1 << (findLog2(NumUserDesc) + 1);
	TRPRINT(TR_BF, TR_LEVEL_2, ("ObjectHashTableSize:%d", ObjectHashTableSize));
	SuspendListSize =	  60;
	NumScanDesc =		  10;
	return esmNOERROR;
}

 int
BF_InitGroups ()
{
    int 		i;		

	TRACE(TR_BF, TR_LEVEL_1);

	GET(BufGroupPtr, BUFGROUP, NumBufGroups);

	initializeList( &BufGroupPool );
	initializeList( &UsedBufGroups );

    for (i = 0; i < NumBufGroups; i++) {
		initializeList( &(BufGroupPtr[i].fixedList) );
		initializeList( &(BufGroupPtr[i].unfixedList) );
		initializeListElement( &(BufGroupPtr[i].groupList), 
			(char *) &(BufGroupPtr[i]) );
		initializeListElement( &(BufGroupPtr[i].transList), 
			(char *) &(BufGroupPtr[i]) );
		BufGroupPtr[i].flags = G_FREE;

		listEnq( &BufGroupPool, (&BufGroupPtr[i].groupList) );

		/*
		 *	Record the index of this entry
		 *	for use in index->ptr conversion
		 */
		BufGroupPtr[i].index = i;	
		INIT_BUFGROUP_MAGIC( &(BufGroupPtr[i]) );
    }

	GET(GroupLinkPtr, GROUPLINK, NumGroupLink);

	initializeList( &GroupLinkPool );

	for (i = 0; i < NumGroupLink; i++)	{
		initializeListElement( &(GroupLinkPtr[i].hashList), 
			(char *) &(GroupLinkPtr[i]) );
		initializeListElement( &(GroupLinkPtr[i].groupList), 
			(char *) &(GroupLinkPtr[i]) );
		GroupLinkPtr[i].bufGroup = NULL;
		GroupLinkPtr[i].fixedCount = 0;
		GroupLinkPtr[i].pageHash = NULL;
		GroupLinkPtr[i].type = TYPE_PAGE;
		INIT_GROUPLINK_MAGIC( &(GroupLinkPtr[i]) );

		listEnq( &(GroupLinkPool), &(GroupLinkPtr[i].hashList) );
	}
	return esmNOERROR;
}

 int
BF_InitObjectHash ()
{
	int		i;

	TRACE(TR_BF, TR_LEVEL_1);
	GET(ObjectHashTable, LIST, ObjectHashTableSize);
	for (i = 0; i < ObjectHashTableSize; i++)	{
		initializeList( &(ObjectHashTable[i]) );
	}

	ObjectHashMask = ObjectHashTableSize - 1;
	return esmNOERROR;
}

static LIST *SuspendListMalloc = NULL;

 int
BF_InitUserDesc ()
{
	int		i;

	TRACE(TR_BF, TR_LEVEL_1);
	GET(UserDescTable, USERDESC, NumUserDesc);

	initializeList( &UserDescPool );
	for (i = 0; i < NumUserDesc; i++)	{
	/* recycle the user descriptor */
		UserDescTable[i].flags = USERDESC_FREE; 
		UserDescTable[i].type = TYPE_FREE; 
		initializeListElement( &(UserDescTable[i].userList), 
			(char *) &(UserDescTable[i]) );
		initializeListElement( &(UserDescTable[i].oidList), 
			(char *) &(UserDescTable[i]) );
		initializeListElement( &(UserDescTable[i].transList), 
			(char *) &(UserDescTable[i]) );
		INIT_USERDESC_MAGIC( &(UserDescTable[i]) );
		listEnq( &(UserDescPool), &(UserDescTable[i].userList) );
	}

	GET(SuspendListMalloc, LIST, SuspendListSize);

	initializeList( &SuspendListPool );
	for (i = 0; i < SuspendListSize; i++)	{
		initializeListElement( &(SuspendListMalloc[i]), (char *) &(SuspendListMalloc[i]) );
		listEnq( &(SuspendListPool), &(SuspendListMalloc[i]) );
	}
	return esmNOERROR;
}

 int
BF_InitPageBufs ()
{
	int		i;

	TRACE(TR_BF, TR_LEVEL_1);
	GET(PageInfoPtr, PAGEINFO, NumBuffers);

	initializeList( &PageInfoPool );
	for (i = 0; i < NumBuffers; i++)	{
		initializeListElement( &(PageInfoPtr[i].userList), 
			(char *) &(PageInfoPtr[i]) );
		listEnq( &PageInfoPool, &(PageInfoPtr[i].userList) );
	}
	return esmNOERROR;
}

 int
BF_InitChunks ()
{
	int		i;

	TRACE(TR_BF, TR_LEVEL_1);

	GET(ChunkDescPtr, CHUNKDESC, NumChunkDesc);

	initializeList( &ChunkDescPool );
	for (i = 0; i < NumChunkDesc; i++)	{
		initializeListElement( &(ChunkDescPtr[i].memberList), 
			(char *) &(ChunkDescPtr[i]) );
		ChunkDescPtr[i].type = TYPE_CHUNK;
		ChunkDescPtr[i].flags = CHUNK_INVALID;
		ChunkDescPtr[i].dirtyPages = 0;
		ChunkDescPtr[i].chunkPage = NULL;
		INIT_CHUNKDESC_MAGIC( &(ChunkDescPtr[i]) );
		listEnq( &ChunkDescPool, &(ChunkDescPtr[i].memberList) );
	}
	return esmNOERROR;
}

 int
bf_Init()
{
	static PFI initfuncs[] = {
		BF_InitSpace,
		BF_InitGroups,
		BF_InitPageHash,
		BF_InitObjectHash,
		BF_InitUserDesc,
		BF_InitPageBufs,
		BF_InitBuffers,
		BF_InitChunks,
		(PFI)0
	};
	PFI *func;

	TRACE(TR_BF, TR_LEVEL_1);

	for(func = initfuncs; *func != (PFI) NULL; func++) {
		TRACE(TR_BF, TR_LEVEL_1);
		if( (*(*func))() != esmNOERROR ) 
			return esmFAILURE;
	}
	TRACE(TR_BF, TR_LEVEL_1);
	return esmNOERROR;
}

void
BF_CloseAllBufGroups()
{
	int i;

	if(BufGroupPtr == NULL) return;

    for (i = 0; i < NumBufGroups; i++) {
		if( BufGroupPtr[i].flags & G_OPEN ) 
			bf_DiscardGroup( &BufGroupPtr[i] );
			/* throw away the pages while we're at it */
    }
}

 void 
bf_Shutdown ()
{
	TRACE(TR_BF, TR_LEVEL_1);

	BF_CloseAllBufGroups();
	BF_DeleteHashTable();
	SM_FREE_PTR(GroupLinkPtr);
	SM_FREE_PTR(BufGroupPtr);
	SM_FREE_PTR(PageInfoPtr);
	SM_FREE_PTR(MallocBufferPool);
	SM_FREE_PTR(BufInfoTable);
	SM_FREE_PTR(ChunkDescPtr);
	SM_FREE_PTR(UserDescTable);
	SM_FREE_PTR(ObjectHashTable);
	SM_FREE_PTR(FreeBufTable);
	SM_FREE_PTR(SuspendListMalloc);
}
