/*
 *   $RCSfile: bf_InvalidatePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "volume.h"
#include "load.h"
#include "scan.h"
#include "sm_params.h"
#include "btree.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "lm_extfuncs.h"


 void
bf_InvalidatePage (

	register PAGEHASH	*pageHash, 
	BOOL				 removeFromHT, /* hash table */
	BOOL				forceToServer
)
{

	TRPRINT(TR_BF, TR_LEVEL_1, ("index:%d page: %d size:%d",
			pageHash->pageInfo->bufIndex, 
			pageHash->hashList.hashPid.page,
			BLOCKCOUNT(pageHash->page2size)));

	SM_ASSERT(LEVEL_1, (pageHash->memberCount == 0));
	/*
	 *	check to see if the page is valid  (in the hash table) 
	 */
	if (LIST_MEMBER( &(pageHash->hashList.hashList) ))	{ 

		TRPRINT(TR_BF, TR_LEVEL_2, ("page is valid"));

		if(forceToServer)
			bf_ForcePage(pageHash, NOFLAGS); /* cleans page */
		else 
			bf_CleanPage(pageHash);

		/* check to make sure it is not on the dirty list */
		SM_ASSERT(LEVEL_3, NOT_LIST_MEMBER( &(pageHash->dirtyList) ));

		/* 
		 * For inter-transaction caching of pages, we leave the 
		 * page in the cache.  It's still considered invalid because
		 * it has (will soon have) no lock.  There are some times
		 * when it makes no sense to leave the page in the hash table, like
		 * when we are destroying a file, or aborting a transaction.
		 *
		 * For now, we do inter-tx page caching ONLY for small objects.
		 * When we add large objects, we have to 
		 * 	1) carefully check all places that large object 
		 * 		code does FindHashEntry and modify it to check for 
		 *      H_INVALID
		 *  2) worry about either invalidating or 
		 * 		updating chunks (see bf_ReadChunk.c)
		 */
		if( (removeFromHT==FALSE) && 
			(PPAGE_TYPE(pageHash) == PAGE_SLOTTED) ) {

			pageHash->flags |= H_INVALID;

			sm_AuditSlotPage((SLOTTEDPAGE *)pageHash->pageInfo->bufFrame);
		} else {
			BF_RemovePageFromHT(pageHash);
		}

		/*
		 *	invalidate cached lock (if any)
		 */
		if (pageHash->flags & UNLOCKABLE_PAGE)  {
			lm_InvalidateLock(&(pageHash->hashList.hashPid));
			/* calls us again (bf_InvalidatePage), but that is
			 * ok because by then, this page will be out of the
			 * hash table, and bf_InvalidatePage will return immediately.
			 */
		} else {
			/* invalidate our local copy of the lock */
			SET_PAGEHASHLOCK(pageHash, NL);
		}
	}
}
