/*
 *   $RCSfile: bf_PrintFixed.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


 void
bf_PrintUnfixedBG (
	BUFGROUP	*bufGroup 
)
{

	register GROUPLINK	*groupLink;
	register CHUNKDESC	*chunkDesc;
	register CHUNKPAGE	*chunkPage;
	register int		i;

	
	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	Make sure that the group is valid.
     */
	CHECK_BUFGROUP_MAGIC(bufGroup);

    /*
     *	Print the calculated number of pages in the group
     */
	TRPRINT(TR_BF, TR_LEVEL_2, ("unfixedCharges:%d", bufGroup->unfixedCharges));

	/*
	 *	get a pointer to the first element on the unfixed list
	 */
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->unfixedList) );

    /*
     *	Walk the regions on the unfixed list.
     */
	while (groupLink != NULL)	{

		/*
		 *	check to see if this is a page or chunk
		 */
		if (groupLink->type == TYPE_PAGE)	{

			/*
			 *	check the grouplink magic
			 */
			CHECK_GROUPLINK_MAGIC(groupLink);

			/*
			 *	print the page information
			 */
			TRPRINT(TR_BF, TR_LEVEL_2, ("page:%d", groupLink->pageHash->hashList.hashPid));

			/*
			 *	get the pointer to the next entry
			 */
			groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
		
		} else if (groupLink->type == TYPE_CHUNK)	{

			/*
			 *	get a pointer to the chunk descriptor
			 */
			chunkDesc = (CHUNKDESC *) groupLink;

			/*
			 *	check the magic number
			 */
			CHECK_CHUNKDESC_MAGIC(chunkDesc);

			/*
			 *	get a pointer to the first page
			 */
			chunkPage = chunkDesc->chunkPage;

			/*
			 *	print the chunk page list information
			 */
			for (i = 0; i < chunkDesc->pageCount; i++, chunkPage++)	{

				/*
				 *	print the entry
				 */
				TRPRINT(TR_BF, TR_LEVEL_2, ("chunkpage:%d", chunkPage->pid.page));
			}

			/*
			 *	get the pointer to the next entry
			 */
			groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );

		} else {

			/*
			 *	Unknown type
			 */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	}
}

 void
bf_PrintFixedBG (

	BUFGROUP	*bufGroup 
)
{
	register GROUPLINK	*groupLink;
	register CHUNKDESC	*chunkDesc;
	register CHUNKPAGE	*chunkPage;
	register int		i;

	
	TRACE(TR_BF, TR_LEVEL_1);

	CHECK_BUFGROUP_MAGIC(bufGroup);

	TRPRINT(TR_BF, TR_LEVEL_2, ("fixedCharges:%d", bufGroup->fixedCharges));
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->fixedList) );

	while (groupLink != NULL)	{
		if (groupLink->type == TYPE_PAGE)	{
			CHECK_GROUPLINK_MAGIC(groupLink);

			TRPRINT(TR_BF, TR_LEVEL_2, ("page:%d", groupLink->pageHash->hashList.hashPid));

			groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
		} else if (groupLink->type == TYPE_CHUNK)	{
			chunkDesc = (CHUNKDESC *) groupLink;

			CHECK_CHUNKDESC_MAGIC(chunkDesc);
			chunkPage = chunkDesc->chunkPage;

			for (i = 0; i < chunkDesc->pageCount; i++, chunkPage++)	{
				TRPRINT(TR_BF, TR_LEVEL_2, ("chunkpage:%d", chunkPage->pid.page));
			}

			groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );
		} else {

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	}
}


void
bf_Stats(int *openGroups, int *fixedCharges, BOOL print )
{
	extern BUFGROUP *BufGroupPtr;
	extern int NumBufGroups;
	int totalFixed;
	int i;
	BUFGROUP *bufGroup;

	if(BufGroupPtr == NULL) {
		*openGroups =0;
		*fixedCharges = 0;
		if(print) fprintf(stderr, "client library not initialized\n");
		return;
	}
    for ( *openGroups = 0, totalFixed = 0, i = 0; i < NumBufGroups; i++) {
		bufGroup = &  BufGroupPtr[i];
		if( bufGroup->flags & G_OPEN )  {
			BF_AuditGroup(bufGroup);
			(*openGroups) ++;
			{
				register LIST		*element;
				register int		charges = 0;

				if(print) fprintf(stderr, "Buffer group %d:\n",i);
				/*
				 *	setup initial loop conditions
				 */
				element = (LIST *) bufGroup->fixedList.succ;
				while (element != &(bufGroup->fixedList))	{
					CHECK_LIST_MAGIC(element);

					switch ( *((short *) element->item) )	{
						case TYPE_PAGE: {
							int charge;
							PAGEHASH *pageHash;

							CHECK_GROUPLINK_MAGIC(((GROUPLINK *) element->item));
							pageHash = ((GROUPLINK *)element->item)->pageHash;
							charge = BLOCKCOUNT(pageHash->page2size);
							if(print) fprintf(stderr, 
					"(%d charge for PAGE %d.%d, flags 0x%x fixedCount %d)\n", 
									charge,
									pageHash->hashList.hashPid.page,
									pageHash->hashList.hashPid.volid,
									pageHash->flags,
									((GROUPLINK *)element->item)->fixedCount);
							charges += charge;
							}
							break;

						case TYPE_CHUNK: {
								register CHUNKDESC	*chunkDesc;
								chunkDesc = (CHUNKDESC *) element->item;
								CHECK_CHUNKDESC_MAGIC(chunkDesc);
								charges += chunkDesc->bufCount;
								TRPRINT(TR_BF, TR_LEVEL_3, 
								("chunk fixed charge:%d", chunkDesc->bufCount));

								if(print) fprintf(stderr, 
	"(%d charge for CHUNK) (bufGroup index %d, flags 0x%x, policy %d)\n", 
									chunkDesc->bufCount,
									chunkDesc->bufGroup->index,
									chunkDesc->bufGroup->flags,
									chunkDesc->bufGroup->replacePolicy);
							}
							break;

						default:
							SM_ERROR(TYPE_FATAL, esmINTERNAL);
							break;
					}
					element = element->succ;
				}
				totalFixed += charges;
				if(print) fprintf(stderr, 
				"Buffer group %d has %d fixed pages\n", i, charges);
			}
		} /*if */
	} /* for */
	(*fixedCharges) = totalFixed;
}

int
bf_CountFixed()
{
	int og, fc;
	bf_Stats(&og, &fc, FALSE);
	return fc;
}

void
bf_PrintFixed()
{
	int og, fc;
	bf_Stats(&og, &fc, TRUE);
}

