/*
 *   $RCSfile: bf_PurgeFilePages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 void 
BF_GroupFidFlush (

	register BUFGROUP	*bufGroup,
	register FID		*fid 
)
{

	register GROUPLINK	*groupLink;
	SLOTTEDPAGE			*slottedPage;
	GROUPLINK			*tempLink;
	CHUNKDESC			*chunkDesc;


	TRPRINT(TR_BF, TR_LEVEL_1, ("flushing group:%d", bufGroup->index));

	/*
	 *	check the buffer group magic number
	 */
	CHECK_BUFGROUP_MAGIC(bufGroup);
	TRPRINT(TR_BF, TR_LEVEL_2, ("buffer group:%d", bufGroup->index));

	/*
	 *	remove a buffer from the list
	 */
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->unfixedList) );

	/*
	 *	run down the list
	 */
	while (groupLink != NULL)	{

		/*
		 *	check to see if this is a page or chunk
		 */
		if (groupLink->type == TYPE_PAGE)	{

			/*
			 *	check the grouplink magic
			 */
			CHECK_GROUPLINK_MAGIC(groupLink);

			/*
			 *	get the pointer to the next entry
			 */
			tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );

			/*
			 *	check to see if the page is slotted
			 */
			if (groupLink->pageHash->flags & H_SLOTTED)	{

				TRPRINT(TR_BF, TR_LEVEL_2, ("page is slotted"));

				/*
				 *	get a pointer to the page
				 */
				slottedPage = (SLOTTEDPAGE *) groupLink->bufFrame;

				/*
				 *	check to see if the fid matches
				 */
				if ((slottedPage->header.fid.pid.volid == fid->pid.volid) &&
					(slottedPage->header.fid.pid.page == fid->pid.page) &&
					(slottedPage->header.fid.unique == fid->unique))	{

					/*
					 *	take the link off the list
					 *	NOTE: Maybe it would be better to mark
					 *		  the page clean as well?
					 */
					listRemove( &(groupLink->groupList) );

					/*
					 *	Free the page
					 */
					BF_FreePage(bufGroup, groupLink, FALSE);
				}
			}

			/*
			 *	setup the pointer to the next entry
			 */
			groupLink = tempLink;

		} else if (groupLink->type == TYPE_CHUNK)	{

			/*
			 *	get a chunk descriptor pointer
			 */
			chunkDesc = (CHUNKDESC *) groupLink;

			/*
			 *	check the chunkdescriptor magic
			 */
			CHECK_CHUNKDESC_MAGIC(chunkDesc);

			/*
			 *	save the pointer to the next link
			 */
			tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );

			/*
			 *	check to see if the fid matches
			 */
			if ((chunkDesc->fid.pid.volid == fid->pid.volid) &&
				(chunkDesc->fid.pid.page == fid->pid.page) &&
				(chunkDesc->fid.unique == fid->unique))	{

				/*
				 *	take the chunk off the list
				 */
				listRemove( &(chunkDesc->memberList) );

				/*
				 *	free the chunk
				 */
				BF_FreeChunk(bufGroup, chunkDesc);
			}

			/*
			 *	put the link back
			 */
			groupLink = tempLink;

		} else {

			/*
			 *	this should not happen
			 */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	}
}


/*
 * Get rid of pages in a file.
 * Don't leave them around for reuse.
 */
 void 
bf_PurgeFilePages (

	register FID		*fid 
)
{
    register PAGEHASH   *pageHash;
	register GROUPLINK	*groupLink;
	register CHUNKDESC	*chunkDesc;
	SLOTTEDPAGE			*slottedPage;
	BUFGROUP			*bufGroup;
	int					i;

	
	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	get a pointer to the first in use buffer group
	 */
	bufGroup = (BUFGROUP *) FIRST_LIST_ELEMENT( &UsedBufGroups );

	/*
	 *	run through the open buffer group list
	 *	to see if there are any fixed pages
	 */
	while (bufGroup != NULL)	{

		/*
		 *	check the buffer group magic number
		 */
		CHECK_BUFGROUP_MAGIC(bufGroup);
		TRPRINT(TR_BF, TR_LEVEL_2, ("buffer group:%d", bufGroup->index));

		/*
		 *	free the pages in the group
		 */
		BF_GroupFidFlush(bufGroup, fid);

		/*
		 *	get a pointer to the next buffer group
		 */
		bufGroup = (BUFGROUP *) NEXT_LIST_ELEMENT( &(bufGroup->groupList) );
	}

	/*
	 *	loop through the free lists
	 */
	for (i = 0; i < NumBufferSizes; i++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("processing list:%d", i));
		if (LIST_NOT_EMPTY( &FreeBufTable[i] ))	{

			groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(FreeBufTable[i]) );

			while (groupLink != NULL)	{

				if (groupLink->type == TYPE_PAGE)	{

					CHECK_GROUPLINK_MAGIC(groupLink);
					pageHash = groupLink->pageHash;
					CHECK_PAGEHASH_MAGIC(pageHash);

					TRPRINT(TR_BF, TR_LEVEL_2, ("checking page:%d %d",
							pageHash->hashList.hashPid.page,
							pageHash->hashList.hashPid.volid));

					if (pageHash->flags & H_SLOTTED)	{

						TRPRINT(TR_BF, TR_LEVEL_2, ("page is slotted"));
						slottedPage = (SLOTTEDPAGE *) pageHash->pageInfo->bufFrame;
						if ((slottedPage->header.fid.pid.volid == fid->pid.volid) &&
							(slottedPage->header.fid.pid.page == fid->pid.page) &&
							(slottedPage->header.fid.unique == fid->unique))	{

							bf_InvalidatePage(pageHash, TRUE, FALSE);
						}
					}

					groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );

				} else if (groupLink->type == TYPE_CHUNK)	{

					chunkDesc = (CHUNKDESC *) groupLink;
					CHECK_CHUNKDESC_MAGIC(chunkDesc);

					if ((chunkDesc->fid.pid.volid == fid->pid.volid) &&
						(chunkDesc->fid.pid.page == fid->pid.page) &&
						(chunkDesc->fid.unique == fid->unique))	{

						if (CHUNK_IS_VALID(chunkDesc))	{
							BF_DiscardChunk(chunkDesc);
						}
					}
					groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );
				} else {
					SM_ERROR(TYPE_FATAL, esmINTERNAL);
				}
			}
		}
	}
}
