/*
 *   $RCSfile: bf_ScanPage.c,v $
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:15 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "volume.h"
#include "trans.h"
#include "lsn.h"
#include "loginfo.h"
#include "clog.h"
#include "lm.h"
#include "scan.h"
#include "fi_msg.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "lm_extfuncs.h"
#include "bf_globals.h"
#include "trans_globals.h"
#include "clog_globals.h"
#include "lock_globals.h"
#include "sm_globals.h"



 GROUPLINK 
*bf_ScanPage (
	BUFGROUP			*bufGroup,
	const PID    	    *pid,
	SCANTYPE			flags,
	PID					*neighborPid,
	FID					*fid,
	NEIGHBORFLAGS		direction
)
{
	GROUPLINK			*groupLink;
	BOOL				simpleReadPage; /* no need to BF_ReadPageWithNeighbor */

	SM_ASSERT(LEVEL_2, (((flags & LOGICAL_SCAN)==LOGICAL_SCAN) ||
							((flags & PHYSICAL_SCAN)==PHYSICAL_SCAN)) );

	simpleReadPage = ((flags & LOGICAL_SCAN)==LOGICAL_SCAN);
	if(simpleReadPage) {
		groupLink = bf_ReadPage(bufGroup, pid, 
				PAGE_SLOTTED, SLOTTED_PAGE2SIZE, NL);
	} else {
		SM_ASSERT(LEVEL_1, fid != NULL);
		SM_ASSERT(LEVEL_1, neighborPid != NULL);
		groupLink = BF_ReadPageWithNeighbor(bufGroup, pid, 
				PAGE_SLOTTED, SLOTTED_PAGE2SIZE, NL,
				direction, fid, neighborPid);
		TRPRINT(TR_BF, TR_LEVEL_1, 
			("neighborPid %d.%d, scanType:%d", 
			neighborPid->page, neighborPid->volid, flags));
	}
	/* 
	 * if groupLink != NULL, this page is fixed in this bufGroup
	 */

	if(groupLink == NULL) 
		goto NoMore; /* to be safe */

	/* Now get the neighbor pid. */

	if( flags & LOGICAL_SCAN ) { 
		SLOTTEDPAGE 		*currentPage;

		currentPage = (SLOTTEDPAGE *) groupLink->bufFrame;

		/* the the neighbor pid from the slotted page */
		neighborPid->page  = currentPage->header.nextLogicalPid;
		neighborPid->volid = currentPage->header.pid.volid;
	} /* else: physical scan w/o scan descriptor: used BF_ReadPageWith... */

	SM_ASSERT(LEVEL_1, (neighborPid->page  == NULLPID) ||
		(neighborPid->page > 0));
	SM_ASSERT(LEVEL_1,neighborPid->volid  != 0);

	return groupLink;

NoMore:
	if(direction & NEIGHBOR_BEFORE) {
		sm_errno = esmBEFOREFIRSTPID;
	} else {
		SM_ASSERT(LEVEL_2, (direction&(NEIGHBOR_AFTER | NEIGHBOR_ANY)));
		sm_errno = esmAFTERLASTPID;
	}
	neighborPid->page = NULLPID;
	neighborPid->volid = fid->pid.volid;

	return groupLink;
}
