/*
 *   $RCSfile: bf_UnfixChunk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:13 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


 void
bf_UnfixChunk (

	register CHUNKDESC	*chunkDesc,
	int					replacePolicy 
)
{

	register BUFGROUP	*bufGroup;


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	Check the chunk descriptor
	 */
	CHECK_CHUNKDESC_MAGIC(chunkDesc);

	/*
	 *	make sure the chunk is fixed
	 */
	SM_ASSERT(LEVEL_3, chunkDesc->flags & CHUNK_FIXED);

	/*
	 *	Take the chunk off of the fixed list
	 */
	listRemove( &(chunkDesc->memberList) );

	/*
	 *	get a pointer to the holding buffer group
	 */
	bufGroup = chunkDesc->bufGroup;

	/*
	 *	put it on the unfixed list according to the replacement policy
	 */
	BF_ReplacePolicy(bufGroup, &(chunkDesc->memberList), replacePolicy);	

	/*
	 *	Decrement the fixed charges and increment the unfixed charges
	 */
	TRPRINT(TR_BF, TR_LEVEL_2, ("chunkdesc->bufCount:%d", chunkDesc->bufCount));
	bufGroup->fixedCharges   -= chunkDesc->bufCount;
	bufGroup->unfixedCharges += chunkDesc->bufCount;
	TRPRINT(TR_BF, TR_LEVEL_2, ("fixed:%d unfixed:%d",
			bufGroup->fixedCharges, bufGroup->unfixedCharges));

	/*
	 *	check the fix and unfixed count
	 */
	SM_ASSERT(LEVEL_3, (bufGroup->fixedCharges >= 0) && (bufGroup->unfixedCharges > 0));

	/*
	 *	set the flags as unfixed
	 *	There is no need to update the state space of the bufInfo
	 */
	chunkDesc->flags &= ~CHUNK_FIXED;

	/*
	 *	audit the buffer group
	 */
	BF_AuditGroup(bufGroup);
}
