/*
 *   $RCSfile: bf_UnfixPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "trans.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


extern TRANSREC		TransRec;


 void
bf_UnfixPage (

	register GROUPLINK	*groupLink,
	int					replacePolicy,
	BOOL				dirty 
)
{

	register BUFGROUP	*bufGroup;


	TRPRINT(TR_BF, TR_LEVEL_1, ("unfixing page:%d",
			groupLink->pageHash->hashList.hashPid.page));

	/*
	 *	check the magic number
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);

	/*
	 *	Check to see if the fixed count is correct
	 */
	SM_ASSERT(LEVEL_3, groupLink->fixedCount > 0);
	SM_ASSERT(LEVEL_3, groupLink->pageHash->memberCount > 0);

	/*
	 *	check to see if page needs to be dirtied
	 */
	if (dirty)	{

		bf_DirtyPage(groupLink->pageHash);
		TRPRINT(TR_BF, TR_LEVEL_2, ("marking page dirty"));
	}

	/*
	 *	put the buffer group in a register
	 */
	bufGroup = groupLink->bufGroup;

	/*
	 *	Decrement the fixed count for the group
	 */
	if (--(groupLink->fixedCount) == 0)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("putting on unfixed"));

		/*
		 *	We must take it off the fixed list
		 */
		listRemove( &(groupLink->groupList) );

		/*
		 *	Put it on the free list according to the replacement policy
		 */
		BF_ReplacePolicy(bufGroup, &(groupLink->groupList), replacePolicy);

		/*
		 *	Decrement the fixed charges and increment the unfixed charges
		 */
		bufGroup->fixedCharges   -= BLOCKCOUNT(groupLink->pageHash->page2size);
		bufGroup->unfixedCharges += BLOCKCOUNT(groupLink->pageHash->page2size);

		/*
		 *	check the fix count
		 */
		SM_ASSERT(LEVEL_3, (bufGroup->fixedCharges >= 0) || (bufGroup->unfixedCharges > 0));
	}
	TRPRINT(TR_BF, TR_LEVEL_2, ("fixed:%d unfixed:%d",
			bufGroup->fixedCharges, bufGroup->unfixedCharges));

	/*
	 *	audit the state of the group
	 */
	BF_AuditGroup(bufGroup);
}
