/*
 *   $RCSfile: bf_UpdateChunkPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "util_funcs.h"


 void
bf_UpdateChunkPage(

	register CHUNKPAGE	*homePage,
	int					start,
	int					length,
	char				*data 
)
{

	register CHUNKPAGE	*chunkPage;
	register CHUNKPAGE	*nextPage;
	register PAGEHASH	*pageHash;
	register CHUNKDESC	*chunkDesc;


	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d start:%d length:%d",
			homePage->pid.page, start, length));

	/*
	 *	get a pointer to the page hash structure
	 */
	pageHash = homePage->pageHash;

	/*
	 *	check the pageHash magic
	 */
	CHECK_PAGEHASH_MAGIC(pageHash);

	/*
	 *	check to see if we are the only chunk using the page
	 */
	if (pageHash->memberCount > 1)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("%d chunks using page", pageHash->memberCount));

		/*
		 *	set the initial value of the chunkpage
		 */
		chunkPage = (CHUNKPAGE *) FIRST_LIST_ELEMENT( &(pageHash->memberList) );

		/*
		 *	get rid of all the chunks that reference these pages
		 */
		while (chunkPage != NULL)	{

			/*
			 *	Check to make sure that the list is not empty
			 */
			SM_ASSERT(LEVEL_3, LIST_NOT_EMPTY( &(pageHash->memberList) ));

			/*
			 *	Save a pointer to the next page
			 */
			nextPage = (CHUNKPAGE *) NEXT_LIST_ELEMENT( &(chunkPage->hashList) );

			/*
			 *	check to see if this is the home page
			 */
			if (chunkPage != homePage)	{

				TRPRINT(TR_BF, TR_LEVEL_2, ("not home page"));

				/*
				 *	extract the chunk descriptor pointer
				 */
				chunkDesc = chunkPage->chunkDesc;
				CHECK_CHUNKDESC_MAGIC(chunkDesc);
				CHECK_PAGEHASH_MAGIC(chunkPage->pageHash);

				/*
				 *	if the chunk is fixed we cannot allow this to happen
				 */
				if (!(chunkDesc->flags & CHUNK_VALID))	{

					SM_ERROR(TYPE_FATAL, esmINTERNAL);
				
				} else {
					
					TRPRINT(TR_BF, TR_LEVEL_2, ("chunk is used"));

					/*
					 *	copy in the data from the write
					 */
					bcopy(data, chunkDesc->bufFrame + chunkPage->offset + start, (int) length);
					TRPRINT(TR_BF, TR_LEVEL_2, ("copying to page:%d offset:%d length:%d index:%d",
							chunkPage->pid.page, chunkPage->offset + start, length,
							chunkPage->chunkDesc->bufIndex));

					/*
					 *	dirty the page
					 */
					if (!(chunkPage->flags & PAGE_DIRTY))	{

						/*
						 *	dirty the page
						 */
						chunkPage->flags |= PAGE_DIRTY;
						chunkDesc->dirtyPages++;
						DIRTY_CHUNK(chunkDesc);
						bf_DirtyPage(chunkPage->pageHash);

					} else {

						SM_ASSERT(LEVEL_3, chunkDesc->dirtyPages > 0);
						SM_ASSERT(LEVEL_3, PAGE_IS_DIRTY(chunkPage->pageHash));
						SM_ASSERT(LEVEL_3, CHUNK_IS_DIRTY(chunkPage->chunkDesc));
					}
				}
			}

			/*
			 *	Increment the pointer to the next item
			 */
			chunkPage = nextPage;
		}

	} else {

		TRPRINT(TR_BF, TR_LEVEL_2, ("homePage:%d bufIndex:%d",
				homePage->pid.page, homePage->chunkDesc->bufIndex));
	}
}
