/*
 *   $RCSfile: bf_UpdateLogicalLink.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:14 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "bf_extfuncs.h"
#include "bf_intfuncs.h"


 int 
bf_UpdateLogicalLink (

	register BUFGROUP	*bufGroup,
	PID     		    *lastPid,
	SHORTPID			newPid 
)
{

	register PAGEHASH	*pageHash;
	register GROUPLINK	*groupLink;
	SLOTTEDPAGE			*slottedPage;


	TRPRINT(TR_SM, TR_LEVEL_1, ("lastPid:%d newPid:%d", lastPid->page, newPid));

    /*
     * Check to see if the page is in the cache.  If it's in the 
	 * cache, we must update its pointer to the next page.
	 * The page could be present, but invalid, in which case
	 * we must just remove if from the page hash table, since
	 * it might not even be a slotted page anymore (assuming the
	 * page is really not up-to-date).
     */
    pageHash = bf_FindPage(lastPid);

	if( pageHash == NULL ) {
		/* don't have to do anything */
		return(esmNOERROR);
	}

	if( pageHash->flags & H_INVALID) {
		/* Remove page from hash table. It's already on the free list.  */
		BF_RemovePageFromHT(pageHash);

	} else {
		SM_ASSERT(LEVEL_1, (PPAGE_TYPE(pageHash) == PAGE_SLOTTED));

		TRPRINT(TR_SM, TR_LEVEL_2, ("page present"));

		/*
		 *	read the page
		 */
		if ((groupLink = bf_ReadPage(bufGroup, lastPid, H_SLOTTED,
									 SLOTTED_PAGE2SIZE, EX)) == NULL)	{

			return(esmFAILURE);
		}

		/*
		 *	get a pointer to the page
		 */
		slottedPage = (SLOTTEDPAGE *) groupLink->bufFrame;

		/*
		 *	check the slotted page magic
		 */
		if (CHECK_SLOTTED_MAGIC(slottedPage))	{

			SM_ERROR(TYPE_FATAL, esmBADSLOTTEDMAGIC);
		}

		/*
		 *	fill in the new link
		 */
		slottedPage->header.nextLogicalPid = newPid;

		/*
		 *	Unfix the page, but do not dirty it (since the
		 *	update was performed and logged on the server).
		 */
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
	}

	return(esmNOERROR);
}
