/*
 *   $RCSfile: bf_dump.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:15 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "util_funcs.h"


#ifdef DEBUG

#include "bf_globals.h"

static int 
countElementsInList(
	LIST	*list,
	BOOL	printit
)
{
	int			total=0;
	GROUPLINK 	*groupLink;
	CHUNKDESC 	*chunkDesc;

    groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( list );

    while(groupLink != NULL) {
		total++;

		if (groupLink->type == TYPE_PAGE)	{
			if(printit)
			fprintf(stderr, "TYPE_PAGE, size %d page %d\n",
				groupLink->pageHash->page2size,
				groupLink->pageHash->hashList.hashPid.page);

			groupLink = (GROUPLINK *) 
				NEXT_LIST_ELEMENT( &(groupLink->groupList) );

		} else if (groupLink->type == TYPE_CHUNK)	{

			chunkDesc = (CHUNKDESC *) groupLink;

			if(printit)
			fprintf(stderr, "TYPE_CHUNK, bufCount %d totalLen %d\n",
				chunkDesc->bufCount,
				chunkDesc->totalLength);
			groupLink = (GROUPLINK *) 
				NEXT_LIST_ELEMENT( &(chunkDesc->memberList) );

		} else {

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
		
    }
	return total;
}

void
printFreeBufTable()
{
	register int i, j;
	int	maxBuf2Size, maxPage2Size, numBufferSizes;

	maxBuf2Size = findLog2(NumBuffers);
	maxPage2Size = maxBuf2Size + MIN_PAGE2SIZE;
	numBufferSizes = OFFSET(maxPage2Size) + 1;

	for (i = 0; i < numBufferSizes; i++)	{
		j = countElementsInList( (LIST *)&FreeBufTable[i], FALSE );
		if(j>0) {
			fprintf(stderr, 
			"FreeBufTable[%2d:%5d] contains %4d buffers.\n", 
				i, RAISETWO(i+MIN_PAGE2SIZE), j);
		}
	}

}

void
printUnfixedList( BUFGROUP *bufGroup)
{
	register int j;

	j = countElementsInList( (LIST *)&(bufGroup->unfixedList), TRUE );
	if(j>0) {
		fprintf(stderr, 
			"unfixed list for buf group %d contains %4d buffers.\n",
				bufGroup->index,j);
				
	}
}

void
printFixedList( BUFGROUP *bufGroup)
{
	register int j;

	j = countElementsInList( (LIST *)&(bufGroup->fixedList), TRUE );
	if(j>0) {
		fprintf(stderr, 
			"fixedList for buf group %d contains %4d buffers.\n", 
				bufGroup->index, j);
				
	}
}
#endif DEBUG
