/*
 *   $RCSfile: bf_lrcLoc.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:12 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "page.h"
#include "object.h"
#include "bf.h"
#ifndef SERVER_MAKE
#include "scan.h"
#include "load.h"
#include "chunk.h"
#endif SERVER_MAKE
#include "bf_extfuncs.h"

LRC *bf_lrcLoc(
	PAGEHASH *pageHash
)
{
	register 	char 	*bufFrame;
	register	LRC 	*pageLRC;
	register	unsigned pageType = (unsigned) PPAGE_TYPE(pageHash);

	
	/* Need the following check because on the client side, there is a
	 * pageInfo pointer between pageHash and bufFrame, and it is
	 * null for large data pages
	 */
	if(pageType != PAGE_LARGEDATA) {
		bufFrame = PBUFFRAME(pageHash);
	}

	switch (pageType)	{

		case PAGE_SLOTTED:
			pageLRC = &(((SLOTTEDPAGE*)bufFrame)->header.lrc);
			break;

		case PAGE_INDEX:
			pageLRC = (LRC*) bufFrame; /* at beginning of page */
			break;
			
		case PAGE_INDEXDESC:
			pageLRC = &(((INDEXDESCPAGE*)bufFrame)->lrc);
			break;

		case PAGE_LARGENODE:
			pageLRC = &(((LGNODE*)bufFrame)->header.lrc);
			break;

#ifdef SERVER_MAKE
		/* servers find it in the page as usual */
		case PAGE_LARGEDATA:
			pageLRC = &(((LGDATAPAGE*)bufFrame)->header.lrc);
			break;
#else
		/* client's find it in the page hash */
		case PAGE_LARGEDATA:
			pageLRC = &(pageHash->lrc);
			break;
#endif /* SERVER_MAKE */

		default:
			pageLRC = NULL;
			/* It is the responisibility of the caller
			 * to check for a null result.
			 * In some cases it is fine for the page type
			 * to be 0, for example.
			 */
			break;
	}
	return pageLRC;
}
