/*
 * $RCSfile: BT_Log.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "BT_Log.h"
#ifndef SERVER_MAKE
#   include "clog_intfuncs.h"
#   include "trans_globals.h"
#endif


#ifndef SERVER_MAKE
BOOL btLogActive = TRUE;
#endif


#ifdef SERVER_MAKE

#define lrc_unused_on_client	lrc
typedef LOGRECORDINFO           BT_LOGRECINFO;
#define GENLRC() {\
		recInfo.actionLRC = GENERATE_LRC(&lrc);\
		recInfo.actionPid = &pid;\
	}

#else SERVER_MAKE

#define lrc_unused_on_client	unused
typedef CLIENTLOGRECORDINFO     BT_LOGRECINFO;
#define GENLRC() recInfo.actionPid = pid;

#endif SERVER_MAKE

#define INITRECINFO(info, act, cnt, undoNxt)   {        \
            if (undoNxt)  {                             \
                info.type = LOG_REC_TYPE_COMPENSATION;  \
                info.flags = REDO_ONLY;                 \
                info.nextUndoLSN = undoNxt;             \
                }                                       \
            else {                                      \
                info.type = LOG_REC_TYPE_USER;          \
                info.flags = NOFLAGS;                   \
                info.nextUndoLSN = 0;                   \
                }                                       \
            info.action = act;                          \
            info.imageCount = cnt;                      \
            }
            


//
//  Buff class to ensure threads run correctly on server during
//  restart.
//
class Buff {
	static Buff* freeSpace;
	static void Replenish(size_t sz);
	Buff*& Next()       { return *((Buff**) data); }
	
public:
	char        data[BTREE_PAGESIZE];
	
	Buff()      {};
	~Buff()     {};
	
	void* operator new(size_t sz)  {
		if (!freeSpace) Replenish(sz);
		void* ret = freeSpace;
		freeSpace = freeSpace->Next();
		return ret;
	}
	void operator delete(void* p) {
		((Buff*)p)->Next() = freeSpace;
		freeSpace = (Buff*)p;
	}
};

Buff* Buff::freeSpace = 0;
const chunkSize = 1;


void Buff::Replenish(size_t sz)
{
    register Buff* p;
    
    ASSERT3(!freeSpace);
    freeSpace = p = (Buff*) new char[sz * chunkSize];
    ASSERT1(p);
    
    for (; p != &freeSpace[chunkSize-1]; p->Next() = p + 1, p++);
    p->Next() = 0;
}




void inline SetImage(BT_LOGRECINFO& recInfo, int idx, int size, void* data)
{
    recInfo.imageSize[idx] = size;
    recInfo.imageData[idx] = (VOID*) data;
}


int inline SpaceLeft()
{
#ifdef SERVER_MAKE
    return BTREE_PAGESIZE;
#else
    return GET_LOGINFO->lastUsableByte - (GET_LOGINFO->tailLSN & GET_LOGINFO->pageMask);
#endif
}



#ifndef SERVER_MAKE
/* flaky trick to make the middle argument (used only on client side) 
 * issue a warning that's easy to recognize as something we can disregard:
 */
#define unused fixedSize
#endif

static void LogIt(BT_LOGRECINFO& recInfo, int unused, PAGEHASH* pHash)

#ifndef SERVER_MAKE
#undef unused 
#endif

{

#ifdef SERVER_MAKE
    LSN lsn;
    FORCEMARK forceMark = writeLogRecord((TRANSREC*) Active->transRec, 
										 &recInfo, &lsn, NOFLAGS);
    if (forceMark < 0)  {
        SM_ERROR(TYPE_FATAL, Active->errno);
	}
	
    if (pHash)  {
        DEPEND_LOG(pHash, forceMark, &lsn, recInfo.actionLRC);
	}
#else
	
    if (! btLogActive)  return;
    
    ASSERT3(pHash != 0);
    
    BTREEPAGE* bp = (BTREEPAGE*) pHash->pageInfo->bufFrame;
    
    ASSERT3(bp->SelfID() == recInfo.actionPid);
    
    if ( ! GET_LOGINFO->tailBuffer) 
        CLOG_GetNewLogPage();
	
    LRC lrc = bp->lrc();
	
    int spaceLeft = SpaceLeft();
    
    if (fixedSize > spaceLeft)  {
        CLOG_GetNewLogPage();
        spaceLeft = GET_LOGINFO->usableBytes;
	}
    
    spaceLeft -= fixedSize;
    
    ASSERT1(spaceLeft >= 0);
	
    FORCEMARK forceMark = CLOG_WriteLogRecord(GET_SERVERTID, &recInfo, &lrc, pHash);
	
    ASSERT1(forceMark >= 0);
    
    pHash->forceMark = forceMark;
    
	
    bp->lrc() = lrc;
#endif
}


static int disabled = FALSE;



#ifdef SERVER_MAKE

void BT_LogPureCLR(LSNOFFSET undoNextLSN)
{
    BT_LOGRECINFO recInfo;
	
    recInfo.type = LOG_REC_TYPE_COMPENSATION;
    recInfo.action = LOG_ACTION_PURE_CLR;
    recInfo.imageCount = 0;
    recInfo.actionPid = 0;
    recInfo.actionLRC = 0;
    recInfo.flags = REDO_ONLY;
    recInfo.nextUndoLSN = undoNextLSN;
    
    LogIt(recInfo, ALIGNSIZE(sizeof(LOGRECORDHDR)), 0);
}


int bt_LogFormat(VOLREC*, PID* pid, PAGEHASH* pHash)
{
    TRPRINT(TR_IO, TR_LEVEL_1, ("logging pid:%d", pid->page));
	
    BTREEPAGE* btPage = (BTREEPAGE*) pHash->bufFrame;
    
    btPage->Init(*pid, 1, SM_int);
	
    LRC* lrc = & btPage->lrc();
    INITIALIZE_LRC(lrc);
	
    /*
     *  Initialize the log information
     */
    LOGRECORDINFO   recordInfo;
    recordInfo.action = LOG_ACTION_BTREE_PAGE_FORMAT;
    recordInfo.type = LOG_REC_TYPE_USER;
    recordInfo.imageCount = 0;
    recordInfo.actionPid = & (btPage->SelfID());
    recordInfo.actionLRC = lrc;
    recordInfo.flags = REDO_ONLY|REDO_ALWAYS;
    
    LogIt(recordInfo, ALIGNSIZE(sizeof(LOGRECORDHDR)), pHash);
	
    return esmNOERROR;
}

#endif



void BT_LogPageInit(PAGEHASH* pHash, TWO level, SMDATATYPE keyType)
{
    BT_LOGRECINFO recInfo;
    LogBtreePageInitData* recData = (LogBtreePageInitData*) new Buff;
    recData->newLevel = level;
    recData->keyType = keyType;
    
    recInfo.type = LOG_REC_TYPE_USER;
    recInfo.action = LOG_ACTION_BTREE_PAGE_INIT;
    recInfo.flags = REDO_ONLY;
    recInfo.imageCount = 1;
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    
    if (!disabled)  {
#ifdef SERVER_MAKE
        BTREEPAGE* bp = (BTREEPAGE*) pHash->bufFrame;
        LRC* lrc = &bp->lrc();
        GENERATE_LRC(lrc);
        recInfo.actionPid = &(bp->SelfID());
        recInfo.actionLRC = lrc;
#else
        BTREEPAGE* bp = (BTREEPAGE*) pHash->pageInfo->bufFrame;
        recInfo.actionPid = bp->SelfID();
#endif
        LogIt(recInfo, fixedSize, pHash);
	}
    
    delete (Buff*)recData;
}




void BT_LogModifyLink(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client, 
					  SHORTPID oldNext, SHORTPID newNext, 
					  SHORTPID oldPrev, SHORTPID newPrev, LSNOFFSET undoNxtLSN)
{
    LogBtreeModifyLinkData* recData = (LogBtreeModifyLinkData*) new Buff;
    recData->oldNext = oldNext;
    recData->newNext = newNext;
    recData->oldPrev = oldPrev;
    recData->newPrev = newPrev;
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_MODIFY_LINK, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled) {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    delete (Buff*) recData;
}



void BT_LogModifyLevel(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
					   TWO oldLevel, TWO newLevel, LSNOFFSET undoNxtLSN)
{
    LogBtreeModifyLevelData* recData = (LogBtreeModifyLevelData*) new Buff;
    recData->oldLevel = oldLevel;
    recData->newLevel = newLevel;
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_MODIFY_LEVEL, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled) {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    delete (Buff*) recData;
}

void BT_LogModifyVector0(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client, 
						 SHORTPID oldPid0, SHORTPID newPid0, LSNOFFSET undoNxtLSN)
{
    LogBtreeModifyVector0Data* recData = (LogBtreeModifyVector0Data*) new Buff;
    recData->oldPid0 = oldPid0;
    recData->newPid0 = newPid0;
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_MODIFY_VECTOR0, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    
    delete (Buff*) recData;
}


void BT_LogPhysicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
                        TWO slotNum, TWO numSlot, LSNOFFSET undoNxtLSN)
{

	
#ifdef SERVER_MAKE
    BTREEPAGE* bp = (BTREEPAGE*) pHash->bufFrame;
#else
    BTREEPAGE* bp = (BTREEPAGE*) pHash->pageInfo->bufFrame;
#endif
	
    ASSERT3(slotNum >= 0);
    ASSERT3(slotNum < bp->NumSlots());
    ASSERT3(numSlot > 0);
    ASSERT3(slotNum + numSlot <= bp->NumSlots());
	
    LogBtreePhysicInsertData* recData = (LogBtreePhysicInsertData*)new Buff;
	
    for (int i = 0; i < numSlot; ) {
        char* p;
        int start = i;
		
        int sz;
        for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
            sz = bp->Tuple(i+slotNum).Size();
            if (ALIGNSIZE(p - recData->data + sz) > recData->Max())  break;
            BCOPY((char*) & bp->Tuple(i+slotNum), (char*)p, sz);
		}
		
        BT_LOGRECINFO recInfo;
        INITRECINFO(recInfo, LOG_ACTION_BTREE_PHYSIC_INSERT, 1, undoNxtLSN);
        
        recData->startSlot = slotNum + start;   // different from
        recData->numSlots = i - start;          //  BT_LogPhysicDelete()
        recData->length = p - recData->data;
        
        ASSERT3(recData->numSlots > 0);
        
        SetImage(recInfo, 0, recData->Size(), recData);
		
        int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
			ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= BTREE_PAGESIZE);
		
        if (!disabled)  {
			GENLRC();
            LogIt(recInfo, fixedSize, pHash);
		}
	}
    delete (Buff*) recData;
}



#ifdef SERVER_MAKE
void BT_LogPhysicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc,
                        TWO slotNum, TWO numSlot, 
                        int length, void* buff, LSNOFFSET undoNxtLSN)
{
    ASSERT3(slotNum >= 0);
    ASSERT3(numSlot > 0);
	
    LogBtreePhysicInsertData* recData = (LogBtreePhysicInsertData*)new Buff;
    
    recData->startSlot = slotNum;
    recData->numSlots = numSlot;
    recData->length = length;
    BCOPY(buff, recData->data, length);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_PHYSIC_INSERT, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
    ASSERT3(fixedSize <= BTREE_PAGESIZE);
    
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    delete (Buff*) recData;
}
#endif


void BT_LogPhysicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
                        TWO slotNum, TWO numSlot, LSNOFFSET undoNxtLSN)
{
#ifdef SERVER_MAKE
    BTREEPAGE* bp = (BTREEPAGE*) pHash->bufFrame;
#else
    BTREEPAGE* bp = (BTREEPAGE*) pHash->pageInfo->bufFrame;
#endif
	
    ASSERT3(slotNum >= 0);
    ASSERT3(slotNum < bp->NumSlots());
    ASSERT3(numSlot > 0);
    ASSERT3(slotNum + numSlot <= bp->NumSlots());
	
    LogBtreePhysicDeleteData* recData = (LogBtreePhysicDeleteData*) new Buff;
    
    for (int i = 0; i < numSlot; ) {
        char* p;
        int start = i;
        int sz;
        for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
            sz = bp->Tuple(i+slotNum).Size();
            if (p - recData->data + sz > recData->Max())        break;
            BCOPY( (char*)& bp->Tuple(i+slotNum), p, sz);
		}
		
        BT_LOGRECINFO recInfo;
        INITRECINFO(recInfo, LOG_ACTION_BTREE_PHYSIC_DELETE, 1, undoNxtLSN);
        recData->startSlot = slotNum;
        recData->numSlots = i - start;
        recData->length = p - recData->data;
        ASSERT3(recData->numSlots > 0);
        
        SetImage(recInfo, 0, recData->Size(), recData);
		
        int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
			ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= BTREE_PAGESIZE);
        
        if (!disabled)  {
			GENLRC();
            LogIt(recInfo, fixedSize, pHash);
		}
	}
    
    delete (Buff*) recData;
}


#ifdef SERVER_MAKE
void BT_LogPhysicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc, 
                        TWO slotNum, TWO numSlot, 
                        int length, void* buff, LSNOFFSET undoNxtLSN)
{
    ASSERT3(slotNum >= 0);
    ASSERT3(numSlot > 0);
	
    LogBtreePhysicDeleteData* recData = (LogBtreePhysicDeleteData*) new Buff;
	
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_PHYSIC_DELETE, 1, undoNxtLSN);
	
    recData->startSlot = slotNum;
    recData->numSlots = numSlot;
    recData->length = length;
    BCOPY(buff, recData->data, length);
    ASSERT3(recData->numSlots > 0);
	
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
    ASSERT3(fixedSize <= BTREE_PAGESIZE);
	
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    
    delete (Buff*) recData;
}
#endif


void BT_LogLogicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
					   const PID& rootPid, const void* key, 
					   TWO keyLen, const void* el, TWO elSize, TWO maxKeyLen, 
					   TWO unique, SMDATATYPE keyType, LSNOFFSET undoNxtLSN)
{
    BT_LOGRECINFO recInfo;
    LogBtreeLogicInsertData* recData = (LogBtreeLogicInsertData*) new Buff;
    
    recData->rootPid = rootPid;
    recData->elSize = elSize;
    recData->maxKeyLen = maxKeyLen;
    recData->unique = unique;
    recData->keyType = keyType;
    recData->keyLen = keyLen;
    BCOPY(key, recData->Key(), keyLen);
    BCOPY(el, recData->El(), elSize);
    
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_INSERT, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
	
    delete (Buff*) recData;
}



void BT_LogLogicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
					   const PID& rootPid, const void* key, 
					   TWO keyLen, const void* el, TWO elSize, TWO maxKeyLen,
					   TWO unique, SMDATATYPE keyType, LSNOFFSET undoNxtLSN)
{
    BT_LOGRECINFO recInfo;
    LogBtreeLogicDeleteData* recData = (LogBtreeLogicDeleteData*) new Buff;
    
    recData->rootPid = rootPid;
    recData->elSize = elSize;
    recData->maxKeyLen = maxKeyLen;
    recData->unique = unique;
    recData->keyType = keyType;
    recData->keyLen = keyLen;
    BCOPY(key, recData->Key(), keyLen);
    BCOPY(el, recData->El(), elSize);
    
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_DELETE, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    delete (Buff*) recData;
}



void BT_LogLogicSetOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
							const PID& rootPid, 
							const void* key, TWO keyLen, TWO maxKeyLen, 
							SMDATATYPE keyType,
							int numEl, int elSize, const void* elList, 
							const PID& ovPid, LSNOFFSET undoNxtLSN)
{
    LogBtreeLogicSetOverflowData* recData = 
		(LogBtreeLogicSetOverflowData*) new Buff;
    
    recData->rootPid = rootPid;
    recData->ovPid = ovPid;
    recData->maxKeyLen = maxKeyLen;
    recData->numEl = numEl;
    recData->elSize = elSize,
		recData->keyLen = keyLen;
    recData->keyType = keyType;
    BCOPY(key, recData->KeyValue(), keyLen);
    BCOPY(elList, recData->ElList(), numEl * elSize);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_SET_OVERFLOW, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
    
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    
    delete (Buff*) recData;
}


#ifdef SERVER_MAKE

void BT_LogLogicResetOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc,
							  const PID& rootPid, 
							  const void* key, TWO keyLen, TWO maxKeyLen, 
							  SMDATATYPE keyType, 
							  int numEl, int elSize, const void* elList, 
							  const PID& ovPid, LSNOFFSET undoNxtLSN)
{
    LogBtreeLogicResetOverflowData* recData = 
		(LogBtreeLogicResetOverflowData*) new Buff;
    
    
    recData->rootPid = rootPid;
    recData->ovPid = ovPid;
    recData->maxKeyLen = maxKeyLen;
    recData->numEl = numEl;
    recData->elSize = elSize,
		recData->keyLen = keyLen;
    recData->keyType = keyType;
    BCOPY(key, recData->KeyValue(), keyLen);
    BCOPY(elList, recData->ElList(), numEl * elSize);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_RESET_OVERFLOW, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
    delete (Buff*) recData;
}


#endif



void BT_LogLogicIncrElCnt(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client, 
						  const PID& rootPid, const void* key, 
						  TWO keyLen, TWO maxKeyLen, SMDATATYPE keyType,
						  LSNOFFSET undoNxtLSN)
{
    LogBtreeLogicIncrElCntData* recData = 
		(LogBtreeLogicIncrElCntData*) new Buff;
    
    recData->rootPid = rootPid;
    recData->maxKeyLen = maxKeyLen;
    recData->keyType = keyType;
    recData->keyLen = keyLen;
    BCOPY(key, recData->key, keyLen);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_INCR_OIDCNT, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
    
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
	
    delete (Buff*) recData;
}



void BT_LogLogicDecrElCnt(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
						  const PID& rootPid, const void* key, 
						  TWO keyLen, TWO maxKeyLen, SMDATATYPE keyType, 
						  LSNOFFSET undoNxtLSN)
{
    LogBtreeLogicDecrElCntData* recData = 
		(LogBtreeLogicDecrElCntData*) new Buff;
    
    recData->rootPid = rootPid;
    recData->maxKeyLen = maxKeyLen;
    recData->keyType = keyType;
    recData->keyLen = keyLen;
    BCOPY(key, recData->key, keyLen);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTREE_LOGIC_DECR_OIDCNT, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
    
    if (!disabled)  {
		GENLRC();
        LogIt(recInfo, fixedSize, pHash);
	}
	
    delete (Buff*) recData;
}



void OV_LogPageInit(PAGEHASH* pHash, TWO elSize)
{
    BT_LOGRECINFO recInfo;
    LogBtOvPageInitData* recData = (LogBtOvPageInitData*) new Buff;
    recData->elSize = elSize;
    
    recInfo.type = LOG_REC_TYPE_USER;
    recInfo.action = LOG_ACTION_BTOV_PAGE_INIT;
    recInfo.flags = REDO_ONLY;
    recInfo.imageCount = 1;
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
    
#ifdef SERVER_MAKE
    OVPAGE* op = (OVPAGE*) pHash->bufFrame;
    LRC* lrc = &op->lrc();
    GENERATE_LRC(lrc);
    recInfo.actionPid = &(op->SelfID());
    recInfo.actionLRC = lrc;
#else
    OVPAGE* op = (OVPAGE*) pHash->pageInfo->bufFrame;
    recInfo.actionPid = op->SelfID();
#endif
    LogIt(recInfo, fixedSize, pHash);
    
    delete (Buff*) recData;
}





void OV_LogModifyNextLink(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
						  const PID& rootPid, SHORTPID oldNext, SHORTPID newNext,
						  LSNOFFSET undoNxtLSN)
{
    LogBtOvModifyNextLink* recData = (LogBtOvModifyNextLink*) new Buff;
    recData->rootPid = rootPid;
    recData->oldNext = oldNext;
    recData->newNext = newNext;
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTOV_MODIFY_NEXT_LINK, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
	GENLRC();
    LogIt(recInfo, fixedSize, pHash);
    
    delete (Buff*) recData;
}


void OV_LogModifyPrevLink(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
						  const PID& rootPid, SHORTPID oldPrev, SHORTPID newPrev,
						  LSNOFFSET undoNxtLSN)
{
    LogBtOvModifyPrevLink* recData = (LogBtOvModifyPrevLink*) new Buff;
    recData->rootPid = rootPid;
    recData->oldPrev = oldPrev;
    recData->newPrev = newPrev;
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTOV_MODIFY_PREV_LINK, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
	
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	GENLRC();
    LogIt(recInfo, fixedSize, pHash);
    
    delete (Buff*) recData;
}


void OV_LogInsert(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
				  const PID& rootPid, TWO elSize, TWO slotNum, 
				  TWO numSlot, void* buff, LSNOFFSET undoNxtLSN)
{
    LogBtOvInsert* recData = (LogBtOvInsert*) new Buff;
    recData->rootPid = rootPid;
    recData->startSlot= slotNum;
    recData->numSlots = numSlot;
    recData->elSize = elSize;
    
    //
    // assume everything fits into one log record
    //
    ASSERT3(numSlot < recData->Max() / elSize);
    
    BCOPY(buff, recData->data, numSlot * elSize);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTOV_INSERT, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
	GENLRC();
    LogIt(recInfo, fixedSize, pHash);
    
    delete (Buff*) recData;
}


void OV_LogDelete(PID& pid, PAGEHASH* pHash, LRC& lrc_unused_on_client,
				  const PID& rootPid, TWO elSize,
				  TWO slotNum, TWO numSlot, void* buff, LSNOFFSET undoNxtLSN)
{
    LogBtOvDelete* recData = (LogBtOvDelete*) new Buff;
    recData->rootPid = rootPid;
    recData->startSlot = slotNum;
    recData->numSlots = numSlot;
    recData->elSize = elSize;
    
    //
    // assume everything fits into one log record
    //
    ASSERT3(numSlot < recData->Max() / elSize);
    
    BCOPY(buff, recData->data, numSlot * elSize);
    
    BT_LOGRECINFO recInfo;
    INITRECINFO(recInfo, LOG_ACTION_BTOV_DELETE, 1, undoNxtLSN);
    SetImage(recInfo, 0, recData->Size(), recData);
    
    int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
		ALIGNSIZE(recData->Size());
	
	GENLRC();
    LogIt(recInfo, fixedSize, pHash);
    
    delete (Buff*) recData;
}
