/*
 * $RCSfile: BT_NewNodeTuple.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*		     Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "SMOPages.h"


int BT_NewNodeTuple(TID tid, BUFGROUP* bufGroup, SMOPages* smo, 
		    ParentStack& pl, TWO keyLen, void* key, 
		    PID* subtreePid, PFC compFunc)
{
    TWO		slot;
    
    BTREEPAGE* thisNode = pl.Top();
    
    if (thisNode->Search(keyLen, key, compFunc, slot) == TRUE)	{
	// duplicate key in internal nodes
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }
    
    //
    //	Check that there is enough room on the page. If so, insert
    //	the key/pid pair directly into the page; otherwise, split the
    //	node and insert the key recursively.
    //
    
    BT_Tuple buff;
    buff.Create(keyLen, key, subtreePid->page);
    int byteNeeded = buff.Size() + sizeof(SLOT);
    
    //
    //	Insert into thisNode if there is enough space
    //
    if (byteNeeded <= thisNode->FreeSpace())  {
	thisNode->InsertTuple(buff, slot); 
	BT_LogPhysicInsert(thisNode->SelfID(), pl.Top(), thisNode->lrc(), 
			   slot, 1, 0);
	if (smo) smo->Insert(thisNode->SelfID());
	pl.Top().Dirty();
	return esmNOERROR;
    }
    
    //
    //	Split the node which is too crowded
    //
    PageDesc newNodeLink;
    if (BT_SplitPage(tid, bufGroup, smo, pl, slot, buff.Size(), newNodeLink))
	return esmFAILURE;
    
    thisNode = pl.Top();
    BTREEPAGE* newNode = newNodeLink;
    
    //
    //	Determine which page the new tuple should go
    //
    if ((slot < thisNode->NumSlots()) || 
	(slot == thisNode->NumSlots() && 
	 thisNode->FreeSpace() > newNode->FreeSpace())) {
	thisNode->InsertTuple(buff, slot);
	BT_LogPhysicInsert(thisNode->SelfID(), pl.Top(), thisNode->lrc(),
			   slot, 1, 0);
    } else {
	newNode->InsertTuple(buff, slot - thisNode->NumSlots());
	BT_LogPhysicInsert(newNode->SelfID(), newNodeLink, newNode->lrc(),
			   slot - thisNode->NumSlots(), 1, 0);
	
	// ---- Update parent list 
	
	pl.DiscardTop();
	if (pl.ReadNPush(newNode->SelfID(), EX))
	    return esmFAILURE;
    }
    
    //
    //	Finally, the first <key,pid> pair of the right node has to be 
    //	taken out. The pid should go into pid0 of the newNode, and the 
    //	key with the pid of newNode should be inserted into its
    //	parent recursively.
    //
    SHORTPID oldPid0 = newNode->FirstVector();
    newNode->SetFirstVector(newNode->Tuple(0).Vector()); // set pid0 of newNode
    BT_LogModifyVector0(newNode->SelfID(), newNodeLink, newNode->lrc(),
			oldPid0, newNode->FirstVector(), 0);
    
    char keyBuffer[MAXKEYLEN];
    int length = newNode->Tuple(0).KeyLen();
    BCOPY(newNode->Tuple(0).KeyValue(), keyBuffer, length);
    
    BT_LogPhysicDelete(newNode->SelfID(), newNodeLink, 
		       newNode->lrc(), 0, 1, 0);
    newNode->RemoveTuple(0);
    
    PageDesc acc = pl.Pop();
    if (BT_NewNodeTuple(tid, bufGroup, smo, pl, length, keyBuffer,
			&(newNode->SelfID()), compFunc))  {
	pl.Push(acc);
	return esmFAILURE;
    }
    pl.Push(acc);
    
    newNodeLink.UnfixDirty();
    
    return esmNOERROR;
}


