/*
 * $RCSfile: BT_ResetOverflow.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "SMOPages.h"


#ifdef SERVER_MAKE

int BT_ResetOverflow(TID tid, const PID& rootPid, BUFGROUP* bufGroup, 
					void* key, int keyLen, int maxKeyLen, PFC compFunc,
					const void* elList, int numEl, LSNOFFSET undoNxtLSN)
{
	TWO				slot;
	ParentStack		pl(bufGroup);
	int				found;

	//
	//	Traverse the tree
	//
	if (BT_Traverse(rootPid, keyLen, key, found,
							slot, pl, BT_INSERT, maxKeyLen, compFunc))  {
		return esmFAILURE;
		}
		
	if (!found)
		return esmFAILURE;

	BT_Tuple* tp = & (((BTREEPAGE*) pl.Top())->Tuple(slot));
	ASSERT3(tp->ElCnt() == numEl);
	PID ovPid = tp->OverflowRoot();
	
	int length = numEl * tp->ElSize();
	
	if (((BTREEPAGE*) pl.Top())->FreeSpace() < length) {
		SMOPages smo;
		
		BT_TRACE(("btree: page split\n"));
		
		if (BT_SetAnchor(tid)) 
			return esmFAILURE;
			
		PageDesc newLeafLink;
		if (BT_SplitPage(tid, bufGroup, &smo, pl, slot, length, newLeafLink)) {
			pl.Push(newLeafLink);
			return esmFAILURE;
			}
			
		BTREEPAGE* leaf = pl.Top();
		BTREEPAGE* newLeaf = newLeafLink;
		PID newLeafPid = newLeaf->SelfID();
		
		int oldSlotCnt = leaf->NumSlots();
		BT_Tuple* propagateTuple = &newLeaf->Tuple(0);
		if (slot >= oldSlotCnt)  {
			pl.DiscardTop();
			
			if (pl.ReadNPush(newLeafPid, EX)) {
				pl.Push(newLeafLink);
				return esmFAILURE;
				}
				
			slot -= oldSlotCnt;
			}
		newLeafLink.UnfixDirty();
		
		PageDesc acc = pl.Pop();
		if (BT_NewNodeTuple(tid, bufGroup, &smo, pl, propagateTuple->KeyLen(),
							propagateTuple->KeyValue(),
							&newLeafPid, compFunc))  {
			pl.Push(acc);
			return esmFAILURE;
			}
		pl.Push(acc);
		
		smo.Flush();
		
		if (BT_SetCLR(tid))
			return esmFAILURE;
		}
	
	BTREEPAGE* bp = pl.Top();
	bp->SetUnderflow(slot, numEl, elList);
	ASSERT3(bp->CheckContent());
	
	tp = & bp->Tuple(slot);
	
	BT_LogLogicResetOverflow(bp->SelfID(), pl.Top(), bp->lrc(),
							rootPid, key, keyLen, maxKeyLen, bp->KeyType(),
							tp->ElCnt(), tp->ElSize(), tp->ElArray(0), ovPid,
							undoNxtLSN);
	
	pl.Top().Dirty();


	return esmNOERROR;
}

#endif
