/*
 * $RCSfile: BT_SetCLR.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "BTREEPAGE.h"

#ifndef SERVER_MAKE
#	include "msgdefs.h"
#	include "msg_funcs.h"
BOOL btCLRLogActive = FALSE;
#endif


#ifdef DEBUG
int crashAfterSplitRoot = 0;
int crashInCLR = 0;
extern int rootSplitPerformed;
#endif


int BT_SetCLR(TID tid)
{
#ifdef DEBUG
	if (crashAfterSplitRoot && rootSplitPerformed)  {
		printf("User's flag : crash after split root\n");
		exit(0);
		}
	if (crashInCLR)  {
		printf("User's flag : crash in CLR\n");
		exit(0);
		}
#endif
#ifndef SERVER_MAKE
	MESSAGE		message;

	/*
	 *	load up a message
	 */
	message.header.type = SET_CLR;
	message.header.replyRequested = TRUE;
	message.header.params.in.tid	= GET_SERVERTID;

	/*
	 *	write the message
	 */
	if (callServer(GET_SERVERINFO, &message, NULL, 0) < 0)  {
		return esmFAILURE;
		}

	return esmNOERROR;
#else
    TRANSREC* transRec = findTransRec(tid);
    if ( ! transRec)  {
        SM_ERROR(TYPE_USER, esmBADTRANSID);
        return esmFAILURE;
        }


	LOGRECORDINFO	recordInfo;
	recordInfo.action = LOG_ACTION_PURE_CLR;
	recordInfo.type = LOG_REC_TYPE_COMPENSATION;
	recordInfo.imageCount = 0;
	recordInfo.actionPid = 0;
	recordInfo.flags = REDO_ONLY;
	recordInfo.nextUndoLSN = transRec->rememberedLSN;

	LSN lsn;
	FORCEMARK forceMark = writeLogRecord(transRec, &recordInfo, &lsn, NOFLAGS);
	if (forceMark < 0) 
		return esmFAILURE;

	BT_TRACE(("Setting CLR to %d\n", recordInfo.nextUndoLSN));

    transRec->rememberedLSN = NULL_LSN;

    return esmNOERROR;

#endif
}
