/*
 * $RCSfile: BT_SetOverflow.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "SMOPages.h"
#include "OVPAGE.h"


#ifdef SERVER_MAKE

int BT_SetOverflow(TID tid, const PID& rootPid, BUFGROUP* bufGroup, 
					void* key, int keyLen, int maxKeyLen, PFC compFunc,
					const void* elList, int numEl, LSNOFFSET undoNxtLSN)
{
	TWO				slot;
	ParentStack		pl(bufGroup);
	int				found;

	//
	//	Traverse the tree
	//
	if (BT_Traverse(rootPid, keyLen, key, found,
							slot, pl, BT_INSERT, maxKeyLen, compFunc))  {
		return esmFAILURE;
		}
		
	if (!found)
		return esmFAILURE;

	BTREEPAGE* leaf = pl.Top();
	ASSERT3(leaf->IsLeaf());
	
	BT_Tuple& tp = leaf->Tuple(slot);
	ASSERT3( ! tp.IsOverflow());
	ASSERT3(tp.WillOverflow());
	
	ASSERT3(tp.ElCnt() == numEl);
	ASSERT3( ! bcmp((char*)elList, (char*)tp.ElArray(0), numEl * tp.ElSize()));
	
	PID ovPid;
	if (ov_NewCluster(tid, rootPid.volid, bufGroup, tp.ElSize(), ovPid, tp))
		return esmFAILURE;
		
	BT_LogLogicSetOverflow(leaf->SelfID(), pl.Top(), leaf->lrc(), rootPid, 
						key, keyLen, maxKeyLen, leaf->KeyType(), 
						tp.ElCnt(), tp.ElSize(), tp.ElArray(0), ovPid,
						undoNxtLSN);
									
	leaf->SetOverflow(slot, ovPid);
	ASSERT3(tp.IsOverflow());
	
	pl.Top().Dirty();

	return esmNOERROR;
}

#endif
