/*
 * $RCSfile: BT_Traverse.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "ParentStack.h"




#define BT_LOCKWANTED(OPER)	((OPER == BT_READ) ? SH : EX)


//
//	Function	: int BT_Traverse(PID& const rootPid, int keyLen, 
//						void* keyVal, int& found, TWO& returnSlot, 
//						ParentStack& pl, BT_OPER oper, int maxKeyLen, 
//						PFC compFunc)
//	Description	: Travers the btree starting from 'rootPid' in search of
//				  keyVal. If found, a TRUE value is returned in 'found'.
//				  The traverser path from root to leaf is locked 
//				  appropriately according to 'oper'. On termination without
//				  error, 'returnSlot' and 'pl' is filled with the slot the
//				  key is in (or should be in, if not found), and all 
//				  pages in the traversal path.
//				  
int BT_Traverse(const PID& rootPid, int keyLen, const void* keyVal, 
							int& found, TWO& returnSlot, ParentStack& pl, 
							BT_OPER oper, int maxKeyLen, PFC compFunc)
{
	LOCKMODE	lockMode = BT_LOCKWANTED(oper);

	if (pl.ReadNPush(rootPid, lockMode))
		return esmFAILURE;

	BTREEPAGE* thisPage = pl.Top();
	ASSERT3(thisPage->CheckContent());
	PID pid = thisPage->SelfID();

	while ( ! thisPage->IsLeaf() )  {
		TWO			slotNum;

		if (thisPage->Search(keyLen, keyVal, compFunc, slotNum) == FALSE)
			slotNum--;

		pid.page = (slotNum < 0) ? thisPage->FirstVector() : 
									thisPage->Tuple(slotNum).Vector();

		if (pl.ReadNPush(pid, lockMode))
			return esmFAILURE;

		thisPage = pl.Top();

		if (oper == BT_READ)  {
			// 	---- release previous lock  and unfix previous buffer 
			pl.ReleasePrevious();
			}
		else if (thisPage->IsSafe(oper, maxKeyLen))   {
				// 	---- release all previous locks and 
				//	---- all previous buffers with lock
				pl.ReleaseAllPrevious();
			}
		}
		
	ASSERT3(thisPage->IsLeaf());
		
	//
	//	Search the leaf 
	//
	found = thisPage->Search(keyLen, keyVal, compFunc, returnSlot);
	
	return esmNOERROR;
}

