/*
 * $RCSfile: BT_Tuple.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */
/**********************************************************************
 * EXODUS Database Toolkit Software
 * Copyright (c) 1991 Computer Sciences Department, University of
 *                    Wisconsin -- Madison
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
 * MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
 * THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * The EXODUS Project Group requests users of this software to return 
 * any improvements or extensions that they make to:
 *
 *   EXODUS Project Group 
 *     c/o David J. DeWitt and Michael J. Carey
 *   Computer Sciences Department
 *   University of Wisconsin -- Madison
 *   Madison, WI 53706
 *
 *	 or exodus@cs.wisc.edu
 *
 * In addition, the EXODUS Project Group requests that users grant the 
 * Computer Sciences Department rights to redistribute these changes.
 **********************************************************************/
#define BT_TUPLE_C

#include "BTREEPAGE.h"
#ifdef SERVER_MAKE
#	include "log_globals.h"
#endif


void BT_Tuple::Create(int keyLen, const void* keyPtr,
		      const void* elem, int elSize) 
{
    // Create a leaf tuple
    ASSERT3(elSize > 0 && elSize <= SM_MAXELEMLEN);
    klen = keyLen;
    align = ALIGNSIZE(klen) - klen;
    type = LEAFTUPLE;
    attr.leafAttr.elCnt = 1;
    attr.leafAttr.elSize = elSize;
    attr.leafAttr.ovFlag = FALSE;
    BCOPY(keyPtr, key, keyLen);
    BCOPY(elem, ElArray(0), elSize);
}


void BT_Tuple::Create(int keyLen, void* keyPtr, SHORTPID child) 
{
    // Create a node tuple
    klen = keyLen;
    align = ALIGNSIZE(klen) - klen;
    type = NODETUPLE;
    attr.nodeAttr.childPid = child;
    BCOPY(keyPtr, key, keyLen);
}




//
//	Class		: BT_Tuple
//	Method		: int AddElem(void* const newElem)
//	Description	: Add 'newElem' into the elArray of this tuple (in order).
//
int BT_Tuple::AddElem(const void* newElem)
{
    ASSERT3(!IsOverflow() && !WillOverflow());
    
    int idx;
    
    //
    //	Find position in oidArray to insert 'newEl'
    //
    if (BT_SearchElemArray(ElSize(), newElem, 
			   ElCnt(), (char*) ElArray(0), idx))  {
	SM_ERROR(TYPE_USER, esmOIDEXISTS);
	return esmFAILURE;
    }
    
    attr.leafAttr.elCnt++;	// one more oid
    
    //
    //	Make space for new entry
    //		
    for (int i = ElCnt() - 1; i > idx; i--)  {
	// ElArray(i) = ElArray(i-1);
	BCOPY(ElArray(i - 1), ElArray(i), ElSize()); 
    }
    
    //
    //	Insert new entry
    //
    BCOPY(newElem, ElArray(idx), ElSize());
    
    return esmNOERROR;
}





int BT_Tuple::RemoveElem(const void* doomedElem)
{
    ASSERT3(!IsOverflow() && ElCnt() > 0);
    
    //
    //	Find oid to remove
    //
    int idx;
    if (! BT_SearchElemArray(ElSize(), doomedElem, 
			     ElCnt(), (char*)ElArray(0), idx))  {
	SM_ERROR(TYPE_USER, esmOIDNOTFOUND);
	return esmFAILURE;
    }
    
    //
    //	Close gap in oid array
    //
    for (int i = idx+1, cnt = ElCnt(); i < cnt; i++)  {
	// ElArray(i-1) = ElArray(i);
	BCOPY(ElArray(i), ElArray(i-1), ElSize());
    }
    
    attr.leafAttr.elCnt--;		// one less oid now
    
    return esmNOERROR;
}


