/*
 * $RCSfile: OV_AllocPage.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "io_extfuncs.h"
#include "sm_macro.h"


//
//	Function	: int OV_AllocPage(TID tid, int volume, 
//										BUFGROUP* bufGroup, PID& pid)
//	Description	: Allocates an overflow page in 'volume' and returns
//				  its page id in 'pid'. The page is properly initialized.
//
int OV_AllocPage(TID tid, int volume, BUFGROUP* bufGroup, int elSize, PID& pid)
{
#ifdef SERVER_MAKE
	TRANSREC* transRec = (TRANSREC*) Active->transRec;
	VOLREC	* volRec;
	ASSERT1(transRec->tid == tid);

	if( (volRec = io_FindVolRec((VOLID) volume)) == NULL ) {
        return esmFAILURE;
	}

    if (io_AllocPages(volRec, BTREE_PAGE2SIZE, 1, NULL, &pid))  {
        return esmFAILURE;
        }
#else
	checkTransState((VOLID)volume);
    if (io_AllocPages(volume, BTREE_PAGE2SIZE, 1, NULL, &pid, PAGE_INDEX))  {
        return esmFAILURE;
        }
#endif

    PageDesc pageDesc;
    if (pageDesc.GetPage(bufGroup, pid))  {
        return esmFAILURE;
        }

	OVPAGE* op = pageDesc;
	
#ifdef SERVER_MAKE
	INITIALIZE_LRC( &op->lrc());
#else
	op->lrc() = *(GET_LOGINFO->initLRC);
#endif

	//
	//	Initialize new page
	//
	op->Init(pid, elSize);

	//
	//	Log the initialization
	//
	OV_LogPageInit(pageDesc, op->ElSize());

    pageDesc.UnfixDirty();

    return esmNOERROR;
}

