/*
 * $RCSfile: OV_Check.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "sm_macro.h"

#ifdef DEBUG

//
//	Function	: int OV_Check(PID& const rootPid, BUFGROUP* bufGroup)
//	Description	: Checks the overflow pages rooted at 'rootPid'. All oids
//				  should be in ascending order.
//
int OV_Check(const PID& rootPid, BUFGROUP* bufGroup)
{
	PID pid = rootPid;
	SHORTPID next = rootPid.page;
	SHORTPID prev = NULLPID;
	
//	printf("[");
	
	while (next != NULLPID)  {
		pid.page = next;
		PageDesc pd;
		
//		printf("%d ", pid.page);
		ASSERT1(pd.Read(bufGroup, pid, SH) == esmNOERROR);
		
		OVPAGE* p = pd;

		ASSERT1(p->SelfID().page == pid.page);
		ASSERT1(p->SelfID().volid == pid.volid);		
		ASSERT1(p->CheckPage());
		
		ASSERT1(p->PrevPage() == prev);
		
		next = p->NextPage();
		prev = pid.page;
		pd.Unfix();
		}
		
//	printf("]\n");
		
	return TRUE;
}
#endif

