/*
 * $RCSfile: OV_Search.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "sm_macro.h"



//
//	Function	: int OV_Search(PID& const rootPid, BUFGROUP* bufGroup, 
//								void* const ep, PID& pid, int& found, int& idx)
//	Description	: Search the overflow pages rooted at rootPid for 'ep'.
//				  If found, return TRUE in 'found'. 
//				  Returns the current page being searched in 'pid' and 
//				  index within the page in 'idx'. This would be the
//				  appropriate place to insert 'ep' if it was not found.
//
int OV_Search(const PID& rootPid, BUFGROUP* bufGroup, 
								const void* ep, PID& pid, int& found, int& idx)
{
	pid = rootPid;
	idx = -1;
	found = FALSE;
	SHORTPID next = rootPid.page;

	//
	//	Go through the chain of overflow pages
	//
	while (next != NULLPID)  {
		pid.page = next;
		PageDesc pageDesc;
		if (pageDesc.Read(bufGroup, pid, EX))
			return esmFAILURE;

		OVPAGE* pg = pageDesc;
		
		ASSERT3(pg->ElCnt() >= 0);

		if (pg->ElCnt())  {
			if (BT_ElemDiff(pg->ElSize(), ep, pg->Get(pg->ElCnt()-1)) <= 0)  {
				//
				// 	binary search on this page
				//
				found = BT_SearchElemArray(pg->ElSize(), ep, pg->ElCnt(), 
													(char*) pg->Get(0), idx);
				pageDesc.Unfix();

				return esmNOERROR;
				}
			}

		next = pg->NextPage();
		idx = pg->ElCnt();

		pageDesc.Unfix();
		}

	return esmNOERROR;
}

