/*
 * $RCSfile: bt_BulkLoad.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "SMOPages.h"
#include "OVPAGE.h"
#include "sm_extfuncs.h"


#define READ_ALL -1

extern void IntPrint(int, char*);
extern void (*BT_PrintFunc)(int, char*);


static BulkLoad(TID tid, ParentStack& pl, SCANDESC* sd, 
						int cnt, int elSize, int unique, SMDATATYPE type);
static AddElem(TID tid, BUFGROUP* bufGroup, VOLID volid, 
						BT_Tuple& tp, int elSize, void* elem);

bt_BulkLoad(TID tid, BUFGROUP* bufGroup, VOLID volid, FID& input, int cnt,
					BOOL unique, SMDATATYPE type, int elSize, PID& rootPid)
{
	SCANDESC* sd = 0;	
	ParentStack pl(bufGroup);
		
	//
	//	Read in root page
	//
	if (pl.ReadNPush(rootPid, EX)) {
		return esmFAILURE;
		}
			
	PID pid;	
	if (BT_AllocPage(tid, volid, bufGroup, pid, 1, type))  {
		return esmFAILURE;
		}

	BTREEPAGE* page = pl.Top();	// root
	if (! (page->IsLeaf() && page->NumSlots() == 0))  {
		SM_ERROR(TYPE_USER, esmINDEXNOTEMPTY);
		return esmFAILURE;
		}
	pl.Top().Dirty();	// will be dirtied
			
	
	SHORTPID oldPid0;
	oldPid0 = page->FirstVector();	
	page->SetLevel(2);
	BT_LogModifyLevel(page->SelfID(), pl.Top(), page->lrc(), 1, 2, 0);
	page->SetFirstVector(pid.page);
	BT_LogModifyVector0(page->SelfID(), pl.Top(), page->lrc(), oldPid0, 
													page->FirstVector(), 0);
	page = 0;
		
	if (pl.ReadNPush(pid, EX))  {
		return esmFAILURE;
		}
	
	if (sm_OpenScan(&input, LOGICAL_SCAN, 5, &sd, 0))  {
		return esmFAILURE;
		}
	
	//
	//	Turn off logging, bulkload, restore logging
	//
	btCLRLogActive = FALSE;
	int e = BulkLoad(tid, pl, sd, cnt, elSize, unique, type);
	btCLRLogActive = TRUE;

	sm_CloseScan(sd);
	
	return e ? esmFAILURE : esmNOERROR;
}




static
BulkLoad(TID tid, ParentStack& pl, SCANDESC* sd, 
						int cnt, int elSize, int unique, SMDATATYPE type)
{
	PFC cmp = SMCOMPFUNC[type];
	BOOL eof;
	BUFGROUP* bufGroup = sd->bufGroup;
	USERDESC* ud;
	
	VOLID volid = ((BTREEPAGE*) pl.Top())->Volume();
	if (sm_ScanNextObject(sd, 0, READ_ALL, &ud, &eof))   {
		return esmFAILURE;
		}
		
	while (! eof && cnt > 0)  {
		--cnt;
		ASSERT3(ud->objectSize > sizeof(int));
		int keyLen = * (int*) ud->basePtr;
		char* key = ud->basePtr + ALIGNSIZE(sizeof(int));
		void* elem = key + ALIGNSIZE(keyLen);
		
		// printf("loading %f (length %d) \n", * (double*) key, keyLen);

		BT_Tuple tuple;
		tuple.Create(keyLen, key, elem, elSize);

		while (cnt > 0)  {
			if (sm_ScanNextObject(sd, 0, READ_ALL, &ud, &eof))  {
				return esmFAILURE;
				}
			if (eof) break;
			
			ASSERT3(ud->objectSize > sizeof(int));
			keyLen = * (int*) ud->basePtr;
			key = ud->basePtr + ALIGNSIZE(sizeof(int));
			elem = key + ALIGNSIZE(keyLen);

			if (cmp(keyLen, tuple.KeyValue(), keyLen, key) == 0) {
				if (unique)  {
					SM_ERROR(TYPE_USER, esmKEYALREADYEXISTS);
					return esmFAILURE;
					}
				--cnt;
				AddElem(tid, bufGroup, volid, tuple, elSize, elem);
				}
			else 
				break;
			}
			
		if (((BTREEPAGE*)pl.Top())->FreeSpace() < 
										tuple.Size() + sizeof(SLOT))  {
			PageDesc leftPd = pl.Pop();
			BTREEPAGE* leftPage = leftPd;
			
			PID rightPid;
			if (BT_AllocPage(tid, volid, bufGroup, rightPid, 
												leftPage->Level(), type)) {
				pl.Push(leftPd);
				return esmFAILURE;
				}
				

			if ( pl.ReadNPush(rightPid, EX) )  {
				pl.Push(leftPd);
				return esmFAILURE;
				}

			PageDesc acc = pl.Pop();
			BTREEPAGE* rightPage = acc;
			leftPage->Linkup(rightPage, 0);
				
			leftPd.UnfixDirty();
			leftPage = 0;
				
			if (BT_NewNodeTuple(tid, bufGroup, 0, pl, keyLen, 
									tuple.KeyValue(), &rightPid, cmp)) {
				pl.Push(acc);
				return esmFAILURE;
				}
			pl.Push(acc);
			}
			
		((BTREEPAGE*) pl.Top())->AppendTuple(tuple);
		pl.Top().Dirty();
		}
	SM_ASSERT(LEVEL_1, cnt == 0);
	return esmNOERROR;
}

static 
AddElem(TID tid, BUFGROUP* bufGroup, VOLID volid, 
									BT_Tuple& tp, int elSize, void* elem)
{
	if (tp.WillOverflow())  {
		PID ovPid;
		if (ov_NewCluster(tid, volid, bufGroup, elSize, ovPid, tp)) {
			return esmFAILURE;
			}
		tp.SetOverflow(ovPid);
		}
	if (! tp.IsOverflow())  {
		tp.AddElem(elem);
		}
	else {
		if (ov_AddElem(tid, tp.OverflowRoot(), bufGroup, elem))
			return esmFAILURE;
		tp.IncrElCnt();
		}
	return esmNOERROR;
}
