/*
 * $RCSfile: bt_CheckTree.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*		     Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"

bt_CheckTree(const PID& rootPid, BUFGROUP* bufGroup)
{
    PageDesc	tpd;
    BTREEPAGE*	tp;
    
    if (tpd.Read(bufGroup, rootPid, SH))  {
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }
    
    tp = tpd;
    
    tp->CheckContent();
    
    if (tp->NextPage() != NULLPID)  {
	PID pid = tp->SelfID();
	pid.page = tp->NextPage();
	PageDesc rpd;
	if (rpd.Read(bufGroup, pid, SH))  {
	    SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	ASSERT1(((BTREEPAGE*) rpd)->PrevPage() == tp->SelfID().page);
	rpd.Unfix();
    }
    
    if (tp->PrevPage() != NULLPID)  {
	PID pid = tp->SelfID();
	pid.page = tp->PrevPage();
	PageDesc lpd;
	if (lpd.Read(bufGroup, pid, SH))  {
	    SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	ASSERT1(((BTREEPAGE*) lpd)->NextPage() == tp->SelfID().page);
	lpd.Unfix();
    }
    
    if (tp->Level() != 1)    {
	int i;
	PID subTree;
	subTree.volid = tp->SelfID().volid;
	subTree.page = tp->FirstVector();
	bt_CheckTree(subTree, bufGroup);
	for (i = 0; i < tp->NumSlots(); i++)  {
	    subTree.volid = rootPid.volid;
	    subTree.page = tp->Tuple(i).Vector();
	    bt_CheckTree(subTree, bufGroup);
	}
    }
    
    tpd.Unfix();
    
    return esmNOERROR;
}
