/*
 * $RCSfile: bt_DestroyBtree.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "io_extfuncs.h"

const listSize = 128;

//
//	Function	: int bt_DestroyBtree(const PID& rootPid, int groupIndex)
//	Description	: Destroy the btree rooted at rootPid (recursively).
//
int bt_DestroyBtree(TID tid, const PID& rootPid, BUFGROUP* bufGroup)
{
	// do breadth first traversal, accumulate pids in a list, and, 
	// do 1 io_Deallloc.

	PID lowerPid = rootPid;
#ifdef SERVER_MAKE
	TRANSREC* transRec = (TRANSREC*) Active->transRec;
	ASSERT1(transRec->tid == tid);
#endif

	while (lowerPid.page != NULLPID)  {
		PageDesc lowerPd;
		if (lowerPd.Read(bufGroup, lowerPid, EX))
			return esmFAILURE;
		PID nextPid = lowerPid;
		
		lowerPid.page = (((BTREEPAGE*) lowerPd)->IsNode()) ?
							((BTREEPAGE*)lowerPd)->FirstVector() :
							NULLPID;

		lowerPd.Unfix();
			
		while (nextPid.page != NULLPID)  {
			PID list[listSize];
			int cnt = 0;
			do {
				PageDesc pd;
				if (pd.Read(bufGroup, nextPid, EX))  {
					return esmFAILURE;
					}
				BTREEPAGE* page = pd;
				list[cnt++] = page->SelfID();
				nextPid.page = page->NextPage();
				if (page->IsLeaf())  {
					for (int i = 0; i < page->NumSlots(); i++)  {
						BT_Tuple& tp = page->Tuple(i);
						if (tp.IsOverflow()) {
							if (ov_DestroyCluster(tid, tp.OverflowRoot(), 
															bufGroup))   {
								pd.Unfix();
								return esmFAILURE;
								}
							}
						}
					}
				pd.Unfix();
			} while (cnt < listSize && nextPid.page != NULLPID);

#ifdef SERVER_MAKE
			for (int i = 0; i < cnt; i++)  {
				BT_TRACE(("Dealloc page %d\n", list[i].page));
				}
			if (io_FakeDeallocPages(BTREE_PAGE2SIZE, cnt, list, 
								&(transRec->pageDeallocList), PAGE_INDEX)){
				return esmFAILURE;
				}
#else
			if (io_DeallocPages(BTREE_PAGE2SIZE, cnt, list, FALSE, PAGE_INDEX))
				return esmFAILURE;
#endif
			}
			
		}

	return esmNOERROR;
}
