/*
 * $RCSfile: bt_FetchInit.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*		     Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "OVPAGE.h"


//
//  Function	: int bt_FetchInit(PID& const rootPid, int groupIndex,
//				   KEY* bound1,
//				  int maxKeyLen, BT_CURSOR* cursor,
//				  SMCOND condition, PFC compFunc)
//  Description : Search the btree rooted at rootPid for bound1 (if specified).
//		  If bound1 is NULL, search starts from the first key in the
//		  btree if condition is > or >=, or from the last key in the 
//		  btree if condition is < or <=.
//
int bt_FetchInit(const PID& rootPid, BUFGROUP* bufGroup, void* bound1, 
		 int bound1Len,
		 int maxKeyLen, BT_CURSOR* cursor,
		 SMCOND condition, PFC compFunc)
{
    TWO		slot;
    ParentStack pl(bufGroup);
    int		found;
    PID		pid;
    BTREEPAGE*	leaf = 0;
    
    pid.volid = rootPid.volid;
    cursor->valid = FALSE;
    
    //
    //	Position 'slot' and 'pl' at the appropriate starting position
    //
    if (bound1)	 {
	if (BT_Traverse(rootPid, bound1Len, bound1, found,
			slot, pl, BT_READ, maxKeyLen, compFunc))  {
	    return esmFAILURE;
	}
	leaf = pl.Top();
	switch (condition)  {
	case SM_EQ:
	    if (!found) return esmNOERROR;
	    break;
	case SM_L:
	case SM_LEQ:
	    if (condition == SM_L || !found) {
		// go one slot to the left on "less than" condition
		--slot;
		while (slot < 0)  {
		    pid.page = leaf->PrevPage();
		    
		    if (pid.page == NULLPID)    return esmNOERROR;
		    
		    pl.DiscardTop();
		    
		    if (pl.ReadNPush(pid, SH))  return esmFAILURE;
		    
		    leaf = pl.Top();
		    slot = leaf->NumSlots() - 1;
		}
	    }
	    break;
	    
	case SM_G:
	case SM_GEQ:
	    if ((condition == SM_G && found) || 
		(slot >= leaf->NumSlots())) {
		// go one slot to the right on "greater than" condition
		slot++;
		while (slot >= leaf->NumSlots()) {
		    pid.page = leaf->NextPage();
		    
		    if (pid.page == NULLPID)    return esmNOERROR;
		    
		    pl.DiscardTop();
		    
		    if (pl.ReadNPush(pid, SH))  return esmFAILURE;
		    
		    leaf = pl.Top();
		    slot = 0;
		}
	    }
	    break;
	    
	default:
	    SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
    } else {
	switch (condition) {
	case SM_G:
	case SM_GEQ:
	    if (BT_FirstEntry(rootPid, found, slot, pl))
		return esmFAILURE;
	    if (!found)
		return esmNOERROR;
	    break;
	    
	case SM_L:
	case SM_LEQ:
	    if (BT_LastEntry(rootPid, found, slot, pl))
		return esmFAILURE;
	    if (!found)
		return esmNOERROR;
	    break;
	    
	case SM_EQ:
	default:
	    SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	leaf = pl.Top();
    }
    
    ASSERT3(leaf->IsLeaf());
    
    //
    //	Position oid array index in a leaf tuple
    //
    
    cursor->valid = TRUE;
    cursor->rootPID = rootPid;
    cursor->page = leaf->SelfID();
    cursor->slotNum = slot;
    cursor->index = (condition & SM_L) ? leaf->Tuple(slot).ElCnt() - 1: 0;
    
    return esmNOERROR;
}
