/*
 * $RCSfile: bt_RemoveEntry.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:15 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "ParentStack.h"
#include "OVPAGE.h"


//
//	Function	: int bt_RemoveEntry(PID& const rootPid, int groupIndex,
//							void* doomedKey, int doomedKeyLen,
//							int elSize, const void* elem,
//							int maxKeyLen, BOOL unique, PFC compFunc)
//	Description	: Remove <doomedKey, elem> from the btree rooted
//				  at rootPid.
//
int bt_RemoveEntry(TID tid, const PID& rootPid, BUFGROUP* bufGroup, 
					const void* doomedKey, int doomedKeyLen,
					int elSize, const void* elem, int maxKeyLen, 
					BOOL unique, PFC compFunc, LSNOFFSET undoNxtLSN)
{
	TWO			slot;
	ParentStack	pl(bufGroup);
	int			found;

	//
	//	Traverse the tree
	//
	if (BT_Traverse(rootPid, doomedKeyLen, doomedKey, found,
						slot, pl, BT_DELETE, maxKeyLen, compFunc))  {
		return esmFAILURE;
		}

	if (!found)	 {
		SM_ERROR(TYPE_USER, esmKEYNOTFOUND);
		return esmFAILURE;
		}

	BTREEPAGE* leaf = pl.Top();
	
	BT_Tuple& tp = leaf->Tuple(slot);
	
	//
	//	If overflow, remove from overflow pages
	//
	if (tp.IsOverflow())  {
		if (ov_RemoveElem(tid, tp.OverflowRoot(), bufGroup, elem)) 
			return esmFAILURE;
		tp.DecrElCnt();
		BT_LogLogicDecrElCnt(leaf->SelfID(), pl.Top(), leaf->lrc(),
										rootPid, doomedKey, doomedKeyLen, 
										maxKeyLen, leaf->KeyType(), 0);
		if (leaf->Tuple(slot).ElCnt() == 0)  {
			if (ov_DestroyCluster(tid, tp.OverflowRoot(), bufGroup))
				return esmFAILURE;
			BT_LogPhysicDelete(leaf->SelfID(), pl.Top(), 
										leaf->lrc(), slot, 1, 0);
			leaf->RemoveTuple(slot);
			}
		pl.Top().Dirty();
		return esmNOERROR;
		}
	
	//
	//	Remove the oid
	//
	if (leaf->RemoveElem(slot, elem)) {
		return esmFAILURE;
		}

	//
	//	Remove the tuple if there isn't any more oid in it
	//
	if (leaf->Tuple(slot).ElCnt() == 0)   {
		leaf->RemoveTuple(slot);
		}

	//
	//	Log the deletion
	//
	BT_LogLogicDelete(leaf->SelfID(), pl.Top(), leaf->lrc(), 
						rootPid, doomedKey, doomedKeyLen, elem, elSize, 
						maxKeyLen, unique, leaf->KeyType(), undoNxtLSN);

	pl.Top().Dirty();

	return esmNOERROR;
}
