/*
 * $RCSfile: ov_GetElem.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*		     Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "sm_macro.h"


//
//  Function	: int ov_GetElem(PID& const rootPid, 
//				    BUFGROUP* bufGroup, int idx, void* ret)
//  Description : Get the Elem indexed to 'idx', and return it in 'ret'.
//
int ov_GetElem(const PID& rootPid, BUFGROUP* bufGroup, int idx, void* ret)
{
    PageDesc pageDesc;
    PID pid = rootPid;
    
    //
    //	Follow the link, read page by page
    //
    while (pid.page != NULLPID)	 {
	if (pageDesc.Read(bufGroup, pid, EX))
	    return esmFAILURE;
	
	OVPAGE* pg = pageDesc;
	if (idx < pg->ElCnt())	{   // found
	    BCOPY(pg->Get(idx), ret, pg->ElSize());
	    pageDesc.Unfix();
	    return esmNOERROR;
	}
	
	pid.page = (idx -= pg->ElCnt()) < 0 ? NULLPID : pg->NextPage();
	pageDesc.Unfix();
    }
    
    return esmFAILURE;
}
