/*
 * $RCSfile: ov_NewCluster.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:16 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "sm_macro.h"

//
//	Function	: ov_NewCluster(TID tid, int volume, BUFGROUP* bufGroup, 
//								int elSize, PID& rootPid, BT_Tuple& tuple)
//	Description	: Create an overflow cluster for 'tuple'. Oids in 'tuple'
//				  is copied onto the overflow page and sorted.
//				  Logging of overflow modification is taken care of.
//
int ov_NewCluster(TID tid, int volume, BUFGROUP* bufGroup, 
								int elSize, PID& rootPid, BT_Tuple& tuple)
{

	//
	//	Allocate a page for overflow entry
	//
	if (OV_AllocPage(tid, volume, bufGroup, elSize, rootPid))
		return esmFAILURE;

	//
	//	Read in the newly allocated page
	//
	PageDesc pd;
	if (pd.Read(bufGroup, rootPid, EX))
		return esmFAILURE;
	

	//
	//	Insert oidArray[] into the root page
	//
	OVPAGE* p = pd;
	ASSERT3(tuple.ElCnt() <= p->MaxEl())
	
	p->FillSort(tuple.ElCnt(), tuple.ElArray(0));
	OV_LogInsert(p->SelfID(), pd, p->lrc(), 
						rootPid, p->ElSize(), 0, p->ElCnt(), p->Get(0), 0);
	

	//
	//	Unfix the page
	//
	pd.UnfixDirty();
	
	ASSERT3(OV_Check(rootPid, bufGroup));

	return esmNOERROR;
}

