/*
 *   $RCSfile: CLOG_GetNewLogPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:16 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "logrecs.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "bf_macro.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"

    
/***************************************************************
 *  CLOG_GetNewLogPage
 * 	
 *  PURPOSE	
 *	
 *	Sets up a new log page in the log buffer group.	
 *	May cause other log page to be forced out.
 *	Updates global loginfo structure.
 *  
 ***************************************************************/

int
CLOG_GetNewLogPage()
{
    register LOGINFO 	*logInfo;
    register PAGEHASH   *pageHash;
    PID			pid;
    GROUPLINK       	*tailLink;
    PAGEHASH		*tailBuffer;
    LOGPAGEHDR		*pageHeader;

    /* put logInfo ptr in a register var*/
    logInfo = GET_LOGINFO;

    /* Added MJF 3/22/91 */
    /* force the current log page to disk 
	 * (if dirty) to insure that log pages
	 * arrive at the server in the correct order 
	 */
    if (logInfo->tailBuffer != NULL) {
		SM_ASSERT(LEVEL_1, 
		(logInfo->tailBuffer->hashList.hashPid.volid == logInfo->volid));

        if (PAGE_IS_DIRTY(logInfo->tailBuffer)) {
            bf_ForceLogPage(GET_SERVERINFO, logInfo->tailBuffer, BF_FORCE);
            logInfo->tailBuffer = NULL;
        }
    }

    logInfo->tailPid++;  

    pid.volid = logInfo->volid;
    pid.page  = logInfo->tailPid;

    if ((tailLink = bf_GetPage(logInfo->writeGroup, &pid, H_LOG, NULL,
                                logInfo->page2size)) == NULL)    {

        SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

    pageHash = tailLink->pageHash;

    /*
     *  initialize the force marks
     */
    pageHash->firstMark = pageHash->forceMark = 0;
    /*pageHash->lsn.wrapCount = pageHash->lsn.offset = 0;*/

    /*
     *  Unfix the page.  The current page does not have to be fixed as
     *  we are assuming single-user access at clients.  This
     *  is an important assumption that will need to be maintained.
     */
    bf_UnfixPage(tailLink, BF_DEFAULT, FALSE);

    tailBuffer = tailLink->pageHash;
    tailBuffer->flags |= H_LOG;
    bzero(tailBuffer->pageInfo->bufFrame, (int) logInfo->pageSize);

    logInfo->tailBuffer = tailBuffer;
	SM_ASSERT(LEVEL_1, 
		(tailBuffer->hashList.hashPid.volid == logInfo->volid));

    logInfo->tailLink   = tailLink;
    logInfo->tailLSN   = FIRST_LSN;

    /* note that we have no log records on this page yet */
    logInfo->currentLSN   = ZERO;

    /*
     *  check to make sure the first mark and the force mark are zero
     */
    SM_ASSERT(LEVEL_3, 
		((tailBuffer->forceMark == 0) && (tailBuffer->firstMark == 0)));

    pageHeader = (LOGPAGEHDR *) tailBuffer->pageInfo->bufFrame;

    pageHeader->magic = LOGPAGE_MAGIC;
    pageHeader->wrapCount = logInfo->wrapCount;
    pageHeader->lastRecord = NULL_LSN;

    pageHeader = (LOGPAGEHDR *) (((char *) pageHeader) +
						 logInfo->lastUsableByte);
    pageHeader->magic = LOGPAGE_MAGIC;
    pageHeader->wrapCount = logInfo->wrapCount;
    pageHeader->lastRecord = NULL_LSN;

/***********    bf_DirtyPage(tailBuffer);  **************/

	return esmNOERROR;
}


