/*
 *   $RCSfile: CLOG_LogChangeSmHeader.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:17 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "logrecs.h"
#include "logaction.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"


int
CLOG_LogChangeSmHeader(

    OPERATIONINFO	*opInfo 
)
{

    LRC 		lrc;
    int 		fixedSize;
    CLIENTLOGRECORDINFO 	recInfo;
    register LOGINFO 	*logInfo;
    LSNOFFSET  	tailLSN;
    PAGEHASH 	*pageHash;
    int			offset;
    int 		spaceLeft;
    FORCEMARK 	forceMark;

/***************************************************************
 *  This function is used for logging changes to a small object header
 * 
 * 	image[0] - OID
 *	image[1] - new object header 
 *	image[2] - old object header 
 *
 *  For INSERTS and APPENDS to a moved (already forwarded object)
 *
 *  All log records are fixed length 
 ***************************************************************/
     
    /* Get register pointer to log info structure */
    logInfo = GET_LOGINFO;

    /* first, make sure that we have a valid log buffer at the tail of the
       log.  Get a new log page if this isn't the case.*/

    if (logInfo->tailBuffer == NULL) {
        CLOG_GetNewLogPage();
    }

    /*
     * determine size of the log record 
     */

    /* log record is fixed length: a log record header, oid and 
	the before and after images of the object header*/
    fixedSize = (ALIGN(sizeof(LOGRECORDHDR),BYTESPERWORD)) +
		(ALIGN(sizeof(OID),BYTESPERWORD)) +
		(ALIGN(sizeof(OBJHDR),BYTESPERWORD)) +
		(ALIGN(sizeof(OBJHDR),BYTESPERWORD));

    recInfo.type        = LOG_REC_TYPE_USER;
    recInfo.action      = opInfo->action;
    recInfo.flags       = NOFLAGS;
    recInfo.imageCount  = 3;
    recInfo.imageSize[0] = sizeof(OID);
    recInfo.imageData[0] = (UONE *) &(opInfo->oid);
    recInfo.imageSize[1] = sizeof(OBJHDR);
    recInfo.imageData[1] = (UONE *) opInfo->newData;
    recInfo.imageSize[2] = sizeof(OBJHDR);
    recInfo.imageData[2] = (UONE *) opInfo->oldData;
    recInfo.actionPid.page = opInfo->oid.diskAddr.page;
    recInfo.actionPid.volid = opInfo->oid.diskAddr.volid;

    if ((pageHash = bf_FindPage( &(recInfo.actionPid))) == NULL) {
       return(esmFAILURE);
    }
	SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));

    lrc = ((SLOTTEDPAGE *) pageHash->pageInfo->bufFrame)->header.lrc;

    /*
     * determine placement of log record(s) on the page(s)
     */
    tailLSN = logInfo->tailLSN;
    offset = (int) (tailLSN & logInfo->pageMask);
    spaceLeft = logInfo->lastUsableByte - offset;

    /* see if we can use space on current log page. */
    if (fixedSize > spaceLeft) {

	/* can't take advantage of space in current page, move on to next*/
	CLOG_GetNewLogPage();
        spaceLeft = logInfo->usableBytes;
    }
	    
    spaceLeft -= fixedSize;

    /* check the size calculation*/
    SM_ASSERT(LEVEL_1, spaceLeft >= 0);


    /* write the log record*/
    forceMark = CLOG_WriteLogRecord(TransRec.clientTid, &recInfo, &lrc, pageHash);

    /* mark the page dependency with the last log record written */
     pageHash->forceMark = forceMark;

    /* put the new lrc on the page */
     ((SLOTTEDPAGE *) pageHash->pageInfo->bufFrame)->header.lrc = lrc;

     return(esmNOERROR);
}
