/*
 *   $RCSfile: clog_ForceLog.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:16 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lsn.h"
#include "clog.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "bf_extfuncs.h"
#include "clog_globals.h"


 int
clog_ForceLog (
	SERVERINFO	*serverInfo,
    FORCEMARK	forceMark 
)
{

    PAGEHASH	*pageHash;
    GROUPLINK	*groupLink;
    LOGINFO 	*logInfo = &(serverInfo->logInfo);

	if((serverInfo->flags & SERV_LOGGING_INITD)==0) {
		return esmNOERROR;
	}

    TRPRINT(TR_LOG, TR_LEVEL_1, ("target force mark:%d", forceMark));

    /* check to see if we need to force */
    if (logInfo->forceMark > forceMark) {
        /* force criteria has already been met */
        return(esmNOERROR);
    }

    /*
     *  go through the replaceable queue forcing pages
     */
    for (groupLink = 
	 (GROUPLINK *) FIRST_LIST_ELEMENT( &(logInfo->writeGroup->unfixedList));
         groupLink != NULL;
         groupLink=(GROUPLINK *)NEXT_LIST_ELEMENT(&(groupLink->groupList))) {

        /* check the grouplink magic */
        CHECK_GROUPLINK_MAGIC(groupLink);

        /* get a pointer to the pagehash structure */
        pageHash = groupLink->pageHash;
        CHECK_PAGEHASH_MAGIC(pageHash);
        TRPRINT(TR_LOG, TR_LEVEL_2, ("buf mark:%d log mark:%d",
                pageHash->forceMark, logInfo->forceMark));

        /* check to make sure this is a log buffer */
        SM_ASSERT(LEVEL_3, pageHash->flags & H_LOG);

        /* check to see if we have passed the force mark */
        if ((pageHash->forceMark <= forceMark) || 
				(pageHash->firstMark <= forceMark))   {

            /* check to see if the buffer is dirty */
            if (PAGE_IS_DIRTY(pageHash))    {

                /* check to make sure force marks align */
                SM_ASSERT(LEVEL_3, pageHash->forceMark >= logInfo->forceMark);

                /* force the page to the disk */
                bf_ForceLogPage(serverInfo, pageHash, NOFLAGS);

                /* reset the force mark */
                pageHash->forceMark = pageHash->firstMark = 0;
			} else {
				if (pageHash == logInfo->tailBuffer) {
					logInfo->tailBuffer = NULL;
				}
			}
        } else {

            /* break out of the loop */
            break;
        }
    }

    return(esmNOERROR);
}
