/*
 *   $RCSfile: clog_Initialize.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:16 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "clog.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"

    
/***************************************************************
 *  clog_Initialize
 * 	
 *  PURPOSE	
 *	Perform initial set-up for client-side logging operations.
 *	This includes allocating buffers, initializing the
 *	logInfo structure, etc.  This must be called when the 
 *	client is started if logging is to be performed.
 *  
 ***************************************************************/

int
clog_Initialize()
{
	if (OldData == NULL) {
		if ((OldData = (char *) malloc(RAISETWO(MAX_PAGE2SIZE) ) ) == NULL ) {
			SM_ERROR(TYPE_USER, esmMALLOCFAILED);
			return esmFAILURE;
		}
   	} 
    return esmNOERROR;
}

void
clog_Shutdown()
{
	SM_FREE_PTR(OldData);
}

int
clog_serverInitialize(
	SERVERINFO	*serverInfo
)
{
    LOGINFO *logInfo = &(serverInfo->logInfo);
    int pagesPerLogPage;
    int	logFileBlock2Size = (int)(logInfo->page2size);

	if( serverInfo->flags & SERV_LOGGING_INITD) {
		/* clog init already done */
		return esmNOERROR;
	}

	/*
	 *	Set initLRC to point to the correct initialization lrc
	 *	for new pages.  This is determined by the server.
	 */
	if (logInfo->initLRCisLSN) {
		logInfo->initLRC = (LRC*) &(logInfo->lastTailLSN);
	} else {
		logInfo->initLRC = &LrcNone;
	}

    INIT_LOGINFO_MAGIC(logInfo);

    /* figure out the size of a log page */ 
    pagesPerLogPage = BLOCKCOUNT(logFileBlock2Size); 

	SM_ASSERT(LEVEL_1, (logInfo->writeGroup == NULL));
    if ((logInfo->writeGroup = 
		bf_OpenGroup((CLOG_BUFGROUP_SIZE * pagesPerLogPage),
					BF_LRU, NOFLAGS)) == NULL) {
        return esmFAILURE;
    }

    /* mark buffer group as a log group */
    logInfo->writeGroup->flags |= (LOG_GROUP|FORCE_GROUP);

    /* initialize the logInfo structure */

	/* The volid should already have been initialized in msg_initClient() */
    SM_ASSERT(LEVEL_1, (logInfo->volid != (VOLID)0));

    logInfo->logFileAddr = 0;
    logInfo->writePages  = CLOG_BUFGROUP_SIZE;
	/*
    logInfo->page2size   = (UONE) logFileBlock2Size;
	*/
    logInfo->pageSize    = RAISETWO(logFileBlock2Size);
    logInfo->pageMask = logInfo->pageSize - 1;
    logInfo->usableBytes = logInfo->pageSize - (2 * sizeof(LOGPAGEHDR));
    logInfo->lastUsableByte = logInfo->pageSize - sizeof(LOGPAGEHDR);
    logInfo->tailPid     = NULLPID;
    logInfo->tailLSN     = 0;
    logInfo->tailBuffer  = NULL;
    logInfo->tailLink    = NULL;
    logInfo->forceMark = 0;
    logInfo->logRecordCount = 2;
    logInfo->flags       = NOFLAGS;

	serverInfo->flags |= SERV_LOGGING_INITD;
    return esmNOERROR;
}
