/*
 *   $RCSfile: clog_LogCreateIndex.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:17 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "logrecs.h"
#include "logaction.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"


int
clog_LogCreateIndex(
	PID*			pid,
	PAGEHASH*		pHash
)
{
	INDEXDESCPAGE* p = (INDEXDESCPAGE*) pHash->pageInfo->bufFrame;
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
					ALIGNSIZE(sizeof(p->desc));
	CLIENTLOGRECORDINFO recInfo;
	LRC 		lrc;
	int 		spaceLeft;
	FORCEMARK 	forceMark;
	LOGINFO		*logInfo = GET_LOGINFO;

	SM_ASSERT(LEVEL_3, p->magic == NDXMAGIC);
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_CREATE_INDEX;
	recInfo.actionPid = *pid;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	recInfo.imageSize[0] = sizeof(p->desc);
	recInfo.imageData[0] = (UONE *) & p->desc;
	

    if ( ! logInfo->tailBuffer)
        CLOG_GetNewLogPage();

    lrc = p->lrc;

    spaceLeft = 
		logInfo->lastUsableByte - 
		(int)(logInfo->tailLSN & logInfo->pageMask);

    if (fixedSize > spaceLeft)  {
        CLOG_GetNewLogPage();
        spaceLeft = logInfo->usableBytes;
        }

    spaceLeft -= fixedSize;

    SM_ASSERT(LEVEL_1, spaceLeft >= 0);

    forceMark = CLOG_WriteLogRecord(TransRec.clientTid, &recInfo, &lrc, pHash);

    SM_ASSERT(LEVEL_1, forceMark >= 0);

    pHash->forceMark = forceMark;

	p->lrc = lrc;

	return(esmNOERROR);
}
