/*
 *   $RCSfile: clog_LogModifyLgPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:17 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "clog.h"
#include "loginfo.h"
#include "logrecs.h"
#include "logaction.h"
#include "volume.h"
#include "trans.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"


int
clog_LogModifyLgPage(

    OPERATIONINFO	*opInfo 
)
{

    LRC *lrcPtr;
    int recSize;
    int fixedSize;
    int bodySize;
    CLIENTLOGRECORDINFO recInfo;
    register LOGINFO *logInfo;
    LGWRITEINFO writeInfo;
    LSNOFFSET  tailLSN;
    PAGEHASH *pageHash;
    int	offset;
    int spaceLeft, writtenSize, imageSize, usableBytes, leftOver;
    FORCEMARK forceMark;
    BOOL	isDelete,dataOp;
	int	*prevSizePtr;

/***************************************************************
 *  This function is used for logging removes, inserts, and appends
 * 	 for data on a leaf page of a large object or slots on
 *	 a large object index node.
 *
 * 	NOTE: To determine the number of slots in the case of updates
 *		to index nodes, divide the data size by sizeof(LGSLOT).
 *
 *	NOTE: Removes are not deletes because they do not involve
 *		any shifting of data after the deleted region.  They
 *		are used merely to log data that may eventually be
 *		overwritten.
 *
 *  For INSERTS, the format of a log record is:
 * 	image[0] - PID,start location, length, rootSlot, prevSize
 *	image[1] - new data 
 *
 *  For APPENDS, the format of a log record is:
 * 	image[0] - PID,start location, length, rootSlot, NULL
 *	image[1] - new data 
 *
 *  For REMOVES, the format of a log record is:
 * 	image[0] - PID,start location, length, rootSlot, NULL
 *	image[1] - old data 
 *
 *  Note: rootSlot will be NIL unless the affected object is a root object
 *		stored on a slotted page.
 *
 *
 *  For INSERTS, Additional log records are INSERTS 
 *  For REMOVES, Additional log records are REMOVES
 *  For APPENDS, Additional log records are APPENDS
 ***************************************************************/
     
    /* Get register pointer to log info structure */
    logInfo = GET_LOGINFO;

    SM_ASSERT(LEVEL_3, (opInfo->size + opInfo->start) <= LGDATA_USABLESIZE);


    /*
	 *	first, make sure that we have a valid log buffer at the tail
	 *	of the log.  Get a new log page if this isn't the case.
	 */
    if (logInfo->tailBuffer == NULL) {
        CLOG_GetNewLogPage();
    }

    switch(opInfo->action) {
	case(LOG_ACTION_REMOVE_LARGE_DATA):
	case(LOG_ACTION_DELETE_LARGE_DATA):
	    isDelete = TRUE;
	    dataOp = TRUE;
	    break;
	case(LOG_ACTION_APPEND_LARGE_DATA):
	    isDelete = FALSE;
	    dataOp = TRUE;
	    break;
	case(LOG_ACTION_INSERT_LARGE_DATA):
	    isDelete = FALSE;
	    dataOp = TRUE;
	    break;
	case(LOG_ACTION_REMOVE_LARGE_SLOTS):
	case(LOG_ACTION_DELETE_LARGE_SLOTS):
	    isDelete = TRUE;
	    dataOp = FALSE;
	    break;
	case(LOG_ACTION_APPEND_LARGE_SLOTS):
	case(LOG_ACTION_INSERT_LARGE_SLOTS):
	    isDelete = FALSE;
	    dataOp = FALSE;
	    break;
	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

    /*
     * determine size of the log record 
     */

    /* fixed part of log record is aligned record header, aligned
	write info and enough space to align new image */
    fixedSize = (ALIGN(sizeof(LOGRECORDHDR),BYTESPERWORD)) +
		(ALIGN(sizeof(LGWRITEINFO),BYTESPERWORD)) + BYTESPERWORD;
		
    /* data portion = object data */
    bodySize = (int)opInfo->size;

    recSize = bodySize + fixedSize;

    /*
     * set up record info fields which are common to all
     */

    recInfo.type        = LOG_REC_TYPE_USER;
    recInfo.action      = opInfo->action;
    recInfo.flags       = NOFLAGS;
    recInfo.imageCount  = 2;
    recInfo.imageSize[0] = sizeof(LGWRITEINFO);
    recInfo.imageData[0] = (UONE *) &writeInfo;
    recInfo.actionPid = opInfo->actionPid;

    if ((pageHash = bf_FindPage( &(recInfo.actionPid))) == NULL) {
       return(esmFAILURE);
    }
	SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));

	lrcPtr = bf_lrcLoc(pageHash);

#ifdef DEBUG
    /* for large data pages, the lrc is stored in the pageHash */
    if (dataOp == TRUE) {
		SM_ASSERT(LEVEL_1, (lrcPtr == &(pageHash->lrc)));
    } else {
		if (opInfo->rootSlot == NIL) {
            SM_ASSERT(LEVEL_1, (lrcPtr == &(((LGNODE *) (pageHash->pageInfo->bufFrame))->header.lrc)));
		} else {
			SM_ASSERT(LEVEL_1, (lrcPtr == &(((SLOTTEDPAGE *) (pageHash->pageInfo->bufFrame))->header.lrc)));
		}
    }
#endif DEBUG

    /*
     * determine placement of log record(s) on the page(s)
     */
    
    tailLSN = logInfo->tailLSN;
    offset = (int) (tailLSN & logInfo->pageMask);
    spaceLeft = logInfo->lastUsableByte - offset;

    /* may need to split log record */
    usableBytes = (logInfo->usableBytes - fixedSize);

    /* now start writing out log records */
    writtenSize = 0;

    while (bodySize > 0) {
        leftOver = (bodySize % usableBytes) + fixedSize;
	
	    /* see if we can use space on current log page. */

        if (leftOver > spaceLeft) {

            /* can't take advantage of space in current page, move on to next*/
	        CLOG_GetNewLogPage();
	        spaceLeft = logInfo->usableBytes;
	    }
	    
	    /* compute amount of space available for new Data */
	    imageSize = MIN((spaceLeft - fixedSize),bodySize);
	    bodySize -= imageSize;

	    /*adjust space left to account for the size of this record */
	    spaceLeft -= (imageSize + fixedSize);

	    /* check the size calculation*/
	    SM_ASSERT(LEVEL_1, spaceLeft >= 0);

	    /* prepare to write out this log record */
	    if (isDelete) {
			writeInfo.start = (UTWO) (opInfo->start + opInfo->size - writtenSize - imageSize);
	    } else {
			writeInfo.start = (UTWO) (writtenSize + opInfo->start);
		}
        writeInfo.rootSlot = (TWO) opInfo->rootSlot;

		if (opInfo->action == LOG_ACTION_INSERT_LARGE_DATA) { 
			prevSizePtr = (int*) opInfo->oldData; 
            writeInfo.prevSize = (UTWO) (*prevSizePtr + writtenSize);
		} else if (opInfo->action == LOG_ACTION_DELETE_LARGE_DATA) {
			prevSizePtr = (int*) opInfo->newData; 
            writeInfo.prevSize = (UTWO) (*prevSizePtr - writtenSize);
        }
	    if (isDelete) {
            //recInfo.imageData[1] = (UONE*) opInfo->oldData + writtenSize;
			recInfo.imageData[1] = opInfo->oldData + opInfo->size - writtenSize - imageSize;

	    } else {
            recInfo.imageData[1] = (UONE*) opInfo->newData + writtenSize;
	    }
	    writeInfo.size = (UTWO) imageSize;
        recInfo.imageSize[1] = imageSize;

	    writtenSize += writeInfo.size;

	    /* write the log record*/
	    forceMark = CLOG_WriteLogRecord(TransRec.clientTid, &recInfo,lrcPtr,pageHash);
    }

    /* mark the page dependency with the last log record written */
     pageHash->forceMark = forceMark;

     return esmNOERROR;
}



int
clog_LogModifyLgPageND(

    OPERATIONINFO	*opInfo 
)
{

    LRC *lrcPtr;
    int recSize;
    int fixedSize;
    CLIENTLOGRECORDINFO recInfo;
    register LOGINFO *logInfo;
    LGWRITEINFO writeInfo;
    LSNOFFSET  tailLSN;
    PAGEHASH *pageHash;
    int	offset;
    int spaceLeft;
    FORCEMARK forceMark;

/***************************************************************
 *  This function is used for logging inserts, and appends
 * 	 for zero'd data on a leaf page of a large object or slots on
 *	 a large object index node.
 *
 * 	NOTE: To determine the number of slots in the case of updates
 *		to index nodes, divide the data size by sizeof(LGSLOT).
 *
 *	NOTE: Removes are not deletes because they do not involve
 *		any shifting of data after the deleted region.  They
 *		are used merely to log data that may eventually be
 *		overwritten.
 *
 *  For INSERTS, the format of a log record is:
 * 	image[0] - PID,start location, length, rootSlot, prevSize
 *	image[1] - new data 
 *
 *  For APPENDS, the format of a log record is:
 * 	image[0] - PID,start location, length, rootSlot, NULL
 *	image[1] - new data 
 *
 *  Note: rootSlot will be NIL unless the affected object is a root object
 *		stored on a slotted page.
 *
 *
 *  For INSERTS, Additional log records are INSERTS 
 *  For APPENDS, Additional log records are APPENDS
 ***************************************************************/

	SM_ASSERT(LEVEL_3, opInfo->newData == NULL);
     
    /* Get register pointer to log info structure */
    logInfo = GET_LOGINFO;

    /*
	 *	first, make sure that we have a valid log buffer at the tail
	 *	of the log.  Get a new log page if this isn't the case.
	 */
    if (logInfo->tailBuffer == NULL) {
        CLOG_GetNewLogPage();
    }

	SM_ASSERT(LEVEL_3,
				opInfo->action == LOG_ACTION_APPEND_LARGE_DATA_ND ||
				opInfo->action == LOG_ACTION_INSERT_LARGE_DATA_ND);

    /*
     * determine size of the log record 
     */
    /* fixed part of log record is aligned record header, aligned
	write info and enough space to align new image */
    fixedSize = (ALIGN(sizeof(LOGRECORDHDR),BYTESPERWORD)) +
		(ALIGN(sizeof(LGWRITEINFO),BYTESPERWORD)) + BYTESPERWORD;
    recSize = fixedSize;

    /*
     * set up record info fields which are common to all
     */
    recInfo.type        = LOG_REC_TYPE_USER;
    recInfo.action      = opInfo->action;
    recInfo.flags       = NOFLAGS;
    recInfo.imageCount  = 1;
    recInfo.imageSize[0] = sizeof(LGWRITEINFO);
    recInfo.imageData[0] = (UONE *) &writeInfo;
    recInfo.actionPid = opInfo->actionPid;

    if ((pageHash = bf_FindPage( &(recInfo.actionPid))) == NULL) {
       return(esmFAILURE);
    }
	SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));
    
	lrcPtr = bf_lrcLoc(pageHash);
	SM_ASSERT(LEVEL_1, (lrcPtr == &(pageHash->lrc)));

    /*
     * determine placement of log record(s) on the page(s)
     */
    tailLSN = logInfo->tailLSN;
    offset = (int) (tailLSN & logInfo->pageMask);
    spaceLeft = logInfo->lastUsableByte - offset;

	if (recSize > spaceLeft) {

		/* can't take advantage of space in current page, move on to next*/
		CLOG_GetNewLogPage();
		spaceLeft = logInfo->usableBytes;
	}
	    

	/* prepare to write out this log record */
	writeInfo.start = (UTWO) opInfo->start;
	writeInfo.prevSize = 0;

	writeInfo.size = (UTWO) opInfo->size;

	/* write the log record*/
	forceMark = CLOG_WriteLogRecord(TransRec.clientTid, 
		&recInfo,lrcPtr,pageHash);

    /* mark the page dependency with the last log record written */
     pageHash->forceMark = forceMark;

     return esmNOERROR;
}
