/*
 *   $RCSfile: clog_LogWriteLgNodeHdr.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:17 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "logrecs.h"
#include "logaction.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "clog_intfuncs.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"


int
clog_LogWriteLgNodeHdr(

    OPERATIONINFO	*opInfo 
)
{

    LRC 		*pageLRC;
    int 		fixedSize;
    CLIENTLOGRECORDINFO 	recInfo;
    register LOGINFO 	*logInfo;
    LSNOFFSET  		tailLSN;
    PAGEHASH 		*pageHash;
    int				offset;
    int 			spaceLeft;
    FORCEMARK 		forceMark;

/***************************************************************
 *  This function is used for logging a new large index node header
 *	or a change to an existing one.
 * 
 * 	image[0] - rootSlot (slot if root node on slotted page)
 *	image[1] - new node header 
 *	image[2] - old node header (except for init)
 *
 *  All log records are fixed length 
 ***************************************************************/
     
    /* Get register pointer to log info structure */
    logInfo = GET_LOGINFO;

    /* first, make sure that we have a valid log buffer at the tail of the
       log.  Get a new log page if this isn't the case.*/

    if (logInfo->tailBuffer == NULL) {
        CLOG_GetNewLogPage();
    }


    /*
     * determine size of the log record 
     */

    /* log record is fixed length: a log record header, oid and 
	the before and after images of the object header*/
    fixedSize = (ALIGN(sizeof(LOGRECORDHDR),BYTESPERWORD)) +
		(ALIGN(sizeof(PID),BYTESPERWORD)) +
		(ALIGN(sizeof(LGNODEHDR),BYTESPERWORD)); 

    recInfo.type        = LOG_REC_TYPE_USER;
    recInfo.action      = opInfo->action;
    recInfo.flags       = NOFLAGS;
    recInfo.imageCount  = 2;
    recInfo.imageSize[0] = sizeof(opInfo->rootSlot);
    recInfo.imageData[0] = (UONE *) &(opInfo->rootSlot);
    recInfo.imageSize[1] = sizeof(LGNODEHDR);
    recInfo.imageData[1] = (UONE *) opInfo->newData;
    recInfo.actionPid = opInfo->actionPid;

    /* for non-inits, need to log old value as well */
    if (recInfo.action != LOG_ACTION_INIT_LARGE_NODE) {
        recInfo.imageCount++; 
        recInfo.imageSize[2] = sizeof(LGNODEHDR);
        recInfo.imageData[2] = (UONE *) opInfo->oldData;
  	fixedSize += (ALIGN(sizeof(LGNODEHDR),BYTESPERWORD)); 
    }

    if ((pageHash = bf_FindPage( &(recInfo.actionPid))) == NULL) {
       return(esmFAILURE);
    }
	SM_ASSERT( LEVEL_1, !(pageHash->flags & H_INVALID));

    if (opInfo->rootSlot == NIL) {
	pageLRC = &(((LGNODE *) (pageHash->pageInfo->bufFrame))->header.lrc);
    } else {
	pageLRC	= &(((SLOTTEDPAGE *)(pageHash->pageInfo->bufFrame))->header.lrc);
    }

    /*
     * determine placement of log record(s) on the page(s)
     */
    
    tailLSN = logInfo->tailLSN;
    offset = (int) (tailLSN & logInfo->pageMask);
    spaceLeft = logInfo->lastUsableByte - offset;

    /* see if we can use space on current log page. */
    if (fixedSize > spaceLeft) {

        /* can't take advantage of space in current page, move on to next*/
	CLOG_GetNewLogPage();
        spaceLeft = logInfo->usableBytes;
    }
	    
    spaceLeft -= fixedSize;

    /* check the size calculation*/
    SM_ASSERT(LEVEL_1, spaceLeft >= 0);


    /* write the log record*/
    forceMark = CLOG_WriteLogRecord(TransRec.clientTid, &recInfo, pageLRC, pageHash);
    pageHash->forceMark = forceMark;

    return esmNOERROR;
}
