/*
 *   $RCSfile: clog_WriteLogPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:17 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "loginfo.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "logrecs.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "util_funcs.h"
#include "clog_extfuncs.h"
#include "io_extfuncs.h"
#include "clog_globals.h"
#include "trans_globals.h"

    
/***************************************************************
 *  clog_WriteLogPage
 * 	
 *  PURPOSE	
 *	
 *	Used when a log page needs to be written to the server.
 *	Insures that the log tail is marked as NULL if the page
 *	being written is the tail of the log.
 *  
 ***************************************************************/

int
clog_WriteLogPage(
	SERVERINFO	*serverInfo,
    PAGEHASH	*pageHash 
)
{
    LOGINFO 	*logInfo = &(serverInfo->logInfo);

    if (io_WriteLogPage(serverInfo, pageHash->pageInfo->bufFrame, pageHash->page2size )) {
        return(esmFAILURE);
    }
 
    /* if this is the tail of the log, change loginfo to reflect that the
       tail is no longer there */

    if (pageHash == logInfo->tailBuffer) {
        logInfo->tailBuffer = NULL;
    }

	return(esmNOERROR);
}
