/*
 *   $RCSfile: fi_CreateFile.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:19 $
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "file.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "fi_extfuncs.h"
#include "msg_funcs.h"
#include "trans_globals.h"


 int
fi_CreateFile (

	VOLID			volid,
	PID				*nearPid,
	FID				*fid,
	FLAGS			flags 
)
{

	register VOLREC	*volRec;
	MESSAGE			message;


	TRPRINT(TR_FI, TR_LEVEL_1, ("volid:%d near:%d", volid, nearPid->page));

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 */
	message.header.type			   = CREATE_FILE;
	message.header.replyRequested   = TRUE;
	message.header.params.in.tid	= GET_SERVERTID;
	message.body.file.fid.pid.volid = volid;
	message.body.file.flags		= flags;

	/*
	 *	check to see if a nearpid is given
	 */
	if (nearPid != NULL)	{

		message.body.file.fid.pid.page = nearPid->page;

	} else {

		message.body.file.fid.pid.page = NULLPID;
	}

	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

		return(esmFAILURE);

	} 
	*fid = message.body.file.fid;		
	return(esmNOERROR);
}
