/*
 *   $RCSfile: fi_NeighborPid.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:19 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "lm.h"
#include "lsn.h"
#include "loginfo.h"
#include "clog.h"
#include "file.h"
#include "io_extfuncs.h"
#include "fi_extfuncs.h"
#include "bf_extfuncs.h"
#include "bf_intfuncs.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "trans_globals.h"
#include "clog_globals.h"
#include "io_globals.h"
#include "sm_macro.h"


 int
fi_NeighborPid (
	PID				*currentPid,
	FID				*fid,
	SHORTPID		**begin,
	SHORTPID		**end,
	FLAGS			direction,
	PID				*nextPid
)
{
	register VOLREC	*volRec;
	MESSAGE			message;

	TRPRINT(TR_FI, TR_LEVEL_1, ("pid %d.%d, fid:%d.%d, flags 0x%x", 
		currentPid->page, currentPid->volid, 
		fid->pid.page, fid->pid.volid, direction));

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(fid->pid.volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}
	message.header.type	  	   = NEIGHBOR_PAGE;
	message.header.replyRequested	 = TRUE;
	message.header.params.in.tid	 = GET_SERVERTID;

	message.body.file.fid 	    = *fid;
	message.body.file.pid 		= *currentPid;
	SM_ASSERT(LEVEL_1, (fid->pid.volid == currentPid->volid));

	message.body.file.flags	   = direction ;
		/* do not use REPLY_WITH_DATA or REPLY_WITH_NEIGHBOR */
	if(begin != NULL)
		message.body.file.flags	   |= REPLY_WITH_DATA;

	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{
		return(esmFAILURE);
	} 
	if(nextPid != NULL)
		*nextPid = message.body.page.NeighborPid;

	if( message.body.page.flags & REPLY_WITH_DATA ) {
		int howMany;
		/* Server returned a set of pids in addition to the single
		 * neighbor pid.  Free old malloced space.
		 */

		TRPRINT(TR_FI, TR_LEVEL_1, ("SET OF neighbor pids returned"));

		SM_ASSERT(LEVEL_1, ((begin != NULL) && (end!=NULL)));

		if (receive(volRec->serverInfo, (char *)&howMany, sizeof(howMany)) < 0){
			return(esmFAILURE);
		}
		if(*begin) {
			SM_FREE_PTR((*begin));
			*begin = *end =  NULL;
		}

		if( ((*begin) = (SHORTPID *)malloc(howMany * sizeof(SHORTPID)))
									== NULL) {
			return esmMALLOCFAILED;
		}

		if (receive(volRec->serverInfo, 
			(char *)(*begin), (howMany * sizeof(SHORTPID))) < 0) {
			return esmFAILURE;
		}
		*end = *begin + howMany - 1;

	} else {
		SM_ASSERT(LEVEL_1, (begin == NULL));
	}
	return esmNOERROR;
}
