/*
 *   $RCSfile: fi_NumPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:19 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "file.h"
#include "sm_state.h"
#include "fi_extfuncs.h"
#include "io_extfuncs.h"
#include "lm_extfuncs.h"
#include "msg_funcs.h"
#include "trans_globals.h"


 int
fi_NumPages(

	FID				*fid,
	int*			numPages
	
)
{
	MESSAGE			message;
	VOLREC			*volRec;

	TRPRINT(TR_FI, TR_LEVEL_1, ("fid:%d", fid->pid.page));

    /* check to see if the volume is mounted */
    if ((volRec = io_FindVolId(fid->pid.volid)) == NULL) {
        SM_ERROR(TYPE_USER, esmBADVOLID);
        return(esmFAILURE);
    }
	
	if (lm_LockFile(fid, SH))  {
		return esmFAILURE;
		}

	/* load up a server message */
	message.header.type	  = FILE_NUMPAGES;
	message.header.replyRequested	 = TRUE;
	message.header.params.in.tid	  = GET_SERVERTID;
	message.body.fileNumPages.fid = *fid;

	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

		return(esmFAILURE);

	}
	
	*numPages = (int) message.body.fileNumPages.numPages;
	return(esmNOERROR);
}
