/*
 *   $RCSfile: fi_Page.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:19 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "bf.h"
#include "lock.h"
#include "msgdefs.h"
#include "volume.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "trans.h"
#include "scan.h"
#include "file.h"
#include "fi_macro.h"
#include "io_extfuncs.h"
#include "fi_extfuncs.h"
#include "msg_funcs.h"
#include "trans_globals.h"


/*
 *	fi_Page() finds the first/last physical/logical page in a file.
 *	It attempts to use any hints in the file table, and updates them
 *	if they are invalid.
 */


 int
fi_Page (

	FID				*fid,
	PID				*pid,
	FLAGS			flags
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	SHORTPID		*hint;
	FILEINFO		*fileInfo;

	TRPRINT(TR_FI, TR_LEVEL_1, ("fid:%d, pid %d.%d, flags 0x%x", 
		fid->pid.page, pid->page, flags));

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(fid->pid.volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	fileInfo = fi_FindFileInfo(fid, TRUE);
	if (fileInfo == NULL) return esmFAILURE;

	/*
	 *	load up a disk message
	 */
	if( flags & LOGICAL_SCAN ) {
		if(flags & FI_FIRST) {
			message.header.type	  = FIRST_LOGICAL_PAGE;
			hint = &(fileInfo->firstPageHint);
		} else {
			SM_ASSERT(LEVEL_1, flags & FI_LAST);
			message.header.type	  = LAST_LOGICAL_PAGE;
			hint = &(fileInfo->lastPageHint);
		}
	} else { 
		SM_ASSERT(LEVEL_1, flags & PHYSICAL_SCAN);
		if(flags & FI_FIRST) {
			message.header.type	  = FIRST_PHYSICAL_PAGE;
			hint = &(fileInfo->firstPageHint);
		} else {
			SM_ASSERT(LEVEL_1, flags & FI_LAST);
			message.header.type	  = LAST_PHYSICAL_PAGE;
			hint = &(fileInfo->lastPageHint);
		}
	}

	if ( *hint != NULLPID) {

		/* the hint is valid, so use it */
		pid->page = *hint;
		pid->volid = fid->pid.volid;

	} else {

		/* the hint is invalid, to ask the server */
		message.header.replyRequested	 = TRUE;
		message.header.params.in.tid	  = GET_SERVERTID;
		message.body.file.fid = *fid;

		/*
		 *	write the read message
		 */
		if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

			return(esmFAILURE);

		} 
		pid->page = message.body.file.pid.page;
		pid->volid = fid->pid.volid;

		/* this is the new hint */
		*hint = pid->page;
	}

	TRPRINT(TR_FI, TR_LEVEL_1, ("first/last pid:%d", pid->page));
	return(esmNOERROR);
}
