#ifndef _SM_STATE_H_
#define _SM_STATE_H_
/*
 *   $RCSfile: sm_state.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:20 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "forward.h"

/*
 * This file contains structures used to maintain state in the client.
 * Whenever an sm_*() function is called a global state variable is
 * initialized.
 * As information about logging, servers, etc involved in the 
 * operation is obtained, it is added to the state variable.
 * This eliminates passing extra parameters to called functions.
 */

/* BEGIN visible to user */

/*
 * Logging level flags
 */
#define	LOG_NONE	0x0
#define	LOG_SPACE	0x1
#define	LOG_DATA	0x2
#define LOG_ALL		(LOG_DATA | LOG_SPACE)

/* END visible to user */

typedef struct SMSTATE {
	SERVERINFO			*serverInfo;
	struct LogInfo_s	*logInfo;
	VOLREC				*volRec;
	FLAGS				logLevel;
	FLAGS				defaultLogLevel;
	TID					serverTid; /* temporary kludge */

	/* put new fields over these to avoid recompilation */ 
	int					filler1;
	int					filler2;
	int					filler3;
	/* file info */
	/* trans info */
} SMSTATE;


#define INIT_SMSTATE(_smstate)	\
	(_smstate)->serverInfo = NULL;	\
	(_smstate)->logInfo = NULL;	\
	(_smstate)->volRec = NULL;	\
	(_smstate)->serverTid = NULL_TID;	\
	(_smstate)->logLevel = LOG_ALL;	

#define GET_LOGINFO (SM_State.logInfo)
#define GET_LOGLEVEL(_log) (SM_State.logLevel & (_log))
#define GET_SERVERINFO (SM_State.serverInfo)
#define GET_SERVERTID (SM_State.serverTid)


extern SMSTATE	SM_State;
#endif /* _SM_STATE_H_ */
