#ifndef _SM_USER_H_
#define _SM_USER_H_
/*
 *   $RCSfile: sm_user.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:20 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

/* BEGIN visible to user */

/* use as a parameter on object reads to specify entire object */
#define READ_ALL -1

/* use as a parameter on object delete to specify rest of object */
#define DELETE_ALL -1


/*
 *	Macros to set invalid (null) object, index, file, transaction,
 *	cursor, and buffer group ID's
 */
#define INVALIDATE_OID(_oid)    ((_oid).diskAddr.volid = 0)
#define OID_IS_INVALID(_oid)    ((_oid).diskAddr.volid == 0)
#define INVALIDATE_IID(_iid)    ((_iid).pid.volid = 0)
#define IID_IS_INVALID(_iid)    ((_iid).pid.volid == 0)
#define INVALIDATE_FID(_fid)	((_fid).pid.volid = 0)
#define FID_IS_INVALID(_fid)    ((_fid).pid.volid == 0)
#define INVALIDATE_TID(_tid)	((_tid) = 0)
#define TID_IS_INVALID(_tid)    ((_tid) < 1)
#define INVALIDATE_BUFGROUP(_buf)	((_buf) = -1)
#define BUFGROUP_IS_INVALID(_buf)	((_buf) < 0)
#define INVALIDATE_CURSOR(_cur)	(INVALIDATE_BUFGROUP((_cur).groupIndex))
#define CURSOR_IS_INVALID(_cur) (BUFGROUP_IS_INVALID((_cur).groupIndex))
#define INVALIDATE_VOLID(_volid)  (_volid = 0)
#define VOLID_IS_INVALID(_volid)  (_volid == 0)


/*
 *	Globals required for transaction macros	
 */
extern jmp_buf	sm_TransJmpBuf;
extern int		*sm_TransReturnCode;					


#define SM_BEGIN_TRANSACTION(_tid, _retCodeVariable)				\
	do{														\
		sm_TransReturnCode = &(_retCodeVariable);			\
		*sm_TransReturnCode = esmNOERROR;					\
		if (sm_BeginTransaction((_tid))  != esmNOERROR) {   \
			*sm_TransReturnCode = sm_errno;					\
			break;											\
		}													\
		if (setjmp(sm_TransJmpBuf) != 0) {					\
			sm_AbortTransaction(*(_tid));		\
			break;											\
		}	

#define SM_COMMIT_TRANSACTION(_tid)							\
	} while (0);											\
	if (*sm_TransReturnCode == esmNOERROR) {				\
		if (sm_CommitTransaction((_tid)) != esmNOERROR) {	\
			*sm_TransReturnCode = sm_errno;					\
			sm_AbortTransaction((_tid));		\
		} else {											\
			*sm_TransReturnCode = esmNOERROR;				\
		}													\
	}

#define SM_ABORT_TRANSACTION(_retCode)						\
	{														\
		*sm_TransReturnCode = (_retCode);					\
		longjmp(sm_TransJmpBuf, 0);							\
	}

/*
 *	Example use of these functions
 *
		#include <setjmp.h>

		SM_USE_TRANSACTION_MACROS

		main()
		{
			int abort;
			TID	tid;

			printf("outside\n");

			SM_BEGIN_TRANSACTION(&tid, abort)

				f();
				printf("inside\n");
			
			SM_COMMIT_TRANSACTION()
			if (abort) {
				printf("aborted\n");
			} else {
				printf("commited\n");
			}
		}

		f()
		{
			int a;

			printf("abort? ");
			scanf("%d", &a);
			if (a) {
				SM_ABORT_TRANSACTION(1)
			}
		}

	 *
	 *	End of example
	 */

/* END visible to user */
#endif /* _SM_USER_H_ */
