/*
 *   $RCSfile: io_AllocPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:20 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "sm_state.h"
#include "lsn.h"
#include "loginfo.h"
#include "serverinfo.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "trans_globals.h"

BEGIN_EXTERNC
	extern void     sm_SetAbortIfError(void);
END_EXTERNC

 int
io_AllocPages (

	VOLID			volid,
	PAGE2SIZE		page2size,
	int				numPages,
	PID				*nearPid,
	PID				pageList[],
	int				pageType 
)
{

	VOLREC			*volRec;
	MESSAGE			message;
	int				numToAlloc;
	int				numAlloced;

	TRPRINT(TR_IO, TR_LEVEL_1, ("volid:%d allocating:%d page2size:%d pages near:%d",
			volid, numPages, page2size, 
			nearPid==NULL?-1:nearPid->page));

	FAKEFAIL(esmDISKFULL);

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}


	/*
	 *	Check to see if the near pid is null
	 */
	if (nearPid != NULL)	{

		message.body.alloc.nearPid = *nearPid;

	} else {

		message.body.alloc.nearPid.page = NULLPID;
	}

	/* 
	 *	Allocate a max of MAX_PIDLIST_SIZE pages per message 
	 */
	numToAlloc = numPages;
	numAlloced = 0;
	if (numToAlloc > 0) {
		sm_SetAbortIfError();
	}
	while (numToAlloc > 0) {

		/*
		 *	load up a disk message
		 */
		message.header.type			 = ALLOC_PAGE;
		message.header.replyRequested = TRUE;
		/* message.header.magic is set in callServer */
		message.header.params.in.tid	 = GET_SERVERTID;

		message.body.alloc.volid	 = volid;
		message.body.alloc.page2size = page2size;

		/*
		 * NB: message.body.alloc.flags is used only in the 
		 * io_AllocSlottedPages case, so we need not set it here.
		 */
		message.body.alloc.pageType	 = pageType;

		if (numToAlloc > MAX_PIDLIST_SIZE) {
			message.body.alloc.numPages	 = MAX_PIDLIST_SIZE;
		} else {
			message.body.alloc.numPages	 = numToAlloc;
		}

		if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{
			return(esmFAILURE);
		}

		/*
		 *	ALLOC_PAGE returns a list of PIDs.
		 */
		if (receive(volRec->serverInfo, (char *) (pageList+numAlloced), (sizeof(PID) * message.body.alloc.numPages)) < 0){

			return(esmFAILURE);
		}

#ifdef DEBUG
		{ 	register int i;
			for (i = 0; i < message.body.alloc.numPages ; i++ ) {
				TRPRINT(TR_IO, TR_LEVEL_1, 
				("allocated page:%d", pageList[i].page));
			}
		}
#endif DEBUG

		numToAlloc -= message.body.alloc.numPages;
		numAlloced += message.body.alloc.numPages;
	}

	SM_ASSERT(LEVEL_3, numAlloced == numPages);

	return(esmNOERROR);
}
