/*
 *   $RCSfile: io_AllocSlottedPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "msgdefs.h"
#include "volume.h"
#include "load.h"
#include "scan.h"
#include "trans.h"
#include "file.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "fi_extfuncs.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "trans_globals.h"


 int
io_AllocSlottedPages (

	int				numPages,
	PID				*nearPid,
	PID				pageList[],
	FID				*fid,
	FLAGS			flags,
	UNIQUE			*unique,
	BUFGROUP		*bufGroup 
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	PID				linkPid;
	FILEINFO		*fileInfo;

	TRPRINT(TR_IO, TR_LEVEL_1, ("volid:%d allocating:%d pages near:%d",
			fid->pid.volid, numPages, nearPid->page));

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(fid->pid.volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 */
	message.header.type			   = ALLOC_PAGE;
	message.header.replyRequested  = TRUE;
	/* message.header.magic is set in callServer */
	message.header.params.in.tid	= GET_SERVERTID;

	message.body.alloc.volid	   = fid->pid.volid;
	message.body.alloc.page2size   = SLOTTED_PAGE2SIZE;
	message.body.alloc.numPages	   = numPages;
	message.body.alloc.pageType	   = PAGE_SLOTTED;
	message.body.alloc.flags	   = flags;
	message.body.alloc.slotted.s_input.fid  = *fid;
	message.body.alloc.slotted.s_input.uniqueCount = UNIQUE_ALLOC_COUNT;

	/*
	 *	check to see if there is a near pid
	 */
	if (nearPid != NULL)	{

		message.body.alloc.nearPid = *nearPid;

	} else {

		message.body.alloc.nearPid.page = NULLPID;
	}

	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

		return(esmFAILURE);

	} 
	TRPRINT(TR_IO, TR_LEVEL_2, ("alloc succeeded"));

	/* 
	 * ALLOC_PAGE returns a list of PIDS
	 */

	if (receive(volRec->serverInfo, (char *)pageList, (sizeof(PID) * numPages)) < 0){

		return(esmFAILURE);
	}

	/*
	 *	check to see if the logical link must be updated
	 */
	if (message.body.alloc.slotted.s_output.logicalPredPid != NULLPID)	{

		/*
		 *	construct the pid
		 */
		linkPid.volid = fid->pid.volid;
		linkPid.page  = message.body.alloc.slotted.s_output.logicalPredPid;

		/*
		 *	mark the page
		 */
		bf_UpdateLogicalLink(bufGroup, &linkPid, pageList->page);
	}

	/*
	 *	extract the unique numbers
	 */
	*unique = message.body.alloc.slotted.s_output.unique;
	TRPRINT(TR_IO, TR_LEVEL_2, ("unique:%d", *unique));

	/*
	 *	Update hints about the last physical and logical pages in
	 *	the file.
	 */
	fileInfo = fi_FindFileInfo(fid, FALSE);
	if (fileInfo == NULL) SM_ERROR(TYPE_FATAL, esmINTERNAL);

	if (fileInfo->tempVolume) {
		   fileInfo->lastPageHint = pageList->page;
	} else {
	   if (pageList->page > fileInfo->lastPageHint) {
			   fileInfo->lastPageHint = pageList->page;
	   }
	}

	return(esmNOERROR);
}
