/*
 *   $RCSfile: io_DeallocPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:20 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "file.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_globals.h"

BEGIN_EXTERNC
	extern void     sm_SetAbortIfError(void);
END_EXTERNC

 int
io_DeallocPages (

	PAGE2SIZE		page2size,
	int				numPages,
	const PID		*pidList,
	BOOL			immediateDealloc,  /* tell server to dealloc now 
										 instead of at EOT			*/
	PAGETYPE		pageType
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	int				numToDealloc;
	int				numDealloced;


	TRPRINT(TR_IO, TR_LEVEL_1, ("deallocating:%d pages", numPages));

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(pidList->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 *  (MJZ: 6-23-90 -- added initialization of message.header.magic)
	 */
	message.header.type			 = DEALLOC_PAGE;
	message.header.replyRequested = TRUE;
	/* message.header.magic is set in callServer */
	message.header.params.in.tid  = GET_SERVERTID;
	message.body.alloc.page2size = page2size;
	message.body.alloc.flags	 = immediateDealloc; 
	message.body.alloc.pageType	 = pageType; 

	/*
	 *  Deallocate a max of MAX_PIDLIST_SIZE pages per message
	 */
	numToDealloc = numPages;
	numDealloced = 0;
	if (numToDealloc > 0) {
		sm_SetAbortIfError();
	}
	while (numToDealloc > 0) {


		if (numToDealloc > MAX_PIDLIST_SIZE) {
			message.body.alloc.numPages  = MAX_PIDLIST_SIZE;
		} else {
			message.body.alloc.numPages  = numToDealloc;
		}

		/*
		 *	Send the page list to the server
		 */
		if (callServer(volRec->serverInfo, &message,
						(char*) (pidList+numDealloced),
					 	(sizeof(PID) * message.body.alloc.numPages) ) != esmNOERROR)    {
			return(esmFAILURE);
		}

		numToDealloc -= message.body.alloc.numPages;
		numDealloced += message.body.alloc.numPages;

	}

	SM_ASSERT(LEVEL_3, numDealloced == numPages);

	return(esmNOERROR);
}



 int
io_DeallocSlottedPages (

	PID				*pid,
	FILEINFO		*fileInfo,
	BOOL			immediateDealloc  /* tell server to dealloc now 
										 instead of at EOT			*/
)
{

	register VOLREC	*volRec;
	MESSAGE			message;


	TRPRINT(TR_IO, TR_LEVEL_1, ("deallocating pid:%d:%d", (int) (pid->volid), pid->page ));

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(pid->volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 *  (MJZ: 6-23-90 -- added initialization of message.header.magic)
	 */
	message.header.type			 = DEALLOC_PAGE;
	message.header.replyRequested = TRUE;
	/* message.header.magic is set in callServer */
	message.header.params.in.tid  = GET_SERVERTID;
	message.body.alloc.page2size = SLOTTED_PAGE2SIZE;
	message.body.alloc.flags	 = immediateDealloc; 
	message.body.alloc.pageType	 = PAGE_SLOTTED; 
	message.body.alloc.slotted.s_input.fid  = fileInfo->fid;
	message.body.alloc.numPages  = 1;

	/*
	 *	Send the page list to the server
	 */
	if (callServer(volRec->serverInfo, &message, (char*) (pid),
					(sizeof(PID) * message.body.alloc.numPages) ) != esmNOERROR)    {
		return(esmFAILURE);
	}

	/* invalidate hints about the first and last page in the file */
	if (pid->page >= fileInfo->lastPageHint) {
		fileInfo->lastPageHint = NULLPID;
	}
	if (pid->page <= fileInfo->firstPageHint) {
		fileInfo->firstPageHint = NULLPID;
	}

	return(esmNOERROR);
}
