/*
 *   $RCSfile: io_GetRootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "util_funcs.h"
#include "trans_globals.h"


 int
io_GetRootEntry (

	VOLID			volid,
	char			*name,
	char			*data,
	int				*dataSize 
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	int				nameSize;
	char			tempData[MAX_ROOTDATA_SIZE];

	TRPRINT(TR_IO, TR_LEVEL_1, ("volid:%d", volid));

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 */
	message.header.type		= GET_ROOT_ENTRY;
	message.header.replyRequested = TRUE;
	/* message.header.magic is set in callServer */
	message.header.params.in.tid		= GET_SERVERTID;

	message.body.root.volid	= volid;

	nameSize = strlen(name);
	message.body.root.nameSize	= nameSize;

	if (callServer(volRec->serverInfo, &message, name, nameSize ) < 0)	{
		return(esmFAILURE);
    } 

	/* data returned constitute the root entry's "value" */

	if (receive(volRec->serverInfo, tempData, (int)message.body.root.dataSize) < 0){
		return(esmFAILURE);
	}

	if (*dataSize < 0) {
		*dataSize = (int) message.body.root.dataSize;
	}

	/* copy at most dataSize bytes to the user's data area */
	bcopy(tempData, data, MIN((int)message.body.root.dataSize, *dataSize));

	/* return the length of the data */
	*dataSize = (int) message.body.root.dataSize;

	return(esmNOERROR);
}
