/*
 *   $RCSfile: io_GetUnique.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:20 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_globals.h"

 int
io_GetUnique (

	PID				*pid,
	UNIQUE			*unique,
	int				count 
)
{

	register VOLREC	*volRec;
	MESSAGE			message;	


	TRPRINT(TR_IO, TR_LEVEL_1, ("getting unique for page:%d", pid->page));

	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolId(pid->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	load up a disk message
	 */
	message.header.type	 	 		= GET_UNIQUE;
	message.header.replyRequested   = TRUE;
	/* message.header.magic is set in callServer */
	message.header.params.in.tid	= GET_SERVERTID;

	message.body.unique.pid	 		= *pid;
	message.body.unique.uniqueCount = count;

	if (callServer(volRec->serverInfo, &message, NULL, 0 ) < 0)	{
		return(esmFAILURE);
	} 

	/*
	 *	put the unique in
	 */
	*unique = message.body.unique.unique;
	TRPRINT(TR_IO, TR_LEVEL_1, ("unique:%x", *unique));
	return(esmNOERROR);
}
