/*
 *   $RCSfile: io_ReadPageWithNeighbor.c,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 1996/05/04 21:55:21 $
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "bf.h"
#include "lock.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "lsn.h"
#include "loginfo.h"
#include "clog.h"
#include "chunk.h"
#include "load.h"
#include "scan.h"
#include "sm_params.h"
#include "btree.h"
#include "fi_msg.h"
#include "io_extfuncs.h"
#include "io_intfuncs.h"
#include "sm_globals.h"

 int
io_ReadPageWithNeighbor (

	const PID		*pid,
	char			*buffer,
	LOCKMODE		*lockMode,
	PAGEHASH		*pageHash,
	FLAGS			flags,
	FID				*fid,
	PID 			*neighborPid
)
{
	int err;

	/* server knows about flags BEFORE, AFTER, or ANY, and REPLY_WITH_* */ 
	if( flags & BF_NOREAD ) {
		flags &= ~BF_NOREAD; 
	} else {
		flags |= REPLY_WITH_DATA; 
	}
	if( flags & NEIGHBOR_FLAGS ) {
		flags |= REPLY_WITH_NEIGHBOR; 
	}

	err = IO_ReadPageWithNeighbor(pid, buffer, lockMode, pageHash,
						flags, fid, neighborPid);
	TRPRINT(TR_IO|TR_SM, TR_LEVEL_1,  
		("returning %d, sm_errno=%d, neighborPid %d.%d\n", 
		err, sm_errno, neighborPid?neighborPid->page:0,
		neighborPid?neighborPid->volid:0));
	return err;
}
