/*
 *   $RCSfile: io_SetRootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_funcs.h"
#include "util_funcs.h"
#include "trans_globals.h"


 int
io_SetRootEntry (

	VOLID			volid,
	char			*name,
	char			*data,
	int				dataSize 
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	char			msgSpace[MAX_ROOTNAME_SIZE+MAX_ROOTDATA_SIZE+2/*extra*/];
	int				nameSize;


	TRPRINT(TR_IO, TR_LEVEL_1, ("name:%s", name));

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	nameSize = strlen(name);
	if(nameSize==0) {
		SM_ERROR(TYPE_USER, esmBADLENGTH);
		return esmFAILURE;
	}
	strcpy(msgSpace, name);
	if (data == NULL) {
		bzero(msgSpace+nameSize, (int) dataSize);
	} else {
		bcopy(data, msgSpace+nameSize, (int) dataSize);
	}

	/* load up a disk message */
	message.header.type		   = SET_ROOT_ENTRY;
	message.header.replyRequested = TRUE;
	message.header.params.in.tid		   = GET_SERVERTID;
	message.body.root.volid	   = volid;
	message.body.root.dataSize = dataSize;
	message.body.root.nameSize = nameSize;

	/* send the set message */
	if (callServer(volRec->serverInfo, &message, msgSpace, (int) (dataSize+nameSize) ) < 0)	{
		return(esmFAILURE);
    } 
	return(esmNOERROR);
}



 int
io_RemoveRootEntry (

	VOLID			volid,
	char			*name
)
{

	register VOLREC	*volRec;
	MESSAGE			message;
	char			msgSpace[MAX_ROOTNAME_SIZE+2/*extra*/];
	int				nameSize;


	TRPRINT(TR_IO, TR_LEVEL_1, ("name:%s", name));

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(volid)) == NULL)	{
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	nameSize = strlen(name);
	if(nameSize==0) {
		SM_ERROR(TYPE_USER, esmBADLENGTH);
		return esmFAILURE;
	}
	strncpy(msgSpace, name, MAX_ROOTNAME_SIZE);

	/* load up a disk message */
	message.header.type		   = REMOVE_ROOT_ENTRY;
	message.header.replyRequested = TRUE;
	message.header.params.in.tid		   = GET_SERVERTID;
	message.body.root.volid	   = volid;
	message.body.root.nameSize = nameSize;

	/* send the set message */
	if (callServer(volRec->serverInfo, &message, msgSpace, (int) (nameSize) ) < 0)	{
		return(esmFAILURE);
    } 
	return(esmNOERROR);
}
