/*
 *   $RCSfile: io_SyncTrans.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "page.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "lsn.h"
#include "loginfo.h"
#include "logrecs.h"
#include "load.h"
#include "scan.h"
#include "sm_params.h"
#include "btree.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_funcs.h"
#include "io_globals.h"
#include "msg_globals.h"
#include "trans_globals.h"
#include "clog_globals.h"


BOOL
simpleSendMessage (
	SERVERTRANSREC  *stl,
	MESSAGE 	*msg,
	TRANSREC	*transRec
)
{
	MESSAGE 	message;
	
	message = *msg; /* a copy */
	message.header.params.in.tid    = stl->tid;

	if(callServer(stl->serverInfo, &message, NULL, 0 ) != esmNOERROR) {
		CLIENT_ABORT_TRANS(transRec);
	}
	return FALSE; 
}


 int
io_SyncTrans (
	TRANSREC	*transRec,
	BOOL		toDisk
)
{
	MESSAGE			message;

	TRPRINT(TR_IO, TR_LEVEL_1, ("Syncing trans %d", transRec->clientTid));

	message.header.type	  			= SYNC_TRANS;
	message.header.replyRequested 	= TRUE;

	if(toDisk)
		message.body.trans.syncTrans = TRUE;
	else
		message.body.trans.syncTrans = FALSE;

 
	ForEachServerInTrans(transRec, 3, 
			simpleSendMessage, &message, transRec, 0);
	if(transRec->clientTransState == T_ACTIVE)
		return esmNOERROR; 
	else
		return esmFAILURE; 
}
