/*
 *   $RCSfile: io_WriteLogPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "lsn.h"
#include "loginfo.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_funcs.h"
#include "clog_globals.h"
#include "io_globals.h"
#include "msg_globals.h"
#include "trans_globals.h"

 int
io_WriteLogPage (
	SERVERINFO		*serverInfo,
	char			*buffer,
	PAGE2SIZE		page2size 
)
{

	MESSAGE			message;


	/*
	 *	See if we should fake writing this log page.  Usually this
	 *	is set by a performance testing program.
	 */
	if (FakeLogFlush) {
		return(esmNOERROR);
	}

	TRPRINT(TR_IO, TR_LEVEL_1, ("write log page"));


again:  /* come here to try sending the message again */

	/*
	 *	load up a disk message
	 */
	message.header.type	  = WRITE_LOG_PAGE;
	message.header.replyRequested = FALSE;
	message.header.params.in.tid = 
		*getServerTid(serverInfo, serverInfo->logInfo.volid, &TransRec);

	/* note: no pid for log pages - mjf*/
	message.body.page.page2size = page2size;

	/*
	 * send the page to the server	
	 */
	if (callServer(serverInfo, &message, buffer,
					RAISETWO(page2size)) != esmNOERROR)	{

		/*
		 *	if there was no space in the log, try again, because
		 *	there may be now.
		 */
		if (message.header.params.out.errno == esmNOFREELOGSPACE) {
			SM_ERROR(TYPE_LOG, (int) message.header.params.out.errno);
			goto again;
		}

		/*
		 *	This function never returns an error from callServer
		 *	because BF_Invalidate*() assumes any error from 
		 *	this function is fatal.  So, the transaction must
		 *	be aborted.
		 */
		CLIENT_ABORT_TID ( getLocalTid() );
	}
	TRPRINT(TR_IO|TR_MSG, TR_LEVEL_2, 
		("writePage result:%d", message.header.params.out.errno));

	return(esmNOERROR);
}
