/*
 *   $RCSfile: LG_AllocateNodeListElement.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "ess.h"
#include "sysdefs.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "lgobject.h"
#include "volume.h"
#include "pool.h"
#include "version_graph.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "lg_versionfuncs.h"
#include "error.h"


/*
 * LG_AllocateNodeListElement creates a new element for the pool
 */

#define LGNODECHUNKSIZE 100

 LGNODELIST *
LG_AllocateNodeListElement (
	POOL		*unused			/* pool of node list elements		*/
)
{
			LGNODELIST	*node;
	static	LGNODELIST 	*nodeArray = NULL;
	static 	int			available = 0;

	if (available > 0) {
		
		/*
		 *	There is still nodes available on nodeArray, so return on
		 *	of those.
		 */
		available--;

	} else {

		/*
		 *	Allocate a chunk of nodes
		 */
		nodeArray = (LGNODELIST*) malloc(LGNODECHUNKSIZE*sizeof(LGNODELIST));
		if (nodeArray == NULL) {
			return(NULL);
		} else {
			available = LGNODECHUNKSIZE-1;	
		}
	}

	/*
	 *	Initialize the node and return it
	 */
	node = nodeArray+available;
	initializeListElement(&node->list, (char*)node);
	initializeListElement(&node->hash, (char*)node);

	return(node);
}
