/*
 *   $RCSfile: LG_AppendPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "error.h"


/*
 * Append (page id, byte count, clean/dirty) pair to end of page list.
 */


 void 
LG_AppendPage (

	PAGELIST		*pageList,		/* pointer to page list			*/
	PID				*pid,			/* pointer to page id structure	*/
	int				byteCount,		/* number of bytes on page		*/
	BOOL			dirty,			/* true if page is dirty		*/
	GROUPLINK		*nodeLink,
	SHORTPID		parentPid,
	int				parentSlot,
	int				slot,
	BOOL			shared 			/* true if page shared w/another verison*/
)
{

	register CHUNKPAGE	*chunkPage;    /* index of new list entry */


	TRPRINT(TR_LG, TR_LEVEL_1, ("pid:%d byteCount:%d", pid->page, byteCount));

	/*
	 *	check to see if we have an overflow
	 */
    SM_ASSERT(LEVEL_3, pageList->numPages < pageList->maxPages);
    
	/*
	 *	get a pointer to the new node and increment the
	 *	node count to account for the new node
	 */
    chunkPage = &(pageList->pages[pageList->numPages++]);

	/*
	 *	increment the number of bytes affected
	 */
    pageList->numBytes += byteCount;
	TRPRINT(TR_LG, TR_LEVEL_1, ("total bytes:%d", pageList->numBytes));

	/*
	 *	fill in the node entry
	 */
    chunkPage->flags	  = dirty ? (PAGE_DIRTY|PAGE_VALID) : PAGE_VALID;
    chunkPage->size		  = byteCount;
    chunkPage->pid		  = *pid;
    chunkPage->nodeLink   = nodeLink;
	chunkPage->parentPid  = parentPid;
	chunkPage->parentSlot = parentSlot;
	chunkPage->slot		  = slot;
	chunkPage->shared	  = shared;
}
