/*
 *   $RCSfile: LG_BinarySearch.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "error.h"


/*
 * Binary search a node.  The current semantics of this routine are as
 * follows.  Suppose the counts in the node are:  10 20 30.  That is, byte
 * 10 will lie in the middle subtree (it contains bytes 10 - 19).  Then
 * LG_BinarySearch(nodePtr, 10) will return 1, reflecting the fact that
 * to get to byte 10 we must follow the second slot's pid value.  -1 is
 * returned if an error is encountered.
 */


 int 
LG_BinarySearch (

	LGNODE			*nodePtr,		/* node we're searching			*/
	int				offset 			/* offset we're searching for	*/
)
{

    register int		low;			/* low index				*/
    register int		high;			/* high index				*/
    register int		mid;			/* mid point				*/
    register int		midCount;		/* count of mid slot		*/
    register int		prevCount;		/* count of mid - 1 slot	*/


	TRPRINT(TR_LG, TR_LEVEL_1, ("offset:%d", offset));

    /*
	 *	setup the initial search conditions
     */
    low = 0;
    high = (int)nodePtr->header.numSlots - 1;

	/*
	 *	search until found or goes off the end
	 */
    while (low <= high) {

		/*
		 *	calculate the new midpoint
		 */
        mid = (low + high) / 2;

		/*
		 *	get the count of the middle slot
		 */
        midCount = (int)nodePtr->slot[mid].count;

		/*
		 *	get the count of the previous slot
		 */
        if (mid == 0) {

			prevCount = 0;

        } else {

			prevCount = (int)nodePtr->slot[mid - 1].count;
        }
		TRPRINT(TR_LG, TR_LEVEL_2, ("low:%d high:%d mid:%d midcount:%d prevcount:%d",
				low, high, mid, midCount, prevCount));

		/*
		 *	check to see if we've found our value or must search further
		 */
        if ((offset < midCount) && (offset >= prevCount)) {

			/*
			 *	found the right node
			 */
			TRPRINT(TR_LG, TR_LEVEL_2, ("returning:%d", mid));
            return(mid);

        } else if (offset < midCount) {

			/*
			 *	look in lower half of nodes
			 */
            high = mid - 1;

        } else {

			/*
			 *	look in upper half of nodes
			 */
            low = mid + 1;
        }
    }

	/*
	 *	this should not happen
	 */
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
	/* to keep the compiler quiet */ return esmFAILURE;
}

