/*
 *   $RCSfile: LG_ChangeWritePageId.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "ess.h"
#include "sysdefs.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lgobject.h"
#include "pool.h"
#include "version_graph.h"
#include "sm_macro.h"
#include "lock.h"
#include "volume.h"
#include "lg_versionfuncs.h"

/*
 * This function changes the page id's in writeList to match those 
 * of the pages in the chunk descriptor.  
 */

 int
LG_ChangeWritePageId (

	CHUNKDESC	*chunkDesc,	/* chunk to process						*/
	PAGELIST	*writeList  /* pages possibly needing id's changed	*/
)
{

	CHUNKPAGE	*chunkPage;
	CHUNKPAGE	*writePage;
	int			changeCount; 	/* # of id's changed 	*/
	int			loopCount; 		/* # of times thru loop	*/

	TRPRINT(TR_LG, TR_LEVEL_1, (""));

	/*
	 *	This routine is called only from LG_InsertData() or
	 *	LG_AppendData() and therefore the chunk should only have
	 *	one or two pages, and at least one must be shared.
	 */
	SM_ASSERT(LEVEL_3, chunkDesc->pageCount <= 2);
	SM_ASSERT(LEVEL_3, chunkDesc->sharedPages > 0);

	/*
	 *	For each shared page in the chunk, if its id doesn't match that
	 *	of the corresponding page in the write list, update the 
	 *	id in the write list.
	 */
	for (chunkPage = chunkDesc->chunkPage, 
			 writePage = writeList->pages,
			 loopCount = 0, 
			 changeCount = 0; 
		 changeCount < chunkDesc->sharedPages; /* still shared pages */
		 chunkPage++, writePage++, loopCount++) {

		/*
		 *	Don't visit more pages than in the chunk
		 */
		SM_ASSERT(LEVEL_3, loopCount < chunkDesc->pageCount);

		/*
		 *	See if pid needs to be changed
		 */
		if ( ! PIDEQ(chunkPage->pid, writePage->pid)) {
			/*
			 *	The pages must be on the same volume
			 */
			SM_ASSERT(LEVEL_3, chunkPage->pid.volid == writePage->pid.volid);

			/*
			 *	Change the page id 
			 */
			writePage->pid.page = chunkPage->pid.page;

			/* 
			 *	record that the page id was changed
			 */
			changeCount++;

		}
	}
	return(esmNOERROR);
}
