/*
 *   $RCSfile: LG_FixLast.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "lsn.h"
#include "clog.h"
#include "logaction.h"
#include "loginfo.h"
#include "logrecs.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "lgobject.h"
#include "sm_state.h"
#include "bf_extfuncs.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "clog_globals.h"
#include "clog_extfuncs.h"
#include "error.h"


/*
 * Update the index of the long object following an append or insert
 * operation.
 */


 int 
LG_FixLast (

	PAGELIST			*pageList,		/* list of leaf page pid/cnt pairs			*/
	TSTACK				*tstack 		/* stack that was used to traverse object	*/
)
{

	register LGSLOT		*slotPtr;
	register TSTACKENT	*stackTop;
    PID					origPid;		/* pid of buffered internal node			*/
	GROUPLINK			*origLink;		/* link of the current node					*/
    LGNODE				*origNode;		/* ptr to copy of original node contents	*/
	int					insertSlot;
	int					currentBytes;
	CHUNKPAGE			*currentPage;
    int					i;				/* canonical loop index	*/
	OPERATIONINFO   opInfo;     /* structure for passing info to log calls*/
	SLOTTEDPAGE     *rootPage;
	LARGEOBJROOT    *largeRoot;
	PAGESLOT        *rootSlotPtr;
	BOOL			newSlotRoot;





	TRACE(TR_LG, TR_LEVEL_1);

	/*
	 *	Get the top entry off the stack
	 */
	stackTop = LG_PopTstack(tstack);

	/*
	 *	get out the appropriate stack entries
	 */
	insertSlot = stackTop->slotIndex;
	origPid	   = stackTop->pid;
	origNode   = stackTop->nodePtr;
	origLink   = stackTop->nodeLink;
	TRPRINT(TR_LG, TR_LEVEL_2, ("pid:%d insertSlot:%d", origPid.page, insertSlot));

	/* check if we are working on an object with a slotted root object
		that is in the process of being created */
	newSlotRoot = FALSE;
	if (stackTop->rootSlot != NIL) {
	    rootPage  = (SLOTTEDPAGE *) origLink->bufFrame;
	    rootSlotPtr = (PAGESLOT *) GETSLOTPTR(rootPage, stackTop->rootSlot);
	    largeRoot = (LARGEOBJROOT *) GETOBJECTPTR(rootPage, rootSlotPtr);
		if (largeRoot->objectHdr.properties & P_CREATEINPROGRESS) {
			newSlotRoot = TRUE;
		}
    }
												  
	/*
	 *	Setup a pointer to the first pagelist entry
	 *	and the slot to replace the entries
	 */
	currentPage = pageList->pages;
	slotPtr = &(origNode->slot[insertSlot]);

	/*
	 *	set the current number of bytes
	 */
	if (insertSlot == 0)	{

		currentBytes = 0;

	} else {

		/*
		 *	Get the bytes from the previous slot.  The slots
		 *	up to insertion point were copied into the new
		 *	node so we can get the count from there.
		 */
		currentBytes = (int)origNode->slot[insertSlot - 1].count;
	}
	TRPRINT(TR_LG, TR_LEVEL_2, ("currentBytes:%d", currentBytes));

	/* save old entries for logging */
	if (newSlotRoot == FALSE) {
	    bcopy((char*) slotPtr, OldData,
									(int) sizeof(LGSLOT) * pageList->numPages);
	}
	
	/*
	 *	copy in the entries
	 */
	for (i = 0; i < pageList->numPages; i++)	{

		/*
		 *	fill in the pid and the slot, the page is not
		 *	shared since it is new
		 */
		slotPtr->pid = currentPage->pid;
		slotPtr->shared = FALSE;
		slotPtr->count = currentBytes += currentPage->size;
		TRPRINT(TR_LG, TR_LEVEL_2, ("slot:%d pid:%d count:%d",
				insertSlot + i, slotPtr->pid.page, currentBytes));

		/*
		 *	get the next entry
		 */
		slotPtr++;
		currentPage++;
	}

	if (newSlotRoot == FALSE && GET_LOGLEVEL(LOG_SPACE)) {
        /*log the writing of the slots of the original node*/
        opInfo.action = LOG_ACTION_WRITE_LARGE_SLOTS;
        opInfo.flags = 0;
        opInfo.start =(int) (insertSlot * sizeof(LGSLOT));
        opInfo.size = (int) (sizeof(LGSLOT) * pageList->numPages);
        opInfo.oldData = OldData;
        opInfo.newData = (char *) &(origNode->slot[insertSlot]);
        opInfo.actionPid = origPid;
	    /* mark if it is a root object on a slotted page */
	    opInfo.rootSlot = stackTop->rootSlot;
		if(clog_LogWriteLgPage(&opInfo)) {
			FAIL
		}
	}

	lg_PrintNode(origNode, 0, origPid.page);

	/* Unfix the page */
	bf_UnfixPage(origLink, BF_DEFAULT, TRUE);

	LG_UnfixTstack(tstack, FALSE);

	/* return success to the caller */
    return(esmNOERROR);

FailureReturn:
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
	return esmFAILURE; /* to keep compiler quiet */
}
