/*
 *   $RCSfile: LG_HashPutNode.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "pool.h"
#include "version_graph.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "lgobject.h"
#include "lg_extfuncs.h"
#include "lg_versionfuncs.h"
#include "error.h"


/*
 * LG_HashPutNode adds a node to the hash table, if it's not already
 * present.
 */


 BOOL
LG_HashPutNode (
	LGNODEHASHTABLE *nodeHashTable,
	LGNODELIST		*node 
)
{
	LGNODELIST			*match;
	unsigned	int		index;

	TRPRINT(TR_LG, TR_LEVEL_1, ("page:%d", node->pid.page));

	index = LG_HashPid(&node->pid, nodeHashTable);

	SM_ASSERT(LEVEL_3, (index >= 0) && (index < nodeHashTable->tableSize));

	/*
	 * Find a matching node
	 */
	match = (LGNODELIST*) FIRST_LIST_ELEMENT(&nodeHashTable->table[index]);
	while (match != NULL) {
	
		if (PIDEQ(node->pid, match->pid) && (node->slot == match->slot)) {

			/*
			 *	Node already present
			 */
			return(FALSE);

		} else {
			match = (LGNODELIST*) NEXT_LIST_ELEMENT(&match->hash);
		}
	}

	/*
	 *	Add the node
	 */
	listEnq(&nodeHashTable->table[index], &node->hash);
	
	return(TRUE);
}
