/*
 *   $RCSfile: LG_InsertPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:21 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   EXODUS Project Group
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "util_funcs.h"


/*
 * Insert a (page id, byte count, clean/dirty) pair in front of page list.
 * (Note - this routine will only be used when the list is known for sure
 * to be extremely short.)
 */


 void 
LG_InsertPage (

	PAGELIST			*pageList,		/* pointer to page list		*/
	PID					*pid,			/* pointer to page id structure*/
	int					byteCount,		/* number of bytes on page	*/
	BOOL				dirty,			/* true if page is dirty	*/
	GROUPLINK			*nodeLink,	
	SHORTPID			parentPid,		/* parent page of pid		*/
	int					parentSlot,		/* slot if parent is root on slotted page*/
	BOOL				shared 			/* true if page shared w/another version*/
)
{

	TRACE(TR_LG, TR_LEVEL_1);

	/*
	 *	check for overflow
	 */
    SM_ASSERT(LEVEL_3, pageList->numPages < pageList->maxPages);

    /*
     * Move everything up one slot
     */
    bcopy( (char *) &(pageList->pages[0]), (char *) &(pageList->pages[1]),
          (int) (pageList->numPages * sizeof(CHUNKPAGE)) );

	/*
	 *	Insert the information into the first slot
	 */
    pageList->pages[0].flags	= dirty ? PAGE_DIRTY : NOFLAGS;
    pageList->pages[0].size		= byteCount;
    pageList->pages[0].pid		= *pid;
    pageList->pages[0].nodeLink = nodeLink;
    pageList->pages[0].parentPid = parentPid;
    pageList->pages[0].parentSlot = (TWO) parentSlot;
    pageList->pages[0].shared 	= shared;

	/*
	 *	increment the number of pages and bytes
	 */
    pageList->numPages++;
    pageList->numBytes += byteCount;
}
