/*
 *   $RCSfile: LG_PushTstack.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:23 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "error.h"


/*
 * Push (slotNo, pid, nodePtr) triple onto stack.
 */


 int 
LG_PushTstack (

	TSTACK			*tstack,			/* stack pointer 			*/
	SLOTINDEX		slotIndex,			/* slot number to push 		*/
	PID				*pid,				/* page identifier to push 	*/
	LGNODE			*nodePtr,			/* page pointer 			*/
	BOOL			dangerFlag,			/* true if node in danger 	*/
	int				dangerCount,		/* how many kids in danger 	*/
	GROUPLINK		*nodeLink,
	SLOTINDEX		rootSlot,
	BOOL			shared 				/* shared by other version	*/
)
{

	register TSTACKENT	*topEntry;


	TRPRINT(TR_LG, TR_LEVEL_1, ("slotIndex:%d pid:%d dangerFlag:%d dangerCount:%d, rootSlot:%d",
			slotIndex, pid->page, dangerFlag, dangerCount, rootSlot));

	/*
	 *	Increment the top and check for overflow
	 */
    if (++(tstack->top) >= TSTACKSIZE) {

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

	/*
	 *	get a pointer to the top element
	 */
	topEntry = &(tstack->stack[tstack->top]);

	/*
	 *	fill in the entries
	 */
    topEntry->slotIndex   = slotIndex;
    topEntry->nodePtr	  = nodePtr;
    topEntry->pid		  = *pid;
    topEntry->dangerFlag  = dangerFlag;
    topEntry->dangerCount = dangerCount;
    topEntry->nodeLink	  = nodeLink;
    topEntry->rootSlot	  = rootSlot;
    topEntry->shared	  = shared;

	/*
	 *	return success
	 */
    return(esmNOERROR);
}
