/*
 *   $RCSfile: LG_ReplaceIfShared.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "sm_macro.h"
#include "lock.h"
#include "pool.h"
#include "version_graph.h"
#include "bf_extfuncs.h"
#include "bf_intfuncs.h"
#include "lg_intfuncs.h"
#include "lg_versionfuncs.h"


/*
 *	This routine sees if a large object btree node is shared and 
 *	needs to be replaced and then calls a routine to replace it.
 */


 int
LG_ReplaceIfShared (
	BUFGROUP	*bufGroup,	/* page to possibly replace			*/
	PID			*rootPid,	/* root page of object 				*/
	SLOTINDEX	rootSlot,	/* slot of root, if on slotted page	*/
	PID			*pid,		/* page to possibly replace 		*/
	int			offset,		/* offset into large object			*/
	PID			*newPid,	/* new page							*/
	BOOL		*replaced  	/* true if page replaced			*/
)
{
	TSTACK		tstack;		/* stack for recording traversal	*/
	int			e;			/* error code						*/
	TSTACKENT	*stackEntry;
	int			sharedCount;	/* number of shared pages			*/


	TRPRINT(TR_LG, TR_LEVEL_1, ("might replace pid %d,%d", pid->page, 
			pid->volid));

	e = LG_BuildShareStack(rootPid, rootSlot, pid, offset, &tstack, 
						   bufGroup, &sharedCount);
	if (e != esmNOERROR) return (e);

	/*
	 *	If the page is not shared, then don't do anything
	 */
	if (sharedCount == 0) {

		/*
		 *	Unpin each page on the stack
		 */
		for (stackEntry = LG_TopTstack(&tstack);
			 stackEntry >= &(tstack.stack[0]);
			 stackEntry--) {
			bf_UnfixPage(stackEntry->nodeLink, BF_DEFAULT, FALSE);
		}

		/*
		 *	Return that the page, pid, was not replaced
		 */
		*replaced = FALSE;
		return (esmNOERROR);
	}

	e = LG_ReplaceSharedNodes(bufGroup, rootPid, &tstack, sharedCount, newPid);
	if (e != esmNOERROR) return (e);
							  
	/*
	 *	Unpin each page on the stack
	 */
	for (stackEntry = LG_TopTstack(&tstack);
		 stackEntry >= &(tstack.stack[0]);
		 stackEntry--) {
		bf_UnfixPage(stackEntry->nodeLink, BF_DEFAULT, FALSE);
	}

	/*
	 *	Note that pid was replaced, and return
	 */
	*replaced = TRUE;
	return(esmNOERROR);
}
