/*
 *   $RCSfile: LG_UpdateLeafParent.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "sm_macro.h"
#include "lock.h"
#include "pool.h"
#include "logaction.h"
#include "clog.h"
#include "sm_state.h"
#include "version_graph.h"
#include "bf_extfuncs.h"
#include "bf_intfuncs.h"
#include "clog_extfuncs.h"
#include "lg_versionfuncs.h"


/*
 *	Routine to modify slots in a parent node to reflect the pids
 *	of newly allocated leaf pages.
 */


 int
LG_UpdateLeafParent (
	CHUNKDESC	*chunkDesc,		/* chunk containing the new pages 	*/
	CHUNKPAGE	*firstPage,		/* first new page in chunk			*/
	CHUNKPAGE	*lastPage 		/* last new page in chunk			*/
)
{
	PID				parentPid;		
	GROUPLINK		*parentLink;
	CHUNKPAGE		*chunkPage;		/* generic chunk page pointer	*/
	LGNODE			*parentNode;	/* the actual parent node		*/
	LGNODE			tempNode;		/* to hold old copy of slots	*/
	SLOTINDEX		slotIndex;
	OPERATIONINFO	opInfo;

	TRPRINT(TR_LG, TR_LEVEL_1, ("updating parent page %d", firstPage->parentPid));

	/*
	 *	generate the parent pid
	 */
	parentPid.page = firstPage->parentPid;
	parentPid.volid = firstPage->pid.volid;

	/*
	 *	See if the parent page is the root page and located on a 
	 *	slotted page
	 */
	if (firstPage->parentSlot != NIL) {
		/*
		 *	Pin the root's slotted page 
		 */
		parentLink = bf_ReadPage(chunkDesc->bufGroup, &parentPid, 
								 H_SLOTTED, SLOTTED_PAGE2SIZE, NL);	

	} else {
		/*
		 *	Pin the root page
		 */
		parentLink = bf_ReadPage(chunkDesc->bufGroup, &parentPid, 
								 H_LARGE, LG_PAGE2SIZE, NL);	
	}
	if(parentLink==NULL) {
		return esmFAILURE;
	}

	/*	
	 *	Get a pointer to the actual parent data
	 */
	if (firstPage->parentSlot != NIL) {
		LARGEOBJROOT 	*objectPtr;
		PAGESLOT        *slotPtr;
		SLOTTEDPAGE     *pagePtr;

		/*
		 *	Parent page (node) is on a slotted page, so get the 
		 *	address of the root node on the slotted page
		 */
		pagePtr = (SLOTTEDPAGE *) parentLink->bufFrame;
		slotPtr = GETSLOTPTR(pagePtr, firstPage->parentSlot);
		objectPtr = (LARGEOBJROOT *) GETOBJECTPTR(pagePtr, slotPtr);
		parentNode = (LGNODE *) &(objectPtr->rootPage);
	} else {
		/*
		 *	Parent page is a normal large obj interior node
		 */
		parentNode = (LGNODE*) parentLink->bufFrame;
	}
	
	/*
	 *	Update the parent's child pointers. First, remember the
	 *	original value of the slots
	 */
	for (slotIndex = firstPage->slot; slotIndex <= lastPage->slot; slotIndex++) {
		tempNode.slot[slotIndex] = parentNode->slot[slotIndex];
	}
	for (chunkPage = firstPage; chunkPage <= lastPage; chunkPage++) {
		parentNode->slot[chunkPage->slot].pid = chunkPage->pid;
		parentNode->slot[chunkPage->slot].shared = FALSE;
		SM_ASSERT(LEVEL_3, chunkPage->parentPid == parentPid.page);
	}
	if (GET_LOGLEVEL(LOG_SPACE)) {
		opInfo.action = LOG_ACTION_WRITE_LARGE_SLOTS;
		opInfo.flags = 0;
		opInfo.start = firstPage->slot * sizeof(LGSLOT);
		opInfo.size = (firstPage->slot - lastPage->slot + 1) * sizeof(LGSLOT);
		opInfo.oldData = (char*) &(tempNode.slot[firstPage->slot]);
		opInfo.newData = (char*) &(parentNode->slot[firstPage->slot]);
		opInfo.actionPid = parentPid;
		/* mark if it is a root object on a slotted page */
		opInfo.rootSlot = firstPage->parentSlot;
		if(clog_LogWriteLgPage(&opInfo)) {
			FAIL
		}
	}

	/* Unfix the parent page, marking it dirty */
	bf_UnfixPage(parentLink, BF_DEFAULT, TRUE);

	/* Finished */
	return(esmNOERROR);

FailureReturn:
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
	return esmFAILURE; /* to keep the compiler quiet */
}
