/*
 *   $RCSfile: lg_DeleteFromObject.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "bf_macro.h"
#include "lgobject.h"
#include "bf_extfuncs.h"
#include "lg_intfuncs.h"
#include "lg_extfuncs.h"
#include "error.h"


/*
 * Delete routine for removing bytes from within a large storage object.
 */


 int
lg_DeleteFromObject (
	FID			*fid,
	PID			*rootPid,		/* pid of object to delete data from		*/
	BUFGROUP	*bufGroup,		/* buffer group to be used					*/
	int			start,			/* offset of desired deletion point			*/
	int			length,			/* number of bytes to delete				*/
	SLOTINDEX	rootSlot,
	BOOL		versioned 
)
{
    int			offset;			/* offset of start within leaf block		*/
    TSTACK		lstack;			/* tstack for left cut path					*/
    TSTACK		rstack;			/* tstack for right cut path				*/
    PID			noPid;			/* dummy pid for delete phase routine		*/
    BOOL		noBool;			/* dummy boolean for delete phase routine	*/


	TRPRINT(TR_LG, TR_LEVEL_1, ("root pid:%d rootSlot:%d", rootPid->page, rootSlot));

    /*
     * Delete the desired bytes, then fix up the index and return.
     */
    offset = start;
    LG_ResetTstack(&lstack);
    LG_ResetTstack(&rstack);
    if (LG_DeletePhase(fid, rootPid, rootSlot, rootPid, rootSlot, 
					   offset, length, bufGroup, &noPid, &lstack, 
					   &rstack, &noBool, versioned, 
					   FALSE /* node can't be shared since it is root*/))	{
		
		return(esmFAILURE);
	}

	TRPRINT(TR_LG, TR_LEVEL_2, ("rootpid:%d", rootPid->page));
	TRPRINT(TR_LG, TR_LEVEL_2, ("left cut path stack"));
	LG_PrintTstack(&lstack);
	TRPRINT(TR_LG, TR_LEVEL_2, ("right cut path stack"));
	LG_PrintTstack(&rstack);

    if (LG_ZipUpIndex(fid, rootPid, bufGroup, &lstack, &rstack, versioned))	{
		
		return(esmFAILURE);
	}
    return(esmNOERROR);
}
