/*
 *   $RCSfile: lg_GetRootNodePid.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "file.h"
#include "scan.h"
#include "load.h"
#include "bf_extfuncs.h"
#include "lg_extfuncs.h"


 int
lg_GetRootNodePid (

	BUFGROUP				*bufGroup,
	OID						*oid,
	PID						*rootPid,	/* OUT: root page of obj 	*/
	SLOTINDEX				*rootSlot 	/* OUT: if on slotted page	*/
)
{

    register SLOTTEDPAGE    *homePage;		/* object's home page		*/
    register OBJECT			*objectPtr;		/* ptr to generic object	*/
	GROUPLINK				*homeLink;
    PID						homePid;		/* home page of object		*/
	PAGESLOT				*slotPtr;
    SLOTINDEX				objectSlot;		/* the object's slot		*/



	TRPRINT(TR_LG, TR_LEVEL_1, ("oid page:%d slot:%d unique:%d",
			oid->diskAddr.page, oid->diskAddr.slot, oid->diskAddr.unique));

    /*
     *	Construct the pid of the home page
     */
    homePid.volid = oid->diskAddr.volid;
    homePid.page = oid->diskAddr.page;

	/*
	 *	check the buffer group magic number
	 */
	if (CHECK_BUFGROUP_VALID(bufGroup))	{

		SM_ERROR(TYPE_USER, esmBADGROUP);
		return(esmFAILURE);
	}

	/*
	 *	read the page into a buffer
	 */
	if ((homeLink = bf_ReadPage(bufGroup, &homePid, H_SLOTTED,
								SLOTTED_PAGE2SIZE, SH)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	get a pointer to the slotted page
	 */
	homePage = (SLOTTEDPAGE *) homeLink->bufFrame;

	/*
	 *	check the magic number
	 */
	if (CHECK_SLOTTED_MAGIC(homePage))	{

		bf_UnfixPage(homeLink, BF_DEFAULT, FALSE);
		SM_ERROR(TYPE_WARNING, esmBADSLOTTEDMAGIC);
		return(esmFAILURE);
	}

    /*
     * Make sure we have a valid slot in the oid.
     */
    objectSlot = oid->diskAddr.slot;

	/*
	 *	get a pointer to the slot
	 */
	slotPtr = GETSLOTPTR(homePage, objectSlot);

	/*
	 *	check the validity of the oid
	 */
	if (CHECKOID(objectSlot, homePage, slotPtr, oid->diskAddr.unique))	{

		bf_UnfixPage(homeLink, BF_DEFAULT, FALSE);
		SM_ERROR(TYPE_USER, esmBADOID);
		return(esmFAILURE);
	}

	/*
	 *  get a pointer to the object
  	 */
	objectPtr = GETOBJECTPTR(homePage, slotPtr);

	/*
	 *	Get the root page, first making sure we're dealing with a
	 *	large object
	 */
	if (objectPtr->objectHdr.properties & P_LARGEOBJ)   {

        /*
         *  check to see if we are looking at a header on
         *  the slotted page
         */
        if (objectPtr->objectHdr.properties & P_HEADERONPAGE)   {

			*rootPid = homePid;
			*rootSlot = objectSlot;
		} else {

            /*
             * Dig out the root pid
             */
            rootPid->volid = homePid.volid;
            rootPid->page = ((LARGEOBJHDR *) objectPtr)->rootPid;
			*rootSlot = NIL;
		}

	} else {

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return(esmFAILURE);
	}

	/*
	 *	Unfix the page
	 */
    bf_UnfixPage(homeLink, BF_DEFAULT, FALSE);

	/*
	 *	return success to the user
	 */
    return(esmNOERROR);
}

