/*
 *   $RCSfile: lg_GetVHGPtr.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "lock.h"
#include "sm_macro.h"
#include "lgobject.h"
#include "volume.h"
#include "pool.h"
#include "load.h"
#include "scan.h"
#include "io_extfuncs.h"
#include "version_graph.h"
#include "lg_versionfuncs.h"
#include "bf_extfuncs.h"

/*
 * Get the version history graph pointer for the large object
 */

 int
lg_GetVHGPtr (

	BUFGROUP	*bufGroup,
	LARGEOBJHDR	*largeObjectHdr,
	VOLID		volid,			/* volume containing the object 	*/
	VHGNODEPTR	*vhgPtr 		/* returns the VHG node "pointer"	*/

)
{

    PID				rootPid;		/* pid of root page */
	GROUPLINK		*rootLink;
	LGNODE			*rootPtr;		/* pointer to root node in buffer */


	TRPRINT(TR_LG, TR_LEVEL_1, ("getting VHG info %d"));

	if (largeObjectHdr->objectHdr.properties & P_HEADERONPAGE) {
		/*
		 *	The vhg info is easy to get since the root page is on the
		 *	slotted page.
		 */
		*vhgPtr = ((LARGEOBJROOT*)largeObjectHdr)->rootPage.header.vhgPtr;
		return(esmNOERROR);
	}

	/*
	 *	The root page is on a separate page so we have to read it in
	 *	to the the VHG information.
	 */

	/*
	 *	create a root pid
	 */
	rootPid.page 	= largeObjectHdr->rootPid;
	rootPid.volid	= volid;

	/*
	 *	Pin the parent verison root page
	 */
	if ((rootLink = bf_ReadPage(bufGroup, &rootPid, H_LARGE, 
								LG_PAGE2SIZE, NL)) == NULL)  {
		return(esmFAILURE);
	}

	/*
	 *	get a pointer to the parent root page
	 */
	rootPtr = (LGNODE *) rootLink->bufFrame;
	CHECK_LARGENODE_MAGIC(rootPtr);

	/*
	 *	return the VHG pointer
	 */
	*vhgPtr = rootPtr->header.vhgPtr;

	/*
	 *	Unfix the root page
	 */
	bf_UnfixPage(rootLink, BF_DEFAULT, FALSE);

	/*
	 *	return success
	 */
    return(esmNOERROR);
}

