/*
 *   $RCSfile: lg_SetVHGPtr.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:24 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "trace.h"
#include "checking.h"
#include "error.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "bf_macro.h"
#include "chunk.h"
#include "lock.h"
#include "sm_macro.h"
#include "lgobject.h"
#include "volume.h"
#include "pool.h"
#include "load.h"
#include "scan.h"
#include "logaction.h"
#include "clog.h"
#include "sm_state.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "clog_extfuncs.h"
#include "version_graph.h"
#include "lg_versionfuncs.h"

/*
 * Set the version history graph pointer for the large object
 */

 int
lg_SetVHGPtr (

	BUFGROUP	*bufGroup,
	LARGEOBJHDR	*largeObjectHdr,
	VHGNODEPTR	*vhgPtr,		/* VHG node "pointer"				*/
	OID			*oid 			/* oid of the object				*/

)
{

    PID				rootPid;		/* pid of root page 			*/
	GROUPLINK		*rootLink;
	LGNODE			*rootPtr;		/* pointer to root node in buffer*/
	LGNODEHDR		oldNodeHdr;	
	LARGEOBJROOT	*lgObjRoot;
	OPERATIONINFO	opInfo;			/* log record info 				*/


	TRPRINT(TR_LG, TR_LEVEL_1, ("setting VHG info %d"));

	if (largeObjectHdr->objectHdr.properties & P_HEADERONPAGE) {

		/*
		 *	The vhg info is easy to set since the root page is on the
		 *	slotted page.
		 */
		lgObjRoot = (LARGEOBJROOT*)largeObjectHdr;
		oldNodeHdr = lgObjRoot->rootPage.header;
		((LARGEOBJROOT*)largeObjectHdr)->rootPage.header.vhgPtr = *vhgPtr;

		if (GET_LOGLEVEL(LOG_SPACE)) {
			/*
			 *	Log the changed header;
			 */
			opInfo.action = LOG_ACTION_WRITE_LARGE_NODE;
			opInfo.flags = 0;
			opInfo.start = 0;
			opInfo.size = sizeof(LGNODEHDR);
			opInfo.oldData = (char*) &oldNodeHdr;
			opInfo.newData = (char*) &(lgObjRoot->rootPage.header);
			opInfo.actionPid.volid = oid->diskAddr.volid;
			opInfo.actionPid.page = oid->diskAddr.page;
			opInfo.rootSlot = oid->diskAddr.slot;
			if(clog_LogWriteLgNodeHdr(&opInfo)) {
				FAIL
			}
		}
		return(esmNOERROR);
	}

	/*
	 *	The root page is on a separate page so we have to read it in
	 *	to the the VHG information.
	 */

	/* get the root pid */
	rootPid.page 	= largeObjectHdr->rootPid;
	rootPid.volid	= oid->diskAddr.volid;

	/* Pin the parent verison root page */
	if ((rootLink = bf_ReadPage(bufGroup, &rootPid, H_LARGE, 
								LG_PAGE2SIZE, NL)) == NULL)  {
		return(esmFAILURE);
	}

	/* get a pointer to the parent root page */
	rootPtr = (LGNODE *) rootLink->bufFrame;
	CHECK_LARGENODE_MAGIC(rootPtr);

	if (GET_LOGLEVEL(LOG_SPACE)) {
		/* set the VHG pointer, then log it */
		oldNodeHdr = rootPtr->header;
		rootPtr->header.vhgPtr = *vhgPtr;
		opInfo.action = LOG_ACTION_WRITE_LARGE_NODE;
		opInfo.flags = 0;
		opInfo.start = 0;
		opInfo.size = sizeof(LGNODEHDR);
		opInfo.oldData = (char*) &oldNodeHdr;
		opInfo.newData = (char*) &(rootPtr->header);
		opInfo.actionPid = rootPid;
		opInfo.rootSlot = NIL;
		if(clog_LogWriteLgNodeHdr(&opInfo)) {
			FAIL
		}
	}

	/* Unfix the root page */
	bf_UnfixPage(rootLink, BF_DEFAULT, TRUE);

	/* return success */
    return(esmNOERROR);

FailureReturn:
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
	return esmFAILURE; /* to keep the compiler quiet */
}

