/*
 *	$RCSfile: LHSTACK.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "LHSTACK.h"

//
//	Class		: LHashStack
//	Method		: Destructor
//	Description	: Unfix every page in the stack.  
//
LHashStack::~LHashStack()
{
	while (cnt-- > 0)  {
		if ((GROUPLINK*)descPtr[cnt]) {
			descPtr[cnt].Unfix();
			}
		}
}


//
//	Class		: LHashStack
//	Method		: void DiscardTop()
//	Description	: Pop the top entry in the stack and unfix it.
//
void LHashStack::DiscardTop()
{
	PageDesc pd = Pop();
	LHINDPAGE*	indp = pd;
	if (indp)  {
			pd.Unfix();
		}
}

//
//	Class		: LHashStack
//	Method		: void DiscardEntry(PID& const pid)
//	Description	: remove the entry with pid in stack without unfixing and 
//					unlocking it.
//
void LHashStack::DiscardEntry(PID& const pid)
{
	ASSERT1(cnt > 0);
	for (int i=0; i < cnt; i++)	{
		LHINDPAGE* indp = descPtr[i];
		if (indp->SelfID() == pid)	{
			//
			// shift everything else
			//
			for (int y=i; y<cnt-1; y++)	
				descPtr[y] = descPtr[y+1];
			cnt--;
			return;
			}
		}
	//
	// not found, error
	//
	ASSERT1(FALSE);
}


//
//	Class		: LHashStack
//	Method		: int ReadNPush(PID& const pid, LOCKMODE lockMode)
//	Description	: Read page 'pid' with 'lockMode', and push it onto the stack.
//
int LHashStack::ReadNPush(PID& const pid, LOCKMODE lockMode)
{
	PageDesc pd;

	if (pd.Read(bufGroup, pid, lockMode))
		return esmFAILURE;

	Push(pd);

	return esmNOERROR;
}


//
//	Class		: LHashStack
//	Method		: PageDesc& GetEntry(PID& const pid)
//	Description	: Search for the entry with pid and return it.
//
int LHashStack::GetEntry(PID& const pid, PageDesc& retDesc)
{
	ASSERT1(cnt > 0);
	for (int i=0; i < cnt; i++)	{
		LHINDPAGE* indp = descPtr[i];
		if (indp->SelfID() == pid)	{
			retDesc = descPtr[i];
			return TRUE;
			}
		}
	//
	// not found
	//
	return FALSE;
}

//
//	Class		: LHashStack
//	Method		: PageDesc& GetPrevEntry(PID& const pid)
//	Description	: Search for the entry with pid as overflow page
//					and return it.
//
int LHashStack::GetPrevEntry(PID& const pid, PageDesc& retDesc)
{
	ASSERT1(cnt > 1);
	for (int i=0; i < cnt; i++)	{
		LHINDPAGE* indp = descPtr[i];
		if (indp->OvPage() == pid.page)	{
			retDesc = descPtr[i];
			return TRUE;
			}
		}
	//
	// not found
	//
	return FALSE;
}

//
//	Class		: LHashStack
//	Method		: void DirtyAll()
//	Description	: Dirty all the pages in the stack
//
void LHashStack::DirtyAll()
{
	ASSERT1(cnt > 0);
	for (int i=0; i < cnt; i++)	
		descPtr[i].Dirty();
}

//
//	Class		: LHashStack
//	Method		: void Print()
//	Description	: Print the stack for debugging purposes
//
void LHashStack::Print() 
{ 
	printf("LHashStack: \n"); 
	for (int i = 0; i < cnt; i++)  {
		LHINDPAGE* indp = descPtr[i];
		if (indp == 0) {
			printf("\tdescPtr[%d] is released\n", i);
			continue;
			}
		printf("\tdescPtr[%d] = page %d\n", i, indp->SelfID().page);
		}
}

