/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "io_extfuncs.h"
#ifdef SERVER_MAKE
#	include "log_globals.h"
#else
#include "sm_intfuncs.h"
	extern int LH_BULKLOAD;
#endif

//
//	Function	: int LH_AllocPage(TID tid, int volume, BUFGROUP* bufGroup, 
//											PID& pid, FOUR hashVal,
//											SMDATATYPE keyType)
//	Description	: Allocate and initialize a LH INDEX PAGE in volume.
//				  EX-lock is set on the newly allocated page. 
//
int LH_AllocPage(TID tid, int volume, BUFGROUP* bufGroup, PID& pid,
							FOUR hashVal, SMDATATYPE keyType, int ovFlag)
{
	BOOL log = TRUE;

	//
	//	Allocate a disk page
	//  note: all index pages (btree, lhash directory and index page) have
	//        the same size
	//
#ifdef SERVER_MAKE
	TRANSREC* transRec = (TRANSREC*) Active->transRec;
	VOLREC  * volRec;
	ASSERT1(transRec->tid == tid);

	if( (volRec = io_FindVolRec((VOLID) volume)) == NULL ) {
		return esmFAILURE;
		}

	if (io_AllocPages(volRec, BTREE_PAGE2SIZE, 1, NULL, &pid))
		return esmFAILURE;
#else
	checkTransState(volume);
	if (io_AllocPages(volume, BTREE_PAGE2SIZE, 1, NULL, &pid, PAGE_INDEX))
		return esmFAILURE;
	log = GET_LOGLEVEL(LOG_SPACE) && !LH_BULKLOAD;
#endif


	//
	//	Read in the new page
	//
	PageDesc pageDesc;
	if (pageDesc.GetPage(bufGroup, pid))
		return esmFAILURE;

	LHINDPAGE* indp = pageDesc;

#ifdef SERVER_MAKE
	INITIALIZE_LRC(&(indp->lrc()));
#else
    indp->lrc() = *(GET_LOGINFO->initLRC);
#endif /* SERVER_MAKE */

	// 
	//	Initialize new page 
	// 
	indp->Init(pid, hashVal, keyType, ovFlag);

	//
	//	Log the initialization
	//
	if (log)
		LH_LogIndexPageInit(pageDesc, keyType, hashVal, ovFlag);

	pageDesc.UnfixDirty();

	return esmNOERROR;
}
