/*
 *	$RCSfile: LH_DecrElCnt.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "LHSTACK.h"
#include "LH_SMOPages.h"
#include "OVPAGE.h"

extern void IntPrint(int, char*);
extern void (*BT_PrintFunc)(int, char*);

//
//	This function is called by server during undo for undoing logical
//	insertion on non-unique keys. It decremnts the tuple element count.
//
int LH_DecrElCnt(TID unused /* tid */, PID& const rootPid, BUFGROUP* bufGroup, 
					void* key, int keyLen, int maxKeyLen, PFC compFunc)
{
	TWO         slot;
	int         keyExists;
	PageDesc    curPd;
	LHashStack  dirPl(bufGroup), curPl(bufGroup);
	PID			indPid = rootPid;

	//
	//	Traverse the directory first
	//
	if (LH_DirSearch(rootPid, keyLen, key, dirPl, indPid.page, LH_READ))
		return esmFAILURE;

	//
	//  Find the index page
	//
	if (LH_IndSearch(indPid, keyLen, key, curPd, curPl, keyExists,
						slot, LH_INSERT, compFunc))  
		return esmFAILURE;
	
	if (!keyExists) 
		return esmFAILURE;

	LHINDPAGE* curp = curPd;

	//
	// do the decrement
	//
	curp->Tuple(slot).DecrElCnt();

	LH_LogLogicDecrElCnt(curp->SelfID(), curPd, curp->lrc(),
							rootPid, key, keyLen, maxKeyLen, curp->KeyType());
	
	curPd.Dirty();

	return esmNOERROR;
}
