/*
 *	$RCSfile: LH_DirSearch.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "LHSTACK.h"

#define LH_LOCKWANTED(OPER) ((OPER == LH_READ) ? SH : EX)

//
//	Function	: int LH_DirSearch(PID& const rootPid, int keyLen, 
//						void* keyVal, LHashStack& dirPl, SHORTPID& indPid,
//						LH_OPER oper)
//  Description : Traverse the directory (btree-like) to find the leaf level
//					directory page which has the entry of page id of index
//					page containing the keyVal.
//				  The target index page pid is returned through indPid.
//			      The search path passed out via hash stack dirPl (bottom-up).
//

int LH_DirSearch(PID& const rootPid, int keyLen, void* keyVal, 
					LHashStack& dirPl, SHORTPID& indPid, LH_OPER oper)
{
	LOCKMODE    lockMode = LH_LOCKWANTED(oper);
	LHDIRPAGE*  rootp;
	LHDIRPAGE*  dirp;
	
	//
	// read in the root directory page
	//
	if (dirPl.ReadNPush(rootPid, lockMode))
		return esmFAILURE;
	rootp = dirPl.Top();
	dirp = rootp;

	int hashVal = rootp->IndexCalc(keyLen, keyVal);
	int curIndex = hashVal;

	int curHeight = dirp->Height();
	PID curPid = rootPid;
	
	//
	// traverse the directory to find the index page entry
	//
	while (curHeight!=1) {
		int value = dirp->Capacity(curHeight-1);
		int n = curIndex / value;
		curIndex %= value;
		curPid.page = dirp->GetEntry(n);
		if (dirPl.ReadNPush(curPid, lockMode))
			return esmFAILURE;
		dirp = dirPl.Top();
		curHeight = dirp->Height();
		}

	indPid = dirp->GetEntry(curIndex);

	return esmNOERROR;
}
