/*
 *	$RCSfile: LH_DirSplit.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "LHSTACK.h"
#include "LH_SMOPages.h"

#ifdef DEBUG
 extern int rootSplitPerformed;
#endif


//
//	Function	: int LH_DirSplit(TID tid, int volume, BUFGROUP* bufGroup,
//						LHashStack& dirPl, LH_SMOPages& smo, PageDesc& dirPd,
//						SHORTPID& const dirPid)
//  Description : Promote the current directory page to be one more layer
//					higher, with its old copy stored as the first child page 
//					and the second entry containing the new entry (addPid).
//

int LH_DirSplit(TID tid, int volume, BUFGROUP* bufGroup, LHashStack& dirPl,
				LH_SMOPages& smo, PageDesc& dirPd, SHORTPID& const addPid)
{
	LHDIRPAGE* dirp = dirPd;
	BOOL log = TRUE;

#ifdef DEBUG
      rootSplitPerformed = (dirPl.NumEntries() == 1);
#endif

#ifndef SERVER_MAKE
	log = GET_LOGLEVEL(LOG_SPACE);
#endif
	//
	// Allocate and read one new directory page
	//
	PID newPid;
	if (LH_AllocDirPage(tid, volume, bufGroup, newPid, FALSE, dirp->KeyType()))
		return esmFAILURE;

	dirPl.ReadNPush(newPid, EX);  

	PageDesc newPd = dirPl.Top(); 
	LHDIRPAGE* newp = newPd;
	
	//
	// Copy everything in current directory page to new child
	//
	dirp->CopyData(newp);
	newPd.Dirty();
	if (log) {
		smo.Insert(newPid);
		LH_LogDirSplitCopy(newp->SelfID(), newPd, newp->lrc(), dirp->Height(),
							dirp->CurPos());
		}
	//
	// reset some of the parameters and add the two new entries
	//
	dirp->DirSplitUpdate();
	dirp->SetNextEntry(newPid.page);
	dirp->SetNextEntry(addPid);

	dirPd.Dirty();
	
	//
	// log the above three actions
	//
	if (log) {
		smo.Insert(dirp->SelfID());
		LH_LogDirSplitReset(dirp->SelfID(), dirPd, dirp->lrc(), 
								newPid.page, addPid, newp);
		}

	return esmNOERROR;
}
