/*
 *	$RCSfile: LH_Hash.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include <float.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"


#include "LH_Hash.h"

/* LH_Hash.C 
   Hash the key into a 4-byte integer keynum.
   1. SM_int, SM_long, SM_short need no conversion.
   2. SM_float, SM_double: keynum=(int)key*factor
   3. SM_string,SM_char: divide the string into 4-byte chunks from left,
      then bit-wise exclusive-or the chunks to get the keynum
   Then hash using the function (for now) of
   H sub i(keynum)=keynum mod (2^i) * N,
   where N=initial primary number of buckets
*/


/***********
BasicHash()
************/
FOUR BasicHash(TWO level, FOUR keynum) {
	FOUR retVal = (keynum * HASH_MAGIC + 1) % (NUMBUCKETS << level);
	if (retVal < 0) retVal = - retVal;
    return retVal;
}

/**************
ConvertFloat()
***************/
FOUR ConvertFloat(const void *keyval) {

    float key=*((float *)keyval);
    int intPart = (int) key;
    float fraction = key - intPart;

    return (FOUR)(key + fraction * CONVERT_FACTOR);
}

/**************
ConvertDouble()
***************/
FOUR ConvertDouble(const void *keyval) {

    double key=*((double *)keyval);
    long   intPart = (long) key;
    double fraction = key - intPart;
    return (FOUR)(key + fraction * CONVERT_FACTOR);
}

/***********
ConvertBits() 
used to convert both character strings and
groups of characters that are not null-terminated
************/
FOUR ConvertBits(const void *keyval, TWO length) {

    char *key = (char *)keyval;
    const CHUNKSIZE = 4;
    FOUR result=0;
    char chunk[CHUNKSIZE];
    register TWO position;
    register FOUR *temp;

    for (position=0; position<length; ++key) {
		chunk[position % CHUNKSIZE]=*key;
		position++;
		if (position % CHUNKSIZE==0) { //done with a chunk
	    	temp=(FOUR *)chunk;
	    	result=result ^ *temp; //exclusive or with result so far
			}
    	}
    if (position % CHUNKSIZE != 0) {
	// fill in a partly done chunk with '\0's
    	for ( ; position % CHUNKSIZE != 0; ++position) {
	    	chunk[position % CHUNKSIZE]='\0';
			}
		temp=(FOUR *)chunk;
		result=result ^ *temp;
    	}
    return result;
}

/***********
LH_Hash()
************/
int LH_Hash(TWO keylen, const void *keyval,SMDATATYPE keytype, TWO level) {
    FOUR result,temp;

    switch (keytype) {
	case SM_int:
	case SM_long:
   			temp=*((FOUR *)keyval);
			result=BasicHash(level,temp);
			break;
	case SM_short:
   			temp=(FOUR)(*((TWO *)keyval));
			result=BasicHash(level,temp);
			break;
	case SM_float:
			temp=ConvertFloat(keyval);
			result=BasicHash(level,temp);
			break;
	case SM_double:
			temp=ConvertDouble(keyval);
			result=BasicHash(level,temp);
			break;
	case SM_string:
			result=BasicHash(level,ConvertBits(keyval,strlen((char *)keyval)));
			break;
	case SM_char:
			result=BasicHash(level,ConvertBits(keyval,keylen));
			break;
	default:
			return esmFAILURE;
	}
    return result;
}

