/*
 *	$RCSfile: LH_IndSearch.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "LHSTACK.h"

#define LH_LOCKWANTED(OPER)	((OPER == LH_READ) ? SH : EX)

//
//	Function : int LH_IndSearch(PID& const indPid, int keyLen, void* keyVal, 
//					PageDesc& curPd, LHashStack& curPl,
//					int& found, TWO& returnSlot, LH_OPER oper, 
//					PFC compFunc)
//
//	Description : Find the index page chain corresponding to the hash value
//					of the key, and also get the index page where the key 
//					resides. 
//
int LH_IndSearch(PID& const indPid, int keyLen, void* keyVal, 
							PageDesc& curPd, LHashStack& curPl,
							int& found, TWO& returnSlot, LH_OPER oper,
							PFC compFunc)
{
	LOCKMODE	lockMode = LH_LOCKWANTED(oper);
	BUFGROUP*	bufGroup = curPl.BufGroup();

	//
	//	Read in the index page
	//
	if (curPd.Read(bufGroup, indPid, lockMode))
		return esmFAILURE;
	
	curPl.Push(curPd);
	LHINDPAGE* indp = curPd;

	//
	//	Search the primary index page
	//
	found = indp->Search(keyLen, keyVal, compFunc, returnSlot);

	//
	//	Check the overflow page if unsuccessful
	//
	PID			ovPid = indp->SelfID();
	LHINDPAGE*	ovp = indp;
	while (!found && ovp->HaveOv())	{
		ovPid.page = ovp->OvPage();
		ASSERT3( ovPid.page != NULLPID );
		curPl.ReadNPush(ovPid, lockMode);
		curPd = curPl.Top();
		ovp = curPd;
		found = ovp->Search(keyLen, keyVal, compFunc, returnSlot);
		}

	//
	//	still push rest non-empty overflow pages into the stack        
	//  (useful for splitting)
	//
	if (oper!=LH_READ)
		while (ovp->HaveOv())	{
			ovPid.page = ovp->OvPage();
			ASSERT3( ovPid.page != NULLPID );
			curPl.ReadNPush(ovPid, lockMode);
			ovp = curPl.Top();
			}

	return esmNOERROR;
}

