/*
 * $RCSfile: LH_Log.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "LHDIRPAGE.h"
#include "LHINDPAGE.h"
#include "OVPAGE.h"
#include "LH_Log.h"
#ifndef SERVER_MAKE
#	include "clog_intfuncs.h"
#	include "trans_globals.h"
#endif




#ifdef SERVER_MAKE

typedef LOGRECORDINFO 			LH_LOGRECINFO;
#define lrc_unused_on_client	lrc
#define GENLRC() {\
		recInfo.actionLRC = GENERATE_LRC(&lrc);\
		recInfo.actionPid = &pid;\
	}

#else 

typedef CLIENTLOGRECORDINFO		LH_LOGRECINFO;
#define lrc_unused_on_client	unused
#define GENLRC() recInfo.actionPid = pid;

#endif 
//
//	lhBuff class to ensure threads run correctly on server during
//	restart.
//
class lhBuff {
		static lhBuff* freeSpace;
		static void Replenish(size_t sz);
		lhBuff*& Next()		{ return *((lhBuff**) data); }
		
	public:
		char		data[BTREE_PAGESIZE];
		
		lhBuff()		{};
		~lhBuff()		{};
		
		void* operator new(size_t sz)  {
					if (!freeSpace)	Replenish(sz);
					void* ret = freeSpace;
					freeSpace = freeSpace->Next();
					return ret;
					}
		void operator delete(void* p) {
					((lhBuff*)p)->Next() = freeSpace;
					freeSpace = (lhBuff*)p;
					}
		};

lhBuff* lhBuff::freeSpace = 0;
const chunkSize = 1;


void lhBuff::Replenish(size_t sz)
{
	register lhBuff* p;
	
	ASSERT3(!freeSpace);
	freeSpace = p = (lhBuff*) new char[sz * chunkSize];
	ASSERT1(p);
	
	for (; p != &freeSpace[chunkSize-1]; p->Next() = p + 1, p++);
	p->Next() = 0;
}




void inline SetImage(LH_LOGRECINFO& recInfo, int idx, int size, void* data)
{
	recInfo.imageSize[idx] = size;
	recInfo.imageData[idx] = (VOID *) data;
}


int inline SpaceLeft()
{
#ifdef SERVER_MAKE
	return BTREE_PAGESIZE;
#else
	return GET_LOGINFO->lastUsableByte - (GET_LOGINFO->tailLSN & GET_LOGINFO->pageMask);
#endif
}

/****************
LogIt
*****************/
#ifndef SERVER_MAKE
/* flaky trick to make the middle argument (used only on client side) 
 * issue a warning that's easy to recognize as something we can disregard:
 */
#define unused fixedSize
#endif

static void LogIt(LH_LOGRECINFO& recInfo, int unused, PAGEHASH* pHash)

#ifndef SERVER_MAKE
#undef unused 
#endif
{
#ifdef SERVER_MAKE
	LSN lsn;
	FORCEMARK forceMark = writeLogRecord((TRANSREC*) Active->transRec, 
											&recInfo, &lsn, NOFLAGS);
	if (forceMark < 0)  {
		SM_ERROR(TYPE_FATAL, Active->errno);
		}

	if (pHash)	{
		DEPEND_LOG(pHash, forceMark, &lsn, recInfo.actionLRC);
		}
#else
	
	ASSERT3(pHash != 0);
//	ASSERT3(GET_LOGLEVEL(LOG_SPACE));
	
	LHINDPAGE* indp = (LHINDPAGE*) pHash->pageInfo->bufFrame;
	//cast directory page as an index page to access SelfID and lrc()
	
	ASSERT3(indp->SelfID() == recInfo.actionPid);
	
	if ( ! GET_LOGINFO->tailBuffer) 
		CLOG_GetNewLogPage();

	LRC lrc = indp->lrc();

	FOUR spaceLeft = SpaceLeft();
	
	if (fixedSize > spaceLeft)  {
		CLOG_GetNewLogPage();
		spaceLeft = GET_LOGINFO->usableBytes;
		}
	
	spaceLeft -= fixedSize;
	
	ASSERT1(spaceLeft >= 0);

	FORCEMARK forceMark = CLOG_WriteLogRecord(TransRec.clientTid, &recInfo, &lrc, pHash);

	ASSERT1(forceMark >= 0);
	
	pHash->forceMark = forceMark;
	

	indp->lrc() = lrc;
#endif
}


static int disabled = FALSE;



#ifdef SERVER_MAKE

/****************
LH_LogPureCLR
(Called by undoCreateIndex and undoLHash)
*****************/
void LH_LogPureCLR(LSNOFFSET undoNextLSN)
{
	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_COMPENSATION;
	recInfo.action = LOG_ACTION_PURE_CLR;
	recInfo.imageCount = 0;
	recInfo.actionPid = 0;
	recInfo.actionLRC = 0;
	recInfo.flags = REDO_ONLY;
	recInfo.nextUndoLSN = undoNextLSN;

	LogIt(recInfo, ALIGNSIZE(sizeof(LOGRECORDHDR)), 0);
}

#endif

/****************
LH_LogDirPageInit
(called by LH_AllocDirPage)
*****************/
void LH_LogDirPageInit(PAGEHASH* pHash, int rootFlag, SMDATATYPE keyType)
{
	LH_LOGRECINFO recInfo;
	LogDirPageInitData* recData = (LogDirPageInitData*) new lhBuff;
	recData->keyType = keyType;
	recData->rootFlag = rootFlag;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIRPAGE_INIT;
	recInfo.flags = REDO_ONLY;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
#ifdef SERVER_MAKE
		LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->bufFrame;
		LRC* lrc = &dirp->lrc();
		(void) GENERATE_LRC(lrc);
		recInfo.actionPid = &(dirp->SelfID());
		recInfo.actionLRC = lrc;
#else
		LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->pageInfo->bufFrame;
		recInfo.actionPid = dirp->SelfID();
#endif
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/****************
LH_LogIndexPageInit
(called by LH_AllocPage)
*****************/
void LH_LogIndexPageInit(PAGEHASH* pHash, SMDATATYPE keyType, FOUR hashVal,
						 int ovFlag)
{
	LH_LOGRECINFO recInfo;
	LogIndexPageInitData* recData = (LogIndexPageInitData*) new lhBuff;
	recData->keyType = keyType;
	recData->hashVal = hashVal;
	recData->ovFlag = ovFlag;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_INDEXPAGE_INIT;
	recInfo.flags = REDO_ONLY;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
#ifdef SERVER_MAKE
		LHINDPAGE* indp = (LHINDPAGE*) pHash->bufFrame;
		LRC* lrc = &indp->lrc();
		(void) GENERATE_LRC(lrc);
		recInfo.actionPid = &(indp->SelfID());
		recInfo.actionLRC = lrc;
#else
		LHINDPAGE* indp = (LHINDPAGE*) pHash->pageInfo->bufFrame;
		recInfo.actionPid = indp->SelfID();
#endif
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/****************
LH_LogPhysicInsert
(inserting tuple(s),
called by LH_SplitPage and LH_ResetOverflow)
*****************/
void LH_LogPhysicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc,
							TWO slotNum, TWO numSlot)
{

#ifdef SERVER_MAKE
	LHINDPAGE* indp = (LHINDPAGE*) pHash->bufFrame;
#else
	LHINDPAGE* indp = (LHINDPAGE*) pHash->pageInfo->bufFrame;
#endif

	ASSERT3(slotNum >= 0);
	ASSERT3(slotNum < indp->NumSlots());
	ASSERT3(numSlot > 0);
	ASSERT3(slotNum + numSlot <= indp->NumSlots());

	LogLhashPhysicInsertData* recData = (LogLhashPhysicInsertData*)new lhBuff;

	for (int i = 0; i < numSlot; ) {
		char* p;
		int start = i;

		int sz;
		for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
			sz = indp->Tuple(i+slotNum).Size();
			if (ALIGNSIZE(p - recData->data + sz) > recData->Max())  break;
			BCOPY( & indp->Tuple(i+slotNum), p, sz);
			}

		LH_LOGRECINFO recInfo;
		recInfo.type = LOG_REC_TYPE_USER;
		recInfo.action = LOG_ACTION_LHASH_PHYSIC_INSERT;
		recInfo.flags = NOFLAGS;
		recInfo.imageCount = 1;
		
		recData->startSlot = slotNum + start;	// different from
		recData->numSlots = i - start;			// 	LH_LogPhysicDelete()
		recData->length = p - recData->data;
		
		ASSERT3(recData->numSlots > 0);
		
		SetImage(recInfo, 0, recData->Size(), recData);
	
		int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
		ASSERT3(fixedSize <= recData->Max());

		if (!disabled)	{
			GENLRC();
			LogIt(recInfo, fixedSize, pHash);
			}
		}
	delete (lhBuff*) recData;
}


#ifdef SERVER_MAKE
/****************
LH_LogPhysicInsert
(inserting tuple(s) located in void* buff,
called by undo/undoLHash.C)
*****************/
void LH_LogPhysicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc,
						TWO slotNum, TWO numSlot, int length, void* buff)
{
	ASSERT3(slotNum >= 0);
	ASSERT3(numSlot > 0);

	LogLhashPhysicInsertData* recData = (LogLhashPhysicInsertData*)new lhBuff;
	
	recData->startSlot = slotNum;
	recData->numSlots = numSlot;
	recData->length = length;
	BCOPY(buff, recData->data, length);
	
	LH_LOGRECINFO recInfo;
	recInfo.type= LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_PHYSIC_INSERT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= recData->Max());
	
	if (!disabled)	{
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	delete (lhBuff*) recData;
}
#endif

/****************
LH_LogPhyLogInsert
(physiologically inserting tuple(s),
called by LH_SplitPage and LH_ResetOverflow)
*****************/
void LH_LogPhyLogInsert(PID& pid, PAGEHASH* pHash, LHINDPAGE* indp,
							LRC& lrc, TWO numSlot)
{

	ASSERT3(numSlot > 0);

	LogLhashPhyLogInsertData* recData = (LogLhashPhyLogInsertData*)new lhBuff;

	for (int i = 0; i < numSlot; ) {
		char* p;
		int start = i;

		int sz;
		for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
			sz = indp->Tuple(i).Size();
			if (ALIGNSIZE(p - recData->data + sz) > recData->Max())  break;
			BCOPY( & indp->Tuple(i), p, sz);
			}

		LH_LOGRECINFO recInfo;
		recInfo.type = LOG_REC_TYPE_USER;
		recInfo.action = LOG_ACTION_LHASH_PHYLOG_INSERT;
		recInfo.flags = NOFLAGS;
		recInfo.imageCount = 1;
		
		recData->startSlot = start;				// different from
		recData->numSlots = i - start;			// 	LH_LogPhysicDelete()
		recData->length = p - recData->data;
		
		ASSERT3(recData->numSlots > 0);
		
		SetImage(recInfo, 0, recData->Size(), recData);
	
		int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
		ASSERT3(fixedSize <= recData->Max());

		if (!disabled)	{
			GENLRC();
			LogIt(recInfo, fixedSize, pHash);
			}
		}
	delete (lhBuff*) recData;
}

#ifdef SERVER_MAKE
/****************
LH_LogPhyLogInsert
(deleting tuple(s) located in void* buff,
called by undo/undoLHash.C)
*****************/
void LH_LogPhyLogInsert(PID& pid, PAGEHASH* pHash, LRC& lrc, 
						TWO numSlot, int length, void* buff)
{
	ASSERT3(numSlot > 0);

	LogLhashPhyLogInsertData* recData = (LogLhashPhyLogInsertData*) new lhBuff;

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_PHYLOG_INSERT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
		
	recData->startSlot = 0;
	recData->numSlots = numSlot;
	recData->length = length;
	BCOPY(buff, recData->data, length);
	ASSERT3(recData->numSlots > 0);
		
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= recData->Max());
		
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*) recData;
}
#endif


/****************
LH_LogPhysicDelete
(deleting tuple(s),
called by LH_SplitPage, LH_InsertNewElem, 
LH_ResetOverflow, and lh_RemoveEntry)
*****************/
void LH_LogPhysicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc,
							TWO slotNum, TWO numSlot)
{

#ifdef SERVER_MAKE
	LHINDPAGE* indp = (LHINDPAGE*) pHash->bufFrame;
#else
	LHINDPAGE* indp = (LHINDPAGE*) pHash->pageInfo->bufFrame;
#endif

	ASSERT3(slotNum >= 0);
	ASSERT3(slotNum < indp->NumSlots());
	ASSERT3(numSlot > 0);
	ASSERT3(slotNum + numSlot <= indp->NumSlots());

	LogLhashPhysicDeleteData* recData = (LogLhashPhysicDeleteData*) new lhBuff;
	
	for (int i = 0; i < numSlot; ) {
		char* p;
		int start = i;
		int sz;
		for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
			sz = indp->Tuple(i+slotNum).Size();
			if (ALIGNSIZE(p - recData->data + sz) > recData->Max())  break;
			BCOPY( & indp->Tuple(i+slotNum), p, sz);
			}

		LH_LOGRECINFO recInfo;
		recInfo.type = LOG_REC_TYPE_USER;
		recInfo.action = LOG_ACTION_LHASH_PHYSIC_DELETE;
		recInfo.flags = NOFLAGS;
		recInfo.imageCount = 1;
		
		recData->startSlot = slotNum;
		recData->numSlots = i - start;
		recData->length = p - recData->data;
		ASSERT3(recData->numSlots > 0);
		
		SetImage(recInfo, 0, recData->Size(), recData);
	
		int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
		ASSERT3(fixedSize <= recData->Max());
		
		if (!disabled)  {
			GENLRC();
			LogIt(recInfo, fixedSize, pHash);
			}
		}
	
	delete (lhBuff*) recData;
}


#ifdef SERVER_MAKE
/****************
LH_LogPhysicDelete
(deleting tuple(s) located in void* buff,
called by undo/undoLHash.C)
*****************/
void LH_LogPhysicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc, 
						TWO slotNum, TWO numSlot, int length, void* buff)
{
	ASSERT3(slotNum >= 0);
	ASSERT3(numSlot > 0);

	LogLhashPhysicDeleteData* recData = (LogLhashPhysicDeleteData*) new lhBuff;

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_PHYSIC_DELETE;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
		
	recData->startSlot = slotNum;
	recData->numSlots = numSlot;
	recData->length = length;
	BCOPY(buff, recData->data, length);
	ASSERT3(recData->numSlots > 0);
		
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= recData->Max());
		
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*) recData;
}
#endif


/****************
LH_LogPhyLogDelete
(physiologically deleting tuple(s),
called by LH_SplitPage, LH_InsertNewElem, 
LH_ResetOverflow, and lh_RemoveEntry)
*****************/
void LH_LogPhyLogDelete(PID& pid, PAGEHASH* pHash, LHINDPAGE* indp,
							LRC& lrc, TWO numSlot)
{

	ASSERT3(numSlot > 0);

	LogLhashPhyLogDeleteData* recData = (LogLhashPhyLogDeleteData*) new lhBuff;
	
	for (int i = 0; i < numSlot; ) {
		char* p;
		int start = i;
		int sz;
		for (sz = 0, p = recData->data; i < numSlot; i++, p += sz)  {
			sz = indp->Tuple(i).Size();
			if (ALIGNSIZE(p - recData->data + sz) > recData->Max())  break;
			BCOPY( & indp->Tuple(i), p, sz);
			}

		LH_LOGRECINFO recInfo;
		recInfo.type = LOG_REC_TYPE_USER;
		recInfo.action = LOG_ACTION_LHASH_PHYLOG_DELETE;
		recInfo.flags = NOFLAGS;
		recInfo.imageCount = 1;
		
		recData->startSlot = 0;
		recData->numSlots = i - start;
		recData->length = p - recData->data;
		ASSERT3(recData->numSlots > 0);
		
		SetImage(recInfo, 0, recData->Size(), recData);
	
		int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
		ASSERT3(fixedSize <= recData->Max());
		
		if (!disabled)  {
			GENLRC();
			LogIt(recInfo, fixedSize, pHash);
			}
		}
	
	delete (lhBuff*) recData;
}

#ifdef SERVER_MAKE
/****************
LH_LogPhyLogDelete
(deleting tuple(s) located in void* buff,
called by undo/undoLHash.C)
*****************/
void LH_LogPhyLogDelete(PID& pid, PAGEHASH* pHash, LRC& lrc, 
						TWO numSlot, int length, void* buff)
{
	ASSERT3(numSlot > 0);

	LogLhashPhyLogDeleteData* recData = (LogLhashPhyLogDeleteData*) new lhBuff;

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_PHYLOG_DELETE;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
		
	recData->startSlot = 0;
	recData->numSlots = numSlot;
	recData->length = length;
	BCOPY(buff, recData->data, length);
	ASSERT3(recData->numSlots > 0);
		
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	ASSERT3(fixedSize <= recData->Max());
		
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*) recData;
}
#endif

/****************
LH_LogLogicInsert
(used by LH_InsertNewElem, lh_InsertEntry
and undo/undoLHash)
*****************/
void LH_LogLogicInsert(PID& pid, PAGEHASH* pHash, LRC& lrc,
					PID& const rootPid, void* const key, 
					TWO keyLen, void* const el, TWO elSize, TWO maxKeyLen, 
					TWO unique, SMDATATYPE keyType, BOOL primary)
{

	LH_LOGRECINFO recInfo;
	LogLhashLogicInsertData* recData = (LogLhashLogicInsertData*) new lhBuff;

	recData->rootPid = rootPid;
	recData->elSize = elSize;
	recData->maxKeyLen = maxKeyLen;
	recData->unique = unique;
	recData->keyType = keyType;
	recData->keyLen = keyLen;
	recData->primary = primary;
	BCOPY(key, recData->Key(), keyLen);
	BCOPY(el, recData->El(), elSize);
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_INSERT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
		
	delete (lhBuff*) recData;
}



/****************
LH_LogLogicDelete
(used by LH_DecrElCnt, lh_RemoveEntry
and undoLHash)
*****************/
void LH_LogLogicDelete(PID& pid, PAGEHASH* pHash, LRC& lrc,
						PID& const rootPid, void* const key, 
						TWO keyLen, void* const el, TWO elSize, TWO maxKeyLen,
						TWO unique, SMDATATYPE keyType, BOOL primary)
{

	LH_LOGRECINFO recInfo;
	LogLhashLogicDeleteData* recData = (LogLhashLogicDeleteData*) new lhBuff;
	
	recData->rootPid = rootPid;
	recData->elSize = elSize;
	recData->maxKeyLen = maxKeyLen;
	recData->unique = unique;
	recData->keyType = keyType;
	recData->keyLen = keyLen;
	recData->primary = primary;
	BCOPY(key, recData->Key(), keyLen);
	BCOPY(el, recData->El(), elSize);
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_DELETE;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	delete (lhBuff*) recData;
}

/***************
LH_LogSetIndexOverflow
(used by LH_InsertNewTuple, LH_SplitPage
and undoLHash)
****************/
void LH_LogSetIndexOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc,
							SHORTPID newOvPage, SHORTPID oldOvPage)
{
	LH_LOGRECINFO recInfo;
	LogSetIndexOverflowData* recData = (LogSetIndexOverflowData*) new lhBuff;
	recData->oldOvPage = oldOvPage;
	recData->newOvPage = newOvPage;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_SET_INDEX_OVERFLOW;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/****************
LH_LogResetIndexOverflow
(used by LH_SplitPage.C and undoLHash)
*****************/
void LH_LogResetIndexOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc,
							SHORTPID oldOvPage, int keepFlag)
{	
	LH_LOGRECINFO recInfo;
	LogResetIndexOverflowData* recData = (LogResetIndexOverflowData*) new lhBuff;
	recData->oldOvPage = oldOvPage;
	recData->keepFlag = keepFlag;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_RESET_INDEX_OVERFLOW;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}



/*****************
LH_LogDirUpdate
(used by LH_SplitPage.C)
******************/
void LH_LogDirUpdate(PID& pid, PAGEHASH* pHash, LRC& lrc,
						float oldFromLoad, float fromLoad, float toLoad,
						BOOL growFlag)
{
	LH_LOGRECINFO recInfo;
	LogDirUpdateData* recData = (LogDirUpdateData*) new lhBuff;
	recData->oldFromLoad = oldFromLoad;
	recData->fromLoad = fromLoad;
	recData->toLoad = toLoad;
	recData->growFlag = growFlag;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_UPDATE;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/*****************
LH_LogDirSetNextEntry
(used by lh_CreateIndex & lh_InsertIndex)
******************/
void LH_LogDirSetNextEntry(PID& pid, PAGEHASH* pHash, LRC& lrc,
						SHORTPID newPage, FOUR hashVal)
{
	LH_LOGRECINFO recInfo;
	LogDirSetNextEntryData* recData = (LogDirSetNextEntryData*) new lhBuff;
	recData->newPage = newPage;
	recData->hashVal = hashVal;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_SET_NEXT_ENTRY;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}


/*****************
LH_LogDirUnSetNextEntry
(used by undoLHash )
******************/
/*
#ifdef SERVER_MAKE
void LH_LogDirUnSetNextEntry(PID& pid, PAGEHASH* pHash, LRC& lrc, FOUR hashVal)
{
	LH_LOGRECINFO recInfo;
	LogDirSetNextEntryData* recData = (LogDirSetNextEntryData*) new lhBuff;
	recData->newPage = NULLPID-1;
	recData->hashVal = hashVal;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_SET_NEXT_ENTRY;
	recInfo.flags = REDO_ONLY;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}
#endif
*/

/*****************
LH_LogDirRootCtrl
(used by lh_CreateIndex)
******************/
void LH_LogDirRootCtrl(PID& pid, PAGEHASH* pHash, LRC& lrc, int unused)
{
#ifdef SERVER_MAKE
	LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->bufFrame;
#else
	LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->pageInfo->bufFrame;
#endif

	LH_LOGRECINFO recInfo;
	LogDirRootCtrlData* recData = (LogDirRootCtrlData*) new lhBuff;

	recData->M = dirp->Primary();
	recData->P = dirp->SplitPtr();
	recData->I = dirp->Level();
	recData->curPos = dirp->CurPos();
	recData->height = dirp->Height();
	recData->load = dirp->Load();

	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_ROOT_CTRL;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/*****************
LH_LogDirSplitCopy
(used by lh_CreateIndex, LH_DirSplit)
******************/
void LH_LogDirSplitCopy(PID& pid, PAGEHASH* pHash, LRC& lrc,
						int height, int curPos)
{
#ifdef SERVER_MAKE
	LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->bufFrame;
#else
	LHDIRPAGE* dirp = (LHDIRPAGE*) pHash->pageInfo->bufFrame;
#endif

	LH_LOGRECINFO recInfo;
	LogDirSplitData* recData = (LogDirSplitData*) new lhBuff;

	recData->height = height;
	recData->curPos = curPos;
	BCOPY(dirp->FirstEntry(), recData->data, curPos*sizeof(SHORTPID));
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_SPLIT_COPY;
	recInfo.flags = REDO_ONLY;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/*****************
LH_LogDirSplitReset
(used by lh_CreateIndex)
******************/
void LH_LogDirSplitReset(PID& pid, PAGEHASH* pHash, LRC& lrc,
						 SHORTPID page1, SHORTPID page2,
						 LHDIRPAGE* newp)
{
	LH_LOGRECINFO recInfo;
	LogDirSplitData* recData = (LogDirSplitData*) new lhBuff;
	recData->newPage1 = page1;
	recData->newPage2 = page2;
	recData->height = newp->Height();
	recData->curPos = newp->CurPos();
	BCOPY(newp->FirstEntry(), recData->data, newp->CurPos()*sizeof(SHORTPID));

	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_SPLIT_RESET;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/*****************
LH_LogDirLoadUpdate
(used by lh_InsertEntry, lh_RemoveEntry, undoLhash)
******************/
void LH_LogDirLoadUpdate(PID& pid, PAGEHASH* pHash, LRC& lrc,
					float load1, float load2 )
{
	LH_LOGRECINFO recInfo;
	LogDirLoadUpdateData* recData = (LogDirLoadUpdateData*) new lhBuff;

	recData->load1 = load1;
	recData->load2 = load2;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_LOAD_UPDATE;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
						ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/*****************
LH_LogDirLoadUpdate
overloaded
(used by lh_InsertEntry)
******************/
void LH_LogDirLoadUpdate(PID& pid, PAGEHASH* pHash, LRC& lrc,
					float load )
{
	LH_LOGRECINFO recInfo;
	LogDirLoadUpdateData* recData = (LogDirLoadUpdateData*) new lhBuff;

	recData->load1 = 0.0;  // different from above function !
	recData->load2 = load;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_DIR_LOAD_UPDATE;
	recInfo.flags = UNDO_ONLY;  // the big difference !!
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
						ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}

/******************
LH_LogLogicDecrElCnt
(used by lh_RemoveEntry, LH_DecrElCnt
and undoLHash)
*******************/
void LH_LogLogicDecrElCnt(PID& pid, PAGEHASH* pHash, LRC& lrc,
					PID& const rootPid, void* const key,
					TWO keyLen, TWO maxKeyLen, SMDATATYPE keyType)
{
	LogLhashLogicDecrElCntData* recData =
	(LogLhashLogicDecrElCntData*) new lhBuff;

	recData->rootPid = rootPid;
	recData->maxKeyLen = maxKeyLen;
	recData->keyType = keyType;
	recData->keyLen = keyLen;
	BCOPY(key, recData->key, keyLen);

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_DECR_OIDCNT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) +
	ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}

	delete (lhBuff*) recData;
}

/****************
LH_LogLogicIncrElCnt
(used by lh_InsertEntry, LH_IncrElCnt
and undoLHash)
*****************/
void LH_LogLogicIncrElCnt(PID& pid, PAGEHASH* pHash, LRC& lrc,
					PID& const rootPid, void* const key,
					TWO keyLen, TWO maxKeyLen, SMDATATYPE keyType)
{
	LogLhashLogicIncrElCntData* recData =
	(LogLhashLogicIncrElCntData*) new lhBuff;

	recData->rootPid = rootPid;
	recData->maxKeyLen = maxKeyLen;
	recData->keyType = keyType;
	recData->keyLen = keyLen;
	BCOPY(key, recData->key, keyLen);

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_INCR_OIDCNT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) +
	ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}

	delete (lhBuff*) recData;
}

/***************
LH_LogLogicSetOverflow
(used by lh_InsertEntry, LH_SetOverflow
and undoLHash)
****************/
void LH_LogLogicSetOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc,
		PID& const rootPid, void* const key, TWO keyLen, TWO maxKeyLen,
		SMDATATYPE keyType, int numEl, int elSize, void* const elList,
		PID& const ovPid)
{
	LogLhashLogicSetOverflowData* recData =
	    (LogLhashLogicSetOverflowData*) new lhBuff;

	recData->rootPid = rootPid;
    recData->ovPid = ovPid;
	recData->maxKeyLen = maxKeyLen;
    recData->numEl = numEl;
    recData->elSize = elSize;
	recData->keyLen = keyLen;
    recData->keyType = keyType;
	BCOPY(key, recData->KeyValue(), keyLen);
    BCOPY(elList, recData->ElList(), numEl * elSize);

	LH_LOGRECINFO recInfo;
    recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_SET_OVERFLOW;
    recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
    SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) +
    ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}

    delete (lhBuff*) recData;
}

#ifdef SERVER_MAKE

/***************
LH_LogLogicResetOverflow
(used by LH_ResetOverflow and undoLHash)
****************/
void LH_LogLogicResetOverflow(PID& pid, PAGEHASH* pHash, LRC& lrc,
		PID& const rootPid, void* const key, TWO keyLen, TWO maxKeyLen,
		SMDATATYPE keyType, int numEl, int elSize, void* const elList,
		PID& const ovPid)
{
	LogLhashLogicResetOverflowData* recData =
	    (LogLhashLogicResetOverflowData*) new lhBuff;

	recData->rootPid = rootPid;
    recData->ovPid = ovPid;
	recData->maxKeyLen = maxKeyLen;
    recData->numEl = numEl;
    recData->elSize = elSize;
	recData->keyLen = keyLen;
    recData->keyType = keyType;
	BCOPY(key, recData->KeyValue(), keyLen);
    BCOPY(elList, recData->ElList(), numEl * elSize);

	LH_LOGRECINFO recInfo;
    recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_LOGIC_RESET_OVERFLOW;
    recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
    SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) +
    ALIGNSIZE(recData->Size());

	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}

    delete (lhBuff*) recData;
}

#endif


/***************
LH_LogOvConvert
(used by LH_SplitPage)
****************/
void LH_LogOvConvert(PID& pid, PAGEHASH* pHash, LRC& lrc,
		FOUR oldHashVal, FOUR newHashVal, BOOL cvtFlag)
{
	LogLhashOvConvertData* recData =
	    (LogLhashOvConvertData*) new lhBuff;

	recData->oldHashVal = oldHashVal;
	recData->newHashVal = newHashVal;
	recData->cvtFlag = cvtFlag;

	LH_LOGRECINFO recInfo;
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_OV_CONVERT;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);

	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) +
	ALIGNSIZE(recData->Size());
						   
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}

	delete (lhBuff*)recData;
}

/*****************
LH_LogSetThreshold
(used by LH_SetLoadThreshold.C)
******************/
void LH_LogSetThreshold(PID& pid, PAGEHASH* pHash, LRC& lrc,
						float oldThreshold, float newThreshold)
{
	LH_LOGRECINFO recInfo;
	LogSetThresholdData* recData = (LogSetThresholdData*) new lhBuff;
	recData->oldThreshold = oldThreshold;
	recData->newThreshold = newThreshold;
	
	recInfo.type = LOG_REC_TYPE_USER;
	recInfo.action = LOG_ACTION_LHASH_SET_THRESHOLD;
	recInfo.flags = NOFLAGS;
	recInfo.imageCount = 1;
	SetImage(recInfo, 0, recData->Size(), recData);
	
	int fixedSize = ALIGNSIZE(sizeof(LOGRECORDHDR)) + 
									ALIGNSIZE(recData->Size());
	
	if (!disabled)  {
		GENLRC();
		LogIt(recInfo, fixedSize, pHash);
		}
	
	delete (lhBuff*)recData;
}
