/*
 *	$RCSfile: LH_PrintIndex.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"

int maxOverflow;

void LH_PrintIndex(PID& const dirPid, BUFGROUP* bufGroup)
{
	PageDesc	dirPd;
	LHDIRPAGE*	dirp;
	LHINDPAGE*	indp;

	if (dirPd.Read(bufGroup, dirPid, SH))  {
#ifdef SERVER_MAKE
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
#else
		SM_ERROR(TYPE_LOG, sm_errno);
#endif
		dirPd.UnfixUnlock();
		return;
		}

	dirp = dirPd;
	ASSERT3(dirp->SelfID() == dirPid);
	dirp->Print();

	if (dirp->IsRoot()) maxOverflow = 0;

	if (dirp->Height()>1) {
		//
		// recursively print out all directory pages and index pages
		//
		for (int i=0; i<dirp->CurPos(); i++) {
			PID curPid;
			curPid.volid = dirp->SelfID().volid;
			curPid.page = dirp->GetEntry(i);
			LH_PrintIndex(curPid, bufGroup);
			}
		}
	else {
		for (int i=0; i<dirp->CurPos(); i++)	{
			PageDesc indPd;
			PID	indPid;
			indPid.volid = dirp->SelfID().volid;
			indPid.page = dirp->GetEntry(i);
			if (indPid.page == NULLPID) 
				continue;
			if (indPd.Read(bufGroup, indPid, SH))  {
#ifdef SERVER_MAKE
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
#else
				SM_ERROR(TYPE_LOG, sm_errno);
#endif
				return;
				}
			indp = indPd;
			indp->Print(bufGroup);
			int ovFlag = indp->HaveOv();
			SHORTPID	page = indp->OvPage();
			int ovCount = 0;
			while (ovFlag)	{
				PID	ovPid;
				PageDesc ovPd;
				ovPid.volid = indp->SelfID().volid;
				ovPid.page = page;
				if (ovPd.Read(bufGroup, ovPid, SH))  {
#ifdef SERVER_MAKE
					SM_ERROR(TYPE_FATAL, esmINTERNAL);
#else
					SM_ERROR(TYPE_LOG, sm_errno);
#endif
					return;
					}
				ovCount++;
				LHINDPAGE* ovp = ovPd;
				ovp->Print(bufGroup);
				ovFlag = ovp->HaveOv();
				page = ovp->OvPage();
				ovPd.Unfix();
				}	
			indPd.Unfix();
			if (ovCount > maxOverflow) maxOverflow = ovCount;
			}
		}
	dirPd.UnfixUnlock();
	if (dirp->IsRoot()) {
		printf("\n\n max overflow level = %d\n", maxOverflow);
	        printf("\t# of primary pages = %d:\n", (int) dirp->Primary());
		printf("\t\tcurrent split ptr  = %d:\n", (int) dirp->SplitPtr());
		printf("\t\tcurrent level  = %d:\n", (int) dirp->Level());
		printf("\t\tcurrent load  = %f:\n", dirp->Load());
		}

}
