/*
 *	$RCSfile: LH_ResetOverflow.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "LHSTACK.h"
#include "LH_SMOPages.h"

#ifdef SERVER_MAKE

int LH_ResetOverflow(TID tid, PID& const rootPid, BUFGROUP* bufGroup, 
					void* key, int keyLen, 
					int maxKeyLen, PFC compFunc,
					void* const elList, int numEl)
{
	TWO         slot;
	int         keyExists;
	PageDesc	curPd;
	LHashStack	dirPl(bufGroup), curPl(bufGroup);
	PID			indPid = rootPid;

    //
	//  Traverse the directory first
	//
	if (LH_DirSearch(rootPid, keyLen, key, dirPl, indPid.page, LH_READ))
		return esmFAILURE;

	//
	//  Find the index page
	//
	if (LH_IndSearch(indPid, keyLen, key, curPd, curPl, keyExists,
						slot, LH_INSERT, compFunc))  
		return esmFAILURE;

	if (!keyExists) 
		return esmFAILURE;

	LHINDPAGE* curp = curPd;

	BT_Tuple* tp = & (curp->Tuple(slot));
	ASSERT3(tp->ElCnt() == numEl);
	PID ovPid = tp->OverflowRoot();
	
	int length = numEl * tp->ElSize();
	
	if (curp->FreeSpace() >= length) {
		
		curp->SetUnderflow(slot, numEl, elList);
		ASSERT3(curp->CheckContent());
	
		tp = & curp->Tuple(slot);
	
		LH_LogLogicResetOverflow(curp->SelfID(), curPd, curp->lrc(),
							rootPid, key, keyLen, maxKeyLen, curp->KeyType(),
							tp->ElCnt(), tp->ElSize(), tp->ElArray(0), ovPid);
		curPd.Dirty();
		return esmNOERROR;
	}

	//
	// duplicate the old tuple and append the new Els
	//
	BT_Tuple scratch;
	BCOPY(&(curp->Tuple(slot)), &scratch, curp->Tuple(slot).Size());

	ASSERT3( ! scratch.IsOverflow());
	ASSERT3( ! scratch.WillOverflow());

	scratch.ResetOverflow(numEl,elList);

	ASSERT3(scratch.Size() > curp->Tuple(slot).Size());
    
	//
	// delete the old tuple from the current page it resides in
	//
	LH_LogPhysicDelete(curp->SelfID(), curPd, curp->lrc(), slot, 1);
	curp->RemoveTuple(slot);

	//
	// insert appended tuple
	//
	if (LH_InsertNewTuple(tid, bufGroup, curPl, curPd, slot, 
							scratch, compFunc)) {
		return esmFAILURE;
		}

	curp = curPd;
	LH_LogPhysicInsert(curp->SelfID(), curPd, curp->lrc(), slot, 1);

	return esmNOERROR;
}

#endif
