/*
 *	$RCSfile: LH_SetOverflow.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "LHSTACK.h"
#include "LH_SMOPages.h"
#include "OVPAGE.h"


#ifdef SERVER_MAKE

int LH_SetOverflow(TID tid, PID& const rootPid, BUFGROUP* bufGroup, 
					void* key, int keyLen, 
					int maxKeyLen, PFC compFunc,
					void* const elList, int unused /* elSize */, int numEl)
{
	TWO         slot;
	int         keyExists;
	PageDesc    curPd;
	LHashStack  dirPl(bufGroup), curPl(bufGroup);
	PID         indPid = rootPid;

	//
	//  Traverse the directory first
	//
	if (LH_DirSearch(rootPid, keyLen, key, dirPl, indPid.page, LH_READ))
		return esmFAILURE;

	//
	//  Find the index page
	//
	if (LH_IndSearch(indPid, keyLen, key, curPd, curPl, keyExists,
							slot, LH_INSERT, compFunc))  
		return esmFAILURE;
	

	if (!keyExists) 
		return esmFAILURE;

	LHINDPAGE* curp = curPd;

	BT_Tuple& tp = curp->Tuple(slot);
	ASSERT3( ! tp.IsOverflow());
	ASSERT3(tp.WillOverflow());
	
	ASSERT3(tp.ElCnt() == numEl);
	ASSERT3( ! bcmp((char*)elList, (char*)tp.ElArray(0), numEl * tp.ElSize()));
	
	PID ovPid;
	if (ov_NewCluster(tid, rootPid.volid, bufGroup, tp.ElSize(), 
				ovPid, tp)) {
		return esmFAILURE;
		}

	LH_LogLogicSetOverflow(curp->SelfID(), curPd, curp->lrc(), rootPid, 
						key, keyLen, maxKeyLen, curp->KeyType(), 
						tp.ElCnt(), tp.ElSize(), tp.ElArray(0), ovPid);
									
	curp->SetOverflow(slot, ovPid);
	ASSERT3(tp.IsOverflow());
	
	curPd.Dirty();
	return esmNOERROR;
}

#endif
