/*
 *	$RCSfile: lh_CreateIndex.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "LHDIRPAGE.h"
#include "LH_SMOPages.h"

//
//	Function	: int lh_CreateIndex(int volume, int groupIndex, PID& rootPid)
//	Description	: Create a directory page on volume, and returns the root pid.
//
int lh_CreateIndex(TID tid, int volume, BUFGROUP* bufGroup, 
						PID& rootPid, SMDATATYPE type)
{
	if (BT_SetAnchor(tid))  {
		return esmFAILURE;
	}
		
	//
	//	Allocate the directory page for our index
	//
	if (LH_AllocDirPage(tid, volume, bufGroup, rootPid, TRUE, type)) {
		return esmFAILURE;
		}

	LH_SMOPages smo;
	PageDesc rootPd;
	if (rootPd.Read(bufGroup, rootPid, EX))  {
		return esmFAILURE;
		}
	
   	//
	//  Allocate initial primary buckets
	//
	LHDIRPAGE* rootp = rootPd;

	for (int i=0; i<INIT_PRIMARY; i++)  {
	//
	// change to have no empty index pages at begining, 1/11/93
	//
		rootp->SetNextEntry(NULLPID);
		rootp->IncrCount();
		LH_LogDirSetNextEntry(rootp->SelfID(), rootPd, rootp->lrc(),
								NULLPID, i);
					
		}

	
	smo.Insert(rootPid);
	rootPd.UnfixDirty();
	smo.Flush();

	if (BT_SetCLR(tid))  {
		return esmFAILURE;
	}
		
	return esmNOERROR;
}
