/*
 * $RCSfile: lh_DestroyIndex.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "io_extfuncs.h"

const listSize = 128;

//
//	Function	: int lh_DestroyIndex(TID tid, PID& const dirPid, 
//											BUFGROUP* bufGroup)
//	Description	: Destroy both directory and index pages.
//
int lh_DestroyIndex(TID tid, PID& const dirPid, BUFGROUP* bufGroup)
{
	// accumulate pids in a list, and, 
	// do 1 io_Deallloc for each directory page.

	PageDesc dirPd; 

#ifdef SERVER_MAKE
	TRANSREC* transRec = (TRANSREC*) Active->transRec;
	ASSERT1(transRec->tid == tid);
#endif

	if (dirPd.Read(bufGroup, dirPid, EX))
		return esmFAILURE;

	LHDIRPAGE* dirp = dirPd;
	int total = (int)dirp->CurPos();

	if (dirp->Height()>1) {
		for (int i=0; i < total; i++) {
			PID curPid = dirPid;
			curPid.page = dirp->GetEntry(i);
			lh_DestroyIndex(tid, curPid, bufGroup);
			}
		}
	else {	
		int curr = 0; 
		int overFlow = FALSE;
		while (curr < total)  {
			PID list[listSize];
			PID	curPid = dirPid;
			int cnt = 0;
			do {
				PageDesc pd;
				if (!overFlow)	
					curPid.page = dirp->GetEntry(curr);
				if (curPid.page == NULLPID) {
					curr++;
					continue;
				}
				if (pd.Read(bufGroup, curPid, EX))  {
					dirPd.Unfix();
					return esmFAILURE;
					}
				LHINDPAGE* page = pd;
				if (overFlow && !page->OvLabel()) {
					//
					// the page is promoted to be primary, it should be
					// deallocated later
					//
					pd.UnfixUnlock();
					overFlow = FALSE;
					curr++;
					continue;
					}

				list[cnt++] = page->SelfID();
				for (int i = 0; i < page->NumSlots(); i++)  {
					BT_Tuple& tp = page->Tuple(i);
					if (tp.IsOverflow()) {
						if (ov_DestroyCluster(tid, tp.OverflowRoot(),
												bufGroup)) {
							pd.Unfix();
							dirPd.Unfix();
							return esmFAILURE;
							}
						}
					}
				if (page->OvPage()!=NULLPID) {
					curPid.page = page->OvPage();
					overFlow = TRUE;
					}
				else	{
					overFlow = FALSE;
					curr++;
					}
				pd.Unfix();
				} while (cnt < listSize && curr < total);
		
			if (cnt > 0) {
#ifdef SERVER_MAKE
				for (int i = 0; i < cnt; i++)  {
					LH_TRACE(("Dealloc page %d\n", list[i].page));
					}
				if (io_FakeDeallocPages(BTREE_PAGE2SIZE, cnt, list, 
							&(transRec->pageDeallocList), PAGE_INDEX)){
					dirPd.Unfix();
					return esmFAILURE;
					}
#else
				if (io_DeallocPages(BTREE_PAGE2SIZE, cnt, list, FALSE, PAGE_INDEX)) {
					dirPd.Unfix();
					return esmFAILURE;
					}
#endif
				}	
			}
		}

	//
	//	finally deallocate the directory page
	//
	dirPd.Unfix();

#ifdef SERVER_MAKE
	LH_TRACE(("Dealloc page %d\n", dirPid.page));
	if (io_FakeDeallocPages(BTREE_PAGE2SIZE, 1, &dirPid, 
				&(transRec->pageDeallocList), PAGE_INDEX)){
		return esmFAILURE;
		}
#else
	if (io_DeallocPages(BTREE_PAGE2SIZE, 1, &dirPid, FALSE, PAGE_INDEX))
			return esmFAILURE;
#endif

	return esmNOERROR;
}
