/*
 *	$RCSfile: lh_FetchInit.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "LHSTACK.h"
#include "OVPAGE.h"


//
//	Function	: int lh_FetchInit(PID& const dirPid, int groupIndex,
//										KEY* bound1,
//                                		int maxKeyLen, BT_CURSOR* cursor,
//                                		SMCOND condition, PFC compFunc)
//	Description	: Search the for bound1 (if specified).
//				  Exact match queries only, where bound1==bound2, are
//				  allowed for linear hashing
//
int lh_FetchInit(PID& const rootPid, BUFGROUP* bufGroup, void* bound1, 
			int bound1Len, int maxKeyLen, BT_CURSOR* cursor,
			SMCOND condition, PFC compFunc)
{
	TWO			slot;
	int		found;
	PID			pid;
	PageDesc    curPd;
	LHashStack  dirPl(bufGroup), curPl(bufGroup);
	
	pid.volid = rootPid.volid;
	cursor->valid = FALSE;

	//
	//	Position 'slot' and 'pl' at the appropriate starting position
	//
	if (bound1)  {
	    if (LH_DirSearch(rootPid, bound1Len, bound1, dirPl, pid.page, LH_READ))
			return esmFAILURE;
		
		if (pid.page==NULLPID)  { // empty index
			return esmNOERROR;
		}
		if (LH_IndSearch(pid, bound1Len, bound1, curPd, curPl, found,
							slot, LH_READ, compFunc))  
			return esmFAILURE;

		if (condition == SM_EQ && !found)	
			return esmNOERROR;
		}
	else {
		SM_ERROR(TYPE_USER, esmINVALIDCOND);
		return esmFAILURE;
		}

	LHINDPAGE* curp;

	curp = curPd;

	//
	//	Position element array index in a leaf tuple
	//

	cursor->valid = TRUE;
	cursor->rootPID = rootPid;
	cursor->page = curp->SelfID();
	cursor->slotNum = slot;
	cursor->index = 0;
	
	return esmNOERROR;
}
