/*
 *	$RCSfile: lh_FetchNext.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"


//
//	Function	: int lh_FetchNext(BT_CURSOR& cursor, int groupIndex, 
//										KEY* bound2, SMCOND condition, 
//										PFC compFunc, void* ret)
//	Description	: Given a cursor, look a key that satisfies
//				  condition(key, bound2). The result element of the search 
//			      is returned in ret. 
//
int lh_FetchNext(BT_CURSOR& cursor, BUFGROUP* bufGroup, void* bound2, 
			int bound2Len, SMCOND condition, PFC compFunc, 
			void* valuePtr, TWO* lenPtr, void* ret)
{
	ASSERT1(cursor.valid);

	PageDesc 	pd;

	if (pd.Read(bufGroup, cursor.page, SH))
		return esmFAILURE;

	LHINDPAGE* indp = pd;
	
	BT_Tuple& tp = indp->Tuple(cursor.slotNum);
		
	//
	//	Fill return values
	//
	if (lenPtr)  {
		if (*lenPtr < tp.KeyLen())  {
			SM_ERROR(TYPE_USER, esmKEYTOOLONG);
			pd.Unfix();
			return esmFAILURE;
			}
		*lenPtr = tp.KeyLen();
		BCOPY(tp.KeyValue(), valuePtr, *lenPtr);
		}
	if ( ! tp.IsOverflow()) 
		BCOPY( tp.ElArray(cursor.index), ret, tp.ElSize() );
	else if (ov_GetElem(tp.OverflowRoot(), bufGroup, cursor.index, ret)) {
		pd.Unfix();
		return esmFAILURE;
		}

	//
	//	Move the cursor to the next eligible unit based on 'condition'
	//
	int found = TRUE;
	switch (condition)  {

		case SM_EQ:
			if (++cursor.index >= indp->Tuple(cursor.slotNum).ElCnt()) 
				found = FALSE;
			break;

		default:
			//
			//	Illegal 'condition'
			//
			SM_ERROR(TYPE_USER, esmINVALIDCOND);
			pd.Unfix();
			return esmFAILURE;
		}

	//
	//	Check if condition is still satisfied by the entry in
	//	the new cursor position.
	//
	if ( ! found) 
		cursor.valid = FALSE;

	pd.Unfix();

	return esmNOERROR;
}
