/*
 *	$RCSfile: lh_SetLoadThreshold.C,v $
 *	$Revision: 1.1.1.1 $
 *	$Date: 1996/05/04 21:55:25 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "BTREEPAGE.h"

//
//  Function    :   int lh_SetLoadThreshold(PID& const rootPid,
//			BUFGROUP* bufGroup, float load)
//
//  Description :   Set load threshold for the linear hashing index
//
int lh_SetLoadThreshold(PID& const rootPid, BUFGROUP* bufGroup, float load)
{
	PageDesc rootPd;

	if (rootPd.Read(bufGroup, rootPid, EX))
		return esmFAILURE;

	LHDIRPAGE* rootp = rootPd;

	if (load < 0.0 || load > 1.0) {
		SM_ERROR(TYPE_USER, esmWRONGHASHLOAD);
		rootPd.Unfix();
		return esmFAILURE;
		}

	LH_LogSetThreshold(rootPid, rootPd, rootp->lrc(), 
			rootp->Threshold(), load);

	rootp->SetThreshold(load);

	rootPd.UnfixUnlockDirty();
	
	return esmNOERROR;
}
