/*
 *   $RCSfile: rpc_CheckPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:26 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "lock.h"
#include "bf.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "serverinfo.h"
#include "sm_state.h"
#include "msg_funcs.h"
#include "lm_intfuncs.h"
#include "io_extfuncs.h"
#include "trans_globals.h"


 int
rpc_CheckPage (

	register PID	*pid,
	int				releaseLimit,
	int*			serverReleaseLimit 
)
{
	MESSAGE			message;
	VOLREC			*volRec;

	TRACE(TR_LOCK, TR_LEVEL_1);

	/* check to see if the volume is mounted */
	if ((volRec = io_FindVolId(pid->volid)) == NULL) {
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	*serverReleaseLimit = releaseLimit;

	/*
	 *	load up a disk message
	 */
	message.header.type =			QUERY_PAGE;
	message.header.replyRequested =		TRUE;
	message.header.params.in.tid =		GET_SERVERTID;
	message.body.lockQuery.pid =	*pid;
	message.body.lockQuery.releaseLimit = releaseLimit;

	/*
	 *	write the read message
	 */
	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

		return(esmFAILURE);

	} else {
		*serverReleaseLimit = (int) message.body.lockQuery.releaseLimit;

		return(esmNOERROR);
	}
}
