/*
 *   $RCSfile: rpc_LockPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:26 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "object.h"
#include "lock.h"
#include "bf.h"
#include "lm.h"
#include "sm_state.h"
#include "msgdefs.h"
#include "volume.h"
#include "trans.h"
#include "msg_funcs.h"
#include "io_extfuncs.h"
#include "lm_extfuncs.h"
#include "lm_intfuncs.h"
#include "trans_globals.h"
#include "lock_globals.h"
#include "sm_globals.h"


 int
rpc_LockPage (

	const PID		*pid,
	PAGETYPE		pageType,
	LOCKMODE		*lockMode,
	FID				*fid 
)
{

	MESSAGE			message;
	VOLREC			*volRec;
	int				retryFlag = 1;


	TRACE(TR_LOCK, TR_LEVEL_1);

    /* check to see if the volume is mounted */
    if ((volRec = io_FindVolId(pid->volid)) == NULL) {
        SM_ERROR(TYPE_USER, esmBADVOLID);
        return(esmFAILURE);
    }

	/* 
	printf("Request lock %s on page %d\n", LM_Character[*lockMode], pid->page);
	*/

	/*
	 *	load up a disk message
	 */
again:
	message.header.type =			LOCK_PAGE;
    message.header.replyRequested = TRUE;
	message.header.params.in.tid =	GET_SERVERTID;
	message.body.lock.pid =         *pid;
	if (fid)  
		message.body.lock.fid =		*fid;
	else
		message.body.lock.fid.pid.page = NULLPID;

	message.body.lock.pidLockMode =	*lockMode;

	/*
	 *	check to see what type of lock mode is requested
	 */
	if (*lockMode == SH)	{
		
		message.body.lock.fidLockMode = IS;

	} else {

		message.body.lock.fidLockMode = IX; /* MJZ: change from EX */
	}

	/*
	 *	send the lock message
	 */
	if (callServer(volRec->serverInfo, &message, NULL, 0) < 0)	{

		if ((sm_errno == esmPHANTOMDEADLOCK ||
				sm_errno == esmCAUSEDPHANTOMDEADLOCK) && 
				pageType == H_INDEX && retryFlag)  {
			lm_InvalidateAllLocks();
			if (sm_errno == esmCAUSEDPHANTOMDEADLOCK) retryFlag = 0;
			printf("Phantom deadlock detected. Retrying!\n");
			goto again;
			}
		sm_errno = esmLOCKCAUSEDDEADLOCK;
		return(esmFAILURE);

	} else {
		/* return the current lock mode */
		*lockMode = message.body.lock.pidLockMode;
		return(esmNOERROR);
	}
}
