/*
 *   $RCSfile: connectServer.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:26 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "host.h"
#include "bf.h"
#include "lsn.h"
#include "loginfo.h"
#include "serverinfo.h"
#include "bitvec.h"
#include "link.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "sm_globals.h"
#include "util_funcs.h"

int
connectServer (
	SERVERINFO			*serverInfo
)
{
	int					socketBufSize = 16*1024; /* 16K socket buffers */

	TRACE(TR_MSG, TR_LEVEL_1);

	if( serverInfo->socket == NOSOCKET) {
		int fd;

		if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)	{
			SM_ERROR(TYPE_LOG, errno);
			return esmFAILURE;
		}
		TRPRINT(TR_MSG, TR_LEVEL_1, ("connection socket:%d", fd));

		/*
		 *  set socket to not delay sending data and enlarge the socket
		 *	send and receive buffer size to 16K.
		 */
		if (setSocketOptions(fd, socketBufSize, socketBufSize) != esmNOERROR) {
			return esmFAILURE;
		}
		serverInfo->socket = fd;

		if (connect(serverInfo->socket, (struct sockaddr *) 
			&serverInfo->connectAddr, sizeof(SOCKADDR)) < 0)	{
			if(errno != EISCONN) {
				SM_ERROR(TYPE_LOG, errno);
				if (close(serverInfo->socket) != 0) {
					SM_ERROR(TYPE_LOG, errno);
				}
				serverInfo->socket = NOSOCKET;
				return esmFAILURE;
			}
		}
		setLink(fd, serverInfo);
		serverInfo->flags |= (SERV_CONNECTED|SERV_CONTACTED_IN_PAST);
		TRPRINT(TR_MSG, TR_LEVEL_1, ("connect successful"));
	}

	return esmNOERROR;
}
