/*
 *   $RCSfile: msg_Receive.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:27 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "bf.h"
#include "lsn.h"
#include "loginfo.h"
#include "logrecs.h"
#include "volume.h"
#include "trans.h"
#include "clog.h"
#include "serverinfo.h"
#include "bitvec.h"
#include "link.h"
#include "msg_funcs.h"
#include "trans_funcs.h"
#include "msg_globals.h"
#include "clog_globals.h"
#include "util_funcs.h"
#include "trans_globals.h"
#include "sm_globals.h"

/************************************************************************
 *																		*
 * msg_Receive ()														*
 *																	    *
 *		Receive a message from the server, updating the current 	    *
 *	    knowledge of the tailLSN of the log.							*
 *																		*
 ************************************************************************/

 int
msg_Receive (

 	SERVERINFO	*serverInfo,	
	MESSAGE		*message,
 	int			size 	
)
{
	TID				tidOfMessage = message->header.params.in.tid;
	TRANSREC		*transRec = NULL;

	TRACE(TR_MSG, TR_LEVEL_1);

	/*
	 *	read the reply
	 */
	switch (receive(serverInfo, (char *) message, size))	{
	case esmFAILURE:
		return esmFAILURE;

	case esmPROTOCOLVERSION:
		message->header.params.out.errno = esmPROTOCOLVERSION;
		break;

	case esmNOERROR:
		break;
	}

	/* Grab the latest tailLSN from the message header */
	serverInfo->logInfo.lastTailLSN = message->header.params.out.endOfLog;

	/*
	 *	check to see if the message has an error code
	 */
	if (message->header.params.out.errno != esmNOERROR)	{
		MsgStats.replied_in_error[message->header.type]++;

		/*
		 * 	If the transaction has been aborted, then record the fact
		 */
		if (message->header.params.out.errno == esmBADTRANSID ||
			message->header.params.out.errno == esmBADMESSAGEMAGIC ||
			message->header.params.out.errno == esmPROTOCOLVERSION ||
			message->header.params.out.errno == esmTRANSABORTED) {

			serverAbortedTrans(serverInfo, tidOfMessage);

			/* remember why this trans was aborted */
			sm_reason = (int) message->header.params.out.linkTransError;
		}

#ifdef JUNK
		/*
		 *	if logging is disabled, consider the server to be
		 *	disconnected
		 */
		if (message->header.params.out.errno == esmLOGDISABLED) 
			(void) detectedDisconnect(serverInfo);
#endif JUNK

		/*
		 *	Record the errno.  A bad transid indicates that the
		 *	server aborted the transaction.
		 */
		transRec = findTransRec(getLocalTid());
		if ((message->header.params.out.errno == esmBADTRANSID) &&
			(transRec->clientTransState == T_ACTIVE) ) { 
			message->header.params.out.errno = esmTRANSABORTED;
		}
		SM_ERROR(TYPE_LOG, (int)message->header.params.out.errno);
		return esmNOERROR;
	
	} else {
		MsgStats.replied_ok[message->header.type]++;
		return esmNOERROR;
	}
}
