/*
 *   $RCSfile: sm_AdminFuncs2.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include <sys/resource.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "msgdefs.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "scan.h"
#include "volume.h"
#include "adminmsg.h"
#include "sm_extfuncs.h"
#include "msg_funcs.h"
#include "io_extfuncs.h"
#include "io_intfuncs.h"
#include "msg_globals.h"
#include "trans.h"
#include "trans_funcs.h"
#include "trans_globals.h"



/*
 * helperfunc1a: sends the message and receives a response.
 * Assumes that the server Tid is already in the message or 
 * the message doesn't require a Tid.
 */
static  BOOL
helperfunc1a (
	SERVERINFO  *serverInfo,
	MESSAGE 	*msg,
	IOVECENT	*iov,
	int			iovlength,
	char 		**inbuf,
	int			inbufsize
)
{
	MESSAGE 	message;
	static IOVECTOR	mine;
	
	message = *msg; /* a copy */

	if(server_connectInit(serverInfo, TRUE/* is admin */) != esmNOERROR) {
		return FALSE;
	}
	if(iovlength == 0) {
		SM_ASSERT(LEVEL_1, iov == NULL);
		iov = mine;
		iovlength = 1;
	} else {
		SM_ASSERT(LEVEL_1, iov != NULL);
	}
	iov[0].iov_base = (char *)&message;
	iov[0].iov_len  = (int) sizeof(MESSAGE);

	if(callServerMulti(serverInfo, iov, iovlength) == esmNOERROR) {
		if(inbufsize > 0) {
			if(receive(serverInfo, *inbuf, inbufsize) == esmNOERROR){
				if(message.header.type == ADMIN_STATISTICS)	{
					struct sockaddr     *sa;
					int                 salen = sizeof(struct sockaddr);

					sa =  (struct sockaddr *) &(((SERVERSTATS *)(*inbuf))->serverAddr);
					if(getpeername(serverInfo->socket, sa, &salen) < 0) {
						SM_ERROR(TYPE_WARNING, errno);
					} else {
						if(salen != sizeof(*sa)) {
							fprintf(stderr, "Unexpected address size:%d!\n", salen);
							SM_ERROR(TYPE_FATAL, esmINTERNAL);
						}
					}
				}
				(*inbuf) += inbufsize;
			}
		}
	}
	return FALSE; 
}

static BOOL
helperfunc2a (
	SERVERINFO  *serverInfo,
	MESSAGE 	*msg,
	IOVECENT	*iov,
	int			iovlength,
	char 		**buffer,
	int			bufferSize
)
{
	if(serverInfo->flags & SERV_CONTACTED_IN_PAST) {
		helperfunc1a (serverInfo, msg, 
			iov, iovlength, buffer, bufferSize);
	}
	return FALSE;
}


static BOOL
helperfunc3a (
	SERVERTRANSREC  *stl,
	MESSAGE 	*msg,
	IOVECENT	*iov,
	int			iovlength,
	char 		**buffer,
	int			bufferSize
)
{
	/* No admin func (at present) needs a tid, so this is
	 * really superfluous, but we will leave it in for 
	 * possible extensions.
	 */
	msg->header.params.in.tid = stl->tid;
	helperfunc1a (stl->serverInfo, msg, 
			iov, iovlength, buffer, bufferSize);
	return FALSE;
}

static int 
doit2(FLAGS flags, VOLID volid, MESSAGE *mp, 
	IOVECENT	*iov,
	int			iovlength,
	void *bufp, int bufsz
) /* bufp is a char ** */
{
	extern int sm_errno;

	VOLREC *volRec;
	sm_errno = 0;

	switch(flags) {
	case VOL_BY_VOLID:
		volRec = IO_FindVolId( volid, SERV_ESTAB );
		if( volRec == NULL) {
			return esmFAILURE;
		} 
		if( volRec->serverInfo == NULL) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
		sm_errno = 0;
		(void) helperfunc1a(volRec->serverInfo, mp, 
			iov, iovlength, (char**)bufp, bufsz);
		break;
	case VOL_ALL:
		/* stupid */
		serverForEach(6, helperfunc1a , mp, 
			iov, (void *) iovlength, bufp, (void *)bufsz);
		break;
	case VOL_USED_IN_TRANSACTION:
		ForEachServerInTrans(findTransRec(getLocalTid()), 6, helperfunc3a, 
						mp, iov, (void *)iovlength, bufp, (void *)bufsz);
		break;
	case VOL_MOUNTED_SINCE_INIT:
		serverForEach(6, helperfunc2a,  mp, 
					iov, (void *)iovlength, bufp, (void *)bufsz);
		break;
	}
	if(sm_errno)
		return esmFAILURE;
	return esmNOERROR;
}

static BOOL
countVolume(
	VOLREC *volRec,
	int 	*total
) 
{
	if(volRec->flags & VOL_CLIENT_MOUNTED) {
		(*total)++;
	}
	return FALSE;
}
static BOOL
countServers(
	SERVERINFO *unused,
	int 	   *total
) 
{
	(*total)++;
	return FALSE;
}

static int
count(FLAGS flags, int *total) 
{
	/* can't require more than the total number of servers */
	switch(flags) {
	case VOL_BY_VOLID:
		*total = 1;
		break;

	case VOL_ALL:
		serverForEach(2, countServers, total);
		break;

	case VOL_USED_IN_TRANSACTION:
		if(checkLocalTransState()!= esmNOERROR) {
			return esmFAILURE;
		}
		ForEachServerInTrans(
			findTransRec(getLocalTid()), 2, countServers, total );
		break;

	case VOL_MOUNTED_SINCE_INIT:
		ForEachVolume(2, countVolume,  total);
		break;
	}
	return esmNOERROR;
}


 int
sm_ResetRusage (
	FLAGS		flags,
	VOLID		volid
)
{
	MESSAGE			message;

	/* 
	 * zeroes the server's rusage structure 
	 */
	message.header.type		= RESET_RUSAGE; 
	message.header.replyRequested	= TRUE; 
	return doit2(flags, volid,  &message, NULL, 0, NULL, 0);
}

 int
sm_GetRusage (
	FLAGS			flags,
	VOLID			volid,
	int				*numberOfServers,
	struct rusage	**result
)
{
	MESSAGE			message;
	int				total = 0;
	struct rusage 	*rusage_list;

	message.header.type		= GET_RUSAGE; 
	message.header.replyRequested	= TRUE; 

	if(count(flags, &total) == esmFAILURE) return esmFAILURE;
	if(count==0) {
		*numberOfServers = 0;
		*result = NULL;
		return esmNOERROR;
	}

	rusage_list = (struct rusage *)calloc(total, sizeof(struct rusage) );
	*numberOfServers = total;
	*result = rusage_list; /* rusage_list gets changed below */

	return doit2(flags, volid,  &message, 
		NULL, 0, &rusage_list, (int) sizeof(struct rusage) );
}


 int
sm_TestPageMessage (
	FLAGS		flags,
	VOLID		volid,
	int			numMsgs, 		/* number of page messages to send 	*/
	int			pageBlockCount,	/* size of page (pwr 2 multiple)	*/
	BOOL		do_send 			/* send of receive page 			*/
)
{

	MESSAGE			message;
	char			buffer[MAX_PAGESIZE];
	char			*bufptr = buffer;
	int				i;

	if ( (pageBlockCount < 1) || (pageBlockCount > 5)) {
		SM_ERROR(TYPE_USER, esmBADPAGESIZE);
		return(esmFAILURE);
	}
	message.body.page.page2size   = MIN_PAGE2SIZE + pageBlockCount - 1;


	if (do_send) {
		message.header.type		= TEST_PAGE_SEND; 
		message.header.replyRequested	= FALSE; 

	} else {

		/*
		 *	read numMsgs pages from the server  with one request.
		 */
		message.header.type		= TEST_PAGE_RPC; 
		message.header.replyRequested	= TRUE; 

	}
	for (i = 0; i<numMsgs; i++) {
		if(doit2(flags, volid,  &message, 
			NULL, 0,
			&bufptr, (int) sizeof(buffer) ) != esmNOERROR)
			return esmFAILURE;
	}
	return esmNOERROR;
}


 int
sm_ServerStatistics (
	FLAGS			flags,
	VOLID			volid,
	int				*numberOfServers,
	SERVERSTATS		**result,
	BOOL			reset
)
{
	MESSAGE			message;
	int				total = 0;
	SERVERSTATS  	*stats_list;

	message.header.type				= ADMIN_STATISTICS; 
	message.header.replyRequested	= TRUE; 
	if (reset) 	message.body.stats.flags = STAT_CLEAR;
	else		message.body.stats.flags = NOFLAGS;

	if(count(flags, &total) == esmFAILURE) return esmFAILURE;

	if(total == 0) {
		*numberOfServers = 0;
		*result = NULL;
		return esmNOERROR;
	}
	stats_list = (SERVERSTATS *)calloc(total, sizeof(SERVERSTATS) );
	/* APPLICATION MUST FREE THIS!!!!!!!!!*/

	if (stats_list == NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		*result = NULL;
		return(esmFAILURE);
	}

	*numberOfServers = total;
	*result = stats_list; /* stats_list gets changed below */

	return doit2(flags, volid,  &message, 
		NULL, 0, &stats_list, (int) sizeof(SERVERSTATS) );

}

 int
sm_AddServerVolume (
	FLAGS			flags,
	VOLID			volid,
	char			*optionname,
	char			*formatinfo
)
{
	MESSAGE			message;
	IOVECTOR		iov;

	message.header.type				= ADD_VOLUME; 
	message.header.replyRequested	= TRUE; 
	message.body.volume.optionNameLength	= strlen(optionname);
	message.body.volume.formatLength	= strlen(formatinfo);

	if(message.body.volume.optionNameLength == 0) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
	}
	if(message.body.volume.formatLength == 0) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
	}

	/* iov[0] is set by a lower function, helperfunc1a */
	iov[1].iov_base = optionname;
	iov[1].iov_len = message.body.volume.optionNameLength;
	iov[2].iov_base = formatinfo;
	iov[2].iov_len = message.body.volume.formatLength;

	return doit2(flags, volid,  &message, iov, 3, NULL, 0);
}

 int
sm_RemoveServerVolume (
	FLAGS			flags,
	VOLID			volid,
	VOLID			volid2remove
) 
{
	MESSAGE			message;

	message.header.type				= RM_VOLUME; 
	message.header.replyRequested	= TRUE; 
	message.body.volume.volid		= volid2remove;

	return doit2(flags, volid,  &message, NULL, 0, NULL, 0);
}
