/*
 *   $RCSfile: sm_BufferGroupInfo.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "lock.h"
#include "bf_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "bf_globals.h"


/*
 *  sm_BufferGroupInfo returns information about fixed pages
 *  in the group.
 *    maxPages returns the max #of fixed pages allowed in the group.
 *    fixedPages returns current # of pages fixed in this group
 *    unfixedPages returns current # of unfixed pages in this group
 */


 int
sm_BufferGroupInfo (

	int			groupIndex,		/* the buffer group to close */
	int			*maxPages,      /* max fixed pages allowed      */
	int			*fixedPages,    /* current # of pages fixed     */
	int			*unfixedPages   /* current # of pages unfixed   */
)
{

	register BUFGROUP	*bufGroup;


	TRACE(TR_SM, TR_LEVEL_1);

	/*
	 *	convert the group index into a pointer
	 */
	bufGroup = &BufGroupPtr[groupIndex];

	/*
	 *	Check the buffer group magic
	 */
	if (CHECK_BUFGROUP_VALID(bufGroup))	{

		SM_ERROR(TYPE_USER, esmBADGROUP);
		return(esmFAILURE);
	}

	/*
	 *	return the group info
	 */
    *maxPages = bufGroup->maxCharges;
    *fixedPages = bufGroup->fixedCharges;
    *unfixedPages = bufGroup->unfixedCharges;
    return(esmNOERROR);
}
