/*
 *   $RCSfile: sm_CheckIndex.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "io.h"
#include "page.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "scan.h"
#include "volume.h"
#include "bf_globals.h"
#include "bf_extfuncs.h"
#include "trans.h"
#include "trans_globals.h"
#include "sm_extfuncs.h"
#include "sm_intfuncs.h"
#include "bt_extfuncs.h"
#include "lh_extfuncs.h"
#include "sm_state.h"


void sm_CheckIndex(IID* ndx, int groupIndex)
{
	BUFGROUP* 	bufGroup = & BufGroupPtr[groupIndex];
	INDEXDESC* 	np;
	GROUPLINK* 	gLink;
	
    if (CHECK_BUFGROUP_VALID(bufGroup))  {
        SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

    INIT_SMSTATE(&SM_State);
	if (checkTransState(ndx->pid.volid))  {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	
	int loadNdx;
	if (sm_FindIndexLoad(ndx, &loadNdx)) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	if (loadNdx >= 0)  {
		// cannot be bulkloading
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	
	if ((gLink = bf_ReadPage(bufGroup, &ndx->pid, H_INDEXDESC, 
										INDEXDESC_PAGE2SIZE, SH)) == NULL)  {
		return;
		}
	
	SM_ASSERT(LEVEL_3, ((INDEXDESCPAGE*) gLink->bufFrame)->magic == NDXMAGIC);
	
	np = & (((INDEXDESCPAGE*) gLink->bufFrame)->desc);
	
	switch (np->ndxType)  {
		case SM_BTREENDX:
			bt_CheckTree(np->rootPid, bufGroup);
			break;
			
		case SM_HASHNDX:
			lh_CheckIndex(np->rootPid, bufGroup);
			break;
			
		default:
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	bf_UnfixPage(gLink, BF_LRU, FALSE);
}

