/*
 *   $RCSfile: sm_CloseBufferGroup.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "ess.h"
#include "sysdefs.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "lock.h"
#include "bf_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "bf_globals.h"


/*============================================================================*/
/* 
 * sm_CloseBufferGroup:
 * 	Routine to close a buffer group.
 *
 * IMPORTS:
 *	bf_CloseGroup()
 *
 * RETURNS:
 *      esmNOERROR
 *
 * ERROR RETURNS:
 *	errors returned by lower levels.
 *
 * SIDE EFFECTS:
 *	A buffer group is closed.
 */
/*============================================================================*/


 int
sm_CloseBufferGroup (

	int			groupIndex 		/* the buffer group to close */
)
{

	register BUFGROUP	*bufGroup;


	TRACE(TR_SM, TR_LEVEL_1);

	if(groupIndex < 0) {
		SM_ERROR(TYPE_USER, esmBADGROUP);
		return(esmFAILURE);
	}

	/*
	 *	convert the group index into a pointer
	 */
	bufGroup = &BufGroupPtr[groupIndex];

	/*
	 *	Check the buffer group magic
	 */
	if (CHECK_BUFGROUP_VALID(bufGroup))	{

		SM_ERROR(TYPE_USER, esmBADGROUP);
		return(esmFAILURE);
	}

	/*
	 *	If the buffer group is associated with the transaction,
	 *	remove it.
	 */
	if (bufGroup->flags & G_TRANS) {
		listRemove(&(bufGroup->transList));
	}

	/* 	release the group */
    return(bf_CloseGroup(bufGroup));
}
