/*
 *   $RCSfile: sm_CompFunc.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#ifdef SERVER_MAKE
#include "link.h"
#include "volume.h"
#endif
#include "error.h"
#include "checking.h"
#ifdef SERVER_MAKE
#   include "msgdefs.h"
#   include "thread.h"
#   include "semaphore.h"
#   include "latch.h"
#endif
#ifndef SERVER_MAKE
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#endif
#include "sm_extfuncs.h"



static int SM_CharComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	return (* (char*) kval1) - (* (char*) kval2);
}



//
//	Function	: int SM_IntComp(int klen1, const void* kval1, int klen2, const void* kval2)
//	Description	: returns difference of *((int*)kval1) and *((int*)kval2)
//
static int SM_IntComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	return (* (int*) kval1) - (* (int*) kval2);
}


static int SM_LongComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	return (* (long*) kval1) - (* (long*) kval2);
}


static int SM_ShortComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	return (* (short*) kval1) - (* (short*) kval2);
}


static int SM_FloatComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	float tmp = (* (float*) kval1) - (* (float*) kval2);

	return tmp < 0 ? -1 : ((tmp > 0) ? 1 : 0);
}

static int SM_DoubleComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	SM_ASSERT(LEVEL_3, klen1 == klen2);
	double tmp = (* (double*) kval1) - (* (double*) kval2);

	return tmp < 0 ? -1 : ((tmp > 0) ? 1 : 0);
}


static int SM_StringComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	register char* p1 = (char*) kval1;
	register char* p2 = (char*) kval2;
	int result;
	for (int i = klen1 < klen2 ? klen1 : klen2; 
									i > 0 && ! (result = *p1 - *p2); 
									i--, p1++, p2++);

	return result ? result : klen1 - klen2;		// longer wins;
}


#ifdef JUNK
/* not used at the moment */
/* SAVE: might use later *//
static int SM_BitsComp(int klen1, const void* kval1, int klen2, const void* kval2)
{
	register char* p1 = (char*) kval1;
	register char* p2 = (char*) kval2;
	int result;
	for (int i = klen1 < klen2 ? klen1 : klen2; 
									i > 0 && ! (result = *p1 - *p2); 
									i--, p1++, p2++);

	return result ? result : klen1 - klen2;		// longer wins;
}
#endif



//
//	Array of comparative functions
//
PFC SMCOMPFUNC [ SM_string + 1] = {
		SM_CharComp,
		SM_IntComp,
		SM_LongComp,
		SM_ShortComp,
		SM_FloatComp,
		SM_DoubleComp,
		SM_StringComp
		};

