/*
 *   $RCSfile: sm_CreateFile.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "lock.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "trans.h"
#include "file.h"
#include "sm_state.h"
#include "volume.h"
#include "io_extfuncs.h"
#include "fi_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"
#include "bf_globals.h"
#include "trans_globals.h"


/*============================================================================*/
/* 
 * sm_CreateFile:
 * 	Routine to create a file.
 *	We will probably want to make this a macro in the future.
 *
 * IMPORTS:
 *	fi_CreateFile()
 *
 * RETURNS:
 *      esmNOERROR if all went well. 
 *	The fid is filled in.
 *
 * ERROR RETURNS:
 *	errors returned by lower levels.
 *
 * SIDE EFFECTS:
 *	The fid parameter is filled in and a file is created.
 */
/*============================================================================*/


 int
sm_CreateFile (

	int					groupIndex,
	VOLID				volid,
	FID					*fid
)
{

	BUFGROUP			*bufGroup;
    PID					nearPid;
	VOLREC				*volRec;
	FLAGS				flags;

	TRACE(TR_SM, TR_LEVEL_1);

	INIT_SMSTATE(&SM_State);

	/*
	 *	check to see if a transaction is active
	 */
	if (checkTransState(volid))	{
		return(esmFAILURE);
	}

	bufGroup = &BufGroupPtr[groupIndex];

	if (CHECK_BUFGROUP_VALID(bufGroup))	{
		SM_ERROR(TYPE_WARNING, esmBADGROUP);
		return(esmFAILURE);
	}

	/*
	 *	construct a default pid
	 */
    nearPid.volid = volid;
    nearPid.page  = 0;

	/*
	 *	Determine if the file should be logically ordered.
	 * 	This can only be done for files on temporary volumes.
	 */
	if ( (volRec = io_FindVolId(volid)) == NULL) { 
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}
	if (volRec->properties & VOL_TEMP) {
		flags = LOGICAL_ORDER;
	} else {
		flags = NOFLAGS;
	}

    /*
     *	Call fi_CreateFile to do the dirty work.
     */
    if (fi_CreateFile(volid, &nearPid, fid, flags))	{

		return(esmFAILURE);
	}
	TRPRINT(TR_SM, TR_LEVEL_2, ("file volid:%d root page:%d unique:%x",
					fid->pid.volid, fid->pid.page, fid->unique));

	/*
	 *	return success
	 */
    return(esmNOERROR);
}
