/*
 *   $RCSfile: sm_CreateIndex.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "volume.h"
#include "file.h"
#include "bf_globals.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "scan.h"
#include "trans.h"
#include "clog.h"
#include "sm_params.h"
#include "lsn.h"
#include "loginfo.h"
#include "sm_state.h"
#include "sm_extfuncs.h"
#include "sm_intfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "bt_extfuncs.h"
#include "lh_extfuncs.h"
#include "trans_funcs.h"
#include "trans_globals.h"
#include "clog_extfuncs.h"
#include "clog_globals.h"



int sm_CreateIndex(VOLID volume, int groupIndex, SMTYPE ndxType,
						SMDATATYPE dataType, int maxKeyLen, 
						int elSize, BOOL unique, IID* ndx)
{
	INDEXDESC*	ndxDesc;
	BUFGROUP*	bufGroup = & BufGroupPtr[groupIndex];
	GROUPLINK*	gLink;

    if (CHECK_BUFGROUP_VALID(bufGroup))  {
        SM_ERROR(TYPE_USER, esmBADGROUP);
        return esmFAILURE;
    }

	INIT_SMSTATE(&SM_State);
	
    if (checkTransState(volume))  {
        return(esmFAILURE);
    }
	{
		VOLREC *volRec;
		/* TODO: remove this once logging is completely
		 * turned off for temp volumes in the index code.
		 */
		volRec = io_FindVolId(volume); /* we know it's not null because
			checkTransState() called this successfully */
		if(volRec->properties & VOL_TEMP) {
			SM_ERROR(TYPE_USER, esmNOTIMPLEMENTED);
			return esmFAILURE;
		}
	}

	if (elSize <= 0 || elSize > SM_MAXELEMLEN || elSize % 4)  {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return esmFAILURE;
		}
		
	if (maxKeyLen > SM_MAXKEYLEN)	{
		SM_ERROR(TYPE_USER, esmMAXKEYLENEXCEEDED);
		return esmFAILURE;
		}

    if (io_AllocPages(volume, INDEXDESC_PAGE2SIZE, 1, NULL, 
												& ndx->pid, PAGE_INDEXDESC))  {
        return esmFAILURE;
		}
		
	if ((gLink = bf_FakeReadPage(bufGroup, & ndx->pid, H_INDEXDESC, 
								INDEXDESC_PAGE2SIZE)) == NULL)  {
		return esmFAILURE;
		}
	
	((INDEXDESCPAGE*) gLink->bufFrame)->magic = NDXMAGIC;
	((INDEXDESCPAGE*) gLink->bufFrame)->lrc = *(GET_LOGINFO->initLRC);
	ndxDesc = &(((INDEXDESCPAGE*) gLink->bufFrame)->desc);

	/*
	 *  Initialize the root page
	 */
	switch (ndxDesc->ndxType = ndxType)  {

	  	case SM_BTREENDX:
			if (maxKeyLen > BT_MAXKEYLEN)  {
				SM_ERROR(TYPE_USER, esmMAXKEYLENEXCEEDED);
				return esmFAILURE;
				}
			if (bt_CreateBtree(TransRec.clientTid, volume, & BufGroupPtr[groupIndex], 
											ndxDesc->rootPid, dataType)) {
				return esmFAILURE;
				}
			ndxDesc->maxKeyLen = maxKeyLen;
			break;

	  	case SM_HASHNDX:
            if (maxKeyLen > BT_MAXKEYLEN)  {
				SM_ERROR(TYPE_USER, esmMAXKEYLENEXCEEDED);
				return esmFAILURE;
				}
			if (lh_CreateIndex(TransRec.clientTid, volume, & BufGroupPtr[groupIndex], 
				ndxDesc->rootPid, dataType)) {
				return esmFAILURE;
				}
			ndxDesc->maxKeyLen = maxKeyLen;
			break;

	  	default:
			printf("Illegal index choice!");
			exit(1);
		}

	ndxDesc->dataType = dataType;
	ndxDesc->maxKeyLen = maxKeyLen;
	ndxDesc->elSize = elSize;
	ndxDesc->unique = (unique) ? 1 : 0;
	
	clog_LogCreateIndex(& ndx->pid, gLink->pageHash);
	
	bf_UnfixPage(gLink, BF_LRU, TRUE);
	
	return esmNOERROR;
}


