/*
 *   $RCSfile: sm_DestroyIndex.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "volume.h"
#include "file.h"
#include "bf_globals.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "scan.h"
#include "sm_extfuncs.h"
#include "bf_macro.h"
#include "trans.h"
#include "sm_state.h"
#include "lm_extfuncs.h"
#include "bt_extfuncs.h"
#include "lh_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "sm_extfuncs.h"
#include "sm_intfuncs.h"
#include "trans_funcs.h"
#include "trans_globals.h"


int sm_DestroyIndex(IID* ndx, int groupIndex)
{
	INDEXDESC* ndxDesc;
	BUFGROUP* bufGroup = & BufGroupPtr[groupIndex];
	GROUPLINK* gLink;

    if (CHECK_BUFGROUP_VALID(bufGroup))  {
        SM_ERROR(TYPE_USER, esmBADGROUP);
        return esmFAILURE;
    }

    INIT_SMSTATE(&SM_State);

    int loadNdx;
    if (sm_FindIndexLoad(ndx, &loadNdx))  {
        return esmFAILURE;
        }
    if (loadNdx >= 0)  {
        SM_ERROR(TYPE_USER, esmBULKLOADING);
        return esmFAILURE;
        }


    //  check to see if a transaction is active
    if (checkTransState(ndx->pid.volid))  {
        return(esmFAILURE);
    }

	//
	//	Read and lock index descriptor page in EX mode
	//
	if ((gLink = bf_ReadPage(bufGroup, &ndx->pid, H_INDEXDESC, 
								INDEXDESC_PAGE2SIZE, EX)) == NULL)  {
		return esmFAILURE;
		}
	
	ndxDesc = & (((INDEXDESCPAGE*) gLink->bufFrame)->desc);
	
	SM_ASSERT(LEVEL_3, ((INDEXDESCPAGE*) gLink->bufFrame)->magic == NDXMAGIC);

	switch (ndxDesc->ndxType)  {
		case SM_BTREENDX:
			if (bt_DestroyBtree(TransRec.clientTid, ndxDesc->rootPid,  bufGroup))  {
				return esmFAILURE;
				}
			break;

		case SM_HASHNDX:
            if (lh_DestroyIndex(TransRec.clientTid, ndxDesc->rootPid,  bufGroup))  {
				return esmFAILURE;
				}
			break;

		default:
			printf("Illegal index choice\n");
			exit(1);
		}
	
	bf_DiscardPage(bufGroup, gLink);
	
	return io_DeallocPages(INDEXDESC_PAGE2SIZE, 1, &ndx->pid, FALSE, PAGE_INDEXDESC);
}

