/*
 *   $RCSfile: sm_FetchNext.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "volume.h"
#include "load.h"
#include "scan.h"
#include "trans.h"
#include "bf_globals.h"
#include "bf_extfuncs.h"
#include "sm_params.h"
#include "btree.h"
#include "sm_intfuncs.h"
#include "bt_extfuncs.h"
#include "lh_extfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"
#include "trans_globals.h"



int sm_FetchNext(SMCURSOR* cursor, KEY* retKey, void* retElem, BOOL* eof)
{
	void* 		valuePtr = 0;
	TWO* 		lenPtr = 0;
	BUFGROUP*	bufGroup = & BufGroupPtr[cursor->groupIndex];
	GROUPLINK*	gLink;
	BT_CURSOR*	btCursor = & (cursor->opaque.btCursor);

    if (CHECK_BUFGROUP_VALID(bufGroup))  {
        SM_ERROR(TYPE_USER, esmBADGROUP);
        return esmFAILURE;
    }

    // check to see if a transaction is active
    if (checkTransState(cursor->ndx.pid.volid))  {
        return(esmFAILURE);
    }
	
	if (*eof = cursor->eof)  {
		SM_ASSERT(LEVEL_3, ! btCursor->valid);
		return esmNOERROR;
		}

	if ((gLink = bf_ReadPage(bufGroup, & cursor->ndx.pid, H_INDEXDESC, 
										INDEXDESC_PAGE2SIZE, SH)) == NULL) 
		return esmFAILURE;
	
	SM_ASSERT(LEVEL_3, ((INDEXDESCPAGE*) gLink->bufFrame)->magic == NDXMAGIC);
	
	INDEXDESC* np = & (((INDEXDESCPAGE*) gLink->bufFrame)->desc);
	
	if (retKey)  {
		valuePtr = retKey->valuePtr;
		lenPtr = & (retKey->length);
		}
	
	switch (np->ndxType)  {
		case SM_BTREENDX:
			if (bt_FetchNext(*btCursor, bufGroup, 
							cursor->b2len ? cursor->b2 : 0, cursor->b2len,
							cursor->cond2, SMCOMPFUNC[np->dataType], 
							valuePtr, lenPtr, retElem, cursor->eof)) {
				bf_UnfixPage(gLink, BF_LRU, FALSE);
				return esmFAILURE;
				}
			break;

		case SM_HASHNDX:
			if (lh_FetchNext(*btCursor, bufGroup,
							cursor->b2len ? cursor->b2 : 0, cursor->b2len,
							cursor->cond2, SMCOMPFUNC[np->dataType],
							valuePtr, lenPtr, retElem)) {
				bf_UnfixPage(gLink, BF_LRU, FALSE);
				return esmFAILURE;
				}
			break;

		default:
			SM_ERROR(TYPE_USER, esmWRONGNDXTYPE);
		}
	
	bf_UnfixPage(gLink, BF_LRU, FALSE);

	//
	//	Check if cursor was previously at EOF (applies to btrees only)
	//
	if (*eof = cursor->eof)  {
		return esmNOERROR;
	}
	
	//
	//	Check if cursor is now at EOF
	//
	if ( ! btCursor->valid) {
		cursor->eof = TRUE;
		}

	*eof = FALSE;
	
	return esmNOERROR;
}

