/*
 * $RCSfile: sm_GetClientOption.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:29 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "sm_macro.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "volume.h"
#include "lock.h"
#include "msgdefs.h"
#include "option.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trace_funcs.h"
#include "bf_globals.h"
#include "clog_globals.h"
#include "sm_globals.h"
#include "trans_globals.h"

#ifndef __cplusplus
extern char*	getenv(char*);
#endif

/*
 *	ClientOptions is defined here since space.c can be compiled with C
 */
extern OptionList*	ClientOptions;


/*
 * FUNCTION DESCRIPTION:
 *
 * This function is used to get the value of a client option.
 * The optionValue argument points to a value for the option.
 *
 */

static char *
lockmode2str(LOCKMODE lockmode)
{
	switch(lockmode) {
	case NL: return "NL";
	case IS: return "IS";
	case IX: return "IX";
	case SH: return "SH";
	case SIX: return "SIX";
	case EX: return "EX";
	default:
		SM_ERROR(TYPE_USER, esmBADOPTIONVALUE);
		return NULL;
	}
}

 int
sm_GetClientOption (
	char		*optionName,	/* name of the option 		*/
	void		*optionValue 	/* value for the option 	*/
)
{
	OptionInfo*	opt;

	if (ClientOptions == NULL) {
		if (SM_InitClientOptions() != esmNOERROR) {
			SM_ERROR(TYPE_USER, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}

	if ( (optionName == NULL) || (optionValue == NULL) ) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}

	if (ClientOptions->lookup(optionName, opt) != OPTERR_Success) {
		SM_ERROR(TYPE_USER, esmBADOPTION);
		return(esmFAILURE);
	}

	switch (opt->ID()) {
	case SM_OPT_locktimeout:
		*(int*)optionValue = DefaultLockTimeout;
		break;
	case SM_OPT_bufpages:
		*(int*)optionValue = NumBuffers;
		break;
	case SM_OPT_groups: 
		*(int*)optionValue = NumBufGroups;
		break;
	case SM_OPT_userdesc:
		*(int*)optionValue = NumUserDesc;
		break;
	case SM_OPT_traceflags:
		*(TRACEFLAGS*)optionValue = TraceFlags;
		break;
	case SM_OPT_lognewpages:
		*(BOOL*)optionValue = LogNewPages;
		break;
	case SM_OPT_deallocpages:
		*(BOOL*)optionValue = DeallocSmallObjPages;
		break;
	case SM_OPT_mount:
		*(char **)optionValue = opt->value();
		break;
	case SM_OPT_filelock:
		*(char **)optionValue = opt->value();
		SM_ASSERT(LEVEL_1,
			(strcmp(lockmode2str(sm_DefaultFileLock), opt->value())==0));
		break;

	case SM_OPT_pagelock:
		SM_ASSERT(LEVEL_1,
			(strcmp(lockmode2str(sm_DefaultPageLock), opt->value())==0));
		*(char **)optionValue = opt->value();
		break;

	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		break;
	}

	/*
	 *	return success
	 */
    return(esmNOERROR);
}
