/*
 *   $RCSfile: sm_LoadScan.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "ess.h"
#include "sysdefs.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "file.h"
#include "clog.h"
#include "logaction.h"
#include "loginfo.h"
#include "logrecs.h"
#include "sm_state.h"
#include "sm_params.h"
#include "btree.h"
#include "bf_extfuncs.h"
#include "clog_extfuncs.h"
#include "fi_extfuncs.h"
#include "io_extfuncs.h"
#include "lg_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"
#include "bf_globals.h"
#include "clog_globals.h"
#include "sm_globals.h"
#include "trans_globals.h"
#include "fi_macro.h"


LOADDESC		*LoadTable = NULL;
SCANDESC		*ScanTable = NULL;


 int
SM_ReleaseScanObject (
	SCANDESC		*scanDesc 
)
{
	TRACE(TR_SM, TR_LEVEL_1);

	if(sm_ReleaseObject(scanDesc->currentUser)) {
		return esmFAILURE;
	}
	scanDesc->currentUser = NULL;
	return esmNOERROR;
}

 int
sm_CloseAllLoadsAndScans ()
{
	SCANDESC 	*scanDesc;
	SCANDESC 	*nextScanDesc;
	LOADDESC 	*loadDesc;
	LOADDESC 	*nextLoadDesc;
	
	TRACE(TR_SM, TR_LEVEL_1);

	/*
	 *	Free all scan descriptors
	 */
	scanDesc = (SCANDESC*) FIRST_LIST_ELEMENT(&UsedScanDesc);
	while (scanDesc != NULL) {

		CHECK_SCANDESC_MAGIC(scanDesc);

		/*
		 *	if the scan descriptor has a valid user
		 *	descriptor we must free it
		 */
		if (scanDesc->currentUser != NULL)	{

			SM_ReleaseScanObject(scanDesc);
		}

		/*
		 *	release the last page
		 */
		if (scanDesc->currentLink != NULL)	{
			bf_UnfixPage(scanDesc->currentLink, BF_DEFAULT, FALSE);
		}

		TRPRINT(TR_SM, TR_LEVEL_1, ("group 0x%x fixed %d unfixed %d",
			scanDesc->bufGroup,
			scanDesc->bufGroup->fixedCharges,
			scanDesc->bufGroup->unfixedCharges));
		/*
		 *	close the buffer group
		 */
		bf_CloseGroup(scanDesc->bufGroup);

		if(scanDesc->pidListBegin)  {
			SM_FREE_PTR(scanDesc->pidListBegin);
			scanDesc->pidListBegin = NULL;
		}

		nextScanDesc = (SCANDESC*) NEXT_LIST_ELEMENT( &(scanDesc->scanList) );
		/*
		 *	free up the scan descriptor
		 */
		listRemove( &(scanDesc->scanList) );
		listEnq( &ScanDescPool, &(scanDesc->scanList) );

		scanDesc = nextScanDesc;
	}

	/*
	 *	Free all load descriptors
	 */
	loadDesc = (LOADDESC*) FIRST_LIST_ELEMENT(&UsedLoadDesc);
	while (loadDesc != NULL) {

		CHECK_LOADDESC_MAGIC(loadDesc);

		/*
		 *	release the last page
		 */
		if (loadDesc->currentLink != NULL)	{
			bf_UnfixPage(loadDesc->currentLink, BF_DEFAULT, FALSE);
		}

		/*
		 *	close the buffer group
		 */
		bf_CloseGroup(loadDesc->bufGroup);

		nextLoadDesc = (LOADDESC*) NEXT_LIST_ELEMENT( &(loadDesc->loadList) );
		/*
		 *	free up the load descriptor
		 */
		listRemove( &(loadDesc->loadList) );
		listEnq( &LoadDescPool, &(loadDesc->loadList) );

		loadDesc = nextLoadDesc;
	}

	return(esmNOERROR);
}




 int
SM_InitLoadDesc ()
{
	register int		i;

	TRACE(TR_SM, TR_LEVEL_1);

	GET(LoadTable, LOADDESC, NumLoadDesc);

	initializeList( &LoadDescPool );
	initializeList( &UsedLoadDesc );

	for (i = 0; i < NumLoadDesc; i++)	{
		initializeListElement( &(LoadTable[i].loadList), 
			(char *) &(LoadTable[i]) );
		listEnq( &LoadDescPool, &(LoadTable[i].loadList) );
		INIT_LOADDESC_MAGIC( &(LoadTable[i]) );
	}
	return esmNOERROR;
}


 int
SM_LoadNextObject (

	LOADDESC			*loadDesc,
	int					length,
	char				*data,
	OID					*oid 
)
{
    register SLOTINDEX	currentSlot;
	register PAGESLOT	*slotPtr;
    register SMALLOBJ	*smallObject;
    SLOTTEDPAGE			*currentPage;
	UNIQUE				unique;
	GROUPLINK			*currentLink;	
    PID					currentPid;
	PID					rootPid;
	BOOL				creatingLargeObject;
	BOOL				rootObject;
	int					spaceNeeded;
	int					alignLength;
    OPERATIONINFO		opInfo;


	TRPRINT(TR_SM, TR_LEVEL_1, ("length:%d", length));

	CHECK_LOADDESC_MAGIC(loadDesc);

	/* setup register pointers */
	currentPage = loadDesc->currentPage;
	currentLink = loadDesc->currentLink;
	currentPid  = loadDesc->currentPid;
	currentSlot = loadDesc->currentSlot;

	/* round the size up to the nearest alignment value */
	alignLength = ALIGNSIZE(length);
	TRPRINT(TR_SM, TR_LEVEL_2, ("alignLength:%d", alignLength));

    /*
     *	Are we creating a large or small object?
     *	In calculating how much space we'll need, waste a little space by 
     *	assuming that we'll need to allocate a new slot; this will save 
     *	time in the calculations below and isn't much of a price to pay.
     */
    if (alignLength > LARGETHRESHOLD) {

		/*
		 *	mark the fact that we are creating a large object
		 */
		creatingLargeObject = TRUE;

		/*
		 *	Check to see if the object will fit in a one level tree
		 */
		if (length <= (LSLOTTHRESHOLD))	{

			/*
			 *	allocate space for the header and the root
			 */
			spaceNeeded = ALIGNSIZE(sizeof(LARGEOBJROOT)) + sizeof(PAGESLOT);
			rootObject = TRUE;
			TRPRINT(TR_SM, TR_LEVEL_2, ("creating slotted large"));

		} else {

			/*
			 *	allocate enough space to hold the header only
			 */
			spaceNeeded = sizeof(LARGEOBJHDR) + sizeof(PAGESLOT);
			rootObject = FALSE;
			TRPRINT(TR_SM, TR_LEVEL_2, ("creating large object on seperate"));
		}

    } else {

		/*
		 *	we are creating a small object so calculate appropriate space
		 */
		creatingLargeObject = FALSE;
		spaceNeeded = sizeof(OBJHDR) + alignLength + sizeof(PAGESLOT);
		TRPRINT(TR_SM, TR_LEVEL_2, ("creating small object"));
    }
	TRPRINT(TR_SM, TR_LEVEL_2, ("spaced needed:%d", spaceNeeded));

	/*
	 *	Check to see if there is enough room on the page
	 */
	if (CONTIGSPACE(currentPage) < spaceNeeded) {

		PID		newPid;

		/*
		 *	allocate a page in the file
		 */
		if (io_AllocSlottedPages(1, &currentPid, &newPid, &(loadDesc->fid),
								 creatingLargeObject, &unique, loadDesc->bufGroup))	{

			return esmFAILURE;
		}

		/* assign the current pid */
		currentPid = newPid;

		/* unfix the current page */
		bf_UnfixPage(currentLink, BF_DEFAULT, TRUE);

		/*
		 * Get a buffer for the page.
		 */
		if ((currentLink = bf_FakeReadPage(loadDesc->bufGroup, &currentPid, H_SLOTTED,
								      SLOTTED_PAGE2SIZE)) == NULL)	{
			return esmFAILURE;
		}

		/* get a pointer to the page */
		currentPage = (SLOTTEDPAGE *) currentLink->bufFrame;

		/* Initialize the page.  */
		INITPAGE(currentPage, currentPid, loadDesc->fid, FALSE, unique);
		currentPage->header.lrc = *(GET_LOGINFO->initLRC);

		/*
		 *	initialize the load descriptor
		 */
		loadDesc->currentPage = currentPage;
		loadDesc->currentLink = currentLink;
		loadDesc->currentPid  = currentPid;
		loadDesc->currentSlot = 0;
	}

	if (SM_SetLogServerState(currentLink, NULL) != esmNOERROR) {
		return esmFAILURE;
	}

	/* get a pointer to the next slot */
	currentSlot = loadDesc->currentSlot;
	slotPtr = GETSLOTPTR(currentPage, currentSlot);

	/*
	 *	Check to see if we are creating a large object
	 */
	if (creatingLargeObject == TRUE)	{

		/*
		 *	Check to see if this is the first large object
		 *	that resides on this page
		 */
		if ((currentPage->header.largeCount == 0) && (currentSlot > 0))	{

			/* mark the bitmap also */
			if (fi_MarkPage( &(loadDesc->fid), currentPid.page ))	{
				return esmFAILURE;
			}
		}

		/* increment the number of large obj's on the page */
		currentPage->header.largeCount++;
	}

	/* increment the number of slots in use */
	currentPage->header.slotCount++;

	/*
	 *	check to see if we can use the unique on the page
	 */
	if (currentPage->header.uniqueCount == 0)	{

		/* get a new unique range */
		if (io_GetUnique(&currentPid, &unique, UNIQUE_ALLOC_COUNT))	{
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/* set the unique fields */
		currentPage->header.unique = unique;
		currentPage->header.uniqueCount = UNIQUE_ALLOC_COUNT;
	}

    /*
     * Now create the appropriate type of object.
     */
     if (creatingLargeObject == TRUE) {

		LARGEOBJHDR		*largeObject;

		/*
		 *	fill in the root page in case we have the 
		 *	large object header on the slotted page
		 */
		rootPid = currentPid;

		/* Update the current page info.  */
		slotPtr->offset = currentPage->header.freeStart;

		/*
		 * Creating a large object.
		 * Large object create will fill in the rootPid.
		 */
		if (lg_CreateObject( &(loadDesc->fid), &currentPid,
							 loadDesc->bufGroup, length, data,
							 &rootPid, (rootObject ? currentSlot : NIL)))	{

			return esmFAILURE;
		}

		/* Store the header on the current page.  */
		largeObject = (LARGEOBJHDR *) ((char *) currentPage + currentPage->header.freeStart);
		largeObject->objectHdr.size	 = length;


#if OBJECT_MAGIC IS_ENABLED
		largeObject->objectHdr.magic = OBJECT_HDR_MAGIC;
#endif OBJECT_MAGIC IS_ENABLED

		/* fill in the unique field */
		slotPtr->unique = currentPage->header.unique++;
		currentPage->header.uniqueCount--;
		TRPRINT(TR_SM, TR_LEVEL_2, ("unique:%x", slotPtr->unique));

		/*
		 *	Check to see if we created a root object
		 */
		if (rootObject)	{

			/* set the properties */
			largeObject->objectHdr.properties = P_LARGEOBJ|P_HEADERONPAGE;

			/* update the page usage statistics */
			currentPage->header.freeStart += ALIGNSIZE(sizeof(LARGEOBJROOT));
			currentPage->header.freeBytes -= ALIGNSIZE(sizeof(LARGEOBJROOT));

            /*get size of and ptr to the data part of the root - for logging */
            opInfo.size = sizeof(LARGEOBJROOT) - sizeof(OBJHDR);
		    opInfo.newData = (char *) &(largeObject->rootPid);

		} else {
			
			/* set the properties */
			largeObject->objectHdr.properties = P_LARGEOBJ;

			/* update the page usage statistics */
			currentPage->header.freeStart += sizeof(LARGEOBJHDR);
			currentPage->header.freeBytes -= sizeof(LARGEOBJHDR);

            /*get size of and ptr to the data part of the header - for logging*/
            opInfo.size = sizeof(LARGEOBJHDR) - sizeof(OBJHDR);
		    opInfo.newData = (char *) &(largeObject->rootPid);

			/* record the root pid */
			largeObject->rootPid = rootPid.page;

		}

		/*
		 *	if we are allocating a slot then subtract that space
		 */
		currentPage->header.freeBytes -= sizeof(PAGESLOT);

		/*
		 *	fill in the unique field
		 */
		slotPtr->unique = currentPage->header.unique++;
		currentPage->header.uniqueCount--;
		TRPRINT(TR_SM, TR_LEVEL_2, ("unique:%x", slotPtr->unique));

    } else {

		/*
		 * If we got here, we're creating a small object.
		 * Copy the object to the current page.
		 */
		smallObject = (SMALLOBJ *) (((char *) currentPage) + currentPage->header.freeStart);
		smallObject->objectHdr.properties = 0;
		smallObject->objectHdr.size		  = length;
		
		/* get the data size and pointer for logging purposes */
		opInfo.size = length;
		opInfo.newData = (char *) smallObject->data;

		TRPRINT(TR_SM, TR_LEVEL_2, ("freestart:%d length:%d",
				currentPage->header.freeStart, length));

#if OBJECT_MAGIC IS_ENABLED
		smallObject->objectHdr.magic = OBJECT_HDR_MAGIC;
#endif OBJECT_MAGIC IS_ENABLED

		/*
		 *	Only copy the data in if the pointer is valid
		 */
		if (data != NULL)	{
			bcopy(data, smallObject->data, (int) length);
		} else {
			bzero(smallObject->data, (int) length);
		}

		/*
		 *	Initialize the contents of the slot
		 */
		slotPtr->offset = currentPage->header.freeStart;

		/*
		 *	fill in the unique field
		 */
		slotPtr->unique = currentPage->header.unique++;
		currentPage->header.uniqueCount--;
		TRPRINT(TR_SM, TR_LEVEL_2, ("unique:%x", slotPtr->unique));

		/*
		 *	Update the current page stats to account for the
		 *	space allocated for the object
		 */
		currentPage->header.freeStart += sizeof(OBJHDR) + alignLength;
		currentPage->header.freeBytes -= sizeof(OBJHDR) + alignLength;

		/*
		 *	Update free space if we allocated a new slot
		 */
		currentPage->header.freeBytes -= sizeof(PAGESLOT);
		TRPRINT(TR_SM, TR_LEVEL_2, ("freeStart:%d freeBytes:%d",
				currentPage->header.freeStart, currentPage->header.freeBytes));
	}

	/* store the oid for logging purposes */
	opInfo.oid.diskAddr.page	 = currentPid.page;
	opInfo.oid.diskAddr.slot	 = currentSlot;
	opInfo.oid.diskAddr.unique = slotPtr->unique;
	opInfo.oid.diskAddr.volid  = currentPid.volid;

	/*
	 *	check to see if an oid is returned
	 */
	if (oid != NULL)	{

		/* Set the OID, dirty the home page, unfix it and return.  */
		*oid = opInfo.oid ;
		
		TRPRINT(TR_SM, TR_LEVEL_2, ("returning oid pid:%d slot:%d unique:%x",
				oid->diskAddr.page, currentSlot, slotPtr->unique));
	} 

	if (GET_LOGLEVEL(LOG_SPACE)) {

		opInfo.flags = 0;
		opInfo.start = 0;
		opInfo.oldData = NULL;
		opInfo.actionPid.volid = oid->diskAddr.volid;
		opInfo.actionPid.page  = oid->diskAddr.page;

		/*
		 * data logging is required if the it is turned on or
		 * if the object is large.
		 */
		if ( (GET_LOGLEVEL(LOG_DATA) && (data != NULL))  ||
				creatingLargeObject) {

			/* log the create of the small portion of the object*/
			/* note: oid, size and newData fields are set above */

			opInfo.action = LOG_ACTION_CREATE_SMALL_OBJ;
			clog_LogCreateObj(&opInfo);

		} else {

			/* log the create of the small header portion of object*/
			/* note: oid, size and newData fields are set above */
			opInfo.action = LOG_ACTION_CREATE_SMALL_OBJ_ND;
			clog_LogCreateObjND(&opInfo);
		}
	}

	loadDesc->currentSlot++;
	bf_DirtyPage(currentLink->pageHash);

	return(esmNOERROR);
}

 int
SM_OpenLoad (
	FID					*fid,
	LOADDESC			*loadDesc 
)
{

    SLOTTEDPAGE			*firstPage;
	GROUPLINK			*firstLink;	
	BUFGROUP			*bufGroup;
    PID					firstPid;
	UNIQUE				unique;
	FILEINFO            *fileInfo;
	int					err;

	TRACE(TR_SM, TR_LEVEL_1);

	/*
	 *	get pointer to buffer group
	 */
	bufGroup = loadDesc->bufGroup;

    /*
     *  Determine the type for the scan, ignore the type passed in
     */
    if ((fileInfo = fi_FindFileInfo(fid, TRUE)) == NULL) {
        return(esmFAILURE);
    }
    if (fileInfo->tempVolume) {
		loadDesc->type        = LOGICAL_LOAD;
    } else {
		loadDesc->type        = PHYSICAL_LOAD;
    }

	/*
	 *	attempt to get the last logical page in the file
	 */
	switch (loadDesc->type) {
	case LOGICAL_LOAD:
		err = fi_LastLogicalPage(fid, &firstPid);
		break;
	case PHYSICAL_LOAD:
		err = fi_LastPhysicalPage(fid, &firstPid);
		break;
	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return(esmFAILURE);
	}
	if (err)	{

		/*
		 *	check the error code
		 */
		if (sm_errno == esmEMPTYFILE)	{	

			TRPRINT(TR_SM, TR_LEVEL_2, ("empty file"));

			/*
			 *	allocate the first page in the file
			 */
			if (io_AllocSlottedPages(1, &(fid->pid), &firstPid, fid, FALSE, &unique, bufGroup))	{

				return esmFAILURE;
			}

			/*
			 * Get a buffer for the page.
			 */
			if ((firstLink = bf_FakeReadPage(bufGroup, &firstPid,
											 H_SLOTTED, SLOTTED_PAGE2SIZE)) == NULL)	{

				return esmFAILURE;
			}

			/*
			 *	get a pointer to the page
			 */
			firstPage = (SLOTTEDPAGE *) firstLink->bufFrame;

			/*
			 * Initialize the page.
			 */
			INITPAGE(firstPage, firstPid, *fid, FALSE, unique);
			firstPage->header.lrc = *(GET_LOGINFO->initLRC);

			/*
			 *	Get a pointer to the slotted page
			 */
			firstPage = (SLOTTEDPAGE *) firstLink->bufFrame;

			/*
			 *	set the pointer to the current slot
			 */
			loadDesc->currentSlot = 0;

			/*
			 *	dirty the page
			 */
			bf_DirtyPage(firstLink->pageHash);

		} else {
			return esmFAILURE;
		}

	} else {

		/*
		 *	read in the page
		 */
		if ((firstLink = bf_ReadPage(bufGroup, &firstPid, H_SLOTTED,
									 SLOTTED_PAGE2SIZE, NL)) == NULL)	{
			return esmFAILURE;
		}

		/*
		 *	Get a pointer to the slotted page
		 */
		firstPage = (SLOTTEDPAGE *) firstLink->bufFrame;

		/*
		 *	check to make sure that we have no stray free slots
		 */
		SM_ASSERT(LEVEL_3, firstPage->header.freeSlot != NIL);

		/*
		 *	point to the first free slot
		 */
		loadDesc->currentSlot = firstPage->header.slotCount;
	}
	TRPRINT(TR_SM, TR_LEVEL_2, ("currentSlot:%d", loadDesc->currentSlot));

	/*
	 *	initialize the load descriptor
	 */
	loadDesc->bufGroup    = bufGroup;
	loadDesc->currentPage = firstPage;
	loadDesc->currentLink = firstLink;
	loadDesc->flags		  = LOAD_BEGIN;
	loadDesc->fid		  = *fid;
	loadDesc->currentPid  = firstPid;

	/*
	 *	return the load descriptor
	 */
	return(esmNOERROR);
}



 int
SM_InitScanDesc ()
{
	int		i;

	TRACE(TR_SM, TR_LEVEL_1);

	GET(ScanTable, SCANDESC, NumScanDesc);

	initializeList( &ScanDescPool );
	initializeList( &UsedScanDesc );

	for (i = 0; i < NumScanDesc; i++)	{
		initializeListElement( &(ScanTable[i].scanList), 
			(char *) &(ScanTable[i]) );
		listEnq( &ScanDescPool, &(ScanTable[i].scanList) );
		INIT_SCANDESC_MAGIC( &(ScanTable[i]) );
	}
	return esmNOERROR;
}

 int
SM_OpenScan (
	FID					*fid,
	SCANDESC			*scanDesc,
	OID					*oid,
	SCANTYPE			scanType /* ignored */
)
{
    SLOTTEDPAGE			*firstPage;			/* home page ptr				*/
	GROUPLINK			*firstLink = NULL;	
	PID					firstPid;
	FILEINFO			*fileInfo;


	TRACE(TR_SM, TR_LEVEL_1);

	scanDesc->pidListBegin = scanDesc->pidListEnd = 
		scanDesc->pidListCurrent = scanDesc->nextPid = NULL;

	/*
	 *	Determine the type for the scan, ignore the type passed in
	 */
	if ((fileInfo = fi_FindFileInfo(fid, TRUE)) == NULL) {
		FAIL
	}
	if (fileInfo->tempVolume) {
		scanType = LOGICAL_SCAN;
	} else {
		scanType = PHYSICAL_SCAN;
	}

	if( scanType == LOGICAL_SCAN ) {
		/* the the neighbor pid from the slotted page */
		scanDesc->nextPid = &(scanDesc->nextLogicalPid);
	}

	scanDesc->currentLink = NULL;

	if (oid == NULL)	{
		/*
		 *	Start the scan with the first oid in the file.
		 */
		if (fi_Page(fid, &firstPid, (FLAGS)(FI_FIRST | scanType)))	{

            if (sm_errno == esmEMPTYFILE) {

                /*
                 * file is empty so return an empty scan descriptor
                 */
                scanDesc->currentPage = NULL;
                scanDesc->currentLink = NULL;
                scanDesc->currentUser = NULL;
                scanDesc->flags       = SCAN_END;
                scanDesc->fid         = *fid;
                scanDesc->type        = PHYSICAL_SCAN;

				goto Success;

            } else {
				FAIL
            }

		}
		scanDesc->currentSlot = 0;
	} else {
		/*
		 *	Start the scan with the oid given by the user.
		 */
		firstPid.volid = oid->diskAddr.volid;
		firstPid.page  = oid->diskAddr.page;
	}

	/*
	 *	Read the first page.
	 */
	if ((firstLink = bf_ReadPage(scanDesc->bufGroup, &firstPid, 
			PAGE_SLOTTED, SLOTTED_PAGE2SIZE, SH)) == NULL)	{
		FAIL
	}
	firstPage = (SLOTTEDPAGE *) firstLink->bufFrame;

	if (CHECK_SLOTTED_MAGIC(firstPage))	{
		SM_ERROR(TYPE_FATAL, esmBADSLOTTEDMAGIC);
	}
	sm_AuditSlotPage(firstPage);

#ifdef DEBUG
	/*
	 *	check to make sure that the fids match
	 */
	if ((fid->unique != firstPage->header.fid.unique) ||
		(fid->pid.page != firstPage->header.fid.pid.page) ||
		(fid->pid.volid != firstPage->header.fid.pid.volid))	{

		/* should not happen */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
#endif DEBUG


	if(oid) {
		register SLOTINDEX	objectSlot;
		register PAGESLOT	*slotPtr;

		objectSlot = oid->diskAddr.slot;
		slotPtr = GETSLOTPTR(firstPage, objectSlot);
		/*
		 *	Check the oid for validity
		 */
		if (CHECKOID(objectSlot, firstPage, slotPtr, oid->diskAddr.unique))	{

			SM_ERROR(TYPE_USER, esmBADOID);
			FAIL
		}
		TRPRINT(TR_SM, TR_LEVEL_2, 
			("oid is valid offset is:%d", slotPtr->offset));
		scanDesc->currentSlot = oid->diskAddr.slot;
	}


	/*
	 *	initialize the scan descriptor
	 */
	scanDesc->currentPage = firstPage;
	scanDesc->currentLink = firstLink;
	scanDesc->currentUser = NULL;
	scanDesc->flags		  = SCAN_BEGIN;
	scanDesc->fid		  = *fid;
	scanDesc->currentPid  = firstPid;
	scanDesc->type        = scanType;

	if(scanDesc->type == LOGICAL_SCAN) 
		*(scanDesc->nextPid)  = firstPage->header.nextLogicalPid;
	else {
		return SM_NeighborPids(scanDesc, NEIGHBOR_AFTER);
	}
	/* first page is still fixed */

Success:

#ifdef DEBUG
	if( scanDesc->flags == SCAN_END) {
		/* SM_ASSERT(LEVEL_1, scanDesc->bufGroup->fixedCharges == 0); only true when using a new buffer group for the scan */
		SM_ASSERT(LEVEL_1, (scanDesc->currentLink == NULL));
	} else  {
		SM_ASSERT(LEVEL_1, scanDesc->bufGroup->fixedCharges > 0);
		SM_ASSERT(LEVEL_1, (scanDesc->currentLink != NULL));
	}
#endif DEBUG

	return(esmNOERROR);


FailureReturn:

	if(firstLink) {
		bf_UnfixPage(firstLink, BF_DEFAULT, FALSE);
	}
	SM_ASSERT(LEVEL_1, (scanDesc->currentLink == NULL));
	SM_ASSERT(LEVEL_1, (scanDesc->bufGroup->fixedCharges==0));
	return esmFAILURE;
}

 USERDESC
*SM_ReadScanObject (
	register SCANDESC	*scanDesc 
)
{
	OID			oid;


	TRPRINT(TR_SM, TR_LEVEL_1, ("reading object on page:%d in slot:%d",
			scanDesc->currentPid.page, scanDesc->currentSlot));

	CHECK_SCANDESC_MAGIC(scanDesc);

	SM_ASSERT(LEVEL_1,  (scanDesc->currentPage == 
		(SLOTTEDPAGE *) scanDesc->currentLink->bufFrame));

	{
		/*
		 *	construct an oid for SM_ReadObjectAux()
		 */
		PAGESLOT			*slotPtr;

		oid.diskAddr.page	= scanDesc->currentPid.page;
		oid.diskAddr.slot	= scanDesc->currentSlot;
		oid.diskAddr.volid	= scanDesc->fid.pid.volid;

	    slotPtr = GETSLOTPTR(scanDesc->currentPage, scanDesc->currentSlot);
		oid.diskAddr.unique = slotPtr->unique;
	}
	/*
	it got freed earlier 
	if(scanDesc->currentUser != NULL) {
		if(SM_InvalidateUserDesc(scanDesc->currentUser)) {
			FAIL
		}
	}
	*/

	if(SM_ReadObjectAux(scanDesc->currentLink, &scanDesc->currentPid,
			scanDesc->bufGroup, &oid, 
			scanDesc->currentStart, scanDesc->currentSize,
			&scanDesc->currentUser, scanDesc->currentUser, FALSE)) {

		/* When if fails, SM_ReadObject hasn't set 
		   the ptr scanDesc->currentUser 
		  */
		FAIL
	}
	if(scanDesc->currentUser->groupLink == scanDesc->currentLink) {
		/* The user descriptor refers to an object on this page.
		 * Inc its pin count since SM_ReadObjectAux does not do so.
		 * NB:
		 * This is a short-circuit of bf_ReadPage(),
		 * since we already have the page fixed in this buffer group.
		 */
		scanDesc->currentUser->groupLink->fixedCount++;
	}

Success:
	/* DO NOT UNFIX currentLink, even if it's the home link for a
	 * large or forwarded object.
	 * The highest level scan functions do the unfix.
	 */
	{
		int length = scanDesc->currentSize = scanDesc->currentUser->objectSize;

		if (scanDesc->currentUser->userFlags & P_LARGEOBJ && length != 0){
			scanDesc->flags = SCAN_LARGE;
		} else if(scanDesc->currentUser->userFlags & P_MOVED && length != 0){
			scanDesc->flags = SCAN_FORWARDED;
		} else {
			scanDesc->flags = 0;
		}
	}

    return(scanDesc->currentUser);

FailureReturn:
	/* scanDesc is invalid */
	scanDesc->flags = 0;
	scanDesc->currentSize = 0;
	return NULL;
}


 int
SM_ScanNext (

	SCANDESC			*scanDesc,
	int					start,
	int					length,
	USERDESC			**retDesc,
	BOOL				*eof
)
{
    register OBJECT		*objectPtr;			/* ptr to generic object		*/
	register PAGESLOT	*slotPtr;
    register SLOTINDEX	slotIndex;			/* slot index of next object	*/
    SLOTTEDPAGE			*currentPage;		/* double indirect ref to page	*/
    PID					currentPid;			/* to find next page of file	*/
	GROUPLINK			*currentLink;	



	TRPRINT(TR_SM, TR_LEVEL_1, ("start:%d, length:%d", start, length));

	CHECK_SCANDESC_MAGIC(scanDesc);

	SM_ASSERT( LEVEL_1, (scanDesc->bufGroup->fixedCharges > 0) || scanDesc->currentLink == NULL);

	currentPage = scanDesc->currentPage;
	currentLink = scanDesc->currentLink;
	scanDesc->currentLink = NULL;
	currentPid  = scanDesc->currentPid;

	slotIndex = scanDesc->currentSlot;
	slotPtr	  = GETSLOTPTR(currentPage, slotIndex);

    /*
     * Find the next valid slot in the page.
     */
    for (*eof = FALSE;;) {

		/*
		 *	check to see if we've looked at all the slots on the page
		 */
		if (currentLink == NULL || slotIndex >= currentPage->header.slotCount) {

			TRPRINT(TR_SM, TR_LEVEL_1, 
				("at end of page %d, getting page %d", currentPid.page, 
				scanDesc->nextPid==NULL?-1: *scanDesc->nextPid));

			/*
			 *	check to see if we are at the end of the chain
			 */
			if ((scanDesc->nextPid == NULL)/*phys scan*/ ||
				(*(scanDesc->nextPid)==NULLPID)/*log scan*/ ) {

				TRPRINT(TR_SM, TR_LEVEL_2, ("at end of scan"));

				/*
				 *	Signal eof to the user
				 *	set the fields of the scan descriptor
				 */
				*eof = TRUE;
				*retDesc = NULL;
				scanDesc->flags = SCAN_END;
				scanDesc->currentPage  = currentPage;
				scanDesc->currentLink  = currentLink;
				scanDesc->currentSlot  = NIL;
				scanDesc->currentPid   = currentPid;

				goto Success;
			}

#define SM_PAGEMASK 0x7ffffff
/* TODO: some day this should to into a .h file that's used
 * both by the server and client.
 * When we add storage utilization hints, it would be good
 * to export this leafpage info to the client.
 */
			currentPid.page = (*(scanDesc->nextPid)) & SM_PAGEMASK;
			currentPid.volid = scanDesc->fid.pid.volid;

			/*
			 *	unfix the current page
			 */
			if (currentLink != NULL) bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);

			SM_ASSERT(LEVEL_1, currentPid.volid != 0);
			SM_ASSERT(LEVEL_1, currentPid.page > 0);

			/*
			 * Read in the next page of the file.
			 */
			if ((currentLink = bf_ReadPage(scanDesc->bufGroup, &currentPid, 
				PAGE_SLOTTED, SLOTTED_PAGE2SIZE, SH)) == NULL)	{
				FAIL
			}

			/*
			 *	Get the pointer to the page
			 */
			currentPage = (SLOTTEDPAGE *) currentLink->bufFrame;
			sm_AuditSlotPage(currentPage);

			if( scanDesc->type == LOGICAL_SCAN ) {
				/* the the neighbor pid from the slotted page */
				*(scanDesc->nextPid)  = currentPage->header.nextLogicalPid;
			}  else {
				/* SM_NeighborPids uses currentPid */
				scanDesc->currentPid   = currentPid;

				if(SM_NeighborPids(scanDesc, NEIGHBOR_AFTER)) {
					FAIL
				}
			}
			/*
			 *	set looking at the first slot
			 */
			slotPtr	  = &(currentPage->slot[0]);
			slotIndex = 0;

		}

		/*
		 * If the slot isn't empty, then we can break out of the loop.
		 */
		if (slotPtr->offset != NIL) {

			TRPRINT(TR_SM, TR_LEVEL_2, ("found object in slot:%d", slotIndex));

			/*
			 *	get a pointer to the object to see if it is
			 *	a forwarded object which we want to skip
			 */
			objectPtr = GETOBJECTPTR(currentPage, slotPtr);

			/*
			 *	check to see if this is the forwarded portion or
			 *	a version history graph
			 */
			if (!(objectPtr->objectHdr.properties & INVISIBLE_OBJECT ))	{

				break;
			}
		}
        slotIndex++;
        slotPtr--;
    }

	SM_ASSERT(LEVEL_3, bf_CheckBitmap(&currentPid, PAGE_SLOTTED) == esmNOERROR);

	if (CHECK_SLOTTED_MAGIC(currentPage))	{
		SM_ERROR(TYPE_FATAL, esmBADSLOTTEDMAGIC);
	}

	/*
	 *	set the fields of the scan descriptor
	 */
	scanDesc->currentPage  = currentPage;
	scanDesc->currentLink  = currentLink;		/* fixed */
	scanDesc->currentPid   = currentPid;
	scanDesc->currentSlot  = slotIndex;
	scanDesc->currentStart = start;
	scanDesc->currentSize  = length;

	/*
	 *	actually read the object
	 */
	*retDesc = SM_ReadScanObject(scanDesc);

	if (*retDesc == NULL) {
	    FAIL
	} 

	scanDesc->currentSlot++;

Success:
	SM_ASSERT(LEVEL_1, (scanDesc->currentLink != NULL) || scanDesc->flags & SCAN_END);
	/* at least the currentLink is fixed, for the sake of the scan */

#ifdef DEBUG
	/* we'd like to say that *retDesc->byteCount > 0 IFF its
	 * groupLink is fixed, but the grouplink could be the scan desc's 
	 * current link and it could be fixed from 
	 * an operation other than the scan.
	 * Instead, we make a weaker check:  The grouplink is NULL if no
	 * bytes are described by the user descriptor.
	 * THe scanDesc's currentLink is treated entirely separately from
	 * the user descriptor's group link.
	 */

	/* group link is null iff byteCount is 0 */
	if(*retDesc) {
		if((*retDesc)->byteCount == 0) {
			SM_ASSERT(LEVEL_1, ((*retDesc)->groupLink == scanDesc->currentLink));
		} else {
			if(scanDesc->flags & SCAN_LARGE) {
				SM_ASSERT(LEVEL_1, ((*retDesc)->objInfo != NULL));
			} else {
				SM_ASSERT(LEVEL_1, ((*retDesc)->groupLink != NULL));
				SM_ASSERT( LEVEL_1, ((*retDesc)->groupLink->fixedCount > 0));
			}
		}
	} /* else no return user desc - must be SCAN_END */
#endif DEBUG

	/* for scanDesc's currentLink */
	SM_ASSERT( LEVEL_1, (scanDesc->bufGroup->fixedCharges > 0) || scanDesc->flags & SCAN_END);


	TRPRINT(TR_SM, TR_LEVEL_2, 
		("fixed charges", scanDesc->bufGroup->fixedCharges));

	return(esmNOERROR);


FailureReturn:

	if(currentLink != NULL) {
		bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);
	}
	return esmFAILURE;
}




 int
sm_CloseLoad (

	LOADDESC			*loadDesc 
)
{

	TRACE(TR_SM, TR_LEVEL_1);

	INIT_SMSTATE(&SM_State);

	if (checkTransState(loadDesc->fid.pid.volid))	{
		return esmFAILURE;
    }
	if (loadDesc->currentLink != NULL)	{
		bf_UnfixPage(loadDesc->currentLink, BF_DEFAULT, FALSE);
	}
	bf_CloseGroup(loadDesc->bufGroup);

	listRemove( &(loadDesc->loadList) );
	listEnq( &LoadDescPool, &(loadDesc->loadList) );

	return esmNOERROR;
}




 int
sm_LoadNextObject (

	LOADDESC		*loadDesc,
	int				length,
	void			*data,
	OID				*oid 
)
{

	TRPRINT(TR_SM, TR_LEVEL_1, ("length:%d", length));

	INIT_SMSTATE(&SM_State);

	if (checkTransState(loadDesc->fid.pid.volid))	{
		return esmFAILURE;
	}
	CHECK_LOADDESC_MAGIC(loadDesc);

	if (SM_LoadNextObject(loadDesc, length, (char*)data, oid))	{
		return esmFAILURE;
	} else {
		return esmNOERROR;
	}
}


 int
sm_OpenLoad (

	FID					*fid,
	int					groupSize,
	float				fillFactor,		/* unused at this time */
	LOADDESC			**retDesc 
)
{

	LOADDESC			*loadDesc = NULL;

	TRPRINT(TR_SM, TR_LEVEL_1, ("fid:%d fillFactor:%f", fid->pid.page,
			fillFactor));

	if (checkTransState(fid->pid.volid))	{
		return esmFAILURE;
	}

	if (io_CheckBitmap( &(fid->pid), PAGE_FILEHDR ))	{
		return esmFAILURE;
	}

	if ((loadDesc = (LOADDESC *) listDeq( &LoadDescPool )) == NULL)	{
		SM_ERROR(TYPE_WARNING, esmNOFREELOADDESC);
		FAIL;
	}

	if ((loadDesc->bufGroup = bf_OpenGroup(groupSize, BF_LRU, NOFLAGS)) == NULL)	{
		FAIL;
	}

	if (SM_OpenLoad (fid, loadDesc) != esmNOERROR) {
		FAIL;
	}

	/*
	 *	return the load descriptor
	 */
	listEnq( &UsedLoadDesc, &(loadDesc->loadList) );
	*retDesc = loadDesc;
	return esmNOERROR;

FailureReturn:
	if (loadDesc != NULL) {
		listEnq( &LoadDescPool, &(loadDesc->loadList) );

		if (loadDesc->bufGroup != NULL)  {
			bf_CloseGroup(loadDesc->bufGroup);
		}
	}
	return esmFAILURE;

}




 int
sm_CloseScan (

	SCANDESC			*scanDesc 
)
{

	TRACE(TR_SM, TR_LEVEL_1);

	/*
	INIT_SMSTATE(&SM_State);  index bulk load needs SM_State
	*/

    if (checkTransState(scanDesc->fid.pid.volid))  {
		return esmFAILURE;
    }

	/*
	 *	if the scan descriptor has a valid user
	 *	descriptor we must free it
	 */
	if (scanDesc->currentUser != NULL)	{
		SM_ReleaseScanObject(scanDesc);
	}

	/*
	 *	release the last page
	 */
	if (scanDesc->currentLink != NULL)	{
		bf_UnfixPage(scanDesc->currentLink, BF_DEFAULT, FALSE);
	}

	if (scanDesc->removeGroup) {
		/*
		 *	close the buffer group
		 */
		bf_CloseGroup(scanDesc->bufGroup);
	} else {
		TRPRINT(TR_SM, TR_LEVEL_1, ("not closing buf group"));
	}

	if(scanDesc->pidListBegin) {
		SM_FREE_PTR(scanDesc->pidListBegin);
		scanDesc->pidListBegin = NULL;
	}

	/*
	 *	free up the scan descriptor
	 */
	listRemove( &(scanDesc->scanList) );
	listEnq( &ScanDescPool, &(scanDesc->scanList) );

	/*
	 *	return success
	 */
	return(esmNOERROR);
}




 int
sm_OpenScan (

	FID					*fid,
	int					type,
	int					groupSize,
	SCANDESC			**retDesc,
	OID					*oid 
)
{

	BUFGROUP			*bufGroup;

	TRPRINT(TR_SM, TR_LEVEL_1, ("fid:%d", fid->pid.page));

	if ((bufGroup = bf_OpenGroup(groupSize, BF_MRU, NOFLAGS)) == NULL)	{
		return esmFAILURE;
	}

	if (sm_OpenScanWithGroup(fid, type, bufGroup->index, retDesc, oid) != esmNOERROR) {
		bf_CloseGroup(bufGroup);
		return esmFAILURE;
	}
	SM_ASSERT(LEVEL_1, ((*retDesc)->bufGroup == bufGroup));

	/* remember to remove the buffer group */
	(*retDesc)->removeGroup = TRUE;

	return(esmNOERROR);
}




 int
sm_OpenScanWithGroup (

	FID					*fid,
	int					type,
	int					groupIndex,
	SCANDESC			**retDesc,
	OID					*oid 
)
{

	SCANDESC			*scanDesc;


	TRPRINT(TR_SM, TR_LEVEL_1, ("fid:%d", fid->pid.page));

	if (checkTransState(fid->pid.volid))	{
		return esmFAILURE;
	}
	/*
	 *	attempt to get a scan descriptor
	 */
	if ((scanDesc = (SCANDESC *) listDeq( &ScanDescPool )) == NULL)	{

		/*
		 *	WARNING: This generates a small memory leak if the client
		 *	         is shutdown and then restarted.
		 */
		scanDesc = (SCANDESC*)malloc(sizeof(SCANDESC));
		if (scanDesc == NULL) {
			SM_ERROR(TYPE_WARNING, esmNOFREESCANDESC);
			return esmFAILURE;
		}
		/* NumScanDesc++; */
		initializeListElement( &(scanDesc->scanList), 
			(char *) scanDesc );
		INIT_SCANDESC_MAGIC( scanDesc );
	}

	/*
	 *	remember the buffer group, and remember not to remove it
	 *	at the end of the scan
	 */
	scanDesc->bufGroup = &(BufGroupPtr[groupIndex]);
	if (CHECK_BUFGROUP_VALID(scanDesc->bufGroup)) {
		listEnq( &ScanDescPool, &(scanDesc->scanList) );
		SM_ERROR(TYPE_USER, esmBADGROUP);
		return esmFAILURE;
	}
	scanDesc->removeGroup = FALSE;

	if (SM_OpenScan (fid, scanDesc, oid, type))	{

		listEnq( &ScanDescPool, &(scanDesc->scanList) );
		return esmFAILURE;
	}

	/*
	 *	return the scan descriptor
	 */
	listEnq( &UsedScanDesc, &(scanDesc->scanList) );
	*retDesc = scanDesc;
	return(esmNOERROR);
}




 int
sm_ScanNextBytes (

	SCANDESC			*scanDesc,
	int					length 
)
{

    register OBJECT		*objectPtr;			/* ptr to generic object		*/
	register PAGESLOT	*slotPtr;
    SLOTTEDPAGE			*currentPage;		/* double indirect ref to page	*/
	BUFGROUP			*bufGroup;
	USERDESC			*userDesc;
    PID					rootPid;			/* pid of large object root		*/
	SLOTINDEX			currentSlot;
	int					start;


	TRPRINT(TR_SM, TR_LEVEL_1, ("length:%d", length));

    if (checkTransState(scanDesc->fid.pid.volid))  {
		return esmFAILURE;
	}

	CHECK_SCANDESC_MAGIC(scanDesc);

	/*
	 *	setup local variables
	 */
	currentPage = scanDesc->currentPage;
	currentSlot = scanDesc->currentSlot - 1;
	bufGroup    = scanDesc->bufGroup;
	userDesc    = scanDesc->currentUser;

	/*
	 *	setup the slot pointer
	 */
	slotPtr = GETSLOTPTR(currentPage, currentSlot);

    /*
     * Set the object pointer to point to the object header
     */
    objectPtr = GETOBJECTPTR(currentPage, slotPtr);


#if OBJECT_MAGIC IS_ENABLED
		SM_ERROR(TYPE_FATAL, esmBADOBJECTMAGIC);
	}
#endif

	/*
	 *	pull the start out of the user descriptor
	 */
	start = userDesc->offset + userDesc->byteCount;
	TRPRINT(TR_SM, TR_LEVEL_2, ("start:%d", start));

	/*
	 *	Make sure that the user has specified a valid start
	 */
	if (start >= objectPtr->objectHdr.size)	{

		TRPRINT(TR_SM, TR_LEVEL_2, ("bad start:%d size:%d",
				start, objectPtr->objectHdr.size));
		SM_ERROR(TYPE_USER, esmENDOFOBJECT);
		return esmFAILURE;
	}

	/*
	 *	if the length is less than zero then user wants rest of object
	 */
	if (length < 0) {

		/*
		 *	Calculate remainder of object
		 */
		length = (int)objectPtr->objectHdr.size - start;

	} else {

		/*
		 *	read the minimum of the users requested size
		 *	or the remainder of the object
		 */
		length = MIN(length, ((int)objectPtr->objectHdr.size - start));
	}
	TRPRINT(TR_SM, TR_LEVEL_2, ("returning:%d bytes objsize:%d",
			length, objectPtr->objectHdr.size));

    /*
     * Determine whether we are dealing with a large or small object.
     */
    if (objectPtr->objectHdr.properties & P_LARGEOBJ)	{

		CHUNKDESC		*chunkDesc;

		TRPRINT(TR_SM, TR_LEVEL_2, ("reading large object"));

		bf_UnfixChunk((CHUNKDESC *) userDesc->objInfo, BF_DEFAULT);
		userDesc->objInfo = NULL;

		/*
		 *	check to see if we are looking at a header on
		 *	the slotted page
		 */
		if (objectPtr->objectHdr.properties & P_HEADERONPAGE)	{

			TRPRINT(TR_SM, TR_LEVEL_2, ("slot root"));

			/*
			 *	read the object
			 */
			if (lg_ReadObject( &(currentPage->header.fid), &(scanDesc->currentPid),
							   bufGroup, start, length, &chunkDesc, currentSlot))	{
				return esmFAILURE;
			}

		} else {

			TRPRINT(TR_SM, TR_LEVEL_2, ("page root"));

			/*
			 * Dig out the root pid
			 */
			rootPid.volid = scanDesc->currentPid.volid;
			rootPid.page = ((LARGEOBJHDR *) objectPtr)->rootPid;

			/*
			 *	read the object
			 */
			if (lg_ReadObject( &(currentPage->header.fid), &rootPid, bufGroup,
							   start, length, &chunkDesc, NIL))	{

				return esmFAILURE;
			}
		}

		/*
		 *	initialize the common contents of the user descriptor
		 */
		userDesc->basePtr   = chunkDesc->basePtr;
		userDesc->byteCount = chunkDesc->byteCount;
		userDesc->userFlags	= objectPtr->objectHdr.properties;
		userDesc->objInfo   = (char *) chunkDesc;
		userDesc->offset    = start;

    } else {

		register SMALLOBJ	*smallObject;


		/*
		 * Dealing with a small object.
		 */
		smallObject = (SMALLOBJ *) objectPtr;

		/*
		 *	check to see if we are reading in a forwarded object.
		 */
		if (smallObject->objectHdr.properties & P_MOVED)	{

			GROUPLINK		*forwardLink;
			PAGESLOT		*forwardSlotPtr;
			SLOTTEDPAGE		*forwardPage;
			int				forwardSlot;


			TRPRINT(TR_SM, TR_LEVEL_2, ("forwarded object"));

			/*
			 *	get a pointer to the forward page and slot
			 */
			forwardLink = userDesc->groupLink;
			forwardPage = (SLOTTEDPAGE *) userDesc->groupLink->bufFrame;
    		forwardSlot = ((OID *) smallObject->data)->diskAddr.slot;
			forwardSlotPtr = GETSLOTPTR(forwardPage, forwardSlot);
			smallObject = (SMALLOBJ *) GETOBJECTPTR(forwardPage, forwardSlotPtr);
		}

		/*
		 *	initialize the common contents of the user descriptor
		 */
		userDesc->basePtr   = smallObject->data + start;
		userDesc->byteCount = length;
		userDesc->offset    = start;
		userDesc->userFlags	= objectPtr->objectHdr.properties;
    }

	/*
	 *	set the scan descriptor length
	 */
	scanDesc->currentSize = length;

	/*
	 *	return success
	 */
	return(esmNOERROR);
}




 int
sm_ScanNextObject (

	SCANDESC			*scanDesc,
	int					start,
	int					length,
	USERDESC			**retDesc,
	BOOL				*eof 
)
{

	TRPRINT(TR_SM, TR_LEVEL_1, ("start:%d, length:%d", start, length));

	if (checkTransState(scanDesc->fid.pid.volid))	{
		return esmFAILURE;
	}

	CHECK_SCANDESC_MAGIC(scanDesc);

	/*
	 *	release current object
	 */
	if (scanDesc->currentUser != NULL)	{
		SM_ReleaseScanObject(scanDesc);
		SM_ASSERT( LEVEL_1, (scanDesc->currentUser==NULL) );
	}

    /*
     *  See if the scan has reached the end
     */
    if (scanDesc->flags & SCAN_END) {
        *eof = TRUE;
		*retDesc = NULL;
        return(esmNOERROR);
    }

	SM_ASSERT(LEVEL_1, (scanDesc->type == LOGICAL_SCAN) ||
		(scanDesc->type == PHYSICAL_SCAN));

	return  SM_ScanNext(scanDesc, start, length, retDesc, eof);
}
void
SM_Shutdown()
{
	SM_FREE_PTR(LoadTable);
	SM_FREE_PTR(ScanTable);
}
