/*
 *   $RCSfile: sm_OpenBufferGroup.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "trans.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "lock.h"
#include "bf_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"


/*============================================================================*/
/* 
 * sm_OpenBufferGroup:
 * 	Routine to open a buffer group.
 *
 * IMPORTS:
 *	bf_OpenGroup()
 *	bf_CloseGroup()
 *
 * RETURNS:
 *      esmNOERROR
 *
 * ERROR RETURNS:
 *	esmBADPARAMS  - if there were bad input parameters.
 *	errors returned by lower levels.
 *
 * SIDE EFFECTS:
 *	A buffer group is opened.
 */
/*============================================================================*/


extern TRANSREC	TransRec;


 int
sm_OpenBufferGroup (

	int					groupSize,
	int					replacePolicy,
	int					*groupIndex,
	FLAGS				flags 
)
{

	register BUFGROUP	*bufGroup;


	TRACE(TR_SM, TR_LEVEL_1);

    if (groupSize <= 0) {
	
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
    }

	/*
	 *	check to see if the buffer group is by transaction
	 */
	if (flags & TRANS_GROUP)	{

		/*
		 *	check to see if transaction processing is enabled
		 */
		if (checkLocalTransState())	{

			return(esmFAILURE);
		}
	}

	/*
	 *	Attempt to allocate an open group structure
	 */
	if ((bufGroup = bf_OpenGroup(groupSize, replacePolicy, flags)) == NULL)	{

		/*
		 *	return an error code
		 */
    	return(esmFAILURE);
	
	} else {

		/*
		 *	get the index
		 */
		*groupIndex = bufGroup->index;
		TRPRINT(TR_SM, TR_LEVEL_2, ("buffer group index:%d", *groupIndex));

		/*
		 *	check to see if the buffer group is by transaction
		 */
		if (flags & TRANS_GROUP)	{

			/*
			 *	hang the buffer group off the transaction record
			 */
			listEnq( &(TransRec.bufGroupList), &(bufGroup->transList) );
		}

		/*
		 *	return success
		 */
		return(esmNOERROR);
	}
}
