/*
 * $RCSfile: sm_ParseCommandLine.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:29 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "sm_macro.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "volume.h"
#include "lock.h"
#include "option.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trace_funcs.h"
#include "sm_globals.h"


// extern here since space.c is compiled with C
extern OptionList*	ClientOptions;

/*
 *	Store error messages in global scanErrorString since they should not be popped from
 *	the stack
 */


/*
 * FUNCTION DESCRIPTION:
 *
 * This function examines command line arguments for configuration
 * options.  It returns argv with any sm configuration options
 * removed.  Argc is adjusted as well.
 */

 int
sm_ParseCommandLine (
	int			*argc,			/* number of commnad line arguments */
	char		**argv,			/* command line arguments			*/
	char		**errorMsg		/* if any error occurs, this will
									be set to a string explaining the
									error */
)
{
	OptionInfo*	opt;
	BOOL		errorFound = FALSE;			
	int			i;

	if (errorMsg == NULL) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}

	// set up error message
	strcpy(scanErrorString, "");
	*errorMsg = scanErrorString;

	if (ClientOptions == NULL) {
		if (SM_InitClientOptions() != esmNOERROR) {
			SM_ERROR(TYPE_USER, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}

	i = 0;
	while (i < *argc && !errorFound) {

		if (argv[i][0] == '-' && strlen(argv[i]) > 2) {
			if (ClientOptions->lookup(argv[i]+1, opt) == OPTERR_Success) {
				if (i+1 == *argc) {

					sprintf(scanErrorString, "missing argument for %s\n", argv[i]);
					// remove this option argument
					(*argc)--;

					SM_ERROR(TYPE_USER, esmBADOPTIONVALUE);
					errorFound = TRUE;
				} else {
					if (opt->setValue(argv[i+1], TRUE) != 0) {
						sprintf(scanErrorString, "bad value for argument %s\n", argv[i]);
						errorFound = TRUE;
					}

					// remove these option and value arguments
					for (int j = i+2; j < *argc; j++) {
						argv[j-2] = argv[j];
					}
					(*argc) -= 2;
				}
			} else {
				i++;  // advance to next argument
			}
		} else {
			i++;  // advance to next argument
		}
	}

	if (errorFound) {
		return(esmFAILURE);
	}

    return(esmNOERROR);
}

