/*
 * $RCSfile: sm_ReadConfigFile.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:29 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include <sys/stat.h>
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "sm_macro.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "volume.h"
#include "lock.h"
#include "option.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trace_funcs.h"
#include "sm_globals.h"


// extern here since space.c is compiled with C
extern OptionList*	ClientOptions;

/*
 *	From config/configOptions.c
 */
extern char	*SM_ConfigFileName;

#define	MaxFiles 3
#define SM_CONFIG "sm_config"

/*
 *  Store error messages in global scanErrorString since they should not be popped from
 *  the stack
 */

/*
 * FUNCTION DESCRIPTION:
 *
 * This function reads a configuration file for the client.
 * 
 */

 int
sm_ReadConfigFile (
	char		*configFile,	// config file to read
	char		*programName,	// name of the application	
	char		**errorMsg		// if any error occurs, this will
								//	be set to a string explaining the
								//	error
)
{

	int			errorFound = OPTERR_Success;
	char		fileList[MaxFiles][_POSIX_PATH_MAX+1];
	int			fileListLen = 0;
	int			i;
	struct stat statBuf;	// info from stat() ststem call
	ino_t		lastInode = 0;	// inode for last file read 

	if (errorMsg == NULL) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}

	if (ClientOptions == NULL) {
		if (SM_InitClientOptions() != esmNOERROR) {
			SM_ERROR(TYPE_USER, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}

	// see if we should read the default config files
	if (configFile == NULL) {

		// get file from site dependent location
		// (usually /usr/lib/exodus/sm_config
		strncpy(fileList[0], SM_ConfigFileName, _POSIX_PATH_MAX);

		// get ~/.sm_config
		if (getenv("HOME")) {
			if ( (strlen(getenv("HOME")) + strlen(SM_CONFIG) +2) >= _POSIX_PATH_MAX) {
				sprintf(scanErrorString, "$HOME path name is too long\n");
				SM_ERROR(TYPE_SYS, ENAMETOOLONG);
				return(esmFAILURE);
			}
			strncpy(fileList[1], getenv("HOME"), _POSIX_PATH_MAX);
			strcat(fileList[1], "/.");
			strcat(fileList[1], SM_CONFIG);
		} else {
			sprintf(scanErrorString, "Environment variable $HOME does not exist\n");
			SM_ERROR(TYPE_USER, esmINTERNAL);
			return(esmFAILURE);
		}

		// get ./sm_config
		strcpy(fileList[2], "./.");
		strcat(fileList[2], SM_CONFIG);

		fileListLen = 3;
	} else {
		strncpy(fileList[0], configFile, _POSIX_PATH_MAX);
		fileListLen = 1;
	}
	
	// set up the program name as the second option class level
	if (programName != NULL) {
		ClientOptions->setClassLevel(programName, 1);
	}

	// read each file
	for (i = 0; i < fileListLen; i++) {

		// make sure we have not already read this file
		if (i > 0) {
			if (stat(fileList[i], &statBuf) != 0) {
				// if this is a default file, then just continue
				if (configFile == NULL) continue;
		
				SM_ERROR(TYPE_SYS, errno);
				return(esmFAILURE);
			}
			if (statBuf.st_ino == lastInode) {
				// this file is the same as the previous one,
				// so skip it
				continue;
			}
			lastInode = statBuf.st_ino;
		}

		sm_errno = errno = 0;
		errorFound = processOptionFile(fileList[i], ClientOptions, TRUE, errorMsg);

		if (errorFound != esmNOERROR) {

			// If it was a unix file access error and this is a
			// default file, the just continue
			if (SM_ERR_BASE_DECODE(sm_errno) == UNIX_ERROR_BASE &&
				configFile == NULL ) {
				continue;
			}
			return(esmFAILURE);
		}
	}

    return(esmNOERROR);
}

