/*
 *   $RCSfile: sm_RemoveEntry.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "bf.h"
#include "chunk.h"
#include "lsn.h"
#include "volume.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "scan.h"
#include "trans.h"
#include "sm_state.h"
#include "sm_extfuncs.h"
#include "sm_intfuncs.h"
#include "bf_globals.h"
#include "bf_extfuncs.h"
#include "trans_funcs.h"
#include "trans_globals.h"
#include "bt_extfuncs.h"
#include "lh_extfuncs.h"


int sm_RemoveEntry(IID* ndx, int groupIndex, KEY* key, void* elem)
{
	BUFGROUP* 	bufGroup = & BufGroupPtr[groupIndex];
	INDEXDESC* 	np;
	GROUPLINK* 	gLink;

    if (CHECK_BUFGROUP_VALID(bufGroup))  {
        SM_ERROR(TYPE_USER, esmBADGROUP);
        return esmFAILURE;
    }

    INIT_SMSTATE(&SM_State);
    // check to see if a transaction is active
    if (checkTransState(ndx->pid.volid))  {
        return(esmFAILURE);
    }

    int loadNdx;
    if (sm_FindIndexLoad(ndx, &loadNdx)) {
        return esmFAILURE;
        }
    if (loadNdx >= 0)  {
        SM_ERROR(TYPE_USER, esmBULKLOADING);
	return esmFAILURE;
        }


	
	if ((gLink = bf_ReadPage(bufGroup, &ndx->pid, H_INDEXDESC, 
										INDEXDESC_PAGE2SIZE, SH)) == NULL) 
		return esmFAILURE;
	
	SM_ASSERT(LEVEL_3, ((INDEXDESCPAGE*) gLink->bufFrame)->magic == NDXMAGIC);

	np = & (((INDEXDESCPAGE*) gLink->bufFrame)->desc);
	
	if (key->length > np->maxKeyLen)  {
		SM_ERROR(TYPE_USER, esmKEYTOOLONG);
		bf_UnfixPage(gLink, BF_LRU, FALSE);
		return esmFAILURE;
		}
	
	switch (np->ndxType)  {
		case SM_BTREENDX:
			if (bt_RemoveEntry(TransRec.clientTid, np->rootPid, bufGroup,
									key->valuePtr, key->length, 
									np->elSize, elem,
									np->maxKeyLen, np->unique, 
									SMCOMPFUNC[np->dataType], 0))  {
				bf_UnfixPage(gLink, BF_LRU, FALSE);
				return esmFAILURE;
				}
			break;
			
		case SM_HASHNDX:
           	if (lh_RemoveEntry(TransRec.clientTid, np->rootPid, bufGroup,
									key->valuePtr, key->length,
									np->elSize, elem,
								   	np->maxKeyLen, np->unique,
								   	SMCOMPFUNC[np->dataType], TRUE))  {
				bf_UnfixPage(gLink, BF_LRU, FALSE);
				return esmFAILURE;
		 	}
			break;

		default:
			SM_ERROR(TYPE_USER, esmWRONGNDXTYPE);
		}
	
	bf_UnfixPage(gLink, BF_LRU, FALSE);
		
	return esmNOERROR;
}

