/*
 *   $RCSfile: sm_RootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:28 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include <string.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "volume.h"
#include "io_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"


 int
sm_SetRootEntry (

	VOLID		volid,
	char		*name,
	void		*data,
	int			dataLength 
)
{

	TRACE(TR_SM, TR_LEVEL_1);

	if (checkTransState(volid))	{
		return(esmFAILURE);
	}

	if (strlen(name) >= MAX_ROOTNAME_SIZE)	{
		SM_ERROR(TYPE_USER, esmROOTNAMETOOLONG);
		return(esmFAILURE);
	}

	if (dataLength > MAX_ROOTDATA_SIZE)	{
		SM_ERROR(TYPE_USER, esmBADLENGTH);
		return(-1);
	}

	{
		int prop;

		if(sm_VolumeProperties(volid, &prop) != esmNOERROR) {
			return esmFAILURE;
		}
		if(prop & VOLPROP_TEMP) {
			SM_ERROR(TYPE_USER, esmNOTIMPLEMENTED);
			return esmFAILURE;
		}
	}
    return(io_SetRootEntry(volid, name, (char*)data, dataLength));
}


 int
sm_GetRootEntry (

	VOLID		volid,
	char		*name,
	void		*data,
	int			*dataLength 
)
{
	int	 retLength = *dataLength;

	TRACE(TR_SM, TR_LEVEL_1);

	if (checkTransState(volid))	{
		return(esmFAILURE);
	}

	/* check to see if the name size is OK */
	if (strlen(name) >= MAX_ROOTNAME_SIZE)	{
		SM_ERROR(TYPE_USER, esmROOTNAMETOOLONG);
		return(esmFAILURE);
	}

	if (data == NULL) {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}
	{
		int prop;

		if(sm_VolumeProperties(volid, &prop) != esmNOERROR) {
			return esmFAILURE;
		}
		if(prop & VOLPROP_TEMP) {
			SM_ERROR(TYPE_USER, esmNOTIMPLEMENTED);
			return esmFAILURE;
		}
	}


    if (io_GetRootEntry(volid, name, (char*)data, &retLength) != esmNOERROR) {
		return(esmFAILURE);
	}
	*dataLength = retLength;
    return(esmNOERROR);
}


 int
sm_RemoveRootEntry (

	VOLID		volid,
	char		*name
)
{

	TRACE(TR_SM, TR_LEVEL_1);

	if (checkTransState(volid))	{
		return(esmFAILURE);
	}

	/* check to see if the name size is OK */
	if (strlen(name) >= MAX_ROOTNAME_SIZE)	{
		SM_ERROR(TYPE_USER, esmROOTNAMETOOLONG);
		return(esmFAILURE);
	}

	{
		int prop;

		if(sm_VolumeProperties(volid, &prop) != esmNOERROR) {
			return esmFAILURE;
		}
		if(prop & VOLPROP_TEMP) {
			SM_ERROR(TYPE_USER, esmNOTIMPLEMENTED);
			return esmFAILURE;
		}
	}
    if (io_RemoveRootEntry(volid, name) != esmNOERROR) {
		return(esmFAILURE);
	}
    return(esmNOERROR);
}
