/*
 *   $RCSfile: sm_SetLogLevel.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "lock.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "trans.h"
#include "file.h"
#include "sm_state.h"
#include "fi_extfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"

/* from fi_FileInfoTable.C */
extern BOOL        LoggingOffForSomeFile;


 int
sm_SetLogLevel (

	int				loggingLevel,
	int				fileCount,
	FID				fids[] 
)
{
	FILEINFO		*fileInfo;
	int				i;

	TRACE(TR_SM, TR_LEVEL_1);

	switch (loggingLevel) {
	case LOG_ALL: 
		break;	
	case LOG_SPACE: 
		break;	
	default: 
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}

	if (fileCount == 0) {

		/* Install the default for already known files */
		fi_SetAllFileLogLevel(loggingLevel);

	} else if (fileCount > 0) {

		/* set the logging level for the files specified */
		for (i = 0; i < fileCount; i++) {
			fileInfo = fi_FindFileInfo(fids+i, TRUE);
			if (fileInfo == NULL) return(esmFAILURE);
			if (!fileInfo->tempVolume) fileInfo->logLevel = loggingLevel;
		}

		if (loggingLevel != LOG_ALL) {
			LoggingOffForSomeFile = TRUE;
		}

	} else {
		SM_ERROR(TYPE_USER, esmBADPARAMS);
		return(esmFAILURE);
	}

	return(esmNOERROR);
}
