/*
 *   $RCSfile: SM_CheckPageUse.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:30 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "lock.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"


 int
SM_CheckPageUse (

	register GROUPLINK		*homeLink 
)
{

	register GROUPLINK		*tempLink;


	TRACE(TR_SM, TR_LEVEL_1);

	/*
	 *	we must step through the grouplinks to see if
	 *	any of them have the page in the fixed list
	 */
	tempLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(homeLink->pageHash->memberList) );

	/*
	 *	run through the links
	 */
	while (tempLink != NULL)	{

		/*
		 *	Check the groupLink magic number
		 */
		CHECK_GROUPLINK_MAGIC(tempLink);

		/*
		 *	check to see if we are at our own link
		 */
		if (tempLink != homeLink)	{

			/*
			 *	check the fixed count
			 */
			if (tempLink->fixedCount > 0)	{

				TRPRINT(TR_SM, TR_LEVEL_2, ("other greater than zero"));

				/*
				 *	we cannot compact
				 */
				return(esmFAILURE);
			}

		} else {

			/*
			 *	if it is our link then check for
			 *	any additional fixes
			 */
			if (tempLink->fixedCount > 1)	{

				TRPRINT(TR_SM, TR_LEVEL_2, ("same greater than one"));

				/*
				 *	we cannot compact
				 */
				return(esmFAILURE);
			}
		}

		/*
		 *	look at the next entry
		 */
		tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(tempLink->hashList) );
	}

	/*
	 *	return success
	 */
	return(esmNOERROR);
}


