/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "sm_macro.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "volume.h"
#include "lock.h"
#include "option.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trace_funcs.h"
#include "sm_globals.h"

#ifndef __cplusplus
extern char*	getenv(char*);
#endif

/*
 *	this global is defined here since space.c can be compiled with C
 */
extern OptionList*	ClientOptions;

 int
SM_CheckRequiredOptions ()
{
	OptionInfo	*opt;

	if (ClientOptions == NULL) {
		if (SM_InitClientOptions() != esmNOERROR) {
			return esmFAILURE;
		}
	}

	ClientOptions->initScan();
	for (opt = ClientOptions->getNext(); opt != NULL; opt = ClientOptions->getNext()) {
		if (opt->required() && !opt->set()) {
			switch (opt->ID()) {
			case SM_OPT_bufpages:
				SM_ERROR(TYPE_USER, esmBUFPAGESNOTSET);
				break;
			default:
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			return esmFAILURE;
		}
	}
    return esmNOERROR;
}
