/*
 *   $RCSfile: SM_GetFirstLastOid.c,v $
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:30 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "file.h"
#include "trace.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "lock.h"
#include "file.h"
#include "fi_macro.h"
#include "bf_extfuncs.h"
#include "fi_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"
#include "bf_globals.h"


/*============================================================================*/
/* 
 * sm_GetFirstOid:
 * 	Routine to get the first oid of a file.
 *
 * RETURNS:
 *      esmNOERROR if all went well. 
 *	The oid parameter is filled in.
 *
 * ERROR RETURNS:
 *	errors returned by lower levels.
 *
 * SIDE EFFECTS:
 *	The oid parameter is filled in.
 */
/*============================================================================*/


extern int			sm_errno;


 int
SM_GetFirstLastOid (

	int						groupIndex,
	FID						*fid,
	OID						*nextOid,
	OBJHDR					*objectHdr,
	BOOL					*eof,
	FLAGS					fi_which
)
{

    PID						pid, nextPid;
	BUFGROUP				*bufGroup;
	GROUPLINK				*currentLink;
	SLOTINDEX				slot;
	FLAGS					neighbor_which;


	TRPRINT(TR_SM, TR_LEVEL_1, ("volid:%d page:%d which 0x%x", 
							fid->pid.volid, fid->pid.page, fi_which));

	if (checkTransState(fid->pid.volid))	{
		return(esmFAILURE);
	}

	bufGroup = &BufGroupPtr[groupIndex];

	if (CHECK_BUFGROUP_VALID(bufGroup))	{
		SM_ERROR(TYPE_USER, esmBADGROUP);
		return(esmFAILURE);
	}

    pid.volid = fid->pid.volid;

	if(fi_which ==FI_FIRST) {
		neighbor_which = NEIGHBOR_AFTER;
	} else {
		SM_ASSERT(LEVEL_1, fi_which == FI_LAST);
		neighbor_which = NEIGHBOR_BEFORE;
	}
    /*
     * Get the pid of the first phys page in the file.
     */
    if (fi_Page(fid, &pid, (fi_which | PHYSICAL_SCAN) ))	{

		if (sm_errno == esmEMPTYFILE) {

			TRPRINT(TR_SM, TR_LEVEL_2, ("file is empty"));
			*eof = TRUE;
			return(esmNOERROR);

		} else {
			return(esmFAILURE);
		}
	}
	/* 
	 * read that page 
	 */
	if ((currentLink = bf_ScanPage(bufGroup, &pid, 
			PHYSICAL_SCAN, &nextPid, fid, neighbor_which)) == NULL)	{

		if (sm_errno == esmAFTERLASTPID) {
			*eof = TRUE;
			return(esmNOERROR);
		} else {
			return(esmFAILURE);
		}
	}
	if(fi_which == FI_FIRST) {
		slot = -1;
	} else {
		SLOTTEDPAGE				*currentPage;

		currentPage = (SLOTTEDPAGE *) currentLink->bufFrame;
		slot = currentPage->header.slotCount; /* beyond the end */ 
	}

	return SM_GetOid (bufGroup, currentLink, slot, nextOid, 
						&nextPid, objectHdr, eof, neighbor_which);
}
