/*
 *   $RCSfile: SM_GetOid.c,v $
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:30 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "file.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "bf_extfuncs.h"
#include "fi_extfuncs.h"
#include "sm_intfuncs.h"
#include "sm_extfuncs.h"
#include "trans_funcs.h"
#include "bf_globals.h"


extern int			sm_errno;


 int
SM_GetOid (

	BUFGROUP		*bufGroup,
	GROUPLINK		*currentLink,
	SLOTINDEX		currentSlot,
	OID				*nextOid,
	PID				*nextPid,
	OBJHDR			*objectHdr,
	BOOL			*eof,
	NEIGHBORFLAGS	direction
)
{
	PAGESLOT		*slotPtr;
	OBJECT			*objectPtr;
	PID				currentPid;
	BOOL			endOfPage;
	SLOTTEDPAGE		*currentPage;

	TRPRINT(TR_SM, TR_LEVEL_1, ("slot %d direction %d fixedCharges %d", 
		currentSlot, direction, bufGroup->fixedCharges));
	SM_ASSERT(LEVEL_1, currentLink != NULL);

	currentPage = (SLOTTEDPAGE *) currentLink->bufFrame;
	if (CHECK_SLOTTED_MAGIC(currentPage))   {
		 /* fatal -> no need to unfix page but we put it here, lest we
		  *  decide it should be WARNING
		  */
		 bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);
		 SM_ERROR(TYPE_FATAL, esmBADSLOTTEDMAGIC);
		 return(esmFAILURE);
	}
	currentPid = currentPage->header.pid;
    /*
     * Find the next valid slot in the page.
     */
    for (*eof = FALSE;;) {
		/*
		 *  bump the slot pointer & 
		 *	check to see if we've looked at all the slots on the page
		 */

		if( direction == AFTER ) {
			currentSlot++;
			endOfPage = (currentSlot >= currentPage->header.slotCount);
		} else {
			SM_ASSERT(LEVEL_1, direction == BEFORE);
			currentSlot--;
			endOfPage = (currentSlot < 0);
		}

		if(endOfPage) {
			TRPRINT(TR_SM, TR_LEVEL_1, 
				("no slots left on page:%d currentSlot %d",
				currentPid.page, currentSlot));

			bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);
			if( nextPid->page == NULLPID ) {
				*eof = TRUE;
				TRPRINT(TR_SM, TR_LEVEL_1, 
					("fixedCharges %d", bufGroup->fixedCharges));
				return(esmNOERROR);
			}
			currentPid = *nextPid;
			if ((currentLink = bf_ScanPage(bufGroup, &currentPid, 
				PHYSICAL_SCAN, nextPid, &(currentPage->header.fid),
				direction)) == NULL){

				if (sm_errno == esmAFTERLASTPID) {
					*eof = TRUE;
					return(esmNOERROR);
				} else {
					return(esmFAILURE);
				}
			}
			TRPRINT(TR_SM, TR_LEVEL_1, ("fixedCharges %d", bufGroup->fixedCharges));

			currentPage = (SLOTTEDPAGE *) currentLink->bufFrame;
			if (CHECK_SLOTTED_MAGIC(currentPage))   {
				 bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);
				 SM_ERROR(TYPE_FATAL, esmBADSLOTTEDMAGIC);
				 return(esmFAILURE);
			}

			if(direction == AFTER) {
				currentSlot = 0;
			} else {
				currentSlot = currentPage->header.slotCount - 1;
			}

			if( currentPage->header.slotCount == 0) 
				/* empty page - run through loop again */
				continue; 
		}
		slotPtr = GETSLOTPTR(currentPage, currentSlot);

		/*
		 * If the slot isn't empty, then we can break out of the loop.
		 */
		if (slotPtr->offset != NIL) {

			/*
			 *	get a pointer to the object to see if it is
			 *	a forwarded object that we want to skip
			 */
			objectPtr = GETOBJECTPTR(currentPage, slotPtr);

			/*
			 *	check to see if this is the forwarded portion for a
			 *	version history graph
			 */
			if (!(objectPtr->objectHdr.properties & INVISIBLE_OBJECT))	{

				/* user might want an object header */
				if (objectHdr != NULL)	{
					*objectHdr = objectPtr->objectHdr;
				}
				break;
			}
		}
    }

    /* 
     *	Unfix the last page read
     */
	bf_UnfixPage(currentLink, BF_DEFAULT, FALSE);

    /*
     * Fill in the nextOid.
     */
    nextOid->diskAddr.page	 = currentPid.page;
    nextOid->diskAddr.slot	 = currentSlot;
    nextOid->diskAddr.unique = slotPtr->unique;
	nextOid->diskAddr.volid  = currentPid.volid;

	TRPRINT(TR_SM, TR_LEVEL_1, 
		("nextOid page:%d.%d, slot:%d", nextOid->diskAddr.page, 
		nextOid->diskAddr.volid, nextOid->diskAddr.slot));

	TRPRINT(TR_SM, TR_LEVEL_1, ("fixedCharges %d", bufGroup->fixedCharges));

    return(esmNOERROR);
}
