/*
 *   $RCSfile: SM_NeighborPids.c,v $  
 *   $Revision: 1.1.1.1 $
 *   $Date: 1996/05/04 21:55:30 $
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "volume.h"
#include "trans.h"
#include "lsn.h"
#include "loginfo.h"
#include "clog.h"
#include "lm.h"
#include "file.h"
#include "scan.h"
#include "load.h"
#include "fi_msg.h"
#include "fi_extfuncs.h"
#include "sm_intfuncs.h"
#include "bf_globals.h"
#include "trans_globals.h"
#include "clog_globals.h"
#include "lock_globals.h"
#include "sm_globals.h"



 int 
SM_NeighborPids (
	SCANDESC			*scanDesc,
	NEIGHBORFLAGS		direction
)
{
	BOOL 		needPidList = FALSE;

	SM_ASSERT(LEVEL_1, (scanDesc->type == PHYSICAL_SCAN) );

	if(scanDesc->pidListBegin == NULL) {
		SM_ASSERT(LEVEL_1, (scanDesc->flags == SCAN_BEGIN) );
		needPidList = TRUE;
	} else {
		/* 
		 * Adjust current because it did not get done in SM_ScanNext().
		 * We then get the neighbor.
		 */ 
		if(direction & NEIGHBOR_BEFORE) {
			-- (scanDesc->pidListCurrent);
			scanDesc->nextPid  = (scanDesc->pidListCurrent)-1 ;
			needPidList = (scanDesc->nextPid < scanDesc->pidListBegin);
		} else { 
			/* NEIGHBOR_AFTER or NEIGHBOR_BEFORE */
			scanDesc->pidListCurrent ++;
			scanDesc->nextPid  = (scanDesc->pidListCurrent)+1 ;
			needPidList = (scanDesc->nextPid > scanDesc->pidListEnd);
		}
		SM_ASSERT(LEVEL_1,
			((scanDesc->pidListCurrent >= scanDesc->pidListBegin) &&
			(scanDesc->pidListCurrent <= scanDesc->pidListEnd)));

	}
	TRPRINT(TR_SM, TR_LEVEL_1, 
	("Need list for pg:%d (b 0x%x e 0x%x C 0x%x n 0x%x): %s",
		scanDesc->currentPid.page,
		scanDesc->pidListBegin, scanDesc->pidListEnd,
		scanDesc->pidListCurrent, scanDesc->nextPid,
		needPidList?"YES":"NO"));
	if(needPidList) {
		if( fi_NeighborPid ( &(scanDesc->currentPid), &(scanDesc->fid), 
			&(scanDesc->pidListBegin), &(scanDesc->pidListEnd), 
			direction, NULL) ) {

			if ((sm_errno == esmAFTERLASTPID) ||
					(sm_errno == esmBEFOREFIRSTPID) ) {

				/* phys scan; uses null ptr to indicate eof */
				scanDesc->nextPid = NULL;
				TRPRINT(TR_SM, TR_LEVEL_1, ("No NeighborPids %s page: %d",
					(direction==NEIGHBOR_BEFORE)?"before":"after",
					scanDesc->currentPid.page));
				return esmNOERROR;
			}

			return esmFAILURE;
		}
		if(direction == NEIGHBOR_BEFORE) {
			scanDesc->nextPid  = scanDesc->pidListEnd;
			scanDesc->pidListCurrent =  scanDesc->nextPid + 1;
		} else {
			scanDesc->nextPid  = scanDesc->pidListBegin;
			scanDesc->pidListCurrent =  scanDesc->nextPid - 1;
		}
	}
	TRPRINT(TR_SM, TR_LEVEL_1,
		("NeighborPids go %s from %d.%d",
			(direction==NEIGHBOR_BEFORE)?"backward":"forward",
			*(scanDesc->nextPid), scanDesc->fid.pid.volid));

	return esmNOERROR;
}
