/*
 *   $RCSfile: SM_SetVersionGraph.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:29 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "chunk.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "error.h"
#include "trace.h"
#include "lock.h"
#include "trans.h"
#include "file.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "pool.h"
#include "version_graph.h"
#include "sm_intfuncs.h"
#include "sm_version_intfuncs.h"
#include "sm_extfuncs.h"


extern TRANSREC	TransRec;
extern int		sm_errno;


 int
SM_SetVersionGraph (

	BUFGROUP			*bufGroup,
	register OID		*oid,
	VERSIONGRAPH		*vhg 
)
{
	USERDESC	*vhgUserDesc;

	TRPRINT(TR_SM, TR_LEVEL_1, ("set version"));

	/*
	 *	Read the entire version history graph
	 */
	if (SM_ReadObject(bufGroup, oid, 0, -1, &vhgUserDesc, NULL, TRUE, EX)) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return (esmFAILURE);	
	}

	/*
	 *	Make sure we've read in a version history graph
	 */
	CHECK_VERSIONGRAPH_MAGIC((VERSIONGRAPH*)(vhgUserDesc->basePtr));
	CHECK_VERSIONGRAPH_MAGIC(vhg);
	SM_ASSERT(LEVEL_3, vhgUserDesc->userFlags & P_HISTORYGRAPH);

	/*
	 *	Write out the updated version history graph
	 */
	if (sm_WriteObject(bufGroup->index, 0, vhgUserDesc->byteCount,
						(char*) vhg, vhgUserDesc, TRUE) ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return (esmFAILURE);	
	}

    return(esmNOERROR);
}
