/*
 *   $RCSfile: internalAbort.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:30 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "bf.h"
#include "chunk.h"
#include "msgdefs.h"
#include "msg_funcs.h"
#include "trans.h"
#include "trans_funcs.h"
#include "trans_globals.h"
#include "msg_globals.h"
#include "sm_globals.h"

static  BOOL
helperfunc (
	SERVERTRANSREC  *stl,
	MESSAGE 	*msg
)
{
	MESSAGE 	message;
	
	message = *msg; /* a copy */
	message.header.params.in.tid = stl->tid;

	if(server_connectInit(stl->serverInfo, 
		FALSE/* is not admin */) != esmNOERROR) {
		return FALSE;
	}

	(void) callServer(stl->serverInfo, &message, NULL, 0 );
	return FALSE; 
}

 void
internalAbort()
{
	MESSAGE 	message;

	TRACE(TR_TRANS, TR_LEVEL_1);

	/* 
	 * Aborts the CURRENT trans on the server.
	 * Does NOT free any local resources.
	 * Applic still has to call sm_AbortTransaction() to
	 * free resources.
	 */
	SM_ASSERT( LEVEL_3, (TransRec.clientTransState == T_ACTIVE)); 

	SM_TRANSABORTED(TYPE_USER, sm_errno);

	/*
	 *	setup the begin transaction message
	 */
	message.header.type = ABORT_TRANS;
	message.header.replyRequested = TRUE;
	{
		ForEachServerInTrans(findTransRec(getLocalTid()), 
						4, helperfunc, &message, NULL, 0);
		TransRec.clientTransState  = T_ABORT;
	}
}
