/*
 *   $RCSfile: createVHG.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:32 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "list.h"
#include "tid.h"
#include "bf.h"
#include "trace.h"
#include "error.h"
#include "scan.h"
#include "load.h"
#include "sm_params.h"
#include "btree.h"
#include "pool.h"
#include "lock.h"
#include "version_graph.h"
#include "version_funcs.h"
#include "sm_extfuncs.h"
#include "sm_globals.h"


 int
createVHG(
    int				groupIndex,
	FID   			*fid,
	OID				*oid,  		/* object being versioned 		*/
	OID				*vhgOid,	/* new version graph 			*/
	VHGNODEID		*rootNodeId /* root node of version graph 	*/
)
{
	VERSIONGRAPH	graph; 		/* graph being created 			*/
	OBJHDR			objHdr;		/* header for the VHG object	*/

	TRPRINT(TR_VERSION, TR_LEVEL_1, ("Creating VHG \n") );

	/*
	 *	Initialize the graph
	 */
	if (initializeVHG(&graph, oid, V_frozen)) {
		return(esmFAILURE);
	}

	/*
	 *	Set up the VHG object's header telling sm_CreateObject to
	 *	set the versioned flag
	 */
	objHdr.tag = 0; 
	objHdr.properties = P_HISTORYGRAPH;

	/*
	 *	Create the version history graph object, the global variable
	 *	InternalCall must be set to indicate that the properties
	 *	field should be set in the object header.  It must be unset
	 *	when the call is complete.
	 */
	InternalCall = TRUE;
	if (sm_CreateObject(groupIndex, fid, NEAR_OBJ, oid, &objHdr, 
						sizeof(graph), (char*) &graph, vhgOid) ) {
		InternalCall = FALSE;
		return(esmFAILURE);
	}
	InternalCall = FALSE;

	/*
	 * 	return the rootNodeId
	 */
	*rootNodeId = graph.root;	

	return(esmNOERROR);
}
