/*
 *   $RCSfile: createVersion.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:32 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "bf_external_group.h"
#include "version_graph.h"
#include "version_funcs.h"
#include "sm_macro.h"


 int
createVersion(
	VERSIONGRAPH 	*graph,			/* graph to create version in 	*/
	OID				*parentOid,		/* parent object for version 	*/
	VHGNODEID		parentNodeId, 	/* node in VHG of parentOid		*/
	VHGNODEID		*newNodeId 		/* new version id for version	*/
)
{
	VHGNODE* 	newNode;

	TRPRINT(TR_VERSION, TR_LEVEL_1, ("Adding a new VHG node \n") );

	/*
	 *  Do some defensive error checking.
	 */
	CHECK_VERSIONGRAPH_MAGIC(graph);
	CHECK_VHGNODE_MAGIC(&graph->nodeArray[parentNodeId]);

	/*
	 *  Validate parentNode
	 */
	if (parentNodeId >= graph->nodeCount) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	
	/*
	 *  Validate parentNode and parentOid
	 */
	if ( !OIDEQ(graph->nodeArray[parentNodeId].oid, *parentOid) ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *  Make sure parent version is frozen
	 */
	if ( ! (graph->nodeArray[parentNodeId].flags & V_frozen) ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 * 	It should now be safe to set up the version node
	 */

	/*
	 *	get a new node
	 */
	newNode = listVHGDeq(graph, &graph->freeList);
	if (newNode == NULL) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	Check its magic number
	 */
	CHECK_VHGNODE_MAGIC(newNode);

	/*
	 *	No flags are initially set for a node
	 */
	newNode->flags = NOFLAGS;

	/*
	 *	Set the new node's parent
	 */
	newNode->parentId = parentNodeId;

	/*
	 *	The new node has no children
	 */
	initializeVHGList(graph, &newNode->children);

	/*
	 *  Add the node as a child of the parent	
	 */
	listVHGEnq(graph, &graph->nodeArray[parentNodeId].children, &newNode->siblings);

	/*
	 *	Make sure nothing was messed up
	 */
	CHECK_VERSIONGRAPH_MAGIC(graph);
	CHECK_VHGNODE_MAGIC(newNode);
	CHECK_VHGNODE_MAGIC(&graph->nodeArray[parentNodeId]);

	/*
	 *  Return the new node id
	 */
	*newNodeId = newNode->id;
	return esmNOERROR;
}
