/*
 *   $RCSfile: listVHGDeq.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:32 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "checking.h"
#include "ess.h"
#include "io.h"
#include "object.h"
#include "error.h"
#include "trace.h"
#include "list.h"
#include "pool.h"
#include "bf_external_group.h"
#include "version_graph.h"
#include "version_funcs.h"

 VHGNODE*
listVHGDeq(

	VERSIONGRAPH		*graph,
	VHGNODELIST			*head 
)
{
	VHGNODE				*node;
	VHGNODELISTELEMENT	*element;

    TRPRINT(TR_VHGLIST, TR_LEVEL_1, ("head:%x ", head));

    /*
     *  Check to see if the magic number is valid
     */
    CHECK_VHGLIST_MAGIC(head);

	/*
     *  Check to see if there is anything on the list
     */
    if (VHGLIST_EMPTY(graph, head))   {

        /*
         *  Just return null pointer indicating that
         *  there were no elements
         */
        TRPRINT(TR_VHGLIST, TR_LEVEL_2, ("element:%x", VHGNULLNODE));
        return(NULL);

    } else {

        /*
         *  Take the first element off the list
         */
        element    = (VHGNODELISTELEMENT*) VHGDEREF(graph, head->succ);
        head->succ = element->succ;
        (VHGDEREF(graph, head->succ))->pred = VHGOFFSET(graph, head);
        element->succ = VHGNULLNODE;
        CHECK_VHGLIST_MAGIC(element);

        /*
         *  Return the item pointer
         */
		node = (VHGNODE*) VHGDEREF(graph, element->item);
		CHECK_VHGNODE_MAGIC(node);
        TRPRINT(TR_VHGLIST, TR_LEVEL_2, ("element:%x", element->item));
        return(node);
    }
}
