/*
 *   $RCSfile: markDestroyPending.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:32 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "bf_external_group.h"
#include "version_graph.h"
#include "version_funcs.h"
#include "sm_macro.h"


 int
markDestroyPending(
	VERSIONGRAPH 	*graph,		/* graph containing node	*/
	VHGNODEID		nodeId  	/* node to mark pending		*/
)
{
	VHGNODE* 	node;

	TRPRINT(TR_VERSION, TR_LEVEL_1, ("freezing node %d \n", nodeId) );

	CHECK_VERSIONGRAPH_MAGIC(graph);
	
	/*
	 *  Validate  nodeId
	 */
	if (nodeId >= graph->nodeCount) {
		SM_ERROR(TYPE_WARNING, esmINTERNAL);
		return esmFAILURE;
	}

	/*
	 *  Get a pointer to the node
	 */
	node = &graph->nodeArray[nodeId];

	/*
	 *  Do some defensive error checking.  The node should be frozen and
	 *	not marked destroy_pending or be a tombstone.
	 */
	CHECK_VHGNODE_MAGIC(node);
	SM_ASSERT(LEVEL_3, node->flags & V_frozen);
	SM_ASSERT(LEVEL_3, !(node->flags & 
						 (V_destroy_pending || V_tombstone) ) );


	node->flags |= V_destroy_pending;

	return esmNOERROR;
}
