/*
 *   $RCSfile: parentDestroyPending.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "bf_external_group.h"
#include "version_graph.h"
#include "version_funcs.h"
#include "sm_macro.h"

/*
 *	ParentDestroyPending() sees if the parent of the version is marked 
 *  destroy-pending and has all its children frozen.  If so, it returns 
 *	TRUE, otherwise false
 */

 BOOL
parentDestroyPending(
	VERSIONGRAPH 	*graph,		/* graph containing node	*/
	VHGNODEID		nodeId, 	/* node to check parent of	*/
	OID				*pendingOid /* object pending destruction */
)
{
	VHGNODE* 	node;
	VHGNODE* 	parent;

	TRPRINT(TR_VERSION, TR_LEVEL_1, ("checking node %d \n", nodeId) );

	CHECK_VERSIONGRAPH_MAGIC(graph);
	
	/*
	 *  Validate  nodeId
	 */
	if (nodeId >= graph->nodeCount) {
		SM_ERROR(TYPE_WARNING, esmINTERNAL);
		return(FALSE);
	}

	/*
	 *  Get a pointer to the node
	 */
	node = &graph->nodeArray[nodeId];

	/*
	 *  Do some defensive error checking.
	 */
	CHECK_VHGNODE_MAGIC(node);

	/*
	 *	See if the node has a parent, and if so, get it
	 */
	if (node->parentId != VHGNULLNODE) {
		parent = &graph->nodeArray[node->parentId];
	} else {
		/*
		 * no parent
		 */
		return(FALSE);
	}
	
	if ( (parent->flags & V_destroy_pending) &&
		 !workingChildren(graph, node->parentId)) {

		/*
		 *	Return the parent's oid
		 */
		*pendingOid = parent->oid;
		return(TRUE);
	}

	return(FALSE);
}
