/*
 *   $RCSfile: workingChildren.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:32 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h>
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "bf_external_group.h"
#include "version_graph.h"
#include "version_funcs.h"
#include "sm_macro.h"

/*
 * WorkingChildren() returns true if the node has any children
 * which are working versions.
 */

 BOOL
workingChildren(
	VERSIONGRAPH 	*graph,		/* graph containing node			*/
	VHGNODEID		nodeId  	/* inspect children of this node	*/
)
{
	VHGNODE 			*node;
	VHGNODELISTELEMENT	*child;
	VHGNODE 			*childNode;


	TRPRINT(TR_VERSION, TR_LEVEL_1, 
			("checking for working children: node %d \n", nodeId) );

	CHECK_VERSIONGRAPH_MAGIC(graph);
	
	/*
	 *  Validate  nodeId
	 */
	if (nodeId >= graph->nodeCount) {
		SM_ERROR(TYPE_WARNING, esmINTERNAL);
		return FALSE;
	}

	/*
	 *	Get a pointer to the node
	 */
	node = &(graph->nodeArray[nodeId]);

	/*
	 *  Do some defensive error checking.
	 */
	CHECK_VHGNODE_MAGIC(node);

	/*
	 *	Get a pointer to the first child
	 */
	child = (VHGNODELISTELEMENT*) VHGDEREF(graph, node->children.succ);
	CHECK_VHGLIST_MAGIC(child);

	/*
	 *	Look at each child until a working version is found or 
	 *	the end-of-list is reached
	 */
	while ( child != &node->children ) {
		
		/*
		 * get a pointer to the child node
		 */
		childNode = (VHGNODE*) VHGDEREF(graph, child->item);
		CHECK_VHGNODE_MAGIC(childNode);

		if ( !(childNode->flags & V_frozen) ) {
			
			/*
			 *	Child is a working version, so return TRUE
			 */
			return TRUE;
		}

		/*
		 *	Get next child
		 */
		child = (VHGNODELISTELEMENT*) VHGDEREF(graph, child->succ);
		CHECK_VHGLIST_MAGIC(child);
	}

	/*
	 *	At end of list, so no working children
	 */
	return FALSE;
}
