/*
 *   $RCSfile: errorcodes.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:11 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include <stdio.h> 
#include "ess.h"
#include "error.h"

ERRORBASEINFO ErrorBaseInfo[] =
{
 UNIX_ERROR_BASE,		MAX_UNIX_ERROR,		"UNIX_ERROR_BASE",		"unix error",
 GENERAL_ERROR_BASE,	MAX_GENERAL_ERROR,	"GENERAL_ERROR_BASE",	"general error",
 SM_ERROR_BASE,			MAX_SM_ERROR,		"SM_ERROR_BASE",		"sm layer error",
 BF_ERROR_BASE,			MAX_BF_ERROR, 		"BF_ERROR_BASE",		"buffer manager error",
 LG_ERROR_BASE,			MAX_LG_ERROR,		"LG_ERROR_BASE",		"large object subsystem error",
 EH_ERROR_BASE,			MAX_EH_ERROR,		"EH_ERROR_BASE",		"error handler error",
 FI_ERROR_BASE,			MAX_FILE_ERROR,		"FI_ERROR_BASE",		"file subsystem error",
 IO_ERROR_BASE,			MAX_IO_ERROR,		"IO_ERROR_BASE",		"io subsystem error ",
 MSG_ERROR_BASE,		MAX_MSG_ERROR,		"MSG_ERROR_BASE",		"message subsystem error",
 THREAD_ERROR_BASE,		MAX_THREAD_ERROR,	"THREAD_ERROR_BASE",	"thread manager error",
 BM_ERROR_BASE,			MAX_BM_ERROR,		"BM_ERROR_BASE",		"bitmap subsystem error",
 TRANS_ERROR_BASE,		MAX_TRANS_ERROR,	"TRANS_ERROR_BASE",		"transaction manager error",
 LOCK_ERROR_BASE,		MAX_LOCK_ERROR,		"LOCK_ERROR_BASE",		"lock manager error",
 LOG_ERROR_BASE,		MAX_LOG_ERROR,		"LOG_ERROR_BASE",		"log manager error",
 RESOURCE_ERROR_BASE,	MAX_RESOURCE_ERROR,	"RESOURCE_ERROR_BASE",	"resource manager error",
 RECOVER_ERROR_BASE,	MAX_RECOVER_ERROR,	"RECOVER_ERROR_BASE",	"recovery manager error",
 IM_ERROR_BASE,			MAX_IM_ERROR,		"IM_ERROR_BASE",		"index manager error",
 OPT_ERROR_BASE,		MAX_OPT_ERROR,		"OPT_ERROR_BASE",		"option related errors",
 DISTR_ERROR_BASE,		MAX_DISTR_ERROR,		"DIST_ERROR_BASE",		"2PC=related errors"
};


ERRORINFO	*ErrorBase[MAX_ERROR_BASE];
BOOL	  	ErrorBaseInit = FALSE;

ERRORINFO UNIX_ErrorInfo[] =
{
 UNIX_ERROR_BASE,	"UNIX_ERROR_BASE",		"invalid error code"
};


ERRORINFO GENERAL_ErrorInfo[] =
{
	
 GENERAL_ERROR_BASE,	"GENERAL_ERROR_BASE",	"invalid error code",
 esmINTERNAL,			"esmINTERNAL",			"internal error",
 esmASSERT,				"esmASSERT",			"assert failed - internal error",
 esmMALLOCFAILED,		"esmMALLOCFAILED",		"out of memory - malloc failed",
 esmSERVERDIED,			"esmSERVERDIED",		"server death detected",
 esmNOTIMPLEMENTED,		"esmNOTIMPLEMENTED",	"feature is not implemented",
 esmUNIXSIGNAL,			"esmUNIXSIGNAL",		"Unix signal -- see sigvec(2) for more info",
 esmPOOLEMPTY,			"esmPOOLEMPTY",			"ran out of items in pool"

};

ERRORINFO SM_ErrorInfo[] =
{
	
 SM_ERROR_BASE,			"SM_ERROR_BASE",		"invalid error code",
 esmBADOID,				"esmBADOID",			"invalid object id",
 esmBADSTART,			"esmBADSTART",			"invalid object start location",
 esmNOFREEUSERDESC,		"esmNOFREEUSERDESC",	"out of free user descriptors",
 esmBADUSERDESC,		"esmBADUSERDESC",		"invalid user descriptor",
 esmBADPARAMS,			"esmBADPARAMS",			"invalid input parameters",
 esmBADGROUP,			"esmBADGROUP",			"invalid buffer group",
 esmBADLENGTH,			"esmBADLENGTH",			"invalid length for object or root entry",
 esmROOTNAMETOOLONG,	"esmROOTNAMETOOLONG",	"root name length too long",
 esmNOFREESCANDESC,		"esmNOFREESCANDESC",	"out of free scan descriptors",
 esmENDOFOBJECT,		"esmENDOFOBJECT",		"end of object",
 esmBADSLOTTEDMAGIC,	"esmBADSLOTTEDMAGIC",	"corrupt slotted page",
 esmBADOBJECTMAGIC,		"esmBADOBJECTMAGIC",	"corrupt object",
 esmBADOBJECTFLAGS,		"esmBADOBJECTFLAGS",	"invalid object flags",
 esmNOFREELOADDESC,		"esmNOFREELOADDESC",	"out of free load descriptors",
 esmHISTORYGRAPHOID,	"esmHISTORYGRAPHOID",	"illegal attempt to reference a history graph object",
 esmNOTVERSIONED,		"esmNOTVERSIONED",		"object is not versioned",
 esmNOTFROZEN,			"esmNOTFROZEN",			"object is not frozen",
 esmFROZEN,				"esmFROZEN",			"object is frozen",
 esmVHGTOOLARGE,		"esmVHGTOOLARGE",		"version history graph too large"
};


ERRORINFO BF_ErrorInfo[] =
{

 BF_ERROR_BASE,			"BF_ERROR_BASE",		"invalid error code",
 esmCANTSWAP,			"esmCANTSWAP",			"too many fixed buffers in buffer group",
 esmFIXEDPAGES,			"esmFIXEDPAGES",		"some pages are still fixed",
 esmNOFREEPAGEHASH,		"esmNOFREEPAGEHASH",	"out of page hash structures",
 esmNOUNRESERVED,		"esmNOUNRESERVED",		"insufficient unreserved buffers",
 esmNOFREEGROUPS,		"esmNOFREEGROUPS",		"out of buffer groups",
 esmBADREPLACEPOLICY,	"esmBADREPLACEPOLICY",	"invalid replacement policy",
 esmPAGEGONE,			"esmPAGEGONE",			"page is no longer in the buffer pool",
 esmNOBUFSPACE,			"esmNOBUFSPACE",		"insufficient contiguous buffer space",
 esmREQUESTTOOBIG,		"esmREQUESTTOOBIG",		"buffer group size request too big",
 esmGROUPGONE,			"esmGROUPGONE",			"buffer group has been closed by another thread",
 esmNOSHMSPACE,			"esmNOSHMSPACE",		""
};


ERRORINFO LG_ErrorInfo[] =
{

 LG_ERROR_BASE,			"LG_ERROR_BASE",		"invalid error code",
 esmBADRANGE,			"esmBADRANGE",			"",
 esmNOFREELGNODELIST,	"esmNOFREELGNODELIST",	"out of large node list elements"
};


ERRORINFO EH_ErrorInfo[] =
{

 EH_ERROR_BASE,		"EH_ERROR_BASE",		"invalid error code"
};


ERRORINFO FI_ErrorInfo[] =
{
	
 FI_ERROR_BASE,		"FI_ERROR_BASE",		"invalid error code",
 esmEMPTYFILE,			"esmEMPTYFILE",			"file is empty",
 esmBADNEARTYPE,		"esmBADNEARTYPE",		"illegal near hint type",
 esmBADFILEPAGEMAGIC,	"esmBADFILEPAGEMAGIC",	"corrupt file page",
 esmAFTERLASTPID,		"esmAFTERLASTPID",		"beyond end of file",
 esmBEFOREFIRSTPID,		"esmBEFOREFIRSTPID",	"before beginning of file",
 esmFILESTACKOVERFLOW,	"esmFILESTACKOVERFLOW",	"file stack overflow",
 esmFILEPAGENOTFOUND,	"esmFILEPAGENOTFOUND",	"file page not found",
 esmNOFREEFILESTACK,	"esmNOFREEFILESTACK",	"no free file stack",
 esmPAGEEXISTS,			"esmPAGEEXISTS",		"page already exists",
 esmPAGEMARKED,			"esmPAGEMARKED",		"page already marked",
 esmBADFID,				"esmBADFID",			"invalid file id",
 esmNOFREEFIDHASH,		"esmNOFREEFIDHASH",		"",
 esmFILENOTLOGICAL,		"esmFILENOTLOGICAL",	"file not in logical order",
 esmBADPAGESIZE,		"esmBADPAGESIZE",		"incorrect page size",
 esmBADFILEHEADERMAGIC,	"esmBADFILEHEADERMAGIC","corrupt file header",
 esmNOFREEOPENFILE,		"esmNOFREEOPENFILE",	"open file limit exceeded"
};


ERRORINFO IO_ErrorInfo[] =
{

 IO_ERROR_BASE,			"IO_ERROR_BASE",		"invalid error code",
 esmBADVOLHEADER,		"esmBADVOLHEADER",		"corrupt volume header",
 esmDUPVOLID,			"esmDUPVOLID",			
	"duplicate volume IDs with different volume names",
 esmDISKPROCDIED,		"esmDISKPROCDIED",		"disk process died",
 esmBADVOLID,			"esmBADVOLID",			"invalid volume id",
 esmDUPVOLNAME,			"esmDUPVOLNAME",		
	"duplicate volume names in mount table",
 esmFRAGMENTED,			"esmFRAGMENTED",		"volume is too fragmented",
 esmDISKFULL,			"esmDISKFULL",			"volume is full",
 esmDISKMOUNTED,		"esmDISKMOUNTED",		"volume with this id is mounted",
 esmBADROOTNAME,		"esmBADROOTNAME",		"root name does not exist",
 esmROOTTABLEFULL,		"esmROOTTABLEFULL",		"out of root entry space",
 esmNOVOLINFO,			"esmNOVOLINFO",			"out of volume information space",
 esmNOFREECONTEXT,		"esmNOFREECONTEXT",		"out of bitmap page contexts",
 esmBADPID,				"esmBADPID",			"invalid page id",
 esmSERVPROCDIED,		"esmSERVPROCDIED",		"server process died",
 esmNOSUCHVOLUME,		"esmNOSUCHVOLUME",		"no such volume to mount",
 esmMOUNTTABLEFULL,		"esmMOUNTTABLEFULL",	"max # volumes already mounted"
};


ERRORINFO MSG_ErrorInfo[] =
{

 MSG_ERROR_BASE,		"MSG_ERROR_BASE",		"invalid error code",
 esmPROTOCOLVERSION,	"esmPROTOCOLVERSION",	
							"client, server have different protocol versions",
 esmBADMESSAGENUMBER,	"esmBADMESSAGENUMBER",	"invalid message number",
 esmBADMESSAGEMAGIC,	"esmBADMESSAGEMAGIC",	"client-server protocol error",
 esmALREADYCONNECTED,	"esmALREADYCONNECTED",	"already connected to server",
 esmNOTCONNECTED,		"esmNOTCONNECTED",		"not connected to server",
 esmNOFREETIMERS,		"esmNOFREETIMERS",		"ran out of timer structures",
 esmUNKNOWNHOSTNAME,	"esmUNKNOWNHOSTNAME",	"unknown host (machine) name",
 esmCONFIGMISMATCH,		"esmCONFIGMISMATCH",	
							 "client and server are incompatible",
 esmBADFAMILY,		"esmBADFAMILY",		"address received not AF_INET"
};


ERRORINFO THREAD_ErrorInfo[] =
{

 THREAD_ERROR_BASE,		"THREAD_ERROR_BASE",	"invalid error code",
 esmNOFREETHREAD,		"esmNOFREETHREAD",		"no free threads"
};


ERRORINFO BM_ErrorInfo[] =
{

 BM_ERROR_BASE,			"BM_ERROR_BASE",		"invalid error code",
 esmNOFREEPAGECONTEXT,	"esmNOFREEPAGECONTEXT",	"out of bitmap page contexts",
 esmTOOMANYBITS,		"esmTOOMANYBITS",		"too many free pages requested"
};


ERRORINFO TRANS_ErrorInfo[] =
{

 TRANS_ERROR_BASE,		"TRANS_ERROR_BASE",		"invalid error code",
 esmNOFREETRANSREC,		"esmNOFREETRANSREC",	"out of transaction records",
 esmTRANSDISABLED,		"esmTRANSDISABLED",		"transaction processing is disabled",
 esmBADTRANSID,			"esmBADTRANSID",		"invalid transaction id",
 esmNOTTRANSMASTER,		"esmNOTTRANSMASTER",	"client socket is not the transaction master, so commit or abort reqeust is rejected",
 esmTRANSABORTED,		"esmTRANSABORTED",		"transaction is aborted",
 esmTRANSCOMMITTED,		"esmTRANSCOMMITTED",	"transaction is commited",
 esmNOFREETRANSVOLREC,	"esmNOFREETRANSVOLREC",	"out of transcation volume records",
 esmTRANSINPROGRESS,	"esmTRANSINPROGRESS",	"transaction already in progress -- nested transactions not supported",
 esmNOACTIVETRANS,		"esmNOACTIVETRANS",		"no transaction is active",
 esmCLIENTREQUEST,		"esmCLIENTREQUEST",		"at the client's request"
};


ERRORINFO LM_ErrorInfo[] =
{

 LOCK_ERROR_BASE,		"LOCK_ERROR_BASE",		"invalid error code",
 esmBADLOCKTABLESIZE,	"esmBADLOCKTABLESIZE",	"bad lock table size",
 esmBADLOCKMODE,		"esmBADLOCKMODE",		"invalid lock mode",
 esmNOFREELOCKHEADER,	"esmNOFREELOCKHEADER",	"out of lock headers",
 esmNOFREELOCKENTRY,	"esmNOFREELOCKENTRY",	"out of lock entries",
 esmNOFREELOCKCLASSREC,	"esmNOFREELOCKCLASSREC","",
 esmNOFREELOCKCLASS,	"esmNOFREELOCKCLASS",	"",
 esmBADLOCKCLASS,		"esmBADLOCKCLASS",		"invalid lock class",
 esmLOCKBUSY,			"esmLOCKBUSY",			"lock is busy - already held by another transaction",
 esmLOCKCAUSEDDEADLOCK,	"esmLOCKCAUSEDDEADLOCK","lock reqeust causes deadlock",
 esmNOSUCHLOCK,			"esmNOSUCHLOCK",		"no such lock",
 esmWAITEREXISTS,		"esmWAITEREXISTS",		"a transaction is waiting for the lock",
 esmPHANTOMDEADLOCK,	"esmPHANTOMDEADLOCK",	"lock request causes phantom deadlock",
 esmCAUSEDPHANTOMDEADLOCK, "esmCAUSEDPHANTOMDEADLOCK", "lock request generates phantom deadlock",
 esmLOCKPREEMPTED, "esmLOCKPREEMPTED", "lock was taken by another transaction"
};


ERRORINFO LOG_ErrorInfo[] =
{

 LOG_ERROR_BASE,		"LOG_ERROR_BASE",		"invalid error code",
 esmLOGDISABLED,		"esmLOGDISABLED",		"logging is disabled",
 esmLOGWRAPPED,			"esmLOGWRAPPED",		"the log has wrapped",
 esmNOFREELOGSPACE,		"esmNOFREELOGSPACE",	"no free log space",
 esmLOGRECORDTOOBIG,	"esmLOGRECORDTOOBIG",	"log record is too large",
 esmBADLOGPAGEHEADER,	"esmBADLOGPAGEHEADER",	"corrupt log page header",
 esmBADLOGRECORDHEADER,	"esmBADLOGRECORDHEADER","corrupt log record header",
 esmBADLOGVOLUME,		"esmBADLOGVOLUME",		"volume does not have a log",
 esmMOUNTLSNTOOHIGH,	"esmMOUNTLSNTOOHIGH",	"LSN on volume is too high to allow mounting",
 esmLOGTOOSMALL,		"esmLOGTOOSMALL",		"log space on volume is too small",
 esmLOGGINGOFF,			"esmLOGGINGOFF",		"transaction has logging turned off",
 };


ERRORINFO RESOURCE_ErrorInfo[] =
{

 RESOURCE_ERROR_BASE,	"RESOURCE_ERROR_BASE",	"invalid error code",
 esmBADRESOURCE,		"esmBADRESOURCE",		"invalid resource",

 };


ERRORINFO RECOVER_ErrorInfo[] =
{
 RECOVER_ERROR_BASE,	"RECOVER_ERROR_BASE",	"invalid error code",
 esmCANNOTRECOVER,		"esmCANNOTRECOVER", 	"cannot recover"
};


ERRORINFO IM_ErrorInfo[] =
{

 IM_ERROR_BASE,			"IM_ERROR_BASE",		"invalid error code",
 esmKEYNOTFOUND,		"esmKEYNOTFOUND",		"index key not found",
 esmKEYALREADYEXISTS,	"esmKEYALREADYEXISTS",	"index key already exists",
 esmDUPLICATEKEYPID,	"esmDUPLICATEKEYPID",	"duplicate key/page pair",
 esmKEYTOOLONG,			"esmKEYTOOLONG",		"index key too long",
 esmMAXKEYLENEXCEEDED,	"esmMAXKEYLENEXCEEDED",	"maximum index key length exceeded",
 esmOIDNOTFOUND,		"esmOIDNOTFOUND",		"oid not found",
 esmOIDEXISTS,			"esmOIDEXISTS",			"oid already exists",
 esmINVALIDDIRECTION,	"esmINVALIDDIRECTION",	"invalid index scan direction",
 esmSCANEXHAUST,		"esmSCANEXHAUST",		"index scan exhausted",
 esmINVALIDCURSOR,		"esmINVALIDCURSOR",		"invalid index cursor",
 esmINVALIDCOND,		"esmINVALIDCOND",		"invalid index condition",
 esmCONFLICTCOND,		"esmCONFLICTCOND",		"index search conditions conflict",
 esmBULKLOADING,        "esmBULKLOADING",       "index is (is not) bulkloading",
 esmINDEXNOTEMPTY,      "esmINDEXNOTEMPTY",     "index not empty",
 esmNEEDTEMPVOL,        "esmNEEDTEMPVOL",       "temporary volume needed",
 esmWRONGNDXTYPE,       "esmWRONGNDXTYPE",      "wrong index type",
 esmWRONGHASHLOAD,      "esmWRONGHASHLOAD",     "wrong hash load",
};


ERRORINFO OPT_ErrorInfo[] =
{

 OPT_ERROR_BASE,		"OPT_ERROR_BASE",		"invalid error code",
 esmBADOPTION,			"esmBADOPTION",			"invalid option name",
 esmOPTIONSYNTAX,		"esmOPTIONSYNTAX",		"bad syntax in option configuration file",
 esmOPTIONNOTUNIQUE,	"esmOPTIONNOTUNIQUE",	"option name is not unique",
 esmOPTIONNOTSET,		"esmOPTIONNOTSET",		"required configuration option is not set",
 esmBADOPTIONCLASS,		"esmBADOPTIONCLASS",	"invalid option class name",
 esmBADOPTIONVALUE,		"esmBADOPTIONVALUE",	"illegal or missing option value",
 esmBUFPAGESNOTSET,		"esmBUFPAGESNOTSET",	"bufpages option is not set",
};

ERRORINFO DISTR_ErrorInfo[] =
{
 DISTR_ERROR_BASE,		"DIST_ERROR_BASE",		"invalid error code",
 esmTRANSNOTPREPARED,	"esmTRANSNOTPREPARED",	"transaction not prepared",
 esmBADCOMMAND,	"esmBADCOMMAND",	"invalid command - should be C_COMMIT or C_ABORT",
 esmNOFREEGTIDREC,	"esmNOFREEGTIDREC",	"out of GTIDRECs",
 esmTOOMANYSERVERS,	"esmTOOMANYSERVERS", "coord cannot handle more than MAXPARTICIPANTS servers per transaction",
 esmNOFREESERVERTIDINFO, "esmNOFREESERVERTIDINFO", "out of SERVERTIDINFOs",
 esmBADHANDLE, "esmBADHANDLE", "cannot find server specified in handle",
 esmWOULDBLOCK,	"esmWOULDBLOCK", "connect to server will block ",
 esmBADVOTE, "esmBADVOTE", "invalid vote received",
 esmBADSTATE, "esmBADSTATE", "transaction in invalid state",
 esmCOORDUNKNOWN, "esmCOORDUNKNOWN", "no known coordinator",
 esmTRANSPREPARED, "esmTRANSPREPARED", "transaction in prepared state",
 esmTRANSIN2PC, "esmTRANSIN2PC", "transaction already in 2 phase commit",
 esmTRANSIN2PC, "esmTRANSNOTIN2PC", "transaction not in 2 phase commit"
};


