/*
 *   $RCSfile: list1.c,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 1996/05/04 21:55:11 $
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "checking.h"
#include "ess.h"
#include "list.h"
#include "error.h"
#include "trace.h"


 void
initializeList (

	register LIST	*head 

)
{
	TRPRINT(TR_LIST, TR_LEVEL_1, ("head:%x", head));
	INITIALIZELIST(head);
}

 void
initializeListElement (
	register LIST	*element,
	void			*item 

)
{
	TRACE(TR_LIST, TR_LEVEL_1);
	INITIALIZELISTELEMENT(element, item);
}

 void
*listDeq (

	register LIST			*head 
)
{
	register LISTELEMENT	*element;

	LISTDEQ( head, element);

	if(element == NULL)
		return (void *)NULL;
	return( element->item );
}

 void
listEnq (

	register LIST	*head,
	register LIST	*element 

)
{
	LISTENQ(head,element);
}

 void
listMoveEnq (

	register LIST			*head,
	register LISTELEMENT	*element 

)
{

	TRPRINT(TR_LIST, TR_LEVEL_1, ("head:%x element:%x", head, element));

	CHECK_LIST_MAGIC(head);

	CHECK_LIST_MAGIC(element);

	/*
	 *	Consistency check for element
	 */
	if (element->succ == NULL)	{
		
		/*
		 *	there were no elements
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);

	} else {

		/*
		 *	Remove the element from the list
		 */
		element->succ->pred = element->pred;
		element->pred->succ = element->succ;

		/*
		 *	Put the item last on the list
		 */
		element->succ 	   = head;
		element->pred 	   = head->pred;
		element->pred->succ =element;
		head->pred		   = element;
	}
}

 void
listMovePush (

	register LIST			*head,
	register LISTELEMENT	*element 

)
{
	LISTMOVEPUSH(head,element);
}

 void
listPush (

	register LIST	*head,
	register LIST	*element 

)
{
	LISTPUSH(head,element);	
}

 void
listRemove (

	register LIST	*element 
)
{
	LISTREMOVE(element);
}
