/*
 *   $RCSfile: list2.c,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 1996/05/04 21:55:11 $
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "checking.h"
#include "ess.h"
#include "list.h"
#include "error.h"
#include "trace.h"

 void
listAppend (

	register LIST	*destHead,
	register LIST	*sourceHead 

)
{

	TRPRINT(TR_LIST, TR_LEVEL_1, ("destHead:%x sourceHead:%x", destHead, sourceHead));

	CHECK_LIST_MAGIC(destHead);
	CHECK_LIST_MAGIC(sourceHead);

	/*
	 *	check to see if the source is empty
	 */
	if (LIST_NOT_EMPTY(sourceHead))	{

		/*
		 *	setup the new chain
		 */
		sourceHead->succ->pred	= destHead->pred;
		sourceHead->pred->succ  = destHead;
		destHead->pred->succ	= sourceHead->succ;
		destHead->pred			= sourceHead->pred;
		INITIALIZELIST(sourceHead);
	}
}

 void
listInsertAfter (

	register LISTELEMENT	*current,
	register LISTELEMENT	*newElement 

)
{

	TRPRINT(TR_LIST, TR_LEVEL_1, ("current:%x new:%x", current, newElement));

	/*
	 *	Check to see if the magic number is valid
	 */
	CHECK_LIST_MAGIC(current);

	/*
	 *	Check to see if the magic number is valid
	 */
	CHECK_LIST_MAGIC(newElement);

	/*
	 *	Consistency check for element
	 */
	if (newElement->succ != NULL)	{
		
		/*
		 *	Just return null pointer indicating that 
		 *	there were no elements
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);

	} else {

		/*
		 *	Put the new item in the list before the current
		 */
		newElement->pred	= current;
		newElement->succ	= current->succ;
		current->succ->pred = newElement;
		current->succ		= newElement;
	}
}

 void
listInsertBefore (

	register LISTELEMENT	*current,
	register LISTELEMENT	*newElement 

)
{

	TRPRINT(TR_LIST, TR_LEVEL_1, ("current:%x new:%x", current, newElement));

	/*
	 *	Check to see if the magic number is valid
	 */
	CHECK_LIST_MAGIC(current);

	/*
	 *	Check to see if the magic number is valid
	 */
	CHECK_LIST_MAGIC(newElement);

	/*
	 *	Consistency check for element
	 */
	if (newElement->succ != NULL)	{
		
		/*
		 *	Just return null pointer indicating that 
		 *	there were no elements
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);

	} else {

		/*
		 *	Put the new item in the list before the current
		 */
		newElement->pred	= current->pred;
		newElement->succ	= current;
		current->pred->succ = newElement;
		current->pred		= newElement;
	}
}

int
listLength(LIST *list)
{
	register LISTELEMENT *l = NULL;
	register int i;

	for(i=0,l = FIRST_LIST_ELEMENT_ITSELF(list); 
		l != list; 
		i++, l = NEXT_LIST_ELEMENT_ITSELF(l));
	return i;
}
