/*
 *   $RCSfile: option_c_interface.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:11 $      
 */ 

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*    or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "io.h"
#include "option.h"
#include "error.h"

int	processOptionFile(char* optFile, OptionList* optList, BOOL overRide, char** errorString)
{
	int	errorFound;
	OptionFileScan optFileScan(optFile, optList);

	errorFound = optFileScan.scan(overRide, *errorString);

	/*
	 *	Convert the error into a storage manager error
	 */
	switch(errorFound) {
	case OPTERR_Success:
		errorFound = esmNOERROR;
		break;
	case OPTERR_NoMatch:
		errorFound = esmBADOPTION;
		break;
	case OPTERR_Duplicate:
		errorFound = esmOPTIONNOTUNIQUE;
		break;
	case OPTERR_Syntax:
		errorFound = esmOPTIONSYNTAX;
		break;
	case OPTERR_IllegalClass:
		errorFound = esmBADOPTIONCLASS;
		break;
	default:
		if (errorFound > 0) {
			/*
			 *	the error was a unix file access error or
			 *	a callback function error
			 */
		} else {
			errorFound = esmOPTIONSYNTAX;
		}
		break;
	}
	if (errorFound) SM_ERROR(TYPE_USER, errorFound);


	/*
	 *	it is critical to return errorFound and not just 
	 *	esmFAILURE since at server start up there is no place 
	 *	for SM_ERROR to store the error code
	 */
	return(errorFound);
}


BOOL checkRequiredOptions(OptionList* optList, FILE* errfile)
{
	BOOL		err = FALSE;
	OptionInfo*	opt;	
	
	optList->initScan();

	for (opt = optList->getNext(); opt != NULL; opt = optList->getNext()) {
		if (opt->required() && !opt->set()) {
			err = TRUE;
			fprintf(errfile, "Usage error: option <%s> is required but is not set\n", opt->name());
		}
	}

	return err;
}



