/*
 *   $RCSfile: poolDeq.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:11 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include <stdio.h>
#include "checking.h"
#include "ess.h"
#include "list.h"
#include "pool.h"
#include "trace.h"
#include "error.h"


 void
*poolDeq (

	register POOL	*head 
)
{

	register LIST	*element;
	register char	*structure;


	TRPRINT(TR_LIST, TR_LEVEL_1, ("head:%x", head));

	/*
	 *	check the pool magic
	 */
	CHECK_POOL_MAGIC(head);

	/*
	 *	Check to see if there is anything on the list
	 */
	if (LIST_EMPTY( &(head->freeList)) )	{

		/*
		 *	check to make sure there is an allocation routine
		 */
		if (head->allocMore == NULL)	{

			/*
			 *	just return NULL
			 */
			return(NULL);

		} else {
			
			/*
			 *	Just return null pointer indicating that 
			 *	there were no elements
			 */
			if ((structure = head->allocMore(head)) == NULL)	{

				return(NULL);

			} else {

				/*
				 *	update the usage statistics
				 */
				head->currentUsed++;

				/*
				 *	check to see if this is a new limit
				 */
				if (head->currentUsed > head->maxUsed)	{

					head->maxUsed = head->currentUsed;
				}

				/*
				 *	return the pointer
				 */
				return( (void *) structure );
			}
		}

	} else {

		/*
		 *	Take the first element off the list
		 */
		if ((element = (LIST *) listDeq( &(head->freeList) )) == NULL)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *	update the usage statistics
		 */
		head->currentUsed++;

		/*
		 *	check to see if this is a new limit
		 */
		if (head->currentUsed > head->maxUsed)	{

			head->maxUsed = head->currentUsed;
		}

		/*
		 *	Return the item pointer
		 */
		TRPRINT(TR_LIST, TR_LEVEL_2, ("element:%x", element));
		return( (void *) element );
	}
}
