/*
 *   $RCSfile: SM_Error.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:07 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "queue_consist.h"
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "error.h"
#include "tid.h"
#include "pool.h"
#include "io.h"
#include "bitvec.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "disk.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "bf.h"
#include "link.h"
#include "volume.h"
#include "trace.h"
#include "msgvector.h"

#include "disk_funcs.h"
#include "queues.h"
#include "diskproc_globals.h"
#include "diskproc_intfuncs.h"
#include "diskproc_extfuncs.h"

 void
SM_Error (

	int		type,
	int		code,
	char	*file,
	int		line
)
{

	char	*pointer;

	TRPRINT(TR_DISKRW, TR_LEVEL_1, ("type %d code %d, file %s line %d", 
		type, code, file, line));
	
	if(type != TYPE_QUIET) {
		fprintf(stderr, "DISKRW file: %-22s line:%d error %s -- %s\n", 
			file, line, errorIdString(code), errorCodeString(code));
	}

	if((type == TYPE_STOP) || (type == TYPE_QUIET)) {
		fprintf(stderr, "DISKRW %d exiting.\n", getpid());
		onexit(1);
	}

	if (type == TYPE_FATAL || type == TYPE_CRASH)   {
		/*
		 *	reference the null pointer
		 */
		fprintf(stderr, "DISKRW %d aborting; creating core file...\n", 
			getpid());
#ifdef notdef
		unCatchAbortSignal(SIGILL);
		abort();
#else
		unCatchAbortSignal(SIGSEGV);
		pointer = NULL;
		*pointer = 0;
#endif notdef
	}
}
