/*
 *   $RCSfile: openLocalDisk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:07 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include <sys/file.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "disk.h"
#include "queues.h"
#include "diskproc_globals.h"
#include "diskproc_intfuncs.h"
#include "diskproc_extfuncs.h"


extern int		errno;
extern int		Fd;


 void
openLocalDisk (
	DISKMSG	*message,
	char 	*diskName
)
{

	FLAGS			flags;

	
	TRPRINT(TR_DISKRW, TR_LEVEL_1, ("opening disk:%s", diskName));

	/*
	 *	compose the flags field
	 */
	flags = O_RDWR|message->body.flags;

	/*
	 *	attempt to open the disk
	 */
	if ((Fd = open(diskName, flags, 0604)) < 0)	{

		TRPRINT(TR_DISKRW, TR_LEVEL_2, ("open failed:%d", errno));

		/*
		 *	reply in the negative and die
		 */
		if (errno == 0) SM_ERROR(TYPE_FATAL, esmINTERNAL);
		message->header.params.out.errno = errno;
		message->diskmagic = DISK_MESSAGE_MAGIC;
		replyDiskMessage(message);
#ifdef 	DISKPROC_MAKE
		/* 
		 * If we're the disk process, die.
		 * Error is not TYPE_FATAL because it could very well be a user error,
		 * such as nonexistent file name.  Don't want to dump core
		 * for that.
		 */
		SM_ERROR(TYPE_STOP, errno);
#endif 	DISKPROC_MAKE

	} else {

		TRPRINT(TR_DISKRW, TR_LEVEL_2, ("open successful fd:%d", Fd));

#ifndef	DISKPROC_MAKE
		/*
		 *	Get an exclusive Unix lock on the file so that no
		 *	other server or formatvol will use the volume
		 */

#endif	/*DISKPROC_MAKE*/

		/*
		 *	reply in the positive
		 */
		message->header.params.out.errno = esmNOERROR;
		message->diskmagic = DISK_MESSAGE_MAGIC;
		replyDiskMessage(message);
	}
}
