/*
 *   $RCSfile: formatvol.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:07 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "io.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bitvec.h"
#include "util_funcs.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "init_funcs.h"
#include "disk.h"
#include "disk_funcs.h"
#include "thread_funcs.h"
#include "msg_funcs.h"
#include "server_main_funcs.h"
#include "trace_funcs.h"
#include "sharedmem_globals.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "disk_globals.h"
#include "io_intfuncs.h"
#include "io_globals.h"
#include "msg_globals.h"
#include "sm_params.h"
#include "queue_consist.h"
#include "queue_globals.h"


extern FILE		*sm_ErrorStream;
extern void 	processCommandLine( int , char** );

extern int formatIt(VOLNAMEREC *);
extern void displayIt(VOLNAMEREC *);

static BOOL
doit(VOLNAMEREC *volNameRec)
{
	if( volNameRec->fmtinfo->volflags & VOL_DISPLAY )  {
		(void) displayIt(volNameRec);
	}
	if( volNameRec->fmtinfo->volflags & VOL_FORMAT )  {
		if(formatIt(volNameRec) == esmFAILURE) {
			fprintf(sm_ErrorStream, "FORMATVOL error: %s\n", errorCodeString(Active->errno));
			fprintf(sm_ErrorStream, "FORMATVOL Could not format volume: %s\n", volNameRec->volName);
			return TRUE; /* quit right now */
		}
	}
	return FALSE;
}

main(
	int		argc,
	char	**argv
)
{
#ifdef IPS_SUPPORT
	IPSuareaReadable = 0;
#endif

	Active = NULL;
	formatvolRunning = TRUE; /* see io/space.c */
	FastPathOnly	 = TRUE; /* see io/space.c */

	sm_ErrorStream = stderr;
	/*
	 * map error codes to string messages
	 */
	initErrorCodes();

	/*TODO: display option! */
	NumBuffers = 40; /* bufpages option isn't required for formatvol */
	NumThreads = 5; /* maxthreads option isn't required for formatvol */

	initializeVolNameTable();
	processCommandLine(argc, argv);

	/* kludge to avoid constraints & error msgs that are
	 * pertinent only to the server:
	 */
	MaxAddVolumes = 1;

	/* from this point on, we must exit by using admin_ShutServer() */
	initializeServer();

	if ((UserBufGroup = bf_OpenGroup(20, BF_LRU, NOFLAGS)) == NULL)  {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	
	/*
	 * go through all the volNameRecs and if VOL_FORMAT is in the 
	 * properties, format the volume
	 */
	ForEachVolumeNameRec( 1, doit, NULL, NULL, NULL);

	exit(0);
}
