/*
 *   $RCSfile: processCommandLine.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:07 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include <sys/stat.h>
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "bitvec.h"
#include "link.h"
#include "msgvector.h"
#include "bf.h"
#include "server_options.h"
#include "option.h"
#include "volume.h"
#include "lsn.h"
#include "openlog.h"
#include "init_funcs.h"
#include "server_main_funcs.h"
#include "util_funcs.h"
#include "bf_globals.h"
#include "cl_globals.h"
#include "disk_globals.h"
#include "io_globals.h"
#include "io_extfuncs.h"
#include "log_globals.h"
#include "msg_globals.h"
#include "recover_globals.h"
#include "sharedmem_globals.h"
#include "thread_globals.h"
#include "server_util_funcs.h"

#include <sys/stat.h>

/* from config/configOptions.c */
extern char	*SM_ConfigFileName;
extern char	*SM_DiskProcessName;
extern void usage(char*, BOOL);

#ifdef FORMATVOL_MAKE
	/* parseVolume is used for FMT_OPT_volume and FMT_OPT_display */
extern int parseVolume(OptionInfo* opt, const char* value); 
#endif FORMATVOL_MAKE
extern int parseFormat(OptionInfo* opt, const char* value);


OptionList*	ServerOptions = NULL;


#define SM_CONFIG "sm_config"

int callBack(OptionInfo* opt, const char* value)
{
	switch (opt->ID()) {

#ifdef FORMATVOL_MAKE
	case FMT_OPT_volume:
	case FMT_OPT_display:
		if (parseVolume(opt, value) == esmFAILURE) 
			exit(1);
		else return esmNOERROR;
#endif FORMATVOL_MAKE

	case FMT_OPT_data:
	case FMT_OPT_temp:
	case FMT_OPT_log:
		return parseFormat(opt, value);

	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return esmFAILURE;
	}
}


extern "C" void initializeTrace(void);

 void
processCommandLine(
	int argc,
	char** argv
)
{
	int 		i;
	int			err;
	char*		errMsg;
	OptionInfo*	opt = NULL;
	BOOL		verbose = FALSE;
	BOOL		force = FALSE;
	FILE*		more;
	int			startArg;
	BOOL		skipDefault = FALSE;
	char		configFile[_POSIX_PATH_MAX+1];
    char        *progName;


	/* remember only the file name of the program (not the path) */
	progName = strrchr(argv[0], '/');
	if (progName == NULL) {
		progName = argv[0];
	} else {
		progName += 1; /* skip the '/' */
		if (progName[0] == '\0')  {
			progName = argv[0];
		}
	}

	ServerOptions = new OptionList(32); // 32 == number of options

	// Set the classification hierarchy for naming options.
	// The hierarchy is: server.progName.optionName
#ifdef FORMATVOL_MAKE
	ServerOptions->addClassLevel("formatvol");		// program type 
#else
	ServerOptions->addClassLevel("server");		// program type 
#endif
	ServerOptions->addClassLevel(progName);		// program name

	// Define each option
	ServerOptions->addOpt()->init(SM_OPT_help, "help",
						NULL, NULL, 
						"print a help message", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_skipdefault, "skipdefault",
						NULL, NULL, 
						"skip reading the default configuration files", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_force, "force",
						NULL, NULL, 
						"turn off confirmation requests", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_config, "config",
						"file name", NULL, 
						"read a configuration file", FALSE, NULL);
#ifndef FORMATVOL_MAKE
	ServerOptions->addOpt()->init(SM_OPT_background, "background",
						NULL, NULL,
						"server is to run in the background", FALSE, NULL);
#endif
	ServerOptions->addOpt()->init(SM_OPT_verbose, "verbose",
						"yes no", "no", 
						"print configuration options on startup", FALSE, NULL);

#ifndef FORMATVOL_MAKE
	ServerOptions->addOpt()->init(SM_OPT_bufpages, "bufpages",
						"#>32", NULL, 
						"number of pages in the buffer pool", TRUE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_logvolume, "logvolume",
						"volume ID", NULL, 
						"name of the volume containing the log", TRUE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_portname, "portname",
						"port name/number", "exodussm", 
						"connection port for the server either an entry\n\t\tin /etc/services or a port number", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_errorfile, "errorfile",
						"file name", "std error", 
						"file to write error/warning/progress diagnostics to", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_regenlog, "regenlog",
						"yes no", "no", 
						"regenerate the log", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_shutdown, "shutdown",
						"yes no", "no", 
						"shutdown after recovery or log regeneration", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_checkpoints, "checkpoints",
						"#>1", "100", 
						"checkpoint freq. based on # of log pages", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_wrapcount, "wrapcount",
						"#>=0", NULL, 
						"starting log wrap count", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_maxaddvolumes, "maxaddvolumes",
						"#>=0", "0", 
						"number of volumes added on the fly", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_progress, "progress",
						"yes no", "no", 
						"progress printing turned on", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_diskproc, "diskproc",
						"disk access program", SM_DiskProcessName, 
						"disk access program file name", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_intercache, "intercache",
						"yes no", "yes", 
						"allow inter-transaction caching of pages at the client", FALSE, NULL);
	ServerOptions->addOpt()->init(SM_OPT_maxclients, "maxclients",
						"#>0", "20", 
						"maximum number of clients", FALSE, NULL);
	// maxthreads must be added after maxclients since it's default
	// value depends on maxclients
	ServerOptions->addOpt()->init(SM_OPT_maxthreads, "maxthreads",
						"#>1", "function of maxclients", 
						"maximum number of threads", FALSE, NULL);
/*
	ServerOptions->addOpt()->init(SM_OPT_extrastats, "extrastats",
						"yes no", "no", 
						"print extra stats on logging performance", FALSE, NULL);
*/
/*
	ServerOptions->addOpt()->init(SM_OPT_preflushlog, "preflushlog",
						"yes no", "yes", 
						"allow preflushing of log pages", FALSE, NULL);
*/
#endif FORMATVOL_MAKE

	ServerOptions->addOpt()->init(SM_OPT_traceflags, "traceflags",
						"hex number", "0x0", 
						"sets tracing flags", FALSE, NULL);
	ServerOptions->addOpt()->init(FMT_OPT_data, "dataformat",
						"name;volid;#cyl;#tracks/cyl;#pages/track", 
						NULL, 
						"format info for recoverable data volumes", 
						FALSE, callBack);
	ServerOptions->addOpt()->init(FMT_OPT_temp, "tempformat",
						"name;id;#cyl;#tracks/cyl;#pages/track",
						NULL, 
						"format info for temporary data volumes", 
						FALSE, callBack);
	ServerOptions->addOpt()->init(FMT_OPT_log, "logformat",
					"name;volid;#cyl;#tracks/cyl;#pages/track;Kbytes/log page",
						NULL, 
						"format info for log volumes", 

#ifdef FORMATVOL_MAKE
						FALSE, /* not required for formatvol */
#else FORMATVOL_MAKE
						TRUE,  /* required for the server */
#endif FORMATVOL_MAKE
						callBack);
#ifdef FORMATVOL_MAKE
	ServerOptions->addOpt()->init(FMT_OPT_display, "displayformat",
						"device name -or- volume ID", NULL,
						"Unix path name -or- integer", 
						FALSE, callBack);
	ServerOptions->addOpt()->init(FMT_OPT_volume, "volume",
						"device name -or- volume ID", 
						NULL,
						"Unix path name -or- integer", 
						FALSE, callBack);
#endif FORMATVOL_MAKE

	// see if the first argument says we should skip reading the
	// default configuration files
	skipDefault = FALSE;
	startArg = 1;
	if (ServerOptions->lookup("skipdefault", opt) != OPTERR_Success) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	if (argc > 1) {
		if (opt->match(argv[startArg]+1)) {
			opt->setValue("yes", FALSE);
			startArg++;
			skipDefault = TRUE;
		}
	}

	// read in the 3 default option files if skipdefault is FALSE
	if (!skipDefault) {

		struct stat statBuf;	// info from stat() ststem call
		ino_t		lastInode = 0;	// inode for last config file read

		for (i = 0; i<3; i++) {
			// Generate the option file name
			switch (i) {
			case 0:
				strcpy(configFile, SM_ConfigFileName);
				break;
			case 1:
				if (getenv("HOME")) {
					if ( (strlen(getenv("HOME")) + strlen(SM_CONFIG) +1) >=
						_POSIX_PATH_MAX) {
						fprintf(stderr, "Cannot read default configuration file in home directory\n");
						fprintf(stderr, "$HOME path name is too long\n");
						exit(1);
					}
					strncpy(configFile, getenv("HOME"), _POSIX_PATH_MAX);
					strcat(configFile, "/.");
					strcat(configFile, SM_CONFIG);
				} else {
					fprintf(stderr, "Environment variable $HOME does not exist\n");
					strcpy(configFile, "");
				}
				break;
			case 2:
				strcpy(configFile, "./.");
				strcat(configFile, SM_CONFIG);
				break;
			default:
				SM_ERROR(TYPE_FATAL, esmINTERNAL);	
			}

			// make sure we have not already read this file
			if (i > 0) {
				if (stat(configFile, &statBuf) != 0) {
					continue;	// skip this file on error	
				}
				if (statBuf.st_ino == lastInode) {
					// this file is the same as the previous one,
					// so skip it
					continue;
				}
				lastInode = statBuf.st_ino;
			}

			errMsg = NULL;
			err = processOptionFile(configFile, ServerOptions, TRUE, &errMsg);

            if (err != esmNOERROR ) {

                if (SM_ERR_BASE_DECODE(err) == UNIX_ERROR_BASE) {
                    /* ignore unix file errors for default files */
                } else {
                    if (errMsg) {
						fprintf(stderr, "\t%s\n", errMsg);
					} else {
						fprintf(stderr, "Error with configuration file %s: %s\n", 
												configFile, errorCodeString(err));
					}
					exit(1);
                }
            }
		}
	}

	
	// process each command line argument
	i = startArg;
	while (i < argc) {

		if (argv[i][0] == '-') {
			
			switch (ServerOptions->lookup(argv[i]+1, opt)) {
			case OPTERR_Success:
				if (opt->ID() == SM_OPT_help) {
					more = popen("more", "w");	
					if (more == NULL) {
						fprintf(stderr, "more command not available\n");
						ServerOptions->printUsage(TRUE, progName, stderr);
					} else  {
						ServerOptions->printUsage(TRUE, progName, more);
						pclose(more);
					}
					exit(0);
				}

				if (opt->ID() == SM_OPT_background) {
					// advance to next argument
					RunInBackground = TRUE;
					i++;
					continue;
				}
				// see if this is the force option
				if (opt->ID() == SM_OPT_force) {
					// advance to next argument
					force = TRUE;
					i++;
					continue;
				}

				// option must be valid, make sure it has an argument
				if (i+1 == argc) {
					fprintf(stderr, "Usage error: missing argument for %s\n", argv[i]);
					ServerOptions->printUsage(FALSE, progName, stderr);
					exit(1);
				} else {
					if (argv[i+1][0] == '-') {
						fprintf(stderr, "Usage error: missing argument for %s\n", argv[i]);
						ServerOptions->printUsage(FALSE, progName, stderr);
						exit(1);
					} else {
						if (opt->setValue(argv[i+1], TRUE) != 0) {
							fprintf(stderr, "Usage error: bad value for option %s\n", argv[i]);
							ServerOptions->printUsage(FALSE, progName, stderr);
							exit(1);
						}
						i++;
					}
				}
				break;
			case OPTERR_NoMatch:
				fprintf(stderr, "Usage error: %s is not a valid option\n", argv[i]);
				ServerOptions->printUsage(FALSE, progName, stderr);
				exit(1);
				break;
			case OPTERR_Duplicate:
				fprintf(stderr, "Usage error: %s is not unique - please be more specific\n", argv[i]);
				ServerOptions->printUsage(FALSE, progName, stderr);
				exit(1);
				break;
			default:
				fprintf(stderr, "Command line processing error at: %s\n", argv[i]);
				exit(1);
				break;
			}

			// if a configuration file is given, process it
			if (opt->match("config")) {
				errMsg = NULL;
				err = processOptionFile(opt->value(), ServerOptions, TRUE, &errMsg);
                if (err != esmNOERROR) {
                    if (errMsg) {
						fprintf(stderr, "\t%s\n", errMsg);
					} else {
						fprintf(stderr, "Error with configuration file %s: %s\n", 
												configFile, errorCodeString(err));
					}
					exit(1);
                }
			}

		} else {
			fprintf(stderr, "Usage error: option %s must be preceded by '-'\n", argv[i]);
			fprintf(stderr, "\tWas an option argument ommited for the previous argument?\n", argv[i]);
			ServerOptions->printUsage(FALSE, progName, stderr);
			exit(1);
		}

		i++;
	}

#ifndef FORMATVOL_MAKE

	// validate options, make sure all required options are given
	if (checkRequiredOptions(ServerOptions, stderr)) {
		ServerOptions->printUsage(FALSE, progName, stderr);
		exit(1);
	}
#endif FORMATVOL_MAKE

	// set options
	ServerOptions->initScan();
	for	(opt = ServerOptions->getNext(); opt != NULL; opt = ServerOptions->getNext()) {
		switch (opt->ID()) {
        case SM_OPT_background: /* flags */
        case SM_OPT_help:
        case SM_OPT_skipdefault:
        case SM_OPT_force:
        case SM_OPT_config:
			break;
        case SM_OPT_verbose:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				verbose = TRUE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				verbose = FALSE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;

#ifndef FORMATVOL_MAKE
        case SM_OPT_maxaddvolumes:
			MaxAddVolumes = atoi(opt->value());
			if (MaxAddVolumes < 0) {
				fprintf(stderr, "Option error: <%s> should be: %s\n", opt->name(), opt->possibleValues());
				exit(1);
			}
			break;
#endif FORMATVOL_MAKE

        case SM_OPT_bufpages:
			NumBuffers = atoi(opt->value());
			if (NumBuffers <= 32) {
				fprintf(stderr, "Option error: <%s> should be: %s\n", opt->name(), opt->possibleValues());
				exit(1);
			}
			TRPRINT(TR_BF, TR_LEVEL_2, ("buffer space from environment:%d", NumBuffers));

			break;

        case SM_OPT_logvolume:
			if ( !(isdigit(opt->value()[0])) ) {
				fprintf(stderr, "Not a volume ID: %s\n", opt->value());
				SM_ERROR(TYPE_STOP, errno);
			} 
			LogVolid = (VOLID) (atoi(opt->value()));
			break;

        case SM_OPT_portname:
			if (isalpha(opt->value()[0])) {
				ServeName = opt->value();
				ServePort = 0;
			} else {
				ServePort = htons( (short) (atoi(opt->value())));
				ServeName = 0;
			}
			break;
        case SM_OPT_errorfile:
			if (opt->set() && strcmp(opt->value(), "-") ) {
				sm_ErrorStream = freopen(opt->value(),  "a", stderr);
				if (sm_ErrorStream == NULL) {
					sm_ErrorStream = stderr;
					fprintf(stdout, "Could not open error file: %s\n", optarg);
					fflush(stdout);
					SM_ERROR(TYPE_STOP, errno);
				}
				fprintf(sm_ErrorStream, "\nSERVER: server started.\n");
				setbuf(sm_ErrorStream, NULL);
			} else {
				sm_ErrorStream = stderr;
			}
			break;
        case SM_OPT_regenlog:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				RecoverLog = FALSE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				RecoverLog = TRUE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
        case SM_OPT_shutdown:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				ShutdownAfterRecovery = TRUE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				ShutdownAfterRecovery = FALSE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
        case SM_OPT_checkpoints:
			OpenLog.checkPointInterval = atoi(opt->value());
			if (OpenLog.checkPointInterval < 1) {
				fprintf(stderr, "Option error: <%s> should be: %s\n", opt->name(), opt->possibleValues());
				exit(1);
			}
			break;
        case SM_OPT_wrapcount:
			LogStartWrapCount = atoi(opt->value());
			if (LogStartWrapCount < 0) {
				fprintf(stderr, "Option error: <%s> should be: %s\n", opt->name(), opt->possibleValues());
				exit(1);
			}
			break;
        case SM_OPT_progress:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				PrintProgress = 1;
				break;
			case 'n': case 'N': case 'f': case 'F':
				PrintProgress = 0;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
        case SM_OPT_diskproc:
			DiskProcName = opt->value();
			break;
        case SM_OPT_intercache:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				AlwaysSendPage = FALSE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				AlwaysSendPage = TRUE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
        case SM_OPT_maxclients:
			NumClients = atoi(opt->value());
			if (NumClients < 1) {
				fprintf(stderr, "Option error: <%s> should be: %s\n", opt->name(), opt->possibleValues());
				exit(1);
			}
			break;
        case SM_OPT_maxthreads:
			if (opt->set()) {
				NumThreads = atoi(opt->value()) + CONST_THREADS + 2;
				/*
				 *	there must be at least 2 non-CONST_THREADS threads
				 *	One to execute, and one for spare.
				 */
				if (NumThreads < CONST_THREADS+2/*for safety*/ ) {
					fprintf(stderr, 
					"Option error: <%s> should be at least %d\n", 
					opt->name(), CONST_THREADS+2);
					exit(1);
				}
			} else {
				// use the default
				NumThreads = NumClients + CONST_THREADS + 2;
			}
			break;
/*
        case SM_OPT_extrastats:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				PrintExtraStats = TRUE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				PrintExtraStats = FALSE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
*/
/*
        case SM_OPT_preflushlog:
			switch (opt->value()[0]) {
			case 'y': case 'Y': case 't': case 'T':
				LogPreflushOn = TRUE;
				break;
			case 'n': case 'N': case 'f': case 'F':
				LogPreflushOn = FALSE;
				break;
			default:
				fprintf(stderr, "Option error: <%s> must be one of <%s>\n", opt->name(), opt->possibleValues());
				exit(1);
				break;
			}
			break;
*/
        case SM_OPT_traceflags:
			if (opt->set()) {
#ifndef DEBUG
				fprintf(stderr, 
					"Option error: <%s> is not available.\n", opt->name()); 
				fprintf(stderr, 
					"This program was made without debugging.\n");
#else
				char	val[100];

				sscanf(opt->value(), "%s", val);
				TraceFlags = (TRACEFLAGS) strtol(val, (char **)NULL, 0);
				initializeTrace();
#endif
			}
			break;

        case FMT_OPT_display:
        case FMT_OPT_volume:
        case FMT_OPT_temp:
        case FMT_OPT_data:
        case FMT_OPT_log:
        case SM_OPT_volume:
			/* all these were already  handled by callback */
			break;
		default:
			SM_ERROR(TYPE_FATAL, esmINTERNAL);	
		}
	}

	if (verbose) {
		ServerOptions->printValues(FALSE, sm_ErrorStream);
	}


#ifndef FORMATVOL_MAKE
    /*
     *  Verify request to regenerate the log
     */
    if (RecoverLog == FALSE && !force) {
		char		reply[10];

        printf("Are you sure you wish to regenerate the log? ");
        scanf("%3s", reply);
        if (reply[0] != 'y' && reply[0] != 'Y') {
            RecoverLog = TRUE;
        }
    }
#endif FORMATVOL_MAKE

}

int
addcommand( char *optname, char *formatvalue, FILE *out )
{
	OptionInfo*	opt = NULL;

	if (ServerOptions->lookup(optname, opt) != OPTERR_Success) {
badopt:
		if(out)
			fprintf(out, "Bad option for \"add\" command: %s\n", optname);

		SM_ERROR(TYPE_USER, esmBADOPTION);
		if(out) {
			fprintf(out, "Use dataformat, tempformat, or logformat.\n");
		}
		return esmFAILURE;
	}

	/* it's an option, but is it a sensible one? */

	if ((opt->ID() != FMT_OPT_data) && 
		(opt->ID() != FMT_OPT_log) && (opt->ID() != FMT_OPT_temp)
	   ) {
		goto badopt;
	}

	/* it's a sensible option, but what about the value given? */

	if(formatvalue == NULL) {
		if(out) {
			fprintf(out, "Missing value for option %s\n",  opt->name());
		}
		SM_ERROR(TYPE_USER, esmOPTIONSYNTAX);
		return esmFAILURE;
	}

	/* ok- parse the value */

	if (opt->setValue(formatvalue, TRUE) != esmNOERROR) {

		if(out)
			fprintf(out, "Usage error: bad value for option %s: %s\n", 
				optname, formatvalue);
		SM_ERROR(TYPE_USER, esmBADOPTIONVALUE);
		return esmFAILURE;
	}

	/* We have not yet mounted it and checked out the volid, etc */
	return esmNOERROR;
} 

int
removecommand(VOLID volid, FILE *out) 
{
	if(io_RemoveVolumeName(volid) == esmNOERROR) {
		return esmNOERROR;
	} else if(out)  {
		switch(Active->errno) {
		case esmDISKMOUNTED:
			fprintf(out, 
			"Volume %d is in use; try again later.\n", volid);
			break;

		case esmBADVOLID:
		case esmNOSUCHVOLUME:
			fprintf(out, 
			"There is no volume with volid %d; command ignored.\n", 
			volid);
			break;
		}
	}
	return esmFAILURE;
}
