/*
 *   $RCSfile: volumeOpts.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:07 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "server_options.h"
#include "option.h"
#include "server_main_funcs.h"
#include "io.h"
#include "link.h"
#include "volume.h"
#include "pool.h"
#include "io_extfuncs.h"
#include "io_intfuncs.h"
#include "disk_funcs.h"
#include "util_funcs.h"

extern BOOL PrintUser;

static char *delimiter =  " 	;,:"; 
		/* @, space, tab, semicolon, comma, colon */

#define GetPosNum(a,b) getPosNum(NULL, (a), (b))
#define ReReadPosNum(s, a, b ) getPosNum((s), (a), (b))

static int 
getPosNum(
	char *str,
	BOOL printErr,
	OptionInfo* opt
)
{
	char *p; long int n;

	if(str) {
		p = str;
	} else {
		p = strtok(NULL, delimiter);
	}
	if(p == NULL) {
		if(printErr) {
			fprintf(stderr, "Error in option %s: Missing part in value\n", opt->name());
		}
		goto bad;
	}

	errno = 0;
	n = strtol(p, (char **)NULL, 0);

	if(errno == 0) 
		return (int) n;

	if(printErr) {
		fprintf(stderr, "Error in option %s: Number not in range\n", opt->name(),
			(errno == ERANGE)?"Number not in range":"Cannot parse number" );
	}
	/* drop down to bad */

bad:
	return esmFAILURE;
}

int
parseFormat(
	OptionInfo* opt,
	const char *origstring
)
{
	char	stringbuf[MAXOPTLENGTH+1]; 
	char	*string = stringbuf;
	char 	*volName;
	VOLID 	volid;
	VOLNAMEREC *volNameRec = NULL;


	(void) strcpy(stringbuf, origstring);

	/*
	 * Get the filename.
	 */
	volName = strtok(string, delimiter);
    if(!volName) {
		fprintf(stderr, "Error in option %s: \n", opt->name());
		goto syntaxErr;
    }

	/*  see if we know about this volume name already */

	volid 		= (VOLID) GetPosNum(PrintUser, opt);
	if(volid < 0) {
		goto syntaxErr;
	}

	volNameRec =  io_GetVolNameRec(volName, volid, opt->name());
	if(volNameRec == NULL) {
		/* we had an error.  PrintUser is set to cause
		 * the error to be printed if it came from the server's
		 * terminal; else it ends up going to the client in reply(). 
		 */
		return esmFAILURE;
	}

	SM_ASSERT(LEVEL_1, (volNameRec->fmtinfo == NULL) );

	volNameRec->fmtinfo = (FORMATINFO *) calloc(1, sizeof(FORMATINFO));

	if((volNameRec->fmtinfo->numCylinders = GetPosNum(PrintUser, opt))<0) 
		goto syntaxErr;
	if((volNameRec->fmtinfo->tracksPerCylinder = GetPosNum(PrintUser, opt))<0)
		goto syntaxErr;
	if((volNameRec->fmtinfo->pagesPerTrack = GetPosNum(PrintUser, opt))<0)
		goto syntaxErr;

	switch(opt->ID()) {
	case FMT_OPT_temp:
			volNameRec->fmtinfo->volflags |= VOL_TEMP;
			volNameRec->fmtinfo->volflags &= ~VOL_LOG;
			volNameRec->fmtinfo->numLogFileBlocks = 0;
			volNameRec->fmtinfo->logFileBlock2size =  0;
			break;

	case FMT_OPT_data:
			volNameRec->fmtinfo->volflags &= ~(VOL_TEMP|VOL_LOG);
			volNameRec->fmtinfo->numLogFileBlocks = 0;
			volNameRec->fmtinfo->logFileBlock2size =  0;
			break;
		
	case FMT_OPT_log: {
			int szlp, minsize, maxsize;

			minsize = MIN_PAGESIZE >> 10; /* div by  1K */
			maxsize = MAX_PAGESIZE >> 10; /* div by  1K */
			if((szlp = GetPosNum(PrintUser, opt))<0)
				goto syntaxErr;
			if(!checkLog2(szlp)) {
				fprintf(stderr,
				"Error in option %s: log file block size (%d) must be power of 2\n", opt->name(), szlp);
				goto syntaxErr;
			}
			if((szlp < minsize) || (szlp > maxsize)) {
				fprintf(stderr,
				"Error in option %s: log file block size (%d) must be in the range [%d, %d]\n",
				opt->name(), szlp, minsize, maxsize);
				goto syntaxErr;
			}
			volNameRec->fmtinfo->logFileBlock2size =  szlp * 1024;
			volNameRec->fmtinfo->numLogFileBlocks = -1; /* all */
			volNameRec->fmtinfo->volflags &= ~VOL_TEMP;
			volNameRec->fmtinfo->volflags |= VOL_LOG;
		}
		break;

	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		break;
	}

	return esmNOERROR;

syntaxErr:

	/* don't print Error in option: %s because it's already been done */
	fprintf(stderr,
		"Must be: %s\n", opt->possibleValues());

	/* it's REALLY a syntax problem in the value field */
	SM_ERROR(TYPE_USER, esmBADOPTIONVALUE);

	if(volNameRec) {
		io_RecycleVolNameRec(volNameRec);
	}
	return esmFAILURE; 
}

/* 
 * parseVolume() for FMT_OPT_volume and FMT_OPT_display : volid or /name,
 * we MUST ALREADY have seen its format info.
 * This is reasonable since we expect the normal mode of op
 * to be this:
 *      formatvol*data: /pathname;2000;etc
 *      formatvol*temp: /temppath;2001;etc
 *      formatvol*log: /logpath;2001;etc
 * with the user invoking formatvol thus:
 *      formatvol -volume 2000 -volume /logpath
 */

#ifdef FORMATVOL_MAKE
int
parseVolume(
	OptionInfo* opt,
	const char *origstring
)
{
	char	stringbuf[MAXOPTLENGTH+1]; 
	char	*string = stringbuf;
	char 	*volName;
	VOLID	volid;
	VOLNAMEREC *volNameRec;

	(void) strcpy(stringbuf, origstring);


	/*
	 * Get the filename or volid
	 */
	volName = strtok(string, delimiter);
    if(!volName) {
        fprintf(stderr,
            "Error in option %s: must be one of <%s>\n",
            opt->name(), opt->possibleValues());
            exit(1);
    }

	if((volid = (VOLID) ReReadPosNum(string, FALSE, opt)) < 0) {
		volNameRec = io_FindVolName(volName);
	} else {
		volNameRec = io_FindVolNameById(volid);
	}
	if(volNameRec == NULL) {
		SM_ERROR(TYPE_LOG, esmNOSUCHVOLUME);
		return esmFAILURE;
	}
	/* mark it to be formatted */
	if(opt->ID() == FMT_OPT_volume) {
		volNameRec->fmtinfo->volflags |= VOL_FORMAT;
	}
	if(opt->ID() == FMT_OPT_display) {
		volNameRec->fmtinfo->volflags |= VOL_DISPLAY;
	}
	return esmNOERROR;
}

int
formatIt(
	VOLNAMEREC *volNameRec
)
{
	FORMATINFO *fmtinfo;

	fmtinfo = (FORMATINFO *)(volNameRec->fmtinfo);

	/*
	 * ok - go ahead and format it 
	 */
	if (io_FormatVolume(NULL,  volNameRec,
			fmtinfo->numCylinders, 
			fmtinfo->tracksPerCylinder,
			fmtinfo->pagesPerTrack, 
			fmtinfo->numLogFileBlocks,
			findLog2(fmtinfo->logFileBlock2size), 
			fmtinfo->volflags & VOL_PROPERTIES ) 
				!= esmNOERROR ) {

		//fprintf(stderr, "Format failed:%d\n", errno);
		return esmFAILURE;
	}
	return esmNOERROR;
}

void
displayIt(
	VOLNAMEREC *volNameRec
)
{
	VOLREC	*volRec;

	/* 
	*  The low-level mount requires a volRec for this
	*  thing, so let's create one.
	*/
	volRec = IO_NewVolRec();
	volRec->volNameRec = volNameRec;
	/* 
	 * mount, but don't check header; we're just going
	 * to display it.
	 */
	if(IO_MountVolume(volRec, FALSE) != esmNOERROR) {
		SM_ERROR(TYPE_LOG, esmNOSUCHVOLUME);
		return;
	}
	volRec->volid = volRec->header->volid;
	{
		VOLNAMEREC *vnr2 =io_FindVolNameById(volRec->volid);

		if( vnr2 && (vnr2 != volNameRec) ) {
			/* the volNameRec could already have the volid filled in
			 * by a duplicate entry in the mount table.  Nevertheless,
			 * check its volid
			 */
			fprintf(stderr, 
				"VOLID %d is both %s and %s\n",  volRec->volid,
					volNameRec->volName, vnr2->volName);
			SM_ERROR(TYPE_LOG, esmDUPVOLID);
			volNameRec->volid = 0;
			(void)IO_DismountVolume(volRec);
			return;
		}
	}

	/*
	 *
	 * Display: header
	 * bitmap pages x..y
	 * root entries at x..y
	 * first data page at x
	 * #pages total, free, used
	 */

	{
		VOLHDR *h = volRec->header;
		FILE	*file = stdout;

		fprintf(file,
		"VOLID %d, version %x, is a %s volume\n", h->volid, h->version, 
			(h->properties & VOLPROP_TEMP)? "TEMP":
			(h->properties & VOLPROP_LOG)? "LOG": "DATA");

#define p2sizeToK(x) (1<<((x)-10 ))
		fprintf(file,
			"\tBLOCK SIZES: %d K slotted, %d K lg data,  %d K lg hdr\n",
				p2sizeToK(h->slottedPage2size), 
				p2sizeToK(h->lgPage2size), 
				p2sizeToK(h->lgDataPage2size)
				);
		fprintf(file,
			"\t\t%d K btree, %d K idesc\n",
				p2sizeToK(h->btreePage2size), 
				p2sizeToK(h->indexDescPage2size));

		fprintf(file,
			"\tLAYOUT: %d blk/trk; %d trk/cyl; %d cyl\n",
			h->pagesPerTrack, h->tracksPerCylinder, h->numCylinders);

		fprintf(file,
		"\t\t%d total blocks of %d KB for %.3f KB\n",
			h->numPages,
			p2sizeToK(h->minPage2size),
			((float)(h->numPages * (1<<h->minPage2size)) / (float)1000)
			);

		fprintf(file,
			"\tFREE: %d free, %d used\n",
			h->numFreePages, (h->numPages - h->numFreePages));

		fprintf(file,
			"\tBITMAP: %d blk each, freemap @ %d, slotmap @ %d, filemap @ %d\n",
			h->bitmapPages, h->freeBitmapAddr, 
			h->slotBitmapAddr,
			h->fileBitmapAddr);

		fprintf(file,
			"\tUNIQUE: start @ %d\n", h->uniqueArrayAddr);

		if(h->properties & VOLPROP_LOG) {
			fprintf(file,
				"\tLOG: start @ %d, ctl blk @ %d, blk sz %d K, #blks %d\n",
				h->logFileAddr, h->logControlAddr, 
				p2sizeToK(h->logFileBlock2size),
				h->numLogFileBlocks);
			fprintf(file,
				"\t\tend of log @ dismount: LSN w=%d.o=%d, LRC w=%d.c=%d\n",
				h->dismountEndOfLog.wrapCount,
				h->dismountEndOfLog.offset,
				h->lrc.wrapCount, h->lrc.count);
		}

	}
	(void)IO_DismountVolume(volRec);
}

#endif FORMATVOL_MAKE
