/*
 *   $RCSfile: admin_CollectStats.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/*********************************************************************e
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "list.h"
#include "io.h"
#include "object.h"
#include "pool.h"
#include "tid.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "volume.h"
#include "trans.h"
#include "adminmsg.h"
#include "log.h"
#include "openlog.h"
#include "logrecs.h"
#include "admin_funcs.h"
#include "admin_globals.h"
#include "trans_globals.h"
#include "disk_globals.h"
#include "io_globals.h"
#include "io_extfuncs.h"
#include "log_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "bitvec.h"
#include "msg_funcs.h"

/*
 *	This function gathers statistics and places them in the 
 *	ServerStats global variable.  Then they can be printed or sent
 *	to a client.
 */

 int
admin_CollectStats ()
{
	TRANSREC	*currTrans;
	SHORTPID	limitPid;
	int			freePages;

	TRPRINT(TR_ADMIN, TR_LEVEL_1, ("collecting stats"));
	
	/*
	 *	Get the easy stats
	 */
	ServerStats.totalCommits = TotalCommits;
	ServerStats.totalAborts = TotalAborts;
	ServerStats.chpntFreq = OpenLog.checkPointInterval;
	ServerStats.numClients = NumActiveClients;
	ServerStats.diskReads = IO_DiskReads;
	ServerStats.diskWrites = IO_DiskWrites;
	ServerStats.numActiveCoordinator = numActiveCoordDistrTrans;

	/*
	 *	Count the number of active transactions
	 *	and the number of prepared transactions 
	 *	for which this server is a participant
	 */
	currTrans = (TRANSREC*) FIRST_LIST_ELEMENT(&ActiveTransList);	
	ServerStats.numTrans = ServerStats.numPreparedParticipant = 0;
	while (currTrans) {
		CHECK_TRANSREC_MAGIC(currTrans);
		ServerStats.numTrans++;
		if TRANS_PREPARED_PARTICIPANT(currTrans) {
			ServerStats.numPreparedParticipant++;
		}
		currTrans = (TRANSREC*) NEXT_LIST_ELEMENT(&(currTrans->activeTransList));
	}

	/*
	 *	Count the number of volumes mounted
	 */
	ServerStats.numVolumes = io_NumOpenVolumes();

	/*
 	 *	Calculate available log space.  This is based on code from
	 *	log/calculateLogSpace.c
	 */
	if (OpenLog.activePid < 0) {
        /*
         *  there are no active transactions so the current end of
		 *	the log is the limiting page
         */
        limitPid = OpenLog.tailPid;
        TRPRINT(TR_LOG, TR_LEVEL_2, ("no active transaction"));

    } else {

        /*
         *  the active transaction is the limiting factor
         */
        limitPid = OpenLog.activePid;
        TRPRINT(TR_LOG, TR_LEVEL_2, ("active limiting"));
    }

    /*
     *  calculate the space that can be used
     */
    if (OpenLog.tailPid < limitPid) {

        /*
         *  calculate the space between
         */
        freePages = (int)((limitPid - OpenLog.tailPid) - 1);

    } else if (OpenLog.tailPid >= limitPid)  {

        /*
         *  calculate the space at the end and beginning of file
         */
        freePages = ((int)OpenLog.filePages - ((int)OpenLog.tailPid + 1)) 
							+ (int)limitPid;
    }
	ServerStats.freeLogSpace = freePages << OpenLog.page2size;
	/* subtract log space needed for aborts plus some slack */
	ServerStats.freeLogSpace -= ActiveLogSpace + LOG_SLACK;

	ServerStats.msgStats	= MsgStats;

	/* client has to fill this in */
	BZERO(&ServerStats.serverAddr, sizeof(ServerStats.serverAddr));

	return(esmNOERROR);
}
