/*
 *   $RCSfile: admin_ListDistr.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/*********************************************************************e
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "lock.h"
#include "list.h"
#include "io.h"
#include "object.h"
#include "pool.h"
#include "tid.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "volume.h"
#include "trans.h"
#include "adminmsg.h"
#include "log.h"
#include "openlog.h"
#include "logrecs.h"
#include "admin_funcs.h"
#include "admin_globals.h"
#include "trans_globals.h"
#include "disk_globals.h"
#include "io_globals.h"
#include "io_extfuncs.h"
#include "log_globals.h"
#include "distr.h"
#include "distr_globals.h"

char state_string[10][10] = 
		{"INACTIVE",
		 "ACTIVE",
		 "LATENT",
		 "QUIESCE",
		 "COMMIT",
		 "ABORT",
		 "RECOVER",
		 "PREPARED",
		 "END",
		 ""};
/*
 *	This function lists the active distibuted transactions. 
 *	For transactions for which this server is the coordinator, 
 *	it lists the state of the tranasaction, the set of participants, 
 *	their addresses and TIDs. 
 *	For transactions for which this server is a participant,
 *	it lists the local TID, the state of the transaction, the 
 *	coordinator's address, and the coordinator's TID. 
 */
 void
admin_ListDistr ()
{
	TRANSREC		*transRec;
	SERVERTIDINFO	*serverTidInfo;
	int				serverNum;
	int				transNum;

	TRPRINT(TR_ADMIN, TR_LEVEL_1, ("listing active distributed transactions "));
	
	/*
	 *	go through the list of transactions	which are being 
	 *	coordinated by this server
	 */
	fprintf(stdout, "** List of Transactions being Coordinated:\n");

	transRec = (TRANSREC *) FIRST_LIST_ELEMENT(&CoordDistrTransList);
	transNum = 0;

	while (transRec != NULL) {

		CHECK_TRANSREC_MAGIC(transRec);
		transNum++;

		fprintf(stdout,"%d. TID: %d, State: %s, Number of Participants: %d\n",
				transNum, transRec->tid, 
				state_string[transRec->transState], 
				transRec->numServers);

		for (serverNum=0,
			 serverTidInfo = (SERVERTIDINFO *)
				FIRST_LIST_ELEMENT (&(transRec->serverList));
			 serverTidInfo != NULL;
			 serverTidInfo = (SERVERTIDINFO *)
				NEXT_LIST_ELEMENT(&(serverTidInfo->serverTidInfoList)),
				serverNum++) {
		
			fprintf(stdout, "\tParticipant %d: TID: %d, Address: %s.%d\n",
					serverNum+1, serverTidInfo->tid,
					inet_ntoa(serverTidInfo->connectAddr.sin_addr),
					ntohs(serverTidInfo->connectAddr.sin_port));
		}

		SM_ASSERT(LEVEL_3, (serverNum == transRec->numServers));

		transRec = (TRANSREC *) NEXT_LIST_ELEMENT(&(transRec->distrTransList));
	}

	SM_ASSERT(LEVEL_3, (transNum == numActiveCoordDistrTrans));

	/*
	 *	print the number of transactions being coordinated
	 */
	fprintf(stdout, "* Number of Active Transactions for which this Server is the Coordinator: %d *\n\n", numActiveCoordDistrTrans);


	/*
	 *	next go through the list of transactions to pick out those
	 *	for which this server is a participant in prepared state
	 */
	fprintf(stdout, "** List of Prepared Transactions at this Server:\n");

	transRec = (TRANSREC*) FIRST_LIST_ELEMENT(&ActiveTransList);	
	transNum = 0;

	while (transRec) {
		CHECK_TRANSREC_MAGIC(transRec);
		if TRANS_PREPARED_PARTICIPANT(transRec) {
			transNum++;	
			fprintf(stdout, "%d. TID: %d, State: %s\n", transNum, 
				transRec->tid, state_string[transRec->transState]);
			fprintf(stdout, "\tCoord's Address: %s.%d, Coord's TID: %d \n",
				inet_ntoa(transRec->coordAddr.sin_addr),
				ntohs(transRec->coordAddr.sin_port),
				transRec->coordTid);
		}
		transRec = (TRANSREC*) NEXT_LIST_ELEMENT(&(transRec->activeTransList));
	}

	/*
	 *	print the number of prepared transcations
	 */
	fprintf(stdout, "* Number of Prepared Transactions for which this Server is a Participant: %d *\n\n", transNum);


}
