/*
 *   $RCSfile: analyzeRootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "page.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "trans_extfuncs.h"
#include "analyze_extfuncs.h"
#include "thread_globals.h"


 void
analyzeRootEntry (

	LOGRECORDHDR		*record 
)
{

	PID					pid;
	DIRTYPAGEINFO		*dirtyInfo;

	TRACE(TR_IO|TR_RECOVER, TR_LEVEL_1);

	/*
	 *	calculate the root entry page
	 */
	pid.volid = record->actionPid.volid;
	pid.page = ROOTPAGEADDR;
	TRPRINT(TR_RECOVER, TR_LEVEL_2, ("page:%d volid:%d", pid.page, pid.volid));

	/*
	 *	Make sure this matches the action pid
	 */
	SM_ASSERT(LEVEL_3, PIDEQ(pid, record->actionPid));

	/*
	 *	check to see if the page is in the dirty list
	 */
	if ((dirtyInfo = searchDirtyPageTable( &pid )) == NULL)	{

		/*
		 *	create a new entry
		 */
		if ((dirtyInfo = insertDirtyPageTable( &pid )) == NULL)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *	update the information
		 */
		dirtyInfo->lsn = record->recordLSN;
		dirtyInfo->lrc = record->actionLRC;
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("lsn:%d, lrc%d", dirtyInfo->lsn.offset, dirtyInfo->lrc.count));
	}
}
