/*
 *   $RCSfile: analyzeSplitFileNode.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "trans_extfuncs.h"
#include "analyze_extfuncs.h"
#include "thread_globals.h"


 void
analyzeSplitFileNode (

	LOGRECORDHDR		*record 
)
{

	PID					*pid;
	PID					*pidList;
	LRC					*lrcList;
	DIRTYPAGEINFO		*dirtyInfo;
	register int		i;


	TRACE(TR_IO, TR_LEVEL_1);


	/*
	 *	get pointers to the lists of pages and lrc's involved 
	 */
	pidList = (PID *) GET_LOG_IMAGE(record, 1);
	lrcList = (LRC *) GET_LOG_IMAGE(record, 2);

	/*
	 *	process the split pages
	 */
	for (i = 0; i < 3; i++)	{

		/*
		 *	get a pointer to the allocated pid
		 */
		pid = pidList+i;
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("page%d:%d", i, pid->page));

		/*
		 *	check to see if the page is in the dirty list
		 */
		if ((dirtyInfo = searchDirtyPageTable(pid)) == NULL)	{

			/*
			 *	create a new entry
			 */
			if ((dirtyInfo = insertDirtyPageTable(pid)) == NULL)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			/*
			 *	update the information
			 */
			dirtyInfo->lsn = record->recordLSN;
			dirtyInfo->lrc = lrcList[i];
			TRPRINT(TR_RECOVER, TR_LEVEL_2, ("lsn:%d, lrc:%d", dirtyInfo->lsn.offset, dirtyInfo->lrc.count));

			dirtyInfo->tid = record->tid;
		}
	}
}
