/*
 *   $RCSfile: BF_FreePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 void
BF_FreePage (

	register BUFGROUP	*bufGroup,
	register GROUPLINK	*groupLink 
)
{

	register PAGEHASH	*pageHash;


	TRPRINT(TR_BF, TR_LEVEL_1, ("freeing page:%d",
			groupLink->pageHash->hashList.hashPid.page));

	/*
	 *	check magic numbers
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);
	CHECK_BUFGROUP_MAGIC(bufGroup);

	/*
	 *	Pull out the hash pointer
	 */
	pageHash = groupLink->pageHash;
	CHECK_PAGEHASH_MAGIC(pageHash);
	    
	/*
	 *	Update the group's unfixed size and reset the group links.
	 */
	bufGroup->unfixedCharges -= BLOCKCOUNT(pageHash->page2size);

	/*
	 *	check to see that the fixed charges are reasonable
	 */
	SM_ASSERT(LEVEL_3, bufGroup->unfixedCharges >= 0);

	/*
	 *	check to see if the page is a log page
	 */
	if (bufGroup->flags & FORCE_GROUP)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("force buffer"));

		/*
		 *	flush the page
		 */
		bf_ForcePage(pageHash, BF_SEM);
	}

	/*
	 *	Check to see if this is the only group that has the page as a member
	 */
	if (pageHash->memberCount == 1)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("putting on free list"));

		/*
		 *	Must put it at the head of the LRU queue
		 */
		listEnq( &(FreeBufTable[OFFSET(groupLink->pageHash->page2size)]),
				  &(groupLink->groupList) );

		/*
		 *	set the buffer group pointer to null
		 */
		groupLink->bufGroup = NULL;

		/*
		 *	Set the group count to zero
		 */
		pageHash->memberCount = 0;
		TRPRINT(TR_BF, TR_LEVEL_2, ("freeing index:%d size:%d",
				groupLink->pageHash->bufIndex, groupLink->pageHash->blockCount));

	} else {

		TRPRINT(TR_BF, TR_LEVEL_2, ("member of %d other groups", pageHash->memberCount));

		/*
		 *	put the link on the free pool
		 */
		poolMove( &(GroupLinkPool), &(groupLink->hashList) );

		/*
		 *	Decrement the number of groups it is in
		 */
		pageHash->memberCount--;
	}
}
