/*
 *   $RCSfile: BF_GetLarger.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 GROUPLINK 
*BF_GetLarger (

	int					block2size 
)
{

	register GROUPLINK	*groupLink = NULL;
	register GROUPLINK	*newLink;
	register PAGEHASH	*pageHash;
	register BUFINFO	*bufInfo;
	register int		size;
	register int		bsize;
	register int		start;


	TRPRINT(TR_BF, TR_LEVEL_1, ("looking for block2size:%d", block2size));

	/*
	 *	loop through the larger size queues
	 *	should convert this to pointer operations
	 */
	for (size = block2size + 1; size <= MaxPage2Size; size++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("checking size:%d", size));

		/*
		 *	attempt to get a buffer off the list
		 */
		if (groupLink = (GROUPLINK *) listDeq( &FreeBufTable[OFFSET(size)] ))	{

			TRPRINT(TR_BF, TR_LEVEL_2, ("got buffer of size:%d", size));

			/*
			 *	break out of loop
			 */
			break;
		}
	}

	/*
	 *	check to see if we got a buffer
	 */
	if (groupLink == NULL)	{

		/*
		 *	we didn't find a larger buffer
		 */
		return(NULL);

	}

	/*
	 *	check the groupLink magic
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);

	/*
	 *	Invalidate the page, this could block
	 */
	BF_InvalidatePage(groupLink->pageHash, BF_SEM|BF_FORCE);

	/*
	 *	get the starting address to start splitting the buffers
	 */
	start = groupLink->pageHash->bufIndex + BLOCKCOUNT(block2size);
	TRPRINT(TR_BF, TR_LEVEL_2, ("start:%d", start));

	/*
	 *	create buddy size pages at this address
	 */
	for (bsize = block2size; bsize < size; start += BLOCKCOUNT(bsize), bsize++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("creating block size:%d at:%d", bsize, start));

		/*
		 *	Get a group link structure, should always be enough
		 */
		if ((newLink = (GROUPLINK *) listDeq( &BufStructPool )) == NULL)	{

			SM_ERROR(TYPE_WARNING, esmINTERNAL);
		}

		/*
		 *	set the type to page
		 */
		pageHash = newLink->pageHash;
		CHECK_PAGEHASH_MAGIC(pageHash);
		pageHash->page2size = bsize;
		TRPRINT(TR_BF, TR_LEVEL_2, ("new page size:%d", pageHash->page2size));

		/*
		 *	Initialize the pointer to the actual buffer
		 */
		pageHash->bufFrame = &(BufferPool[start << MIN_PAGE2SIZE]);
		pageHash->bufIndex = start;
		bufInfo = pageHash->bufInfo = &(BufInfoTable[start]);
		pageHash->blockCount = BLOCKCOUNT(pageHash->page2size);

		/*
		 *	Set the group link pointer to the buffer
		 */
		newLink->bufFrame = pageHash->bufFrame;

		/*
		 *	put the buffer on the free list
		 */
		listEnq( &(FreeBufTable[OFFSET(bsize)]), &(newLink->groupList) );

		/*
		 *	clear the slot table
		 */
		BF_SetSlot(bufInfo, BLOCKCOUNT(pageHash->page2size), pageHash);
	}

	/*
	 *	set the size of the page
	 */
	groupLink->pageHash->page2size = block2size;

	/*
	 *	return the found buffer
	 */
	return(groupLink);
}
