/*
 *   $RCSfile: BF_InitializeGroups.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:33 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


/*
 *============================================================================
 *
 * BF_InitializeGroups:
 * 	Routine to initialize the buffer groups.
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	none
 *
 * SIDE EFFECTS:
 *	The data structures associated with buffer groups are initialized.
 *
 * IMPORTS:
 *
 *
 * BUGS:
 *
 *============================================================================
 */


 void
BF_InitializeGroups ()

{

    register int 	i;		
	register int	numbytes;


	TRACE(TR_BF, TR_LEVEL_1);

    /*
	 *	Calculate space needed for group structures
     */
	numbytes = sizeof(BUFGROUP) * NumBufGroups;

    /*
	 *	Allocate the space
     */
	if ((BufGroupPtr = (BUFGROUP *) malloc((unsigned int) numbytes)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	Initialize the free and usedbuffer group list
	 */
	initializePool( &BufGroupPool, (ALLOCFUNC) bf_AllocBufGroup, NumBufGroups );
	initializeList( &UsedBufGroups );

	/*
	 *	Put all the groups on the free list
	 */
    for (i = 0; i < NumBufGroups; i++) {

		/*
		 *	Initialize the list elements
		 */
		initializeList( &(BufGroupPtr[i].fixedList) );
		initializeList( &(BufGroupPtr[i].unfixedList) );
		initializeListElement( &(BufGroupPtr[i].groupList), (char *) &(BufGroupPtr[i]) );
		BufGroupPtr[i].flags = FREE_GROUP;

		/*
		 *	Place the element on the free list
		 */
		listEnq( &(BufGroupPool.freeList), (&BufGroupPtr[i].groupList) );
		INIT_BUFGROUP_MAGIC( &(BufGroupPtr[i]) );
    }

    /*
	 *	Calculate space needed for group link structures
     */
	numbytes = sizeof(GROUPLINK) * NumGroupLink;

	/*
	 *	Allocate the group links
	 */
	if ((GroupLinkPtr = (GROUPLINK *) malloc((unsigned int) numbytes)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	Initialize the Group Link Pool
	 */
	initializePool( &GroupLinkPool, (ALLOCFUNC) bf_AllocGroupLink, NumGroupLink );

	/*
	 *	Put the group link structures on the free list
	 */
	for (i = 0; i < NumGroupLink; i++)	{

		/*
		 *	initialize the element
		 */
		initializeListElement( &(GroupLinkPtr[i].hashList), (char *) &(GroupLinkPtr[i]) );
		initializeListElement( &(GroupLinkPtr[i].groupList), (char *) &(GroupLinkPtr[i]) );
		GroupLinkPtr[i].bufGroup = NULL;
		GroupLinkPtr[i].fixedCount = 0;
		GroupLinkPtr[i].pageHash = NULL;

		/*
		 *	Mark it as free and put it on the free list
		 */
		listEnq( &(GroupLinkPool.freeList), &(GroupLinkPtr[i].hashList) );
		INIT_GROUPLINK_MAGIC( &(GroupLinkPtr[i]) );
	}

	/*
	 * 	Initialize the bufgroup waiting list
	 */
	initializeList( &BufGroupWaitList );
}
