/*
 *   $RCSfile: BF_InvalidatePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "log.h"
#include "openlog.h"
#include "logrecs.h"
#include "trans.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "log_extfuncs.h"
#include "log_globals.h"
#include "thread_globals.h"


 void
BF_InvalidatePage (

	register PAGEHASH	*pageHash,
	FLAGS				flags 
)
{

	TRPRINT(TR_BF, TR_LEVEL_1, ("index:%d size:%d",
			pageHash->bufIndex, BLOCKCOUNT(pageHash->page2size)));

	/*
	 *	get the semaphore
	 */
	if (flags & BF_SEM)	{

		if (waitSemaphore( &(pageHash->semaphore) ))	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
	} else {
		SM_ASSERT(LEVEL_3, pageHash->semaphore.holder == Active);
	}

	/*
	 *	check to see if the page is valid
	 */
	if (LIST_MEMBER( &(pageHash->hashList.hashList) ))	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("page is valid"));

		/*
		 *	check to see if the page is dirty
		 */
		if (PAGE_IS_DIRTY(pageHash))	{

			/*
			 *	check to see if the page should be forced
			 */

			if (flags & BF_FORCE)	{

				/*
				 *	flush it to disk
				 */
				bf_ForcePage(pageHash, NOFLAGS);

				/*
				 *	If no checkpoint is in progress, start on up
				 *  since the buffer pool is probably getting full
				 *  of dirty pages.
				 */
				if ( !(OpenLog.flags & LOG_CHECKPOINT_IN_PROGRESS) ) {
					activateCheckpoint(TRUE, FALSE, FALSE);
				}
			} else {

				/*
				 *	clean the page
				 */
				bf_CleanPage(pageHash);
			}
		}
		SM_ASSERT(LEVEL_3, !LIST_MEMBER(&(pageHash->clientDirtyList)));
		
		/*
		 *	blow away any waiters
		 */
		if (LIST_NOT_EMPTY( &(pageHash->semaphore.waitList) ))	{

			notify( &(pageHash->semaphore.waitList), esmFAILURE, esmPAGEGONE );
		}

		/*
		 *	remove the page from the hash list after the I/O has occurred
		 */
		listRemove( &(pageHash->hashList.hashList) );
		/*
		 * put it in the LRC cache if the page type is known and 
		 * its not a large object data page
		 */
		if ((PPAGE_TYPE(pageHash) != PAGE_GENERIC) && 
			(PPAGE_TYPE(pageHash) != PAGE_LARGEDATA) ) {
			BF_CacheLRCInfo( pageHash );
		}
	}

	/*
	 *	Initialize the semaphore
	 */
	initializeSemaphore( &(pageHash->semaphore), TRUE );

}
