/*
 *   $RCSfile: bf_CheckFixed.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_extfuncs.h"


 BOOL
bf_CheckFixed (

	PAGEHASH			*pageHash 
)
{

	register GROUPLINK	*groupLink;


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	check the pagehash magic number
	 */
	CHECK_PAGEHASH_MAGIC(pageHash);

	/*
	 *	Check quick to see if the item is on the free list
	 */
	if (pageHash->memberCount == 0)	{

		/*
		 *	The page is on a free list, so it is not fixed
		 */
		TRPRINT(TR_BF, TR_LEVEL_2, ("page is on the free list%x", pageHash));
		return(FALSE);
	
	}

	/*
	 *	get a pointer to the first group link
	 */
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(pageHash->memberList) );

	/*
	 *	Traverse down the list looking for one that is fixed
	 */
	while (groupLink != NULL)	{

		/*
		 *	check magic numbers
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);
		CHECK_BUFGROUP_MAGIC(groupLink->bufGroup);

		/*
		 *	Check to see if the page is fixed
		 */
		if (groupLink->fixedCount != 0)	{

			/*
			 *	The page is fixed
			 */
			TRPRINT(TR_BF, TR_LEVEL_2, ("page is fixed"));
			return(TRUE);
		}	

		/*
		 *	Look at the next entry
		 */
		groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
	}

	/*
	 *	The page must be unfixed, since no group has it fixed.
	 */
	return(FALSE);
}
