/*
 *   $RCSfile: bf_ConvertTempToRealPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


/* 
 * bf_ConvertTempToRealPage:
 * 	Routine to put a temporary allocated page into the buffer
 *	area as a named and valid page.
 *
 * SIDE EFFECTS:
 *	The page is entered into the page hash.
 *
 */


 int 
bf_ConvertTempToRealPage (

	BUFGROUP			*bufGroup,
	GROUPLINK			*groupLink,
	PID					*pid,
	FLAGS				flags 
)
{


	TRPRINT(TR_BF, TR_LEVEL_1, ("convert page:%d", pid->page));

	
	/*
	 *	check to make sure the semaphore is free
	 */
	SM_ASSERT(LEVEL_3, SEMAPHORE_IS_FREE( &(groupLink->pageHash->semaphore) ));

	/*
	 *	Manage the buffer group space
	 */
	if (BF_Swap(bufGroup, groupLink->pageHash->page2size))   {
		return(esmFAILURE);
	}

	/*
	 *	grab the semaphore
	 */
	waitSemaphore( &(groupLink->pageHash->semaphore) );

	/*
	 *	Hang the buffer off the fixed list
	 */
	listEnq( &(bufGroup->fixedList), &(groupLink->groupList) );

	/*
	 *	Set the group pointer in the link
	 */
	groupLink->bufGroup = bufGroup;

	/*
	 *	Set the Group count, and pending operation count
	 */
	groupLink->pageHash->memberCount = 1;
	groupLink->pageHash->pendingOpCount = 0;

	/*
	 *	Set the fixed count
	 */
	groupLink->fixedCount = 1;

	/*
	 *	Increment the fixed charges
	 */
	bufGroup->fixedCharges += BLOCKCOUNT(groupLink->pageHash->page2size);

	/*
	 *	set the first and force marks to zero
	 */
	groupLink->pageHash->firstMark = groupLink->pageHash->forceMark = 0;
	groupLink->pageHash->lsn.wrapCount = groupLink->pageHash->lsn.offset = 0;
	groupLink->pageHash->lrc.wrapCount = groupLink->pageHash->lrc.count = 0;

	/*
	 *	Put the page in the hash table
	 */
	BF_InsertHashEntry(groupLink->pageHash, pid, NOFLAGS);

	/*
	 *	Release the semaphore unless user wants it
	 */
	if (!(flags & BF_SEM)) {
		signalSemaphore( &(groupLink->pageHash->semaphore) );
	}
	CHECK_GROUPLINK_MAGIC(groupLink);

	return(esmNOERROR);
}
