/*
 *   $RCSfile: bf_DiscardPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 void
bf_DiscardPage (

	register BUFGROUP	*bufGroup,
	register GROUPLINK	*groupLink 
)
{

	register PAGEHASH	*pageHash;


	TRPRINT(TR_BF, TR_LEVEL_1, ("page:%d", groupLink->pageHash->hashList.hashPid.page));

	/*
	 *	Check the magic number
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);

	/*
	 *	Pull out the hash pointer
	 */
	pageHash = groupLink->pageHash;
	    
	/*
	 *	Update the group's unfixed size and reset the group links.
	 */
	bufGroup->fixedCharges -= BLOCKCOUNT(pageHash->page2size);

	/*
	 *	Check to see if this is the only group that has the page as a member
	 */
	if (pageHash->memberCount == 1)	{

		/*
		 *	Take it off the hash queue
		 */
		listRemove( &(pageHash->hashList.hashList) );

		/*
		 *	Must put it at the head of the LRU queue
		 */
		listMovePush( &(FreeBufTable[OFFSET(groupLink->pageHash->page2size)]),
				  &(groupLink->groupList) );

		/*
		 *	set the group pointer and the member count
		 */
		groupLink->bufGroup = NULL;
		pageHash->memberCount = 0;
		bf_CleanPage(pageHash);

	} else {

		/*
		 *	die for now
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
}
