/*
 *   $RCSfile: bf_DismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "volume.h"
#include "lsn.h"
#include "log.h"
#include "openlog.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "link.h"
#include "volume.h"
#include "io_extfuncs.h"
#include "bf_globals.h"
#include "log_globals.h"


 int 
bf_DismountVolume (

	VOLREC		*volRec,
	BOOL		logDismount 
)
{

	TRPRINT(TR_BF, TR_LEVEL_1, ("dismounting:%d", volRec->header->volid));

	if (io_Checker(volRec) != esmNOERROR) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	Remeber the current end of the log.
	 *	Later, when the volume is mounted, this will be checked
	 *	to verify that LRC's on this volume will be allocated
	 *	monotonically increasing.
	 */
	volRec->header->dismountEndOfLog = OpenLog.nextValidLSN;
	volRec->dirty = TRUE;
	DIRTY_PAGE(volRec->headerLink->pageHash);

	/*
	 *	flush the cached LRC info 
	 */
	BF_FlushLRCCache(volRec->header->volid);

	/*
	 *	flush all volume pages
	 */
	if (bf_FlushVolume(volRec->header->volid))	{

		return(esmFAILURE);
	}

	/*
	 *	dismount the volume
	 */
	if (io_DismountVolume(volRec, logDismount) != esmNOERROR) {
		return(esmFAILURE);
	}

	fprintf(sm_ErrorStream, "SERVER: Volume %d (%s) has been unmounted in a clean state.\n", volRec->header->volid, volRec->volNameRec->volName.name );
	return(esmNOERROR);
}
