/*
 *   $RCSfile: bf_FlushVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "link.h"
#include "volume.h"
#include "io_extfuncs.h"
#include "bf_globals.h"


 int 
bf_FlushVolume (

	register VOLID		volid 
)
{

    register PAGEHASH   *pageHash;
	register GROUPLINK	*groupLink;
	register BUFGROUP	*bufGroup;
	GROUPLINK			*tempLink;
	int					i;


	TRPRINT(TR_BF, TR_LEVEL_1, ("dismounting:%d", volid));

	/*
	 *	get a pointer to the first in use buffer group
	 */
	bufGroup = (BUFGROUP *) FIRST_LIST_ELEMENT( &UsedBufGroups );

	/*
	 *	run through the open buffer group list to see if there are any fixed pages
	 */
	while (bufGroup != NULL)	{

		/*
		 *	check the buffer group magic number
		 */
		CHECK_BUFGROUP_MAGIC(bufGroup);

		/*
		 *	remove a buffer from the list
		 */
		groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(bufGroup->fixedList) );

		/*
		 *	run down the list
		 */
		while (groupLink != NULL)	{

			/*
			 *	check the grouplink magic
			 */
			CHECK_GROUPLINK_MAGIC(groupLink);

			/*
			 *	check the pid to see if the volume matches
			 */
			if ((groupLink->pageHash->hashList.hashPid.volid == volid) &&
				(groupLink->pageHash->hashList.hashPid.page != HEADERADDR))	{

				SM_ERROR(TYPE_LOG, esmFIXEDPAGES);
				return(esmFAILURE);
			}

			/*
			 *	get the next groupLink
			 */
			groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
		}

		/*
		 *	get a pointer to the next buffer group
		 */
		bufGroup = (BUFGROUP *) NEXT_LIST_ELEMENT( &(bufGroup->groupList) );
	}

	/*
	 *	get a pointer to the first in use buffer group
	 */
	bufGroup = (BUFGROUP *) FIRST_LIST_ELEMENT( &UsedBufGroups );

	/*
	 *	run through the open buffer group list
	 *	to flush all the dirty pages
	 */
	while (bufGroup != NULL)	{

		/*
		 *	check the buffer group magic number
		 */
		CHECK_BUFGROUP_MAGIC(bufGroup);

		/*
		 *	remove a buffer from the list
		 */
		groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(bufGroup->unfixedList) );

		/*
		 *	run down the list
		 */
		while (groupLink != NULL)	{

			/*
			 *	check the grouplink magic
			 */
			CHECK_GROUPLINK_MAGIC(groupLink);

			/*
			 *	check the pid to see if the volume matches
			 */
			if (groupLink->pageHash->hashList.hashPid.volid == volid)	{

				/*
				 *	get the pointer to the next entry
				 */
				tempLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );

				/*
				 *	take the link off the list
				 */
				listRemove( &(groupLink->groupList) );

				/*
				 *	flush the page
				 */
				BF_FreePage(bufGroup, groupLink);

				/*
				 *	setup the pointer to the next entry
				 */
				groupLink = tempLink;

			} else {

				/*
				 *	get the pointer to the next entry
				 */
				groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
			}
		}

		/*
		 *	get a pointer to the next buffer group
		 */
		bufGroup = (BUFGROUP *) NEXT_LIST_ELEMENT( &(bufGroup->groupList) );
	}

	/*
	 *	loop through the free lists
	 */
	for (i = 0; i < NumBufferSizes; i++)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("processing list:%d", i));
		
		/*
		 *	check to see if the list is empty
		 */
		if (LIST_NOT_EMPTY( &FreeBufTable[i] ))	{

			/*
			 *	Get a pointer to first element on the list
			 */
			groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(FreeBufTable[i]) );

			/*
			 *	Flush all pages on the list
			 */
			while (groupLink != NULL)	{

				/*
				 *	Check the groupLink magic number
				 */
				CHECK_GROUPLINK_MAGIC(groupLink);

				/*
				 *	Get a pointer to the page hash structure
				 */
				pageHash = groupLink->pageHash;

				/*
				 *	Check the pagehash magic number
				 */
				CHECK_PAGEHASH_MAGIC(pageHash);
				TRPRINT(TR_BF, TR_LEVEL_2, ("checking page:%d %d",
						pageHash->hashList.hashPid.page,
						pageHash->hashList.hashPid.volid));

				/*
				 *	check to see if the page is mapped
				 *	check to see if the page belongs to this volume
				 */
				if (LIST_MEMBER( &(pageHash->hashList.hashList)) &&
					(pageHash->hashList.hashPid.volid == volid)) { 

					/* 	Check to see if the page should be flushed */
					if (PAGE_IS_DIRTY(pageHash))  {

						/* write out the page */
						bf_ForcePage(pageHash, BF_SEM);
					}
					pageHash->flags = H_FREE;

					/*
					 *	now take the page off the hash list
					 */
					listRemove( &(pageHash->hashList.hashList) );
				}

				/*
				 *	Get a pointer to the next element
				 */
				groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
			}
		}
	}

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
