/*
 *   $RCSfile: bf_GetPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"


/*============================================================================*/
/* 
 * bf_GetPage:
 * 	Routine to insert a page into the page hash.
 *
 * RETURNS:
 *	esmNOERROR if all went ok and address of bucket via 
 *	hashPtr parameter.
 *
 * SIDE EFFECTS:
 *	The page is entered into the page hash.
 *
 * SPECIAL NOTE:
 *	No check is made to see whether the page is already in the hash.
 * 	This check should be made by the calling routine.
 */
/*============================================================================*/


 GROUPLINK 
*bf_GetPage (

	BUFGROUP	*bufGroup,
	const PID	*pid,
	FLAGS		flags,
	PAGE2SIZE	page2size 
)
{

	register GROUPLINK	*groupLink;


	TRPRINT(TR_BF, TR_LEVEL_1, ("getting buffer for page:%d page2size:%d",
			pid->page, page2size));

	/*
	 *	Manage the buffer group space
	 */
	if (BF_Swap(bufGroup, page2size))	{

		return(NULL);
	}

	/*
	 *	attempt to get the buffer.  This will increment
	 *	the fixed charges for this group.  Any error
	 *	code will be set by the routine.
	 */
	if ((groupLink = bf_GetPageBuf(bufGroup, page2size)) == NULL)	{

		return(NULL);
	}

	/*
	 *	attempt to insert the page into the hash table
	 */
	BF_InsertHashEntry(groupLink->pageHash, pid, NOFLAGS);

	/*
	 *	give back the semaphore
	 */
	if (!(flags & BF_SEM))	{

		signalSemaphore( &(groupLink->pageHash->semaphore) );
	}

	/*
	 *	return the groupLink
	 */
	return(groupLink);
}
