/*
 *   $RCSfile: bf_GetTempPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"


 GROUPLINK 
*bf_GetTempPage (

	PAGE2SIZE			page2size 
)
{

	GROUPLINK	*groupLink;
	int			attempts = 0;


	TRPRINT(TR_BF, TR_LEVEL_1, ("need page size:%d", BLOCKCOUNT(page2size)));

again:

	/*
	 *	Attempt to get a buffer off of the free list
	 *	of the requested size
	 */
	if (groupLink = (GROUPLINK *) listDeq( &(FreeBufTable[OFFSET(page2size)]) ))	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("got right size page"));

		/*
		 *	check the magic number
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);
		CHECK_PAGEHASH_MAGIC(groupLink->pageHash);
		TRPRINT(TR_BF, TR_LEVEL_2, ("index:%d size:%d", groupLink->pageHash->bufIndex,
				groupLink->pageHash->blockCount));

		/*
		 *	check to see that the page is the right size
		 */
		SM_ASSERT(LEVEL_3, groupLink->pageHash->page2size == page2size);

		/*
		 *	Invalidate the page
		 */
		BF_InvalidatePage(groupLink->pageHash, BF_SEM|BF_FORCE);

	} else if (groupLink = (GROUPLINK *) BF_GetLarger(page2size))	{

		/*
		 *	check the magic number
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);
		CHECK_PAGEHASH_MAGIC(groupLink->pageHash);

	} else if (groupLink = (GROUPLINK *) BF_GetSmaller(page2size))	{

		/*
		 *	check the magic number
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);
		CHECK_PAGEHASH_MAGIC(groupLink->pageHash);

	} else {

		if (attempts < 2) {

			/* 
			 *	Free up space in the primary buffer group and try again.
			 *	This is somewhat of a hack, but this case should
			 *	be rare.
			 */
			BF_SwapAll(UserBufGroup);
			attempts++;
			goto again;
		}

		/*
		 *	return the null pointer
		 */
		SM_ERROR(TYPE_USER, esmNOBUFSPACE);
		return(NULL);
	}
	TRPRINT(TR_BF, TR_LEVEL_2, ("index:%d size:%d", groupLink->pageHash->bufIndex,
			BLOCKCOUNT(groupLink->pageHash->page2size)));

	/*
	 *	set the member count to one to show its used
	 */
	groupLink->pageHash->memberCount = 1;

	/*
	 *	the semaphore has no meaning since page
	 *	does not have a valid pid
	 */
	initializeSemaphore( &(groupLink->pageHash->semaphore), FALSE );

	/*
	 * Set the return value and return.
	 */
	return(groupLink);
}
