/*
 *   $RCSfile: bf_IncreaseGroupSize.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"
#include "threadstate.h"


 int
bf_IncreaseGroupSize (

    register BUFGROUP   *bufGroup,
	register int		pagesNeeded,
	FLAGS				flags 
)
{

	register int		maxCharges;


	TRPRINT(TR_BF, TR_LEVEL_1, ("pagesNeeded:%d", pagesNeeded));

	/*
	 *	calculate the total number of buffers needed
	 */
	maxCharges = pagesNeeded + bufGroup->maxCharges;
	TRPRINT(TR_BF, TR_LEVEL_2, ("maxCharges:%d", maxCharges));

	/*
	 *	check to see that the request can be satisfied
	 */
	if (maxCharges > NumBuffers)	{

		SM_ERROR(TYPE_USER, esmREQUESTTOOBIG);
		return(esmFAILURE);
	}

	/*
	 *	check to make sure that there are no fixed buffers
	 */
	if (bufGroup->fixedCharges > 0)	{

		SM_ERROR(TYPE_USER, esmFIXEDPAGES);
		return(esmFAILURE);
	}

	/*
	 *	free any unfixed pages
	 */
	if (bufGroup->unfixedCharges > 0)	{

		bf_FreeUnfixed(bufGroup);
	}

	/*
	 *	add the currently allocated buffers to the global tally
	 */
	ReservedBufs -= bufGroup->maxCharges;	

again:

    /*
	 *	Check to see if there are sufficient buffers to fill the request
     */
    if (maxCharges + ReservedBufs > NumBuffers) {

		/*
		 *	Check to see if the caller desires to wait
		 */
		if (flags & BF_WAIT)	{

			/*
			 *	place ourselves on the wait list
			 */
			if (waitList(&BufGroupWaitList, THREAD_BUFGROUP_WAIT))	{

				/*
				 *	some problem has occured
				 */
				return(esmFAILURE);
			
			} else {
				
				/*
				 *	try to get the desired amount
				 */
				goto again;
			}

		} else {
				
			SM_ERROR(TYPE_WARNING, esmNOUNRESERVED);
			return esmNOERROR;
		}
    }

	/*
	 *	record the new max charges
	 */
    bufGroup->maxCharges = maxCharges;

	/*
	 *	increment the buffers in use
	 */
    ReservedBufs += maxCharges;

	/*
	 *	return a pointer to the allocated group
	 */
    return esmNOERROR;
}
