/*
 *   $RCSfile: bf_OpenServerGroups.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "bitmap.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "log_globals.h"
#include "bf_globals.h"
#include "io_globals.h"
#include "thread_globals.h"


 int
bf_OpenServerGroups (

	int				logWritePages,
	int				logReadPages 
)
{

	OPENLOG	*openLog;
	int		userGroupSize;


	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 *	Open the buffer group used for general client requests.
	 *	This buffer group should be as large as possible.
	 *	It contains all space not used by the log groups or
	 *	potentially needed for all mounted volumes.
	 *	
	 *	The group should at least be able to hold 2 full size pages
	 *	and is best if it can hold a full size page for each thread.
	 */
	userGroupSize = (int)(NumBuffers - logReadPages - logWritePages);
	userGroupSize -= BITMAP_GROUP_SIZE * NumVolumes;
	if (userGroupSize < NumThreads * BLOCKCOUNT(MAX_PAGE2SIZE)) {

		if (userGroupSize < 2 * BLOCKCOUNT(MAX_PAGE2SIZE)) {
			fprintf(sm_ErrorStream, "SERVER ERROR: insufficient buffer space (%d pages) - use at least %d pages\n", NumBuffers, (NumBuffers - userGroupSize) + 2 * BLOCKCOUNT(MAX_PAGE2SIZE));
			SM_ERROR(TYPE_STOP, esmINTERNAL);
		} else {
		
#ifdef COMMENT
			fprintf(sm_ErrorStream, "SERVER WARNING: low buffer space of only %d pages (%d preferred)\n", NumBuffers, (NumBuffers - userGroupSize) + NumThreads * BLOCKCOUNT(MAX_PAGE2SIZE));
#endif COMMENT

		}
	}
	if ((UserBufGroup = bf_OpenGroup(userGroupSize, BF_LRU, NOFLAGS)) == NULL)	{
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	PRINT_PROGRESS(("User buffer group size = %d\n", userGroupSize));


	/*
	 *	get a register pointer to the openlog structure
	 */
	openLog = &OpenLog;

	/*	attempt to get a buffer group for writing the log */
	if ((openLog->writeGroup = bf_OpenGroup(logWritePages, BF_LRU, NOFLAGS)) == NULL)	{
		return(esmFAILURE);
	}

	/*	attempt to get a buffer group for reading the log */
	if ((openLog->readGroup = bf_OpenGroup(logReadPages, BF_LRU, NOFLAGS)) == NULL)	{
		bf_CloseGroup(openLog->writeGroup);
		return(esmFAILURE);
	}

	return(esmNOERROR);
}
