/*
 *   $RCSfile: bf_PrintFixed.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"



#ifdef DEBUG

 void
bf_PrintFixed (

	BUFGROUP	*bufGroup 
)
{

	register GROUPLINK	*groupLink;

	
	TRACE(TR_BF, TR_LEVEL_1);

	/*
	 * Make sure that the group is open.
     */
	if (CHECK_BUFGROUP_VALID(bufGroup))	{

		SM_ERROR(TYPE_FATAL, esmBADGROUP);
	}

    /*
     *	Print the calculated number of pages in the group
     */
	TRPRINT(TR_BF, TR_LEVEL_2, ("fixedCharges:%d", bufGroup->fixedCharges));

	/*
	 *	get a pointer to the first element on the fixed list
	 */
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(bufGroup->fixedList) );

    /*
     * Free up the regions on the unfixed list.
     */
	while (groupLink != NULL)	{

		/*
		 *	check the grouplink magic
		 */
		CHECK_GROUPLINK_MAGIC(groupLink);

		/*
		 *	print the page information
		 */
		TRPRINT(TR_BF, TR_LEVEL_1, ("page id:%d",
				groupLink->pageHash->hashList.hashPid.page));

		/*
		 *	get the pointer to the next entry
		 */
		groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
	}
}

#endif
