/*
 *   $RCSfile: bf_UnfixPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:34 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "thread_globals.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "thread_funcs.h"

 void
bf_UnfixPage (

	register GROUPLINK	*groupLink,
	int					replacePolicy,
	BOOL				dirty 
)
{

	register BUFGROUP	*bufGroup;


	TRPRINT(TR_BF, TR_LEVEL_1, ("unfixing page:%d",
			groupLink->pageHash->hashList.hashPid.page));

	/*
	 *	check the magic number
	 */
	CHECK_GROUPLINK_MAGIC(groupLink);

	/*
	 *	put the buffer group in a register
	 */
	bufGroup = groupLink->bufGroup;

	/*
	 *	Check to see if the fixed count is correct
	 */
	SM_ASSERT(LEVEL_3, groupLink->fixedCount > 0);

	/*
	 *	check to see if page needs to be dirtied
	 */
	if (dirty)	{

		DIRTY_PAGE(groupLink->pageHash);
		TRPRINT(TR_BF, TR_LEVEL_2, ("marking page dirty"));
#ifdef DEBUG
		SM_ASSERT(LEVEL_3, (PPAGE_TYPE(groupLink->pageHash) == PAGE_LOG) ||
						   (groupLink->pageHash->lsn.offset > 0 &&
						    groupLink->pageHash->lrc.count > 0) );
#endif DEBUG
	}

	/*
	 *	Decrement the fixed count for the group
	 */
	if (--(groupLink->fixedCount) == 0)	{

		TRPRINT(TR_BF, TR_LEVEL_2, ("putting on unfixed"));

		/*
		 *	We must take it off the fixed list
		 */
		listRemove( &(groupLink->groupList) );

		/*
		 *	Put it on the free list according to the replacement policy
		 */
		BF_ReplacePolicy( bufGroup, &(groupLink->groupList), replacePolicy );

		/*
		 *	Decrement the fixed charges and increment the unfixed charges
		 */
		bufGroup->fixedCharges -= BLOCKCOUNT(groupLink->pageHash->page2size);
		bufGroup->unfixedCharges += BLOCKCOUNT(groupLink->pageHash->page2size);

		/*
		 *	check the fix count
		 */
		SM_ASSERT(LEVEL_3, (bufGroup->fixedCharges >= 0) || (bufGroup->unfixedCharges > 0));
	}

	TRPRINT(TR_BF, TR_LEVEL_2, ("fixed:%d unfixed:%d",
			bufGroup->fixedCharges, bufGroup->unfixedCharges));

	/*
	 *	check the status of the buffer group
	 */
	BF_AuditGroup(bufGroup);
}
