/*
 *   $RCSfile: BM_CheckFree.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"


 int
BM_CheckFree (

	FOUR				word,
	FOUR				page,
	FOUR				bit,
	FOUR				freeBits,
	FOUR				leftToAlloc,
	FREECONTEXT			*context,
	GROUPLINK			*groupLink 
)
{

	register BITCONTEXT	*bitContext;


	TRPRINT(TR_BITMAP, TR_LEVEL_1,
			("word:%d page:%d bit:%d freeBits:%d leftToAlloc:%d contextIndex:%d",
			word, page, bit, freeBits, leftToAlloc, context->contextIndex));

	/*
	 *	does the first word have to be unallocated
	 */
	if (context->contextIndex > 0)	{

		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("first bits must be free"));

		/*
		 *	check to see if the first bit is free
		 */
		if (bit != 0)	{

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("first bit is not free"));

			/*
			 *	cannot use this free run
			 */
			return(esmFAILURE);
		}
	}

	/*
	 *	we must use the second array element
	 */
	bitContext = &(context->bitContext[context->contextIndex]);

	/*
	 *	save the information in the context
	 */
	bitContext->bitOffset = bit;
	bitContext->bitCount  = freeBits;
	bitContext->word	  = word;
	bitContext->page	  = page;
	bitContext->groupLink = groupLink;

	/*
	 *	calculate and return the absolute offset of the bits
	 */
	 bitContext->absoluteBitOffset = PAGE_TO_BIT(bitContext->page) +
	     							 WORD_TO_BIT(bitContext->word) +
		 							 bitContext->bitOffset;
	TRPRINT(TR_BITMAP, TR_LEVEL_2, ("absolute offset:%d", bitContext->absoluteBitOffset));

	/*
	 *	decrement the number of needed bits
	 */
	leftToAlloc -= freeBits;
	TRPRINT(TR_BITMAP, TR_LEVEL_2, ("leftToAlloc:%d", leftToAlloc));

	/*
	 *	return how many we have left to find
	 */
	return(leftToAlloc);
}
