/*
 *   $RCSfile: BM_CheckWord.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"


 int
BM_CheckWord (

	register UFOUR	word,
	FOUR			firstBit,
	register FOUR	lastBit,
	register FOUR	leftToAlloc,
	FOUR			*contigStart 
)
{

	register FOUR	contigBits;
	register FOUR	bit;


	TRPRINT(TR_BITMAP, TR_LEVEL_1, ("word:%8.8x firstBit:%d lastBit:%d leftToAlloc:%d",
					word, firstBit, lastBit, leftToAlloc));
			
	/*
	 *	get rid of the uninteresting parts of the word
	 */
	word <<= firstBit;

	/*
	 *	search the remainder for contiguous free bits
	 */
	for (bit = firstBit, contigBits = 0;
		(bit <= lastBit) && (contigBits < leftToAlloc);
		bit++)	{

		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("bit:%d word:%8.8x contigBits:%d",
				bit, word, contigBits));

		/*
		 *	check the most significant bit for a one
		 */
		if (word & MSB_MASK)	{

			/*
			 *	tally another free bit
			 */
			if (contigBits++ == 0)	{

				/*
				 *	save where the free bits start
				 */
				*contigStart = bit;
				TRPRINT(TR_BITMAP, TR_LEVEL_2, ("contigStart:%d", *contigStart));
			}

		} else {

			/*
			 *	start the process over
			 */
			contigBits = 0;
		}

		/*
		 *	look at the next bit in the word
		 */
		word <<= 1;
	}

	/*
	 *	return the starting location and the count
	 */
	TRPRINT(TR_BITMAP, TR_LEVEL_2, ("contigBits:%d contigStart:%d",
			contigBits, *contigStart));
	return(contigBits);
}
