/*
 *   $RCSfile: BM_FreeBits.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"


 void
BM_FreeBits (

	register FREECONTEXT	*context 
)
{

	register BITCONTEXT		*bitContext;
	register FOUR			i;
	register UFOUR			*page;
	register UFOUR			mask;


	TRACE(TR_BITMAP, TR_LEVEL_1);

	/*
	 *	does the first word have to be unallocated
	 */
	for (i = 0, bitContext = &(context->bitContext[0]);
		 i <= context->contextIndex;
		 i++, bitContext++)	{

		 TRPRINT(TR_BITMAP, TR_LEVEL_2, ("word:%d bitOffset:%d bitCount:%d",
				 bitContext->word, bitContext->bitOffset, bitContext->bitCount));

		/*
		 *	get an integer pointer to the page
		 */
		page = (ulong *) bitContext->groupLink->bufFrame;

		/*
		 *	setup a mask for turning off the bit
		 */
		mask = ((((ulong) ALLSET_MASK)			/* mask with all bits set			*/
				<< (32 - bitContext->bitCount)) /* get mask with correct # bits set	*/
				>> (bitContext->bitOffset));	/* shift them into corrent position	*/
		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("mask:%8.8x", mask));

		/*
		 *	set the word in the page to allocate the bits
		 */
		page[bitContext->word] |= mask;
		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("new word:%8.8x", page[bitContext->word]));
	}
}
