/*
 *   $RCSfile: BM_ListUnfix.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "page.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_globals.h"


 void
BM_ListUnfix (

	PID						*pid,
	LIST					*pageList 
)
{

	register PAGECONTEXT	*context;


	TRPRINT(TR_BITMAP, TR_LEVEL_1, ("unfixing pid:%d", pid->page));

	/*
	 *	get a pointer to the first node of the context list
	 */
	context = (PAGECONTEXT *) FIRST_LIST_ELEMENT(pageList);

	/*
	 *	does the first word have to be unallocated
	 */
	while (context != NULL)	{

		/*
		 *	check the context magic number
		 */
		CHECK_PAGECONTEXT_MAGIC(context);
		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("context page:%d", context->pid.page));

		/*
		 *	check to see if the page is found
		 */
		if (PIDEQ(context->pid, *pid))	{

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("found page fixCount:%d", context->fixCount));

			/*
			 *	increment the reference count
			 */
			if (--(context->fixCount) == 0)	{

				/*
				 *	take the element off the page list
				 */
				listRemove( &(context->list) );

				/*
				 *	release the semaphore on the page and
				 *	unfix it in the buffer pool
				 */
				signalSemaphore( &(context->groupLink->pageHash->semaphore) );
				bf_UnfixPage(context->groupLink, BF_DEFAULT, FALSE);

				/*
				 *	give the page back to the pool
				 */
				poolEnq( &PageContextPool, &(context->list) );

				/*
				 *	we are done
				 */
				return;
			}
		}

		/*
		 *	look at the next context
		 */
		context = (PAGECONTEXT *) NEXT_LIST_ELEMENT( &(context->list) );
	}

	/*
	 *	if we get here something is wrong
	 */
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
}
