/*
 *   $RCSfile: bm_FindContigBits.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"
#include "bm_globals.h"
#include "thread_globals.h"


 FREECONTEXT
*bm_FindContigBits (

	register VOLREC		*volRec,
	register FOUR		bitsNeeded,
	register FOUR		initBit,
	register FOUR		finalBit,
	LIST				*pageList 
)
{

	register FOUR		currentWord;
	register FOUR		lastWord;
	register UFOUR		*wordPtr;
	register FOUR		lastBit;
	register FOUR		firstBit;
	PID				currentPid;
	FREECONTEXT			*context;
	GROUPLINK			*groupLink;
	PID					pinList[2];
	SHORTPID			firstPage;
	SHORTPID			lastPage;
	FOUR				leftToAlloc;
	FOUR				contigStart;
	FOUR				freeBits;
	FOUR				currentBuf;


	TRPRINT(TR_BITMAP, TR_LEVEL_1, ("bitsNeeded:%d initBit:%d finalBit:%d",
			bitsNeeded, initBit, finalBit));

	/*
	 *	check to see if we are handling too many bits
	 */
	if (MAX_BIT_PAGE_SPAN(bitsNeeded) > 2)	{

		SM_ERROR(TYPE_WARNING, esmTOOMANYBITS);
		return(NULL);
	}

	/*
	 *	initialize the first and the last pages
	 */
	firstPage =	  BIT_TO_PAGE(initBit);
	lastPage =	  BIT_TO_PAGE(finalBit);
	currentWord = BIT_TO_WORD(BIT_IN_PAGE(initBit));
	firstBit =	  BIT_IN_WORD(initBit);
	TRPRINT(TR_BITMAP, TR_LEVEL_2, ("firstPage:%d lastPage:%d currentWord:%d firstBit:%d",
			firstPage, lastPage, currentWord, firstBit));

	/*
	 *	get a context block
	 */
	if ((context = (FREECONTEXT *) poolDeq( &(FreeContextPool) )) == NULL)	{

		SM_ERROR(TYPE_WARNING, esmNOFREECONTEXT);
		return(NULL);
	}
	CHECK_FREECONTEXT_MAGIC(context);

	/*
	 *	initialize the context to hold no entries
	 */
	context->contextIndex = -1;

	/*
	 *	setup the loop parameters
	 */
	currentPid.volid = volRec->header->volid;
	leftToAlloc = bitsNeeded;
	currentBuf = 0;
	pinList[0].page = pinList[1].page = NIL;

	/*
	 *	loop through the bitmap free pages until we have found
	 *	the number of bits that we want
	 */
	for (currentPid.page = firstPage;
		 currentPid.page <= lastPage;
		 currentPid.page++, currentWord = 0, firstBit = 0)	{

		 TRPRINT(TR_BITMAP, TR_LEVEL_2, ("contextIndex:%d", context->contextIndex));

		/*
		 *	if we get to here we must read in the page
		 */
		if ((groupLink = bm_ListRead(volRec, &currentPid, pageList)) == NULL)	{

			SM_ERROR(TYPE_WARNING, Active->errno);
			return(NULL);
		}

		/*
		 *	save the grouplink in the context
		 *	and in the buffer array
		 */
		pinList[currentBuf] = currentPid;	

		/*
		 *	check to see if we are on the last page
		 */
		if (currentPid.page != lastPage)	{

			/*
			 *	go to last word on page
			 */
			lastWord = WORDS_IN_PAGE - 1;

		} else {

			/*
			 *	calculate the last word to search
			 */
			lastWord = BIT_TO_WORD(finalBit - PAGE_TO_BIT(lastPage));
		}
		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("lastWord:%d", lastWord));

		/*
		 *	look for words in the page that have unallocated bits
		 */
		for (wordPtr = &(((UFOUR *) groupLink->bufFrame)[currentWord]);
			 currentWord <= lastWord;
			 currentWord++, wordPtr++, firstBit = 0)	{

			 TRPRINT(TR_BITMAP, TR_LEVEL_2, ("currentWord:%d contextIndex:%d",
			 				 currentWord, context->contextIndex));

			/*
			 *	save the grouplink in the context
			 *	and in the buffer array
			 */
			context->bitContext[++(context->contextIndex)].groupLink = groupLink;
			context->bitContext[context->contextIndex].page = currentPid.page;
			SM_ASSERT(LEVEL_3, context->contextIndex < MAX_BIT_WORD_SPAN);

			/*
			 *	we must only go to the last valid bit
			 *	in the range specified
			 */
			if ((currentWord == lastWord) && (currentPid.page == lastPage))	{

				/*
				 *	go to the last valid bit on the page
				 */
				lastBit	= BIT_IN_WORD(finalBit);

			} else {

				/*
				 *	go to the last bit in the word
				 */
				lastBit = BITSPERWORD - 1;
			}
			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("lastBit:%d", lastBit));

			/*
			 *	check to see if the word has any free bits
			 */
			if (*wordPtr == 0)	{

				TRPRINT(TR_BITMAP, TR_LEVEL_2, ("current word is zero"));

				/*
				 *	start at the beginning of the process
				 */
				leftToAlloc = bitsNeeded;
				context->contextIndex = -1;
				continue;
			}

			/*
			 *	look within the word for free bits
			 */
			if ((freeBits = BM_CheckWord(*wordPtr, firstBit,
							lastBit, leftToAlloc, &contigStart)) == 0)	{

				TRPRINT(TR_BITMAP, TR_LEVEL_2, ("freeBits:%d contigStart:%d",
						freeBits, contigStart));

				/*
				 *	start looking all over again
				 */
				leftToAlloc = bitsNeeded;
				context->contextIndex = -1;
				continue;
			}

			/*
			 *	determine the location of the first free bit
			 */
			leftToAlloc = BM_CheckFree(currentWord, currentPid.page, contigStart,
								 	   freeBits, leftToAlloc, context, groupLink);

			/*
			 *	check to see if the free bits meet the criteria
			 */
			if (leftToAlloc < 0)	{

				/*
				 *	the bits did not meet the criteria
				 */
				wordPtr--;
				currentWord--;
				leftToAlloc = bitsNeeded;
				context->contextIndex = -1;
				continue;

			} else if (leftToAlloc > 0)	{

				/*
				 *	still need more bits
				 */
				continue;

			} else {

				TRPRINT(TR_BITMAP, TR_LEVEL_2, ("found all bits"));
				
				/*
				 *	mark the bits as allocated
				 */
				BM_AllocBits(context);

				/*
				 *	we have the bits we need
				 */
				return(context);
			}
		}

		/*
		 *	at this point we must determine if we have
		 *	any previous page context that we need
		 */
		if (context->contextIndex < 0)	{

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("don't need previous page context"));

			/*
			 *	unfix this page
			 */
			BM_ListUnfix( &(pinList[currentBuf]), pageList );
			pinList[currentBuf].page = NIL;

			/*
			 *	wrap to the previous page
			 */
			currentBuf ^= 1;

		} else {

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("need current page context"));

			/*
			 *	wrap the pointer to the next element
			 */
			currentBuf ^= 1;
		}

		/*
		 *	free the 2nd last page if necessary
		 */
		if (pinList[currentBuf].page != NIL)	{

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("unfixing previous page:%d",
					pinList[currentBuf]));
			BM_ListUnfix( &(pinList[currentBuf]), pageList );
			pinList[currentBuf].page = NIL;
		}
	}
	TRPRINT(TR_BITMAP, TR_LEVEL_2, ("could not find bits"));

	/*
	 *	unfix last page if necessary
	 */
	currentBuf ^= 1;
	if (pinList[currentBuf].page != NIL)	{

		TRPRINT(TR_BITMAP, TR_LEVEL_2, ("unfixing previous page:%d", pinList[currentBuf]));
		BM_ListUnfix( &(pinList[currentBuf]), pageList );
	}

	/*
	 *	free up the context block
	 */
	poolEnq( &FreeContextPool, &(context->list) );	

	/*
	 *	return that we did not find any suitable bits
	 */
	return(NULL);
}
