/*
 *   $RCSfile: bm_ListDepend.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "bitmap.h"
#include "log.h"
#include "bm_extfuncs.h"


 void
bm_ListDepend (

	LIST					*pageList,
	FORCEMARK				forceMark,
	LSN						*lsn,
	LRC						lrcList[] 
)
{

	register PAGECONTEXT	*context;
	register PAGEHASH		*pageHash;
	LRC		 *lrc;


	TRACE(TR_BITMAP, TR_LEVEL_1);

	/*
	 *	does the first word have to be unallocated
	 */
	lrc = lrcList;  /* start at the first lrc */
	context = (PAGECONTEXT *) FIRST_LIST_ELEMENT(pageList);
	while (context != NULL)	{

		/*
		 *	check the context magic number
		 */
		CHECK_PAGECONTEXT_MAGIC(context);
		CHECK_GROUPLINK_MAGIC(context->groupLink);

		/*
		 *	check to see if the forcemark and lsn need to be set
		 */
		SM_ASSERT(LEVEL_3, forceMark != 0);

		/*
		 *	get a pointer to the pagehash
		 */
		pageHash = context->groupLink->pageHash;

		/*
		 *	setup the specified log dependency
		 */
		DEPEND_LOG(pageHash, forceMark, lsn, lrc);

		/*
		 *	Go to the next lrc
		 */
		lrc++;
		context = (PAGECONTEXT *) NEXT_LIST_ELEMENT(&(context->list));
	}
}
