/*
 *   $RCSfile: bm_ListRead.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "page.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "bm_intfuncs.h"
#include "bm_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_globals.h"
#include "thread_globals.h"


 GROUPLINK
*bm_ListRead (

	VOLREC					*volRec,
	PID						*pid,
	LIST					*pageList 
)
{

	register PAGECONTEXT	*context;
	register GROUPLINK		*groupLink;
	PID						localPid;


	TRPRINT(TR_BITMAP, TR_LEVEL_1, ("reading pid:%d", pid->page));

	/*
	 *	get a pointer to the first node of the context list
	 */
	context = (PAGECONTEXT *) FIRST_LIST_ELEMENT(pageList);

	/*
	 *	does the first word have to be unallocated
	 */
	while (context != NULL)	{

		/*
		 *	check the context magic number
		 */
		CHECK_PAGECONTEXT_MAGIC(context);

		/*
		 *	check to see if the groupLink is found
		 */
		if (PIDEQ(context->pid, *pid))	{

			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("found page already read"));

			/*
			 *	increment the reference count
			 */
			context->fixCount++;
			TRPRINT(TR_BITMAP, TR_LEVEL_2, ("fixed count:%d", context->fixCount));

			/*
			 *	return a pointer to the buffer
			 */
			return(context->groupLink);
		}

		/*
		 *	look at the next context
		 */
		context = (PAGECONTEXT *) NEXT_LIST_ELEMENT( &(context->list) );
	}

	/*
	 *	attempt to get another context struct
	 */
	if ((context = (PAGECONTEXT *) poolDeq( &PageContextPool )) == NULL)	{

		SM_ERROR(TYPE_WARNING, esmNOFREEPAGECONTEXT);
		return(NULL);
	}
	CHECK_PAGECONTEXT_MAGIC(context);

	/*
	 *	we must read from the offset of the bitmap
	 */
	localPid.volid = pid->volid;
	localPid.page = pid->page + volRec->header->freeBitmapAddr;

	/*
	 *	if we get to here we must read in the page
	 */
	if ((groupLink = bf_ReadPage(volRec->bufGroup, &localPid,
								 BITMAP_PAGE2SIZE, BF_SEM)) == NULL)	{

		/*
		 *	free the context since we don't need it
		 */
		poolEnq( &(PageContextPool), &(context->list) );

		/*
		 *	record an error here
		 */
		SM_ERROR(TYPE_WARNING, Active->errno);
		return(NULL);

	} else {

		/*
		 *	set the context information
		 */
		context->pid = *pid;
		context->groupLink = groupLink;
		context->fixCount = 1;
		listPush( pageList, &(context->list) ); 

		/*
		 *	return the read pointer
		 */
		return(groupLink);
	}
}
