/*
 *   $RCSfile: cl_AllocPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "consist.h"
#include "file.h"
#include "openlog.h"
#include "logrecs.h"
#include "deallocinfo.h"
#include "sm_macro.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "lm_extfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "fi_extfuncs.h"
#include "fi_intfuncs.h"
#include "lg_extfuncs.h"
#include "log_extfuncs.h"
#include "sm_extfuncs.h"
#include "cl_funcs.h"
#include "bf_globals.h"
#include "io_globals.h"
#include "thread_globals.h"

#define INCLUDED
#include "BTREEPAGE.h"



 void
cl_AllocPages (

	register MESSAGE	*message,
	register LINK		*link 
)
{
	TRANSREC			*transRec;
	int					i;
	PID					pidList[MAX_PIDLIST_SIZE];
	PID					*nearPid;
	GROUPLINK			*newLink;
	SLOTTEDPAGE			*newPage;
	LSNOFFSET			saveLSN = NULL_LSN;
	BOOL				do_crash = TRUE; /* most cases of
		failures aren't implemented */
	PAGE2SIZE			page2size;
	FOUR				numPages;
	VOLREC				*volRec;
	FID					*fid;
	GROUPLINK			*rootLink;
	LOCKMODE			fileLock;
	LOCKMODE			pageLock;
	LGNODE				*lgNode;
	LGDATAPAGE			*lgDataPage;
	UNIQUE      		output_unique;
	SHORTPID    		output_logicalPredPid;
	PAGEDEALLOCINFO		*pageDeallocInfo ;

/*
 * a debugging macro: helps identify the source of the
 * crash -- see label Crash:
 */
#ifdef DEBUG
	int						__line__;
#define GOTO				__line__ = __LINE__; goto
#else
#define GOTO goto
#endif DEBUG



	TRPRINT(TR_CL, TR_LEVEL_1, ("allocating %d pages", message->body.alloc.numPages));

	/*
	 *	attempt to find the transaction record, TRUE => writing log
	 */
	if ((transRec = registerTrans(message->header.params.in.tid, TRUE)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	check to see if we have a near pid
	 */
	if (message->body.alloc.nearPid.page != NULLPID)	{

		nearPid = &(message->body.alloc.nearPid);
		TRPRINT(TR_CL, TR_LEVEL_2, ("nearPid:%d", nearPid->page));

	} else {

		nearPid = NULL;
	}

	/*
	 *	put variables in registers
	 */
	page2size = message->body.alloc.page2size;
	numPages  = message->body.alloc.numPages;
	SM_ASSERT(LEVEL_1, message->body.alloc.numPages <= MAX_PIDLIST_SIZE);

	/*
	 *	get the volume record
	 */
	if ((volRec = io_FindVolumeIdWithLink(message->body.alloc.volid, link)) == NULL){
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	switch (message->body.alloc.pageType) {
	case PAGE_SLOTTED:
		/*
		 *	If this code is to handle the allocation of more
		 *	than one slotted page, it will need some for loops
		 *	added.
		 */
		SM_ASSERT(LEVEL_3, numPages == 1);

		/*
		 *	get a pointer to the fid
		 */
		fid = &(message->body.alloc.slotted.s_input.fid);
		if (fi_ValidateFid(UserBufGroup, fid) != esmNOERROR) {
			reply(esmFAILURE, Active->errno, 1, TRUE);

		}
		/* prepare for undoing the transaction if error occurs */
		saveLSN = transRec->lastLSN;
		do_crash = FALSE;
		break;

	case PAGE_LARGENODE:

		SM_ASSERT(LEVEL_3, page2size == LG_PAGE2SIZE);
		break;

	case PAGE_LARGEDATA:
	
		SM_ASSERT(LEVEL_3, page2size == LGDATA_PAGE2SIZE);
		break;
		
	case PAGE_INDEXDESC:
	
		SM_ASSERT(LEVEL_3, page2size == INDEXDESC_PAGE2SIZE);
		break;

	case PAGE_INDEX:
		SM_ASSERT(LEVEL_3, page2size == BTREE_PAGE2SIZE);
		break;

	default:
		SM_ERROR(TYPE_LOG, esmINTERNAL);
		reply(esmFAILURE, Active->errno, 1, TRUE);

	} /* end switch */

	if (io_AllocPages(volRec, page2size,
						  numPages, nearPid, pidList)) {
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	if (message->body.alloc.pageType == PAGE_SLOTTED) {
		/*
		 *	get unique numbers for the page
		 */
		if (io_GetUnique(pidList, &output_unique,
					 message->body.alloc.slotted.s_input.uniqueCount))	{
			GOTO UndoTx;
		}	

		/*
		 *	Determine what type of file this is, so we can lock it
		 *	correctly.  First read in the root page of the file.
		 */
		if ((rootLink = bf_ReadPage(UserBufGroup, &fid->pid, 
									FILE_PAGE2SIZE, BF_SEM)) == NULL) {
			GOTO UndoTx;
		}

		if ( ((NODEPAGE*)rootLink)->header.flags & LOGICAL_ORDER) {
			 /* File is ordered */
			fileLock = SIX;
		} else {
			/* File is not ordered */
			fileLock = IX;
		}

		/*
		 *	Release the semaphore, and unfix the file root page
		 */
		signalSemaphore( &(rootLink->pageHash->semaphore) );
		bf_UnfixPage(rootLink, BF_DEFAULT, FALSE);

		/*
		 *	attempt to lock the file
		 */
		if (lockFile(transRec, fid, &fileLock, NOFLAGS))	{
			GOTO UndoTx;
		}
	} /* end (message->body.alloc.pageType == PAGE_SLOTTED)  */
	if( (message->body.alloc.pageType == PAGE_SLOTTED)
		|| (message->body.alloc.pageType == PAGE_INDEX)) {

		/*
		 *	set the write locks for all of the pages
		 */
		pageLock = EX;
		for (i = 0; i < message->body.alloc.numPages; i++)	{
			if (lockPage(transRec, &(pidList[i]), &pageLock, NOFLAGS))	{
				/* we could have timed out */
				GOTO UndoTx;
			}
			SM_ASSERT(LEVEL_3, pageLock == EX);
		}
	}

	for (i = 0; i < numPages; i++) {
		SM_ASSERT(LEVEL_3, bf_FindPage(pidList+i, NOFLAGS) == NULL);

		/*
		 *	Buffer the page, there is no need to read it
		 */
		if ((newLink = bf_ReadPage(UserBufGroup, pidList+i, page2size,
									   BF_NOREAD|BF_SEM)) == NULL) {
			GOTO Crash;
		}
		newLink->pageHash->pageType = message->body.alloc.pageType;

		switch (message->body.alloc.pageType) {
		case PAGE_SLOTTED:
			newPage = (SLOTTEDPAGE *) newLink->bufFrame;

			/*
			 *	format the page
			 */
			INITPAGE(newPage, pidList[0], *fid, FALSE, output_unique);

			/*
			 *	log the slotted page format
			 */
			if (sm_LogSlottedFormat(pidList, newLink->pageHash))	{
				GOTO Crash;
			}

			/*
			 *	mark the page as slotted
			 */
			if (io_MarkPage(volRec, pidList, PAGE_SLOTTED, TRUE))	{
				GOTO Crash;
			}

			/* 	insert the page in the file */
			if (fi_InsertPage(UserBufGroup, fid, 
				pidList[0].page, message->body.alloc.flags,
							  &output_logicalPredPid))	{
				GOTO Crash;
			}
			TRPRINT(TR_CL, TR_LEVEL_2, ("page:%d", pidList[0].page));

#ifdef JUNK
			/*
			 *	See if this file is a newly created one.  If so, there
			 * 	there is no need to log a compensation for the allocation
			 *	since no other transaction can depend upon the allocation.
			 *	Otherwise, log the allocation
			 */
			fileCreateInfo = (FILEDEALLOCINFO*) FIRST_LIST_ELEMENT(&(transRec->fileCreateList));
			while(fileCreateInfo != NULL) {
				if (FIDEQ(fileCreateInfo->fid, *fid)) {
					break;
				}
				fileCreateInfo = (FILEDEALLOCINFO*) NEXT_LIST_ELEMENT(&(fileCreateInfo->list));
			}
			if (fileCreateInfo == NULL) { /*}*/
#endif /* JUNK */

			/*
			 *	log a compensation record
			 */
			if (FI_LogKeyInsertDone(fid, pidList[0].page, saveLSN))	{
				/* don't roll back, don't crash either */
				do_crash = FALSE;
				saveLSN = NULL_LSN;
				GOTO Crash;
			}
			STORE_THREAD_SWITCH(Active);
			break;

		case PAGE_LARGENODE:
			lgNode = (LGNODE*) newLink->bufFrame;
			INIT_LARGENODE_MAGIC(lgNode);

			/*	log the format of the lg object index node */
			if (lg_LogNodeFormat(&(pidList[i]), newLink->pageHash) 
													!= esmNOERROR)	{
				GOTO Crash;
			}
			break;

		case PAGE_LARGEDATA:
			lgDataPage = (LGDATAPAGE*) newLink->bufFrame;
			INIT_LGDATAHDR_MAGIC(lgDataPage);

			/*	log the format of the lg object index node */
			if (lg_LogDataFormat(&(pidList[i]), newLink->pageHash) 
													!= esmNOERROR)	{
				GOTO Crash;
			}
			break;
			
		case PAGE_INDEXDESC:
			/* 	Log the format of the index descriptor page */
			if (sm_LogIndexDescFormat(pidList+i, newLink->pageHash))  {
				GOTO Crash;
			}
			break;

		case PAGE_INDEX:
			/*	Log the format of the btree page */
		if (bt_LogFormat(volRec, &(pidList[i]), newLink->pageHash)) {
				GOTO Crash;
			}
			break;

		default:
			SM_ERROR(TYPE_CRASH, esmINTERNAL);
		} /* end switch */

		signalSemaphore( &(newLink->pageHash->semaphore) );
		CHECK_THREAD_SWITCH(Active);
		bf_UnfixPage(newLink, BF_DEFAULT, TRUE);
	} /* end for each of the numPages  */

	/*
	 *	If this is a temporary volume, and the pages are not slotted
	 *	then record them as pages to be deallocated at end-of-tran.
	 */
	if ( (volRec->volflags & VOL_TEMP) &&
		 (message->body.alloc.pageType != PAGE_SLOTTED) ) {

		for (i = 0; i < numPages; i++) {

			/* get a place to record that the page is to be dealloced */
			pageDeallocInfo = (PAGEDEALLOCINFO*) poolDeq(&PageDeallocInfoPool);
			if (pageDeallocInfo == NULL) {
				SM_ERROR(TYPE_CRASH, esmINTERNAL);
			}
			CHECK_PAGEDEALLOCINFO_MAGIC(pageDeallocInfo);

			/* attach the page info to the transaction */
			pageDeallocInfo->pid = pidList[i];
			pageDeallocInfo->page2size = page2size;
			pageDeallocInfo->pageCount = 1;
			listEnq(&(transRec->tempPageList), &(pageDeallocInfo->list));
		}
	}

	/*
	 *	setup the iovec for the transfer back
	 */
	Active->iovec[1].iov_base = (char *) pidList;
	Active->iovec[1].iov_len = sizeof(PID) * message->body.alloc.numPages;

	message->body.alloc.slotted.s_output.unique = output_unique;
	message->body.alloc.slotted.s_output.logicalPredPid = output_logicalPredPid;

	reply(esmNOERROR, esmNOERROR, 2, TRUE);
	/* not reached */

Crash: 
	/*
	 *	In order to handle this error, all fixed pages
	 *	must be discared.  The code following the error	
	 * 	handler does not discard them all.  A routine
	 *	should be implemented to discard them (it 
	 *	could be used for lg obj data pages as well.
	 *
	 *  Under a debugger, we can look at __line__ to see
	 *  the source of the problem.
	 */
	if(do_crash) {
		SM_ERROR(TYPE_CRASH, esmINTERNAL);
	}
	notify( &(newLink->pageHash->semaphore.waitList),
									esmFAILURE, esmPAGEGONE);
	initializeSemaphore( &(newLink->pageHash->semaphore), TRUE );
	bf_DiscardPage(UserBufGroup, newLink);

	/* drop through */

UndoTx:
	if(saveLSN != NULL_LSN) {
		/*
		 *	roll back the transaction to the save point
		 */
		undoTransaction(saveLSN, NULL_LSN, NOFLAGS);
	}
	reply(esmFAILURE, Active->errno, 1, TRUE);
	/* end of UndoTx, Crash */
}
