/*
 *   $RCSfile: cl_CreateFile.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "consist.h"
#include "deallocinfo.h"
#include "cl_funcs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "fi_extfuncs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "io_globals.h"


 void
cl_CreateFile (

	register MESSAGE	*message,
	register LINK		*unused 
)
{

	TRANSREC		*transRec;

	TRPRINT(TR_FI, TR_LEVEL_1, ("creating file")); 

	/*
	 *	attempt to find the transaction record, TRUE => writing log
	 */
	if ((transRec = registerTrans(message->header.params.in.tid, TRUE)) == NULL) {

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	attempt to allocate the pages
	 */
	if (message->body.file.fid.pid.page != NULLPID)	{

		if (fi_CreateFile(message->body.file.fid.pid.volid,
						  UserBufGroup, &(message->body.file.fid.pid),
						  &(message->body.file.fid), message->body.file.flags) )	{

			reply(esmFAILURE, Active->errno, 1, TRUE);

		}

	} else {

		if (fi_CreateFile(message->body.file.fid.pid.volid,
						  UserBufGroup, NULL, &(message->body.file.fid),
						  message->body.file.flags) )	{

			reply(esmFAILURE, Active->errno, 1, TRUE);

		}
	}

	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}
