/*
 *   $RCSfile: cl_DestroyFile.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "consist.h"
#include "deallocinfo.h"
#include "page.h"
#include "cl_funcs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "fi_extfuncs.h"
#include "lm_extfuncs.h"
#include "io_extfuncs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "io_globals.h"


 void
cl_DestroyFile (

	register MESSAGE	*message,
	register LINK		*link 
)
{

	TRANSREC	*transRec;
	FID			*fid;
	LOCKMODE	lockMode;
	VOLREC		*volRec;


	TRPRINT(TR_FI, TR_LEVEL_1, ("creating file"));

	fid = &(message->body.file.fid);

	/*
	 *	attempt to find the transaction record
	 */
	if ((transRec = findTransRec(message->header.params.in.tid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	put the transrec in the tcb
	 */
	Active->transRec = (char *) transRec;

	if (fi_ValidateFid(UserBufGroup, fid) != esmNOERROR) {
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	See if the volume is temporary
	 */
	volRec = io_FindVolumeIdWithLink(fid->pid.volid, link);
	if (volRec == NULL) {
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}
	if (volRec->volflags & VOL_TEMP) {

		FILEDEALLOCINFO		*currFile;

		/*
		 *	destroy the file now rather than delaying until commit
		 *
		 *	The destroy should be shallow since large object pages
		 *	will be deallocated at commit.  Actually, they
		 *	should be deallocated now to, but support for this
		 *	requires extra checks at commit time.
		 */
		if (fi_DestroyFile(UserBufGroup, &(fid->pid), NULL, TRUE, FALSE) != esmNOERROR) {
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}

		/* Mark the root page of the file as no longer a file page. */
		if (io_MarkPage(volRec, &(fid->pid), PAGE_FILEHDR, FALSE) != esmNOERROR)   {
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}

		/*
		 *	Remove the file from the list of those created by this
		 *	transaction.
		 */
		currFile = (FILEDEALLOCINFO*) FIRST_LIST_ELEMENT(&(transRec->tempFileList));
		while (currFile != NULL) {

			if (PIDEQ(currFile->fid.pid, fid->pid) &&
				(currFile->fid.unique == fid->unique) ) {
				break;
			}
	
			currFile = (FILEDEALLOCINFO*) NEXT_LIST_ELEMENT(&(currFile->list));
		}
		if (currFile == NULL) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
		poolMove(&FileDeallocInfoPool, &(currFile->list));

	} else {

		/*
		 *	attempt to lock the file
		 */
		lockMode = EX;
		if (lockFile(transRec, fid, &lockMode, NOFLAGS))	{

			reply(esmFAILURE, Active->errno, 1, TRUE);
		}

		/*
		 *	Record that the file should be destroyed at commit
		 */
		if (fi_RecordFileForDestruction(fid) ) {
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}
	}
	reply(esmNOERROR, esmNOERROR, 1, TRUE);

}
