/*
 *   $RCSfile: cl_DismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "disk.h"
#include "bitvec.h"
#include "admin_funcs.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "disk_funcs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"

/*
 *	Includes for temporary hack, in code below
 */
#include "openlog.h"
#include "log_globals.h"

/*
 *	Note that there cannot be any outstanding disk requests when the
 *	last close request comes in.  This is enforced both in client
 *	and the server processes.
 */


 void
cl_DismountVolume(

	MESSAGE	*message,
	LINK		*link 
)
{

	register VOLREC		*volRec;


	TRPRINT(TR_ADMIN, TR_LEVEL_1, ("dismounting volume:%d", message->body.mount.volid));

	/*
	 *	check to see if the volid is valid
	 */
	if (message->body.mount.volid == NULLVOLID)	{

		reply(esmFAILURE, esmBADVOLID, 1, TRUE);
	}

	/*
	 *	find the entry in the volume table
	 */
	if ((volRec = io_FindVolRec(message->body.mount.volid)) == NULL)	{

		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	take the volume off the link
	 */
	unlinkVolume(link, volRec);

	/*
	 *	check to see if there are any other opens on the volume
	 *	NOTE: Make sure volume is no the log volume, before dismounting
	 *		  it.  This is a hack until a better solution is found.
	 */
	if (LIST_EMPTY( &(volRec->openList) ) && 
		(OpenLog.volid != volRec->header->volid) )	{

		TRPRINT(TR_ADMIN, TR_LEVEL_2, ("last open on volume"));

		/*
		 *	check to see if there are active transactions for this volume
		 */
		if (LIST_EMPTY( &(volRec->activeTransList) ))	{

			/*
			 *	flush the buffers for this volume
			 */
			if (bf_DismountVolume(volRec, TRUE) != esmNOERROR)	{

				SM_ERROR(TYPE_FATAL, Active->errno);
			}

		} else {

			/*
			 *	Active transactions, so don't do anything
			 */
#ifdef JUNK
			/*
			 *	mark the disk state as quiesce
			 */
			volRec->flags |= VOL_QUIESCE;
#endif /* JUNK */
			TRPRINT(TR_ADMIN, TR_LEVEL_2, ("active trans on volume"));
		}
	}

	/*
	 *	reply with success
	 */
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}


 void
cl_RemoveVolume (
	register MESSAGE	*message,
	register LINK		*link 
)
{
	int e;

	TRACE(TR_CL, TR_LEVEL_1);
	SM_ASSERT(LEVEL_1, message->header.type == RM_VOLUME);

	suspendLink(link);
	Active->errno = esmNOERROR;

	if( (e = removecommand(message->body.volume.volid, NULL)) != esmNOERROR) {
		SM_ERROR(TYPE_USER, Active->errno);
	}
	resumeLink(link);

	if(e == esmNOERROR) {
		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	} else {
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}
}
