/*
 *   $RCSfile: cl_ExternalActions.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "openlog.h"
#include "logrecs.h"
#include "log_globals.h"
#include "log_intfuncs.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "threadstate.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_intfuncs.h"
#include "distr_globals.h"
#include "server_util_funcs.h"


static 
void 
junkServerAddrTids (

	LINK		*link,
	VOID		*unused,
	int			totalToRead
)
{
	SERVERADDRTID 	serverAddrTid;
	int				numRead = 0;
	int				length = sizeof(SERVERADDRTID); 
	TCB				*tcb;

	/*
	 *	get a pointer to this thread
	 */
	tcb = Active;

	/*
	 *	read the list of SERVERADDRTIDs and discard them
	 */
	while (numRead < totalToRead) {

		if (receive(link, (char *) &serverAddrTid, length) < 0) {

			/*
			 *	restart this thread
			 */
			transThreadDeath(tcb);
			threadRestart();
		}
		else 
			numRead++;
	}
}

/*
 *	This routine is called by the coordinator upon receiving a 
 *	command message from a client for a distributed transaction
 *	The coordinator tries to commit or abort the distributed
 *	transaction based on the command
 */

 void
cl_ExternalCommand (

	MESSAGE				*message,
	LINK				*link 
)
{

	register TRANSREC	*transRec;
	COMMAND				command =  message->body.command.command;
	TCB					*tcb;


	TRPRINT(TR_TRANS|TR_DISTR, TR_LEVEL_1, 
		("cl_ExternalCommand: tid:%d", message->header.params.in.tid));

	/*
	 *	get a pointer to this thread
	 */
	tcb = Active;

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findTransRec(message->header.params.in.tid)) == NULL)	{

		/*
		 *	leave it upto the client to interpret this
		 *	or maybe even the application
		 */
		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	check to see if the request has come in on the
	 *	controlling link for the transaction
	 */
	if (transRec->masterLink != link)	{

		reply(esmFAILURE, esmNOTTRANSMASTER, 1, TRUE);
	}

	/*
	 *	check validity of command
	 */
	if ((command != C_COMMIT) && (command != C_ABORT)) {

		reply(esmFAILURE, esmBADCOMMAND, 1, TRUE);
	}

	/*
	 *	check to see if the transaction is prepared
	 */
	if (transRec->transState != T_PREPARED)	{

		switch (transRec->transState) {
				
			case T_ACTIVE:
				if (command == C_COMMIT) {
					reply(esmFAILURE, esmTRANSNOTPREPARED, 1, TRUE);
				} else {

					/*
					 *	this transaction is not yet in the T_PREPARED 
					 *	state - so, we'll make do by putting it on the 
					 *	list of distributed transactions and then 
					 *	notifying the coord
					 */

					/*
					 *	initialize the number of server, and the 
					 *	number of votes and acks received
					 */
					transRec->numServers = 0;
					transRec->numVotesRecd = 0;
					transRec->numAcksRecd = 0;


					/*
					 *	add this transRec to the list of distr trans
					 */
					listEnq(&(CoordDistrTransList), &(transRec->distrTransList));
					/*
					 *	increment the number of active distr trans
					 */
					numActiveCoordDistrTrans++;
				}

				break;

			case T_COMMIT:
				if (command == C_ABORT)
					reply(esmFAILURE, esmTRANSCOMMITTED, 1, TRUE);
				/* 
				 *	otherwise do nothing special 
				 *	the coord thread will reply in due course 
				 */
				break;

			case T_END:
				if (command == C_ABORT)
					reply(esmFAILURE, esmTRANSCOMMITTED, 1, TRUE);
				else
					reply(esmNOERROR, esmNOERROR, 1, TRUE);

				break;
				
			case T_ABORT:
			default:
				if (command == C_ABORT)
					reply(esmNOERROR, esmNOERROR, 1, TRUE);
				else
					reply(esmFAILURE, esmTRANSABORTED, 1, TRUE);

				break;

		}
	}

	/*
	 *	store the command 
	 */
	transRec->clientCommand = command;

	/*
	 *  the coord thread should be awakened
	 */
	TRPRINT(TR_TRANS|TR_DISTR, TR_LEVEL_1, ("cl_ExternalCommand notifying coord"));
	notify(&CoordWaitList, esmNOERROR, esmNOERROR);

	/*
	 *  restart the thread 
	 */
	transThreadDeath(tcb);
	threadRestart();
}

/*
 *	This routine is called by the coordinator upon receiving a 
 *	prepare message from a client for a distributed transaction
 *	It reads the information about the servers involved in the
 *	transaction and enters them in the transRec.
 */

 void
cl_ExternalPrepare (

	MESSAGE				*message,
	LINK				*link 
)
{
	TRANSREC			*transRec;
	SERVERTIDINFO		*serverTidInfo;
	SERVERADDRTID		serverAddrTid;
	register OPENLOG	*openLog;
	int					length = sizeof(SERVERADDRTID); 
	int					numRead = 0;
	int					totalToRead = (int )message->body.prepare.numParticipants;
	TID					tid;
	VOLINFO				*volInfo;
	TCB					*tcb;


	TRPRINT(TR_TRANS|TR_DISTR, TR_LEVEL_1, ("cl_ExternalPrepare: tid:%d", message->header.params.in.tid));

	/*
	 *	get a pointer to this thread
	 */
	tcb = Active;

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, NULL, totalToRead);

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *  See if transaction processing is being shutdown
	 */
	if (TransShutdownState != T_ACTIVE) {

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, NULL, totalToRead);

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	see if this is supposed to be a new transaction
	 */
	if (message->header.params.in.tid == NULL_TID) {

	    /*
  		 *  get a register pointer to the openlog structure
		 */
    	openLog = &OpenLog;

    	/*
  		 *  check the open log magic number
  		 */
    	CHECK_OPENLOG_MAGIC(openLog);

		/*
		 *	Get a transactions id.  The log must be active to get
		 *	a usable id
		 */
		if (TransInfo.logState != L_ACTIVE) {

        	/*
    		 *  check to see if recovery is happening
     		 */
        	if (TransInfo.logState == L_RECOVER)    {

            	/*
            	 *  queue the request pending recovery completion
           		 */
            	if (waitList( &(openLog->recoverList) , THREAD_LOG_RECOVERY_WAIT)) {

					reply(esmFAILURE, Active->errno, 1, TRUE);
            	}

        	} else {

            	/*
           	 	 *  reject the request
            	 */
            	SM_ERROR(TYPE_LOG, esmLOGDISABLED);
				reply(esmFAILURE, Active->errno, 1, TRUE);
        	}
    	}

		/*
		 *	Get a transaction record
		 */
		if ((transRec = allocTrans()) == NULL)	{

			reply(esmFAILURE, Active->errno, 1, TRUE);
		}

		/*
		 *  plug the tid into the transRec
		 */
		tid = transRec->tid = TransInfo.tidSequence++;

		TRPRINT(TR_TRANS, TR_LEVEL_2, ("tid:%d hash:%d", tid, HASH_TID(tid)));
		/*
		 *  hang the transaction off the hash table
		 */
		listPush( &(TransHashTable[HASH_TID(tid)]), &(transRec->tidList) );

		/*
		 *  hang the transaction record off the link
		 *  and record link as the master link
		 */
		listEnq( &(link->transList), &(transRec->linkList) );
		transRec->masterLink = link;

		/*
		 *  Add each volume mounted by the link to the transaction's
		 *  volume list.
		 */
		volInfo = (VOLINFO*) FIRST_LIST_ELEMENT(&(link->volList));
		while (volInfo != NULL) {

			if (addTransVolRec(transRec, volInfo->volRec) == NULL) {
			reply(esmFAILURE, Active->errno, 1, TRUE);
			}
			volInfo = (VOLINFO*) NEXT_LIST_ELEMENT(&(volInfo->volList));
		}

		/*
		 *  plug the transaction id into the message
		 *  reply with the new tid
		 */
		message->header.params.in.tid = tid;

	} else {
		/*
		 *	try to find the transaction
		 */
		if ((transRec = findTransRec(message->header.params.in.tid)) == NULL)	{

			/*
			 *	read the list of SERVERADDRTIDs and discard them
			 */
			junkServerAddrTids(link, transRec, totalToRead);
	
			SM_ERROR(TYPE_USER, esmBADTRANSID);
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}
	}

	/*
	 *	check to see if the request has come in on the
	 *	controlling link for the transaction
	 */

	if (transRec->masterLink != link)	{

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, transRec, totalToRead);

		SM_ERROR(TYPE_USER, esmNOTTRANSMASTER);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	check to see if the transaction has logging turned off 
	 */
	if (transRec->loggingOn == FALSE)	{

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, transRec, totalToRead);

		SM_ERROR(TYPE_USER, esmLOGGINGOFF);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	check to see if the transaction is active
	 */
	if (transRec->transState != T_ACTIVE)	{

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, transRec, totalToRead);

		switch (transRec->transState) {

			case T_PREPARED:
				message->body.vote.vote = YESVOTE;
				reply(esmNOERROR, esmNOERROR, 1, TRUE);
				break;

			case T_COMMIT:
			case T_END:
				reply(esmFAILURE, esmTRANSCOMMITTED, 1, TRUE);
				break;
				
			case T_ABORT:
				reply(esmFAILURE, esmTRANSABORTED, 1, TRUE);
				break;

			default:
				reply(esmFAILURE, esmTRANSABORTED, 1, TRUE);
				break;
		}
	}

	/*
	 *	check to see if the number of servers is managable
	 */
	if (totalToRead > MAXPARTICIPANTS) {

		/*
		 *	read the list of SERVERADDRTIDs and discard them
		 */
		junkServerAddrTids(link, transRec, totalToRead);

		SM_ERROR(TYPE_USER, esmTOOMANYSERVERS);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	TRPRINT(TR_TRANS|TR_DISTR, TR_LEVEL_1, ("cl_ExternalPrepare reading %d SERVERTIDINFO's", totalToRead));
	/* 
	 *  read the list of SERVERADDRTIDs 
	 */
	 for (numRead = 0; numRead < totalToRead; numRead++) {

		/*
		 *  get a serverTidInfo structure 
		 */

		if ((serverTidInfo = (SERVERTIDINFO *) poolDeq( &ServerTidInfoPool)) == NULL) {

			/*
			 *	read the list of SERVERADDRTIDs and discard them
			 */
			junkServerAddrTids(link, transRec, (totalToRead - numRead));

			SM_ERROR(TYPE_WARNING, esmNOFREESERVERTIDINFO);
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}
		else {

			/*
			 *	initialize the serverTidInfo
			 */
			 initServerTidInfo(serverTidInfo);

			if (receive(link, (char *) &serverAddrTid, length) < 0) {

				/*
				 *	restart this thread
				 */
				threadRestart();
			}

			/*
			 *	fill in the fields of the serverTidInfo
			 */
			serverTidInfo->connectAddr = serverAddrTid.connectAddr;
			serverTidInfo->tid = serverAddrTid.tid;


			/*
			 *	add this entry to the list of servers in the transRec
			 */
			listEnq( &(transRec->serverList), &(serverTidInfo->serverTidInfoList) );
		}
	}


	/*
	 *	store the number of servers
	 */
	transRec->numServers = message->body.prepare.numParticipants;
	transRec->prepareAndCommit = message->body.prepare.prepareAndCommit;

	/*
	 *	initialize the number of votes and acks received
	 */
	transRec->numVotesRecd = 0;
	transRec->numAcksRecd = 0;

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	add this transRec to the coord list of distributed transactions
	 */
	listEnq( &(CoordDistrTransList), &(transRec->distrTransList) );

	/*
	 *	increment the number of active distributed transactions
	 */
	numActiveCoordDistrTrans++;

	/*
	 *  the coord thread should be awakened
	 */
	TRPRINT(TR_TRANS|TR_DISTR, TR_LEVEL_1, ("cl_ExternalPrepare notifying coord"));
	 notify(&CoordWaitList, esmNOERROR, esmNOERROR);

	/*
	 *	restart the thread 
	 *	because the coord thread will take care of the rest 
	 */
	transThreadDeath(tcb);
	threadRestart();
}
