/*
 *   $RCSfile: cl_ExternalDist2PC.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "openlog.h"
#include "logrecs.h"
#include "log_globals.h"
#include "log_intfuncs.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "threadstate.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_extfuncs.h"


 void
cl_ExternalContinue2PC (

	MESSAGE				*message,
	LINK				*link 
)
{

	register TRANSREC	*transRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%d", message->header.params.in.tid));

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *  See the transaction processing is being shutdown
	 */
	if (TransShutdownState != T_ACTIVE) {
		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findDistrTransRec(message->header.params.in.tid,
		&CoordDistrTransList)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	check if the transaction already has a controlling
	 *	client
	 */
	if (LIST_MEMBER(&(transRec->linkList)))	{
		
		if (transRec->masterLink != link) {
			
			SM_ERROR(TYPE_USER, esmNOTTRANSMASTER);
			reply(esmFAILURE, Active->errno, 1, TRUE);

		} else {
			/*
			 *	reply with success
			 */
			reply(esmNOERROR, esmNOERROR, 1, TRUE);
		}
	}

	/*
	 *  hang the transaction record off the link
	 *  and record link as the master link
	 */
	listEnq( &(link->transList), &(transRec->linkList) );
	transRec->masterLink = link;

	/*
	 *	reply with success
	 */
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}


 void
cl_ExternalEnter2PC (
	MESSAGE				*unused1,
	LINK				*link 
)
{
	TRANSREC			*transRec;
	TID					tid;
	register OPENLOG	*openLog;
	GTIDREC	 			*gtidRec;
	GTID	 			gtid;
	VOLINFO	 			*volInfo;

	TRACE(TR_TRANS, TR_LEVEL_1);

	if (receive(link, (char *)&gtid, sizeof(GTID)) < 0) {
		threadRestart();
	}

	if (TransInfo.transState != T_ACTIVE)	{
		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	if (TransShutdownState != T_ACTIVE) {
		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

    /*
     *  get a register pointer to the openlog structure
     */
    openLog = &OpenLog;

    /*
     *  check the open log magic number
     */
    CHECK_OPENLOG_MAGIC(openLog);

	/*
	 *	Get a transactions id.  The log must be active to get
	 *	a usable id
	 */
    if (TransInfo.logState != L_ACTIVE) {

        /*
         *  check to see if recovery is happening
         */
        if (TransInfo.logState == L_RECOVER)    {

            /*
             *  queue the request pending recovery completion
             */
            if (waitList( &(openLog->recoverList) , THREAD_LOG_RECOVERY_WAIT)) {

				reply(esmFAILURE, Active->errno, 1, TRUE);
            }

        } else {

            /*
             *  reject the request
             */
            SM_ERROR(TYPE_LOG, esmLOGDISABLED);
			reply(esmFAILURE, Active->errno, 1, TRUE);
        }
    }

	/*
	 *	Get a transaction record
	 */
	if ((transRec = allocTrans()) == NULL)	{

		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *  Get a gtidRec  
	 */
	if ((gtidRec = (GTIDREC *) poolDeq( &GtidRecPool )) == NULL) {

		SM_ERROR(TYPE_WARNING, esmNOFREEGTIDREC);
		reply(esmFAILURE, Active->errno, 1, TRUE);
    }

	/*
	 * fill in the gtidRec 
	 */
	gtidRec->gtid = gtid;

	/*
	 *	plug the tid into the transRec
	 */
	tid = transRec->tid = TransInfo.tidSequence++;

	/*
	 * plug the gtid into the transRec
	 */
	transRec->gtidRec = gtidRec;


	TRPRINT(TR_TRANS, TR_LEVEL_2, ("tid:%d hash:%d", tid, HASH_TID(tid)));

	/*
	 *	hang the transaction off the hash table
	 */
	listPush( &(TransHashTable[HASH_TID(tid)]), &(transRec->tidList) );

	/*
	 *	hang the transaction record off the link
	 *	and record link as the master link
	 */
	listEnq( &(link->transList), &(transRec->linkList) );
	transRec->masterLink = link;

	/*
	 *	Add each volume mounted by the link to the transaction's
	 *	volume list.
	 */
	volInfo = (VOLINFO*) FIRST_LIST_ELEMENT(&(link->volList));
	while (volInfo != NULL) {
		
		if (addTransVolRec(transRec, volInfo->volRec) == NULL) {
			reply(esmFAILURE, Active->errno, 1, TRUE);
		}
		volInfo = (VOLINFO*) NEXT_LIST_ELEMENT(&(volInfo->volList));
	}

	/*
	 *	plug the transaction id into the message
	 *	reply with the new tid
	 */
	Active->message.body.trans.tid = tid;
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}


 void
cl_ExternalRecover2PC (

	MESSAGE				*message,
	LINK				*link 
)
{

	register TRANSREC	*transRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%d", message->header.params.in.tid));

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *  See the transaction processing is being shutdown
	 */
	if (TransShutdownState != T_ACTIVE) {
		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findDistrTransRec(message->header.params.in.tid,
		&CoordDistrTransList)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	check if the transaction already has a controlling
	 *	client
	 */
	if (LIST_MEMBER(&(transRec->linkList)))	{
		
		if (transRec->masterLink != link) {
			
			SM_ERROR(TYPE_USER, esmNOTTRANSMASTER);
			reply(esmFAILURE, Active->errno, 1, TRUE);

		} else {
			/*
			 *	reply with success
			 */
			reply(esmNOERROR, esmNOERROR, 1, TRUE);
		}
	}

	/*
	 *  hang the transaction record off the link
	 *  and record link as the master link
	 */
	listEnq( &(link->transList), &(transRec->linkList) );
	transRec->masterLink = link;

	/*
	 *	reply with success
	 */
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}
