/*
 *   $RCSfile: cl_GetRootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bitvec.h"
#include "volume.h"
#include "trans.h"
#include "cl_funcs.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "thread_funcs.h"
#include "server_util_funcs.h"


 void
cl_GetRootEntry (

	register MESSAGE	*message,
	register LINK		*link 
)
{
	register TRANSREC	*transRec;
	char				namebuf[MAX_ROOTNAME_SIZE+1];
	char				databuf[MAX_ROOTDATA_SIZE];
	register VOLREC	    *volRec;

	TRACE(TR_CL, TR_LEVEL_1);

	/*
	 *	get the name of the desired root entry 
	 */
	if (receive(link, namebuf, (int)message->body.root.nameSize) < 0)	{
		/*
		 *	blow the thread away, the link is gone
		 */
		transThreadDeath(Active);
		threadRestart();
	}

	/* the name will be used as a zero-terminated string */
	namebuf[message->body.root.nameSize] = '\0'; 

	/*
	 *	attempt to find the transaction record, FALSE => not writing log
	 */
	if ((transRec = registerTrans(message->header.params.in.tid, FALSE)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}
	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolumeIdWithLink(message->body.root.volid, link)) 
		== NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	if (io_GetRootEntry(volRec, namebuf, databuf,
						(int *)&(message->body.root.dataSize)))	{

		reply(esmFAILURE, Active->errno, 1, TRUE);
	
	} else {
		Active->iovec[1].iov_base   = databuf;
		Active->iovec[1].iov_len = (int)message->body.root.dataSize;

		reply(esmNOERROR, esmNOERROR, 2, TRUE);
	}
}
