/*
 *   $RCSfile: cl_GetUnique.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "cl_funcs.h"
#include "msg_funcs.h"
#include "io_extfuncs.h"
#include "thread_globals.h"
#include "trans_extfuncs.h"


 void
cl_GetUnique (

	register MESSAGE	*message,
	register LINK		*unused 
)
{

	UNIQUE				unique;
	register TRANSREC   *transRec;

	TRPRINT(TR_CL, TR_LEVEL_1, ("unique for page:%d", message->body.unique.pid.page));

    /*
     *  attempt to find the transaction record, TRUE => writing log
     */
    if ((transRec = registerTrans(message->header.params.in.tid, TRUE)) == NULL) { 

        SM_ERROR(TYPE_USER, esmBADTRANSID);
        reply(esmFAILURE, Active->errno, 1, TRUE);
    }

	/*
	 *	attempt to allocate the pages
	 */
	if (io_GetUnique( &(message->body.unique.pid), &unique,
					  message->body.unique.uniqueCount ))	{

		reply(esmFAILURE, Active->errno, 1, TRUE);

	} else {

		/*
		 *	stuff the unique number in the message
		 */
		message->body.unique.unique = unique;
		TRPRINT(TR_CL, TR_LEVEL_2, ("unique:%d", unique));

		/*
		 *	reply to the caller
		 */
		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	}
}
