/*
 *   $RCSfile: cl_InitClient.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "io.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "bitvec.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "threadstate.h"
#include "cl_funcs.h"
#include "msg_funcs.h"
#include "log_globals.h"
#include "trans_globals.h"
#include "thread_globals.h"
#include "softwareVersion.h"


 void
cl_InitClient (

	register MESSAGE	*message,
	register LINK		*link 
)
{

	if(
		(message->body.init.slottedPage2size != SLOTTED_PAGE2SIZE) ||
		(message->body.init.lgPage2size != LG_PAGE2SIZE) ||
		(message->body.init.lgDataPage2size != LGDATA_PAGE2SIZE) ||
		(message->body.init.btreePage2size != BTREE_PAGE2SIZE) ||
		(message->body.init.indexDescPage2size != INDEXDESC_PAGE2SIZE) ||
		(message->body.init.minPage2size != MIN_PAGE2SIZE)  ||
		(message->body.init.softwareVersion != SOFTWARE_VERSION) ) {

		/* send back my values, so the client can tell the user
		 * exactly what values are wrong, if it so chooses
		 */
		message->body.init.slottedPage2size = SLOTTED_PAGE2SIZE;
		message->body.init.lgPage2size = LG_PAGE2SIZE;
		message->body.init.lgDataPage2size = LGDATA_PAGE2SIZE;
		message->body.init.btreePage2size = BTREE_PAGE2SIZE;
		message->body.init.indexDescPage2size = INDEXDESC_PAGE2SIZE;
		message->body.init.minPage2size = MIN_PAGE2SIZE;
		message->body.init.softwareVersion = SOFTWARE_VERSION;
		SM_ERROR(TYPE_USER,  esmCONFIGMISMATCH);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}


	/*
	 *  Wait until recovery is complete
	 */
    if (TransInfo.logState != L_ACTIVE) {

        /*
         *  check to see if recovery is happening
         */
        if (TransInfo.logState == L_RECOVER)    {

            /*
             *  queue the request pending recovery completion
             */
            if (waitList( &(OpenLog.recoverList) , THREAD_LOG_RECOVERY_WAIT)) {

                reply(esmFAILURE, Active->errno, 1, TRUE);
            }

        } else {

            /*
             *  reject the request
             */
            SM_ERROR(TYPE_LOG, esmLOGDISABLED);
            reply(esmFAILURE, Active->errno, 1, TRUE);
        }
    }

	/*
	 *	Initialize the log record information for the link
	 *	This records that the last log record generated by the
	 *	link is the last log record in the log.
	 */
	link->lastForceMark = OpenLog.logRecordCount;

	SM_ASSERT(LEVEL_1, (OpenLog.volid != 0));
    message->body.init.volid = OpenLog.volid;
    /*
     *  reply with the log page size, and an indication whether
     *  or not the end-of-log LSN should be used as the first
     *  pageLRC for a newly allocated page.
     */
    message->body.init.logPage2size = OpenLog.page2size;
#ifdef INIT_LRC_IS_LSN
    message->body.init.initLRCisLSN = TRUE;
#else
    message->body.init.initLRCisLSN = FALSE;
#endif /* INIT_LRC_IS_LSN */

	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}
