/*
 *   $RCSfile: cl_MountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "bf.h"
#include "link.h"
#include "lsn.h"
#include "volume.h"
#include "disk.h"
#include "bitvec.h"
#include "admin_funcs.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "disk_funcs.h"
#include "thread_globals.h"
#include "thread_funcs.h"
#include "server_util_funcs.h"


 void
cl_MountVolume (
	MESSAGE	*message,
	LINK		*link 
)
{
	register VOLREC		*volRec;

	TRPRINT(TR_CL, TR_LEVEL_1, 
			("mounting volume:%d", message->body.mount.volid));

	
	if ((volRec = io_MountVolume(message->body.mount.volid, TRUE)) == NULL)	{
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	put another user on the list
	 */
	if (linkVolume(link, volRec))	{

		/*
		 *	reply with error
		 */
		reply(esmFAILURE, Active->errno, 1, TRUE);

	} else {
		VOLNAMEREC *volNameRec = io_FindVolNameById(volRec->header->volid);

		/*
		 *	return success along with the volid
		 */
		message->body.mount.volid = volRec->header->volid;
		message->body.mount.volumeProperties = volRec->header->properties;
		Active->iovec[1].iov_base = (char *) volNameRec->volName.name;

		/* receiver expects the full amount: */
		Active->iovec[1].iov_len = sizeof(volNameRec->volName);

		reply(esmNOERROR, esmNOERROR, 2, TRUE);
	}
}

 void
cl_AddVolume (
	MESSAGE	*message,
	LINK		*link 
)
{
	char			*formatvalue = NULL;
	char			*optionname = NULL;
	int				e;

	TRACE(TR_CL, TR_LEVEL_1);

	suspendLink(link);

	Active->errno = esmNOERROR;

	optionname = (char *)calloc( 1, (int)message->body.volume.optionNameLength + 1);
	if(optionname==NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		goto bad;
	}
	if (receive(link, optionname, 
		(int)message->body.volume.optionNameLength) < 0)	{
		goto bad2;
	}
	formatvalue = (char *)calloc(1,  (int)message->body.volume.formatLength + 1);
	if(formatvalue==NULL) {
		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		goto bad;
	}
	if (receive(link, formatvalue, 
		(int)message->body.volume.formatLength) < 0)	{
		goto bad2;
	}
	if(message->header.type == ADD_VOLUME) {
		e = addcommand( optionname, formatvalue, NULL);
	} else {
		SM_ASSERT(LEVEL_1, message->header.type == FORMAT_VOLUME);
		SM_ERROR(TYPE_SYS, esmNOTIMPLEMENTED);
	}
	if(e != esmNOERROR) {
		SM_ERROR(TYPE_USER, Active->errno);
		goto bad;
	}
	resumeLink(link);
	SM_FREE_PTR(optionname);
	SM_FREE_PTR(formatvalue);
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
bad:
	resumeLink(link);
	SM_FREE_PTR(optionname);
	SM_FREE_PTR(formatvalue);
	reply(esmFAILURE, Active->errno, 1, TRUE);

bad2:
	SM_FREE_PTR(optionname);
	SM_FREE_PTR(formatvalue);
	transThreadDeath(Active);
	threadRestart();
}
