/*
 *   $RCSfile: cl_NeighborPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:36 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "consist.h"
#include "log.h"
#include "openlog.h"
#include "cl_funcs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "fi_extfuncs.h"
#include "lm_extfuncs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"
#include "thread_funcs.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "sm_macro.h"


 void
cl_NeighborPage (

	register MESSAGE	*message,
	register LINK		*unused 
)
{
	TRANSREC	*transRec;
	LOCKMODE	fileLockMode;
	SHORTPID	neighborPid;
	VOLID		volid;
	TREEDATA	*resultLeaf;
	int			resultCount, iovecParts;

	TRACE(TR_CL | TR_FI, TR_LEVEL_1);

	/*
	 *	attempt to find the transaction record
	 */
	if ((transRec = findTransRec(message->header.params.in.tid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	save the transaction record in the tcb
	 */
	Active->transRec = (char *) transRec;

    if (fi_ValidateFid(UserBufGroup, &(message->body.file.fid)) != esmNOERROR) {
		reply(esmFAILURE, Active->errno, 1, TRUE);
    }

	/*
	 *	attempt to lock the file, record the lock mode for the file
	 */
	fileLockMode = SH;
	if (lockFile(transRec, &(message->body.file.fid), &fileLockMode, NOFLAGS))	{

		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	volid = message->body.file.fid.pid.volid;

	if( message->body.file.flags & REPLY_WITH_DATA ) {
		resultCount = SLOTTED_PAGESIZE;
		if ((resultLeaf = (TREEDATA *)malloc(resultCount)) == NULL) {
			reply(esmFAILURE, esmMALLOCFAILED, 1, TRUE);
		}
	} else {
		resultCount = 0;
		resultLeaf = NULL;
	}

	if ((fi_NeighborPage(UserBufGroup, &(message->body.file.fid.pid),
						 message->body.file.pid.page,
				   		 &neighborPid,
						 message->body.file.flags & ~(REPLY_WITH_DATA),
						 resultLeaf, 
						 resultLeaf==NULL?NULL:&resultCount)) < esmNOERROR)	{

		if(resultLeaf)
			(void) free((char*)resultLeaf);
		reply(esmFAILURE, Active->errno, 1, TRUE);

	} 

	/*	
	 * the message response will be of the type for a page request,
	 * but the only output param is the NeighborPid part
	 */

	message->body.page.NeighborPid.volid = volid;
	message->body.page.NeighborPid.page = neighborPid;

	if(resultCount > 0) {
		message->body.page.flags = REPLY_WITH_DATA;

		SM_ASSERT(LEVEL_3, sizeof(SHORTPID) == sizeof(KEYS));

		Active->iovec[2].iov_base = (char *)resultLeaf;
		Active->iovec[2].iov_len = resultCount * sizeof(SHORTPID);
		/* NB: client has to know in what order these come */

		Active->iovec[1].iov_base = (char *)&resultCount;
		Active->iovec[1].iov_len = sizeof(resultCount);
		iovecParts = 3;
	} else {
		message->body.page.flags = 0; /* REPLY_WITH_NEIGHBOR  not necessary */
		iovecParts = 1;
	}

	reply(esmNOERROR, esmNOERROR, iovecParts, FALSE);

	(void) free((char*)resultLeaf);

}
