/*
 *   $RCSfile: cl_SetRootEntry.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bitvec.h"
#include "volume.h"
#include "trans.h"
#include "cl_funcs.h"
#include "io_extfuncs.h"
#include "msg_funcs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "thread_funcs.h"
#include "server_util_funcs.h"


 void
cl_SetRootEntry (

	register MESSAGE	*message,
	register LINK		*link 
)
{
	register TRANSREC	*transRec;
	char				namebuf[MAX_ROOTNAME_SIZE];
	char				databuf[MAX_ROOTDATA_SIZE];
	VOLREC				*volRec;


	TRACE(TR_CL, TR_LEVEL_1);

	suspendLink(link);
	/*
	 *	read the name of the root entry and its data into a buffer
	 */
	if (receive(link, namebuf, (int)message->body.root.nameSize) < 0)	{
		/*
		 *	blow the thread away, the link is gone
		 */
		transThreadDeath(Active);
		threadRestart();
	}
	namebuf[message->body.root.nameSize] = '\0';

	if (receive(link, databuf, (int)message->body.root.dataSize) < 0)	{
		/*
		 *	blow the thread away, the link is gone
		 */
		transThreadDeath(Active);
		threadRestart();
	}
	resumeLink(link);

	/*
	 *	attempt to find the transaction record, TRUE => writing log
	 */
	if ((transRec = registerTrans(message->header.params.in.tid, TRUE)) == NULL) {

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}
	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolumeIdWithLink(message->body.root.volid, link)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	if (io_SetRootEntry(volRec, namebuf, databuf,
						(int)message->body.root.dataSize))	{

		reply(esmFAILURE, Active->errno, 1, TRUE);
	
	} else {

		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	}
}


 void
cl_RemoveRootEntry (

	register MESSAGE	*message,
	register LINK		*link 
)
{
	register TRANSREC	*transRec;
	char				namebuf[MAX_ROOTNAME_SIZE];
	VOLREC				*volRec;


	TRACE(TR_CL, TR_LEVEL_1);

	suspendLink(link);
	/*
	 *	read the name of the root entry and its data into a buffer
	 */
	if (receive(link, namebuf, (int)message->body.root.nameSize) < 0)	{
		/*
		 *	blow the thread away, the link is gone
		 */
		transThreadDeath(Active);
		threadRestart();
	}
	namebuf[message->body.root.nameSize] = '\0';

	resumeLink(link);

	/*
	 *	attempt to find the transaction record, TRUE => writing log
	 */
	if ((transRec = registerTrans(message->header.params.in.tid, TRUE)) == NULL) {

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}
	/*
	 *	check to see if the volume is mounted
	 */
	if ((volRec = io_FindVolumeIdWithLink(message->body.root.volid, link)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	if (io_RemoveRootEntry(volRec, namebuf)) {

		reply(esmFAILURE, Active->errno, 1, TRUE);
	
	} else {

		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	}
}
