/*
 *   $RCSfile: cl_SyncTransaction.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "trans_globals.h"


/*
 * the work is done here - this is a separate func only
 * for debugging purposes.
 */

 void
CL_FlushTid (

	TID					tid
)
{

	register TRANSREC	*transRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", tid ));

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findTransRec(tid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		reply(esmFAILURE, Active->errno, 1, TRUE);
	}

	Active->transRec = (char *) transRec;

	/*
	 *	check to see if the transaction is active
	 */
	if (transRec->transState != T_ACTIVE)	{

		reply(esmFAILURE, esmTRANSABORTED, 1, TRUE);
	}

	trans_FlushDirtyPages(transRec);
	reply(esmNOERROR, esmNOERROR, 1, TRUE);
}

 void
cl_SyncTransaction (

	MESSAGE				*message,
	LINK				*unused 
)
{
	if( message->body.trans.syncTrans == TRUE) {
		CL_FlushTid(message->header.params.in.tid);
	} else {
		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	}
}

