/*
 *   $RCSfile: cl_TestPageSend.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:37 $      
 */ 
/*********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "io.h"
#include "object.h"
#include "tid.h"
#include "lock.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "bitvec.h"
#include "latch.h"
#include "link.h"
#include "bf.h"
#include "cl_funcs.h"
#include "msg_funcs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"
#include "thread_funcs.h"
#include "bf_globals.h"


 void
cl_TestPageSend (

	register MESSAGE	*message,
	register LINK		*link 
)
{
	GROUPLINK   *tempLink;

	TRPRINT(TR_CL, TR_LEVEL_1, ("receiving page size %d", message->body.page.page2size));

	/*
	 *	suspend the link so that it does not get selected on
	 *	the basis of the data that is sent
	 */
	suspendLink(link);

	tempLink = bf_GetTempPage(message->body.page.page2size);
	if (tempLink == NULL) {
		if (message->header.replyRequested) {
			reply(esmFAILURE, Active->errno, 1, TRUE);
		} else {
			/* do nothing */
			threadRestart();
		}
	}

	/*
	 *  resume the link so that the data can be read
	 */
	resumeLink(link);

	if (receive(link, tempLink->bufFrame, RAISETWO(message->body.page.page2size)) < 0) {
		/*
		 *  blow the thread away, the link is gone
		 *  discard the page
		 */
		bf_FreeTempPage(tempLink);
		threadRestart();
	}

    /*
	 *	Return the page to the buffer pool and finish handling the msg
	 */
	bf_FreeTempPage(tempLink);
	if (message->header.replyRequested) {
		reply(esmNOERROR, esmNOERROR, 1, TRUE);
	} else {
		threadRestart();
	}

	/* cannot get here */
	SM_ASSERT(LEVEL_3, esmINTERNAL);

}
