/*
 *   $RCSfile: initializeEnv.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "disk.h"
#include "bitvec.h"
#include "trans.h"
#include "bitmap.h"
#include "file.h"
#include "consist.h"
#include "resource.h"
#include "openlog.h"
#include "logrecs.h"
#include "pool.h"
#include "undo.h"
#include "msgvector.h"
#include "log.h"
#include "init_funcs.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "msg_globals.h"
#include "thread_globals.h"
#include "io_globals.h"
#include "disk_globals.h"
#include "recover_globals.h"
#include "sharedmem_globals.h"
#include "sm_params.h"
#include "queue_consist.h"



 void
initializeEnv()
{
#ifdef PROFIL
	{
		char	*param;

		/*
		 * See what the profiling directory is, if any 
		 */
		if (param = getenv("PROFDIR"))	{
			HasProfDir = TRUE;
		} else  {
			HasProfDir = FALSE;
		}
	}
#endif PROFIL

	/* 
	 * For every (client link) thread, there is a MESSAGE buffer.
	 * It's cast as a * DISKMSG buffer for ipc with the disk process.
	 */ 
	NumMsgBufs = (int)NumThreads; 

	/*
	 * Increase QPairSize to reflect the size of the elements part.
	 * The elements part has one ShmOffset for each DISKMSG (which
	 * is also one per thread).
	 * QPairSize was BSS-ed in disk/space.c to be the size of
	 * the QUEUEPAIR structure.  We increase it here.
	 */
	QPairSize +=	 (NumMsgBufs * sizeof(ShmOffset)); 

}
