/*
 *   $RCSfile: initializeSharedMemory.c,v $  
 *   $Revision: 1.2 $  
 *   $Date: 1996/05/04 23:51:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "io.h"
#include "tid.h"
#include "lock.h"
#include "latch.h"
#include "msgdefs.h"
#include "disk.h"
#include "thread.h"
#include "semaphore.h"
#include "bf.h"
#include "bf_intfuncs.h"
#include "bf_globals.h"
#include "link.h"
#include "bitvec.h"
#include "volume.h"
#include "disk_globals.h"
#include "io_globals.h"
#include "sharedmem_globals.h"
#include "msg_globals.h"
#include "sm_params.h"
#include "init_funcs.h"
#include "queue_consist.h"

#define READWRITE (0400 | 0200) 

 void 
initializeSharedMemory()

{
	int size4bufferPool = (int) (NumBuffers + 1) * MIN_PAGESIZE; 
							/* +1 for alignment */
	int size4messages	= 	(NumMsgBufs * sizeof(DISKMSG));
	int size4queues;
	int size;

	/* ok - we already know that at least one "logformat"
	 * option appeared, but we don't know if its volid matches
	 * that of the "logvolume" option.
	 */
	if( NumVolumes + MaxAddVolumes <= 1 ) {
		fprintf(stderr, 
		"Cannot mount data volumes:\n");
		fprintf(stderr, 
		"No volumes are given by <dataformat> or <tempformat> options\n");
		fprintf(stderr, 
		"and the  <maxaddvolumes> option is zero or is not set.\n");
		SM_ERROR(TYPE_STOP, esmMOUNTTABLEFULL);
	}

	NumVolumes += MaxAddVolumes; 	

	/* set MaxAddVolumes to something to show that its value was
	 * already added to NumVolumes:
	 */
	MaxAddVolumes = -1;

	size4queues = 	(NumVolumes * QPairSize);
	size 	 	= 	size4bufferPool + size4messages + 
					size4queues + sizeof(MUTEX) /* for Replies */;

	TRPRINT(TR_INIT, TR_LEVEL_1, 
		("allocating:%d bytes(((%d+1)*%d)+%d+%d+%d", 
			size, NumBuffers, MIN_PAGESIZE, 
			size4messages, size4queues,
			sizeof(MUTEX)));

	/*
	 * All we do here is get the shared memory and set 
	 * global pointers into the proper places in the shared memory.
	 */

	SM_ASSERT(LEVEL_1, (NumBuffers >0));
	SM_ASSERT(LEVEL_1, (NumMsgBufs >0));

	if(FastPathOnly) {
		ShmKey = (u_long) -1;
		ShmId = 0;
		ShmAddress = (char *) calloc(1, size);
		TRPRINT(TR_INIT, TR_LEVEL_2, ("private memory attached at:%x", ShmAddress));
	} else {
#ifdef vax
		char    *shmSpace;
#endif vax
		/*
		 *	create the shared memory segment
		 */
		ShmKey = IPC_PRIVATE;

		if ( (ShmId = shmget(ShmKey, size, IPC_CREAT|READWRITE)) < 0)	{
			TRPRINT(TR_INIT, TR_LEVEL_1, 
				("shmget ShmId %d, errno %d", ShmId, errno));

			fprintf(stderr, 
			"\nCannot allocate buffer pool shared memory (of size %d bytes).\n", size);
			fprintf(stderr, 
			"   Either %d (from the bufpages option X the %d byte page size),\n", size, MIN_PAGESIZE);
			fprintf(stderr, 
			"   is too large, no shared memory segments are left,\n");
			fprintf(stderr, 
			"   or shared memory is not installed on your system.\n\n");
			SM_ERROR(TYPE_STOP, errno);
		}

		TRPRINT(TR_INIT, TR_LEVEL_2, ("shared memory id:%d", ShmId));

#if defined(mips) || defined (sparc) || defined(_AIX) || defined(hpux) || defined(linux)

		/*
		 *	attach the shared memory segment
		 *  brain-damaged hpux turns shmat into a void * func
		 *  for C++ compilers - despite its man page...
		 */
		if ((ShmAddress = (char *)
			shmat(ShmId, 0, 0)) == (char *) -1)	{

			SM_ERROR(TYPE_FATAL, errno);
		}

#elif defined(vax)

		/*
		 *  Attach the shared memory segment and avoid bug in shmat.
		 *  Apparently shmat interferes with malloc on the vax so
		 *  here we allocate the space for the shared memory ourselves
		 *  and tell the os to put the shared memory there.
		 *  Actually the above idea doesn't work so instead we'll put it
		 *  10Meg out in the address space.
		 */

		/*
		 * make it plenty big
		 * shmSpace = (char*)(((long)(malloc(size*2) + 4096)) & 0xffffe000);
		 */

		/*
		 * put it far out
		 */
		shmSpace = (char*)(10 * 1024 * 1024); /* 10 meg */
		if ((ShmAddress = shmat(ShmId, shmSpace, SHM_RND)) == (char *) -1) {

			SM_ERROR(TYPE_FATAL, errno);
		}
#elif !(defined(mips) || defined(sparc) || defined(vax) || defined(_AIX) || defined(hpux) || defined(linux))
		not supported
#else
		/* broken cpp always includes whatever is in here */
#endif
		TRPRINT(TR_INIT, TR_LEVEL_2, ("shared memory attached at:%x", ShmAddress));
	}

	/*
	 *	Record the shared memory segment size
	 */
	ShmSize = size;
	ShmDiskMessages = ShmAddress + size4bufferPool;
	ShmDiskQueues =  ShmDiskMessages + size4messages;
	ShmReplies = 	ShmDiskQueues + size4queues;
	SM_ASSERT(LEVEL_1, ((ShmDiskQueues+size4queues+sizeof(MUTEX))==(ShmAddress+ShmSize)));

}
