/*
 *   $RCSfile: closeDisk.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "disk.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "bf.h"
#include "volume.h"
#include "threadstate.h"
#include "disk_funcs.h"
#include "bitvec.h"
#include "msgvector.h"
#include "thread_globals.h"
#include "msg_globals.h"
#include "pool.h"
#include "io_globals.h"
#include "queue_consist.h"

 int
closeDisk (
	VOLREC 		*volRec
)
{
	LINK		*diskLink = volRec->volLink;
	DISKMSG			*message = Active->diskMessage;

	TRPRINT(TR_DISK, TR_LEVEL_1, ("closeDisk volid %d", volRec->volid));

    /*
     *  Make sure the list structures in the link are ok
     */
#ifdef DEBUG
	if (diskLink != NULL) {
		CHECK_LIST_MAGIC(&(diskLink->list));
		CHECK_LIST_MAGIC(&(diskLink->tcbList));
		CHECK_LIST_MAGIC(&(diskLink->volList));
		CHECK_LIST_MAGIC(&(diskLink->transList));
		CHECK_LIST_MAGIC(&(diskLink->resourceList));
		TRPRINT(TR_DISK, TR_LEVEL_1, ("closeDisk fd %d", diskLink->id));
	}
#endif DEBUG

	/*
	 *	setup the thread id in the message
	 */
	message->header.type = CLOSE_DISK;

	/*
	 *	send the message
	 */
	if (callDisk( volRec, message))	{
		SM_ERROR(TYPE_LOG, errno);
	}
#ifdef DEBUG
	if(FastPathOnly) {
		/*
			no longer the case: see admin_ShutServer(),
			comments about closing disks; where it sets
			FastPathOnly.
			SM_ASSERT(LEVEL_1, (volRec->diskrwPid == 0));
			SM_ASSERT(LEVEL_1, (diskLink == NULL));
		*/
	}
	if(diskLink == NULL) {
		SM_ASSERT(LEVEL_1, (FastPathOnly == TRUE));
	}
	if(volRec->diskrwPid == 0) {
		SM_ASSERT(LEVEL_1, (FastPathOnly == TRUE));
	}
#endif DEBUG

	if (volRec->diskrwPid != 0) {
		TRPRINT(TR_DISKRW|TR_DISK, TR_LEVEL_1,
			("Waiting on child %d for volume %s", 
			volRec->diskrwPid, volRec->volNameRec->volName.name));

		/* 
		 * The volume has a disk process.  
		 * Wait for it to die. 
		 */

		if( waitpid(volRec->diskrwPid, 0, 0) < 0 ) {
			TRPRINT(TR_DISKRW, TR_LEVEL_1, 
				("Error %d waiting on child %d", errno, volRec->diskrwPid));
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		TRPRINT(TR_DISKRW|TR_DISK, TR_LEVEL_1,
			("DISKRW %d for volume %s is gone.", 
			volRec->diskrwPid, volRec->volNameRec->volName.name));
		/* 
		 * disk process doesn't exist anymore
		 */
		volRec->diskrwPid = 0;

		/*
		 *	free up all resources associated with the link
		 */
		freeDiskLink(diskLink, esmFAILURE, esmDISKPROCDIED);
	}

	/*
	 *	reply to the caller
	 */
	return(esmNOERROR);
}
