/*
 *   $RCSfile: unlinkVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:38 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "bf.h"
#include "volume.h"
#include "bitvec.h"
#include "threadstate.h"
#include "disk.h"
#include "disk_funcs.h"
#include "msg_funcs.h"
#include "disk_globals.h"


 void
unlinkVolume (

	register LINK		*link,
	register VOLREC		*volRec 
)
{

	register VOLINFO	*volInfo;


	TRPRINT(TR_DISK, TR_LEVEL_1, ("unlinking vol:%d from link:%d",
			volRec->header->volid, link->id));

	/*
	 *	get a pointer to the first volinfo off the vol record
	 */
	volInfo = (VOLINFO *) FIRST_LIST_ELEMENT(&(link->volList));

	/*
	 *	run through the open list for this link
	 */
	while (volInfo != NULL)	{

		/*
		 *	check to see if the vol is the target vol
		 */
		if (volInfo->volRec == volRec)	{

			TRPRINT(TR_DISK, TR_LEVEL_2, ("vol is open by link"));

			/*
			 *	remove the link from the lists and put on pool
			 */
			listRemove( &(volInfo->volList) );
			listMoveEnq( &VolInfoPool, &(volInfo->linkList) );
			return;
		}

		volInfo = (VOLINFO *) NEXT_LIST_ELEMENT(&(volInfo->linkList));
	}

	/*
	 *	should not get here
	 */
	SM_ERROR(TYPE_FATAL, esmINTERNAL);
}
