/*
 *   $RCSfile: coord_LogActions.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:40 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "logaction.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logprepare.h"
#include "log_globals.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "thread_globals.h"
#include "distr.h"
#include "distr_intfuncs.h"


 int
coord_LogCommit (

	register TRANSREC	*transRec
)
{

	LOGRECORDINFO		recordInfo;


	TRPRINT(TR_DISTR|TR_LOG|TR_TRANS, TR_LEVEL_1, ("tid:%d", transRec->tid));

	/*
	 *	initialize the record info fields
	 */
	recordInfo.type		  = LOG_REC_TYPE_COMMIT;

	if (transRec->numServers == 0) {
		if (writeTransStateRecord(transRec, LOG_REC_TYPE_COMMIT, 
			LOG_ACTION_DISTR_COORD_COMMIT_READONLY, T_COMMIT, TRUE) 
			< esmNOERROR) {

			return(esmFAILURE);
		}
	} else {
		if (writeTransStateRecord(transRec, LOG_REC_TYPE_COMMIT, 
			LOG_ACTION_DISTR_COORD_COMMIT, T_COMMIT, TRUE) 
			< esmNOERROR) {

			return(esmFAILURE);
		}
	}

	return(esmNOERROR);
}


 int
coord_LogPrepare (

	TRANSREC			*transRec
)
{

	register FOUR		forceMark;
	LSN					lsn;
	LOGRECORDINFO		recordInfo;
	COORDPREPAREHDR		prepareHdr;
	PREPARESERVERINFO	serverInfo[MAXSERVERSINIMAGE];
	SERVERTIDINFO		*serverTidInfo;
	int					count;


	TRPRINT(TR_IO, TR_LEVEL_1, ("coord logging prepare: tid %d", transRec->tid));

	/*
	 *	record the prepareLSN for this transaction 
	 */
	transRec->prepareLSN = OpenLog.nextValidLSN;

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_COORD_PREPARE;
	recordInfo.action		= LOG_ACTION_DISTR_COORD_PREPARE_SERVERS;
	recordInfo.actionPid	= NULL;

	/*
	 *	make sure the number of servers is manageable
	 */
	SM_ASSERT(LEVEL_3, (transRec->numServers <= MAXPARTICIPANTS));

	if (transRec->numServers <= MAXSERVERSINIMAGE) {

		/*
		 *	initialize the log information	
		 */
		recordInfo.imageCount	= 1;
		recordInfo.imageSize[0] = (UTWO) transRec->numServers * sizeof(PREPARESERVERINFO);
		recordInfo.imageData[0] = (char *) serverInfo;
		recordInfo.flags		= NOFLAGS;

		/*
		 *	fill in the PREPARESERVERINFO records;
		 */
		for (count = 0; count < transRec->numServers; count++) {
				
			/*
			 *  get the serverTidInfo for this server
			 */
 			serverTidInfo = (SERVERTIDINFO *) listDeq(&(transRec->serverList));
			serverInfo[count].connectAddr = serverTidInfo->connectAddr;
			serverInfo[count].tid = serverTidInfo->tid;

			/*
			 *  put the serverTidInfo back on the list
			 */
			listEnq( &(transRec->serverList), &(serverTidInfo->serverTidInfoList) );

		}
	} else {

		/*
		 *	TODO: crash for now - 
		 *	will have to write multiple log records
		 */
		 SM_ERROR(TYPE_CRASH, esmTOOMANYSERVERS);
	}

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord(transRec, &recordInfo, &lsn, NOFLAGS)) < 0)	{
		
		return(esmFAILURE);

	}

	/*
	 *	write the prepare log header
	 */
	prepareHdr.numServers = transRec->numServers;
	prepareHdr.prepareAndCommit = transRec->prepareAndCommit;
	prepareHdr.prepareLSN = transRec->prepareLSN;
	prepareHdr.firstLogPid = transRec->firstLogPid;
	prepareHdr.logUnique = transRec->logUnique;
	prepareHdr.firstLSN = transRec->firstLSN;
	prepareHdr.logSpace = transRec->logSpace;
	prepareHdr.logRecordCount = transRec->logRecordCount;

	/*
	 *	initialize the log information
	 */
	recordInfo.type         = LOG_REC_TYPE_COORD_PREPARE;
	recordInfo.action       = LOG_ACTION_DISTR_COORD_PREPARE_HDR;
	recordInfo.actionPid    = NULL;
	recordInfo.flags        = NOFLAGS;
	recordInfo.imageCount   = 1;
	recordInfo.imageSize[0] = sizeof(COORDPREPAREHDR);
	recordInfo.imageData[0] = (char*) &prepareHdr;

	/*
	 *  write the record to the log
	 */
	if ((forceMark = writeLogRecord(transRec, &recordInfo, 
									&lsn, NOFLAGS)) < 0) {

		return(esmFAILURE);
    }

	/*
	 *	write the actual prepare log record
	 *	and change the state to T_PREPARED
	 */
	if (writeTransStateRecord(transRec, LOG_REC_TYPE_COORD_PREPARE, ZERO, T_PREPARED, TRUE) < esmNOERROR) {

		return(esmFAILURE);
	}

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
