/*
 *   $RCSfile: coord_RecordAck.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_extfuncs.h"
#include "server_util_funcs.h"


 void
coord_RecordAck (

	MESSAGE				*message,
	LINK				*unused 
)
{

	register TRANSREC	*transRec;
	TID					coordTid = message->body.ack.coordTid;
	SERVERTIDINFO		*serverTidInfo;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("coordTid:%x", coordTid));

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		/* do nothing */
		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *  check the state of the log
	 */
	if (TransInfo.logState != L_ACTIVE) {

		/* do nothing */
		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findTransRec(coordTid)) == NULL)	{

		/* not found implies duplicate ACK - do nothing */
		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	transaction state should be either T_COMMIT, T_ABORT,
	 */
	if ((transRec->transState != T_COMMIT) && (transRec->transState != T_ABORT))	{

		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *  check if all acks have already been received
	 */
	if (transRec->numAcksRecd < transRec->numServers) {

		/*
		 *	search for this server in the list of servers
		 */
		for ( serverTidInfo = (SERVERTIDINFO *) FIRST_LIST_ELEMENT(&(transRec->serverList));
			 serverTidInfo != NULL;
			 serverTidInfo = (SERVERTIDINFO *) NEXT_LIST_ELEMENT(&(serverTidInfo->serverTidInfoList))) {

			/*
			 *	check if this is the correct server
			 */
			if (INET_ADDR_EQ(serverTidInfo->connectAddr, Active->replyAddr)) {
				/*
				 * check for earlier ack
				 */
				if (serverTidInfo->ack == FALSE) {
	
					serverTidInfo->ack = TRUE;
					transRec->numAcksRecd++;

					/*
					 *	the coord thread should be awakened
					 */
					notify(&CoordWaitList, esmNOERROR, esmNOERROR);
				}

				/*
				 *	exit from here
				 */
				transThreadDeath(Active);
				threadRestart();
			}
		}
	
		/*
		 *	we got here means we could not find the server
		 *	TODO: for now, ignore the ack 
		 */
		transThreadDeath(Active);
		threadRestart();

	} else {
			/* 
			 *	do nothing - some other thread is already
			 *	taking care of writing the END log recorcd
			 *	and replying to the client
			 */
			transThreadDeath(Active);
			threadRestart();
	}
}
