/*
 *   $RCSfile: coord_RecordVote.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_intfuncs.h"
#include "distr_extfuncs.h"
#include "server_util_funcs.h"


 void
coord_RecordVote (

	MESSAGE				*message,
	LINK				*unused 
)
{

	register TRANSREC	*transRec;
	SERVERTIDINFO		*serverTidInfo;
	TID					coordTid = message->body.vote.coordTid; 
	TID					serverTid = message->body.vote.localTid;
	VOTE				vote = message->body.vote.vote;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("coordTid:%x", coordTid));

	/*
	 *  initialize the type, coordTid, and localTid of the reply message
	 */
	message->header.type = COORD_COMMAND;
	message->body.command.coordTid = coordTid;
	message->body.command.localTid = serverTid;

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		/* do nothing */
		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *  check the state of the log
	 */
	if (TransInfo.logState != L_ACTIVE) {

		/* do nothing */
		transThreadDeath(Active);
		threadRestart();
	}

	/*
	 *	try to find the transaction
	 */
	if ((transRec = findTransRec(coordTid)) == NULL)	{

		/* not found implies ABORT */
		message->body.command.command = C_ABORT;
		replyDatagram(esmNOERROR, esmNOERROR, 1, TRUE);
	}

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	/*
	 *	check the transaction state 
	 */
	if (transRec->transState == T_COMMIT)	{

		/*
		 *	this is a duplicate vote and the client has asked
		 *	for the transaction to be committed - send the 
		 *	command to this server 
		 */
		message->body.command.command = C_COMMIT;
		replyDatagram(esmNOERROR, esmNOERROR, 1, TRUE);

	} else if (transRec->transState == T_END) {

		/*
		 *	this is a duplicate vote and all servers have since 
		 *	been informed of the vote and have acknowledged
		 *	do nothing
		 */
		transThreadDeath(Active);
		threadRestart();

	} else if (transRec->transState == T_PREPARED) {

		/*
		 *	this is a duplicate vote and the client hasn't replied 
		 *	yet - do nothing
		 */
		transThreadDeath(Active);
		threadRestart();

	} else if (transRec->transState == T_ACTIVE) {

		/*
		 *	check if all votes have already been received 
		 */
		if (transRec->numVotesRecd < transRec->numServers) {
			
			/*
			 *	search for this server in the list of servers
			 */
			for (serverTidInfo = (SERVERTIDINFO *) FIRST_LIST_ELEMENT(&(transRec->serverList));
				 serverTidInfo != NULL;
				serverTidInfo = (SERVERTIDINFO *) NEXT_LIST_ELEMENT(&(serverTidInfo->serverTidInfoList))) {

				/*
				 *  check if this is the correct server
				 */
				if (INET_ADDR_EQ(serverTidInfo->connectAddr, Active->replyAddr)) {
					/*
					 * check for earlier vote
					 */
					if (serverTidInfo->vote == VOTEUNKNOWN) {
						
						serverTidInfo->vote = vote;
						transRec->numVotesRecd++;

						/*
						 *	the coord thread should be awakened
						 */
						notify(&CoordWaitList, esmNOERROR, esmNOERROR);
					}

					/* 
					 *	exit from here
					 */
					transThreadDeath(Active);
					threadRestart();
				}
			}

			/*
			 *	we got here means we could not find the server
			 *	TODO: for now, ignore the vote 
			 */
			transThreadDeath(Active);
			threadRestart();

		} else {
			/*
			 *	do nothing - some other thread is already taking 
			 *	care of evaluating the votes and writing the
			 *	appropriate log records and eventually replying 
			 *	to the servers
			 */
			transThreadDeath(Active);
			threadRestart();
		}

	} else if (transRec->transState == T_QUIESCE)	{
		/*
		 *	means some other thread is going to reply
		 */
		transThreadDeath(Active);
		threadRestart();

	} else {

		/*
		 *	any other state means abort
		 *	reply with an abort message
		 */
		message->body.command.command = C_ABORT;
		replyDatagram(esmNOERROR, esmNOERROR, 1, TRUE);
	}
}
