/*
 *   $RCSfile: findDistrTransRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_extfuncs.h"


 TRANSREC
*findDistrTransRec (

	register TID		tid, 
	LIST				*list

)
{

	register TRANSREC	*transRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", tid));

	/*
	 *	Get the first element on the list  
	 */
	transRec = (TRANSREC *) FIRST_LIST_ELEMENT(list); 

	/*
	 *	search down the list
	 */
	while (transRec != NULL)	{

		TRPRINT(TR_TRANS, TR_LEVEL_2, ("looking at tid:%x", GETTID(transRec)));

		/*
		 *	check the entry magic number
		 */
		CHECK_TRANSREC_MAGIC(transRec);

		/*
		 *	try to match the transaction ids
		 */
		if (transRec->tid == tid)	{

			/*
			 *	return the current transRec
			 */
			return(transRec);

		} else {

			/*
			 *	look at the next element in the list
			 */
			transRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(transRec->distrTransList) );
		}
	}

	/*
	 *	if we are here we haven't found the item
	 */
	return(NULL);
}
