/*
 *   $RCSfile: initializeDistr.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:40 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "util_funcs.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "bf_globals.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_intfuncs.h"
#include "distr_extfuncs.h"


 void
initializeDistr ()

{

	GTIDREC			*gtidRec;
	SERVERTIDINFO	*serverTidInfo;
	int				spaceNeeded;
	int				i;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	Initialize the gtidRec pool 
	 */
	initializePool( &GtidRecPool, (ALLOCFUNC) allocGtidRec, NumGtidRecs );
	
	/*
	 *	calculate how much space to allocate for the gtids 
	 */
	spaceNeeded = sizeof(GTIDREC) * NumGtidRecs;

	/*
	 *	Attempt to allocate the space
	 */
	if ((GtidRecPtr = gtidRec = (GTIDREC *) malloc(spaceNeeded)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	initialize the individual gtids 
	 */
	for (i = 0; i < NumGtidRecs; i++, gtidRec++)	{

		/*
		 * initialize the length field to zero
		 */
		gtidRec->gtid.length = 0;

		/* 
		 * initialize the list magic 
		 */
		INIT_LIST_MAGIC(&(gtidRec->gtidList));
		initializeListElement( &(gtidRec->gtidList), (char *) gtidRec );

		/*
		 *	put the record in the pool
		 */
		listEnq( &(GtidRecPool.freeList), &(gtidRec->gtidList) );

	}

	/*
	 *	Initialize the serverTidInfo pool 
	 */
	initializePool( &ServerTidInfoPool, (ALLOCFUNC) allocServerTidInfo, NumServerTidInfos );
	
	/*
	 *	calculate how much space to allocate for the serverTidInfos 
	 */
	spaceNeeded = sizeof(SERVERTIDINFO) * NumServerTidInfos;

	/*
	 *	Attempt to allocate the space
	 */
	if ((ServerTidInfoPtr = serverTidInfo = (SERVERTIDINFO *) malloc(spaceNeeded)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	initialize the individual serverTidInfos 
	 */
	for (i = 0; i < NumServerTidInfos; i++, serverTidInfo++)	{

		initServerTidInfo(serverTidInfo);

		/*
		 *	put the record in the pool
		 */
		listEnq( &(ServerTidInfoPool.freeList), &(serverTidInfo->serverTidInfoList));

	}

	/*
	 *	initialize the lists of active distributed transactions 
	 */
	initializeList(&CoordDistrTransList);
	initializeList(&ServerDistrTransList);
}
