/*
 *   $RCSfile: replyDatagram.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:40 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "io.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "openlog.h"
#include "logrecs.h"
#include "msgvector.h"
#include "msg_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "msg_globals.h"
#include "log_globals.h"
#include "server_util_funcs.h"
#include "distr.h"
#include "distr_intfuncs.h"


extern int		errno;


 int
replyDatagram (

	int					error,
	int					code,
	int					vecCount,
	BOOL				taskDie 
)
{
	/*
	 *	NOTE: this function builds the reply on the original message.  
	 */

	register TCB		*tcb;
	int					vecIndex;


	TRPRINT(TR_MSG, TR_LEVEL_1, 
			("thread:%d replying on link:%d code:%d vecCount:%d",
			Active->id, Active->linkId, code, vecCount));

	/*
	 *	get a register pointer to this thread
	 */
	tcb = Active;

	SM_ASSERT(LEVEL_1, (tcb->message.header.type <= MAX_MSG_NUMBER));

	if(code == esmBADMESSAGEMAGIC || code == esmPROTOCOLVERSION) {
		SM_ASSERT(LEVEL_1, (error != esmNOERROR));
		tcb->message.header.version = MESSAGE_VERSION;
		tcb->message.header.type = ERROR_REPLY;
	}
	SM_ASSERT(LEVEL_3, ((error == esmNOERROR)&&(code == esmNOERROR)) ||
                       ((error != esmNOERROR)&&(code != esmNOERROR)));
	MsgStats.sent[tcb->message.header.type] ++;
	/* fill in the errno */
	tcb->message.header.params.out.errno = code;

#if TRACING IS_ENABLED
	{
		register int i;

		for (i = 0; i < vecCount; i++)	{
			TRPRINT(TR_MSG, TR_LEVEL_2, 
				("vector:%d count:%d", i, tcb->iovec[i].iov_len));
		}

	}
#endif


	TRPRINT(TR_MSG, TR_LEVEL_2, ("first length:%d", tcb->iovec[0].iov_len));

	for (vecIndex = 0; vecIndex < vecCount ; vecIndex++) {

		if (sendto(fdUdp, tcb->iovec[vecIndex].iov_base, 
			(int )tcb->iovec[vecIndex].iov_len, 0, 
			(struct sockaddr *) &(tcb->replyAddr), sizeof(SOCKADDR))
			< 0) {

			/* these 2 errors in this case also */
			if ( (errno == EBADF) || (errno == EPIPE) )	{

				TRPRINT(TR_MSG, TR_LEVEL_2, ("bad socket:%d", tcb->linkId));

				/*
			 	 *	check to see if the task should die
			 	 */
				if (taskDie)	{
	
					threadRestart();
	
				} else {
	
					return(esmFAILURE);
				}

			} else { /* errno neither EBADF or EPIPE */

				SM_ERROR(TYPE_FATAL, errno);
			}
		} /* end of sendto */
	} /* end of for vecCount */

	{
		register int i, totalBytes;

		for (totalBytes=0, i = 0; i < vecCount; i++)	totalBytes += tcb->iovec[i].iov_len;
		incPages(totalBytes, SENT, TRUE); /* MsgStats */
	}

	/*
	 *	check to see if this is a transaction thread
	 */
	transThreadDeath(tcb);

	/*
	 *	restart this thread
	 */
	if (taskDie)	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("task restarting"));
		threadRestart();

	} 
	return esmNOERROR;
}
