/*
 *   $RCSfile: replyWithAck.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:40 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "cl_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "distr.h"
#include "server_util_funcs.h"
#include "distr_intfuncs.h"
#include "distr_globals.h"
#include "admin_funcs.h"


 void
replyWithAck (

	register TRANSREC	*transRec
)
{

	TID				localTid = Active->message.body.vote.localTid;
	TID				coordTid = Active->message.body.vote.coordTid;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("localTid:%x", localTid));

	/*
	 *	initialize the type, coordTid, and localTid of the reply message
	 */
	Active->message.header.type = SERVER_ACK;
	Active->message.body.ack.coordTid = coordTid;
	Active->message.body.ack.localTid = localTid;

	/*
	 *	mark the active transaction
	 */
	Active->transRec = (char *) transRec;

	SM_ASSERT(LEVEL_3, (transRec->transState == T_PREPARED));
	SM_ASSERT(LEVEL_3, ((transRec->coordCommand == C_COMMIT) || 
						(transRec->coordCommand == C_ABORT)));

	/*
	 * carry out the command
	 */
	if (transRec->coordCommand == C_COMMIT) {

		/*
		 *  mark the intent to commit
		 */
		transRec->intent = T_COMMIT;

		/*
		 *	commit the transaction
		 */
		if (serverCommitTrans(transRec)) {

				 /* TODO: operator recovery required */
				 SM_ERROR(TYPE_CRASH, esmINTERNAL);

		 } else {

#ifdef DEBUG
	if (DebugCrashLoc == 15)
		admin_ShutServer(0);
#endif DEBUG
		 	 replyDatagram(esmNOERROR, esmNOERROR, 1, TRUE);
		 }

	} else if (transRec->coordCommand == C_ABORT) {

		/*
		 *  mark the intent to abort
		 */
		transRec->intent = T_ABORT;

		/*
		 *	abort the transaction
		 */
		abortTrans(transRec, esmCLIENTREQUEST);

		/*
		 *	restart this thread
		 */
		transThreadDeath(Active); 
		threadRestart();
	}
}
