/*
 *   $RCSfile: serverTidInfo.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:40 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "consist.h"
#include "file.h"
#include "deallocinfo.h"
#include "fi_extfuncs.h"
#include "trans_extfuncs.h"
#include "io_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_intfuncs.h"


 SERVERTIDINFO 
*allocServerTidInfo (

	POOL		*head 

)
{

	SERVERTIDINFO	*serverTidInfo;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	Attempt to allocate the space
	 */
	if ((serverTidInfo = (SERVERTIDINFO *) malloc(sizeof(SERVERTIDINFO))) == NULL)	{

		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return(NULL);
	}

	/*
	 *	initialize the serverTidInfo record 
	 */
	initServerTidInfo(serverTidInfo);

	/*
	 *	Increment the number of elements in the pool
	 */
	head->elements++;

	/*
	 *	return a pointer to the serverTidInfo record
	 */
	return(serverTidInfo);
}


/*
 *	This function traverses the list of serverTidInfos 
 *	and returns them to the pool
 */

 int
freeServerTidInfo (

	TRANSREC		*transRec 
)
{

	SERVERTIDINFO	*serverTidInfo;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	Process each serverTidInfo 
	 */
	while ((serverTidInfo = (SERVERTIDINFO*) listDeq(&(transRec->serverList))) != NULL) {

		/*
		 *	Return to the pool
		 */
		poolEnq(&ServerTidInfoPool, &(serverTidInfo->serverTidInfoList));
	}

	return(esmNOERROR);
}


 void
initServerTidInfo (
	register SERVERTIDINFO	*serverTidInfo 
)
{

	TRACE(TR_DISTR, TR_LEVEL_1);

	/*
	 *	initialize the list components
	 */
	initializeListElement( &(serverTidInfo->serverTidInfoList), (char *) serverTidInfo );

	/*
	 *	initialize the tid, vote and ack
	 *	TODO: initialize the connectAddr ?
	 */
	serverTidInfo->tid = NULL_TID;
	serverTidInfo->vote = VOTEUNKNOWN;
	serverTidInfo->ack = FALSE;

}
