/*
 *   $RCSfile: server_LogActions.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logprepare.h"
#include "logaction.h"
#include "log.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "log_globals.h"
#include "bf_macro.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"
#include "distr.h"
#include "distr_intfuncs.h"
#include "TempBufSpace.h"


 int
server_LogCommit (

	register TRANSREC	*transRec
)
{

	TRPRINT(TR_LOG|TR_TRANS, TR_LEVEL_1, ("server_LogCommit: tid :%d", transRec->tid));

	/*
	 *	write the log record
	 */
	if (writeTransStateRecord(transRec, LOG_REC_TYPE_COMMIT, LOG_ACTION_DISTR_SERVER_COMMIT, T_COMMIT, TRUE) < esmNOERROR) {
		return(esmFAILURE);
	
	} else {

		return(esmNOERROR);
	}
}


 int
server_LogPrepare (

	TRANSREC			*transRec,
	VOTE				*vote
)
{

	register FOUR		forceMark;
	LSN					lsn;
	LOGRECORDINFO		recordInfo;
	SERVERPREPAREHDR	prepareHdr;
	PREPARELOCKINFO		*lockInfoPtr, *lockInfoBegin;
	VOLIDNAME			*volidNamePtr, *volidNameBegin;
	LOCKENTRY			*lockEntry;
	TRANSVOLREC			*transVolRec;
	int					lockCount = 0;
	int					volCount = 0;
	int					numLocks;
	int					numVols;
	BOOL				mallocedLocks = FALSE;
	BOOL				mallocedVols = FALSE;
	TempBufSpace		tempSpace1(MAXLOCKSONSTACK * sizeof(PREPARELOCKINFO));
	PREPARELOCKINFO		*lockInfo = (PREPARELOCKINFO *) tempSpace1.location();
	TempBufSpace		tempSpace2(MAXVOLSONSTACK * sizeof(VOLIDNAME));
	VOLIDNAME			*volidName = (VOLIDNAME *)tempSpace2.location();

	TRPRINT(TR_IO, TR_LEVEL_1, ("server logging prepare: tid %d", transRec->tid));
	/*
	 *	initialize the vote
	 */
	*vote = READVOTE;

	/*
	 *	find out how many X, IX, and SIX locks the trans holds
	 */
	numLocks = countLocks(transRec, vote);

	if (*vote == READVOTE) {
		/*
		 *	read-only transaction
		 *	return success
		 */
		return esmNOERROR;
	}
	/*
	 *	find out how many volumes are mounted by this trans 
	 */
	numVols = countVols(transRec);

	/*
	 *	decide if all the lock and volume information will 
	 *	fit into one log record
	 */
	if (sizeof(LOGRECORDHDR) + 
		numLocks * sizeof(PREPARELOCKINFO) +
		numVols * sizeof(VOLIDNAME) > MAX_LOGREC_LEN) {

		/*
		 *	we'll need more than one log record -
		 *	we'll use atleast three - 	one for the locks, one for 
		 *	the volumes, and one for the actual preapre log record 
		 */

		/*
		 *	log the file locks
		 */
		if (server_LogPrepareLocks(transRec, vote) != esmNOERROR)	{
			/*
			 *	return failure
			 */
			return(esmFAILURE);
		}
	
		/*
		 *	log the mounted volumes 
		 */
		if (server_LogPrepareVols(transRec) < esmNOERROR)	{
			/*
			 *	return failure
			 */
			return(esmFAILURE);
		}

	} else {
		
		/*
		 *	initialize the log information
		 */
		recordInfo.type			= LOG_REC_TYPE_SERVER_PREPARE;
		recordInfo.action		= LOG_ACTION_DISTR_SERVER_PREPARE_VOL_LOCK;
		recordInfo.actionPid	= NULL;
		recordInfo.flags        = NOFLAGS;
		recordInfo.imageCount	= 2;
	
	
		/*
		 *	decide if space needs to be malloced 
		 *	for the locks to be logged
		 */
		if (numLocks > MAXLOCKSONSTACK) {
	
			/*
			 *	malloc the space
			 */
			if ((lockInfoBegin = (PREPARELOCKINFO *) malloc(numLocks * sizeof(PREPARELOCKINFO))) 
				== NULL)	{

				SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
				return(esmFAILURE);
			}
			else
				mallocedLocks = TRUE;
		}
		else
			lockInfoBegin = lockInfo;

		/*
		 *	get hold of the first LOCKENTRY record on the 
		 *	lockGrantedList for this transaction
		 */
		lockEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT (&(transRec->lockGrantedList));

		/*
		 *	initialize the current pointer for the array of lockInfos
		 */
		lockInfoPtr = lockInfoBegin;
	
		/*
		 *	go through the list of locks to pick out all the locks
		 *	to be logged
		 */
		while (lockEntry != NULL)	{
	
			/*
			 *	see if the lock needs to be logged	
			 */
			if (PREPARELOG(lockEntry)) {
					
				/*
				 *	check if this is an exclusive (write) lock
				 */
				if (lockEntry->lockHeader->supremum == EX)
					*vote = YESVOTE;
	
				/*
				 *	put this in the list to be logged
				 */
				lockInfoPtr->lockid = lockEntry->lockHeader->hashList.lockid;
				lockInfoPtr->mode = lockEntry->lockHeader->supremum;
	
				/*
				 *	advance the pointer
				 */
				lockInfoPtr++;
	
				/*
				 *	increment the count
				 */
				lockCount++;
			}
	
			/*
			 *  get a pointer to the next link element
			 */
			lockEntry = (LOCKENTRY *) NEXT_LIST_ELEMENT(&(lockEntry->transList));
		}
	
		/*
		 *	decide if space needs to be malloced for volumes to be logged
		 */
		if (numVols > MAXVOLSONSTACK) {
	
			/*
			 *	malloc the space
			 */
			if ((volidNameBegin = (VOLIDNAME *) malloc(numVols * sizeof(VOLIDNAME))) 
				== NULL)	{
				SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
				return(esmFAILURE);
			}
			else
				mallocedVols = TRUE;
		}
		else
			volidNameBegin = volidName;
	
		/*
		 *	get hold of the first TRANSVOLREC record on the volumeList 
		 *	for this transaction
		 */
		transVolRec = (TRANSVOLREC *) FIRST_LIST_ELEMENT(&(transRec->volumeList));
			
		/*
		 *	initialize the current pointer for the array of volidNames
		 */
		volidNamePtr = volidNameBegin;
	
		/*
		 *	go through the list of transVolRecs to record the volumes 
		 *	mounted by this tranaction
		 */
		while (transVolRec != NULL)	{
	
			if ((transVolRec->volRec->volflags & VOL_TEMP) == 0) {
				volidNamePtr->volid = transVolRec->volRec->header->volid;
				strncpy(volidNamePtr->volName, 
					transVolRec->volRec->volNameRec->volName.name,
					sizeof(transVolRec->volRec->volNameRec->volName.name));
	
				volidNamePtr++;
	
				volCount++;
			}
	
				transVolRec = (TRANSVOLREC *) NEXT_LIST_ELEMENT(&(transVolRec->volumeList));
		}
	
		/*
		 * fill in the remaining log info 
		 */
		recordInfo.imageSize[0] = numLocks * sizeof(PREPARELOCKINFO);
		recordInfo.imageData[0] = (char *) lockInfoBegin;
		recordInfo.imageSize[1] = numVols * sizeof(VOLIDNAME);
		recordInfo.imageData[1] = (char *) volidNameBegin;

		/*
		 *	record the prepareLSN for this transaction 
		 */
		transRec->prepareLSN = OpenLog.nextValidLSN;	

		/*
		 *  write the record to the log
		 */
		if ((forceMark = writeLogRecord(/*(TRANSREC *) Active->*/transRec,
			&recordInfo, &lsn, NOFLAGS)) < 0) {
	
			return(esmFAILURE);
		}

		/*
		 *	free space allocated for locks
		 */
		if (mallocedLocks == TRUE) {
			free(lockInfoBegin);
		}
	
		/*
		 *	free space allocated for vols
		 */
		if (mallocedVols == TRUE) {
			free(volidNameBegin);
		}
	}

	/*
	 *	write the log record containing the SERVERPREPAREHDR
	 */
	prepareHdr.coordAddr = transRec->coordAddr;
	prepareHdr.coordTid = transRec->coordTid;
	prepareHdr.prepareLSN = transRec->prepareLSN; 
	prepareHdr.firstLogPid = transRec->firstLogPid; 
	prepareHdr.logUnique = transRec->logUnique; 
	prepareHdr.firstLSN = transRec->firstLSN; 
	prepareHdr.logRecordCount = transRec->logRecordCount; 
	prepareHdr.logSpace = transRec->logSpace; 

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_SERVER_PREPARE;
	recordInfo.action		= LOG_ACTION_DISTR_SERVER_PREPARE_HDR;
	recordInfo.actionPid	= NULL;
	recordInfo.flags        = NOFLAGS;
	recordInfo.imageCount	= 1;
	recordInfo.imageSize[0] = sizeof(SERVERPREPAREHDR);
	recordInfo.imageData[0] = (char *) (&prepareHdr);
	
	/*
	 *  write the record to the log
	 */
	if ((forceMark = writeLogRecord(/*(TRANSREC *) Active->*/transRec,
		&recordInfo, &lsn, NOFLAGS)) < 0) {

		return(esmFAILURE);
	}

	/*
	 *  write the prepare record to the log
	 */
	if (writeTransStateRecord(transRec, LOG_REC_TYPE_SERVER_PREPARE, ZERO, T_PREPARED, TRUE) < esmNOERROR) {

		return(esmFAILURE);
	}

	/*
	 *	return success
	 */
	return(esmNOERROR);
}


 int
server_LogPrepareLocks (

	TRANSREC			*transRec,
	VOTE				*vote
)
{

	register FOUR		forceMark;
	LOGRECORDINFO       recordInfo;
	LSN					lsn;
	BOOL				lsnRecorded = FALSE;
	PREPARELOCKINFO		*lockInfoPtr, *lockInfoBegin;
	LOCKENTRY			*lockEntry;
	int					lockCount = 0;
	int					numRemaining = 0;
	int					numLocks = (int)transRec->numLocks;


	TRPRINT(TR_IO, TR_LEVEL_1, ("server logging prepare: tid %d", transRec->tid));

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_SERVER_PREPARE;

	/*
	 *	NOTE: overloading action field to mark that this log record
	 *	contains the info about locks for a server prepare record
	 */
	recordInfo.action		= LOG_ACTION_DISTR_SERVER_PREPARE_LOCK;
	recordInfo.flags        = NOFLAGS;
	recordInfo.actionPid	= NULL;
	recordInfo.imageCount	= 1;

	/*
	 *	decide if all the information will fit into one log record
	 */
	if (numLocks < MAXLOCKINFOINIMAGE) {

		/*
		 *	allocate space as required
		 */
		if ((lockInfoBegin = (PREPARELOCKINFO *) malloc(numLocks * sizeof(PREPARELOCKINFO))) 
			== NULL)	{

			SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}
	else	{
		/*
		 *	allocate the maximum possible size of an image space
		 */
		if ((lockInfoBegin = (PREPARELOCKINFO *) malloc(MAXLOCKINFOINIMAGE * sizeof(PREPARELOCKINFO))) 
			== NULL)	{

			SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}

	/*
	 *	get hold of the first LOCKENTRY record on the lockGrantedList 
	 *	for this transaction
	 */
	lockEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT (&(transRec->lockGrantedList));

	while (lockCount < numLocks)	{
		
		/*
		 *	if this is the first log record, record the lsn
		 */
		if (lsnRecorded == FALSE) {
			
			lsnRecorded = TRUE;
			transRec->prepareLSN = OpenLog.nextValidLSN;
		}

		/*
		 *	initialize numRemainig 
		 */
		numRemaining = numLocks - lockCount;
		if (numRemaining > MAXLOCKINFOINIMAGE)
			numRemaining = MAXLOCKINFOINIMAGE;
		
		/*
		 *	initialize the current pointer for the array of lockInfos
		 */
		lockInfoPtr = lockInfoBegin;

		/*
		 *	fill in the image size and data pointer
		 */
		recordInfo.imageSize[0] = numRemaining * sizeof(PREPARELOCKINFO);
		recordInfo.imageData[0] = (char*) lockInfoBegin;

		/*
		 *	go thru the list of locks to fill in the log record
		 */
		while (numRemaining	> 0) {
			
			if (lockEntry == NULL)	{
				
				/*
				 *	this should not happen
				 */
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

				/*
				 *  see if the lock needs to be logged
				 */
				if (PREPARELOG(lockEntry)) {
				
				/*
				 *  check if this is an exclusive (write) lock
				 */
				if (lockEntry->lockHeader->supremum == EX)
					*vote = YESVOTE;

				/*
				 *	put this in the list to be logged
				 */
				lockInfoPtr->lockid = lockEntry->lockHeader->hashList.lockid;
				lockInfoPtr->mode = lockEntry->lockHeader->supremum;

				/*
				 *	advance the pointer
				 */
				lockInfoPtr++;

				/*
				 *	update the counts
				 */
				lockCount++;
				numRemaining--;

			}
			
			/*
			 *  get a pointer to the next link element
			 */
			lockEntry = (LOCKENTRY *) NEXT_LIST_ELEMENT(&(lockEntry->transList));
		}

		/*
		 *  write the record to the log
		 */
		if ((forceMark = writeLogRecord(/*(TRANSREC *) Active->*/transRec,
			&recordInfo, &lsn, LOG_FORCE)) < 0) {

			return(esmFAILURE);
		}

	}

	/*
	 *	free space allocated 
	 */
	free(lockInfoBegin);

	return(esmNOERROR);

}


 int
server_LogPrepareVols (

	TRANSREC			*transRec
)
{

	register FORCEMARK	forceMark;
	LOGRECORDINFO		recordInfo;
	LSN					lsn;
	VOLIDNAME			*volidNamePtr, *volidNameBegin;
	TRANSVOLREC			*transVolRec;
	int					volCount = 0;
	int					numRemaining = 0;
	int					numVols = (int)transRec->numVols;


	TRPRINT(TR_IO, TR_LEVEL_1, ("server logging prepare: tid %d", transRec->tid));

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_SERVER_PREPARE;

	/*
	 *	NOTE: overloading action field to mark that this log record
	 *	contains the info about volumes for a server prepare record
	 */
	recordInfo.action		= LOG_ACTION_DISTR_SERVER_PREPARE_VOL;
	recordInfo.flags        = NOFLAGS;
	recordInfo.actionPid	= NULL;
	recordInfo.imageCount	= 1;

	/*
	 *	decide if all the information will fit into one log record
	 */
	if (numVols < MAXVOLINIMAGE) {

		/*
		 *	allocate space as required
		 */
		if ((volidNameBegin = (VOLIDNAME *) malloc(numVols * sizeof(VOLIDNAME))) 
			== NULL)	{

			SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}
	else	{
		/*
		 *	allocate the maximum possible size of an image space
		 */
		if ((volidNameBegin = (VOLIDNAME *) malloc(MAXVOLINIMAGE * sizeof(VOLIDNAME))) 
			== NULL)	{

			SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
			return(esmFAILURE);
		}
	}

	/*
	 *  get hold of the first TRANSVOLREC record on the volumeList
	 *  for this transaction
	 */

	transVolRec = (TRANSVOLREC *) FIRST_LIST_ELEMENT (&(transRec->volumeList));

	while (volCount < numVols)	{
		
		/*
		 *	initialize numRemainig 
		 */
		numRemaining = numVols - volCount;
		if (numRemaining > MAXVOLINIMAGE)
			numRemaining = MAXVOLINIMAGE;
		
		/*
		 *	initialize the current pointer for the array of volidNames
		 */
		volidNamePtr = volidNameBegin;

		/*
		 *	fill in the image size and data pointer
		 */
		recordInfo.imageSize[0] = numRemaining * sizeof(VOLIDNAME);
		recordInfo.imageData[0] = (char*) volidNameBegin;

		/*
		 *	go through the list of transVolRecs to record the volumes
		 *	mounted by this tranaction
		 */
		while (numRemaining	> 0) {
			
			if (transVolRec == NULL)	{
				
				/*
				 *	this should not happen
				 */
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			if ((transVolRec->volRec->volflags & VOL_TEMP) == 0) {

				/*
				 *	put this in the list to be logged
				 */
				volidNamePtr->volid = transVolRec->volRec->header->volid;
				strncpy(volidNamePtr->volName, 
					transVolRec->volRec->volNameRec->volName.name,
					sizeof(transVolRec->volRec->volNameRec->volName.name));

				/*
				 *	advance the pointer
				 */
				volidNamePtr++;

				/*
				 *	update the counts
				 */
				volCount++;
				numRemaining--;

			}

			/*
			 *  get a pointer to the next link element
			 */
			transVolRec = (TRANSVOLREC *) NEXT_LIST_ELEMENT(&(transVolRec->volumeList));
		}

		/*
		 *  write the record to the log
		 */
		if ((forceMark = writeLogRecord(/*(TRANSREC *) Active->*/transRec,
			&recordInfo, &lsn, LOG_FORCE)) < 0) {

			return(esmFAILURE);
		}

	}

	/*
	 *	free space allocated 
	 */
	free(volidNameBegin);

	return(esmNOERROR);

}
