/*
 *   $RCSfile: FI_CheckFull.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "fi_macro.h"
#include "consist.h"
#include "fi_intfuncs.h"


 int
FI_CheckFull (

	PID			*fid,
	BUFGROUP	*bufGroup,
	PAGESTACK	*parentStack,
	GROUPLINK	*nodeLink,
	PID			*readPid 
)
{

	NODEPAGE	*nodePage;


	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	get a pointer to the node
	 */
	nodePage = (NODEPAGE *) nodeLink->bufFrame;
    
	/*
	 *	check to see what type of node this is
	 */
	if (FI_Leaf(nodePage->header.nodeType))	{

		TRPRINT(TR_FI, TR_LEVEL_2, ("node is leaf"));

		/*
		 *	check to see if the node is full
		 */
		if (nodePage->header.nodeCount == FULLEAFNODE)	{

			TRPRINT(TR_FI, TR_LEVEL_2, ("node is full"));

			/*
			 *	we must split the node
			 */
			if (FI_SplitNode(fid, bufGroup, parentStack, nodeLink, readPid))	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			/*
			 *	return that we have split the node
			 */
			return(TRUE);
		
		} else {

			/*
			 *	return that we have not split the node
			 */
			return(FALSE);
		}

	} else {

		TRPRINT(TR_FI, TR_LEVEL_2, ("node is not leaf"));

		/*
		 *	check to see if the node is full
		 */
		if (nodePage->header.nodeCount == FULLNONLEAFNODE)	{

			TRPRINT(TR_FI, TR_LEVEL_2, ("node is full"));

			/*
			 *	we must split the node
			 */
			if (FI_SplitNode(fid, bufGroup, parentStack, nodeLink, readPid))	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}

			/*
			 *	return that we have split the node
			 */
			return(TRUE);
		
		} else {

			/*
			 *	return that we have not split the node
			 */
			return(FALSE);
		}
	}
}
