/*
 *   $RCSfile: FI_DeleteFromArray.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "util_funcs.h"
#include "fi_intfuncs.h"


/*============================================================================*/
/* 
 * FI_DeleteFromArray
 * 	Deletes the keyPtr pair at location 'deleteLoc' within the array in page
 *	'deletePage
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	efiBADPARAMS	NULL page pointer
 *	efiNEGSUBSCRIPT	Negative delete location
 *
 * SIDE EFFECTS:
 *	The keyPtr pair at the specified location is deleted from the page and 
 *	the node count is decremented
 */
/*============================================================================*/


 void
FI_DeleteFromArray (

	NODEPAGE			*deletePage,
	FOUR				deleteLoc,
	SHORTPID			deleteKey 
)
{

    register TREEDATA	*tempArray;
    int					nodeType;
    int					nodeCount;
   	int					i;


	TRPRINT(TR_FI, TR_LEVEL_1, ("deleteLoc:%d", deleteLoc));

	/*
	 *	Check the validity of the delete location
	 */
    SM_ASSERT(LEVEL_3, deleteLoc >= 0);

    /*
     *	Grab the nodeCount, type and array of key/keyPtrs
     */
    nodeCount = (int) deletePage->header.nodeCount;
    nodeType  = deletePage->header.nodeType;
    tempArray = &(deletePage->contents);

    /*
     *	Separate cases for non leaves and leaves:
     *	Overwrite the deleted pair by moving everything after it one step back
     */
    if (FI_NotLeaf(nodeType))  {

		/*
		 *	check to make sure we are deleting the right key
		 */
		SM_ASSERT(LEVEL_1, tempArray->keyPtr[deleteLoc].key == deleteKey);

#ifdef JUNK
		/* BCOPY does the equivalent of the for loop below: */
		bcopy( (char *) &(tempArray->keyPtr[deleteLoc + 1]),
			   (char *) &(tempArray->keyPtr[deleteLoc]),
			   (int) ((nodeCount - deleteLoc - 1) * KEYPTRSIZE) );
#endif

		 for (i=deleteLoc; i < nodeCount - 1; i++)  {
			 tempArray->keyPtr[i] = tempArray->keyPtr[i+1];
		 }

    } else  {

		/*
		 *	check to make sure we are deleting the right key
		 */
		SM_ASSERT(LEVEL_1, (tempArray->keyPid.allKeys[deleteLoc].key|SETLARGE) == (deleteKey|SETLARGE));

#ifdef JUNK
		/* BCOPY does the equivalent of the for loop below: */
		bcopy( (char *) &(tempArray->keyPid.allKeys[deleteLoc + 1]),
			   (char *) &(tempArray->keyPid.allKeys[deleteLoc]),
			   (int) ((nodeCount - deleteLoc - 1) * KEYSIZE) );
#endif

		for (i=deleteLoc; i < nodeCount - 1; i++)  {
			tempArray->keyPid.allKeys[i] = tempArray->keyPid.allKeys[i+1];
		}

    }

    /*
     *	update node count
     */
    deletePage->header.nodeCount -= 1;
}
