/*
 *   $RCSfile: FI_InitLinkConsistRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "link.h"
#include "bf_macro.h"
#include "volume.h"
#include "trans.h"
#include "consist.h"
#include "file.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "fi_globals.h"


 void
FI_InitLinkConsistRec ()

{
	register int	i;
	register int	numbytes;


	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	calculate size of hash table to use
	 */
	numbytes = NumLinkConsistRecs * sizeof(LINKCONSISTREC);

	/*
	 *	allocate the hash table
	 */
	if ((LinkConsistRecs = (LINKCONSISTREC *) malloc(numbytes)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	Initialize the Page Hash Pool
	 */
	initializePool( &LinkConsistPool, NULL, NumLinkConsistRecs );

	/*
	 *	Put the page hash structures on the free list
	 */
	for (i = 0; i < NumLinkConsistRecs; i++)	{

		/*
		 *	initialize the element
		 */
		initializeListElement( &(LinkConsistRecs[i].lockHeaderList),
							   (char *) &(LinkConsistRecs[i])  );

		/*
		 *	put the free function in
		 */
		LinkConsistRecs[i].deallocFunc = (PFI) fi_FreeLinkConsist;

		/*
		 *	put it into the pool
		 */
		listEnq( &(LinkConsistPool.freeList), &(LinkConsistRecs[i].lockHeaderList) );
		INIT_LINKCONSISTREC_MAGIC( &(LinkConsistRecs[i]) );
	}
}
