/*
 *   $RCSfile: FI_InitStack.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "link.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_globals.h"


/*============================================================================*/
/* 
 * FI_StackInit()
 * 	Links the 'numNodes' nodes in the array to be used as a free node list.
 *	Returns NULL in topOfStack, head of list in freeNodes.
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	None
 *
 * SIDE EFFECTS:
 *	None other than that described above
 */
/*============================================================================*/


 void
FI_InitStack ()

{
	register PAGESTACK	*pageStack;
	register int		numBytes;
	register int		i;


	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	initialize the page stack pool
	 */
	initializeList( &PageStackPool );

	/*
	 *	Calculate the number of bytes to allocate
	 */
	numBytes = NumPageStacks * sizeof(PAGESTACK);

	/*
	 *	Attempt to malloc the space
	 */
	if ((pageStack = (PAGESTACK *) malloc(numBytes)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	link the stacks onto the free list
	 */
	for (i = 0; i < NumPageStacks; i++)	{

		/*
		 *	Initialize the list element
		 */
		initializeListElement( &(pageStack[i].stackList), (char *) &(pageStack[i]) );

		/*
		 *	Put the item on the list
		 */
		listEnq( &PageStackPool, &(pageStack[i].stackList) );
	}
}

