/*
 *   $RCSfile: FI_InsertSearch.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "bf_extfuncs.h"


 int
FI_InsertSearch (

	BUFGROUP			*bufGroup,
	PID					*rootPid,
	SHORTPID			insertKey,
	PAGESTACK			*pageStack,
	GROUPLINK			**foundLink,
	int					*foundLoc 
)
{

    register TREEDATA	*pageArray;		/* array of keys */
    register SHORTPID	lastKey;		/* last, middle key in page */
    register SHORTPID	midKey;
    register int		mid, first, last;/* indices for Binary search */
    NODEPAGE			*nodePage;		/* current page handled */
    int					nodeCount;		/* number of keys in node to do B search on */
	GROUPLINK			*nodeLink;		
    int					nodeType;		/* node Type */
    PID					readPid;		/* PID of page to read */


	TRPRINT(TR_FI, TR_LEVEL_1, ("searching for page:%d", insertKey));

    /*
     * Read in first page of file and grab node type
     */
    if ((nodeLink = bf_ReadPage(bufGroup, rootPid, FILE_PAGE2SIZE, BF_SEM)) == NULL)	{

		return(esmFAILURE);
    }

	/*
	 *	Get the current page out of the grouplink
	 */
	nodePage = (NODEPAGE *) nodeLink->bufFrame;
	nodeType = nodePage->header.nodeType;

    /*
     *	Check if first page read in is ROOT PAGE
     */
    SM_ASSERT(LEVEL_3, FI_Root(nodeType));

    /*
     * Initialize array pointer, # of keys, flag 
     */
    pageArray = &(nodePage->contents);
    nodeCount = (int) nodePage->header.nodeCount;

	/*
	 *	Set the readpage value
	 */
	readPid = *rootPid;

    /*
     *	Search nodes until leaf is reached
	 *
	 *	(The structure of this loop must be changed - DMH)
     */
    for (;;)	{

		/*
		 *	print the contents of the node
		 */
		fi_PrintNode(nodePage);

		/*
		 *	check the magic number of the page
		 */
		if (nodePage->header.magic != FILEPAGE_MAGIC)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *	check to see if the node is full
		 */
		if (FI_CheckFull(rootPid, bufGroup, pageStack, nodeLink, &readPid))	{

			register STACKNODE	*stackTop;

			TRPRINT(TR_FI, TR_LEVEL_2, ("split occurred"));

			/*
			 *	relook at the previous stack element if there is one
			 */
			if (pageStack->stackTop >= 0)	{

				TRPRINT(TR_FI, TR_LEVEL_2, ("backup one place on stack"));

				stackTop = &(pageStack->stackNode[pageStack->stackTop--]);
				nodeLink = stackTop->groupLink;
				nodePage = (NODEPAGE *) stackTop->groupLink->bufFrame;
				pageArray = &(nodePage->contents);
				readPid.page = stackTop->pageID;
			}

			/*
			 *	reset node type and count as they may have changed
			 */
			nodeType = nodePage->header.nodeType;
			nodeCount = nodePage->header.nodeCount;
		}
		fi_PrintNode(nodePage);

		/*
		 *	If reached a non leaf, decrement nodeCount to do binary search 
		 *	on because non-leaf nodes have 1 less key than pointers
		 */
		if (FI_NotLeaf(nodeType))  {

			--nodeCount;
		}

		/*
		 * Initialize Binary search indices 
		 */
		first = 0;
		last  = nodeCount - 1;
		mid   = (first+last)/2;
			
		/*
		 * Check if any keys in node
		 */
		if (nodeCount > 0)  {

			/*
			 *	Get the middle and last keys on the page
			 */
			if (FI_Leaf(nodeType))  {

				midKey  = (pageArray->keyPid.allKeys[mid].key & LARGEMASK);
				lastKey = (pageArray->keyPid.allKeys[last].key & LARGEMASK);

			} else {

				midKey  = pageArray->keyPtr[mid].key;
				lastKey = pageArray->keyPtr[last].key;
			}

			/*
			 *	If the searchkey is larger than the last key entry in array, 
			 *	don't bother doing a binary search, use subscript to  
			 *	last pointer.
			 */
			if (insertKey > lastKey)  {

				mid = nodeCount;

			} else {

				/*
				 ********************
				 * do BINARY SEARCH *
				 ********************
				 *
				 * do until first and last 'cross over' 
				 */
				while (last > first)  {

					/*
					 *	check to see where the target key resides
					 *	
					 *	(Put in equal check - DMH)
					 */
					if (insertKey <= midKey) {

						/*
						 *	its in the first half of the search space
						 */
						last = mid;

					} else {

						/*
						 *	Its in the 2nd half of the search space
						 */
						first = mid + 1;
					}
			
					/*
					 * Calculate the new mid point
					 */
					mid = (first+last)/2;

					/*
					 *	Grab the middle key
					 */
					if (FI_Leaf(nodeType))  {

						midKey = (pageArray->keyPid.allKeys[mid].key & LARGEMASK);

					} else {

						midKey = pageArray->keyPtr[mid].key;
					}
				}
			}
		}

		/*
		 * we're done if we've reached a leaf node
		 */
		if (FI_Leaf(nodeType))  {

			TRPRINT(TR_FI, TR_LEVEL_2, ("at leaf"));

			/*
			 *	Exit the loop
			 */
			break;

		} else {

			TRPRINT(TR_FI, TR_LEVEL_2, ("not at leaf"));

			/* 
			 *	Otherwise, push onto stack the following information:
			 *		the page #
			 *		the *page address and.....
			 *		the 'path' taken from this page, i.e. the array subscript
			 */
			FI_PushStack(pageStack, readPid.page, nodePage, nodeLink, mid, NOFLAGS);

			/*
			 * Prepare to read the page the link points to.
			 */
			readPid.page = pageArray->keyPtr[mid].pagePtr;

			/*
			 * read the page in and grab the array address, # of keys, node type
			 */
    		if ((nodeLink = bf_ReadPage(bufGroup, &readPid,
										FILE_PAGE2SIZE, BF_SEM)) == NULL)	{

				return(esmFAILURE);
			}

			/*
			 *	Get a pointer to the buffer
			 */
			nodePage = (NODEPAGE *) nodeLink->bufFrame;

			/*
			 *	Setup the variables
			 */
			pageArray = &(nodePage->contents);
			nodeCount = nodePage->header.nodeCount;
			nodeType  = nodePage->header.nodeType;
		}
    }

    /*
     * return the insert location, page number, and *page address
     */
    *foundLoc  = mid;
    *foundLink = nodeLink;
	TRPRINT(TR_FI, TR_LEVEL_2, ("location:%d", *foundLoc));
    return(esmNOERROR);
}
