/*
 *   $RCSfile: FI_LogKeyInsert.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "fi_intfuncs.h"
#include "thread_globals.h"


 int
FI_LogKeyInsert (

	PID					*pid,
	FOUR				insertLoc,
	SHORTPID			key,
	PAGEHASH			*pageHash,
	BOOL				large 
)
{

	FORCEMARK			forceMark;
	LSN					lsn;
	LOGRECORDINFO		recordInfo;
	LRC					*lrc;

	TRPRINT(TR_IO, TR_LEVEL_1, ("logging pid:%d key:%d", pid->page, key));

    /*
     *  Increment the lrc on the page
     */
    lrc = &(((NODEHEADER *) pageHash->bufFrame)->lrc);
    INCREMENT_LRC(lrc);
	
	/*
	 *	initialize the log information
	 */
	recordInfo.action		= LOG_ACTION_INSERT_FILE_KEY;
	recordInfo.type			= LOG_REC_TYPE_USER;
	recordInfo.imageCount	= 3;
	recordInfo.actionPid    = pid;
    recordInfo.actionLRC    = lrc;
	recordInfo.imageSize[0] = sizeof(FOUR);
	recordInfo.imageData[0] = (VOID *) &insertLoc;
	recordInfo.imageSize[1] = sizeof(SHORTPID);
	recordInfo.imageData[1] = (VOID *) &key;
	recordInfo.imageSize[2] = sizeof(BOOL);
	recordInfo.imageData[2] = (VOID *) &large;
	recordInfo.flags		= NOFLAGS;

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		return(esmFAILURE);
	}

	/*
	 *	mark the page dependency
	 */
	DEPEND_LOG(pageHash, forceMark, &lsn, lrc);

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
