/*
 *   $RCSfile: FI_LogPageSplit.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "fi_intfuncs.h"
#include "thread_globals.h"


 int
FI_LogPageSplit (

	VOLID				unused,
	GROUPLINK			*link1,
	GROUPLINK			*link2,
	GROUPLINK			*link3,
	LSNOFFSET			saveLSN 
)
{
	register NODEHEADER	*nodeHeader;
	FORCEMARK			forceMark;
	LSN					lsn;
	LRC					*lrc;
	LOGRECORDINFO		recordInfo;
	PID					pidList[3];		/* 3 pids in the 3 links 	*/
	LRC					lrcList[3];		/* lrc's for the pidLisT 	*/
	char				*filePageList;  /* 3 file pages 			*/

	TRACE(TR_IO, TR_LEVEL_1);

	/*
	 *	Put the pid's in one array, so that they take up only
	 *	one log image 
	 */
	pidList[0] = link1->pageHash->hashList.hashPid;
	pidList[1] = link2->pageHash->hashList.hashPid;
	pidList[2] = link3->pageHash->hashList.hashPid;

	/*
	 *	print the pids that are being logged
	 */
	TRPRINT(TR_FI|TR_LOG, TR_LEVEL_2, ("pid1:%d, pid2:%d, pid3:%d", 
			pidList[0], pidList[1], pidList[2]));

	/*
	 *	Increment the lrc's on the affected pages
	 */
	lrc = &(((NODEHEADER *) link1->pageHash->bufFrame)->lrc);
	INCREMENT_LRC(lrc);
	lrc = &(((NODEHEADER *) link2->pageHash->bufFrame)->lrc);
	INCREMENT_LRC(lrc);
	lrc = &(((NODEHEADER *) link3->pageHash->bufFrame)->lrc);
	INCREMENT_LRC(lrc);

	/*
	 *	Put the lrc's in one array, so that they take up only
	 *	one log image, and then put them in a log image 
	 */
	lrcList[0] = ((NODEHEADER *) link1->pageHash->bufFrame)->lrc;
	lrcList[1] = ((NODEHEADER *) link2->pageHash->bufFrame)->lrc;
	lrcList[2] = ((NODEHEADER *) link3->pageHash->bufFrame)->lrc;

	/*
	 *	Put the file pages in one array, so that they take up only
	 *	one log image .
	 */
	filePageList = (char*) malloc(3*FILE_PAGESIZE);
	bcopy(link1->pageHash->bufFrame, filePageList, FILE_PAGESIZE);
	bcopy(link2->pageHash->bufFrame, filePageList+FILE_PAGESIZE, FILE_PAGESIZE);
	bcopy(link2->pageHash->bufFrame, filePageList+2*FILE_PAGESIZE, FILE_PAGESIZE);

	/*
	 *	initialize the log information
	 */
	recordInfo.action		= LOG_ACTION_SPLIT_FILE_NODE;
	recordInfo.type			= LOG_REC_TYPE_COMPENSATION;
	recordInfo.imageCount	= 3;
	recordInfo.actionPid	= NULL;	/* action pid's are in pidList */
	recordInfo.imageSize[0] = 3*FILE_PAGESIZE;
	recordInfo.imageData[0] = (VOID *) filePageList;
	recordInfo.imageSize[1] = sizeof(pidList);
	recordInfo.imageData[1] = (VOID *) pidList;
	recordInfo.imageSize[2] = sizeof(lrcList);
	recordInfo.imageData[2] = (VOID *) lrcList;
	recordInfo.nextUndoLSN	= saveLSN;
	recordInfo.flags		= NOFLAGS;

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		free(filePageList);
		return(esmFAILURE);
	}
	free(filePageList);

	/*
	 *	get a pointer to the node page
	 */
	nodeHeader = (NODEHEADER *) link1->bufFrame;

	/*
	 *	mark the page dependency
	 */
	DEPEND_LOG(link1->pageHash, forceMark, &lsn, &(nodeHeader->lrc));

	/*
	 *	get a pointer to the node page
	 */
	nodeHeader = (NODEHEADER *) link2->bufFrame;

	/*
	 *	mark the page dependency
	 */
	DEPEND_LOG(link2->pageHash, forceMark, &lsn, &(nodeHeader->lrc));

	/*
	 *	get a pointer to the node page
	 */
	nodeHeader = (NODEHEADER *) link3->bufFrame;

	/*
	 *	mark the page dependency
	 */
	DEPEND_LOG(link3->pageHash, forceMark, &lsn, &(nodeHeader->lrc));

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
