/*
 *   $RCSfile: FI_PopStack.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"


/*============================================================================*/
/* 
 * FI_PopStack
 *	pops the top element of the stack and returns a pointer to it
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	None
 *
 * SIDE EFFECTS:
 *	None other than that described above
 */
/*============================================================================*/


 STACKNODE
*FI_PopStack(

	PAGESTACK	*pageStack 
)
{

	register int		stackTop;


	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	Check to see if the stack is empty
	 */
	if (FI_StackEmpty(pageStack))	{

		return(NULL);
	}

	/*
	 *	save current top and decrement
	 */
	stackTop = (pageStack->stackTop)--;


#if TRACING IS_ENABLED

	{
		int		i;

		for (i = 0; i <= pageStack->stackTop; i++)	{

			TRPRINT(TR_FI, TR_LEVEL_3, ("element:%d page:%d",
					i, pageStack->stackNode[i].pageID));
		}
	}

#endif

	/*
	 *	return a pointer to the previous top element
	 */
	return( &(pageStack->stackNode[stackTop]) );
}
