/*
 *   $RCSfile: FI_PushStack.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"


/*============================================================================*/
/* 
 * FI_PushStack
 *	pushes the page entry to the top of the stack
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	None
 *
 * SIDE EFFECTS:
 *	None other than that described above
 */
/*============================================================================*/


 STACKNODE
*FI_PushStack(

	register PAGESTACK	*pageStack,
	SHORTPID			pageID,
	NODEPAGE			*nodePage,
	GROUPLINK			*groupLink,
	FOUR				path,
	FLAGS				flags 
)
{

	register int		stackTop;
	register STACKNODE	*stackNode;
	register STACKNODE	*signalNode;


	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	Check to see if the stack is full
	 */
	SM_ASSERT(LEVEL_3, pageStack->stackTop != STACKSIZE);

	/*
	 *	increment the top of stack and put it in a register
	 */
	stackTop = ++(pageStack->stackTop);
	TRPRINT(TR_FI, TR_LEVEL_2, ("stackTop:%d", stackTop));

	/*
	 *	get a pointer to the top element
	 */
	stackNode = &(pageStack->stackNode[stackTop]);

	/*
	 *	Assign the values to the array
	 */
	stackNode->pageID	 = pageID;
	stackNode->nodePage	 = nodePage;
	stackNode->groupLink = groupLink;
	stackNode->path		 = path;
	stackNode->flags	 = flags;


#if TRACING IS_ENABLED

	{
		int		i;

		for (i = 0; i <= stackTop; i++)	{

			TRPRINT(TR_FI, TR_LEVEL_3, ("element:%d page:%d",
					i, pageStack->stackNode[i].pageID));
		}
	}

#endif

	/*
	 *	check to see if there is a node two levels up
	 */
	if (stackTop > 2)	{

		/*
		 *	get a pointer to the node to signal
		 */
		signalNode = &(pageStack->stackNode[stackTop - 2]);
		signalSemaphore( &(signalNode->groupLink->pageHash->semaphore) );
		TRPRINT(TR_FI, TR_LEVEL_2, ("signaling semaphore on page:%d",
				signalNode->groupLink->pageHash->hashList.hashPid.page));
	}

	/*
	 *	Return a pointer to the element
	 */
	return(stackNode);
}
