/*
 *   $RCSfile: FI_UnfixStack.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "bf_extfuncs.h"
#include "fi_globals.h"


/*============================================================================*/
/* 
 *	FI_UnfixStack
 *	 Unfixes the pages in the passed page stack
 *
 *	IMPORTS:
 *	 bf_UnfixPage()
 *	 FI_PopStack()
 *
 *	RETURNS
 *	 void
 *
 *	ERROR RETURNS:
 *
 *	SIDE EFFECTS:
 *	 None
 *
 *	NOTES:
 *	 This routine could be sped up by replacing the call to
 *	 popstack by inline code
 */
/*============================================================================*/


 void
FI_UnfixStack (

	register PAGESTACK	*pageStack,
	BOOL				freeStack 

)
{
	register STACKNODE	*stackNode;
	register int		i;
    

	TRPRINT(TR_FI, TR_LEVEL_1, ("stack top:%d", pageStack->stackTop));

	/*
	 *	signal the semaphore on the pages that need it
	 */
	if (pageStack->holdAllSemaphores) {

		for (i = 0; i <= pageStack->stackTop; i++)	{

			TRPRINT(TR_FI, TR_LEVEL_2, ("signalling:%d", i));
			signalSemaphore( &(pageStack->stackNode[i].groupLink->pageHash->semaphore) );
		}
	} else {

		for (i = MAX(0, pageStack->stackTop - 2); i <= pageStack->stackTop; i++)	{

			TRPRINT(TR_FI, TR_LEVEL_2, ("signalling:%d", i));
			signalSemaphore( &(pageStack->stackNode[i].groupLink->pageHash->semaphore) );
		}
	}
    /*
     *	Unfix all pages on the stack
     */
    while (!FI_StackEmpty(pageStack))  {

		/*
		 *	Get the next node off the stack
		 */
		stackNode = FI_PopStack(pageStack);

		/*
		 *	Unfix the page
		 */
		bf_UnfixPage(stackNode->groupLink, BF_DEFAULT, FALSE);
    }

	/*
	 *	if the flag is set then we must free the stack
	 */
	if (freeStack)	{

		/*
		 *	hang it off the free queue
		 */
		listEnq( &PageStackPool, &(pageStack->stackList) );
	}
}
