/*
 *   $RCSfile: fi_CheckPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "bf_extfuncs.h"


/*============================================================================*/
/* 
 * fi_CheckPage
 * 	Searches the B+ tree for the passed page
 *
 * IMPORTS:
 *	FI_Releasepages()
 *	FI_SearchNode()
 *	FI_StackInit()
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *
 * SIDE EFFECTS:
 *	None.
 */
/*============================================================================*/


 int
fi_CheckPage (

	BUFGROUP		*bufGroup,
	PID				*pid,
	SHORTPID		targetPid,
	BOOL			*present 
)
{

    PAGESTACK		*pageStack;		/* stack to store traverse nodes		*/
	GROUPLINK		*foundLink;		/* buffer that holds found node			*/
    SHORTPID		foundPid;		/* PID of found page					*/
    NODEPAGE		*foundPage;		/* buffer addr of found page			*/
    TREEDATA		*foundArray;	/* key array of found page				*/
    FOUR			foundLoc;		/* array subscript of found key			*/


	TRACE(TR_FI, TR_LEVEL_1);

    /*
	 *	Get stack to store traversed nodes
     */
	if ((pageStack = FI_AllocStack(FALSE)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}	

    /*
     *	Search for page with checkPage or page that would contain 
     *	checkPage, if it was in the file.
     *	Check for errors returned by search
     */
    if (FI_SearchNode(bufGroup, pid, targetPid, pageStack,
					  &foundPid, &foundLink, &foundLoc))	{

		FI_UnfixStack(pageStack, TRUE);
		return(esmFAILURE);
	}

    /*
     *	Unfix all pages on stack
     */
	FI_UnfixStack(pageStack, TRUE);

	/*
	 *	Get a pointer to the buffer
	 */
	foundPage = (NODEPAGE *) foundLink->bufFrame;
		
    /*
     *	Grab array of keys
     */
    foundArray = &(foundPage->contents);

    /*
     *	If the key at the found location in the array matches the
     *	checkPage, return TRUE, FALSE otherwise
     */
    if ((foundArray->keyPid.allKeys[foundLoc].key & LARGEMASK) == targetPid)	{

		*present = TRUE;
		TRPRINT(TR_FI, TR_LEVEL_2, ("page is present"));

    } else {

		*present = FALSE;
		TRPRINT(TR_FI, TR_LEVEL_2, ("page is not present"));
    }

	/*
	 *	Unfix Target page
	 */
	signalSemaphore(&(foundLink->pageHash->semaphore));
	bf_UnfixPage(foundLink, BF_DEFAULT, FALSE);

	/*
	 *	Return appropriate error code
	 */
    return(esmNOERROR);
}
