/*
 *   $RCSfile: fi_DoLinkConsist.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "link.h"
#include "volume.h"
#include "trans.h"
#include "consist.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "lm_extfuncs.h"
#include "lm_intfuncs.h"
#include "fi_globals.h"
#include "thread_globals.h"


 void
fi_DoLinkConsist (

	register PID			*pid,
	PAGEHASH				*pageHash,
	SLOTTEDPAGE				*slottedPage 
)
{

	register LOCKHEADER		*lockHeader;
	register LINKCONSISTREC	*linkConsistRec;


	TRPRINT(TR_FI, TR_LEVEL_1, ("pid:%d", pid->page));

	/*
	 *	check to see if the lock exists
	 */
	if ((lockHeader = lm_GetLockHeader(pid)) != NULL)	{

		TRPRINT(TR_FI, TR_LEVEL_2, ("lock header found"));

		/*
		 *	get a pointer to link consistency record, if the lock header
		 *	has a link consistency structure on its auxList.
		 */
		if (LIST_NOT_EMPTY(&(lockHeader->auxList)) && 
			(linkConsistRec = (LINKCONSISTREC *) lockHeader->auxList.succ->item))	{

			TRPRINT(TR_FI, TR_LEVEL_2, ("consistency record found"));

			/*
			 *	check the magic number
			 */
			CHECK_LINKCONSISTREC_MAGIC(linkConsistRec);

			/*
			 *	fill in the link fields
			 */
			slottedPage->header.nextLogicalPid = linkConsistRec->linkPid;
			TRPRINT(TR_FI, TR_LEVEL_2, ("next logical:%d", linkConsistRec->linkPid));

			/*
			 *	hang the record off the lock structure
			 */
			poolMove( &LinkConsistPool, &(linkConsistRec->lockHeaderList) );

			/*
			 *	dirty the page
			 */
			DIRTY_PAGE(pageHash);
		}
	}
}

