/*
 *   $RCSfile: fi_FirstPhysicalPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:41 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "bf_extfuncs.h"


/*============================================================================*/
/* 
 * fi_FirstPhysicalPage
 * 	returns the first page in the tree or subtree.
 *
 * IMPORTS:
 *	bf_ReadPage()
 *	FI_Leaf()
 *	FI_PushStack()
 *	FI_UnfixStack()
 *	FI_StackInit()
 *
 * RETURNS:
 *	esmNOERROR
 *	efiEMPTYFILE
 *
 * ERROR RETURNS:
 *	efiBADPARAMS	bad parameters
 *	Errors returned by bf_ReadPage()
 *
 * SIDE EFFECTS:
 *	Returns the first SHORTPID in the tree in 'firstPid'.
 *	if the tree is empty, -1 is returned
 *
 *	All pages are UNFIXED .
 */
/*============================================================================*/


 int
fi_FirstPhysicalPage (

	BUFGROUP	*bufGroup,
	PID			*rootPid,
	SHORTPID	*firstPid,
#ifdef notdef
	PAGESTACK	**retStack 
#endif notdef
	TREEDATA	*resultLeaf,
	int			*resultCount
)
{
    
    PAGESTACK	*pageStack;		/* stack to remember path to leaf		*/
	GROUPLINK	*groupLink;		/* pointer to link of buffer			*/
    NODEPAGE	*curPage;		/* addr of current page buffer being	*/
    TREEDATA	*curArray;		/* array of node/keyPtr pairs			*/
    PID			curPid;			/* current page Id being handled		*/
	int			retval;			/* the value to return to the user		*/
#ifdef DEBUG
	static int	nth_call = 0;

	nth_call ++;
#endif DEBUG


	TRACE(TR_FI, TR_LEVEL_1);

#ifdef notdef
	/*
	 *	check to see if we have to allocate path stack
	 */
	if ((retStack == NULL) || (*retStack == NULL))	{

#endif notdef
		/*
		 *	Get the path stack
		 */
		if ((pageStack = FI_AllocStack(FALSE)) == NULL)	{

			return(esmFAILURE);
		}

#ifdef notdef
	} else {

		/*
		 *	use the given page stack
		 */
		pageStack = *retStack;
	}
#endif notdef

	/*
	 *	Setup initial loop conditions
	 */
    curPid = *rootPid;

	/*
	 *	Traverse the subtree
	 */
    for (;;)	{

		/*
		 * Read in current page, and grab it's type, and array
		 */
		if ((groupLink = bf_ReadPage(bufGroup, &curPid,
									 FILE_PAGE2SIZE, BF_SEM)) == NULL)	{

			return(esmFAILURE);
		}

		/*
		 *	Pull out page pointer
		 */
		curPage = (NODEPAGE *) groupLink->bufFrame;

		/*
		 *	check the magic number
		 */
		if (curPage->header.magic != FILEPAGE_MAGIC)	{

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *	pull out type and contents
		 */
		curArray = &(curPage->contents);

		/*
		 * If reached leaf, get the first key on that page
		 */
		if (FI_Leaf(curPage->header.nodeType))  {

			/*
			 * if node count is 0 return efiEMPTYFILE
			 * else return first key
			 */
			if (curPage->header.nodeCount == 0)  {

				/*
				 *	Set the return value
				 */
				retval = esmEMPTYFILE;
				SM_ERROR(TYPE_USER, esmEMPTYFILE);
				TRPRINT(TR_FI, TR_LEVEL_2, ("empty file"));

			} else {

				/*
				 *	Pull out the leftmost key
				 */
				*firstPid = (curArray->keyPid.allKeys[0].key & LARGEMASK);
				fi_PrintNode(curPage);
				TRPRINT(TR_FI, TR_LEVEL_2, ("first pid:%d", *firstPid));
				retval = esmNOERROR;
				if( resultLeaf != NULL ) {
					SM_ASSERT(LEVEL_1, resultCount != NULL);
					/* 
					 * Copy the rest of the leaf file page to the result leaf 
					 * and tell the caller how many keys we are copying.
					 * NB: The keys are in REVERSE ORDER.  The client
					 * must know that!
					 */

					*resultCount = (int) MIN(*resultCount, curPage->header.nodeCount);
					SM_ASSERT(LEVEL_3, sizeof(KEYS) == sizeof(SHORTPID));
					BCOPY( &(curArray->keyPid.allKeys[0]), resultLeaf,
								(sizeof(KEYS) * (*resultCount)) );
				} else {
					SM_ASSERT(LEVEL_1, resultCount == NULL);
				}
			}

#ifdef notdef
			/*
			 *	check to see if we should save the stack
			 */
			if (retStack == NULL)	{
#endif notdef

				TRPRINT(TR_FI, TR_LEVEL_2, ("unfixing stack"));

				/*
				 *	Unfix the current page
				 */
				signalSemaphore(&(groupLink->pageHash->semaphore));
				bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);

				/*
				 *	Release pages on the stack
				 */
				FI_UnfixStack(pageStack, TRUE);
			
#ifdef notdef
			} else {

				TRPRINT(TR_FI, TR_LEVEL_2, ("saving stack"));

				/* 
				 * push leaf page on path stack
				 */
				FI_PushStack(pageStack, curPid.page, curPage, groupLink, 0, NOFLAGS);

				/*
				 *	return the stack
				 */
				*retStack = pageStack;
			}
#endif notdef

			/*
			 *	return the status of search
			 */
			if (retval)	{

				return(esmFAILURE);

			} else {

				return(retval);
			}

		} else {

			/* 
			 * Otherwise, push current page on path stack
			 */
			FI_PushStack(pageStack, curPid.page, curPage, groupLink, 0, NOFLAGS);

			/*
			 *	go on to the leftmost page
			 */
			curPid.page = curArray->keyPtr[0].pagePtr;
			TRPRINT(TR_FI, TR_LEVEL_2, ("going to page:%d", curPid.page));
		}
    }
}
