/*
 *   $RCSfile: fi_NumPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"

#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "consist.h"
#include "link.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "sm_extfuncs.h"


/*============================================================================*/
/* 
 * fi_NumPages()
 * 	Returns the number of pages in the file
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	efiBADPARAMS
 *	Errors returned by bf_ReadPage()
 *
 * SIDE EFFECTS:
 *	Calculate the number of pages in the file
 */
/*============================================================================*/

static Descend(BUFGROUP* bufGroup, PID* rootPid, int* numPages);

int
fi_NumPages(BUFGROUP* bufGroup, PID* rootPid, int* numPages)
{
	*numPages = 0;
	return (Descend(bufGroup, rootPid, numPages));
}


static 
Descend (

	BUFGROUP		*bufGroup,
	PID				*rootPid,
	int*			numPages
)
{
   
    NODEPAGE		*rootPage;
    TREEDATA		*rootArray;
	GROUPLINK		*groupLink;
	PID				childPid;
	PID				currentPid;
    int				rootCount;
    int				i;

    /*
     *	Grab volume Id 
     */
    currentPid.volid = childPid.volid = rootPid->volid;

    /*
     *	Read in root of subtree in.  No lock is needed since
	 *	the root has been locked
     */
	if ((groupLink = bf_ReadPage(bufGroup, rootPid, FILE_PAGE2SIZE, NOFLAGS)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	get a pointer to the root page and check the magic number
	 */
	rootPage = (NODEPAGE *) groupLink->bufFrame;

	/*
	 *	check to see if the magic number is set
	 */
	if (rootPage->header.magic != FILEPAGE_MAGIC)	{

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

    /*
     *	Grab array, node count, node Type
     */
    rootArray = &(rootPage->contents);
    rootCount = (int) rootPage->header.nodeCount;

    /*
     *	Check to see if the page is a leaf
     */
    if (FI_NotLeaf(rootPage->header.nodeType))  {

		/*
		 *	We must check all subtrees rooted here
		 */
		for (i = 0; i < rootCount; i++)  {

			/*
			 *	get the pid of the child
			 */
			childPid.page = rootArray->keyPtr[i].pagePtr;
				
			/*
			 *	traverse the subtree rooted at the child
			 */
			if (Descend(bufGroup, &childPid, numPages))	{
				
				return(esmFAILURE);
			}
		}

    } else {
		*numPages += rootCount;
    }
	
	bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);

	/*
	 *	return success to the caller
	 */
    return(esmNOERROR);
 }



