/*
 *   $RCSfile: fi_RecordFileForDestruction.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"

#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "consist.h"
#include "link.h"
#include "volume.h"
#include "trans.h"
#include "deallocinfo.h"
#include "fi_extfuncs.h"
#include "io_extfuncs.h"
#include "thread_globals.h"
#include "io_globals.h"


/*
 *	Record the fact that this file should be destroyed at the
 *	end of the transaction.
 */

 int
fi_RecordFileForDestruction (

	FID				*fid 
)
{
   
	TRANSREC		*transRec;
	FILEDEALLOCINFO	*fileDeallocInfo;
	VOLREC			*volRec;


	TRPRINT(TR_FI, TR_LEVEL_1, ("rootPid:%d", fid->pid.page));

	transRec = (TRANSREC*) Active->transRec;

    /*
     *  Mark the root page of the file as no longer a
     *  file page.
     */
    volRec = io_FindVolRec(fid->pid.volid);
    if (volRec == NULL) {
        /*
         *  this cannot happen since io_CheckBitmap already called
         *  io_FindVolumeId() successfully
         */
        SM_ERROR(TYPE_FATAL, esmINTERNAL);
    }

    if (io_MarkPage(volRec, &(fid->pid), PAGE_FILEHDR, FALSE) != esmNOERROR)   {
        return(esmFAILURE);
    }

	/*
	 *	If the volume is not temporary, then add the file to 
	 *	a list of those to deallocate at commit.  If the volume
	 *	is temporary, this will be done automatically.
	 */
	if ( !(volRec->volflags & VOL_TEMP) )  {
		/*
		 *  Obtain a place to record that the file is to be deallocated
		 *  at transaction commit.
		 */
		fileDeallocInfo = (FILEDEALLOCINFO*) poolDeq(&FileDeallocInfoPool);
		if (fileDeallocInfo == NULL) {
			SM_ERROR(TYPE_CRASH, esmINTERNAL);
		}

		/*
		 *  Remember the file info and attach it to the transaction
		 */
		fileDeallocInfo->fid = *fid;
		listEnq(&(transRec->fileDeallocList), &(fileDeallocInfo->list));
	} else {
		/* this should not be called for files on temporary volumes */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

    return(esmNOERROR);
 }
