/*
 *   $RCSfile: fi_SetLinkConsist.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "fi_macro.h"
#include "link.h"
#include "volume.h"
#include "trans.h"
#include "consist.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "lm_extfuncs.h"
#include "lm_intfuncs.h"
#include "fi_globals.h"


 int
fi_SetLinkConsist (

	FLAGS					consistType,
	register PID			*pid,
	SHORTPID				linkPid 
)
{

	register LOCKHEADER		*lockHeader;
	register LINKCONSISTREC	*linkConsistRec;


	TRPRINT(TR_FI, TR_LEVEL_1, ("volid:%d page:%d linkPid:%d",
			pid->volid, pid->page, linkPid));


	/*
	 *	check to see if the lock exists
	 */
	if ((lockHeader = lm_GetLockHeader(pid)) != NULL)	{

		TRPRINT(TR_FI|TR_LOCK, TR_LEVEL_2, ("lock header found"));

		/*
		 *	attempt to get a link consistency record
		 */
		if ((linkConsistRec = (LINKCONSISTREC *) poolDeq( &LinkConsistPool )) == NULL)	{

			SM_ERROR(TYPE_FATAL, esmFAILURE);
		}

		/*
		 *	fill in the link fields
		 */
		linkConsistRec->linkPid = linkPid;
		linkConsistRec->flags   = consistType;

		/*
		 *	hang the record off the lock structure
		 */
		listEnq( &(lockHeader->auxList), &(linkConsistRec->lockHeaderList) );
	}

	/*
	 *	return success
	 */
	return(esmNOERROR);
}

