/*
 *   $RCSfile: fi_ValidateFid.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "error.h"
#include "trace.h"
#include "io.h"
#include "page.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "consist.h"
#include "link.h"
#include "volume.h"
#include "fi_intfuncs.h"
#include "fi_extfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"


 int
fi_ValidateFid (

	BUFGROUP			*bufGroup,
	FID					*fid
)
{

    NODEPAGE			*rootPage;
	GROUPLINK			*rootLink;	
	BOOL				bitSet;

	TRACE(TR_FI, TR_LEVEL_1);

	/*
	 *	check the bitmap
	 */
	if (io_CheckBitmap( &(fid->pid), PAGE_FILEHDR, &bitSet ))	{
		return(esmFAILURE);
	}
    if (!bitSet)    {
		SM_ERROR(TYPE_USER, esmBADFID);
		return(esmFAILURE);
	}

	/*
	 *	Read the target page from disk
	 */
	if ((rootLink = bf_ReadPage(bufGroup, &(fid->pid),
								FILE_PAGE2SIZE, NOFLAGS)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	get a pointer to the page
	 */
	rootPage = (NODEPAGE *) rootLink->bufFrame;

	/*
	 *	check the magic number
	 */
	if (rootPage->header.magic != FILEPAGE_MAGIC)	{

		bf_UnfixPage(rootLink, BF_DEFAULT, FALSE);
		SM_ERROR(TYPE_FATAL, esmBADFILEPAGEMAGIC);
		return(esmFAILURE);
	}

	/*
	 *	check the file unique
	 */
	if (rootPage->header.unique != fid->unique)	{

		bf_UnfixPage(rootLink, BF_DEFAULT, FALSE);
		SM_ERROR(TYPE_USER, esmBADFID);
		return(esmFAILURE);
	}

	bf_UnfixPage(rootLink, BF_DEFAULT, FALSE);
	return(esmNOERROR);
}
