/*
 *   $RCSfile: bf.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
#ifndef __BF_H__
#define __BF_H__
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

/*
 * bf.h
 *	Global defines, typedefs, etc. for the buffer manager.
 */

/*
 *	Any module using this file should have SERVER_MAKE defined
 *	in the makefile.
 */
#ifndef SERVER_MAKE
generate an error message from the compiler
#endif

#include "ess.h"
#include "checking.h"
#include "forward.h"
#include "io.h"
#include "latch.h"
#include "list.h"
#include "semaphore.h"

/* max length of sequential io request */
#define MAX_IO_LIST 8

#include "PAGEHASH.h"

#define PAGE_IS_DIRTY(_pageHash)	\
	((_pageHash)->flags & H_DIRTY)

#define PAGE_IS_CLEAN(_pageHash)	\
	(!((_pageHash)->flags & H_DIRTY))

#	define DIRTY_PAGE(_pageHash)	\
		(_pageHash)->flags |= H_DIRTY

#	define CLEAN_PAGE(_pageHash)	\
		bf_CleanPage(_pageHash)

#define PAGE_IS_USED(_pageHash)	\
	((_pageHash)->memberCount > 0)

#define PAGE_IS_FREE(_pageHash)	\
	((_pageHash)->memberCount == 0)

#define PAGE_IS_LOG(_pageHash)	\
	(PPAGE_TYPE(_pageHash) == PAGE_LOG)


/*
 * Definition for the BufGroup table.
 * Buffer groups are used to implement a DBMIN-like replacement strategy.
 * There are two lists associated with a buffer group, the group's
 * fixed list and the groups unfixed list.
 * 
 * No fixed list is kept.
 * The group's unfixed list is organized as a true list so that
 * different replacement policies can be used.
 * Note that a buffer region can simultaneously reside in several
 * fixed lists but never in more than one unfixed list.
 */
typedef struct {

    UONE		flags;					/* state flags of group					*/
    UONE		replacePolicy;			/* replacement policy for unfixed pages	*/
    int			maxCharges;				/* max charges allotted to the group	*/
    int			fixedCharges;			/* current fixed charges				*/
    int			unfixedCharges;			/* current unfixed charges				*/
	LIST		fixedList;				/* queue of fixed elements				*/
	LIST		unfixedList;			/* queue of unfixed elements			*/
	LISTELEMENT	groupList;				/* used for free/used lists				*/	
	MAGIC	magic;					/* magic number for structure verify	*/

} BUFGROUP;


#define BUFGROUP_MAGIC	0xef401cad


/*
 *	Definition of flags for buffer groups
 */
#define FREE_GROUP	0x0
#define OPEN_GROUP	0x1
#define LOG_GROUP	0x2
#define FORCE_GROUP	0x4


/*
 *	Definition of the entry to record 
 */
typedef struct {

	LISTELEMENT	hashList;		/* hung off list of structures		*/
	LISTELEMENT	groupList;		/* member of buffer group			*/
	PAGEHASH	*pageHash;		/* pointer to hash structure		*/
	BUFGROUP	*bufGroup;		/* pointer to the buffer group		*/
	char		*bufFrame;		/* pointer to the actual data		*/
	UTWO		fixedCount;		/* number of times fixed in group	*/
	MAGIC		magic;			/* for verification					*/

} GROUPLINK;

/*
 *	define magic number
 */
#define GROUPLINK_MAGIC	0xbe4a20c8


/*
 *	Return values for the buffer groups
 */
#define BF_NOT_PRESENT	0
#define BF_FREE			1
#define BF_OTHER		2
#define BF_FIXED		3
#define BF_UNFIXED		4


/* 
 * Buffer replacement policies for buffer groups.
 */
#define BF_DEFAULT		0
#define BF_LRU			1
#define BF_MRU			2


/*
 *	define some flags for use in the bf layer
 */
#define BF_SEM			0x1
#define BF_NOREAD		0x2
#define BF_FORCE		0x4
#define BF_WAIT			0x8


/*
 * Definition for the BufTable.
 * The BufTable is an array that describes how each buffer frame is being used.
 * If a frame is marked as valid and contains chunk data, then the 
 * chunkDesc field will be non-NULL and will point to the chunk's descriptor.
 * If a frame is marked as valid and contains a slotted page, then the 
 * hashPtr field will be non-NULL and will point to the page's hash entry.
 *
 * When a frame is in the pool of free frames (i.e., it is part of the
 * free list), the the frame is tagged as free, and the fields size, 
 * nextBuf, prevBuf, and firstInBlk are set accordingly to link the 
 * frame into the free list.
 *
 * Note that in a free block, the fields free and firstInBlk are set for
 * each buffer of the free block, whereas, the fields size, freeBlk, 
 * and numClean are set in just the first buffer of the free block.
 */
typedef struct BufInfo {       

	int			index;
    PAGEHASH	*pageHash;	

}; 

/*
 * For inter-tx page caching: keep a hash table of PID,LRC pairs.
 * Add to this table when swapping a page out.
 */
class LRCCACHEENTRY {
public:
	FID			fid;
	LRC 		lrc;
	FLAGS		flags;
	PAGEHASHQ 	hashList;
	BOOL 		Dump(FILE *f);
	PID 		*keylocation (int unique, PID **unused);
	LISTELEMENT *listlocation (int unique);
	void 		ReInit();
	void 		Init();
};

#endif __BF_H__
