/*
 *   $RCSfile: bf_macro.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
#ifndef __BF_MACRO_H__
#define __BF_MACRO_H__
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
/*
 * bf_macro.h
 * Macros for the buffer manager.
 * 
 * These macros are for doing in-line expansion of
 * routines need to execute quickly.
 * They are only expanded if the #ifdef DEBUG flag is set;
 * otherwise, the C-routine version of the macro is used instead.
 *
 * Each macro was built up by initially writing a C-routine 
 * version of the macro.
 * Once the C-routine version was debugged, the code in the routine
 * was essentially ripped out and put into a macro.
 * This is the recommended procedure for adding new macros.
 *
 * Some of these macros are long and it's questionable whether
 * they should actually be macros, but the ones that are long 
 * are called in only one or two places so the macro expansion
 * isn't quite as bad as it appears on the surface.
 */


/*
 *	define out dependencies for now
 */
#define bf_AddPageDependency(a,b,c)


/*
 *	define magic number checks
 */
#if MAGIC_CHECKING IS_ENABLED


#define CHECK_BUFGROUP_MAGIC(_bufGroup)							\
																\
	if ((_bufGroup)->magic != BUFGROUP_MAGIC)	{				\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);						\
	}

#define CHECK_PAGEHASH_MAGIC(_pageHash)							\
																\
	if ((_pageHash)->magic != PAGEHASH_MAGIC)	{				\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);						\
	}

#define CHECK_GROUPLINK_MAGIC(_groupLink)						\
																\
	if ((_groupLink)->magic != GROUPLINK_MAGIC)	{				\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);						\
	}

#define CHECK_BUFGROUP_VALID(_bufGroup)							\
																\
	(((_bufGroup)->magic != BUFGROUP_MAGIC) ||					\
	 (!((_bufGroup)->flags & OPEN_GROUP)))


#define INIT_BUFGROUP_MAGIC(_bufGroup)							\
																\
	(_bufGroup)->magic = BUFGROUP_MAGIC;


#define INIT_PAGEHASH_MAGIC(_pageHash)							\
																\
	(_pageHash)->magic = PAGEHASH_MAGIC;


#define INIT_GROUPLINK_MAGIC(_groupLink)						\
																\
	(_groupLink)->magic = GROUPLINK_MAGIC;


#else


/*
 *	macros to define out magic checks
 */
#define CHECK_BUFGROUP_MAGIC(_bufGroup)

#define CHECK_PAGEHASH_MAGIC(_pageHash)

#define CHECK_GROUPLINK_MAGIC(_groupLink)

#define CHECK_BUFGROUP_VALID(_bufGroup)							\
																\
	 (!((_bufGroup)->flags & OPEN_GROUP))

#define INIT_BUFGROUP_MAGIC(_bufGroup)

#define INIT_PAGEHASH_MAGIC(_pageHash)

#define INIT_GROUPLINK_MAGIC(_groupLink)


#endif




#ifndef DEBUG


#define BF_ReplacePolicy(_bufGroup, _list, _replacePolicy)		\
																\
{																\
																\
	TRACE(TR_BF, TR_LEVEL_1);												\
																\
	/*															\
	 *	Check to see which policy is specified					\
	 */															\
	if (_replacePolicy == BF_DEFAULT)	{						\
																\
		/*														\
		 *	Set to the default for the buffer group				\
		 */														\
		_replacePolicy = _bufGroup->replacePolicy;				\
	}															\
																\
	/*															\
	 *	take action based on the policy							\
	 */															\
	switch (_replacePolicy)	{									\
																\
		case BF_LRU:											\
																\
			/*													\
			 *	Put on the end of the list						\
			 */													\
			listEnq( &(_bufGroup->unfixedList), _list );		\
			break;												\
																\
		case BF_MRU:											\
																\
			/*													\
			 *	Put on the end of the list						\
			 */													\
			listPush( &(_bufGroup->unfixedList), _list );		\
			break;												\
																\
		default:												\
																\
			/*													\
			 *	This should not happen							\
			 */													\
			SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
			break;												\
	}															\
}

#endif




#define BF_AuditFreeList()

#endif __BF_MACRO_H__
