/*
 *   $RCSfile: deallocinfo.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:42 $      
 */ 
#ifndef __DEALLOCINFO_H__
#define __DEALLOCINFO_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
/*
 *	Structures to support the delayed (until EOT) deallocation of
 *	files and pages.
 */

typedef struct {
	FID			fid;
	LISTELEMENT	list;
	MAGIC		magic;
} FILEDEALLOCINFO;


typedef struct {
	PID			pid;
	TWO			pageCount;	/* each page deallocation request results
							   in a number of pages being deallocated.
							   The first page in the request will have
							   pageCount set to the number of pages.  
							   All other pages in the request will have
							   it set to zero. */
	PAGE2SIZE	page2size;
	LISTELEMENT	list;
	MAGIC		magic;
} PAGEDEALLOCINFO;


#define FILEDEALLOCINFO_MAGIC 0xb4a987ae
#define PAGEDEALLOCINFO_MAGIC 0xc5ba98bf

#if MAGIC_CHECKING IS_ENABLED

#define INIT_FILEDEALLOCINFO_MAGIC(_info)	\
	(_info)->magic = FILEDEALLOCINFO_MAGIC;

#define INIT_PAGEDEALLOCINFO_MAGIC(_info)	\
	(_info)->magic = PAGEDEALLOCINFO_MAGIC;

#define CHECK_FILEDEALLOCINFO_MAGIC(_info)	\
	if ((_info)->magic != FILEDEALLOCINFO_MAGIC) {	\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);			\
	}

#define CHECK_PAGEDEALLOCINFO_MAGIC(_info)	\
	if ((_info)->magic != PAGEDEALLOCINFO_MAGIC) {	\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);			\
	}

#else

#define INIT_FILEDEALLOCINFO_MAGIC(_info)

#define INIT_PAGEDEALLOCINFO_MAGIC(_info)

#define CHECK_FILEDEALLOCINFO_MAGIC(_info)	

#define CHECK_PAGEDEALLOCINFO_MAGIC(_info)

#endif
#endif __DEALLOCINFO_H__
