/*
 *   $RCSfile: fi_macro.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:44 $      
 */ 
#ifndef __FI_MACRO_H__
#define __FI_MACRO_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


/*
 * fi_macro.h
 * Macros for the file routines.
 * 
 * These macros are for doing in-line expansion of
 * routines need to execute quickly.
 * They are only expanded if the #ifdef DEBUG flag is set;
 * otherwise, the C-routine version of the macro is used instead.
 *
 * Each macro was built up by initially writing a C-routine 
 * version of the macro.
 * Once the C-routine version was debugged, the code in the routine
 * was essentially ripped out and put into a macro.
 * This is the recommended procedure for adding new macros.
 *
 * 
 * EXPORTS:
 *	FI_Root (nodeType)
 *	FI_Leaf (nodeType)
 *	FI_Empty (topOfStack)
 *
 */

/*
 * Macro to check if node type is ROOT
 */
#define FI_Root(_NodeType)						\
		 (_NodeType & FI_ROOTPAGE)

/*
 * Macro to check if node type is not ROOT
 */
#define FI_NotRoot(_NodeType)					\
		 (!(_NodeType & FI_ROOTPAGE))

/*
 * Macro to check if node type is LEAF
 */
#define FI_Leaf(_NodeType)						\
		(_NodeType & FI_LEAFPAGE)

/*
 * Macro to check if node type is not LEAF
 */
#define FI_NotLeaf(_NodeType)						\
		(!(_NodeType & FI_LEAFPAGE))


/*
 * Macro to check if stack is EMPTY
 */
#define FI_StackEmpty(_Stack)						\
		( (_Stack->stackTop < 0) )

/*
 * Macro to check if stack is not EMPTY
 */
#define FI_StackNotEmpty(_Stack)						\
		( (_Stack->stackTop >= 0) )


/*
 *	turn off debugging
 */
#ifndef DEBUG

#define fi_PrintNode(node)

#endif
#endif __FI_MACRO_H__
