/*
 *   $RCSfile: latch.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:45 $      
 */ 
#ifndef __LATCH_H__
#define __LATCH_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
/*
 *	define the type for the short term latches
 */
#define LATCH_MAGIC		0xdf451068


#define SHARE_LATCH		0x1
#define EXCLUSIVE_LATCH	0x2


typedef struct {

	LIST		waitList;
	short		exclusiveFlag;
	short		shareCount;
	MAGIC		magic;
} LATCH;



#if MAGIC_CHECKING IS_ENABLED


#define CHECK_LATCH_MAGIC(_latch)							\
															\
	if (_latch->magic != LATCH_MAGIC)	{					\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}

#define INIT_LATCH_MAGIC(_latch)							\
															\
	_latch->magic = LATCH_MAGIC;


#else


#define CHECK_LATCH_MAGIC(_latch)

#define INIT_LATCH_MAGIC(_latch)


#endif


/*
 *	define the function prototypes
 */
#ifdef __cplusplus


extern "C" void		initializeLatch(LATCH *);
extern "C" int		waitLatch(LATCH *, int);
extern "C" void		signalLatch(LATCH *);


#else


extern void		initializeLatch(LATCH *);
extern int		waitLatch(LATCH *, int);
extern void		signalLatch(LATCH *);


#endif
#endif __LATCH_H__
