/*
 *   $RCSfile: resource.h,v $  
 *   $Revision: 1.2 $  
 *   $Date: 1996/05/05 03:09:42 $      
 */ 
#ifndef __RESOURCE_H__
#define __RESOURCE_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
/*
 *	define the types of resources that can be controlled by a link
 */
typedef enum { SEQUENTIALLOAD, SEQUENTIALSCAN } RESOURCETYPE;


/*
 *	define a structure that controls the resource
 */
typedef struct	{
	
	RESOURCEID		id;
	UNIQUE			unique;			/* unique identifier of the resource	*/
	RESOURCETYPE	type;			/* type of the resource					*/
	struct Link_s		*link;			/* pointer to the controlling link		*/
	char			*pointer;		/* pointer to the actual resource		*/
	FLAGS			flags;
	LISTELEMENT		linkList;
	SEMAPHORE		semaphore;
	MAGIC			magic;
} RESOURCE;


/*
 *	define the states of the resource
 */
#define RESOURCE_FREE		0x0
#define RESOURCE_ACTIVE		0x1


/*
 *	define ttt magic number
 */
#define RESOURCE_MAGIC	0x9fa34208


#if MAGIC_CHECKING IS_ENABLED


#define CHECK_RESOURCE_MAGIC(_resource)							\
																\
	if ((_resource)->magic != RESOURCE_MAGIC)	{				\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);						\
	}

#define INIT_RESOURCE_MAGIC(_resource)							\
																\
	(_resource)->magic = RESOURCE_MAGIC;


#else


#define CHECK_RESOURCE_MAGIC(_resource)

#define INIT_RESOURCE_MAGIC(_resource)	


#endif
#endif __RESOURCE_H__
