/*
 *   $RCSfile: semaphore.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:45 $      
 */ 
#ifndef __SEMAPHORE_H__
#define __SEMAPHORE_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "ess.h"
#include "checking.h"
#include "thread.h"
#include "list.h"

/*
 *	define the structure for a semaphore
 */

#define SEMAPHORE_MAGIC		0x452f67c9


typedef struct {

	LIST	waitList;
	TCB		*holder;
	MAGIC	magic;

} SEMAPHORE;


/*
 *	define a macro to check if there are semaphore waiters
 */
#define SEMAPHORE_WAITERS(_semaphore)				\
													\
	(LIST_NOT_EMPTY( &((_semaphore)->waitList) ))


/*
 *	define a macro to check if the semaphore is held
 */
#define SEMAPHORE_IS_HELD(_semaphore)				\
													\
	(((_semaphore)->holder) != NULL)


/*
 *	define a macro to check if the semaphore is held
 */
#define SEMAPHORE_IS_FREE(_semaphore)				\
													\
	( ((_semaphore)->holder == NULL) )




#if MAGIC_CHECKING IS_ENABLED


#define CHECK_SEMAPHORE_MAGIC(_semaphore)					\
															\
	if (_semaphore->magic != SEMAPHORE_MAGIC)	{			\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}

#define INIT_SEMAPHORE_MAGIC(_semaphore)					\
															\
	_semaphore->magic = SEMAPHORE_MAGIC;

#else


#define CHECK_SEMAPHORE_MAGIC(_semaphore)

#define INIT_SEMAPHORE_MAGIC(_semaphore)


#endif


/*
 *	define the function prototypes for semaphores
 */
#ifdef __cplusplus
extern "C" {
#endif

extern void		initializeSemaphore(SEMAPHORE *, BOOL);
extern int		waitSemaphore(SEMAPHORE *);
extern void		signalSemaphore(SEMAPHORE *);
extern int		waitList(LIST *, int);

#ifdef __cplusplus
}
#endif

#endif __SEMAPHORE_H__
