/*
 *   $RCSfile: trans.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
#ifndef __TRANS_H__
#define __TRANS_H__

/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
/*
 *	define states of the log
 */
#define L_OFFLINE		0x1
#define L_ACTIVE		0x2
#define L_QUIESCE		0x4
#define L_RECOVER		0x8


/*
 *	define a record that hold all global transaction info
 */
typedef struct {

	FLAGS		transState;
	FLAGS		logState;
	TID			tidSequence;

} TRANSINFO;

#include "trans_states.h"

struct _MissingUpdateInfo;

/*
 *	define the transaction control block
 */
typedef struct	{

	/*
	 *	for distr trans
	 */

	GTIDREC		*gtidRec;			/* global transaction id record */
	LIST		serverList;			/* list of servers and local TIDs */
	LISTELEMENT	distrTransList;		/* link onto list of distr trans */
	int			numServers;			/* number of servers in this distr trans*/
	int			numVotesRecd;		/* number of votes received */
	int			numAcksRecd;		/* number of acks received */
	UONE 		clientCommand;		/* command recd from client */
									/*COMMAND*/
	UONE		coordCommand;		/* command recd from coord */
									/*COMMAND*/
	int			numLocks;			/* number of locks held */
	int			numVols;			/* number of volumes on volumeList*/
	LSN			prepareLSN;			/* LSN after end of active phase*/
	SOCKADDR	coordAddr;			/* used after recovery only */
	TID			coordTid;			/* used after recovery only */
	BOOL		prepareAndCommit;	/* for the optimized case */
	LIST		pendingBitmapList;	/* list of pending deallocations */

	TID			tid;
	LISTELEMENT	tidList;			/* list used to search by TID	*/
	LISTELEMENT	activeTransList;	/* list of active transactions	*/
	LIST		threadList;			/* list of threads active for TID*/
	LIST		linkList;			/* list of links active for TID	*/
	LISTELEMENT	logActiveList;		/* link onto log for operations	*/
	LIST		quiesceList;		/* list for holding quiesce master*/
	LIST		volumeList;			/* list of volume used in trans	*/
	LINK		*masterLink;		/* controlling link for TID		*/
	TCB			*masterThread;		/* master thread for commit/abort*/
	ONE			threadCount;		/* number of threads for trans	*/
	ONE			transState;			/* state of the transaction		*/
	ONE			intent;				/* commit/abort intent			*/
	ONE			privilege;			/* priviledge of this transaction*/
	LSN			firstLSN;			/* lsn of first log record for tid*/
	LSNOFFSET	lastLSN;			/* lsn of last log record for tid*/
	LSNOFFSET	nextUndoLSN;		/* lsn of next undo record		*/
	LSNOFFSET	rememberedLSN;		/* used for fancy CLR processing*/
	int			logRecordCount;		/* count of log records for tid	*/
	int			logSpace;			/* log space used by this trans	*/
	int			undoLogSpace;		/* log space used by this trans	in undo	*/
	SHORTPID	firstLogPid;		/* pid of first log record		*/
	UFOUR		logUnique;
	BOOL        loggingOn;          /* indicates if client has turned
                                       off logging for this trans  	*/
	LIST		fileDeallocList;	/* list of files to deallocate		*/
	LIST		pageDeallocList;	/* list of pages to deallocate		*/
	LIST		tempFileList;		/* temporary files created 		*/
	LIST		tempPageList;		/* tempoarary pages allocated 	*/
	LIST		lockGrantedList;
	LIST		lockWaitList;
	LIST		lockUpgradeList;
	LIST		lockClassList[NUM_LOCK_CLASSES];
	CLASSMAP	lockClassMap;
	UFOUR		lockWeight;
	int			visitCount;
	struct _MissingUpdateInfo *missingUpdateInfo; /* table for undo processing */
	LIST		clientDirtyList; 	/* dirty pages shipped from client*/
	SEMAPHORE	logSemaphore;		/* for threads writing log for this trans */
	int			lock_timeout;		/* in seconds, >= 0 */
	MAGIC		magic;

} TRANSREC;


/*
 *	define the transrec structure magic number
 */
#define TRANSREC_MAGIC	0x2ad505f0


/*
 *	define the transaction control block used in recovery
 */
typedef struct	{

	TID			tid;
	LISTELEMENT	tidList;			/* list used to search by TID		*/
	ONE			transState;			/* state of the transaction			*/
	ONE			intent;				/* commit/abort intent				*/
	ONE			privilege;			/* priviledge of this transaction	*/
	LSNOFFSET	firstLSN;			/* lsn of first log record for tid	*/
	LSNOFFSET	lastLSN;			/* lsn of last log record for tid	*/
	LSNOFFSET	nextUndoLSN;		/* the next undo lsn				*/
	MAGIC		magic;
} UNDOTRANSREC;


/*
 *	define the magic number
 */
#define UNDOTRANSREC_MAGIC	0x4bc539a1


/*
 *	define a list structure for searching for the transaction id
 */
typedef struct	{

	LISTELEMENT	list;
	TRANSREC	*transRec;

} TRANSLIST;


/*
 *	define the structure that tracks the dirty pages
 */

class DIRTYPAGEINFO {
public:
	PID			pid;
	LSN			lsn;
	LRC			lrc;
	LISTELEMENT	hashList;
	TID			tid;	/* tid which added page during analysis: 
						   (tid == NULL_TID) ==> page already in table*/
	FLAGS		flags;

	BOOL Dump(FILE *f);
	PID *keylocation (int unique, PID **unused);
	LISTELEMENT *listlocation (int unique);
	void ReInit();
	void Init();
};

/*
 *	Flags for dirty page table entries information
 *	DPT_DEALLOC: page was deallocated
 *	DPT_FIXFIRSTLRC: page was placed in DPT by log record generated
 *					 by a client.
 */
#define DPT_DEALLOC			0x1
#define DPT_FIXFIRSTLRC		0x2

/*
 *	define the structure that tracks the dirty pages
 */
typedef struct	{

	PID			pid;
	LSN			firstLSN;
	LRC			firstLRC;

} DIRTYPAGELOGINFO;

/*
 *	define the lockid so that it can accomodate all necessary lock types
 */
typedef union	{

	struct	{

		UFOUR	word1;
		UFOUR	word2;
		UFOUR	word3;
	
	} lockid;


	struct	{

		ONE		type;
		PID		pid;

	} page;


	struct	{

		ONE		type;
		ONE		slot;
		PID		pid;

	} rootSlot;


	struct	{

		ONE		type;
		PID		fid;

	} file;

} LOCKID;




/*
 *	typedef for linked list with LOCKID as the key
 */
typedef struct	{

	LISTELEMENT	list;
	LOCKID		lockid;

} LOCKIDLIST;


/*
 *	typedef for linked list with  as the key
 */
typedef struct	{

	LISTELEMENT	list;
	LOCKCLASS	lockClass;

} LOCKCLASSLIST;


/*
 *	structure that defines an instance of a lock
 */
typedef struct	{

	LOCKIDLIST	hashList;				/* hangs off the hash list				*/
	LOCKMODE	supremum;				/* current supremum of granted group	*/
	UONE		modeCount[NUM_LOCK_MODES];	/* number of locks in given mode	*/
	UTWO		lockCount;				/* number of locks granted				*/
	LIST		grantedList;			/* list of granted locks				*/
	LIST		waitList;				/* list of waiters for locks			*/
	LIST		upgradeList;			/* list of upgrade waiters				*/
	LIST		auxList;				/* list of auxillary structures			*/

	MAGIC		magic;

} LOCKHEADER;


/*
 *	define the lockheader structure magic number
 */
#define LOCKHEADER_MAGIC	0x45cd7509


/*
 *	structure defines an instance of a lock request
 */
typedef struct	{

	TRANSLIST	headerList;			/* hangs off the lock header		*/
	LISTELEMENT	transList;			/* hangs off the transaction record	*/
	LIST		threadList;			/* list of threads waiting for lock	*/
	LOCKHEADER	*lockHeader;		/* pointer to lock header record	*/
	LOCKMODE	lockMode;
	UONE		flags;
	TWO			time2wakeup;		/* age left on the lock (counts down) */
	UFOUR		weight;
	LISTELEMENT	ageList;			/* hangs off a list for the aging thread */
	MAGIC		magic;
} LOCKENTRY;


/*
 *	define the lock entry structure magic number
 */
#define LOCKENTRY_MAGIC	0xcfeba54d


/*
 *	define the flags for the lock entries
 */
#define LOCK_GRANTED			0x1
#define LOCK_WAITER				0x2
#define LOCK_UPGRADE			0x4
#define LOCK_UPGRADE_PENDING	0x8


/*
 *	structure to implement lock classes
 */
typedef struct	{

	LOCKCLASSLIST	entryList;
	LISTELEMENT		transList;
	LOCKENTRY		*lockEntry;
	ONE				lockCount;
	MAGIC		magic;
} LOCKCLASSREC;


/*
 *	define the lock entry structure magic number
 */
#define LOCKCLASSREC_MAGIC	0xa790f8cb


/*
 *	define a structure to link a volume by a transaction
 */
typedef struct	{

	LISTELEMENT	volumeList; /* for transRec->volumeList, and to TransVolRecPool.freeList */
	LISTELEMENT	transList;	/* for volRec->activeTransList */
	TRANSREC	*transRec;
	VOLREC		*volRec;
	MAGIC		magic;
} TRANSVOLREC;


/*
 *	define the magic number
 */
#define TRANSVOLREC_MAGIC	0x5e47a97f 


/*
 *	define the magic checking macros
 */
#if MAGIC_CHECKING IS_ENABLED


#define INIT_TRANSREC_MAGIC(_transRec)						\
															\
	(_transRec)->magic = TRANSREC_MAGIC;

#define INIT_UNDOTRANSREC_MAGIC(_transRec)					\
															\
	(_transRec)->magic = UNDOTRANSREC_MAGIC;


#define INIT_LOCKHEADER_MAGIC(_lockHeader)					\
															\
	(_lockHeader)->magic = LOCKHEADER_MAGIC;


#define INIT_LOCKENTRY_MAGIC(_lockEntry)					\
															\
	(_lockEntry)->magic = LOCKENTRY_MAGIC;


#define INIT_LOCKCLASSREC_MAGIC(_lockClassRec)				\
															\
	(_lockClassRec)->magic = LOCKCLASSREC_MAGIC;


#define INIT_TRANSVOLREC_MAGIC(_transVolRec)				\
															\
	(_transVolRec)->magic = TRANSVOLREC_MAGIC;


#define CHECK_TRANSREC_MAGIC(_transRec)						\
															\
	if ((_transRec)->magic != TRANSREC_MAGIC)	{			\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#define CHECK_UNDOTRANSREC_MAGIC(_transRec)					\
															\
	if ((_transRec)->magic != UNDOTRANSREC_MAGIC)	{		\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#define CHECK_TRANSVOLREC_MAGIC(_transVolRec)				\
															\
	if ((_transVolRec)->magic != TRANSVOLREC_MAGIC)	{		\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#define CHECK_LOCKHEADER_MAGIC(_lockHeader)					\
															\
	if ((_lockHeader)->magic != LOCKHEADER_MAGIC)	{		\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#define CHECK_LOCKENTRY_MAGIC(_lockEntry)					\
															\
	if ((_lockEntry)->magic != LOCKENTRY_MAGIC)	{			\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#define CHECK_LOCKCLASSREC_MAGIC(_lockClassRec)				\
															\
	if ((_lockClassRec)->magic != LOCKCLASSREC_MAGIC)	{	\
															\
		SM_ERROR(TYPE_FATAL, esmINTERNAL);					\
	}


#else


#define INIT_TRANSREC_MAGIC(_transRec)

#define INIT_UNDOTRANSREC_MAGIC(_transRec)

#define INIT_TRANSVOLREC_MAGIC(_transRec)

#define INIT_LOCKHEADER_MAGIC(_lockHeader)

#define INIT_LOCKENTRY_MAGIC(_lockEntry)

#define INIT_LOCKCLASSREC_MAGIC(_lockClassRec)

#define CHECK_TRANSREC_MAGIC(_transRec)

#define CHECK_UNDOTRANSREC_MAGIC(_transRec)

#define CHECK_TRANSVOLREC_MAGIC(_transRec)

#define CHECK_LOCKHEADER_MAGIC(_lockHeader)

#define CHECK_LOCKENTRY_MAGIC(_lockEntry)

#define CHECK_CLASSREC_MAGIC(_lockClassRec)


#endif


/*
 *	define some convenience macros
 */
#define GETTID(_transRec)							\
	(_transRec)->tid


/*
 *	define states for the various lists
 */
#define GRANT_LIST		1
#define WAIT_LIST		2
#define UPGRADE_LIST	3


/*
 *	define some macros for the deadlock detection algorithm
 */
#define VISITED(_transRec)					\
											\
	((_transRec)->visitCount == DeadlockVisitCount)


#define NOT_VISITED(_transRec)				\
											\
	((_transRec)->visitCount != DeadlockVisitCount)


#define SET_VISITED(_transRec)				\
											\
	((_transRec)->visitCount = DeadlockVisitCount)


/*
 *	define maximum depth that the deadlock stack can reach
 */
#define DEAD_STACK_SIZE		(10)


/*
 *	define the hash function to hash a tid
 */
#define HASH_TID(_tid)		((_tid) & TransHashMask)

#endif __TRANS_H__
