/*
 *   $RCSfile: IO_CheckDirection.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:47 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"


/*
 * Determine the starting pages for the current and next track to be searched.
 *  Also check to see if a direction change must occur.
 *
 *   Returns:
 *        A new current track, next track and possibly a new search direction.
 *
 *   Side Effects:
 *	None
 *
 *   Errors:
 *        None
 */


#define RIGHT 0
#define LEFT  1


 void
IO_CheckDirection (

	int			*curTrack,  	/* Pointer to first page of the current track */
	int			*nextTrack, 	/* Pointer to first page of the next track    */
	int			*direction, 	/* Pointer to the direction indicator         */
	int			startTrack, 	/* track where original search began          */
	VOLREC		*volRec     	/* points to the volume header entry          */
)
{

	TRACE(TR_IO, TR_LEVEL_1);

	/*
	 *	check to see which direction we are travelling in
	 */
    if (*direction == RIGHT) {

		*curTrack = *nextTrack;

		/*
		 *	If we reached the end of the volume, change direction of search
		 */
		if (*curTrack >= volRec->header->numPages) {

			/*
			 *	switch direction to the left
			 */
			*direction = LEFT;
			*curTrack  =  startTrack - (int) volRec->header->pagesPerTrack;
			*nextTrack = *curTrack + (int) volRec->header->pagesPerTrack;

		} else {

			/*
			 *	look at the next track
			 */
			*nextTrack += (int) volRec->header->pagesPerTrack;
		}

	} else {

		/*
		 *	begin searching at the previous track
		 */
		*nextTrack  = *curTrack;
		*curTrack  -= (int) volRec->header->pagesPerTrack;
    }
}
