/*
 *   $RCSfile: IO_Checker.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "io_intfuncs.h"
#include "bit_funcs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"

/*
 * Module IOconcheck: volume header consistency checker 
 * This routine checks the consistency of a volume header
 *
 *  Returns:
 *	None
 *
 *  Side effects:
 *	Error messages, if any, go to the standard output.
 *
 *  Errors:
 *	eioBADHEADER - the volume header is inconsistent 
 *
 *  IMPORTS:
 *	VolDev[]
 *	countmap()
 *	wsetmap()
 *	checkset()
 *	setbit()
 *	clearbit()
 *	malloc()
 *	free()
 *
 *  EXPORTS:
 *	IO_checker()	level 0 volume header checker
 */


 int
IO_Checker (

	register VOLREC	*volRec 
)
{

	register GROUPLINK  *groupLink;
	PID                 bitmapPid;
	int					total;
	int					validBits;
	SHORTPID			lastPage;
	int					Bit;

	TRACE(TR_IO, TR_LEVEL_1);

	/*
	 *	Calculate the last bitmap page
	 */
	lastPage = volRec->header->freeBitmapAddr + volRec->header->bitmapPages - 1;

	/*
	 *	Total the number of set bits on each page of the free-page
	 *	bitmap.  
	 */
	for (total = 0, 
			bitmapPid.volid = volRec->header->volid, 
			bitmapPid.page = volRec->header->freeBitmapAddr; 
		 bitmapPid.page <= lastPage; 
		 bitmapPid.page++) {

		/*
		 *	Get the corresponding bitmap page
		 */
		if ((groupLink = bf_ReadPage(volRec->bufGroup, &bitmapPid,
									 BITMAP_PAGE2SIZE, NOFLAGS)) == NULL)  {

			return(esmFAILURE);
		}

		/*
		 *	Figure out how many bits are valid to look at on the page.
		 *	Is this the last bitmap page?
		 */
		if (bitmapPid.page == lastPage) {
			validBits =  (int) ( volRec->header->numPages - 
				(BITMAP_USABLESIZE * 
				(volRec->header->bitmapPages-1) * BITSPERBYTE));
		} else {
			validBits = BITMAP_USABLESIZE * BITSPERBYTE;
		}

		/*
		 *	Count the number of bits on this page, and unfix it.
		 */
		for (Bit = 0; Bit < validBits; Bit++) {
			if (bm_CheckSet((UFOUR*)groupLink->bufFrame, Bit)) total++;
		
		}

		/* 
		 *	Don't use this code, since countmap is not realible for
		 *	(validBits % BITSPERWORD) != 0
		 *
		 * total += countmap((unsigned char*)groupLink->bufFrame, 
		 *				  validBits);
		 */

		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);

	}

	/*
	 *	check to see if the map is consistent
	 */
	if (total != volRec->header->numFreePages) {

		SM_ERROR(TYPE_FATAL, esmBADVOLHEADER);
		return(esmFAILURE);
	}

	/*
	 *	return success to the user
	 */
	return(esmNOERROR);
}
