/*
 *   $RCSfile: IO_DeallocPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:47 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "io_intfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"


/*
 * Deallocate the specified page.
 *
 *  Returns:
 *      The number of SHORT pages actually deallocated.
 *
 *  Side Effects:
 *	page bit map updated in volume header entry
 *
 *  Errors:
 *          None 
 */


 int
IO_DeallocPages (

	SHORTPID			page,
	PAGESIZE			blockSize,
	VOLREC				*volRec,
	LIST				*bitmapList,
	BOOL				doDeallocation 	/* perform the dealloc	*/
)
{

	GROUPLINK	*groupLink;
    int			index;
	SHORTPID	bitmapPage;
	PID			bitmapPid;


	TRPRINT(TR_IO, TR_LEVEL_1, ("deallocating page:%d blockSize:%d", page, blockSize));

	/*
	 *	calculate the page that the first bit lies on
	 */
	bitmapPid.page = NULLPID;
	bitmapPid.volid = volRec->header->volid;

    /*
	 *	Deallocate each page in the block
	 */
    for (index = 0; index < blockSize; index++)	{

		/*
		 *	calculate the page that the first bit lies on
		 */
		bitmapPage = BIT_TO_PAGE(page + index);
		TRPRINT(TR_IO, TR_LEVEL_2, ("bitmap page:%d", bitmapPid.page));

		/*
		 *	check to see if we access the same bitmap page
		 */
		if (bitmapPage != bitmapPid.page)	{

			/*
			 *	setup the new pid
			 */
			bitmapPid.page = bitmapPage;

			/*
			 *	attempt to read the page
			 */
			if ((groupLink = bm_ListRead(volRec, &bitmapPid, bitmapList)) == NULL)	{

				return(esmFAILURE);
			}
		}

        /*
         *  Make sure the page isn't already marked free
         */
        SM_ASSERT(LEVEL_3, !bm_CheckSet((UFOUR *) groupLink->bufFrame,
                                    (FOUR) BIT_IN_PAGE(page + index)));

		/*
		 *	actually set the bit
		 */
        if (doDeallocation) bm_SetBit((UFOUR *) groupLink->bufFrame, (FOUR) BIT_IN_PAGE(page + index));
	}

	/*
	 *	unfix the page
	 */
    return(esmNOERROR);
}
