/*
 *   $RCSfile: IO_DismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "disk.h"
#include "io_intfuncs.h"
#include "bf_extfuncs.h"
#include "disk_funcs.h"
#include "io_globals.h"


/*
 * Dismount a device described by the "Vix"th entry of the volume table.
 * 
 *   Returns:
 *	None
 *
 *  Side Effects:
 *	volume header on the device flushed if necessary &
 *	the memory buffers for the header are released 
 *
 *  Errors:
 *	eioDISMOUNTFAILED - device dismount failed
 */


extern int		errno;


 int
IO_DismountVolume (

	register VOLREC	*volRec 
)
{

	TRPRINT(TR_IO, TR_LEVEL_1, ("dismounting entry:%d", volRec->header->volid));

	SM_ASSERT(LEVEL_1, ( LIST_EMPTY(&(volRec->openList)) && 
		 LIST_EMPTY(&(volRec->activeTransList)))); 

	bf_UnfixPage(volRec->headerLink, BF_DEFAULT, FALSE);
	/*  we can lose the cpu here, but having set the
     *  QUIESCE flags should keep anyone else from
     *  trying to use the volume.
     */
	bf_FlushVolume(volRec->header->volid);

	/*
	 *	"physically" dismount the device
	 *	after sync'ing it if its not a raw disk 
	 *	(ie. guarantee all pages flushed)
	 */
	if (!(volRec->volflags & VOL_RAWDEV)) {
		if (fsyncDisk(volRec) != esmNOERROR) {
			SM_ERROR(TYPE_CRASH, esmINTERNAL);
		}
	}

	/*
	 * Unset the VOL_OPEN flag before:
	 * - freeing the buffer group, 
	 * - closing the disk
	 * so that noone tries to do anything with this volume.
	 */
	IO_RemoveVolRec(volRec);
	closeDisk(volRec);

	if (volRec->bufGroup != NULL)	{
		bf_CloseGroup(volRec->bufGroup);
		volRec->bufGroup = NULL;
	}
	/* 
	 * someone could be waiting with THREAD_DISKCLOSE_WAIT
	 * (or THREAD_DISKOPEN_WAIT, for that matter)
	 */
	notify( &(volRec->tcbList), Active->error, Active->errno );

	IO_FreeVolRec(volRec);
	return(esmNOERROR);
}
