/*
 *   $RCSfile: IO_FlushHeader.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bf_extfuncs.h"
#include "io_intfuncs.h"


/*
 * Flush header pages for the volRec described by the "slot"th in the table.
 *
 *  Returns:
 *	NONE
 *
 *  Side Effects:
 *	All modified header pages are written to vol
 *
 *  Errors:
 *	None originated by this routine.
 */


 int
IO_FlushHeader (

	register VOLREC	*volRec 
)
{

	TRPRINT(TR_IO, TR_LEVEL_1, ("flushing volRec:%d", volRec->header->volid));

	/*
	 *	Check to see if the volRec has been modified
	 */
	if (volRec->dirty) {

		/*
		 *	Write volRec header.  Note that the following write
		 *	will write both the desired volRec header and some
		 *	of the subsequent volRec headers since I/O has to
		 *	occur in page size units.  This is not a problem
		 *	since subsequent mounts of this volRec read the page
		 *	containing the volRec header into a separate page sized
		 *	buffer before loading the information into VolTable
		 */
		bf_ForcePage(volRec->headerLink->pageHash, BF_SEM);

	}

	/*
	 *	return success to the caller
	 */
	return(esmNOERROR);
}
