/*
 *   $RCSfile: IO_LogPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_intfuncs.h"
#include "thread_globals.h"


 int
IO_LogPageUnique (

	PID					*pid,
	int					word,
	UNIQUE				unique,
	PAGEHASH			*pageHash 
)
{

	FORCEMARK			forceMark;
	LSN					lsn;
	LRC					*lrc;
	LOGRECORDINFO		recordInfo;


	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("logging pid:%d", pid->page));

    /*
     *  get the lrc on the bitmap page and increment it
     */
    lrc = &(((BITMAPPAGE *) pageHash->bufFrame)->lrc);
    INCREMENT_LRC(lrc);

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_USER;
	recordInfo.action		= LOG_ACTION_INCREMENT_UNIQUE;
	recordInfo.flags		= REDO_ONLY;
	recordInfo.imageCount	= 2;
    recordInfo.actionPid	= pid;
	recordInfo.actionLRC    = lrc;
	recordInfo.imageSize[0] = sizeof(word);
	recordInfo.imageData[0] = (VOID *) &word;
	recordInfo.imageSize[1] = sizeof(unique);
	recordInfo.imageData[1] = (VOID *) &unique;
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("word:%d unique:%d", word, unique));

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		return(esmFAILURE);
	}

	/*
	 *	put the dependency on the page
	 */
	DEPEND_LOG(pageHash, forceMark, &lsn, lrc);

	/*
	 *	return success
	 */
	return(esmNOERROR);
}


 int
IO_LogPageMark (
	PID					*pid,
	int					pageType,
	BOOL				mark,
	PAGEHASH			*pageHash 
)
{

	FORCEMARK			forceMark;
	LSN					lsn;
	LOGRECORDINFO		recordInfo;
	LRC					*lrc;

	TRPRINT(TR_IO, TR_LEVEL_1, ("logging pid:%d", pid->page));

	/*
	 *  get the lrc on the bitmap page and increment it
	 */
	lrc = &(((BITMAPPAGE *) pageHash->bufFrame)->lrc);
	INCREMENT_LRC(lrc);

	/*
	 *	initialize the action type
	 */
	switch (pageType)	{

		case PAGE_FILEHDR:

			if (mark) recordInfo.action = LOG_ACTION_MARK_FILE_PAGE;
			else recordInfo.action = LOG_ACTION_UNMARK_FILE_PAGE;
			break;

		case PAGE_SLOTTED:

			if (mark) recordInfo.action = LOG_ACTION_MARK_SLOTTED_PAGE;
			else recordInfo.action = LOG_ACTION_UNMARK_SLOTTED_PAGE;
			break;

		default:

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}

	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_USER;
	recordInfo.flags		= NOFLAGS;
	recordInfo.imageCount	= 1;
    recordInfo.actionPid    = &(pageHash->hashList.hashPid);
    recordInfo.actionLRC    = lrc;
	recordInfo.imageSize[0] = (unsigned short) sizeof(PID);
	recordInfo.imageData[0] = (VOID *) pid;


	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		return(esmFAILURE);
	}

	/*
	 *	put the dependency on the page
	 */
	DEPEND_LOG(pageHash, forceMark, &lsn, lrc);

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
