/*
 *   $RCSfile: IO_ReadPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "threadstate.h"
#include "disk.h"
#include "disk_funcs.h"
#include "util_funcs.h"
#include "io_globals.h"
#include "bf_globals.h"
#include "sharedmem_globals.h"
#include "thread_globals.h"
#include "queue_consist.h"


/*============================================================================*/
/* 
 * IO_ReadPage:
 * 	Read one or more physical pages into a buffer.
 *
 * RETURNS:
 *	smNOERROR
 *
 * ERROR RETURNS:
 *	eioDEVSEEKERROR - device seek error.
 *	eioDEVREADERROR - device read error.
 *
 * SIDE EFFECTS:
 *	An lseek is done and data is read into the buffer.
 */
/*============================================================================*/


extern int		errno;

 int
IO_ReadPage (

	VOLREC 			*volRec, 	/* volume rec hold link & queues */
	SHORTPID		page,		/* starting "physical" address of the page	*/
	int				count,		/* number of bytes to read					*/
	char			*buffer		/* target of the read 						*/
)

{
	DISKMSG			*diskmessage = Active->diskMessage;
	int				offset;

	TRPRINT(TR_IO, TR_LEVEL_2, ("reading page:%d volid %d count:%d",
			page, volRec->volid, count));

#if IO_STATISTICS IS_ENABLED

	/*
	 *	increment the number of reads
	 */
	IO_DiskReads++;

#endif

	/*
	 *	check to see that we have a multiple of the page size
	 */
	SM_ASSERT(LEVEL_3, (count % MIN_PAGESIZE) == 0);

	/*
	 *	calculate the offset of the read
	 */
	offset = (int) (page << MIN_PAGE2SIZE);
	TRPRINT(TR_IO, TR_LEVEL_2, ("offset:%d", offset));

	/*
	 *	setup the message to the driver
	 */
	diskmessage->header.type = READ_BLOCK;
	diskmessage->body.offset = offset;
	diskmessage->body.diskVec[0].length = count;
	diskmessage->body.diskVec[0].bufIndex = buffer - ShmAddress;
	diskmessage->body.diskVecCount = 1;
	SM_ASSERT(LEVEL_3, (diskmessage->body.diskVec[0].bufIndex < ShmSize) && 
					   (diskmessage->body.diskVec[0].bufIndex >= 0) );

	/*
	 *	call the disk
	 */
	if(callDisk(volRec, diskmessage))  {

		return(esmFAILURE);
	}

	/*
	 *	return success to the caller
	 */
	return(esmNOERROR);
}
