/*
 *   $RCSfile: IO_WritePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "threadstate.h"
#include "disk.h"
#include "disk_funcs.h"
#include "util_funcs.h"
#include "io_globals.h"
#include "bf_globals.h"
#include "thread_globals.h"
#include "sharedmem_globals.h"
#include "queue_consist.h"

/*============================================================================*/
/* 
 * IO_WritePage:
 * 	Write one or more physical pages to disk.
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	eioDEVSEEKERROR  - device seek error.
 *	eioDEVWRITEERROR - device write error.
 *
 * SIDE EFFECTS:
 *	An lseek is done and data is written to the device.
 */
/*============================================================================*/

/*
 *	For write timing of log pages
 */
struct timeval startTime;
struct timeval stopTime;
int StatTotalTime_usec=0;
int StatPagesWritten=0; 
int StatWriteCalls=0; 
int startClientMsgs;
int stopClientMsgs;
int StatClientMsgs=0;
extern sel_clients;

 int
IO_WritePage (
	VOLREC			*volRec,
	SHORTPID		page[],		/* starting "physical" address of the page	*/
	int				count[],	/* number of bytes to read			*/
	char			*buffer[],	/* address of the memory buffer		*/
	int				pageListLen /* number of pages to write 		*/
)
{
	DISKMSG			*diskmessage = Active->diskMessage;
	int				offset;
	int 			i;

	TRPRINT(TR_IO, TR_LEVEL_1, 
	("writing %d pages page:%d volid %d count:%d ", 
		pageListLen, page[0], volRec->volid, count[0] ));

	if (PrintExtraStats && (count[0] == 8192)) {
		gettimeofday(&startTime, NULL);
		startClientMsgs = sel_clients;
	}

#if IO_STATISTICS IS_ENABLED

	IO_DiskWrites += pageListLen;

#endif

	for (i = 0; i < pageListLen; i++) {

		/* check to see that we have a multiple of the page size */
		SM_ASSERT(LEVEL_3, (count[i] % MIN_PAGESIZE) == 0);

		diskmessage->body.diskVec[i].length = count[i];
		diskmessage->body.diskVec[i].bufIndex = buffer[i] - ShmAddress;

		SM_ASSERT(LEVEL_3, (diskmessage->body.diskVec[i].bufIndex < ShmSize) &&
						   (diskmessage->body.diskVec[i].bufIndex >= 0) );
	}

	/* calculate the offset of the read */
	offset = (int) page[0] << MIN_PAGE2SIZE;
	TRPRINT(TR_IO, TR_LEVEL_2, ("offset:%d", offset));

	/*
	 *	setup the message to the driver
	 */
	diskmessage->header.type = WRITE_BLOCK;
	diskmessage->body.offset = offset;
	diskmessage->body.diskVecCount = pageListLen;


	/*
	 *	call the disk
	 */
	if (callDisk(volRec, diskmessage))	{

		return(esmFAILURE);
	}

	if (PrintExtraStats && (count[0] == 8192)) {
		gettimeofday(&stopTime, NULL);
		stopClientMsgs = sel_clients;
		StatPagesWritten += pageListLen;
		StatWriteCalls++;
		StatClientMsgs += stopClientMsgs - startClientMsgs;
		StatTotalTime_usec += 
			(int) (1000000 * (stopTime.tv_sec - startTime.tv_sec) + (stopTime.tv_usec - startTime.tv_usec));
	}

	/*
	 *	return success to the caller
	 */
	return(esmNOERROR);
}
