/*
 *   $RCSfile: io_AllocPageDeallocInfo.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "bf.h"
#include "link.h"
#include "volume.h"
#include "deallocinfo.h"
#include "io_extfuncs.h"

#ifdef DEBUG
static int	chunkSize = 2;
#else
static int	chunkSize = 100;
#endif

 char
*io_AllocPageDeallocInfo (
	POOL	*pageDeallocInfoPool 
)
{
	PAGEDEALLOCINFO	*pageDeallocInfo;
	int				i;


	TRACE(TR_IO, TR_LEVEL_1);

	/*
	 *	Allocate a chunk of structs
	 */
	if ((pageDeallocInfo = (PAGEDEALLOCINFO *) malloc(sizeof(PAGEDEALLOCINFO) * chunkSize)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	initialize the elements
	 */
	for (i = 0; i < chunkSize; i++) {
		initializeListElement( &(pageDeallocInfo[i].list), (char *) &(pageDeallocInfo[i]) );
		INIT_PAGEDEALLOCINFO_MAGIC(pageDeallocInfo+i);
		listEnq(&(pageDeallocInfoPool->freeList), &(pageDeallocInfo[i].list));
	}
	pageDeallocInfoPool->elements += chunkSize;	


	/*
	 *	return a pointer to the first newly allocated element
	 */
	pageDeallocInfo = (PAGEDEALLOCINFO*) listDeq(&(pageDeallocInfoPool->freeList));
	return((char*)pageDeallocInfo);
}
