/*
 *   $RCSfile: io_CheckBitmap.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:47 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "latch.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bitmap.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"


 int
io_CheckBitmap (

	PID					*pid,
	int					pageType,
	BOOL				*bitSet
)
{

	register VOLREC		*volRec;
	register GROUPLINK	*groupLink;
	PID					bitmapPid;


	TRPRINT(TR_IO, TR_LEVEL_1, ("checking page:%d type:%d", pid->page, pageType));

	if ((volRec = io_FindVolRec(pid->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	check to see if the bit address is valid
	 */
	if ((pid->page < 0) || (pid->page > volRec->header->numPages))	{

		SM_ERROR(TYPE_USER, esmBADPID);
		return(esmFAILURE);
	}

	/*
	 *	get the page the bit resides in
	 */
	bitmapPid.page = BIT_TO_PAGE(pid->page);
	SM_ASSERT(LEVEL_1, volRec->header->volid==pid->volid);
	bitmapPid.volid = pid->volid;

	TRPRINT(TR_IO, TR_LEVEL_1, ("bitmap page:%d", bitmapPid.page));

	/*
	 *	Check the type to see if we should set any of the bitmaps
	 */
	switch (pageType)	{

		case PAGE_SLOTTED:

			TRPRINT(TR_IO, TR_LEVEL_1, ("type slotted"));
			bitmapPid.page += volRec->header->slotBitmapAddr;
			break;

		case PAGE_FILEHDR:

			TRPRINT(TR_IO, TR_LEVEL_1, ("type fileHdr"));
			bitmapPid.page += volRec->header->fileBitmapAddr;
			break;

		default:
			TRPRINT(TR_IO, TR_LEVEL_1, ("type other"));
			bitmapPid.page += volRec->header->freeBitmapAddr;
			break;
	}

	/*
	 *	read in the bitmap page
	 */
	if ((groupLink = bf_ReadPage(volRec->bufGroup, &bitmapPid,
								 MIN_PAGE2SIZE, NOFLAGS)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	Check to see if the bit is set
	 */
	if (bm_CheckSet((UFOUR *) groupLink->bufFrame, (FOUR) BIT_IN_PAGE(pid->page)))	{

		/*
		 *	bit is correct type
		 */
		*bitSet = TRUE;
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
		return(esmNOERROR);
	
	} else {

		/*
		 *	bit is not correct type
		 */
		*bitSet = FALSE;
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
		return(esmNOERROR);
	}
}
