/*
 *   $RCSfile: io_DismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "openlog.h"
#include "logrecs.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "log_extfuncs.h"
#include "thread.h"
#include "thread_globals.h"


/* Module io_dismount: dismount devices 
 *
 *  Returns:
 *	NONE
 *
 *  Side Effects:
 *	volume header flushed if necessary; 
 *	the device information removed from DiskTable
 *
 *  Errors generated here:
 *	eioVOLNOTMOUNTED - volume not mounted
 *
 *
 *  IMPORTS:
 *	DiskTable[]
 *	IO_DisMount()
 *
 *  EXPORTS:
 *	io_dismount()	dismount a active device
 */


 int
io_DismountVolume (

	VOLREC			*volRec,
	BOOL			logDismount 
)
{
	int				returnvalue = esmNOERROR;
	LOGRECORDINFO   recordInfo;
	LSN             lsn;

	/* cache the following  stuff because we need the info after 
	 * volRec is gone */
	BOOL			isTemp = (volRec->volflags & VOL_TEMP)==VOL_TEMP;
	VOLID			volid = volRec->header->volid;

	/*
	 *	Make sure it's ok to unmount the volume
	 */
	if ( LIST_NOT_EMPTY(&(volRec->openList)) || 
		 LIST_NOT_EMPTY(&(volRec->activeTransList)) )  {

		/* in use; can't dismount */
		return esmNOERROR;
	}
	TRPRINT(TR_IO, TR_LEVEL_1, ("dismounting volume:%d", volid)); 
	Active->errno = esmNOERROR;
	volRec->volflags |= VOL_QUIESCE;
	returnvalue = IO_DismountVolume(volRec);

	volRec = NULL; /* volRec is no longer valid after IO_Dismount */

	if(returnvalue == esmNOERROR) {

		/*
		 *  See if the dismount should be logged.
		 *	Do not log the dismount of temporary volumes
		 */
		if (logDismount && !isTemp) {

			TRPRINT(TR_IO, TR_LEVEL_1, ("logging dismount of volume:%d", volid));

			recordInfo.action       = 0;
			recordInfo.type         = LOG_REC_TYPE_DISMOUNT_VOLUME;
			recordInfo.imageCount   = 1;
			recordInfo.actionPid    = NULL;
			recordInfo.actionLRC    = NULL;
			recordInfo.imageSize[0] = sizeof(VOLID);
			recordInfo.imageData[0] = (VOID *) &volid;
			recordInfo.flags        = NOFLAGS;

			if (writeNonTransLogRecord(&recordInfo, &lsn, LOG_FORCE) < 0)  {
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}
		}
	}

	return returnvalue;
}
