/*
 *   $RCSfile: io_GetUnique.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bitmap.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"


 int
io_GetUnique (

	PID					*pid,
	UNIQUE				*unique,
	int					count 
)
{

	VOLREC		*volRec;
	GROUPLINK	*groupLink;
	UNIQUE		*mapPtr;
	int			word;
	PID			bitmapPid;


	TRPRINT(TR_IO, TR_LEVEL_1, ("unique for page:%d count:%d", pid->page, count));

	/*
	 *	get a pointer to the vol record
	 */
	if ((volRec = io_FindVolRec(pid->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	check to make sure the page is valid
	 */
	SM_ASSERT(LEVEL_1, (pid->page >= 0) && (pid->page < volRec->header->numPages));

	/*
	 *	calulate the bitmap page the word resides on
	 */
	bitmapPid.page = BIT_TO_PAGE(pid->page);
	bitmapPid.volid = pid->volid;
	TRPRINT(TR_IO, TR_LEVEL_2, ("word on page:%d", bitmapPid.page));
	bitmapPid.page += volRec->header->uniqueArrayAddr;

	/*
	 *	attempt to read in the page
	 */
	if ((groupLink = bf_ReadPage(volRec->bufGroup, &bitmapPid,
								 BITMAP_PAGE2SIZE, BF_SEM)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	get the pointer to the word map
	 */
	mapPtr = (UNIQUE *) groupLink->bufFrame;

	/*
	 *	get the word offset
	 */
	word = (int) BIT_TO_WORD(BIT_IN_PAGE(pid->page));

	/*
	 *	get the current unique value
	 */
	*unique = mapPtr[word];
	TRPRINT(TR_IO, TR_LEVEL_2, ("unique:%x", *unique));

	/*
	 *	log the increment
	 */
	if (IO_LogPageUnique(&bitmapPid, word, *unique + count, groupLink->pageHash)) {

		signalSemaphore( &(groupLink->pageHash->semaphore) );
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
		return(esmFAILURE);
	}

	/*
	 *	increment the unique value
	 */
	mapPtr[word] += count;

	/*
	 *	signal the semaphore
	 */
	signalSemaphore( &(groupLink->pageHash->semaphore) );

	/*
	 *	unfix and dirty the page
	 */
	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
