/*
 *   $RCSfile: io_MarkPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "bitmap.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "thread_globals.h"


 int
io_MarkPage (

	VOLREC				*volRec,
	PID					*pid,
	int					pageType,
	BOOL				mark 
)
{

	GROUPLINK			*groupLink;
	int					pageOffset;
	PID					bitmapPid;


	TRPRINT(TR_IO, TR_LEVEL_1, ("marking page:%d as:%d", pid->page, pageType));

	/*
	 *	initialize the pid with the volume
	 */
	bitmapPid.volid = volRec->header->volid;

	/*
	 *	calculate the offset to the correct page
	 */
	pageOffset = (int) BIT_TO_PAGE(pid->page);
	TRPRINT(TR_IO, TR_LEVEL_2, ("pageOffset:%d", pageOffset));

	/*
	 *	Check the type to see if we should set any of the bitmaps
	 */
	switch (pageType)	{

		case PAGE_SLOTTED:

			/*
			 *	get the pid of the correct slotted bitmap page
			 */
			bitmapPid.page = volRec->header->slotBitmapAddr + pageOffset;
			TRPRINT(TR_IO, TR_LEVEL_2, ("slotted bitmap page:%d", bitmapPid.page));
			break;

		case PAGE_FILEHDR:

			/*
			 *	get the pid of the correct file bitmap page
			 */
			bitmapPid.page = volRec->header->fileBitmapAddr + pageOffset;
			TRPRINT(TR_IO, TR_LEVEL_2, ("fileheader bitmap page:%d", bitmapPid.page));
			break;

		default:
	
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}

	/*
	 *	read in the page
	 */
	if ((groupLink = bf_ReadPage(volRec->bufGroup, &bitmapPid,
								 BITMAP_PAGE2SIZE, BF_SEM)) == NULL)	{

		return(esmFAILURE);
	}

	/*
	 *	log the page mark
	 */
	if (IO_LogPageMark( pid, pageType, mark, groupLink->pageHash))	{

		signalSemaphore( &(groupLink->pageHash->semaphore) );
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);
		return(esmFAILURE);
	}

	/*
	 *	check to see if we are to set or unset the bit
	 */
	if (mark)	{

		/*
		 *	mark the bit
		 */
		bm_SetBit((UFOUR *) groupLink->bufFrame, BIT_IN_PAGE(pid->page));
	
	} else {

		/*
		 *	mark the bit
		 */
		bm_ClearBit((UFOUR *) groupLink->bufFrame, BIT_IN_PAGE(pid->page));
	}

	/*
	 *	signal the semaphore
	 */
	signalSemaphore( &(groupLink->pageHash->semaphore) );

	/*
	 *	unfix the page
	 */
	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);

	/*
	 *	return success to the caller
	 */
	return(esmNOERROR);
}
