/*
 *   $RCSfile: io_ReadPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "queue_consist.h"


/*============================================================================*/
/* 
 * io_readpage:
 * 	Read one or more physical pages into a buffer.
 *
 * IMPORTS:
 *      IO_ReadPage()
 *
 * EXPORTS:	
 *	io_readpage (pid, buffer, Size)
 *
 * BUGS:
 *      IO_ReadPage should be in-line expanded here for efficiency.
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	eioNULLPIDPTR    - null PID pointer.
 *	eioNULLBUFPTR    - null memory buffer.
 *	eioVOLNOTMOUNTED - volume not mounted.
 *	eioINVALIDPID    - invalid page ID.
 *
 * SIDE EFFECTS:
 *	Data is read into the buffer.
 */
/*============================================================================*/


 int
io_ReadPage (
	const PID		*pid,
	char			*buffer,
	PAGE2SIZE		page2size
)
{

	VOLREC		*volRec;
	int			count;
	SHORTPID	page;


	TRPRINT(TR_IO, TR_LEVEL_1, ("reading page:%d page2size:%d", pid->page, page2size));

	/*
	 *	get a pointer to the first element of the vol array
	 */
	if ((volRec = io_FindVolRec(pid->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	get the page number from the pid
	 */
	page = pid->page;

	/*
	 *	check to make sure the pid is valid
	 */
	if ((page < 0) || (page > volRec->header->numPages))	{

		SM_ERROR(TYPE_USER, esmBADPID);
		return(esmFAILURE);
	}

	/*
	 * Set count to the number of bytes to read,
	 * then call ReadPage to actually do the read.
	 */
    count = RAISETWO(page2size);

	/*
	 *	call to read the page
	 */
	return(IO_ReadPage(volRec, page, count, buffer));
}
