/*
 *   $RCSfile: io_SyncVolumes.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "openlog.h"
#include "io_extfuncs.h"
#include "disk_funcs.h"
#include "io_globals.h"


extern int		errno;

/*
 *	DESCRIPTION:	for each volume stored in a unix file,
 *				 	io_SyncUnixFileVolumes will fsync the unix file.
 */

static BOOL
syncit(
	VOLREC  *volRec,
	int		*error
)
{
	/*
	 *	If the volume is open and on a unix file and not
	 *	a temporary volume, sync it.
	 */
	if ( (volRec->volflags & (VOL_OPEN | VOL_RAWDEV | VOL_TEMP)) == VOL_OPEN) {

		PRINT_PROGRESS(("syncing volume %d: %s\n", 
			volRec->header->volid, volRec->volNameRec->volName.name));

		if ((*error = fsyncDisk(volRec)) != esmNOERROR) {
			return TRUE;
		}
	}
	return FALSE;
}

 int
io_SyncUnixFileVolumes ()
{
	int error = esmNOERROR;

	TRPRINT(TR_IO, TR_LEVEL_1, ("sync'ing all volumes"));

	ForEachVolume(2, syncit, &error, 0, 0);
	return error;
}



/*
 *	DESCRIPTION: 	for each volume used by a transaction and stored
 *					in a unix file, io_SyncUnixFileVolumes will fsync
 *					the unix file.
 */
 int
io_SyncTransVolumes (LIST* volList)
{
	TRANSVOLREC		*transVolRec;

	TRPRINT(TR_IO, TR_LEVEL_1, ("sync'ing all volumes for transaction"));

	transVolRec = (TRANSVOLREC*) FIRST_LIST_ELEMENT(volList);
	CHECK_TRANSVOLREC_MAGIC(transVolRec);
	while(transVolRec != NULL ) {
		SM_ASSERT(LEVEL_3, transVolRec->volRec->volflags & VOL_OPEN);

		/*
		 *	If the volume is on a unix file and not
		 *	a temporary volume, sync it.
		 */
		if ( (transVolRec->volRec->volflags & (VOL_RAWDEV | VOL_TEMP)) == 0) {

			PRINT_PROGRESS(("syncing volume %d: %s\n", transVolRec->volRec->header->volid, transVolRec->volRec->volNameRec->volName.name));
			if (fsyncDisk(transVolRec->volRec) != esmNOERROR) {
				return(esmFAILURE);
			}
		}

		transVolRec = (TRANSVOLREC*) NEXT_LIST_ELEMENT(&(transVolRec->volumeList));
	}

	return(esmNOERROR);
}

