#ifndef _IO_VOLUMETABLE_C_
#define _IO_VOLUMETABLE_C_
/*
 *   $RCSfile: io_VolumeTable.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

/*
 *****************************************************************
 *       c-callable wrapper funcs for page hash table and pool
 *****************************************************************
 */


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "io.h"
#include "object.h"
#include "tid.h"
#include "bf.h"
#include "lock.h"
#include "link.h"
#include "volume.h"
#include "resources.h"
#include "HashTable.h"
#include "Pool.h"
#include "pool.h"
#include "queues.h"
#include "io_extfuncs.h"
#include "io_intfuncs.h"
#include "init_funcs.h"
#include "io_globals.h"
#include "bf_extfuncs.h"
#include "threadstate.h"
#include "thread_funcs.h"
/* 
 * include the code for the class funcs.
 */
#include "VOLREC.h"
#include "class_alias_macros.h"


/* VOLNAMERECs are hashed on VOLUMENAMES */
void 
VOLNAMEREC:: Dump(FILE *f) {
	fprintf(f,  "DUMP OF VOLNAMEREC 0x%x\n", this);
	fprintf(f, "volid %d volName %s\n", this->volid, this->volName.name);
}; 

VOLIDCLASS *
VOLNAMEREC:: keylocation (int unique, VOLIDCLASS **unused) {
	SM_ASSERT(LEVEL_1, (unique == VOLID_TABLE) );
	return (VOLIDCLASS *) (void *)&(this->volid); 
};

VOLUMENAME *
VOLNAMEREC:: keylocation (int unique, VOLUMENAME **unused) {
	SM_ASSERT(LEVEL_1, (unique == VOLNAME_TABLE) );
	return (VOLUMENAME *)&(this->volName); 
};

LISTELEMENT *
VOLNAMEREC:: listlocation (int unique) {
	if((unique == VOLNAMEREC_POOL)  || (unique == VOLNAME_TABLE) ) {
		return &(nameHashList); 
	} else if(unique == VOLID_TABLE) {
		return &(volidHashList); 
	} else { 
		SM_ASSERT(LEVEL_1, 0); 
		/* to shut the compiler UP */
		return &(volidHashList); 
	}
};

void 
VOLNAMEREC:: ReInit() {
	this->volid = 0;
	bzero(this->volName.name, DEVNAMEMAX);
};

void 
VOLNAMEREC:: Init() {
	initializeListElement(this->listlocation(VOLNAME_TABLE), this);
	initializeListElement(this->listlocation(VOLID_TABLE), this);
	this->ReInit();
};


#define VOLNAME_HASHTSIZE	7

HashTable<VOLIDCLASS,VOLREC>   	*volRecTable = NULL; /* hashed on volid */
Pool<VOLREC>						*volRecPool = NULL;

HashTable<VOLUMENAME,VOLNAMEREC>  *volNameTable = NULL;
HashTable<VOLIDCLASS,VOLNAMEREC>   	*volIdTable = NULL;
Pool<VOLNAMEREC>						*volNameRecPool = NULL;

int											NumVolumes = 0; /* limits the # volumes 
					we can possibly mount, based on the number of volume names given in 
					the config file (mount table) */

static	BOOL	ioIsInitializing = FALSE; /* when TRUE we can in NumVolumes */

static	BOOL	savePrintUser;

void
initializeVolNameTable()
{
	/* this has to be called BEFORE options processing starts */
	TRACE(TR_IO, TR_LEVEL_1);

	ioIsInitializing = TRUE;
	savePrintUser = PrintUser;

	volNameRecPool = new Pool<VOLNAMEREC> ( "volName", VOLNAMEREC_POOL,
		10, POOL_NOMORE_GETMORE );

	volNameTable = new HashTable<VOLUMENAME,VOLNAMEREC>( "volName",
		VOLNAME_TABLE, VOLNAME_HASHTSIZE);

	volIdTable = new HashTable<VOLIDCLASS,VOLNAMEREC>( "volId",
		VOLID_TABLE, VOLNAME_HASHTSIZE);

}

static BOOL
initQ(
	VOLREC	*volRec,
	int		*ordinal
)
{
	volRec->queues = initDiskQueue(*ordinal, volRec);
	(*ordinal)++;
	return FALSE;
}

 int
io_Init ()
{
	/* This assumes that options processing has been done,
	 * so that NumVolumes reflects the number of volume names
	 * in the mount table, including the log volume.
	 *
	 * It also assumes that initializeQueues() has been done.
	 */

	TRACE(TR_IO, TR_LEVEL_1);

	SM_ASSERT(LEVEL_1, (MaxAddVolumes == -1)); 
	/* already added in to NumVolumes */

	/*
		This was done in initializeSharedMemory:
		NumVolumes += MaxAddVolumes;
		MaxAddVolumes = -1;
	*/

	volRecPool = new Pool<VOLREC> ( "volRec", VOLREC_POOL,
		NumVolumes, POOL_NOMORE_ISERROR ); 
		/* This pool is a static size because it is tied to 
		 * the amount of shared memory that we allocate.
		 * (We have one shm disk queue for each possible mounted volume.)
		 */
	/* From this point on, we cannot change NumVolumes; so we
	 * unset the flag that allows us to bump it up
	 */
	ioIsInitializing = FALSE;
	PrintUser = savePrintUser;

	{
		int ordinal = 0;
#ifdef __GNUC__
		volRecPool->ForEach(2, initQ, &ordinal);
#else __GNUC__
		volRecPool->ForEach(2, (FOREACHFUNC) initQ, &ordinal);
#endif __GNUC__
	}

	volRecTable = new HashTable<VOLIDCLASS,VOLREC>( "volRec", 
		VOLREC_TABLE, VOLNAME_HASHTSIZE);

	if(formatvolRunning)
		return esmNOERROR;

	/* 
	 * get the volids for all the volumes listed in the config file.
	 */
	/* Policy change: we'll believe the volid info in the 
	 * options and it'll get checked at mount time.
	 * This is the way it should be if we are to allow the 
	 * server to do the formatting (not yet implemented). 
	 * Furthermore, if the volumes get re-formatted between
	 * now and when they get mounted, they could become illegit,
	 * so why bother now?
	 */
	io_GetVolIds(); 


	io_ClearStat();

	/*
	 *	Initialize the delayed deallocation information pools
	 */
	initializePool(&PageDeallocInfoPool, (ALLOCFUNC) io_AllocPageDeallocInfo, 0);
	initializePool(&FileDeallocInfoPool, (ALLOCFUNC) io_AllocFileDeallocInfo, 0);

	return esmNOERROR;
}

/* 
 * THE NEXT TWO FUNCS 
 * take char * args for volName
 * because they're called from the volumeOpts.C (options
 * processing stuff for formatvol and server.
 */
VOLNAMEREC *
io_FindVolName( const	char *volName )
{
	VOLUMENAME	vn;
	VOLNAMEREC	*volNameRec;

	if(strlen(volName) > sizeof(volNameRec->volName.name)) {
		SM_ERROR(TYPE_USER, esmBADPARAMS); /* TODO: really need 
			an error code for "path name too long" */
		return NULL;
	}
	/* grot - cfront doesn't handle struct copy */
	(void) bcopy( volName, (char *)vn.name, sizeof(vn.name));

	volNameRec = volNameTable->Find(&vn);

	return volNameRec;
}

/* a function for use by parseFormat */
VOLNAMEREC *
io_GetVolNameRec(const char  *volName, VOLID volid, const char *const optionname)
{
	VOLNAMEREC *volNameRec;
	VOLNAMEREC *volIdRec;
	int			caseOf4;
	BOOL		mightreuse = FALSE;
	
	volNameRec  = io_FindVolName(volName);
	volIdRec = io_FindVolNameById( volid );

	if(volNameRec!=NULL && volIdRec!=NULL) {
		caseOf4 = 1;
	} else if(volNameRec!=NULL) {
		caseOf4 = 2;
	} else if(volIdRec!=NULL) {
		caseOf4 = 3;
	} else {
		caseOf4 = 4;
	}

	switch(caseOf4) {
	case 1: /* both volid and volname are in the tables. */
		/* are they the same? */
		if(volIdRec != volNameRec) {
			/* nope - there's a diff path with this volid */
			SM_ERROR(ioIsInitializing?TYPE_LOG:TYPE_USER, esmDUPVOLID);
			/*
			 * Use TYPE_LOG instead of TYPE_STOP so the caller
			 * can print more info before stopping.
			 */
			volNameRec = NULL;
		} else {
			mightreuse = TRUE;
		}
		break;

	case 2: /* volname is there but volid is not */
		SM_ASSERT(LEVEL_1, (volNameRec->fmtinfo != NULL));
		mightreuse = TRUE;
		break;

	case 3: /* volid is there but vol name is not */
		if(volIdRec->fmtinfo == NULL) {
			SM_ASSERT(LEVEL_1, (ioIsInitializing));
			/* 
			 * The only way an entry can get in here w/o any fmtinfo
			 * is from the "logvolume" option, in which case it has
			 * only the volid in it.
			 */
			/* this is ok - go ahead and fill in the fmt info */
			volNameRec = volIdRec;

		} else {
			/* there's already an entry for this volid, but
			 * not for this volume name!
			 * Use TYPE_LOG instead of TYPE_STOP so the caller
			 * can print more info before stopping.
			 */
			SM_ERROR(ioIsInitializing?TYPE_LOG:TYPE_USER, esmDUPVOLID);
			/* volNameRec == NULL */
		}
		break;

	case 4: /* neither volid nor vol name is in the tables */
		volNameRec = IO_NewVolNameRec(volName, volid);
		SM_ASSERT(LEVEL_1, (volNameRec != NULL));
		break;

	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	if(mightreuse) {
		if(ioIsInitializing) {
			fprintf(stderr,
			"\n******WARNING:\nDuplicate option (%s) for path %s (volid %d)\n\n",
					optionname, volName, volid);
			SM_ERROR(TYPE_WARNING, esmDUPVOLNAME);

			SM_FREE_PTR(volNameRec->fmtinfo); /* sets it to null */
			/* because caller mallocs a new one */
		} else {
			/* consider it an error instead of overriding it.
			 * caller can remove the old one, then
			 * add the new one.
			 */
			SM_ERROR(TYPE_USER, esmDUPVOLNAME);
			volNameRec = NULL;
		}
	}

	return volNameRec;
}


VOLNAMEREC *
io_FindVolNameById( VOLID volid )
{
	CLASS_TYPE(VOLID)		*CLASS_ALIAS(volid) = (CLASS_TYPE(VOLID) *) &volid;
	VOLNAMEREC	*volNameRec;

	volNameRec = volIdTable->Find(CLASS_ALIAS(volid));

	return volNameRec;
}

static int
WaitFor(
	VOLREC *volRec
)
{
	int waitstatus;

	for(;;) {
		if( (volRec->volflags & VOL_OPEN) == VOL_OPEN ) {
			return esmNOERROR;
		} else if (volRec->volflags & VOL_INIT)   {
			TRPRINT(TR_IO, TR_LEVEL_2, ("in init state"));
			waitstatus = THREAD_DISKOPEN_WAIT;
		} else if (volRec->volflags & VOL_INVALID) {
			TRPRINT(TR_IO, TR_LEVEL_2, ("link invalid"));
			SM_ERROR(TYPE_SYS, esmDISKPROCDIED);
			return esmFAILURE;

		} else if (volRec->volflags & VOL_QUIESCE) {
			TRPRINT(TR_IO, TR_LEVEL_2, ("disk quiesced"));
			(void) waitList( &(volRec->tcbList), THREAD_DISKCLOSE_WAIT);
			/* is closed */
			return esmFAILURE;

		} else if (volRec->volflags & VOL_FORMAT)  {
			TRPRINT(TR_IO, TR_LEVEL_2, ("disk formatting"));
			waitstatus = THREAD_DISKFORMAT_WAIT;

		} else if (volRec->volflags & VOL_FREE) {
			return esmFAILURE; /* no longer in use */
		}

		TRPRINT(TR_IO, TR_LEVEL_1, 
			("Await vol %d in state %d", 
				volRec->queues->semNum, waitstatus));

		if (waitList( &(volRec->tcbList), waitstatus))   {
			return esmFAILURE;
		} 
	}
}

static VOLREC *
FindVolRec (
	VOLID	volid,
	BOOL	updateCache
)
{
	CLASS_TYPE(VOLID)	*CLASS_ALIAS(volid) = (CLASS_TYPE(VOLID) *) &volid;
	VOLREC				*volRec;

	static struct {
		VOLID 	volid;
		VOLREC 	*ptr;
	} cached = { 0, NULL };

	TRPRINT(TR_IO, TR_LEVEL_1, ("volid:%d", volid));

	if(volid == cached.volid) {
		volRec = cached.ptr;
		if (volRec->volflags == VOL_FREE) {
			/* cache is out-of-date, verify this with the following */
#ifdef DEBUG
			volRec = volRecTable->Find(CLASS_ALIAS(volid));
			if (volRec != NULL) {
				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			}
#endif
			/* invalidate cache */
			cached.volid = 0;	
			cached.ptr = NULL;	
			volRec = NULL;
		}
	} else {
		volRec = volRecTable->Find(CLASS_ALIAS(volid));
	}

/* found: */
	if(volRec == NULL) {
		return NULL;
	}
	SM_ASSERT(LEVEL_3, volRec->volflags != VOL_FREE);

	if( WaitFor(volRec) == esmFAILURE )
		return NULL;

	if(updateCache) {
		cached.volid = volid;
		cached.ptr 	 = volRec;
	}
	return volRec;
}

void
io_RecycleVolNameRec(
	VOLNAMEREC *volNameRec
)
{
	SM_FREE_PTR(volNameRec->fmtinfo);

	volNameTable->Remove( volNameRec );
	volIdTable->Remove( volNameRec );
	volNameRecPool->Put( volNameRec );
}

void
IO_RemoveVolRec(
	VOLREC *volRec
)
{

	volRec->volflags &=  ~VOL_OPEN;
	volRecTable->Remove( volRec );
}
 
int
io_RemoveVolumeName(
	VOLID volid
)
{
	VOLNAMEREC *volNameRec;

	if( (volNameRec = io_FindVolNameById( volid )) == NULL ) {
		SM_ERROR(TYPE_USER, esmBADVOLID);
		return esmFAILURE;
	}
	/* had better not be in use ... */
	{
		VOLREC *volRec;

		volRec =  FindVolRec (volid, FALSE);
		if(volRec) {
			/* WaitFor returned esmNOERROR, so it's open */
			SM_ERROR(TYPE_USER, esmDISKMOUNTED);
			return esmFAILURE;
		}
	}
	io_RecycleVolNameRec(volNameRec);

	return esmNOERROR;
}

static BOOL
mountit(
	VOLNAMEREC 	*volNameRec
)
{
	VOLREC	*volRec;

	/* volNameRec has the name only, not the volid 
	*  Our job is to fill in the volid.
	*/

	SM_ASSERT(LEVEL_1, (formatvolRunning == FALSE));

	/* 
	*  The low-level mount requires a volRec for this
	*  thing, so let's create one.
	*/
	volRec = IO_NewVolRec();
	if(volRec == NULL) {
		return FALSE;
	}
	volRec->volNameRec = volNameRec;
	/* 
	 * mount and check for header consistency 
	 */
	if(IO_MountVolume(volRec, TRUE) != esmNOERROR) {
		fprintf(stderr, "Volume %s: ", volNameRec->volName.name);
		SM_ERROR(TYPE_LOG, esmNOSUCHVOLUME);
		return FALSE;
	}
	volRec->volid = volRec->header->volid;
	TRPRINT(TR_IO, TR_LEVEL_1, ("volume:%s has volid %d", volNameRec->volName.name, 
		volRec->volid));
	{
		VOLNAMEREC *vnr2 =io_FindVolNameById(volRec->volid);

		if( vnr2 && (vnr2 != volNameRec) ) {
			/* the volNameRec could already have the volid filled in
			 * by a duplicate entry in the mount table.  Nevertheless,
			 * check its volid
			 */
			fprintf(sm_ErrorStream, 
				"VOLID %d is both %s and %s\n",  volRec->volid,
					volNameRec->volName.name, vnr2->volName.name);
			SM_ERROR(TYPE_LOG, esmDUPVOLID);
			volNameRec->volid = 0;
			(void)IO_DismountVolume(volRec);
			return FALSE;
		}
		TRPRINT(TR_IO, TR_LEVEL_1, ("volid %d is new", volRec->volid));
	}

	volNameRec->volid = volRec->volid;

	{ 	/* 
		 * It's in the volume NAME table already; insert it in the 
		 * volume ID table 
		 */
		if( io_FindVolNameById(volRec->volid) == NULL) {
			SM_ERROR(TYPE_WARNING, esmINTERNAL); /* TODO: remove this 
			and this whole section: should already be in both */
			volIdTable->Insert( volNameRec );
		}
	}

	volRec->volflags |= VOL_OPEN;
	IO_InsertVolRec(volRec); /* only so that dismountit works */

	return FALSE;
}

static BOOL
dismountit(
	VOLREC 	*volRec,
	int		*error,
	BOOL	flush,
	BOOL	logit
)
{
	int err;

	if(flush) {
		err = bf_DismountVolume(volRec, logit);
	} else {
		err = IO_DismountVolume(volRec);
	}
	if(err != esmNOERROR) {
		*error = err;
		return TRUE;
	} else 
		return FALSE;
}

void
io_GetVolIds()
{
	BOOL	FPOSave = FastPathOnly;

	FastPathOnly = TRUE;
	SM_ASSERT(LEVEL_1, (PreventFastPath == 0));

	if(volNameTable == NULL) return;

	/* ForEachVolumeNameRec( 1, mountit ); */
#ifdef __GNUC__
	volNameTable->ForEach(1, mountit);
#else __GNUC__
	volNameTable->ForEach(1, (FOREACHFUNC)mountit);
#endif __GNUC__


	/* ForEachVolume( 3, dismountit, FALSE, FALSE ); */
#ifdef __GNUC__
	volRecTable->ForEach(3, dismountit, FALSE, FALSE);
#else __GNUC__
	volRecTable->ForEach(3, (FOREACHFUNC)dismountit, FALSE, FALSE);
#endif __GNUC__

	FastPathOnly = FPOSave;
}

VOLNAMEREC *
IO_NewVolNameRec( const char *volName, VOLID volid )
{
	VOLNAMEREC				*volNameRec;

	/* By the time we get here, io_FindVolName() has already
	 * chacked the length of the path name
	 */

	volNameRec = volNameRecPool->Get(); /* POOL_NOMORE_GETMORE */
	SM_ASSERT(LEVEL_1, (volNameRec != NULL)); 

	(void) strncpy(volNameRec->volName.name, volName, sizeof(volNameRec->volName.name));
	volNameRec->volid = volid;
	volNameTable->Insert( volNameRec );

	/* 
	 * WE ASSUME THAT THE CALLER ALREADY MADE SURE THAT THIS VOLID
	 * ISN'T ALREADY IN THE TABLE
	 */

	SM_ASSERT(LEVEL_1, volid != 0); /* for now */
	volIdTable->Insert( volNameRec );

	if(ioIsInitializing) {
		/* only bump this up if we are in the early phase of
		 * init (reading the config file).
		 * We can't change the value of NumVolumes once
		 * io_Init() has been called.
		 */

		NumVolumes ++;
	}
	return volNameRec;
}

VOLREC *
IO_NewVolRec()
{
	VOLREC *volRec;

	volRec = volRecPool->Get(); /* POOL_NOMORE_ISERROR */
	if(volRec==NULL) {
		SM_ERROR(TYPE_LOG, esmMOUNTTABLEFULL);
		return NULL;
	}
	return volRec;
}


 void
ForEachVolume(
	int		nargs,
	void 	*func, 
	void 	*arg1,
	void 	*arg2,
	void 	*arg3
)
{
	TRACE(TR_IO, TR_LEVEL_1);
	SM_ASSERT(LEVEL_1, (nargs <= 4));
	SM_ASSERT(LEVEL_1, (nargs >= 1));

	if(volRecTable == NULL) return;
		/* sometimes it's legitimate to call this before
		 * the table was initialized - e.g. from admin_ShutServer
		 * when an error is encountered while initializing
		 */

#ifdef __GNUC__
	volRecTable->ForEach(nargs, func, arg1, arg2, arg3);
#else __GNUC__
	volRecTable->ForEach(nargs, (FOREACHFUNC)func, arg1, arg2, arg3);
#endif __GNUC__
}

 void
ForEachVolumeNameRec(
	int		nargs,
	void 	*func, 
	void 	*arg1,
	void 	*arg2,
	void 	*arg3
)
{
	TRACE(TR_IO, TR_LEVEL_1);
	SM_ASSERT(LEVEL_1, (nargs <= 4));
	SM_ASSERT(LEVEL_1, (nargs >= 1));

	if(volNameTable == NULL) return;

#ifdef __GNUC__
	volNameTable->ForEach(nargs, func, arg1, arg2, arg3);
#else __GNUC__
	volNameTable->ForEach(nargs, (FOREACHFUNC)func, arg1, arg2, arg3);
#endif __GNUC__
}



VOLREC *
io_FindVolRec (
	VOLID	volid
)
{
	/* TODO: have it take an arg for BOOL updateCache  */
	return FindVolRec (volid, TRUE);
}

void
IO_InsertVolRec(
	VOLREC *volRec
)
{

	TRPRINT(TR_IO, TR_LEVEL_1, ("Inserting %d(%s)", 
		volRec->volNameRec->volid, volRec->volNameRec->volName.name));

	SM_ASSERT(LEVEL_1, (volRec->volid != 0));

	volRecTable->Insert( volRec );
}


void
IO_FreeVolRec(
	VOLREC *volRec
)
{
	SM_ASSERT(LEVEL_1, ((volRec->volflags & VOL_OPEN) == 0));

	volRec->volflags = VOL_FREE;
	/* Don't zero out the volLink yet; leave it around
	 * only for debugging purposes.  Let the Get func 
	 * reinitialize it.
	 */
	volRecPool->Put( volRec );
}

/*
 * Gets the volRec associated with the volid and link.
 * Don't update the cache.
 * Meant to be used by cl_* functions, so they don't 
 * interfere with other server activity (bf*, io*) that might
 * be going on with other volumes.
 */
VOLREC *
io_FindVolumeIdWithLink ( 
	VOLID			volid,
	LINK			*link = (LINK *)NULL
)
{
	if(link) {
		/* try the link's cache */
		if( volid == (VOLID) link->cached.volid )  {
			return  (VOLREC *) (link->cached.ptr);
		} 
	}
	return FindVolRec(volid, FALSE);
} 	/* io_FindVolRec */

static BOOL
countOpen(
	VOLREC  *volRec,
	int		*total
)
{
	if(volRec->volflags & VOL_OPEN)
		(*total)++;
	return FALSE;
}

int
io_NumOpenVolumes()
{
	int total = 0;

#ifdef __GNUC__
	volRecTable->ForEach(2, countOpen, &total);
#else __GNUC__
	volRecTable->ForEach(2, (FOREACHFUNC)countOpen, &total);
#endif __GNUC__

	/* not everything in the table is open: could be INIT, or QUIESCE, etc */
	
	return total;
}

 int
io_Final ()
{
	TRACE(TR_IO, TR_LEVEL_1);

	/* ForEachVolume( 1, dismountit ); */
#ifdef __GNUC__
	volRecTable->ForEach(3, dismountit, TRUE, FALSE);
#else __GNUC__
	volRecTable->ForEach(3, (FOREACHFUNC)dismountit, TRUE, FALSE);
#endif __GNUC__

	return esmNOERROR;
}


/*
 * io_CheckAllVolumes:
 *	Verify the the bitmaps on all volumes are consistent.  For
 *	now this means making sure the number of free pages matches the
 *	the number of bits in the free map.
 */

static BOOL
CheckVolume(
	VOLREC  *volRec,
	int		*error
)
{
	int err = 0;

	if(WaitFor(volRec) == esmFAILURE) {
		return FALSE;
	}
	if ((err = IO_Checker(volRec)) != esmNOERROR) {
		*error = err;
	}
	return FALSE;
}

 int
io_CheckAllVolumes ()
{
	int error = 0;
	TRACE(TR_IO, TR_LEVEL_1);

	ForEachVolume(1, CheckVolume, &error, 0, 0);

	return error;
}

#endif _IO_VOLUMETABLE_C_
