/*
 *   $RCSfile: io_WritePage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:46 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "queue_consist.h"


/*============================================================================*/
/* 
 * io_writepage:
 * 	Write one or more physical pages.
 *
 * IMPORTS:
 *      IO_WritePage()
 *
 * EXPORTS:	
 *
 * BUGS:
 *      IO_WritePage should be in-line expanded here for efficiency.
 *
 *
 * RETURNS:
 *	esmNOERROR
 *
 * ERROR RETURNS:
 *	eioNULLPIDPTR    - null PID pointer.
 *	eioNULLBUFPTR    - null memory buffer.
 *	eioVOLNOTMOUNTED - volume not mounted.
 *	eioINVALIDPID    - invalid page ID.
 *
 * SIDE EFFECTS:
 *	Data is written to the device.
 */
/*============================================================================*/


 int
io_WritePage (

	PID				*pid,
	char			*buffer,
	PAGE2SIZE		page2size
)
{

	VOLREC		*volRec;
	SHORTPID	page;
	int			count;


	TRPRINT(TR_IO, TR_LEVEL_1, ("writing page:%d page2size:%d", pid->page, page2size));

   { 	extern int PrintProgress;
		/*
		 * PrintProgress is in log_globals.h, but if we include it,
		 * we have to include all sorts of other log-related header
		 * files, too.  This is cleaner.
		 */

	   if (pid->page == 0) {
	       PRINT_PROGRESS(("writing header\n"));
		}
	}

	if ((volRec = io_FindVolRec(pid->volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	get the page number out of the pid
	 */
	page = pid->page;

	/*
	 *	check to make sure the pid is valid
	 */
	if ((page < 0) || (page > volRec->header->numPages))	{

		SM_ERROR(TYPE_USER, esmBADPID);
		return(esmFAILURE);
	}

	/*
	 * Figure out how many bytes to write, then
	 * call WritePage to actually do the vol write.
	 */
	count = RAISETWO(page2size);

	/*
	 *	do the actual write (of 1 page)
	 */
	return (IO_WritePage(volRec, &page, &count, &buffer, 1));
}
