/*
 *   $RCSfile: io_WritePageList.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "io_intfuncs.h"
#include "io_extfuncs.h"
#include "queue_consist.h"



 int
io_WritePageList (

	PID*			*pidList,
	char*			*bufferList,
	PAGE2SIZE*		page2sizeList,
	int				pageListLen
)
{

	VOLREC			*volRec;
	VOLID			volid;
	SHORTPID		pageList[MAX_IO_LIST];
	int				countList[MAX_IO_LIST];
	int				i;


	TRPRINT(TR_IO, TR_LEVEL_1, ("writing %d pages at page:%d page2size:%d", pageListLen, pidList[0]->page, page2sizeList[0]));


	/*
	 *	get a pointer to the first element of the vol array
	 */
	volid = pidList[0]->volid;
	if ((volRec = io_FindVolRec(volid)) == NULL)	{

		SM_ERROR(TYPE_USER, esmBADVOLID);
		return(esmFAILURE);
	}

	/*
	 *	Generate the read vector
	 */
	for (i = 0; i < pageListLen; i++) {

		/*
		 *	get the page number out of the pid
		 */
		pageList[i] = pidList[i]->page;

		/*
		 *	check to make sure the pid is valid
		 */
		if ((pageList[i] < 0) || (pageList[i] > volRec->header->numPages))	{

			SM_ERROR(TYPE_USER, esmBADPID);
			return(esmFAILURE);
		}

		/*
		 * Figure out how many bytes to write, then
		 * call WritePage to actually do the vol write.
		 */
		countList[i] = RAISETWO(page2sizeList[i]);

		/* make sure the page is in the same volume and is contiguous */
		SM_ASSERT(LEVEL_3, volid == pidList[i]->volid);
		if (i > 0) {
			SM_ASSERT(LEVEL_3, pageList[i-1]+BLOCKCOUNT(page2sizeList[i-1]) == pageList[i] );
		}	

	}

	/*
	 *	do the actual write
	 */
	return (IO_WritePage(volRec, pageList, countList, bufferList, pageListLen));
}
