/*
 *   $RCSfile: LG_PrintNode.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "lgobject.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "sm_macro.h"
#include "trace.h"
#include "error.h"
#include "lg_intfuncs.h"


/*
 * Print out the contents of a node (for debugging purposes).
 */

#ifdef DEBUG


 void 
LG_PrintNode (

	LGNODE			*nodePtr,	/* node to be printed out */
	int				indent,		/* number of tabs to indent */
	SHORTPID		pid 
)
{

    register int	i;


	/*
	 *	print out the pid of the node
	 */
	TRPRINT(TR_LG, TR_LEVEL_1, ("pid of page:%d height:%d", pid, nodePtr->header.height));

	/*
	 *	print out the contents of the node
	 */
    for (i = 0; i < nodePtr->header.numSlots; i++) {

		/*
		 *	print the node contents
		 */
		TRPRINT(TR_LG, TR_LEVEL_3, ("slot[%d]: cnt:%d, cum:%d pid:%d",
				i, LG_SlotCount(nodePtr, i),
				nodePtr->slot[i].count, nodePtr->slot[i].pid.page));

		/*
		 *	Check for consistnecy
		 */
		if ((nodePtr->header.height == 1) &&
			((LG_SlotCount(nodePtr, i) > LGDATA_PAGESIZE) ||
			 (LG_SlotCount(nodePtr, i) < 0) ||
			 (nodePtr->slot[i].count < 0))) 	{

			/*
			 *	Die
			 */
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
    }
}

#endif
