/*
 *   $RCSfile: lg_LogDataFormat.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "file.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "lg_extfuncs.h"
#include "thread_globals.h"
#include "log_globals.h"


 int
lg_LogDataFormat (

	PID					*pid,
	PAGEHASH			*pageHash 
)
{

	FORCEMARK			forceMark;
	LGDATAPAGE			*lgDataPage;
	LSN					lsn;
	LRC                 *lrc;
	LOGRECORDINFO		recordInfo;


	TRPRINT(TR_IO, TR_LEVEL_1, ("logging pid:%d", pid->page));

	/*
	 *	get a pointer to the slotted page
	 */
	lgDataPage = (LGDATAPAGE *) pageHash->bufFrame;

    /*
     *  get the lrc on the slotted page and initialize it
     */
    lrc = &(lgDataPage->header.lrc);
    INITIALIZE_LRC(lrc);

	/*
	 *	initialize the log information
	 */
	recordInfo.action		= LOG_ACTION_LG_DATA_PAGE_FORMAT;
	recordInfo.type			= LOG_REC_TYPE_USER;
	recordInfo.imageCount	= 0;
	recordInfo.actionPid    = pid;
    recordInfo.actionLRC    = lrc;
	recordInfo.flags		= REDO_ONLY|REDO_ALWAYS;

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		return(esmFAILURE);
	}

	/*
	 *	mark the page dependency
	 */
	DEPEND_LOG(pageHash, forceMark, &lsn, lrc);

	/*
	 *	return success
	 */
	return(esmNOERROR);
}
