/*
 *   $RCSfile: allocRuntimeClass.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"


 int
allocRuntimeClass (

	register TRANSREC	*transRec 
)
{

	register CLASSMAP	classMap;
	register int		current;


	TRPRINT(TR_LOCK, TR_LEVEL_1, ("classMap:%x", transRec->lockClassMap));

	/*
	 *	shift the classmap over one to get rid of the default class
	 */
	classMap = (transRec->lockClassMap >> 1);
	
	/*
	 *	search for an empty class
	 */
	for (current = 1; current < NUM_LOCK_CLASSES; current++, classMap >>= 1)	{

		/*
		 *	check to see if the current class is free
		 */
		if (!(classMap & 0x1))	{

			/*
			 *	mark the class in the transrec
			 */
			TRPRINT(TR_LOCK, TR_LEVEL_2, ("found free class:%d", current));
			transRec->lockClassMap |= (1 << current);
			return(current);
		}
	}

	/*
	 *	return failure
	 */
	return(esmFAILURE);
}
