/*
 *   $RCSfile: checkUpgradeDeadlock.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lock_globals.h"


 int
checkUpgradeDeadlock (

	register LOCKHEADER	*lockHeader,
	register LOCKMODE	heldMode,
	register LOCKMODE	requestMode 
)
{

	register LOCKENTRY	*upgradeEntry;
	register LOCKENTRY	*lockEntry;


	TRPRINT(TR_LOCK, TR_LEVEL_1, ("heldMode:%s requestMode:%s",
			GETMODE(heldMode), GETMODE(requestMode)));

	/*
	 *	Get a pointer to the bucket
	 */
	upgradeEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT( &(lockHeader->upgradeList) );

	/*
	 *	search down the list
	 */
	while (upgradeEntry != NULL)	{

		TRPRINT(TR_LOCK, TR_LEVEL_2, ("looking at tid:%x",
				GETTID(upgradeEntry->headerList.transRec)));

		/*
		 *	check the entry magic number
		 */
		CHECK_LOCKENTRY_MAGIC(upgradeEntry);

		/*
		 *	check to see if the requested mode by waiting upgrader is
		 *	compatible with the mode held by new upgrader
		 */
		if (!LM_Compat[heldMode][upgradeEntry->lockMode])	{

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("not compatible:%s",
					GETMODE(upgradeEntry->lockMode)));

			/*
			 *	get a pointer to the granted record of the upgrader
			 */
			lockEntry = (LOCKENTRY *) upgradeEntry->lockHeader;
			CHECK_LOCKENTRY_MAGIC(lockEntry);

			/*
			 *	check to see if the mode requested by new upgrader is
			 *	compatible with the mode held by the waiting upgrader
			 */
			if (!LM_Compat[lockEntry->lockMode][requestMode])	{
			
				TRPRINT(TR_LOCK, TR_LEVEL_2, ("not compatible:%s",
						GETMODE(lockEntry->lockMode)));
				fprintf(sm_ErrorStream, "SERVER: deadlock detected (due to lock upgrades) -- aborting a transaction.\n");
				fprintf(sm_ErrorStream, "  Deadlock from lock upgrades with transaction %d on lock %s\n",
					   lockEntry->headerList.transRec->tid,
					   printLockId(lockHeader->hashList.lockid)); 

				SM_ERROR(TYPE_USER, esmLOCKCAUSEDDEADLOCK);
				return(TRUE);
			}
		}

		/*
		 *	look at the next element in the list
		 */
		upgradeEntry = (LOCKENTRY *) NEXT_LIST_ELEMENT( &(upgradeEntry->headerList.list) );
	}

	/*
	 *	if we are here we all locks are compatible
	 */
	TRPRINT(TR_LOCK, TR_LEVEL_2, ("all locks compatible"));
	/*
	 * MJZ: 6-22-90 (change from true to false)
	 */
	return(FALSE);
}
