/*
 *   $RCSfile: findLockHeader.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lm_globals.h"


 LOCKHEADER
*findLockHeader (

	register LOCKID		*lockid,
	register int		slot 
)
{

	register LOCKHEADER	*lockHeader;


	TRPRINT(TR_LOCK, TR_LEVEL_2, ("lockid:%x:%x:%x", lockid->lockid.word1,
			lockid->lockid.word2, lockid->lockid.word3));

	/*
	 *	Get a pointer to the bucket
	 */
	lockHeader = (LOCKHEADER *)	FIRST_LIST_ELEMENT( &(LockHashTable[slot]) );

	/*
	 *	search down the list
	 */
	while (lockHeader != NULL)	{

		TRPRINT(TR_LOCK, TR_LEVEL_2, ("looking at lockid %x:%x:%x",
				lockHeader->hashList.lockid.lockid.word1,
						  lockHeader->hashList.lockid.lockid.word2,
						  lockHeader->hashList.lockid.lockid.word3));

		/*
		 *	check the lock header magic number
		 */
		CHECK_LOCKHEADER_MAGIC(lockHeader);

		/*
		 *	try to match the lock ids
		 */
		if ((lockid->lockid.word1 == lockHeader->hashList.lockid.lockid.word1) &&
			(lockid->lockid.word2 == lockHeader->hashList.lockid.lockid.word2) &&
			(lockid->lockid.word3 == lockHeader->hashList.lockid.lockid.word3))	{

			/*
			 *	return the current lock
			 */
			return(lockHeader);

		} else {

			/*
			 *	look at the next element in the list
			 */
			lockHeader = (LOCKHEADER *) NEXT_LIST_ELEMENT( &(lockHeader->hashList.list) );
		}
	}

	/*
	 *	if we are here we haven't found the item
	 */
	return(NULL);
}
