/*
 *   $RCSfile: getLockMode.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:53 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lm_globals.h"
#include "lock_globals.h"


 LOCKMODE
getLockMode (

	register TRANSREC	*transRec,
	register LOCKID		*lockid 
)
{

	register LOCKHEADER	*lockHeader;
	register LOCKENTRY	*lockEntry;


	TRPRINT(TR_LOCK, TR_LEVEL_1, ("object:%x:%x:%x tid:%x", lockid->lockid.word1,
			lockid->lockid.word2, lockid->lockid.word3, GETTID(transRec)));

	/*
	 *	Get a lock header structure
	 */
	if ((lockHeader = findLockHeader(lockid, hashLockId(lockid))) == NULL)	{

		TRPRINT(TR_LOCK, TR_LEVEL_2, ("object not locked"));
		return(NL);
	}

	/*
	 *	try to find the lock entry
	 */
	if ((lockEntry = findLockEntry( &(lockHeader->grantedList), transRec)) == NULL)	{

		TRPRINT(TR_LOCK, TR_LEVEL_2, ("object not locked"));
		return(NL);
	}

	/*
	 *	return the lock mode
	 */
	TRPRINT(TR_LOCK, TR_LEVEL_2, ("lockMode:%s", GETMODE(lockEntry->lockMode)));
	return(lockEntry->lockMode);
}
