/*
 *   $RCSfile: grantLockUpgrades.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lock_globals.h"


 void
grantLockUpgrades (

	register LOCKHEADER	*lockHeader 
)
{

	register LOCKENTRY	*masterEntry;
	register LOCKENTRY	*upgradeEntry;
	register LOCKENTRY	*nextEntry;


	TRACE(TR_LOCK, TR_LEVEL_1);

	/*
	 *	Get a pointer to the bucket
	 */
	upgradeEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT( &(lockHeader->upgradeList) );

	/*
	 *	search down the list
	 */
	while (upgradeEntry != NULL)	{

		TRPRINT(TR_LOCK, TR_LEVEL_2, ("looking at tid:%x mode:%s",
				GETTID(upgradeEntry->headerList.transRec),
						  GETMODE(upgradeEntry->lockMode)));

		/*
		 *	check the entry magic number
		 */
		CHECK_LOCKENTRY_MAGIC(upgradeEntry);

		/*
		 *	get a pointer to the master lock entry
		 */
		masterEntry = (LOCKENTRY *) upgradeEntry->lockHeader;
		CHECK_LOCKENTRY_MAGIC(masterEntry);

		/*
		 *	look at the next element in the list
		 */
		nextEntry = (LOCKENTRY *) NEXT_LIST_ELEMENT( &(upgradeEntry->headerList.list) );

		/*
		 *	remove the master from the granted list
		 *	and decrement the granted mode
		 */
		listRemove( &(masterEntry->headerList.list) );
		lockHeader->modeCount[masterEntry->lockMode]--;

		/*
		 *	check to see if the request is compatible
		 */
		if (lockCompatible(lockHeader, upgradeEntry->lockMode) )	{

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("lock compatible"));

			/*
			 *	update the mode counts in the lock header
			 */
			lockHeader->modeCount[upgradeEntry->lockMode]++;

			/*
			 *	Check to see if there is a new supremum
			 */
			if (!LM_Compat_Upgrade[upgradeEntry->lockMode][lockHeader->supremum])	{

				TRPRINT(TR_LOCK, TR_LEVEL_2, ("new supremum:%s",
						GETMODE(upgradeEntry->lockMode)));
				lockHeader->supremum = LM_Supremum[upgradeEntry->lockMode][lockHeader->supremum];
			}

			/*
			 *	mark the new mode of the lock
			 */
			masterEntry->lockMode = upgradeEntry->lockMode;

			/*
			 *	mark the entry as no upgrade pending
			 */
	SM_ASSERT(LEVEL_1, (masterEntry->flags & LOCK_UPGRADE_PENDING));
			masterEntry->flags &= ~LOCK_UPGRADE_PENDING;

			/*
			 *	free the upgrade entry
			 */
			freeLockEntry(upgradeEntry, esmNOERROR);

		} else {

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("lock not compatible"));

			/*
			 *	put the original entry on the granted list
			 *	and restore the original lock mode count
			 */
			lockHeader->modeCount[masterEntry->lockMode]++;
		}

		/*
		 *	put the original entry on the granted list
		 *	and restore the original lock mode count
		 */
		listPush( &(lockHeader->grantedList), &(masterEntry->headerList.list) );

		/*
		 *	set the next upgrade entry
		 */
		upgradeEntry = nextEntry;
	}
}
