/*
 *   $RCSfile: grantLockWaiters.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lock_globals.h"


 void
grantLockWaiters (

	register LOCKHEADER	*lockHeader 
)
{

	register LOCKENTRY	*lockEntry;
	register LOCKMODE	lockMode;


	TRACE(TR_LOCK, TR_LEVEL_1);

	/*
	 *	search down the list
	 */
	while (LIST_NOT_EMPTY( &(lockHeader->waitList) ))	{

		/*
		 *	get a pointer to the lock entry
		 */
		lockEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT( &(lockHeader->waitList) );
		CHECK_LOCKENTRY_MAGIC(lockEntry);

		/*
		 *	put the requested lock mode in a register
		 */
		lockMode = lockEntry->lockMode;
		TRPRINT(TR_LOCK, TR_LEVEL_2, ("supremum:%s lockMode:%s",
				GETMODE(lockHeader->supremum), GETMODE(lockMode) ));

		/*
		 *	check to see if the request can be granted
		 */
		if (LM_Compat[lockHeader->supremum][lockMode])	{

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("modes are compatible"));	

			/*
			 *	move the entry to the lock grant list
			 */
			listMovePush( &(lockHeader->grantedList), &(lockEntry->headerList.list) );

			/*
			 *	move the entry onto the transaction granted list
			 */
			listMovePush( &(lockEntry->headerList.transRec->lockGrantedList),
						  &(lockEntry->transList) );

			TRPRINT(TR_LOCK, TR_LEVEL_1, 
				("lockEntry 0x%x changed from %d -> %d", lockEntry->flags,
					LOCK_GRANTED));
			/*
			 *	record that the state is granted
			 */
			lockEntry->flags = LOCK_GRANTED;

			/*
			 *	notify the thread waiting
			 */
			notify( &(lockEntry->threadList), esmNOERROR, esmNOERROR);

			/*
			 *	increment the mode count in the lock header
			 */
			lockHeader->modeCount[lockMode]++;
			lockHeader->lockCount++;

			/*
			 *	take care of the new supremum mode
			 */
			if (!LM_Compat_Upgrade[lockMode][lockHeader->supremum])	{
				
				TRPRINT(TR_LOCK, TR_LEVEL_2, ("new supremum:%s", GETMODE(lockMode)));
				lockHeader->supremum = LM_Supremum[lockMode][lockHeader->supremum];
			}

		} else {

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("lock not compatible"));

			/*
			 *	break out of the loop for fair FCFS scheduling
			 */
			break;
		}
	}
}
