/*
 *   $RCSfile: lm_CheckPage.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:53 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"


#define RELEASELIMIT 64

#ifdef LM_DEBUG
#define LM_TRACE(x)		printf x
#else
#define LM_TRACE(x)
#endif


int lm_CheckPage(
	TRANSREC*	transRec,
	PID*		pid,
	FOUR*		limit 
)
{
	LOCKID		lockid;
	LOCKHEADER*	lockHeader;
	int			slot;
	FOUR		tmp;

	lockid.lockid.word1 = lockid.lockid.word2 = lockid.lockid.word3 = 0;

	lockid.page.type = PAGE_LOCK;
	lockid.page.pid.volid = pid->volid;
	lockid.page.pid.page = pid->page;

	if (transRec->transState != T_ACTIVE)  {

		SM_ERROR(TYPE_USER, esmBADTRANSID);
		return esmFAILURE;
		}

	slot = hashLockId(&lockid);

	/*
	 * 	Make sure page is locked
	 */
	lockHeader = findLockHeader(&lockid, slot);
	SM_ASSERT(LEVEL_1, lockHeader);

	if ((LIST_NOT_EMPTY( &(lockHeader->waitList) )) || 
						(LIST_NOT_EMPTY( &(lockHeader->upgradeList)))) {
		/*
		 *	Someone is waiting on the page
		 */
		*limit = 0;
		LM_TRACE(("Waiters exist for page %d\n", pid->page));
		return esmNOERROR;
		}

	/*
	 *	Nobody is waiting -- double releaseLimit and return
	 */
	tmp = (*limit) * 2;
	if (tmp > RELEASELIMIT)  tmp = RELEASELIMIT;

	LM_TRACE(("page %d, limit %d --> %d\n", pid->page, *limit, tmp));
	*limit = tmp;

	return esmNOERROR;
}
