/*
 *   $RCSfile: lockCompatible.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:51 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "lm_intfuncs.h"
#include "lm_extfuncs.h"
#include "lock_globals.h"


 int
lockCompatible (

	register LOCKHEADER	*lockHeader,
	register LOCKMODE	requestMode 
)
{

	TRPRINT(TR_LOCK, TR_LEVEL_1, ("requestMode:%s", GETMODE(requestMode)));

	/*
	 *	check to see how many locks are held
	 */
	if (lockHeader->lockCount < NUM_LOCK_MODES)	{

		register LOCKENTRY	*lockEntry;

		/*
		 *	Get a pointer to the bucket
		 */
		lockEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT( &(lockHeader->grantedList) );

		/*
		 *	search down the list
		 */
		while (lockEntry != NULL)	{

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("looking at tid:%x lockMode:%s",
							  GETTID(lockEntry->headerList.transRec),
							  GETMODE(lockEntry->lockMode)));

			/*
			 *	check the entry magic number
			 */
			CHECK_LOCKENTRY_MAGIC(lockEntry);

			/*
			 *	check to see if the lock is compatible
			 */
			if (!LM_Compat[lockEntry->lockMode][requestMode])	{

				TRPRINT(TR_LOCK, TR_LEVEL_2, ("not compatible:%s",
						GETMODE(lockEntry->lockMode)));
				return(FALSE);
			}

			/*
			 *	look at the next element in the list
			 */
			lockEntry = (LOCKENTRY *) NEXT_LIST_ELEMENT( &(lockEntry->headerList.list) );
		}

		/*
		 *	if we are here we all locks are compatible
		 */
		TRPRINT(TR_LOCK, TR_LEVEL_2, ("all locks compatible"));
		return(TRUE);

	} else {

		register LOCKMODE	lockMode;
		register UONE		*lockCount;

		/*
		 *	search starting at one less than the current supremum
		 */
		for (lockMode = lockHeader->supremum,
			 lockCount = &(lockHeader->modeCount[lockMode]);
			 lockMode > NL;
			 lockMode--, lockCount--)	{

			TRPRINT(TR_LOCK, TR_LEVEL_2, ("checking mode:%s count:%d",
					GETMODE(lockMode), *lockCount));

			/*
			 *	check to see if the count is non zero
			 */
			if (*lockCount != 0)	{

				/*
				 *	check to see if the lock is compatible
				 */
				if (!LM_Compat[lockMode][requestMode])	{

					TRPRINT(TR_LOCK, TR_LEVEL_2, ("not compatible:%s", GETMODE(lockMode)));
					return(FALSE);
				}
			}
		}

		/*
		 *	if we get to here we are compatible
		 */
		TRPRINT(TR_LOCK, TR_LEVEL_2, ("all locks compatible"));
		return(TRUE);
	}
}
