/*
 *   $RCSfile: abortLogSpace.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "trans_extfuncs.h"


 void
abortLogSpace (

	OPENLOG		*openLog,
	int			spaceNeeded 
)
{

	SHORTPID	limitPid;
	TRANSREC	*transRec;
	TRANSREC	*nextTransRec;
	int			pages;
	BOOL		wrap;


	TRPRINT(TR_LOG, TR_LEVEL_1, ("spaceNeeded:%d", spaceNeeded));

	/*
	 *	check to see if the list is empty
	 */
	SM_ASSERT(LEVEL_3, LIST_NOT_EMPTY( &(openLog->activeList) ));

	/*
	 *	calculate the number of pages that need to be aborted
	 */
	pages	 = ALIGN(spaceNeeded, openLog->pageSize) >> openLog->page2size;
	limitPid = (openLog->tailPid + pages) % openLog->filePages;
	wrap	 = (BOOL) ((openLog->tailPid + pages) / openLog->filePages);
	TRPRINT(TR_LOG, TR_LEVEL_2, ("pages:%d limit page:%d wrap:%d", pages, limitPid, wrap));

	/*
	 *	get a pointer to the oldest transaction hanging off the list
	 */
	transRec = (TRANSREC *) FIRST_LIST_ELEMENT( &(openLog->activeList) );

	/*
	 *	while the transactions are on the same page abort them
	 */
	while (transRec != NULL)	{
		
		/*
		 *	check the magic number of the transaction record
		 */
		CHECK_TRANSREC_MAGIC(transRec);

		/*
		 *	check to see if the first page of the transaction falls within
		 *	the target range that is specified by space
		 */
		if (wrap)	{

			/*
			 *	check two ranges
			 */
			if ((transRec->firstLogPid > openLog->tailPid) ||
				(transRec->firstLogPid <= limitPid))	{

				/*
				 *	save the next transaction pointer
				 */
				nextTransRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(transRec->logActiveList) );

				/*
				 *	abort this transaction
				 */
				fprintf(sm_ErrorStream, "SERVER: transaction %d is being aborted due to lack of log space\n", transRec->tid); 
				internalAbort(transRec, esmNOFREELOGSPACE);

				/*
				 *	restore the next record
				 */
				transRec = nextTransRec;

				/*
				 *	go to the end of the loop
				 */
				continue;
			}

		} else {

			/*
			 *	check to see if the first page falls in the target range
			 */
			if ((transRec->firstLogPid > openLog->tailPid) &&
				(transRec->firstLogPid <= limitPid))	{

				/*
				 *	save the next transaction pointer
				 */
				nextTransRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(transRec->logActiveList) );

				/*
				 *	abort this transaction
				 */
				fprintf(sm_ErrorStream, "SERVER: transaction %d is being aborted due to lack of log space\n", transRec->tid); 
				internalAbort(transRec, esmNOFREELOGSPACE);

				/*
				 *	restore the next record
				 */
				transRec = nextTransRec;

				/*
				 *	go to the end of the loop
				 */
				continue;
			}
		}

		/*
		 *	if we are here we should return
		 */
		break;
	}
}
