/*
 *   $RCSfile: asyncForceDirtyPages.c,v $  
 *   $Revision: 1.2 $  
 *   $Date: 1996/05/04 23:51:53 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "util_funcs.h"
#include "bf_extfuncs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "thread_funcs.h"
#include "bf_globals.h"
#include "io_globals.h"
#include "thread_globals.h"
#include "log_globals.h"
#include "trans_globals.h"

/*
 *	AsyncFlushPage is forked by asyncForceDirtyPages to flush
 *	a page at certain times.
 */
#if defined(mips) || defined(sparc) || defined(_AIX) || defined(hpux) || defined(linux)
 void
asyncFlushPage ()
#elif !(defined(mips) || defined(sparc) || defined(vax) || defined(_AIX) || defined(hpux) || defined(linux))
    not supported
#else
    /* this else should have the not supported code, but the
       ultrix cpp is broken */
#endif

{
	/*
	 *	This variable holds an argument passed the this function
	 *	when the thread was forked.
	 */
#if defined(mips) || defined(sparc) || defined(_AIX) || defined(hpux) || defined(linux)
    PID		pid;
#elif defined(vax)
	/* not needed */
#elif !(defined(mips) || defined(sparc) || defined(vax) || defined(_AIX) || defined(hpux) || defined(linux))
    not supported
#else
    /* this else should have the not supported code, but the
       ultrix cpp is broken */
#endif 

	PAGEHASH	*pageHash;

	TRACE(TR_LOG, TR_LEVEL_1);

	/*
	 *	get the page from the tcb buffer
	 */
#ifdef mips
    pid.page = Active->currentBuf[JB_A0];
    pid.volid = Active->currentBuf[JB_A0+1];
#elif defined(sparc) || defined(_AIX) || defined(hpux) || defined(linux)
	/*
	 *	Sparc and other architectures
	 */
    pid.page = ((TCB *) Active)->args[0];
    pid.volid = (VOLID) Active->args[1];
#elif defined(vax)
	/* not needed */
#elif !(defined(mips) || defined(sparc) || defined(vax) || defined(_AIX) || defined(hpux) || defined(linux))
    not supported
#else
    /* this else should have the not supported code, but the
       ultrix cpp is broken */
#endif 

	if ((pageHash = bf_FindPage(&pid, BF_SEM)) != NULL)     {
		bf_ForcePage(pageHash, NOFLAGS);
		signalSemaphore(&(pageHash->semaphore));
	}

	threadRestart();
}


/*
 *  collectAndForcePage takes pages from asyncForceDirtyPages
 *  and attempts to force them in chunks to the disk process.
 *	It returns true if the thread could have been switched by
 *	a flush and does not add the page to the list in that case.
 */
static
BOOL
collectAndForcePage(PAGEHASH* pageHash)
{
	static PAGEHASH*	pageHashList[MAX_IO_LIST];
	static int			numPages = 0;

	SHORTPID			nextContig;

	TRPRINT(TR_LOG, TR_LEVEL_1, ("collected:%d", numPages));

	if (pageHash == NULL) {
		/* flush all */
		if (numPages > 0) {
			bf_ForcePageList(pageHashList, numPages, BF_SEM);
			numPages = 0;
		}
		return FALSE;
	}

	if (numPages == 0) {
		pageHashList[numPages] = pageHash;
		numPages++;
		STORE_THREAD_SWITCH(Active);
		return FALSE;
	}

	SM_ASSERT(LEVEL_3, numPages <= MAX_IO_LIST && numPages > 0);
	CHECK_THREAD_SWITCH(Active);

	/*
	 *	Make sure the list is not full and the pages are contiguous
	 *	and on the same volume.
	 */
	nextContig = pageHashList[numPages-1]->hashList.hashPid.page +
				 BLOCKCOUNT(pageHashList[numPages-1]->page2size);
	if ((numPages == MAX_IO_LIST) ||
		(pageHash->hashList.hashPid.page != nextContig) ||
		(pageHash->hashList.hashPid.volid !=
		 pageHashList[numPages-1]->hashList.hashPid.volid)) {

		bf_ForcePageList(pageHashList, numPages, BF_SEM);
		numPages = 0;
		STORE_THREAD_SWITCH(Active);
		return TRUE;  // lost CPU in bf_ForcePageList
	}

	pageHashList[numPages] = pageHash;
	numPages++;

	SM_ASSERT(LEVEL_3, numPages <= MAX_IO_LIST);

	return FALSE;
}



static PAGEHASH *PrevOldestPage = NULL;

 void
asyncForceDirtyPages(
    int			pageCount,
	LSN			checkpointLSN
)
{

	register DIRTYPAGEINFO	*dirtyInfo;
	register PAGEHASH		*pageHash;
	PAGEHASH				*oldestPage;
	register OPENLOG		*openLog;
	BOOL					pageNotFlushed = FALSE;
	LSN						oldestNotFlushed;
	BOOL					lostCPU;

	TRPRINT(TR_LOG, TR_LEVEL_1, ("pageCount:%d", pageCount));

	/*
	 *	Check parameters
	 */
	SM_ASSERT(LEVEL_1, checkpointLSN.offset != NULL_LSN);

	/*
	 *	get a pointer to the openlog
	 */
	openLog = &OpenLog;

	/* start with the highest LSN */
	oldestNotFlushed = checkpointLSN;	

	/* make sure I/O is asynchronous */
	PREVENT_FAST_IO();

	/*
	 *	initialize the count and get the pointer to
	 *	the first bucket on the page hash table
	 */
	for (dirtyInfo = DirtyPageTable; pageCount > 0; dirtyInfo++, pageCount--)	{

		/*
		 *	This "do" loop is repeated if the thread is switched
		 *	between the time the flush request is given and the
		 *	actual flush occurs.
		 */
		do {
			/*
			 *	get a pointer to the first element in the bucket
			 */
			if ((pageHash = bf_FindPage( &(dirtyInfo->pid), NOFLAGS )) == NULL)	{

				break;
			}

			/*
			 *	Make sure no one holds the semaphore on the page
			 */
			if (SEMAPHORE_IS_HELD(&(pageHash->semaphore)) ) {
				
				/*
				 *	Just skip this page, since we would have to block
				 *	on the semaphore
				 */
				pageNotFlushed = TRUE;
				if (compareLSN(&(pageHash->lsn), &oldestNotFlushed) < 0) {
					oldestNotFlushed = pageHash->lsn;
					oldestPage = pageHash;
				}
				break;
			}

			/*
			 *	Make sure the page has no pending operations 
			 */
			if (pageHash->pendingOpCount != 0) {

				/*
				 *	Just skip this page, since a page with pending 
				 * 	operations cannot be written
				 *	since it would not reflect pending operations.
				 */
				pageNotFlushed = TRUE;
				if (compareLSN(&(pageHash->lsn), &oldestNotFlushed) < 0) {
					oldestNotFlushed = pageHash->lsn;
					oldestPage = pageHash;
				}
				break;
			}

			/*
			 *	The btree index code requires that fixed pages
			 *	not be forced to disk.  But, volume header pages
			 *	are always fixed, so they are an exception.
			 */
			if (bf_CheckFixed(pageHash) && pageHash->hashList.hashPid.page != HEADERADDR) {
				/*
				 *	Just skip this page, since its fixed and not
				 *	the volume header
				 */
				pageNotFlushed = TRUE;
				if (compareLSN(&(pageHash->lsn), &oldestNotFlushed) < 0) {
					oldestNotFlushed = pageHash->lsn;
					oldestPage = pageHash;
				}
				break;
			}

			/*
			 *	force the page
			 */
			lostCPU = collectAndForcePage(pageHash);
		} while (lostCPU);

		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2,
				("pid:%d", pageHash->hashList.hashPid.page));
	}
	collectAndForcePage(NULL);  /* done, so flush any still cached */

	/* allow server to perform I/O itself (synchrounous) */
	ALLOW_FAST_IO();

	/*
	 *	If all pages were flushed, record this checkpoint's lsn
	 *	It will be used in assigning firstLSNs to pages
	 */
	if (!pageNotFlushed) {
		LastFlushedCheckpoint = checkpointLSN;
		PRINT_PROGRESS( ("CHECKPOINT LSN:%d:%d sucessfuly flushed\n", 
			   checkpointLSN.offset, checkpointLSN.wrapCount) );
		PrevOldestPage = NULL;
	} else {

		/*
		 *	Not everything was flushed, so the oldest dirty page
		 *	determines how much log space was freed
		 */
		LastFlushedCheckpoint = oldestNotFlushed;
		PRINT_PROGRESS( ("NOTE: checkpoint LSN:%d:%d NOT flushed - stopped at %d\n", 
			   checkpointLSN.offset, checkpointLSN.wrapCount, LastFlushedCheckpoint.offset) );
		if (PrevOldestPage != oldestPage) {
			PrevOldestPage = oldestPage;
		} else {
			FOUR args[2];
			/* we need to forcibly flush this page */
			args[0] = PrevOldestPage->hashList.hashPid.page;
			args[1] = PrevOldestPage->hashList.hashPid.volid;
			threadFork(T_PARENT, (PFI) asyncFlushPage, 2, args);
			PrevOldestPage = NULL;
		}
	}

	/*
	 *	mark the checkpoint as done
	 */
	openLog->flags ^= LOG_CHECKPOINT_IN_PROGRESS;

	/*
	 *	signal any waiters
	 */
	notify( &(openLog->checkPointList), esmNOERROR, esmNOERROR);
}


/*
 *	asyncForceDirtyPagesThread() is a thread which is always waiting
 *	to asynchrounously forces dirty pages after a checkpoint.
 */

 void
asyncForceDirtyPagesThread ()
{

	TRPRINT(TR_LOG, TR_LEVEL_1, ("setting up log preflush thread"));

	initializeList(&DirtyPageFlushWaitList);

	while (TRUE) {   /* forever */

		/* wait for activation */
		if (waitList( &DirtyPageFlushWaitList, THREAD_PAGEFLUSH_WAIT)) {
			SM_ERROR(TYPE_FATAL, Active->errno);
		}

		/* make sure initiator set the flag */
		SM_ASSERT(LEVEL_3, (OpenLog.flags & LOG_CHECKPOINT_IN_PROGRESS));
		asyncForceDirtyPages(DirtyPageTableCount, OpenLog.checkPointLSN);
	}

}


