/*
 *   $RCSfile: checkPointTrans.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "thread_globals.h"
#include "log_globals.h"
#include "trans_globals.h"


 int
checkPointTrans (

	register CHECKTRANS	*transList 
)
{

	register TRANSREC	*transRec;
	register int		transCount;


	TRACE(TR_LOG, TR_LEVEL_1);

	/*
	 *	initialize the count and get the pointer to the first transaction
	 */
	transCount = 0;
	transRec = (TRANSREC *) FIRST_LIST_ELEMENT( &(ActiveTransList) );

	/*
	 *	run down the list of active transactions
	 */
	while (transRec != NULL)	{

		/*
		 *	check the transrec magic number
		 */
		CHECK_TRANSREC_MAGIC(transRec);

		/*
		 *	record the information from the transaction
		 *  if the transaction is one that will need recovery
		 */
		switch (transRec->transState)	{

			case T_ACTIVE:
			case T_LATENT:
			case T_QUIESCE:
			case T_ABORT:

				transList->state = T_ACTIVE;
				break;

			case T_COMMIT:
				/*
				* commented out for minor efficiency:
				* do not include committed transactions in the checkpoint 
				transList->state = T_COMMIT;
				break;
				*/

				/*
				 *	for distr trans 
				 */
				if (transRec->numServers == 0) {

					goto nextTransRec; /* continue */
				}
				else {
					transList->state = T_COMMIT;
				}
				break;

			case T_PREPARED:

				transList->state = T_PREPARED;
				break;

			case T_END:
				
				transList->state = T_END;
				break;
				
		}
		transList->firstLSN	= transRec->firstLSN;
		transList->lastLSN	= transRec->lastLSN;
		transList->tid		= transRec->tid;
		transList->nextUndoLSN = transRec->nextUndoLSN;
		/*
		 *	for distr trans
		 */
		transList->numServers = transRec->numServers;
		transList->prepareLSN = transRec->prepareLSN;

		TRPRINT(TR_LOG, TR_LEVEL_2, ("tid:%x", GETTID(transRec)));

		/*
		 *	increment the count of transactions
		 */
		transCount++;

		/*
		 *	go to the next element
		 */
		transList++;

	nextTransRec:
		transRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(transRec->activeTransList) );
	}

	/*
	 *	return the count of transactions
	 */
	return(transCount);
}
