/*
 *   $RCSfile: checkpointThread.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "thread_globals.h"
#include "io_globals.h"
#include "log_globals.h"


/*
 *	checkpointThread() is a thread which is always waiting
 *	to perform a checkpoint.  It is activated by activateCheckpoint().
 */

 void
checkpointThread ()
{

	TRPRINT(TR_LOG, TR_LEVEL_1, ("setting up log preflush thread"));

	initializeList(&CheckpointWaitList);
	CheckpointSyncForce = FALSE;

	while (TRUE) {   /* forever */

		/* wait for activation */
		if (waitList( &CheckpointWaitList, THREAD_CHECKPOINT_WAIT)) {
			SM_ERROR(TYPE_FATAL, Active->errno);
		}


		/* 
		 *	start the checkpoint
		 */

		PREVENT_FAST_IO();

		/* make sure initiator set the flag */
		SM_ASSERT(LEVEL_3, (OpenLog.flags & LOG_CHECKPOINT_IN_PROGRESS));
		if (checkPoint(CheckpointSyncForce) != esmNOERROR) {
			SM_ERROR(TYPE_CRASH, Active->errno);
		}
		CheckpointSyncForce = FALSE;
		ALLOW_FAST_IO();
	}

}
