/*
 *   $RCSfile: compareLRC.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#ifdef INIT_LRC_IS_LSN
#	include "log_globals.h"
#endif

 int
compareLRC (

	register LRC	*lrc1,
	register LRC	*lrc2 
)
{

	TRPRINT(TR_LOG, TR_LEVEL_1, ("lrc1:%d:%d lrc2:%d:%d",
			lrc1->wrapCount, lrc1->count, lrc2->wrapCount, lrc2->count));
#ifdef DEBUG

#ifdef INIT_LRC_IS_LSN

	/*
	 *	For debugging only
	 */
	SM_ASSERT(LEVEL_1, (lrc1->wrapCount <= 100000) );
	SM_ASSERT(LEVEL_1, (lrc2->wrapCount <= 100000) );

#else

	/*
	 *	For debuggin purposes, consider any non-zero wrap count
	 *	and any count > 200000 to indicate an error
	 */
	SM_ASSERT(LEVEL_1, (lrc1->wrapCount == 0) && (lrc1->count <= 200000));
	SM_ASSERT(LEVEL_1, (lrc2->wrapCount == 0) && (lrc2->count <= 200000));
#endif /* INIT_LRC_IS_LSN */

#endif

	/*
	 *	compare the lrcs
	 */
	if (lrc1->wrapCount < lrc2->wrapCount)	{

		/*
		 *	lrc 1 is less
		 */
		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("lrc 1 is less"));
		return(-1);

	} else if (lrc1->wrapCount == lrc2->wrapCount)	{

		/*
		 *	check to see if the count are equal
		 */
		if (lrc1->count < lrc2->count)	{

			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("lrc 1 is less"));
			return(-1);

		} else if (lrc1->count == lrc2->count)	{

			/*
			 *	lrcs are equal
			 */
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("lrcs are equal"));
			return(0);
		
		} else {

			/*
			 *	lrc 1 is greater
			 */
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("lrc 2 is less"));
			return(1);
		}

	} else {

		/*
		 *	lrc 1 is greater
		 */
		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("lrc 2 is less"));
		return(1);
	}
}
