/*
 *   $RCSfile: forceLog.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:49 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "disk.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "bf_extfuncs.h"
#include "io_extfuncs.h"
#include "disk_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "log_globals.h"
#include "trans_globals.h"


auditLogPage(GROUPLINK *bufLink)
{
	OPENLOG			*openLog;
	LOGPAGEHDR		*pageHeader;
	LOGPAGEHDR		*endHeader;
	LOGRECORDHDR	*current;
	int				offset;
	int				i;

	/*
	 *	Get the log information
	 */
	openLog = &OpenLog;

	pageHeader = (LOGPAGEHDR*) bufLink->bufFrame;
	endHeader = (LOGPAGEHDR *)(((char *) pageHeader) + openLog->lastUsableByte);
	if (CHECK_LOGPAGE_MAGIC(pageHeader) || 
		CHECK_LOGPAGE_MAGIC(endHeader) ) {

		SM_ERROR(TYPE_FATAL, esmBADLOGPAGEHEADER);
		return(esmFAILURE);
	}

	if (pageHeader->lastRecord == FIRST_LSN) {
		current = (LOGRECORDHDR*) (bufLink->bufFrame + pageHeader->lastRecord);
		if (CHECK_LOGRECORD_MAGIC(current)) {
			/* 
			 *	Then entire page is an image and contains no log record
			 *	header.
			 */
			return(esmNOERROR);
		}
	}

	if (pageHeader->lastRecord >= FIRST_LSN) {

		current = (LOGRECORDHDR*) (bufLink->bufFrame + pageHeader->lastRecord);
		if (CHECK_LOGRECORD_MAGIC(current)) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);	
		}

		offset = (int) (pageHeader->lastRecord + current->length);

		if (offset >= openLog->lastUsableByte) {
			/* nothing to check */
		} else {
			/* all bytes left on page should be zero */
			for (i=offset; i<openLog->lastUsableByte; i++) {
				if (bufLink->bufFrame[i] != '\0') {
					SM_ERROR(TYPE_FATAL, esmINTERNAL);
				}
			}
		} 
	}

	/* return success */
	return(esmNOERROR);
}



 int
forceLog (

	register FORCEMARK	forceMark 
)
{

	register OPENLOG	*openLog;
	register PAGEHASH	*pageHash;
	register GROUPLINK	*groupLink;
	register VOLREC		*volRec;
	BOOL				logLatched;
	PAGEHASH*			pageHashList[MAX_IO_LIST];
	int					pageHashListLen;

	TRPRINT(TR_LOG, TR_LEVEL_1, ("target force mark:%d", forceMark));

	/* get a register pointer to the openlog structure */
	openLog = &OpenLog;

again:

	logLatched = FALSE;

	/* check to see if we need to force */
	if (openLog->forceMark >= forceMark)	{

		/* force criteria has already been met */
		return(esmNOERROR);
	}

	SM_ASSERT(LEVEL_2, TransInfo.logState != L_OFFLINE);

	/*
	 *	go through the replaceable queue forcing pages
	 */
	pageHashListLen = 0;
	groupLink = (GROUPLINK *) FIRST_LIST_ELEMENT( &(openLog->writeGroup->unfixedList));
	while ( groupLink != NULL)  {

		/* check the grouplink magic */
		CHECK_GROUPLINK_MAGIC(groupLink);

		auditLogPage(groupLink);

		/* get a pointer to the pagehash structure */
		pageHash = groupLink->pageHash;
		CHECK_PAGEHASH_MAGIC(pageHash);
		TRPRINT(TR_LOG, TR_LEVEL_2, ("buf mark:%d log mark:%d",
				pageHash->forceMark, openLog->forceMark));

		/* check to make sure this is a log buffer */
		SM_ASSERT(LEVEL_3, PAGE_IS_LOG(pageHash));

		/*
		 *	see if some other thread holds the semaphore on this page
		 */
		if (pageHash->semaphore.holder != NULL)	{	
			
			/* Flush what we have to far */
			if (pageHashListLen > 0) bf_ForcePageList(pageHashList, pageHashListLen, BF_SEM);
			pageHashListLen = 0;

			/* attempt to get a semaphore on the page */
			if (waitSemaphore( &(pageHash->semaphore) ))	{	

				/* check to see if the page is gone */
				if (Active->errno != esmPAGEGONE)	{
					SM_ERROR(TYPE_FATAL, Active->errno);
				}
			}
			
			/*
			 *	since the semaphore is now obtainable, release it
			 *	and start all over.  But, before starting over
			 *	give up the cpu so that any signaled thread can run.
			 *	Otherwise two threads can get stuck in this loop.
			 *	
			 *	The log should not be latched since this would be
			 *	beyond the end of the log.
			 */
			signalSemaphore( &(pageHash->semaphore) );
			SM_ASSERT(LEVEL_3, !logLatched);
			Sleep(SLEEP_DONT_SLEEP);
			goto again;

		} else {


			/* 
			 *	see if page is dirty and needs to be forced
			 *	It is ok to do these checks
			 *	since we will eventually get the semaphore before
			 *	any other thread
			 */
			if ((pageHash->forceMark <= forceMark) || (pageHash->firstMark <= forceMark))	{
			
				/*
				 *	If this page is the current end of the log, the
				 *	we must grab a latch and the writeSemaphore on the 
				 *	log so that no one will be changing it.  We don't
				 *	grab it if we already have it.
				 */
				if (pageHash == openLog->tailLink->pageHash &&
					openLog->writeSemaphore.holder != Active) {
			
					if (waitLatch(&(openLog->logLatch), SHARE_LATCH)) {
						return(esmFAILURE);
					}

					/*
					 *	Make sure no other thread has the semaphore.
					 */
					if (openLog->writeSemaphore.holder != NULL) {

						/* 
						 * another thread has it, so flush what we have
						 * so far and then wait for the semaphore.
						 */
						if (pageHashListLen > 0) 
							bf_ForcePageList(pageHashList, pageHashListLen, BF_SEM);
						pageHashListLen = 0;

						if (waitSemaphore(&(openLog->writeSemaphore))) {
							signalLatch( &(openLog->logLatch) );
							return(esmFAILURE);
						}
						
						/*
						 *	since the semaphore is now obtainable,
						 *	release it, let another thread run, and
						 *	start all over.  
						 */
						signalSemaphore( &(openLog->writeSemaphore) );
						signalLatch( &(openLog->logLatch) );
						Sleep(SLEEP_DONT_SLEEP);
						goto again;

					} else {

						/* no other thread has it */
						if (waitSemaphore( &(openLog->writeSemaphore) ))  {
							signalLatch( &(openLog->logLatch) );
							return(esmFAILURE);
						}
					}

					logLatched = TRUE;
				}

				if (PAGE_IS_DIRTY(pageHash)  ) {

					/* add page to list of pages to flush */
					SM_ASSERT(LEVEL_1, (pageHashListLen == 0) ||
						((pageHashList[pageHashListLen-1] -> 
							hashList.hashPid.page + 
							BLOCKCOUNT(pageHash->page2size) ) == 
								pageHash->hashList.hashPid.page ));

					pageHashList[pageHashListLen] = pageHash;
					pageHashListLen++;

				} else {
					/* 
					 *	if the page is not dirty, the page list must be
					 *	empty since log pages are written sequentially
					 */
					if (pageHashListLen != 0) {
						SM_ERROR(TYPE_FATAL, esmINTERNAL);
					}
				}
			} else {
				
				/*
				 *	Nothing else needs to be force, so exit the loop
				 *	after forcing all pages.
				 */
				/* Flush what we have to far */
				if (pageHashListLen > 0) bf_ForcePageList(pageHashList, pageHashListLen, BF_SEM);
				pageHashListLen = 0;

				if (logLatched) {
					signalSemaphore( &(openLog->writeSemaphore) );
					signalLatch( &(openLog->logLatch) );
				}

				/* break out of the loop */
				break;
			}
		}

		/*
		 *	See if the group link is not on a list.  It may
		 *	not be if it was removed while the pages were being
		 *	flushed.
		 */
		if (!LIST_MEMBER(&(groupLink->groupList))) {

			/* signal log the semaphore */
			if (logLatched) {
				signalSemaphore( &(openLog->writeSemaphore) );
				signalLatch( &(openLog->logLatch) );
			}

			/* try all over again */
			Sleep(SLEEP_DONT_SLEEP);
			goto again;
		}

		/* Get the next page to force */
		groupLink = (GROUPLINK *) NEXT_LIST_ELEMENT( &(groupLink->groupList) );
	
		/*
		 *	Write the pages so far if the list is full or we
		 *	are at the end of the unfixed list.
		 *	Also write the pages is the next page is not
		 *	contiguous (ie. the log wraps).
		 */
		if (pageHashListLen == MAX_IO_LIST || groupLink == NULL ||
			( (pageHashListLen > 0) && 
			   (pageHashList[pageHashListLen-1]->hashList.hashPid.page >
				groupLink->pageHash->hashList.hashPid.page)) ) {

			if (pageHashListLen > 0) 
				bf_ForcePageList(pageHashList, pageHashListLen, BF_SEM);
			pageHashListLen = 0;
		}

		/* 	signal the log semaphore */
		if (logLatched) {
			/* we must be at the end of the log */
			SM_ASSERT(LEVEL_1, groupLink == NULL);
			signalSemaphore( &(openLog->writeSemaphore) );
			signalLatch( &(openLog->logLatch) );
		}

	}
	SM_ASSERT(LEVEL_3, pageHashListLen == 0);

	/*
	 *	See if the log volume is not a raw disk
	 *	first get info on the volume
	 */
	volRec = openLog->volRec;
	if (!(volRec->volflags & VOL_RAWDEV) )	{

		/*
		 * 	not a raw disk, so sync it to force the pages
		 */
		if (fsyncDisk(volRec) != esmNOERROR) {
			SM_ERROR(TYPE_CRASH, esmINTERNAL);
			return(esmNOERROR);
		}
	}

	/* return success */
	return(esmNOERROR);
}
