/*
 *   $RCSfile: preflushLog.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "log_intfuncs.h"
#include "thread_globals.h"
#include "log_globals.h"
#include "io_globals.h"


/*
 *	preflushLog() works with the thread started by preflushLogThread()
 *	to asynchronously force dirty log pages to disk.
 */

 void
preflushLog ()
{

	TRPRINT(TR_LOG, TR_LEVEL_1, ("PotentialDirtyLogPages %d", PotentialDirtyLogPages));

	/*
	 *	A log page is ready to flush.  If we have more than one,
	 *	then perform the flush.
	 */
	PotentialDirtyLogPages++;	
	if ( (PotentialDirtyLogPages > LogPreflushTarget) && LogPreflushOn) {
		
		/*
		 *	perform flush by activating flush thread if it is not
		 *	already running.  Log is flushed up to LogPreflushMark.
		 */
		if (LIST_NOT_EMPTY(&LogPreflushWaitList)) {

			PREVENT_FAST_IO(); /* preflushLogThread() turns it on */
			/* LogPreflushMark = OpenLog.logRecordCount; */
			LogPreflushMark = OpenLog.tailBuffer->forceMark;
			notify(&LogPreflushWaitList, esmNOERROR, esmNOERROR);
		}

	}

	return;
}
