/*
 *   $RCSfile: releaseLogBlocked.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "openlog.h"
#include "log_extfuncs.h"
#include "trans_extfuncs.h"
#include "lock_globals.h"
#include "log_globals.h"


 void
releaseLogBlocked (

	register TRANSREC	*transRec,
	int					error 
)
{

	register OPENLOG	*openLog;


	TRPRINT(TR_LOG, TR_LEVEL_1, ("tid:%d", GETTID(transRec)));

	/*
	 *	get a pointer to the openlog
	 */
	openLog = &OpenLog;

	/*
	 *	release those on the latch list
	 */
	notifyTrans( transRec, &(openLog->logLatch.waitList), error );

	/*
	 *	release those on the read semaphore list
	 */
	notifyTrans( transRec, &(openLog->readSemaphore.waitList), error );

	/*
	 *	release those on the write semaphore list
	 */
	notifyTrans( transRec, &(openLog->writeSemaphore.waitList), error );

	/*
	 *	release those on the checkPoint list
	 */
	notifyTrans( transRec, &(openLog->checkPointList), error );
}
