/*
 *   $RCSfile: removeLogList.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:50 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "threadstate.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "log_globals.h"
#include "trans_globals.h"
#include "thread_globals.h"


 void
removeLogList (

	register TRANSREC	*transRec 
)
{

	register TRANSREC	*firstTrans;
	register OPENLOG	*openLog;


	TRACE(TR_LOG, TR_LEVEL_1);

	/*
	 *	check to see if this transaction is a member of the log list
	 */
	if (LIST_MEMBER( &(transRec->logActiveList) ))	{

		/*
		 *	get a pointer to the open log
		 */
		openLog = &OpenLog;

		/*
		 *	get a pointer to the first transaction on the log list
		 */
		firstTrans = (TRANSREC *) FIRST_LIST_ELEMENT( &(openLog->activeList) );
		CHECK_TRANSREC_MAGIC(transRec);

		/*
		 *	check to see if this record is at the head of the list
		 */
		if (transRec == firstTrans)	{

			TRPRINT(TR_LOG, TR_LEVEL_2, ("first on active log list"));

			/*
			 *	take the record off the list
			 */
			listRemove( &(transRec->logActiveList) );

			/*
			 *	check to see if the list is empty
			 */
			if (LIST_EMPTY( &(openLog->activeList) ))	{

				TRPRINT(TR_LOG, TR_LEVEL_2, ("active list is empty"));

				/*
				 *	set the active page to the null pid
				 */
				openLog->activePid = NULLPID;

			} else {

				/*
				 *	get a pointer to the new first transaction record
				 */
				firstTrans = (TRANSREC *) openLog->activeList.succ->item;
				CHECK_TRANSREC_MAGIC(transRec);

				/*
				 *	update the position information of the new last record
				 */
				openLog->activePid    = firstTrans->firstLogPid;
				openLog->activeUnique = firstTrans->logUnique;
				openLog->activeLSN    = firstTrans->firstLSN.offset;
				TRPRINT(TR_LOG, TR_LEVEL_2, ("new last pid:%d lsn:%d",
						openLog->activePid, openLog->activeLSN));
			}
		
		} else {

			/*
			 *	just take the transaction off the list
			 */
			listRemove( &(transRec->logActiveList) );
			TRPRINT(TR_LOG, TR_LEVEL_2, ("no new active log"));
		}

		/*
		 *	check to see if the close log is set
		 */
		if (openLog->flags & LOG_CLOSE)	{

			/*
			 *	assert that the list is not empty
			 */
			SM_ASSERT(LEVEL_1, LIST_NOT_EMPTY( &(openLog->closeList) ));

			notify( &(openLog->closeList), esmNOERROR, esmNOERROR );
		}
	}
}
