#ifndef _COM_PACKET_H_
#define _COM_PACKET_H_
/*
 *   $RCSfile: com_packet.h,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#ifndef PACKET_DEF
#define PACKET_DEF

#define OK 1

/*
 * The following is all the possible packet types & the corresponding
 * structures. 
 *
 * Any additional packets that are to be added must be included here &
 * implemented in packet.c 
 *
 */

typedef enum packet_type {
	RQ_LOG_VOL_HDR = 1,			/* Message from MR to get the volume header of
								   the log page, upon initialization */
	RE_LOG_VOL_HDR,				/* Message from the server in response to the
								   request for the volume header of the log */
	INIT_DUMP,   				/* Message from server to initiate dump */
	MR_RQ_DATA_PAGE,			/* Request by the MR to get the next data page
								   from the server */
	SR_RE_DATA_PAGE,			/* Response from the server to a request from 
								   the MR for a data page from the server */
	DUMP_DONE,					/* Message from the MR to server signalling the
								   completion of the dump for a given vol */
	SR_RQ_DATA_PAGE,			/* Request by the Server to get a data page
								   from the media recovery manager */
	MR_RE_DATA_PAGE,			/* Response from the MR to a request from the 
								   server for a data page from the MR */
	DUMP_LOG_PAGE,				/* Message from the server to the MR to dump
								   a log page */
	SR_RQ_LOG_PAGE,				/* Message from the server to MR to retrieve a 
								   log page corresponding to some data page */
	MR_RE_LOG_PAGE,				/* Message from the MR to the server in 
								   response to a request for a log page */
	ACK							/* Acknowledgement packet type */
} packet_type;


/*
 * Message from MR to get the volume header of the log page, 
 * upon initialization 
 */

typedef struct {
	UONE filler;	/* Just a dummy field which serves no purpose */
} rqlvh_pkt;

/*
 * Message from the server in response to the request for the volume header 
 * of the log 
 */

typedef struct {
	VOLHDR log_vol_hdr;		/* The volume header of the log */
} relvh_pkt;

/*
 * Message from server to initiate dump 
 */

typedef struct {
	VOLHDR dump_vol_hdr;	/* The volume header of the dump */
	LSN	   startLSN;		/* Start LSN for this dump */
} initdump_pkt;

/* 
 * Request by the MR to get the next data page from the server 
 */

typedef struct {
	PID pid;				/* Pid of the next page to be fetched from server */
} mrrqdp_pkt;

/* 
 * Response from the server to a request from the MR for a data page from 
 * the server 
 */

typedef struct {
	PID pid;				/* Pid of the requested page */
	int length;			/* Length of the page */
	ONE *page;
} srredp_pkt;
	
/* 
 * Message from the MR to signalling the completion of the dump for a 
 * given vol.
 */

typedef struct {
	PID pid;				/* Pid which has the vol. no of the dump that
							   has just been completed. */
} dumpdone_pkt;

/* 
 * Request by the Server to get a data page from the media recovery manager 
 */

typedef struct {
	PID pid;				/* Pid of the requested page */
} srrqdp_pkt;

/* 
 * Response from the MR to a request from the server for a data page from 
 * the MR 
 */

typedef struct {
	PID pid;				/* Pid of the page being shipped */
	LSN startLSN;			/* Start LSN to be later required for log 
							   recovery */
	int length;
	ONE *page;
} mrredp_pkt;

/* 
 * Message from the server to the MR to dump a log page 
 */

typedef struct {
	int length;
	ONE *page;
} dumplp_pkt;

/* 
 * Message from the server to MR to retrieve a log page corresponding to 
 * some data page 
 */

typedef struct {
	PID pid;				/* Pid of the data page for which the corresponding
							   log pages have to be retrieved */
	LSN startLSN;			/* StartLSN for that corresponding log */
} srrqlp_pkt;

/* 
 * Message from the MR to the server in response to a request for a log page 
 */

typedef struct {
	PID pid;				/* Pid of the log page that is being shipped */
	ONE last_page;			/* Boolean field indicating whether this is the
							   last page that applies to the given data page */
	int length;
	ONE *page;
} mrrelp_pkt;

/* 
 * Acknowledgement packet type 
 */

typedef struct {
	UONE code;		/* packet code */
	ONE error_code;	/* Success|failure code */
	PID pid;
} ack_pkt;

/* 
 * Structure of a packet
 */

typedef struct {
	UONE 	ptype;
	union {							/* Union of the various packet formats */
		rqlvh_pkt		rqlvh;
		relvh_pkt		relvh;
		initdump_pkt	initdump;
		mrrqdp_pkt		mrrqdp;
		srredp_pkt		srredp;
		dumpdone_pkt	dumpdone;
		srrqdp_pkt		srrqdp;
		mrredp_pkt 		mrredp;
		dumplp_pkt		dumplp;
		srrqlp_pkt		srrqlp;
		mrrelp_pkt		mrrelp;
		ack_pkt			ack;
	} upacket;
} packet;
	

#endif

#ifdef __cplusplus
extern "C" {
#endif

extern int send_packet(int sock, packet* pstruct);
extern int rcv_packet(int sock, packet* pstruct);

#ifdef __cplusplus
}
#endif
#endif /* _COM_PACKET_H_ */
