#ifdef MEDIA_RECOVERY
/*
 *   $RCSfile: mr_Receive.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"

#include "logdump.h"
#include "pagedump.h"
#include "com_packet.h"

#include "bitvec.h"
#include "msgvector.h"
#include "mr_extfuncs.h"
#include "msg_funcs.h"
#include "thread_funcs.h"
#include "msg_globals.h"
#include "server_stub.h"
#include "mr_extfuncs.h"
#include "mr_intfuncs.h"

extern void clearLinkTemp(LINK*, BOOL);

 int
mr_Receive(
	LINK 	*link 
)
{

	int	newSocket;

	if (PollMR(&newSocket)) {
		/*
		 *	Remove the connection link
		 */
		fprintf(stdout, "TODO: Replace clearLinkTemp with clearLink later\n");
		clearLinkTemp(link, TRUE);

		/*
		 * Set up a new link for the media recovery server to 
		 * connect fresh
		 */
        if (mr_InitializeSM_Stub(&newSocket, TRUE) != esmNOERROR) {
            SM_ERROR(TYPE_CRASH, esmINTERNAL);
        }
        Links[newSocket].linkClass = CL_MEDIA_RECOVERY;
        setLink(&(Links[newSocket]));

		/*
		 * restart this thread
		 */
		threadRestart();
	}

	/*
	 *	See if there is a new socket we should be using.  This
	 *	will only happen once at connect.
	 */
	if (newSocket >= 2) {	/* 2 == stderr */
		Links[newSocket].linkClass = CL_MEDIA_RECOVERY;
		setLink(&(Links[newSocket]));
		SM_ASSERT(LEVEL_1, &Links[newSocket] != link);

		/*
		 *	Remove the connection link
		 */
		clearLinkTemp(link, FALSE);
	}

    /*
     * restart this thread
     */
    threadRestart();
	return(esmNOERROR); /* never executed, but pleases compiler */
}


 void
clearLinkTemp (

	LINK	*link,
	BOOL	shutdownFlag 
)
{

	TRPRINT(TR_MSG, TR_LEVEL_1, ("clearing receive on link:%d", link->id));

	/*
	 *	set the select mask
	 */
	if (link->linkClass == CL_DISK)	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("type is disk"));

		/*
		 *	set the disk mask
		 */
		bic( &(DiskMask), link->id);
		bic( &(SelectMask), link->id);

	} else if (!SelectRestrict)	{

		/*
		 *	unset the select mask
		 */
		bic( &(SelectMask), link->id);
	}
		
	/*
	 *	set the all mask bits
	 */
	bic( &(AllMask), link->id);

	/*
	 *	close the file
	 */
	if (shutdownFlag)
		shutdown(link->id, 2);
	else
		close(link->id);

}
#endif MEDIA_RECOVERY
