/*
 *   $RCSfile: server_stub.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:53 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#ifdef _AIX
#       include <sys/select.h>
#endif
#include <fcntl.h>
#include "logdump.h"
#include "pagedump.h"
#include "com_packet.h"
#include "adminmsg.h"
#include "bitvec.h"
#include "log_intfuncs.h"
#include "log_globals.h"
#include "server_stub.h"
#include "thread_funcs.h"
#include "mr_extfuncs.h"
#include "mr_intfuncs.h"
#include "admin_funcs.h"
#include "msg_funcs.h"
#include "msg_globals.h"
#include "server_util_funcs.h"

#define SERVER_PROMPT "SERVER > "
#ifdef JUNK 
int	 get_str(char * buf, int limit, int * eoln_ptr );
#endif JUNK
void get_line(char * buf, int limit);

#ifdef MEDIA_RECOVERY
static TWO port_to_MR_g = 0;  /* Port no. used to communicate with the recovery
				 manager */
static fd_set monitored_sock_list_g; /* Sockets to be monitored using select */
static int sock_passive_g;	/* Passive socket used to establish connection with
							   the recovery manager */
static int mr_sock_g;		/* Socket returned by the accept call, which will
							   be the actual socket used for comm. with the
							   media recovery manager */
static fd_set *writefds_g = NULL;
static fd_set *exceptfds_g = NULL;
static connected_to_mr_g = 0;
static struct timeval zero_timeout;
extern char MRerrno;

int GetLogVolHdr( VOLHDR *log_vol_header);
int SendLogVolHdr(VOLHDR log_vol_hdr);
int SendNextDataPage( PID pid, int length, GROUPLINK *groupLink );

/* ======================================================================== */

int
GetLogVolHdr(
	VOLHDR *log_vol_header 
)
{

	if (MR_CopyLogVolHdr(log_vol_header)) {
		return(esmFAILURE);
	}
	return(esmNOERROR);
}

/* ======================================================================== */


int
DeliverRqLogPage(
	PID pid,
	ONE last_page,
	int length,
	ONE *page
)
{
	int ret;
	LSN startLSN;

	TRPRINT(TR_MR, TR_LEVEL_1, 
	("Received a log page applying to the data page %d.%d", pid.page, pid.volid));

	if (!last_page) {
		startLSN.offset = 0;
		ret = GetLogPageFromMR(pid, startLSN);
		if (ret == esmFAILURE) {
			fprintf(stdout,"Failed in GetLogPageFromMR.\n");
			return(esmFAILURE);
		}
	}
	else {
		fprintf(stdout,"All log pages for page %d.%d received.\n", pid.page, pid.volid);
	}

	return(esmNOERROR);
}


/* ======================================================================== */

/* To remove a socket from the list of sockets being monitored using select */

void 
remove_sock_from_monitored_list(
	int sock
)
{
	FD_CLR(sock, &monitored_sock_list_g);
}

/* ======================================================================== */

/* To add a socket to the list of sockets being monitored using select */

void 
add_sock_to_monitor_list(
	int new_socket
)
{
	if ((new_socket >= 0) && (new_socket <= FD_SETSIZE)) 
		FD_SET(new_socket, &monitored_sock_list_g);
	else 
		fprintf(stdout,"Invalid socket no. given to add to monitored list.\n");
}

/* ======================================================================== */

int
mr_InitializeSM_Stub(
	int *mr_socket,
	BOOL allocateMediaPort
)
{

	if (allocateMediaPort) {

		port_to_MR_g = SM_PORT_NO;

		/* Initialize the list of monitored sockets */

		FD_ZERO(&monitored_sock_list_g);
	}

	/* Add the standard input to the list of monitored sockets */

	FD_SET(0, &monitored_sock_list_g);

	if (allocateMediaPort) {

		/* Create the passive tcp socket for connecting to media recovery mgr */

		if ((sock_passive_g = create_passive_tcp_socket(port_to_MR_g)) == 
															esmFAILURE) {
			fprintf(stdout,"Passive socket creation failed\n");
			shutdown(sock_passive_g, 2);
			return(esmFAILURE);
		}

		add_sock_to_monitor_list(sock_passive_g);
		*mr_socket = sock_passive_g;
		TRPRINT(TR_MR, TR_LEVEL_1, 
			("Created passive socket for Media Recovery bound to port %d", port_to_MR_g));
		
	}
	return(esmNOERROR);
}

/* ======================================================================== */

int
get_socket_to_receive_from(
	fd_set *readyfd_list
)
{
	int i;

	for (i = 0; i < FD_SETSIZE; i++)  {
	   if (FD_ISSET(i, readyfd_list))
		return(i);
	}
	return(esmFAILURE);
}

/* ======================================================================== */

int 
PollMR( 
	int *return_socket
)
{
	fd_set readfds;
	int ret;
	int cur_sock;
	int new_sock;
	PID pid;
	LSN startLSN;
	packet *packet_ptr; 
	GROUPLINK *groupLink;
	packet local_packet;

	packet_ptr = &local_packet; 

	*return_socket = -1;
	readfds = monitored_sock_list_g;


	/* do a select to see if there is something to receive */

		zero_timeout.tv_sec = 0;  /* To set up a poll using select */
		zero_timeout.tv_usec = 0;

#ifdef hpux
    ret = select(FD_SETSIZE, (int*)&readfds, (int*)writefds_g, (int*)exceptfds_g
, &zero_timeout);
#else
    ret = select(FD_SETSIZE, &readfds, writefds_g, exceptfds_g, &zero_timeout);
#endif hpux

	if (ret < 0) {
		perror("select");
		return esmFAILURE;
	} else if (ret == 0)
	return esmNOERROR;

	/* Handle standard input */

    if (FD_ISSET(0, & readfds)) {
		ret = handle_input(&pid, &startLSN);
		if (ret == PROCESS_DATA_PAGE) {
			ret = GetDataPageFromMR(pid);
			if (ret == esmFAILURE) {
				fprintf(stdout,"Failed in GetDataPageFromMR\n");
				return(esmFAILURE);
			}
		}
		else if (ret == PROCESS_LOG_PAGE) {
			ret = GetLogPageFromMR(pid, startLSN);
			if (ret == esmFAILURE) {
				fprintf(stdout,"Failed in GetLogPageFromMR\n");
				return(esmFAILURE);
			}
		}
			
		return esmNOERROR;
	}

    /*
	 * If we got here the select call indicated available input.
	 * Read the next message.
	 */

	cur_sock = get_socket_to_receive_from(&readfds);
	if (cur_sock == esmFAILURE) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Trouble: get_socket_to_receive_from returned "));
		return(esmFAILURE);
	}
	
	if (cur_sock == sock_passive_g) {
		if (!connected_to_mr_g) {
			new_sock = accept_new_connection(sock_passive_g);
			if (new_sock == esmFAILURE) {
			  TRPRINT(TR_MR, TR_LEVEL_1, ("Failed while trying to accept a new connection.\n"));
			  return(esmFAILURE);
			}
			connected_to_mr_g = 1;
			add_sock_to_monitor_list(new_sock);
			remove_sock_from_monitored_list(sock_passive_g);
			mr_sock_g = new_sock;
			*return_socket = new_sock;
			return esmNOERROR;
		}
		else { /* Assuming there is only one media recovery manager for this 
																	server */
			TRPRINT(TR_MR, TR_LEVEL_1,
				("Received another connect request on the passive socket"));
			return(esmFAILURE);
		}
	}

	ret = rcv_packet(cur_sock, packet_ptr);

	if (ret < END_OF_FILE) {
		fprintf(stdout,"MR process died or the link is broken.\n");
		return esmFAILURE;
	}


   	switch (packet_ptr->ptype) {

		case RQ_LOG_VOL_HDR:
		{
			VOLHDR log_vol_hdr;
			int ret;
            PID pid; /*Just a dummy to call send_ack_packet_to_server */

			TRPRINT(TR_MR, TR_LEVEL_1, ("filler = %d\n",packet_ptr->upacket.rqlvh.filler));

			ret = GetLogVolHdr(&log_vol_hdr);
			if (ret != esmNOERROR) {
			   TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in GetLogVolHdr"));
			   pid.page = 0;
			   ret = send_ack_packet_to_mr(RQ_LOG_VOL_HDR, pid);
			   return(esmFAILURE);
		   	}

			ret = SendLogVolHdr(log_vol_hdr);
			if (ret != esmNOERROR) {
			   TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in SendLogVolHdr"));
			   return(esmFAILURE);
		   	}

			break;
		}

		case MR_RQ_DATA_PAGE:
		{
			int ret;
			int length;

		   TRPRINT(TR_MR, TR_LEVEL_1, ("Received MR_RQ_DATA_PAGE"));

			ret = GetNextDataPage(packet_ptr->upacket.mrrqdp.pid, &length,
													&groupLink);
			if (ret != esmNOERROR) {
			   TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in  GetNextDataPage"));
			   ret = send_ack_packet_to_mr(MR_RQ_DATA_PAGE, 
										packet_ptr->upacket.mrrqdp.pid);
			   return(esmFAILURE);
		   	}

			ret = SendNextDataPage(packet_ptr->upacket.mrrqdp.pid, 
													length, groupLink);
			if (ret != esmNOERROR) {
			   TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in  SendNextDataPage"));
			   return(esmFAILURE);
		   	}
			
			break;
		}

		case DUMP_DONE:
		{
			int ret;

			TRPRINT(TR_MR, TR_LEVEL_1, ("Received a DUMP_DONE packet.")); 

			ret = InformDumpDone(packet_ptr->upacket.dumpdone.pid);
			if (ret != esmNOERROR) {
				TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in InformDumpDone")); 
			   ret = send_ack_packet_to_mr(DUMP_DONE, 
										packet_ptr->upacket.dumpdone.pid);
			   return(esmNOERROR);
		   	}

			break;
		}

		case MR_RE_DATA_PAGE:
		{
			int ret;

			TRPRINT(TR_MR, TR_LEVEL_1, ("Recvd MR_RE_DATA_PAGE")); 

			ret = DeliverRqDataPage(packet_ptr->upacket.mrredp.pid,
						packet_ptr->upacket.mrredp.startLSN,
						packet_ptr->upacket.mrredp.length,
						packet_ptr->upacket.mrredp.page);
			if (ret != esmNOERROR) {

				TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in DeliverRqDataPage")); 
			   ret = send_ack_packet_to_mr(MR_RE_DATA_PAGE, 
										packet_ptr->upacket.mrredp.pid);
			   return(esmFAILURE);
		   	}
			free(packet_ptr->upacket.mrredp.page);

			break;
		}
			
		case MR_RE_LOG_PAGE:
		{
			int ret;

			TRPRINT(TR_MR, TR_LEVEL_1, ("Received MR_RE_LOG_PAGE")); 

			ret = DeliverRqLogPage(packet_ptr->upacket.mrrelp.pid,
						packet_ptr->upacket.mrrelp.last_page,
						packet_ptr->upacket.mrrelp.length,
						packet_ptr->upacket.mrrelp.page);
			if (ret != esmNOERROR) {
				TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in DeliverRqLogPage")); 
			   ret = send_ack_packet_to_mr(MR_RE_LOG_PAGE, 
										packet_ptr->upacket.mrrelp.pid);
			   return(esmFAILURE);
		   	}
			free(packet_ptr->upacket.mrrelp.page);

			break;
		}

		case ACK:
			handle_acks(packet_ptr);
			break;

		default:
			TRPRINT(TR_MR, TR_LEVEL_1, ("Invalida packet type %d", packet_ptr->ptype)); 

			return(esmFAILURE);
	}
	return(esmNOERROR);
}

/* ======================================================================== */

int 
InitiateDump(
	VOLHDR dump_vol_hdr,
	LSN startLSN 
)
{
	packet temp_packet;
	int ret;

	TRPRINT(TR_MR, TR_LEVEL_1, ("InitiadeDump")); 
	temp_packet.ptype = INIT_DUMP;
	temp_packet.upacket.initdump.dump_vol_hdr = dump_vol_hdr;
	temp_packet.upacket.initdump.startLSN = startLSN;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent INIT_DUMP")); 
	return(esmNOERROR);
}

/* ======================================================================== */

int 
SendLogVolHdr(
	VOLHDR log_vol_hdr
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = RE_LOG_VOL_HDR;
	temp_packet.upacket.relvh.log_vol_hdr = log_vol_hdr;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent RE_LOG_VOL_HDR")); 
	return(esmNOERROR);
}

/* ======================================================================== */

int 
SendNextDataPage(
	PID pid,
	int length,
	GROUPLINK *groupLink 
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = SR_RE_DATA_PAGE;
	temp_packet.upacket.srredp.pid = pid;
	temp_packet.upacket.srredp.length = length;
	temp_packet.upacket.srredp.page = groupLink->pageHash->bufFrame;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent SR_RE_DATA_PAGE %d:%d\n", pid.volid, pid.page));

	mr_ReleaseDataPage(groupLink);

	return(esmNOERROR);
}

/* ======================================================================== */

int 
GetDataPageFromMR(
	PID pid
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = SR_RQ_DATA_PAGE;
	temp_packet.upacket.srrqdp.pid = pid;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent SR_RQ_DATA_PAGE"));
	return(esmNOERROR);
}

/* ======================================================================== */

int 
DumpLogPage(
	int length,
	ONE *page
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = DUMP_LOG_PAGE;
	temp_packet.upacket.dumplp.length = length;
	temp_packet.upacket.dumplp.page = page;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent DUMP_LOG_PAGE")); 
	return(esmNOERROR);
}

/* ======================================================================== */

int 
GetLogPageFromMR(
	PID pid, LSN startLSN
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = SR_RQ_LOG_PAGE;
	temp_packet.upacket.srrqlp.pid = pid;
	temp_packet.upacket.srrqlp.startLSN = startLSN;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent SR_RQ_LOG_PAGE")); 
	return(esmNOERROR);
}

/* ======================================================================== */

int 
send_ack_packet_to_mr(
	int code, PID pid
)
{
	packet temp_packet;
	int ret;

	temp_packet.ptype = ACK;
	temp_packet.upacket.ack.code = code;
	temp_packet.upacket.ack.error_code = MRerrno;
	temp_packet.upacket.ack.pid = pid;

	ret = send_packet(mr_sock_g, &temp_packet);
	if (ret != esmNOERROR) {
		TRPRINT(TR_MR, TR_LEVEL_1, ("Failed in send_packet")); 
		return(esmFAILURE);
	}
	TRPRINT(TR_MR, TR_LEVEL_1, ("sent ACK")); 
	return(esmNOERROR);
}

int
mr_Retrieve(
	PID* pid, 
	LSN* startLSN
}
{
	fprintf(stdout,"Type of page:(1 for data page, any other number for log page): ");
	scanf("%d", &type);
	if (type == 1) 
		fprintf(stdout,"Enter the pid of the data page to be retrieved.\n");
	else
		fprintf(stdout,"Enter the pid of the data page for which the log pages have to be fetched:\n ");
	fprintf(stdout,"pid.page = ");
	scanf("%d", &(pid->page));
	fprintf(stdout,"pid.volid = ");
	scanf("%hd", &(pid->volid));
	if (type != 1) {
		fprintf(stdout,"startLSN->wrapCount = ");
		scanf("%d", &(startLSN->wrapCount));
		fprintf(stdout,"startLSN->offset = ");
		scanf("%d", &(startLSN->offset));
	}
	fprintf(stdout,"\n");
	fprintf(stdout, SERVER_PROMPT);
	fflush(stdout);
	if (type == 1) {
		return(PROCESS_DATA_PAGE);
	}
	else {
		return(PROCESS_LOG_PAGE);
	}
}
#else MEDIA_RECOVERY

 int
terminal_input(
	LINK 	*link 
)
{
	PID pid;
	LSN startLSN;

	return handle_input(&pid, &startLSN);
}
#endif


/* ======================================================================== */

void
stdin_usage()
{
	fprintf(stdout,"The following commands are available to the user:\n\n");
	/* quit, exit, bye --> msg about kill, crash, shutdown */
	fprintf(stdout,"help - To list the commands that are available.\n");
	fprintf(stdout,"shutdown,shut - To gracefully complete activity and shut down.\n");
	fprintf(stdout,"kill - To stop everything immediately, without grace.\n");
	fprintf(stdout,"crash - To abort the server and produce a core file.\n");
#ifdef DEBUG
	/* setvar does not get printed if DEBUG not on, but it's still there.... */
	fprintf(stdout,"setvar <variable> <value> - To set certain server variables.\n");

	fprintf(stdout,"traceflags,flags <value> - To change tracing flags.\n");
	fprintf(stdout,"tracelevel,level n - To set trace level to n for each trace flag.\n");
#endif DEBUG
#ifdef PROFIL
	fprintf(stdout,"profil - To dump the profiling info into a file.\n");
#endif PROFIL
#ifdef notdef
	fprintf(stdout,"dump - To dump a  volume. [not implemented]\n");
	fprintf(stdout,"retreive - To retrieve a data page from the MR. [not implemented]\n");
#endif notdef
	fprintf(stdout,"checkpoint,check - To initiate a checkpoint.\n");
	fprintf(stdout,"printstats,print - To print server statistics.\n");
	fprintf(stdout,"clearstats,clear - To clear counters for server statistics.\n");
	fprintf(stdout,"listdistr,distr - To list active distributed transactions.\n");
	fprintf(stdout,"listmount,mount - To list mounted volumes.\n");
	fprintf(stdout,"listvolumes,volumes - To list mountable volumes.\n");
	fprintf(stdout,"progress - To turn on/off progress printing.\n");
	fprintf(stdout,"user - To turn on/off printing of user errors.\n");
	fprintf(stdout,"source <file> - To read a file of these commands.\n");
	fprintf(stdout,"redirect [errorfile|output] <file> - To redirect output to a file.\n");
	fprintf(stdout,
		"addvolume <formatinfo> - To add a volume to those mountable.\n");
	fprintf(stdout,
		"rmvolume <volid> - To remove a volume from those mountable.\n");
}

void
setStdinLink()
{
	LINK *link;

	TRPRINT(TR_CL, TR_LEVEL_1, ("setStdinLink, fileno(stdin) %d\n",
		fileno(stdin)));
	if(fileno(stdin) == -1) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	link = &(Links[fileno(stdin)]);
	Links[fileno(stdin)].linkClass = CL_STDIN;
	setLink(link);
}

void
clearStdinLink()
{
	LINK *link;

	TRPRINT(TR_CL, TR_LEVEL_1, ("ClearStdinLink, fileno(stdin) %d\n",
		fileno(stdin)));
	if(fileno(stdin) == -1) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
	link = &(Links[fileno(stdin)]);
	clearLink(link); /* closes the file associated with the link */
}
#define BUF_SIZE 256

static char *Bufp;
static char *
first_word(char *buf)
{
	Bufp = strtok(buf, " ");
	TRPRINT(TR_CL, TR_LEVEL_1, ("first word: \"%s\"\n", Bufp));
	return Bufp;
}
static char *
next_word()
{
	int		word_len;
	char 	*new_word;
	/* CANNOT be called to get the first word! */

	/* assumes words are separated by one blank */
	/* first: find location of NULL inserted by 
	 * previous strtok call 
	 */
	word_len = strcspn(Bufp, " ");
	/* get new token */
	new_word = strtok(NULL, " ");
	/* replace the null written by the previous strtok */
	Bufp[word_len]  = ' ';
	TRPRINT(TR_CL, TR_LEVEL_1, ("next word: \"%s\"\n", new_word));
	return new_word;
}

void
redirect(int whichfile, char *path)
{
	static char *ctermpath = "/dev/tty";
	FILE *newstream;
	FILE *oldstream;
	LINK *link;
	char *flags;
	int	linkClass;

	TRPRINT(TR_CL, TR_LEVEL_1, ("redirect %d to %s", whichfile, path));

	path = next_word();
	if( path == NULL) {
		path = ctermpath;
	} 
	if(RunInBackground && 
		((path == ctermpath) || !strcmp(path,ctermpath) ) ) {
nobg:
		fprintf(stdout, "\"-background\" option overrides.\n");
		fprintf(stdout, "Cannot redirect to terminal.\n");
		/* stdin is closed now, should never get selected */
		return;
	}
	switch(whichfile) {
		case 0:
			/* flags = O_RDONLY; */
			flags = "r";
			oldstream = stdin;
			linkClass = CL_STDIN;
			break;

		case 1:
			flags = "a";
			oldstream = stdout;
			linkClass = CL_STDOUT;
			break;

		case 2:
			/* flags = O_WRONLY; */
			flags = "a";
			oldstream = stderr;
			linkClass = CL_STDERR;
			break;

		default:
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			return;
	}
	clearLink( &Links[fileno(oldstream)] );
	newstream = freopen(path, flags,  oldstream);
	if(newstream == NULL) {
		SM_ERROR(TYPE_SYS, errno);
		if(!RunInBackground) {
			newstream = freopen(ctermpath, flags,  oldstream);
			if(newstream == NULL) {
				SM_ERROR(TYPE_FATAL, errno);
			}
		} else {
			goto nobg;
		}
	}

	link = &(Links[fileno(newstream)]);
	link->linkClass = linkClass; 
	link->flags = 0;
	setSelectBits(fileno(newstream));
	setLink(link);
}


/*
 * handle_input is used to parse the input command and most probably
 * report some state to the user.
 */

int 
handle_input(
	PID* pid, 
	LSN* startLSN
)
{
	extern		BOOL PrintUser;
	int  		word_len;
	static char	Buf[BUF_SIZE];
	static char	Buf1[BUF_SIZE];
	char		*word;

	if( fileno(stdin) == -1 ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	if(feof(stdin)) 
		return IGNORE;
	if(RunInBackground) {
		/* We get some glop from the Bourne shell
		 * before it closes stdin; ignore it.
		 */
		return IGNORE;
	}

	get_line(Buf1, BUF_SIZE);
	/* words separated by 1 blank; possibly a blank at end */
	word_len = strcspn(Buf1, " ");

	TRPRINT(TR_CL, TR_LEVEL_1,
		("Buf1:%s, word_len 0x%x\n", Buf1, word_len));

	if (word_len > 0) {
		bzero(Buf, sizeof(Buf));
		strcpy(Buf, Buf1); /* Buf <-- Buf1 if it's not just a <cr> */
	} else {
		/* print the command that we're repeating */
		fprintf(stdout,"%s\n",Buf);
	}
	if((word = first_word(Buf)) != NULL) {
		if ( (strcmp(word, "quit") == 0) 
			|| (strcmp(word, "exit") == 0)
			|| (strcmp(word, "bye") == 0)
			) {
			/* ***** QUIT EXIT BYE ***** */
			fprintf(stdout,"Here are the ways you can quit:\n");
			fprintf(stdout,"\tshutdown - to gracefully complete activity and shut down.\n");
			fprintf(stdout,"\tkill - to stop everything immediately, gracelessly.\n");
			fprintf(stdout,"\tcrash - to abort the server and produce a core file.\n");

		} else if ( (strcmp(word, "kill") == 0) || (strcmp(word, "crash") == 0)) {
			/* ***** ABORT CRASH ******
			 * Graceless close.
			 * Abort does not produce core file.  Crash does.
			 */
			if(strcmp(word, "crash") == 0) {
				admin_ShutServer(SHUT_DUMP_CORE);
				return esmNOERROR;
			} else {
				admin_ShutServer(NOFLAGS);
				return esmNOERROR;
			}

		} else if ( (strcmp(word, "shutdown") == 0) || (strcmp(word, "shut") == 0) ) {
			/* ***** SHUTDOWN ******
			 * Cleanly shut down the server.
			 */
			admin_ShutServer(SHUT_ABORT_TRANS | SHUT_CLEAN_VOLUMES);
			return esmNOERROR;

#ifdef PROFIL
		} else if (strcmp(word, "profil") == 0) {
			/* ***** PROFIL *****
			 * dump profiling information into the mon.out files
			 */

			MonDump(TRUE, 0);
#endif PROFIL
		} else if ( (strcmp(word, "printstats") == 0) || (strcmp(word, "print") == 0) ) {
			admin_CollectStats();
			dumpStats();
		} else if ( (strcmp(word, "listdistr") == 0) || (strcmp(word, "distr") == 0) ) {
			admin_ListDistr();
		} else if ( (strcmp(word, "listmount") == 0) || (strcmp(word, "mount") == 0) ) {
			admin_ListMount();
		} else if ( (strcmp(word, "listvolumes") == 0) || (strcmp(word, "volumes") == 0) ) {
			admin_ListVolNames();
		} else if ( (strcmp(word, "clearstats") == 0) || (strcmp(word, "clear") == 0) ) {
			admin_ClearStats();
			clearStats();
		} else if ( (strcmp(word, "addvolume") == 0) || (strcmp(word, "add") == 0) ) {
			if( (word = next_word()) != NULL) {
				BOOL oldPrintUser = PrintUser;
				char *restofline = word + strlen(word) + 1;

				PrintUser = TRUE; /* temporarily */
				(void) addcommand(word, restofline, stdout );
				PrintUser =  oldPrintUser;
			} else {
				fprintf(stdout, "Try addvolume|add optionname formatinfo\n");
			}
		} else if ( (strcmp(word, "rmvolume") == 0) || (strcmp(word, "remove") == 0) ) {
			if( (word = next_word()) != NULL) {
				VOLID volid = atoi(word);
				if(volid) {
					BOOL oldPrintUser = PrintUser;

					PrintUser = TRUE; /* temporarily */
					(void) removecommand(volid, stdout);
					PrintUser =  oldPrintUser;
				} else {
					fprintf(stdout, 
					"Bad volid: %s: Try rmvolume|remove volid\n", word);
				}
			} else {
				fprintf(stdout, "Try rmvolume|remove volid\n");
			}
#ifdef JUNK
		} else if ( strcmp(word, "testdismount") == 0 ) {
			if( (word = next_word()) != NULL) {
				VOLID volid = atoi(word);
				if(volid) {
					BOOL oldPrintUser = PrintUser;

					PrintUser = TRUE; /* temporarily */
					(void) testdismmount(volid, stdout);
					PrintUser =  oldPrintUser;
				} else {
					fprintf(stdout, 
					"Bad volid: %s: Try testdismount volid\n", word);
				}
			} else {
				fprintf(stdout, 
					"Bad volid: %s: Try testdismount volid\n", word);
			}

		} else if ( strcmp(word, "testmount") == 0 ) {
			if( (word = next_word()) != NULL) {
				VOLID volid = atoi(word);
				if(volid) {
					BOOL oldPrintUser = PrintUser;

					PrintUser = TRUE; /* temporarily */
					(void) testmount(volid, stdout);
					PrintUser =  oldPrintUser;
				} else {
					fprintf(stdout, 
					"Bad volid: %s: Try testmount volid\n", word);
				}
			} else {
				fprintf(stdout, 
					"Bad volid: %s: Try testmount volid\n", word);
			}
#endif JUNK

		} else if ( (strcmp(word, "user") == 0) ) {
			PrintUser = !PrintUser;  
			fprintf(stdout, "User errors will %s be printed.\n", 
				PrintUser? "":"not");
		} else if ( (strcmp(word, "progress") == 0) ) {
			PrintProgress = !PrintProgress;  
			fprintf(stdout, "Progress will %s be printed.\n", 
				PrintProgress? "":"not");
#ifdef MEDIA_RECOVERY
		} else if (strcmp(word, "dump") ==0) {
			int ret;
			VOLID dump;

			fprintf(stdout,"Volume of the dump = ");
			scanf("%d", &dump);
			fprintf(stdout,"\n");
			fprintf(stdout, SERVER_PROMPT);
			fflush(stdout);
			ret = mr_TriggerDump(dump);
			if (ret == esmFAILURE)
				fprintf(stdout,"Bad volume Id.  Try again.\n");
			return(esmNOERROR);
#endif MEDIA_RECOVERY
		} else if ( (strcmp(word, "traceflags") == 0) || (strcmp(word, "flags") == 0)) {

			if((word = next_word())  != NULL) {
				TraceFlags = (TRACEFLAGS) strtol(word, (char **)NULL, 0);
			}
			fprintf(stdout, " Trace flags are now 0x%x\n", TraceFlags);

		} else if ( (strcmp(word, "tracelevel") == 0) || (strcmp(word, "level") == 0)) {
			register int level, i;
			BOOL assign;

			if((word = next_word())  != NULL) {
				level = (int) strtol(word, (char **)NULL, 0);
				fprintf(stdout," Setting tracelevel as follows: ", level);
				assign = TRUE;
			} else {
				if(TraceFlags == 0) {
					fprintf(stdout, "No trace flags set.");
				} else {
					fprintf(stdout,"Trace levels are as follows : ");
				}
				assign = FALSE;
			}

			for(i=0; i < (8 * sizeof (int)); i++) {
				if( TraceFlags & (1<<i) ) {
					if(assign)
						TraceLevel.level[i] = level;
					fprintf(stdout," [bit %d : lev %d]", i, TraceLevel.level[i]);
				}
			}
			fprintf(stdout, "\n");
		} else if (strcmp(word, "help") == 0) {
			stdin_usage();
#ifdef MEDIA_RECOVERY
		} else if ((strcmp(word, "retrieve") == 0) || (strcmp(word, "retreive") == 0)) {
			return mr_Retrieve(pid, startLSN);
#endif MEDIA_RECOVERY
		} else if ((strcmp(word, "checkpoint") == 0) || (strcmp(word, "check") == 0)) {
				
				if ((OpenLog.flags & LOG_CHECKPOINT_IN_PROGRESS)) {
					fprintf(stdout,"checkpoint already in progress\n");	
				} else {

					fprintf(stdout,"forking checkpoint thread\n");	
					activateCheckpoint(TRUE, FALSE, FALSE);
				}
		} else if (strcmp(word, "setvar") == 0) {
#ifdef DEBUG
			int var = -1, val = -1;

			if( (word = next_word()) == NULL) {
				for(var=1; ;var++) {
					if(DebugGetVar(var, &val) == esmFAILURE) 
						break;
					fprintf(stdout, "Variable #%10d = %10d / 0x%8x\n", var, val, val);
				}
			} else {
				var = (TRACEFLAGS) strtol(word, (char **)NULL, 0);
				if( (word = next_word()) != NULL) {
					val = (TRACEFLAGS) strtol(word, (char **)NULL, 0);
				}
				if( (var < 0) || (val < 0)) {
					fprintf(stdout,"syntax error. try setvar var# val#\n");
				} else {
					if(DebugSetVar(var, val) != esmNOERROR) {
						fprintf(stdout,"error : try again\n");
					}
				}
			} 
#else DEBUG
			fprintf(stdout,"Server was not made with debugging.\n");
#endif DEBUG
		} else if (strcmp(word, "source") == 0) {
			/* 2nd arg is file name; none -> use /dev/tty */
			redirect(0, word);
		} else if (strcmp(word, "redirect") == 0) {
			/* 2nd arg tells which to redirect; 3rd arg is path;
			 * /dev/tty is back to tty 
			 */
			int whichfile= -1;

			if( (word = next_word()) != NULL) {
				if(strcmp(word, "stderr")==0) {
					whichfile = 2;
				} else if(strcmp(word, "errorfile")==0) {
					whichfile = 2;
				} else if(strcmp(word, "stdout")==0) {
					whichfile = 1;
				} else if(strcmp(word, "output")==0) {
					whichfile = 1;
				} else if(strcmp(word, "stdin")==0) {
					whichfile = 0;
				} else if(strcmp(word, "input")==0) {
					whichfile = 0;
				} else {
					whichfile = -1;
				}
			}
			if(whichfile == -1) {
				fprintf(stdout, 
					"syntax error. try: redirect [errorfile|output] <path> \n");
				return(IGNORE);
			}
			redirect(whichfile, word);
		} else {
			fprintf(stdout,"%s: Invalid command.\n\n",word );
			stdin_usage();
		}
		if((word = next_word())!= NULL) {
			fprintf(stdout,"\"%s ...\" ignored.\n",word );
		}
		fflush(stdout);
	} 
	fprintf(stdout, SERVER_PROMPT);
	fflush(stdout);
	return IGNORE;
}

void
get_line(
	char * buf,
	int limit
)
{
	int i = 0;
	int c;
	BOOL skip_white;

	buf[0] = '\0';
	skip_white = TRUE; /* skip all leading white space */
	while ((c = getchar()) != '\n') {
		if (c == EOF) {
			fflush(stdout);
			/* happens only if we're in the foreground */
			cont(0, 0); /* see catchSignals.c */
			return;
		} else if (isspace(c)) {
			if (skip_white) {
				/* Skip leading white space */
				continue;
			} else {
				c = (int)(' ');
				/* compress white space into one blank */
				skip_white = TRUE;
			}
		} else {
			/* use a blank after this word */
			skip_white = FALSE;
		}
		if (i < (limit-1)) {
			buf[ i ++ ] = c;
		}
	}
	buf[ i ] = '\0';  
	TRPRINT(TR_CL, TR_LEVEL_1, ("new line: \"%s\"\n", buf));
	return;
}

#ifdef JUNK 
/*
 * get_str reads in a string until the '\n' is encountered copying only the
 * first limit characters to the buffer.
 */

get_str(
	char * buf,
	int limit,
	int * eoln_ptr 
)
{
	int i = 0;
	int c, start;

	if (eoln_ptr != NULL)
		*eoln_ptr = 1;

	buf[0] = '\0';
	start = 1;
	while ((c = getchar()) != '\n') {
		if (c == EOF) {
			fflush(stdout);
			/* happens only if we're in the foreground */
			cont(0, 0); /* see catchSignals.c */
		} else if (isspace(c)) {
			if (start == 1)
				/* Skip leading white space */
				continue;
			else {
				/* Now skip any trailing white space */
				while ((c == ' ') || (c == '\t')) {
					c = getchar();
				}
				if (eoln_ptr != NULL) {
					if (c != '\n') {
						ungetc(c, stdin);
						*eoln_ptr = 0;
					} else {
						*eoln_ptr = 1;
					}
				}
				break;
			}
		}

		start = 0;
		if (i < (limit-1)) {
			buf[ i ++ ] = c;
		}
	}
	buf[ i ] = '\0';
	return(0);
}
#endif JUNK

/* ======================================================================== */
#ifdef MEDIA_RECOVERY

int 
DoDump(
	VOLHDR myvolinfo
)
{
char	fname[32];
int stillmore =1;
char pagebuf[MIN_PAGESIZE];
LOGPAGEHDR head;
int fd;

	sprintf(fname,"%d", myvolinfo.volid);

	if ((fd=open(fname, O_RDONLY))<0) {
		fprintf(stdout,"Error opening file %s\n", fname);
	}

	if (lseek(fd, (off_t)(7)*MIN_PAGESIZE, SEEK_SET)< 0) {
		close(fd);
	}
	while (stillmore) {

	/* there are more log pages in the file */

		bzero(pagebuf, MIN_PAGESIZE);

		if (read(fd, pagebuf, MIN_PAGESIZE) <0)	{
			close(fd);
			return(0);
		}

	/*
	 * check to see that this is a valid log page - if it is, then ship else
	 * set stillmore to 0 
	 */


		lseek(fd,(off_t)(-(MIN_PAGESIZE)),SEEK_CUR);
		read(fd,(char *) &head,sizeof(LOGPAGEHDR));
		if (head.magic == LOGPAGE_MAGIC) {
			lseek(fd, (off_t)(MIN_PAGESIZE - sizeof(LOGPAGEHDR)),SEEK_CUR);	
			if (DumpLogPage(MIN_PAGESIZE,(char *)pagebuf) == esmFAILURE) {
				fprintf(stdout,"Error in dumping log page \n");
				return (esmFAILURE);
			}
		}
		else {
			 stillmore =0;
		 }
	}
	close(fd);
	return esmNOERROR;
}


#ifdef SERVER_MAKE
#else
static VOLHDR myvolhdr;
static get_out_of_poll_g = 0;

main(
	int argc,
	ONE *argv[]
)
{
FILEID fd;
LSN		startLSN;
int ret;



	mr_InitializeSM_Stub();


	if (argc < 2)
	{
		fprintf(stdout,"Format: mrecs volname\n");
		exit(-1);
	}

	if ((fd = open(argv[1], O_RDONLY))<0)
	{
		fprintf(stdout,"Unable to open file %s\n", argv[1]);
		exit(-1);
	}

	read(fd, (char *)&myvolhdr, sizeof(VOLHDR));

	close(fd);
	TRPRINT(TR_MR, TR_LEVEL_1, ("The log page size is 2**%d\n",myvolhdr.logFileBlock2size)); 

	while (get_out_of_poll_g != 1) {
		ret = PollMR();
		if (ret == esmFAILURE) {
			fprintf(stdout,"Failed in PollMR()\n");
			exit(-1);
		}
	}

	fprintf(stdout,"Dump for volume %d initiated\n", myvolhdr.volid);
	fflush(stderr);
	if (InitiateDump(myvolhdr, startLSN)==esmFAILURE) {
		fprintf(stdout,"Could not initiate the dump for the volume %d\n", 
														myvolhdr.volid);
		return(esmFAILURE);
	}

	ret = DoDump(myvolhdr);
	if (ret == esmFAILURE) {
		fprintf(stdout,"Failed in DoDump\n");
		return(esmFAILURE);
	}

	while (1) {
		ret = PollMR();
		if (ret == esmFAILURE) {
			fprintf(stdout,"Failed in PollMR()\n");
			exit(-1);
		}
	}

}
#endif SERVER_MAKE



int 
GetNextDataPage(
	PID			pid,
	int		*length,
	GROUPLINK 	**groupLink 
)
{

	if (mr_CopyDataPage(pid, length, groupLink) != esmNOERROR) {
		return(esmFAILURE);	
	}
	return(esmNOERROR);
}


int 
InformDumpDone(
	PID pid
)
{

	fprintf(stdout,"Dump for the volume %d is done.\n", pid.volid);
	return(esmNOERROR);
}

int 
DeliverRqDataPage(
	PID		pid,
	LSN		startLSN,
	int	length,
	ONE		*page 
)
{

#ifdef JUNK
	char 	pagebuf[MIN_PAGESIZE];
	char	fname[32];
	FILEID	fd;

#endif

	int page_length;
	GROUPLINK *groupLink;

	fprintf(stdout," Page that has been delivered is %d.%d\n", pid.volid, pid.page);
#ifdef JUNK
	sprintf(fname,"%d", pid.volid);

	if ((fd=open(fname, O_RDONLY))<0)
		{
		fprintf(stdout,"Error opening file %s\n", fname);
		return(esmFAILURE);	
		}

	if (lseek(fd, (off_t)(pid.page-1)*MIN_PAGESIZE, SEEK_SET)< 0)
	{
		TRPRINT(TR_MR, TR_LEVEL_1, 
			("Error seeking to the desired page %d in volume %d\n", 
														pid.page, pid.volid)_;
		close(fd);
		return(esmFAILURE);
	}

	bzero(pagebuf, MIN_PAGESIZE);

	if (read(fd, pagebuf, MIN_PAGESIZE) <0)	
	{
		TRPRINT(TR_MR, TR_LEVEL_1, 
			("Error reading the desired page %d in volume %d\n", 
													pid.page, pid.volid));
		close(fd);
		return(esmFAILURE);
	}
#endif

	if (mr_CopyDataPage(pid, &page_length, &groupLink)) {
		return(esmFAILURE);
	}
	if (bcmp(groupLink->pageHash->bufFrame, page, length) != 0)
	{

		fprintf(stdout," The retrieved page is not the same as the page dumped\n");
		return(esmFAILURE);
	}
	else
		fprintf(stdout,"The retrieved page matches the original page.\n");

#ifdef JUNK
	close(fd);
#endif

	mr_ReleaseDataPage(groupLink);
	return(esmNOERROR);
}




/* ======================================================================== */

void 
handle_acks(
	packet* temp_packet
)
{
	
	switch (temp_packet->upacket.ack.code) {
		
		case INIT_DUMP:
			fprintf(stdout,"INIT_DUMP came back with a failure\n");
			fprintf(stdout,"Failure code = %d\n", temp_packet->upacket.ack.error_code);
			break;

		case DUMP_LOG_PAGE:
			fprintf(stdout,"DUMP_LOG_PAGE came back with a failure\n");
			fprintf(stdout,"Failure code = %d\n", temp_packet->upacket.ack.error_code);
			break;

		case SR_RE_DATA_PAGE:
		case SR_RQ_DATA_PAGE: 

		case SR_RQ_LOG_PAGE:
			fprintf(stdout,"%d came back with a failure\n",
									temp_packet->upacket.ack.code);
			fprintf(stdout,"Failure code = %d\n", temp_packet->upacket.ack.error_code);

			fprintf(stdout,"The pid involved was: ");
			fprintf(stdout,"pageno = %d\n",temp_packet->upacket.ack.pid.page);
			fprintf(stdout,"                      volid = %d\n",temp_packet->upacket.ack.pid.volid);
			break;

		default:
			fprintf(stdout,"Received an ACK packet with unknown code = %d\n", temp_packet->upacket.ack.code);
	}
}

/* ======================================================================== */
#endif MEDIA_RECOVERY
