/*
 *   $RCSfile: freeClientLink.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "host.h"
#include "bitvec.h"
#include "msgvector.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "util_funcs.h"
#include "thread_funcs.h"
#include "server_util_funcs.h"
#include "disk_globals.h"
#include "thread_globals.h"


static void
_freeClientLink (

	register LINK		*link, 
	int					linkClass
)
{
	TRPRINT(TR_MSG, TR_LEVEL_1, 
		("freeing client link:%d type:%d, newtype %d", 
		link->id, link->linkClass, linkClass));

	/* do not allow this link to be selected */
	suspendLink(link);

	if (link->linkClass == CL_UNUSED) {
		/* link has already been freed by another thread */
		return;
	} 
	if (link->linkClass != CL_CLIENT) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/* decrement the count of active clients for "fast"-path.  */
	NumActiveClients--;

	/* abort the transactions associated with this link */
	abortLink(link);

	/* free the resources associated with this link */
	freeLinkResources(link);

	/* 	free all vols associated with this link */
	freeLinkVolumes(link);

	clearLink(link);
	if(linkClass != link->linkClass) {
		SM_ASSERT(LEVEL_1, (linkClass == CL_TIMEDOUT)); 
		link->linkClass = linkClass; 
		TRPRINT(TR_MSG, TR_LEVEL_1, 
		("link:%d is now type %d", link->id, link->linkClass));
	}

	/* If we're running under a transaction, remove us from it */
	transThreadDeath(Active);
}

void
timeoutClientLink (

	register LINK		*link
)
{
	_freeClientLink(link, CL_TIMEDOUT);
}

void
freeClientLink (

	register LINK		*link
)
{
	_freeClientLink(link, CL_UNUSED);
}
