/*
 *   $RCSfile: freeLinkVolumes.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "host.h"
#include "bitvec.h"
#include "msgvector.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "openlog.h"
#include "bf_extfuncs.h"
#include "trans_extfuncs.h"
#include "msg_funcs.h"
#include "util_funcs.h"
#include "thread_funcs.h"
#include "disk_globals.h"
#include "log_globals.h"


 void
freeLinkVolumes (

	register LINK		*link 
)
{

	register VOLINFO	*volInfo;


	TRPRINT(TR_MSG, TR_LEVEL_1, ("freeing client link:%d", link->id));

	/*
	 *	free all vols associated with this link
	 */
	while ((volInfo = (VOLINFO *) listDeq( &(link->volList) )) != NULL)	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("closing vol:%d", volInfo->volRec->header->volid));

		/*
		 *	take the volinfo off the vol
		 */
		listRemove( &(volInfo->volList) );

		/*
		 *	check to see if this is the last open on this vol and
		 *	the volume is not the one containing the log.
		 */
		if (LIST_EMPTY( &(volInfo->volRec->openList) ) &&
			LIST_EMPTY( &(volInfo->volRec->activeTransList) ) &&
			(OpenLog.volid != volInfo->volRec->header->volid) )	{

			TRPRINT(TR_MSG, TR_LEVEL_2, ("last open on vol"));

			bf_DismountVolume(volInfo->volRec, TRUE);
		}

		/*
		 *	put the volinfo on the pool
		 */
		listEnq( &VolInfoPool, &(volInfo->linkList) );
	}
}
