/*
 *   $RCSfile: initializeLinks.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "bitvec.h"
#include "msgvector.h"
#include "volume.h"
#include "msg_funcs.h"
#include "disk_globals.h"
#include "io_globals.h"
#include "msg_globals.h"


 void
initializeLinks ()

{

	register LINK	*link;
	register int	i;


	TRACE(TR_MSG, TR_LEVEL_1);

	/*
	 *	Base the number of links on the max number of clients
	 *	plus the max number of volumes, + 5 for safety.
	 *  The absolute limit is the max for the system.
	 */
	NumLinks = NumClients + NumVolumes + 5;
	{
#ifdef hpux
        int mx = _POSIX_OPEN_MAX;
#else
        int mx = getdtablesize();
#endif hpux

		if(NumLinks > mx) NumLinks = mx;
	}
	

	/*
	 *	attempt to allocate space for the messages
	 */
	if ((Links = link = (LINK *) malloc(sizeof(LINK) * NumLinks)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}

	/*
	 *	initialize the lists
	 */
	initializeList( &FreeLinks );
	initializeList( &UsedLinks );

	/*
	 *	initialize the links
	 */
	for (i = 0; i < NumLinks; i++, link++)	{

		link->id = i;
		link->flags = LINK_FREE;
		initializeListElement( &(link->list), (char *) link );
		listEnq( &FreeLinks, &(link->list) );
		initializeList( &(link->tcbList) );
		initializeList( &(link->volList) );
		initializeList( &(link->transList) );
		initializeList( &(link->resourceList) );
		link->lastForceMark = 0;
	}
}
