/*
 *   $RCSfile: selectReceive.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "threadstate.h"
#include "semaphore.h"
#include "link.h"
#include "host.h"
#include "bitvec.h"
#include "msgvector.h"
#include "msg_funcs.h"
#include "msg_globals.h"


 int
selectReceive (
	
	register BITVEC	*readyMask 
)
{

	register int	limitIndex;
	register int	currentIndex;
	register int	startIndex;
	register BITVEC *diskMask;
	register BITVEC	*allMask;
	register BITVEC	*workMask;
	register int	wrap;


	TRACE(TR_MSG, TR_LEVEL_1);

	/*
	 *	get pointers to the globalbit vectors
	 */
	diskMask = &DiskMask;
	allMask  = &AllMask;
	workMask = &WorkMask;

	/*
	 *	create the disk bit vector
	 */
	if (!SelectRestrict)	{

		bitcom(workMask, readyMask, diskMask);
	}

	/*
	 *	check to see if any disk requests are satisfied
	 */
	if (btest(workMask))	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("disk requests ready"));

		/*
		 *	use the round robin technique for searching for bits
		 */
		currentIndex = startIndex = DiskIndex;

	} else {

		TRPRINT(TR_MSG, TR_LEVEL_2, ("no disk requests ready"));

		/*
		 *	set the mask
		 */
		workMask = readyMask;
		currentIndex = startIndex = AllIndex;
	}

	/*
	 *	search initially to end of select bits
	 */
	limitIndex = SelectBits;
	wrap = FALSE;

	/*
	 *	search to see which selector is ready
	 */
	while (currentIndex < limitIndex)	{

		if (bit(workMask, currentIndex))	{

			TRPRINT(TR_MSG, TR_LEVEL_2, ("got index at:%d", currentIndex));

			/*
			 *	Check to see if this is a disk receive
			 */
			if (workMask == readyMask)	{

				/*
				 *	set the state for the next receive and return
				 */
				AllIndex = currentIndex + 1;
				TRPRINT(TR_MSG, TR_LEVEL_2, ("setting next all receive to:%d", AllIndex));
				return(currentIndex);

			} else {

				/*
				 *	set the state for the next receive and return
				 */
				DiskIndex = currentIndex + 1;
				TRPRINT(TR_MSG, TR_LEVEL_2, ("setting next disk receive to:%d", DiskIndex));
				return(currentIndex);
			}
		}

		/*
		 *	increment and check for the end of select bits
		 */
		if (++currentIndex == limitIndex)	{

			/*
			 *	check to see if we've already wrapped
			 */
			if (wrap)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
			
			} else {

				TRPRINT(TR_MSG, TR_LEVEL_2, ("wrapping index to:%d", startIndex));

				/*
				 *	wrap the index
				 */
				wrap = TRUE;
				limitIndex = startIndex;
				currentIndex = 0;
			}
		}
	}

	/*
	 *	if we get to here we're dead
	 */
	SM_ERROR(TYPE_FATAL, esmINTERNAL); 
	return esmFAILURE; /* to keep the compiler quiet */
}
