/*
 *   $RCSfile: serverReceive.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "host.h"
#include "bitvec.h"
#include "msgvector.h"
#include "msg_funcs.h"
#include "util_funcs.h"
#include "thread_funcs.h"
#include "thread_globals.h"
#include "msg_globals.h"
#include "distr.h"
#include "distr_intfuncs.h"


extern int			errno;


 void
serverReceive (
	register LINK		*link 
)
{
	register MESSAGE	*message;
	register MSGVECTOR	*msgVector;


	TRPRINT(TR_MSG, TR_LEVEL_1, ("got message on link:%d", link->id));

	/*
	 *	check to see if there is a waiter on that link
	 *	NOTE: this should never be the case for the datagram link
	 */
	SM_ASSERT(LEVEL_1, 	(LIST_EMPTY(&(link->tcbList))));

	/*
	 *	get a register pointer to the message buffer
	 */
	message = &(Active->message);

	/*
	 *	save the link that the request came in on
	 */
	Active->linkId = link->id;

	/*
	 *	read the message
	 *  MsgStats.pages_received, etc  are updated in receiveDatagram()
	 */
	if (receiveDatagram(link, (char *) message, sizeof(MESSAGE)) < 0)	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("receiveDatagram failed:%d", errno));
		/*
		 *	just restart this task
		 */
		threadRestart();
	}

	/*
	 *	take care to see that the protocol versions match
	 */
	if (message->header.version != MESSAGE_VERSION) {

		SM_ERROR(TYPE_WARNING, esmPROTOCOLVERSION);
		replyDatagram(esmFAILURE, Active->errno, 1, TRUE);
	}

	TRPRINT(TR_MSG, TR_LEVEL_2, ("got message of type:%d", message->header.type));

	/*
	 *	check to see if the message header type is correct
	 */
	if (message->header.type > MAX_MSG_NUMBER)	{

		/*
		 *	just restart this task
		 */
		threadRestart();
	}

	if( !(message->header.replyRequested) )
		MsgStats.no_reply_requested[message->header.type]++;
	MsgStats.received [message->header.type]++;

	/*
	 *	get a pointer to the message function table
	 */
	msgVector = &(MsgVectorTable[message->header.type]);
	SM_ASSERT(LEVEL_1, (msgVector->msgtype == message->header.type));

	/*
	 *	check to see that the class is appropriate
	 */
	if (msgVector->linkClass != CL_DGRAM)	{

		TRPRINT(TR_MSG, TR_LEVEL_2, ("bad message type:%d", message->header.type));
		/*
		 *	just restart this task
		 */
		threadRestart();
	}


	/*
	 *	call the function
	 */
	(*(msgVector->function))(message, link);

	/* ok to get here now.  Return to main loop */
}
