/*
 *   $RCSfile: space.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "host.h"
#include "bitvec.h"
#include "bf.h"
#include "volume.h"
#include "lsn.h"
#include "trans.h"
#include "msgvector.h"
#include "resource.h"
#include "cl_funcs.h"
#include "trans_extfuncs.h"
#include "admin_funcs.h"
#include "resource_funcs.h"
#include "msg_globals.h"
#include "sm_params.h"
#include "distr_extfuncs.h"


					/* stdin, stdout, stderr, media recovery, listen, spare*/
int			NumLinks = 0;
LIST		FreeLinks;
LIST		UsedLinks;
LINK		*Links;

int			NumMsgBufs; /* initialized in initializeEnv() */

int		    ServePort = 0;
int			SelectBits = 0;
BITVEC		SelectMask;	/*  the fds that are to be inspected by a select().
						 *	This is either DiskMask or AllMask .
						 */
BITVEC		ReadMask;   /* a copy of SelectMask that's used for the output 
						 *parameter in the select() system call 
						 */
BITVEC		DiskMask;   /* which of the select fds are disk fds .
						 *	suspended links are not present in this mask 
						 */
BITVEC		AllMask;	 /* all the fds that are open and not suspended. */
BITVEC		WorkMask;   /* used by selectReceive(). Shouldn't be global */
int			DiskIndex = 0;
int			AllIndex = 0;
BOOL		SelectRestrict = FALSE; /* When we resort to using the spare
	                      * thread, this becomes true, and the spare 
						  * thread  listens ONLY for disk requests 
						  * (SelectMask=DiskMask) This takes place in 
						  * dispatch().  At all other  times, 
						  * SelectRestrict is false, and SelectMask 
						  * is AllMask.
						  */
TIMEVAL		*SelectTime;
int			ConnectReply;
int			fdUdp, fdTcp;

#ifdef JUNK

#ifdef __cplusplus

extern "C" void cl_CreateSequentialFile(MESSAGE *, LINK *);
extern "C" void cl_OpenSequentialLoad(MESSAGE *, LINK *);
extern "C" void cl_OpenSequentialScan(MESSAGE *, LINK *);
extern "C" void cl_LoadNextSequential(MESSAGE *, LINK *);
extern "C" void	cl_ScanNextSequential(MESSAGE *, LINK *);


#else


extern void cl_CreateSequentialFile(MESSAGE *, LINK *);
extern void cl_OpenSequentialLoad(MESSAGE *, LINK *);
extern void cl_OpenSequentialScan(MESSAGE *, LINK *);
extern void cl_LoadNextSequential(MESSAGE *, LINK *);
extern void	cl_ScanNextSequential(MESSAGE *, LINK *);


#endif

#else 

#define cl_CreateSequentialFile 0
#define cl_OpenSequentialLoad 0
#define cl_OpenSequentialScan 0
#define cl_LoadNextSequential 0
#define cl_ScanNextSequential 0
#define cl_StatVolume 0

#endif


/*
 *	define the master jump table for calling routines for messages
 */
MSGVECTOR MsgVectorTable[] =	{

	NULL,						0	,				      0    ,
	cl_MountVolume,				CL_CLIENT,			      1    ,
	cl_DismountVolume,			CL_CLIENT,			      2    ,
	cl_AddVolume,			CL_CLIENT,			      3    ,
	cl_ReadPage,				CL_CLIENT,			      4    ,
	cl_WritePage,				CL_CLIENT,			      5    ,
	cl_AllocPages,				CL_CLIENT,			      6    ,
	cl_DeallocPages,			CL_CLIENT,			      7    ,
	cl_CheckBitmap,				CL_CLIENT,			      8    ,
	cl_GetUnique,				CL_CLIENT,			      9    ,
	cl_StatVolume,				CL_CLIENT,			     10    ,
	cl_SetRootEntry,			CL_CLIENT,			     11    ,
	cl_GetRootEntry,			CL_CLIENT,			     12    ,
	cl_RemoveRootEntry,			CL_CLIENT,			     13    ,
	cl_ExternalBegin,			CL_CLIENT,			     14    ,
	cl_ExternalCommit,			CL_CLIENT,			     15    ,
	cl_ExternalAbort,			CL_CLIENT,			     16    ,
	cl_LockPage,				CL_CLIENT,			     17    ,
	cl_LockFile,				CL_CLIENT,			     18    ,
	cl_CreateFile,				CL_CLIENT,			     19    ,
	cl_DestroyFile,				CL_CLIENT,			     20    ,
	cl_FirstPhysicalPage,		CL_CLIENT,			     21    ,
	cl_LastPhysicalPage,		CL_CLIENT,			     22    ,
	cl_NeighborPage,			CL_CLIENT,			     23    ,
	cl_FirstLogicalPage,        CL_CLIENT,               24    ,
	cl_LastLogicalPage, 		CL_CLIENT,               25    ,
	cl_MarkPage,				CL_CLIENT,			     26    ,
	cl_ResetRusage,				CL_CLIENT,			     27    ,
	cl_GetRusage,				CL_CLIENT,			     28    ,
	cl_CreateSequentialFile,	CL_CLIENT,			     29    ,
	cl_OpenSequentialLoad,		CL_CLIENT,			     30    ,
	cl_OpenSequentialScan,		CL_CLIENT,			     31    ,
	cl_ScanNextSequential,		CL_CLIENT,			     32    ,
	cl_LoadNextSequential,		CL_CLIENT,			     33    ,
	cl_CloseResource,			CL_CLIENT,			     34    ,
	cl_TestNullRpc,				CL_CLIENT,			     35    ,
	cl_TestPageRpc,				CL_CLIENT,			     36    ,
	cl_WriteLogPage,			CL_CLIENT,			     37    ,
	cl_InitClient,				CL_CLIENT,			     38    ,
	cl_UnlockPage,				CL_CLIENT,			     39    ,
	cl_UnlockFile,				CL_CLIENT,			     40    ,
	cl_QueryPage,				CL_CLIENT,			     41    ,
	cl_SetAnchor,				CL_CLIENT,			     42    ,
	cl_SetCLR,					CL_CLIENT,       	     43    ,
	cl_MonDump,					CL_CLIENT,			   	 44    ,
	cl_TakeCheckpoint,			CL_CLIENT,			   	 45    ,
	cl_ChangeCheckpointFreq,	CL_CLIENT,			   	 46    ,
	cl_ShutServer,				CL_CLIENT,			     47    ,
	cl_Statistics,				CL_CLIENT,			     48    ,
	cl_DebugSetVar,				CL_CLIENT,			     49    ,
	cl_TestPageSend,			CL_CLIENT,			     50    ,
	cl_SyncTransaction,			CL_CLIENT,			     51    ,
	cl_FileNumPages,			CL_CLIENT,			     52    ,
	cl_ExternalEnter2PC,		CL_CLIENT,				 53    ,
	cl_ExternalPrepare,			CL_CLIENT,				 54    ,
	server_ExternalPrepare,		CL_DGRAM,				 55    ,
	coord_RecordVote,			CL_DGRAM,				 56    ,
	cl_ExternalCommand,			CL_CLIENT,				 57    ,
	cl_ExternalContinue2PC,		CL_CLIENT,				 58    ,
	cl_ExternalRecover2PC,		CL_CLIENT,				 59    ,
	server_ExternalCommand,		CL_DGRAM,				 60    ,
	coord_RecordAck,			CL_DGRAM,				 61	   ,
	cl_AddVolume,				CL_CLIENT,				 62	   ,
	cl_RemoveVolume,			CL_CLIENT,				 63	   
	/* 
	 * IF YOU MOVE ANY ENTRIES TO THIS TABLE, 
	 * you must rearrange the list of names in common/util/messageStats.c.
	 */
};

char *ServeName = SERVENAME;
