/*
 *   $RCSfile: timeOutClient.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "ess.h"
#include "checking.h"
#include "sysdefs.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "timer.h"
#include "lock.h"
#include "tid.h"
#include "link.h"
#include "thread.h"
#include "threadstate.h"
#include "thread_globals.h"
#include "msg_globals.h"
#include "thread_funcs.h"
#include "msg_funcs.h"


void
timeOutClient()
{
	LINK			*link;
	register int 	i;

#define MINUTES(x) (60*(x))
#define STALE_TIME 4

	while(TRUE) {
		Sleep(MINUTES( STALE_TIME+1 ));

		for( i=0,link=Links; i < NumLinks; i++,link++) {
			/* mess only with client links */
			if(link->linkClass != CL_CLIENT)
				continue;

			/* consider only links with no transactions */
			if(LIST_NOT_EMPTY(&(link->transList)))
				continue;

			/* If there has been an abort or commit since the
			 * last time any thread (particularly, me) slept,
			 * leave it alone.  Consider the link still "fresh".
			 */
			if( (TIME)link->lastTxEndTime >  ServerTimer.LastSleepTime() )
				continue;

			/* else no activity on this link since the last sleep ... */

			/* If the last sleep was STALE_TIME ago,
			 * consider the link stale and close it.
			 */

			if( ServerTimer.Elapsed() >= MINUTES(STALE_TIME)) {
				TRPRINT(TR_CL|TR_THREAD|TR_MSG,
					TR_LEVEL_1, ("TIMING OUT a client"));
				timeoutClientLink(link);
			}
		}
	}
}
