/*
 *   $RCSfile: analyzeAbort.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "recover_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_extfuncs.h"


 void
analyzeAbort (

	register LOGRECORDHDR	*record,
	register LSNOFFSET		recordLSN 
)
{
	
	register TRANSREC		*transRec;


	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d tid:%x type:%d action:%d",
			record->recordLSN.offset, record->tid, record->type, record->action));

	/*
	 *	find the transaction record
	 */
	if ((transRec = findTransRec(record->tid)) == NULL)	{

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	Remove all pages from the dirty page table which were
	 *	deallocated by this transaction.
	 */
	removeDeallocedDirtyPages(transRec);

#ifdef JUNK
	/*
	 *	set the state to abort
	 */
	transRec->transState = T_ABORT;

	/*
	 *	set the last LSN
	 */
	transRec->lastLSN = recordLSN;
#endif

	/*
	 *	remove the transaction from any list of distributed 
	 *	transactions it is on
	 */
	if (LIST_MEMBER(&(transRec->distrTransList))) {

		if (findDistrTransRec(record->tid, &ServerDistrTransList) 
			!= NULL)	
			numActiveServerDistrTrans--;
		else if (findDistrTransRec(record->tid, &CoordDistrTransList) 
				 != NULL)	
			numActiveCoordDistrTrans--;
		else
			SM_ERROR(TYPE_FATAL, esmINTERNAL);

		/*
		 *  take it off the list of active distr transactions
		 */
		listRemove( &(transRec->distrTransList) );
	}

	/*
	 *	The transaction is now inactive, so remove it from the active list 
	 *	and return it to the free pool
	 */
	transRec->transState = T_INACTIVE;
	listRemove( &(transRec->activeTransList) );
	poolMove( &TransRecPool, &(transRec->tidList) );
}
