/*
 *   $RCSfile: analyzeCommit.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"


 void
analyzeCommit (

	register LOGRECORDHDR	*record 
)
{
	
	register TRANSREC		*transRec;


	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d tid:%x type:%d action:%d",
			record->recordLSN.offset, record->tid, record->type, record->action));

	/*
	 *	check to see if there is a transaction record
	 */
	if ((transRec = findTransRec(record->tid)) == NULL)	{

		SM_ERROR(TYPE_LOG, esmINTERNAL);
		return;
	}

	/*
	 *	process the record
	 */
	switch (record->action) {

		case LOG_ACTION_DISTR_COORD_COMMIT:
			
			/*
			 *  change the state to commit
			 */
			transRec->transState = T_COMMIT;

			/*
			 *	note the client command
			 */
			transRec->clientCommand = C_COMMIT;

			break;
		
		case LOG_ACTION_DISTR_COORD_COMMIT_READONLY:

			/*
			 *	initialize the state to inactive
			 */
			transRec->transState = T_INACTIVE;

			/*
			 *	initialize the list components
			 */
			listRemove( &(transRec->activeTransList) );

			/*
			 *	check to see if the transaction is on the list of 
			 *	distr trans - this may not be so because we don't
			 *	write a prepare record for READONLY transactions
			 */
			if (LIST_MEMBER(&(transRec->distrTransList))) {

				/*
				 *	remove the transRec from the list of distr trans
				 */
				listRemove( &(transRec->distrTransList) );
				numActiveCoordDistrTrans--;
			}

			/*
			 *	move the transaction to the free pool
			 */
			poolMove( &TransRecPool, &(transRec->tidList) );
			break;

		case LOG_ACTION_DISTR_SERVER_COMMIT:
		case NULL:

			/*
			 *  Remove all pages from the dirty page table which were
			 *  deallocated by this transaction.
			 */
			removeDeallocedDirtyPages(transRec);

			/*
			 *	initialize the state to inactive
			 */
			transRec->transState = T_INACTIVE;
		
			/*
			 *	initialize the list components
			 */
			listRemove( &(transRec->activeTransList) );

			/*
			 *	check if this is a distributed transaction
			 */
			if (LIST_MEMBER(&(transRec->distrTransList))) {

				/*
				 *	take it off the list of active distr transactions
				 */
				listRemove( &(transRec->distrTransList) );

				/*
				 *	decrement the number of active distr transactions
				 */
				numActiveServerDistrTrans--;
			}

			/*
			 *	move the transaction to the free pool
			 */
			poolMove( &TransRecPool, &(transRec->tidList) );
			break;

		default:

			TRPRINT(TR_RECOVER, TR_LEVEL_1, 
				("unknown action %d", record->action));
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;

	}
}
