/*
 *   $RCSfile: analyzeDismountVolume.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "log.h"
#include "recover_intfuncs.h"
#include "io_globals.h"
#include "log_globals.h"
#include "trans_globals.h"
#include "trans_extfuncs.h"
#include "bf_globals.h"


/*
 *	This function handles log records which contain information
 *	about the dismounting of a volume
 */


 void
analyzeDismountVolume (
	LOGRECORDHDR		*record 
)
{
	VOLID				volid;
	int					i;
	int					location;

	TRACE(TR_IO, TR_LEVEL_1);

	/*
	 *	Get the volume id
	 */
	volid = *(VOLID*) GET_LOG_IMAGE(record, 0);

	/*
	 *	Make sure the volume is already in the recovery mount table
	 *	and find out which entry it is.
	 */
	location = -1;
	for (i = 0; i < CheckpointVolumeCount; i++) {
		if (CheckpointVolumes[i].volid == volid) location = i;
	}
	if (location == -1) {
		/*
		 *	Not in the table
		 */
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	Remove the volume from the table
	 *	Shift all volumes after the one to delete, down in the table
	 */
	for (i = location+1; i < CheckpointVolumeCount; i++) {
		CheckpointVolumes[i-1] = CheckpointVolumes[i];
	}
	CheckpointVolumeCount--;

	removeDirtyPagesForVolume(volid);

	return;
}
