/*
 *   $RCSfile: analyzeRecord.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "thread_globals.h"


 void
analyzeRecord (

	register LOGRECORDHDR	*record,
	register LSN			*recordLSN 
)
{
	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("record type:%d", record->type));

	/*
	 *	process the record
	 */
	switch (record->type)	{

		case LOG_REC_TYPE_COMMIT:

			analyzeTrans(record, recordLSN);
			analyzeCommit(record);
			break;

		case LOG_REC_TYPE_ABORT:

			analyzeTrans(record, recordLSN);
			analyzeAbort(record, recordLSN->offset);
			break;

		case LOG_REC_TYPE_SERVER_PREPARE:

			analyzeTrans(record, recordLSN);
			analyzeServerPrepare(record);
			break;

		case LOG_REC_TYPE_COORD_PREPARE:

			analyzeTrans(record, recordLSN);
			analyzeCoordPrepare(record /*,recordLSN->offset*/);
			break;

		case LOG_REC_TYPE_END:

			analyzeTrans(record, recordLSN);
			analyzeEnd(record /*, recordLSN->offset*/);
			break;

		case LOG_REC_TYPE_CHECKPOINT:

			/*
			 *	Analyze the checkpoint record.  It is possible that
			 *	more than one checkpoint record can be analyzed.  This
			 *	can happen if the update to the log header does not
			 *	make it to disk before the crash and after a checkpoint.
			 */
			analyzeCheckpoint(record, recordLSN->offset);
			break;

		case LOG_REC_TYPE_COMPENSATION:
		case LOG_REC_TYPE_USER:
		case LOG_REC_TYPE_USER_COMPENSATION:

			analyzeTrans(record, recordLSN);
			analyzeUser(record, recordLSN->offset);
			break;

		case LOG_REC_TYPE_DIRTYPAGELIST:
			/*
			 * analyzeTrans(record, recordLSN);
			 */
			analyzeDirtyPages(record);
			break;

		case LOG_REC_TYPE_MOUNT_VOLUME:
			analyzeMountVolume(record);
			break;

		case LOG_REC_TYPE_DISMOUNT_VOLUME:
			analyzeDismountVolume(record);
			break;

		default:

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}
}
