/*
 *   $RCSfile: processLogForward.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:54 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "log.h"
#include "log_globals.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"


 LSNOFFSET
processLogForward (

	OPENLOG					*openLog,
	register LSNOFFSET		recordLSN,
	FLAGS					phase 
)
{

	register LOGRECORDHDR	*record;
	register FOUR			offset;
	register LOGPAGEHDR		*pageHeader;
	register FOUR			numPages;
	register FOUR			length;
	FOUR					left;
	GROUPLINK				*groupLink;
	LOGPAGEHDR				*endHeader;
	PID						pid;
	SHORTPID				page;
	LSNOFFSET				previousLSN;
	GROUPLINK				*logRecordBuffer = NULL;
#ifdef DEBUG
	LOGRECORDHDR			*previousLogRecord;
	LSN						debugLSN;
#endif DEBUG

	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));

	/*
	 *	fill in the volume id
	 */
	pid.volid = openLog->volid;	

	/*
	 *	read all the records out of the log
	 */
	previousLSN = 0;
	for (openLog->wrapCount = NULL_LOG_WRAP_COUNT;;)	{

		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("current recordLSN:%d", recordLSN));

		/*
		 *	calculate the page
		 */
		page	 = LSN_TO_LOG_PAGE(recordLSN, openLog);
		pid.page = LOG_PAGE_TO_BLOCK(page, openLog);
		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("page:%d", page));

		/*
		 *	check to see if the log has wrapped
		 */
#ifdef JUNK
		if ((page == 0) && (openLog->wrapCount != NULL_LOG_WRAP_COUNT))	{

			openLog->wrapCount++;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("wrapCount:%d", openLog->wrapCount));
		}
#endif JUNK

		/*
		 *	calculate the offset within the page
		 */
		offset = LOG_PAGE_OFFSET(recordLSN, openLog);
		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("offset:%d", offset));

		/*
		 *	read in the next record
		 */
		if ((groupLink = bf_ReadPage(openLog->readGroup, &pid,
									 openLog->page2size, NOFLAGS)) == NULL)	{

			SM_ERROR(TYPE_FATAL, Active->errno);
		}

		/*
		 *	unfix the page
		 */
		bf_UnfixPage(groupLink, BF_DEFAULT, FALSE);

		/*
		 *	get a pointer to the page header and the end header
		 */
		pageHeader = (LOGPAGEHDR *) groupLink->bufFrame;
		endHeader = (LOGPAGEHDR *)(((char *) pageHeader) + openLog->lastUsableByte);

		/*
		 *	check the page numbers
		 */
		SM_ASSERT(LEVEL_1, pageHeader->pageNumber == page);
		SM_ASSERT(LEVEL_1, endHeader->pageNumber == page);

		/*
		 *	check to see if the magic numbers are valid
		 */
		if (CHECK_LOGPAGE_MAGIC(pageHeader) || CHECK_LOGPAGE_MAGIC(endHeader))	{

			SM_ERROR(TYPE_FATAL, esmBADLOGPAGEHEADER);
		}

		/*
		 *	check to see if the wrapcount is zero
		 */
		if (openLog->wrapCount == NULL_LOG_WRAP_COUNT)	{

			/*
			 *	this is the first page
			 */
			openLog->wrapCount = pageHeader->wrapCount;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("first page"));

		} else {
			
			/*
			 *	See if we have wrapped
			 */
			if (previousLSN > recordLSN) {
				
				openLog->wrapCount++;
				TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("wrapCount:%d", openLog->wrapCount));
			}
		}
		
		/*
		 *	check to see if the wrap counts match
		 */
		if (pageHeader->wrapCount != endHeader->wrapCount)	{

			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("wrap count mismatch:%d", pid.page));

			/*
			 *	 check to see if this is the redo phase
			 */
			if (phase == REDO_PHASE)	{

				setupEndOfLog(recordLSN, TRUE);
			}
			break;
		}

		if (pageHeader->wrapCount < openLog->wrapCount)	{

			/*
			 *	this is the end of the log
			 */
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("wrapCount pid:%d", pid.page));

			/*
			 *	 check to see if this is the redo phase
			 */
			if (phase == REDO_PHASE)	{

				setupEndOfLog(recordLSN, TRUE);
			}
			break;

		} 
		
		SM_ASSERT(LEVEL_1, pageHeader->wrapCount == openLog->wrapCount);

#ifdef DEBUG
		/* make sure we don't overwrite important log records */
		debugLSN.offset = openLog->tailLSN;
		debugLSN.wrapCount = openLog->wrapCount;
		debugLSN.wrapCount--;
		if(compareLSN(&debugLSN, &OldestDirtyPageLSN) >= 0) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
#endif

		/*
		 *	check to see if the lastRecords match for analysis purposes
		 */
		if (pageHeader->lastRecord != endHeader->lastRecord)	{

			/*
			 *	endHeader should always be less
			 */
			if (pageHeader->lastRecord <= endHeader->lastRecord)	{

				SM_ERROR(TYPE_FATAL, esmINTERNAL);

			} else {

				SM_ERROR(TYPE_LOG, esmINTERNAL);
			}
		}

		/*
		 *	check the lastRecord field of the ENDHEADER!!
		 */
		if (endHeader->lastRecord < offset)	{

			/*
			 *	go to next page
			 */
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("offset:%d lastRecord:%d", 
					offset, endHeader->lastRecord));

			/*
			 *	get a pointer to the record
			 */
			record = (LOGRECORDHDR *) (groupLink->bufFrame + offset);

			/*
			 *	check to make sure its zeros
			 *
			 *  Whenever a new log page is started, the entire page
			 *	is zero'd (see client--CLOG_GetNewLogPage() and
			 *	serverlib--writeLogPageHeader. When processLogForward is
			 *	scanning for log records, it advances "offset" to
			 *	the next log record expected on the page.  As you see
			 *	in the code, if offset is < the last record on the page,
			 *	then it will skip to the next page.  But, the SM_ASSERT
			 *	is a sanity check the the rest of the page is still
			 *	zeroed.
			 */
			SM_ASSERT(LEVEL_1, record->magic == 0);

			/*
			 *	calculate the offset onto the next page
			 */
			recordLSN = LOG_PAGE_TO_LSN((page + 1), openLog) + FIRST_LSN;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));

			/*
			 *	wrap the log if necessary
			 */
			recordLSN %= openLog->fileBytes;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));

			/*
			 *	continue at the bottom of the loop
			 */
			continue;
		}

		/*
		 *	get a pointer to the record
		 */
		record = (LOGRECORDHDR *) (groupLink->bufFrame + offset);

		/*
		 *	check the log record magic number
		 */
		if (CHECK_LOGRECORD_MAGIC(record))	{

			SM_ERROR(TYPE_FATAL, esmBADLOGRECORDHEADER);
		}

		/*
		 *	save the record length
		 */
		length = record->length;

		/*
		 *	check to see if the record resides on a page
		 */
		if ((openLog->lastUsableByte - offset) < record->length)	{

			if (logRecordBuffer == NULL) {
				logRecordBuffer = getLogRecordBuffer(record->length);
				SM_ASSERT(LEVEL_3, logRecordBuffer != NULL);
			}

			/*
			 *	combine the record
			 */
			if (combineRecord(openLog, record, (openLog->lastUsableByte - offset), FALSE, logRecordBuffer) != esmNOERROR)	{

				/*
				 *	 check to see if this is the redo phase
				 */
				if (phase == REDO_PHASE)	{

					setupEndOfLog(recordLSN, FALSE);

					SM_ASSERT(LEVEL_3, previousLSN > 0);

					bzero(openLog->tailLink->bufFrame + offset, (int)(openLog->lastUsableByte - offset));
					if (LSN_TO_LOG_PAGE(recordLSN, openLog) == LSN_TO_LOG_PAGE(previousLSN, openLog)) {
						pageHeader->lastRecord = LOG_PAGE_OFFSET(previousLSN, openLog);
					} else {
						pageHeader->lastRecord = FIRST_LSN;
					}
					endHeader->lastRecord = pageHeader->lastRecord;
#ifdef DEBUG
					waitSemaphore(&(openLog->tailBuffer->semaphore));
#endif
					DIRTY_PAGE(openLog->tailBuffer);
#ifdef DEBUG
					signalSemaphore(&(openLog->tailBuffer->semaphore));
#endif
					SM_ASSERT(LEVEL_3, auditLogPage(openLog->tailLink) == esmNOERROR);

					PRINT_PROGRESS(("Cleaning up end of log -- was:%d now:%d\n", recordLSN, previousLSN));
				}
				break;
			}

			/*
			 *	get a pointer to the record header
			 */
			record = (LOGRECORDHDR *) logRecordBuffer->bufFrame;
		}

		SM_ASSERT(LEVEL_3, recordLSN == record->recordLSN.offset);

#ifdef DEBUG
		/* make sure we don't overwrite important log records */
		debugLSN.offset = openLog->tailLSN;
		debugLSN.wrapCount = openLog->wrapCount;
		debugLSN.wrapCount--;
		if(compareLSN(&debugLSN, &OldestDirtyPageLSN) >= 0) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}
#endif
		/*
		 *	switch on the phase
		 */
		switch (phase)	{

			case ANALYSIS_PHASE:

				analyzeRecord(record, &(record->recordLSN));
				break;

			case REDO_PHASE:

				redoRecord(record, recordLSN);
				break;

			default:

				SM_ERROR(TYPE_FATAL, esmINTERNAL);
				break;
		}
		previousLSN = recordLSN;

#ifdef DEBUG
		previousLogRecord = record;
#endif DEBUG

		/*
		 *	check to see if the record crosses a page boundary
		 */
		if ((offset + length) >= openLog->lastUsableByte)	{

			/*
			 *	calculate the number of bytes on the next page
			 */
			left = length - (openLog->lastUsableByte - offset);
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("left:%d", left));

			/*
			 *	check how many more pages this record will reside on
			 *
			 *  Bug fix: added by MJZ: 6/6/92
			 */
			numPages = 1 + left/openLog->usableBytes;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("numPages:%d", numPages));

			/*
			 *	calculate the address of the next record
			 */
			recordLSN = (recordLSN + length + (numPages * (2 * FIRST_LSN)));
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));

		} else {

			/*
			 *	the next record header lies on the same page
			 *	this cannot cause a log wrap
			 */
			recordLSN += length;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));
		}

		/*
		 *	calculate the record offset in the page
		 */
		offset = (recordLSN & openLog->pageMask);

		/*
		 *	check to see if the record can fit on this page
		 */
		if ((openLog->lastUsableByte - offset) < sizeof(LOGRECORDHDR))	{

			/*
			 *	go to the next page
			 */
			page = LSN_TO_LOG_PAGE(recordLSN, openLog);
			recordLSN = LOG_PAGE_TO_LSN((page + 1), openLog) + FIRST_LSN;
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));
		}

		/*
		 *	wrap the log if necessary
		 */
		recordLSN %= openLog->fileBytes;
		TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("recordLSN:%d", recordLSN));
	}

	if (logRecordBuffer != NULL) {
		freeLogRecordBuffer(logRecordBuffer);
		logRecordBuffer = NULL;
	}

	/*
	 *	return the record lsn
	 */
	TRPRINT(TR_RECOVER|TR_LOG, TR_LEVEL_1, ("last LSN:%d", recordLSN));
	return(recordLSN);
}
