/*
 *   $RCSfile: redoRecord.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"
#include "redo_extfuncs.h"


 void
redoRecord (

	register LOGRECORDHDR	*record,
	register LSNOFFSET		recordLSN 
)
{

	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("record type: %d", record->type));

	/*
	 *	process the record
	 */
	switch (record->type)	{

		case LOG_REC_TYPE_COMMIT:

			/*
			 *	we should not do this here because we now have atomic
			 *	logging cum state change
			 *
			 *	redoCommit(record, recordLSN);
			 */
			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("commit record"));
			break;

		case LOG_REC_TYPE_ABORT:

			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("abort record"));
			break;

		case LOG_REC_TYPE_COORD_PREPARE:

			redoCoordPrepare(record, recordLSN);
			break;

		case LOG_REC_TYPE_SERVER_PREPARE:

			redoServerPrepare(record, recordLSN);
			break;

		case LOG_REC_TYPE_END:

			/*
			 *	we should not do this here because we now have atomic
			 *	logging cum state change
			 *
			 *	redoEnd(record, recordLSN);
			 */

			break;

		case LOG_REC_TYPE_CHECKPOINT:

			TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_2, ("checkpoint record"));
			break;

		case LOG_REC_TYPE_COMPENSATION:
		case LOG_REC_TYPE_USER:
		case LOG_REC_TYPE_USER_COMPENSATION:

			redoUser(record, recordLSN);
			break;

		case LOG_REC_TYPE_DIRTYPAGELIST:
		case LOG_REC_TYPE_MOUNT_VOLUME:
		case LOG_REC_TYPE_DISMOUNT_VOLUME:
			break;

		default:

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}
}
