/*
 *   $RCSfile: sortUndoList.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "openlog.h"
#include "logrecs.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"
#include "recover_intfuncs.h"
#include "recover_globals.h"

/*
 *	Sorts the list of transaction by the LSN of the last log record
 *	generated by the transaction.  The order is most recent LSN to 
 *	oldest LSN.
 *
 * 	The algorithm is to find the largest LSN on the list, and move
 *	that element to a new list.  Repeat this until the original list
 *	is empty.  The new list is now sorted.  Sure, its and n**2
 *	algorithm, but we're only dealing with a small number of 
 *	transactions.
 */


 void
sortUndoList (
	LIST					*transList,
	LSNOFFSET				lastLSN 
)
{

	TRANSREC	*undoTransRec;
	TRANSREC	*saveTransRec;

	LIST		tempList;

	TRACE(TR_RECOVER, TR_LEVEL_1);

	initializeList(&tempList);

	/* move the list to the temp location */
	listAppend(&tempList, transList);


	while (LIST_NOT_EMPTY(&tempList)) {

		/*
		 *	get a pointer to the first list element
		 */
		undoTransRec = (TRANSREC *) FIRST_LIST_ELEMENT( &tempList );

		/*
		 *	assume undoTransRec is the one we want to undo first
		 */
		saveTransRec = undoTransRec;

		/*
		 *	search down the list
		 */
		while (undoTransRec != NULL)	{

			/*
			 *	check the entry magic number
			 */
			CHECK_TRANSREC_MAGIC(undoTransRec);

			TRPRINT(TR_RECOVER, TR_LEVEL_2, ("nextUndoLSN:%d", undoTransRec->nextUndoLSN));

			/*
			 *		|-----|newestLSN|------|lastLSN|---------|
			 */
			if (saveTransRec->nextUndoLSN < lastLSN)	{

				/*
				 *	check to see if the current last lsn is newer
				 */
				if ((undoTransRec->nextUndoLSN > saveTransRec->nextUndoLSN) &&
					(undoTransRec->nextUndoLSN < lastLSN))	{

					saveTransRec = undoTransRec;
					TRPRINT(TR_RECOVER, TR_LEVEL_2, ("nextUndoLSN:%d",
							saveTransRec->nextUndoLSN));
				}
				
			} else if (saveTransRec->nextUndoLSN > lastLSN)	{

				/*
				 *	|-------|lastLSN|--------|newestLSN|------|
				 */
				if ((undoTransRec->nextUndoLSN > saveTransRec->nextUndoLSN) ||
					(undoTransRec->nextUndoLSN < lastLSN))	{

					saveTransRec = undoTransRec;
					TRPRINT(TR_RECOVER, TR_LEVEL_2, ("nextUndoLSN:%d",
							saveTransRec->nextUndoLSN));
				}

			} else {

				saveTransRec = undoTransRec;
				TRPRINT(TR_RECOVER, TR_LEVEL_2, ("nextUndoLSN:%d",
						saveTransRec->nextUndoLSN));
			}

			/*
			 *	get a pointer to the next record
			 */
			undoTransRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(undoTransRec->activeTransList) );
		}

		/*	move the selected transrec to the active list */
		listMoveEnq(&ActiveTransList, &(saveTransRec->activeTransList));
	}

#ifdef DEBUG
	/* print the transaction undo order */
	PRINT_PROGRESS(("Transaction undo order\n"));	

	/* get a pointer to the first list element */
	undoTransRec = (TRANSREC *) FIRST_LIST_ELEMENT( &ActiveTransList );

	/*
	 *	search down the list
	 */
	while (undoTransRec != NULL)	{

		/* check the entry magic number */
		CHECK_TRANSREC_MAGIC(undoTransRec);

		PRINT_PROGRESS(("  tid:%d  nextUndoLSN:%d\n", undoTransRec->tid, undoTransRec->nextUndoLSN));

		/* get a pointer to the next record */
		undoTransRec = (TRANSREC *) NEXT_LIST_ELEMENT( &(undoTransRec->activeTransList) );
	}
#endif

	return;
}
