/*
 * $RCSfile: redoBtOv.C,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1996/05/04 21:55:58 $
 */
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "OVPAGE.h"
#include "BT_Log.h"
#include "bf_macro.h"
#include "redo_intfuncs.h"


#ifdef DEBUG
int redoBtOvTrace = FALSE;
#undef BT_TRACE
#define BT_TRACE(x)  { if (redoBtOvTrace)  printf x; }
#else
#define BT_TRACE(x)  
#endif 	/* def DEBUG */


void redoBtOv(LOGRECORDHDR* recHdr)
{

	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", recHdr->recordLSN.offset));

	PID* pid = & recHdr->actionPid;
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	//
	//	Check to see if the operation needs to be redone
	//
	GROUPLINK* gLink;
	if (!redoCheckPage(recHdr, &gLink, PAGE_INDEX))  {
		//
		//	Don't need to redo
		//
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
		return;
		}
	CHECK_GROUPLINK_MAGIC(gLink);

    OVPAGE* op = (OVPAGE*) gLink->bufFrame;
	void* vptr = GET_LOG_IMAGE(recHdr, 0);

	switch (recHdr->action) {
	
		case LOG_ACTION_BTOV_PAGE_INIT:
			{
			LogBtOvPageInitData* rp = (LogBtOvPageInitData*) vptr;
			BT_TRACE(("R: btov init %d\n", pid->page));
			op->Init(*pid, rp->elSize);
			}
			break;
	
		case LOG_ACTION_BTOV_MODIFY_NEXT_LINK:
			{
			LogBtOvModifyNextLink* rp = (LogBtOvModifyNextLink*) vptr;
			BT_TRACE(("R: btov modify next (%d)\n", pid->page));
			ASSERT3(op->NextPage() == rp->oldNext);
			op->NextPage() = rp->newNext;
			}
			break;
			
		case LOG_ACTION_BTOV_MODIFY_PREV_LINK:
			{
			LogBtOvModifyPrevLink* rp = (LogBtOvModifyPrevLink*) vptr;
			BT_TRACE(("R: btov modify prev (%d)\n", pid->page));
			ASSERT3(op->PrevPage() == rp->oldPrev);
			op->PrevPage() = rp->newPrev;
			}
			break;
			
		case LOG_ACTION_BTOV_INSERT:
			{
			LogBtOvInsert* rp = (LogBtOvInsert*) vptr;
			BT_TRACE(("R: btov insert (%d)\n", pid->page));
			op->Insert(rp->startSlot, rp->numSlots, rp->data);
			}
			break;
		
		case LOG_ACTION_BTOV_DELETE:
			{
			LogBtOvDelete* rp = (LogBtOvDelete*) vptr;
			BT_TRACE(("R: btov delete (%d)\n", pid->page));
			op->Remove(rp->startSlot, rp->numSlots);
			}
			break;
	
		default:
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	op->lrc() = recHdr->actionLRC;

	signalSemaphore(&gLink->pageHash->semaphore);

	bf_UnfixPage(gLink, BF_DEFAULT, TRUE);
}
