/*
 *   $RCSfile: redoBtreePageFormat.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/
#include "BTREEPAGE.h"
#include "redo_intfuncs.h"


/*
 *	Function	: void redoBtreePageFormat(LOGRECORDHDR* record)
 *	Description	: Redo a btree page format (only the LRC field)
 */
void redoBtreePageFormat(LOGRECORDHDR* record)
{

	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	PID* pid = & record->actionPid;
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	/*
	 *	Check to see if the page is in the dirty page list
	 */
	DIRTYPAGEINFO*	dirtyInfo = searchDirtyPageTable(pid);
	if ( ! dirtyInfo )  {
		/*
		 *	Don't need to redo
		 */
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
		return;
		}

	/*
	 *	Check to see if the lrc on the page is greater than
	 *	the lrc in the log record
	 */
	if (CHECK_PAGE_LRC_LESS_DIRTYINFO(record->actionLRC, record->recordLSN, dirtyInfo))  {
		/*
		 *	Don't need to redo
		 */
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty info page lrc later"));
		return;
		}


	GROUPLINK* groupLink = bf_ReadPage(UserBufGroup, pid, 
											BTREE_PAGE2SIZE, BF_SEM|BF_NOREAD);
	if ( ! groupLink ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return;
		}

	BTREEPAGE* btPage = (BTREEPAGE*) groupLink->bufFrame;

	/*
	 * 	Note: for REDO_ALWAYS operation, no need to check if operation
	 *		  has already been performed.
	 */
	

	/*
	 *	Mark LSN on the page.
	 *	Set up the firstLSN/LRC for the page
	 */
	btPage->lrc() = record->actionLRC;
	DEPEND_LOG(groupLink->pageHash, 0, &record->recordLSN, &record->actionLRC);
	TRPRINT(TR_RECOVER, TR_LEVEL_2, ("marking new page lrc:%d, ", 
															btPage->lrc()));
	
	signalSemaphore( & groupLink->pageHash->semaphore);

	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);
}
