/*
 *   $RCSfile: redoCommit.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "logrecs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "log_intfuncs.h"
#include "log_extfuncs.h"
#include "recover_intfuncs.h"
#include "recover_extfuncs.h"
#include "redo_extfuncs.h"
#include "bf_extfuncs.h"
#include "thread_globals.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_extfuncs.h"


 void
redoCommit (

	register LOGRECORDHDR	*record,
	register LSNOFFSET		unused 
)
{
	TRANSREC				*transRec;

	TRACE(TR_LOG|TR_RECOVER, TR_LEVEL_1);

	if ((transRec = findTransRec(record->tid)) == NULL)	{
		return;
	}

	TRPRINT(TR_LOG|TR_RECOVER, TR_LEVEL_1, ("action %d", record->action));
	/*
	 *	process the record
	 */
	switch (record->action)	{

		case LOG_ACTION_DISTR_COORD_COMMIT:

			if (transRec->transState != T_COMMIT) {

				/*
				 *	mark the state of the transaction
				 */
				transRec->transState = T_COMMIT;

				/*
				 *  initialize the number of acks received
				 */
				transRec->numAcksRecd = 0;

				/*
				 *  make sure this trans is on list of distr trans
				 */
				SM_ASSERT(LEVEL_3, 
						  (findDistrTransRec(transRec->tid, &CoordDistrTransList) != NULL));
			}

			break;

		case LOG_ACTION_DISTR_SERVER_COMMIT:

			/*
			 *	Remove all pages from the dirty page table which were
			 *	deallocated by this transaction.
			 */
			removeDeallocedDirtyPages(transRec);

			/*
 			 *  initialize the state to inactive
 			 */
			transRec->transState = T_INACTIVE;

			/*
			 *  initialize the list components
			 */
			listRemove( &(transRec->activeTransList) );

			/*
 			 *  check if this is a distributed transaction
 			 */
			if (LIST_MEMBER(&(transRec->distrTransList))) {

				/*
				 *  take it off the list of active distr transactions
				 */
				listRemove( &(transRec->distrTransList) );

				/*
				 *  decrement the number of active distr transactions
				 */
				numActiveServerDistrTrans--;
			}

			/*
 			 *  move the transaction to the free pool
 			 */
			poolMove( &TransRecPool, &(transRec->tidList) );

			break;

		case NULL:
			
			break;

		default:

			TRPRINT(TR_LEVEL_1, TR_RECOVER,
				("redoCommit() : unknown action %d", record->action));
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}
}
