/*
 *   $RCSfile: redoCreateIndex.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "pool.h"
#include "volume.h"
#include "logrecs.h"
#include "trans.h"
#include "openlog.h"
#include "bitmap.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "trans_extfuncs.h"
#include "redo_intfuncs.h"
#include "redo_extfuncs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "log_extfuncs.h"
#include "sm_extfuncs.h"
#include "sm_macro.h"
#include "bf_macro.h"

static int check(LOGRECORDHDR* record, GROUPLINK** gLinkPtr);

 void
redoCreateIndex (

	LOGRECORDHDR		*record  
)
{
	INDEXDESC* desc;
	GROUPLINK* gLink;
	PID* pid;
	INDEXDESCPAGE* ip;
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	pid = & record->actionPid;
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	desc = (INDEXDESC*) GET_LOG_IMAGE(record, 0);

	/*
	 *	Check to see if the operation needs to be done
	 */
	if (!check(record, &gLink))  {
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
		return;
		}

	CHECK_GROUPLINK_MAGIC(gLink);

	ip = (INDEXDESCPAGE*) gLink->bufFrame;

	/*
	 * 	Redo 
	 */
	ip->desc = *desc;
	ip->magic = NDXMAGIC;

	ip->lrc = record->actionLRC;

	signalSemaphore(&gLink->pageHash->semaphore);

	bf_UnfixPage(gLink, BF_DEFAULT, TRUE);
}



static int check(LOGRECORDHDR* record, GROUPLINK** gLinkPtr)
{
	PID* pid = &record->actionPid;
	INDEXDESCPAGE* ip;
	DIRTYPAGEINFO* dirtyInfo = searchDirtyPageTable(pid);
	if ( ! dirtyInfo) {
		return FALSE;
		}

	if ( (compareLRC(&record->actionLRC, &dirtyInfo->lrc) < 0) || 
	     (compareLSN(&(record->recordLSN), & (dirtyInfo->lsn)) < 0)) {
		return FALSE;
		}

	SM_ASSERT(LEVEL_3, compareLSN(&record->recordLSN, &dirtyInfo->lsn) >= 0);

	if ((*gLinkPtr = bf_ReadPage(UserBufGroup, pid, 
							INDEXDESC_PAGE2SIZE, BF_SEM)) == NULL)  {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	ip = (INDEXDESCPAGE*) (*gLinkPtr)->bufFrame;

	if (compareLRC( &record->actionLRC, & ip->lrc) <= 0) {
		dirtyInfo->lrc = ip->lrc;
		signalSemaphore( & (*gLinkPtr)->pageHash->semaphore);
		bf_UnfixPage(*gLinkPtr, BF_DEFAULT, FALSE);
		return FALSE;
		}

	DEPEND_LOG((*gLinkPtr)->pageHash, 0, & record->recordLSN, 
												& record->actionLRC);

#ifdef DEBUG
#ifdef INIT_LRC_IS_LSN
#else
	LRC tempLRC = ip->lrc;
	INCREMENT_LRC(&tempLRC);
	SM_ASSERT(LEVEL_1, compareLRC(&record->actionLRC, &tempLRC) == 0);
#endif
#endif

	return TRUE;
}
