/*
 *   $RCSfile: redoCreateSmallObj.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "pool.h"
#include "volume.h"
#include "logrecs.h"
#include "trans.h"
#include "openlog.h"
#include "logprepare.h"
#include "bitmap.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "trans_extfuncs.h"
#include "redo_intfuncs.h"
#include "redo_extfuncs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "log_extfuncs.h"
#include "sm_extfuncs.h"
#include "sm_macro.h"
#include "bf_macro.h"


 void
redoCreateSmallObj (

	LOGRECORDHDR		*record  
)
{ 
	PID					*pid; 
	GROUPLINK			*groupLink;
	SLOTTEDPAGE			*slottedPage;
	SMALLOBJ			*logObj;
	WRITEINFO			*opInfo; 		/* info on logged operation */
	int					objSize;

	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	/*
	 *	get a pointer to the page for the record
	 */
	pid = &(record->actionPid);
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	/*
	 *	get pointer to oid, header info
	 *	get a pointer to the new data and its length
	 */
	opInfo = (WRITEINFO*) GET_LOG_IMAGE(record, 0);
	logObj = (SMALLOBJ*) GET_LOG_IMAGE(record, 1);
	if (record->action == LOG_ACTION_CREATE_SMALL_OBJ) {
		objSize = GET_LOG_IMAGE_SIZE(record, 1) - sizeof(OBJHDR);
	} else {
		SM_ASSERT(LEVEL_3, record->action == LOG_ACTION_CREATE_SMALL_OBJ_ND);
		/* only small and forwarded objects allowed */
		SM_ASSERT(LEVEL_3, (logObj->objectHdr.properties | P_FORWARD) == P_FORWARD);
		objSize = (int) logObj->objectHdr.size;
	}

    /*
     *  See if the operation needs to be redone
     */
    if (!redoCheckPage(record, &groupLink, PAGE_SLOTTED)) {
        /* don't need to redo */
        TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
        return;
    }
    CHECK_GROUPLINK_MAGIC(groupLink);

	/* get a pointer to the slotted page */
	slottedPage = (SLOTTEDPAGE *) groupLink->bufFrame;
	if (CHECK_SLOTTED_MAGIC(slottedPage)) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/* Recreate the object */
	if (logObj->objectHdr.properties & P_LARGEOBJ) {
		slottedPage->header.largeCount++;
	}
	if (sm_RecreateSmallObj(record->type, slottedPage, logObj, 
							opInfo->slot, opInfo->unique, objSize,
							(record->action == LOG_ACTION_CREATE_SMALL_OBJ_ND) )) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/* mark the lrc on the page */
	slottedPage->header.lrc = record->actionLRC;
	TRPRINT(TR_RECOVER, TR_LEVEL_2, ("marking new page lrc:%d, ", slottedPage->header.lrc.count));

	/* release the semaphore */
	signalSemaphore( &(groupLink->pageHash->semaphore) );

	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);
}
