/*
 *   $RCSfile: redoEntirePage.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "pool.h"
#include "volume.h"
#include "logrecs.h"
#include "trans.h"
#include "openlog.h"
#include "bitmap.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "trans_extfuncs.h"
#include "redo_extfuncs.h"
#include "redo_intfuncs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "log_extfuncs.h"
#include "sm_macro.h"
#include "bf_macro.h"


 void
redoEntirePage (

	LOGRECORDHDR		*record 
)
{

	PID					*pid;
	GROUPLINK			*groupLink;
	char				*data;
	int					dataLength;
	PAGETYPE			pageType;


	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	/* get a pointer to the page for the record */
	pid = &(record->actionPid);
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	switch(record->action) {
	case LOG_ACTION_ENTIRE_SLOTTED:
		pageType = PAGE_SLOTTED;
		break;
	case LOG_ACTION_ENTIRE_LARGE_NODE:
		pageType = PAGE_LARGENODE;
		break;
	case LOG_ACTION_ENTIRE_LARGE_DATA:
		pageType = PAGE_LARGEDATA;
		break;
	case LOG_ACTION_ENTIRE_BTREE_PAGE:
		pageType = PAGE_INDEX;
		break;
	case LOG_ACTION_ENTIRE_INDEXDESC:
		pageType = PAGE_INDEXDESC;
		break;
	default:
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	data = GET_LOG_IMAGE(record, 0);
	dataLength = GET_LOG_IMAGE_SIZE(record, 0);

	/*
	 *	See if the operation needs to be redone
	 */
	if (!redoCheckPage(record, &groupLink, pageType)) {
		/* don't need to redo */
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
		return;
	}
	CHECK_GROUPLINK_MAGIC(groupLink);


	/*
	 *	redo the entire page
	 */
	bcopy(data, groupLink->bufFrame,  dataLength);

	/* signal the semaphore */
	signalSemaphore( &(groupLink->pageHash->semaphore) );

	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);
}
