/*
 *   $RCSfile: redoGetServers.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "bitmap.h"
#include "openlog.h"
#include "logaction.h"
#include "io_extfuncs.h"
#include "io_logfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "trans_extfuncs.h"
#include "redo_intfuncs.h"
#include "redo_extfuncs.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "log_globals.h"
#include "log_extfuncs.h"
#include "logprepare.h"
#include "distr.h"
#include "distr_globals.h"
#include "distr_intfuncs.h"


 void
redoGetServers (

	LOGRECORDHDR			*record,
	TRANSREC				*transRec
)
{

	register int			i;	
	PREPARESERVERINFO		*serverInfo;
	SERVERTIDINFO			*serverTidInfo;
	int						numServers;


	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	/*
	 *	store the number of servers
	 */
	numServers = transRec->numServers =  (int) GET_LOG_IMAGE_SIZE(record, 0)/sizeof(PREPARESERVERINFO);

	for(serverInfo = (PREPARESERVERINFO *)GET_LOG_IMAGE(record, 0), i=0;
		i < numServers;
		i++, serverInfo++) {

		/*
		 *  get a serverTidInfo structure
		 */
		if ((serverTidInfo = (SERVERTIDINFO *) poolDeq( &ServerTidInfoPool)) == NULL) {
			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

		/*
		 *  initialize the serverTidInfo
		 */
		initServerTidInfo(serverTidInfo);

		/*
		 *	copy the fields into the serverTidInfo
		 */
		serverTidInfo->connectAddr = serverInfo->connectAddr;
		serverTidInfo->tid = serverInfo->tid;

		/*
		 *	note that the server returned a yes vote
		 */
		if ((transRec->transState == T_PREPARED) || 
			(transRec->transState == T_COMMIT)) {

			serverTidInfo->vote = YESVOTE;
		}

		/*
		 *  add this entry to the list of servers in the transRec
		 */
		listEnq( &(transRec->serverList), &(serverTidInfo->serverTidInfoList) );
	}

	/*
	 *	initialize the number of votes received
	 */
	transRec->numVotesRecd = numServers;

	/*
	 *	return success
	 */
	return;

}
