/*
 *   $RCSfile: redoLgDataPageFormat.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:58 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "pool.h"
#include "volume.h"
#include "logrecs.h"
#include "trans.h"
#include "openlog.h"
#include "bitmap.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "trans_extfuncs.h"
#include "redo_extfuncs.h"
#include "logaction.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "bf_globals.h"
#include "log_globals.h"
#include "log_extfuncs.h"


 void
redoLgDataPageFormat (

	LOGRECORDHDR		*record 
)
{

	register PID		*pid;
	register GROUPLINK	*groupLink;
	LGDATAPAGE			*lgDataPage;
	DIRTYPAGEINFO		*dirtyInfo;


	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_1, ("lsn:%d", record->recordLSN.offset));

	/*
	 *	get a pointer to the page for the record
	 */
	pid = &(record->actionPid);
	TRPRINT(TR_IO|TR_LOG, TR_LEVEL_2, ("pid:%d", pid->page));

	/*
	 *	check to see if the page is in the dirty page list
	 */
	if ((dirtyInfo = searchDirtyPageTable(pid)) == NULL)	{

		/*
		 *	don't need to redo
		 */
		TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty page not present"));
		return;
	}

    /*
     *  check to see if the lrc on the page is greater than
     *  the lrc in the log record.
     *  Also check the lsn for the case of pages which never made it
     *  back from the client.
     */
    if (CHECK_PAGE_LRC_LESS_DIRTYINFO(record->actionLRC, record->recordLSN, dirtyInfo)) {

        /*
         *  don't need to redo
         */
        TRPRINT(TR_RECOVER, TR_LEVEL_2, ("dirty info page lrc later"));
        return;
    }

	/*
	 *	read in the page
	 */
	if ((groupLink = bf_ReadPage(UserBufGroup, pid, LGDATA_PAGE2SIZE, BF_SEM|BF_NOREAD)) == NULL) {

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

	/*
	 *	get a pointer to the node page 
	 */
	lgDataPage = (LGDATAPAGE *) groupLink->bufFrame;

/*****************************************************************
 *	NOTE: the format of a lg obj data page is a REDO_ALWAYS operation,
 *	      therefore, no check is made to see if the operation has
 *        already been performed.
 *****************************************************************/

	/*
	 *	mark the lsn on the page, and fill in the magic number
	 *	Set up the firstLSN/LRC for the page
	 */
	lgDataPage->header.lrc = record->actionLRC;
	INIT_LGDATAHDR_MAGIC(lgDataPage);
	DEPEND_LOG(groupLink->pageHash, 0, &(record->recordLSN), &(record->actionLRC));
	TRPRINT(TR_RECOVER, TR_LEVEL_2, ("marking new page lrc:%d, ", lgDataPage->header.lrc.count));

	/*
	 *	signal the semaphore
	 */
	signalSemaphore( &(groupLink->pageHash->semaphore) );

	/*
	 *	unfix the page
	 */
	bf_UnfixPage(groupLink, BF_DEFAULT, TRUE);
}
