/*
 *   $RCSfile: findResource.c,v $  
 *   $Revision: 1.2 $  
 *   $Date: 1996/05/05 03:16:56 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "bitvec.h"
#include "msgvector.h"
#include "resource.h"
#include "resource_funcs.h"
#include "resource_globals.h"


 RESOURCE
*findResource (

	RESOURCEID		id,
	UNIQUE			unique,
	struct Link_s			*link 
)
{

	register RESOURCE	*resource;


	TRPRINT(TR_MSG, TR_LEVEL_1, ("resource id:%d", id));

	/*
	 *	check to make sure the resource id is valid
	 */
	if ((id < 0) || (id >= NumResources))	{

		SM_ERROR(TYPE_USER, esmBADRESOURCE);
		return(NULL);
	}

	/*
	 *	get a pointer to the resource
	 */
	resource = &Resources[id];

	/*
	 *	check the resource magic
	 */
	CHECK_RESOURCE_MAGIC(resource);

	/*
	 *	check the active flags
	 */
	if (!(resource->flags & RESOURCE_ACTIVE))	{

		SM_ERROR(TYPE_USER, esmBADRESOURCE);
		return(NULL);
	}

	/*
	 *	initialize the resource
	 */
	if (resource->unique != unique)	{

		SM_ERROR(TYPE_USER, esmBADRESOURCE);
		return(NULL);
	}

	/*
	 *	check to see that the link controls
	 */
	if (resource->link != link)	{

		SM_ERROR(TYPE_USER, esmBADRESOURCE);
		return(NULL);
	}

	/*
	 *	return the resource
	 */
	return(resource);
}
