/*
 *   $RCSfile: signalSemaphore.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:59 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "thread_globals.h"
#include "threadstate.h"


 void
signalSemaphore (

	register SEMAPHORE	*semaphore 

)
{

	register TCB		*tcb;


	TRACE(TR_SEM, TR_LEVEL_1);

	/*
	 *	check the semaphore magic number
	 */
	CHECK_SEMAPHORE_MAGIC(semaphore);

	/*
	 *	check to see if we hold the semaphore
	 */
	SM_ASSERT(LEVEL_3, semaphore->holder == Active);

	/*
	 *	check to see if there are waiters
	 */
	if (LIST_EMPTY( &(semaphore->waitList) ))	{

		/*
		 *	reset the holder information
		 */
		semaphore->holder = NULL;

	} else {

		/*
		 *	get the first waiter off the queue
		 */
		tcb = (TCB *) listDeq( &(semaphore->waitList) );
		SM_ASSERT(LEVEL_3, tcb->state == THREAD_SEMAPHORE_WAIT);

		/*
		 *	record that the semaphore has been given
		 */
		semaphore->holder = tcb;
		
		/*
		 *	put the holder on the ready list
		 */
		TRPRINT(TR_SEM, TR_LEVEL_2, ("signalling thread:%d", tcb->id));
		listEnq( &(ReadyList), &(tcb->controlList) );
	}

	Active->semCount--;
}

