/*
 *   $RCSfile: waitSemaphore.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:55:59 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "threadstate.h"
#include "thread_globals.h"


 int
waitSemaphore (

	register SEMAPHORE	*semaphore 
)
{

	TRACE(TR_SEM, TR_LEVEL_1);

	/*
	 *	check the semaphore magic number
	 */
	CHECK_SEMAPHORE_MAGIC(semaphore);

	/*
	 *	check to see if we hold the semaphore
	 */
	SM_ASSERT(LEVEL_3, semaphore->holder != Active);

	/*
	 *	check to see if it is free
	 */
	if (semaphore->holder == NULL)	{

		TRPRINT(TR_SEM, TR_LEVEL_2, ("semaphore is free"));

		/*
		 *	get the semaphore
		 */
		semaphore->holder = Active;

	} else {

		TRPRINT(TR_SEM, TR_LEVEL_2, ("thread:%d waiting for semaphore", Active->id));

		/*
		 *	wait for semaphore
		 */
		Active->error = esmNOERROR;
		listEnq( &(semaphore->waitList), &(Active->controlList) );
		dispatch(THREAD_SEMAPHORE_WAIT);

		/*
		 *	return the code that is placed in the tcb
		 */
		if (Active->error == esmNOERROR) Active->semCount++;
		return(Active->error);
	}

	/*
	 *	return success
	 */
	Active->semCount++;
	return(esmNOERROR);
	
}
