/*
 *   $RCSfile: threadFork.c,v $  
 *   $Revision: 1.2 $  
 *   $Date: 1996/05/04 23:51:55 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "threadstate.h"
#include "thread_funcs.h"
#include "thread_globals.h"


 int
threadFork (

	int				runThread,
	PFI				routine,
	register int	argCount,
	register FOUR	*arguments 
)
{
#ifdef _IBMR2
	FCN_DESC*  fcn_desc;
#endif

	register TCB	*tcb;
	register int	i;


	TRACE(TR_THREAD, TR_LEVEL_1);

	/*
	 *	attempt to get a thread off the free queue
	 */
	while ((tcb = (TCB *) listDeq( &FreeList )) == NULL)	{


		PRINT_PROGRESS(("Waiting to fork a thread -- thread\n"));

		/*
		 *	Wait for a free thread
		 */
		if (waitList(&ThreadForkWaitList, THREAD_FORK_WAIT)) {
			SM_ERROR(TYPE_FATAL, Active->errno);
		}
	}
	TRPRINT(TR_THREAD, TR_LEVEL_2, ("dispatching thread:%d", tcb->id));

	/*
	 *	load in the pc of the routine to run
	 */
#ifdef mips
	tcb->currentBuf[JB_PC] = (int) routine;
	tcb->currentBuf[JB_SP] = tcb->initBuf[JB_SP];
#elif defined(linux)
     LINUX_JB_PC(tcb->currentBuf) = (int) routine;
     LINUX_JB_SP(tcb->currentBuf) = LINUX_JB_SP(tcb->initBuf);
#elif defined(sparc)
	tcb->currentBuf[JB_PC_SPARC] = (int) routine;
	tcb->currentBuf[JB_SP_SPARC] = tcb->initBuf[JB_SP_SPARC];
		/*
		 *	NPC is set to 4 from the pc
		 */
	tcb->currentBuf[JB_NPC_SPARC] = ((int) routine) + sizeof(int);  
#elif defined(_IBMR2)
	fcn_desc = (FCN_DESC *) routine;
	setjmp(tcb->currentBuf);
	tcb->currentBuf[JB_SP_AIX] = (int) tcb->sp;
	tcb->currentBuf[JB_PC_AIX] = (int) fcn_desc->address;
#elif defined(hpux)
     HPPA_JB_PC(tcb->currentBuf) = (int) routine;
     HPPA_JB_SP(tcb->currentBuf) = HPPA_JB_SP(tcb->initBuf);
#elif !(defined(mips) || defined(sparc) || defined(rs6000) || defined(hpux) || defined(linux))
    not supported
#else
    /* this else should have the not supported code, but the
       ultrix cpp is broken */
#endif
	/*
	 *	load in the arguments
	 */
	SM_ASSERT(LEVEL_3, argCount <= MAX_FORK_ARGS);
#ifdef mips
	for (i = 0; i < argCount; i++, arguments++)	{

		TRPRINT(TR_THREAD, TR_LEVEL_2, ("argument %d:%x", i, *arguments));
		tcb->currentBuf[JB_A0 + i] = *arguments;
	}
#else
	/*
	 *	all other arch's should use this more portable method
	 */
	for (i = 0; i < argCount; i++, arguments++)	{

		TRPRINT(TR_THREAD, TR_LEVEL_2, ("argument %d:%x", i, *arguments));
		tcb->args[i] = *arguments;
	}
#endif

	SM_ASSERT(LEVEL_3, tcb->error == esmNOERROR && tcb->errno == esmNOERROR);

	/*
	 *	check to see if the child should run
	 */
	if (runThread == T_CHILD)	{

		TRPRINT(TR_THREAD, TR_LEVEL_2, ("starting child:%d ", tcb->id));

		/*
		 *	save the current state the the thread
		 */
		if (setjmp(Active->currentBuf) == 0)	{

			/*
			 *	put us on the ready list
			 */
			listEnq( &ReadyList, &(Active->controlList) );

			/*
			 *	jump to the new routine
			 */
			threadSwitch(tcb);

			/*
			 *	return success
			 */
			return(esmNOERROR);

		} else {

			TRPRINT(TR_THREAD, TR_LEVEL_2, ("thread:%d returned from longjmp", Active->id));
			return(esmNOERROR);
		}
	
	} else if (runThread == T_PARENT)	{

		TRPRINT(TR_THREAD, TR_LEVEL_2, ("putting child on ready list:%d ", tcb->id));

		/*
		 *	put the child on the ready list
		 */
		listEnq( &ReadyList, &(tcb->controlList) );
		return(esmNOERROR);

	} else {

		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		return(esmNOERROR);
	}
}
