/*
 *   $RCSfile: allocTrans.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "trans_globals.h"
#include "distr.h"


 TRANSREC
*allocTrans ()

{

	TRANSREC	*transRec;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	Get a transaction record
	 */
	if ((transRec = (TRANSREC *) poolDeq( &TransRecPool )) == NULL)	{

		SM_ERROR(TYPE_WARNING, esmNOFREETRANSREC);
		return(NULL);
	}

	/*
	 *	Initialize the transaction record
	 */
	transRec->transState	 = T_ACTIVE;
	transRec->firstLSN.offset = NULL_LSN;
	transRec->firstLSN.wrapCount = 0;
	transRec->lastLSN		 = NULL_LSN;
	transRec->rememberedLSN	 = NULL_LSN;
	transRec->threadCount	 = 1;
	transRec->lockWeight	 = 0;
	transRec->visitCount	 = 0;
	transRec->logRecordCount = 0;
	transRec->nextUndoLSN	 = NULL_LSN;
	transRec->masterThread	 = NULL;
	transRec->masterLink	 = NULL;
	transRec->loggingOn	 	 = TRUE;
	transRec->logSpace	 	 = 0;
	transRec->undoLogSpace	 = 0;
	transRec->gtidRec	 	 = NULL;  		/* for distr trans */
	transRec->intent	 	 = T_INACTIVE;  /* for distr trans */
	transRec->numServers	 = 0;			/* for distr trans */
	transRec->numVotesRecd	 = 0;			/* for distr trans */
	transRec->numAcksRecd	 = 0;			/* for distr trans */
	transRec->clientCommand	 = C_UNKNOWN;	/* for distr trans */
	transRec->coordCommand	 = C_UNKNOWN;	/* for distr trans */
	transRec->numLocks		 = 0;			/* for distr trans */
	transRec->prepareLSN.offset = NULL_LSN;	/* for distr trans */
	transRec->prepareLSN.wrapCount = 0; 	/* for distr trans */
	transRec->prepareAndCommit	 = FALSE;	/* for distr trans */
	initializeList(&(transRec->fileDeallocList));
	initializeList(&(transRec->pageDeallocList));
	initializeList(&(transRec->tempFileList));
	initializeList(&(transRec->tempPageList));
	initializeList(&(transRec->pendingBitmapList)); /* for distr trans */
	initializeList(&(transRec->serverList)); /* for distr trans */

	/*
	 *	hang it on the active list
	 */
	listEnq( &ActiveTransList, &(transRec->activeTransList) );

	/*
	 *	hang it off the current tcb
	 */
	listEnq( &(transRec->threadList), &(Active->transList) );

	/*
	 *	initialize the thread to point to the transaction record
	 */
	Active->transRec = (char *) transRec;
	TRPRINT(TR_TRANS, TR_LEVEL_2, ("transRec:%x", transRec));

	/*
	 *	return a pointer to the transaction record
	 */
	return(transRec);

}
