/*
 *   $RCSfile: allocTransRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "log.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"


 TRANSREC
*allocTransRec (

	POOL		*head 

)
{

	TRANSREC	*transRec;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	Allow no more transactions than can fit in a checkpoint record.
	 */
	if (head->elements >= MAXTRANSCHKPNT) {
		SM_ERROR(TYPE_USER, esmNOFREETRANSREC);
		return(NULL);
	}

	/*
	 *	Attempt to allocate the space
	 */
	if ((transRec = (TRANSREC *) malloc(sizeof(TRANSREC))) == NULL)	{

		SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
		return(NULL);
	}

	/*
	 * Initialize the transaction record
	 */
	initTransRec(transRec);

	/*
	 *	Increment the number of elements in the pool
	 */
	head->elements++;

	/*
	 *	return a pointer to the transaction record
	 */
	return(transRec);
}
