/*
 *   $RCSfile: findTransVolRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"


 TRANSVOLREC
*findTransVolRec (

	register TRANSREC		*transRec,
	register VOLREC			*volRec 
)
{

	register TRANSVOLREC	*transVolRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", GETTID(transRec)));

	/*
	 *	Get a pointer to the bucket
	 */
	transVolRec = (TRANSVOLREC *) FIRST_LIST_ELEMENT( &(transRec->volumeList) );

	/*
	 *	search down the list
	 */
	while (transVolRec != NULL)	{

		TRPRINT(TR_TRANS, TR_LEVEL_2, ("volid:%d", transVolRec->volRec->header->volid));

		/*
		 *	check the entry magic number
		 */
		CHECK_TRANSVOLREC_MAGIC(transVolRec);

		/*
		 *	try to match the volumes
		 */
		if (transVolRec->volRec == volRec)	{

			/*
			 *	return the volume 
			 */
			return(transVolRec);

		} else {

			/*
			 *	look at the next element in the list
			 */
			transVolRec = (TRANSVOLREC *) NEXT_LIST_ELEMENT( &(transVolRec->volumeList) );
		}
	}

	/*
	 *	if we are here we haven't found the item
	 */
	return(NULL);
}
