/*
 *   $RCSfile: freeTransRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "openlog.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"
#include "log_globals.h"
#include "distr.h"
#include "distr_globals.h"


 void
freeTransRec (

	register TRANSREC	*transRec 
)
{

	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	initialize the state to inactive
	 */
	transRec->transState = T_INACTIVE;

	/*
	 * 	Make sure all other resources are free
	 */
	SM_ASSERT(LEVEL_1, LIST_EMPTY(&(transRec->fileDeallocList)));
	SM_ASSERT(LEVEL_1, LIST_EMPTY(&(transRec->pageDeallocList)));
	SM_ASSERT(LEVEL_1, LIST_EMPTY(&(transRec->serverList)));

	/*
	 *	check if gtidRec needs to be returned
	 */
	if (transRec->gtidRec != NULL) {

		/*
		 *	move the gtid to the free pool
		 */
		poolEnq( &GtidRecPool, &(transRec->gtidRec->gtidList) );
	}

	/*
	 *	take the record off the communication link
	 */
	if (LIST_MEMBER( &(transRec->linkList) ))	{

		listRemove( &(transRec->linkList) );
	}

	/*
	 *	take the record off the distributed transaction link
	 */
	if (LIST_MEMBER( &(transRec->distrTransList) ))	{

		listRemove( &(transRec->distrTransList) );
	}

	/* remove log space used by trans */
	ActiveLogSpace -= transRec->logSpace;

	/*
	 *	take the record off the active list
	 */
	listRemove( &(transRec->activeTransList) );

	/*
	 *	If the active list is now empty, notify any waiter wishing
	 *	to shutdown transaction processing
	 */
	if (LIST_EMPTY(&ActiveTransList)) {
		SM_ASSERT(LEVEL_3, ActiveLogSpace == 0);
		notify(&TransShutdownWaitList, esmNOERROR, esmNOERROR);
	}

	/*
	 *	move the transaction to the free pool
	 */
	poolMove( &TransRecPool, &(transRec->tidList) );
}
