/*
 *   $RCSfile: freeTransVolRecs.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "bf_extfuncs.h"
#include "trans_globals.h"


 void
freeTransVolRecs (

	register TRANSREC		*transRec 
)
{
	register TRANSVOLREC	*transVolRec;
	register VOLREC	*volRec;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", GETTID(transRec)));

	while ((transVolRec = (TRANSVOLREC *) listDeq( &(transRec->volumeList) )) != NULL)	{

		CHECK_TRANSVOLREC_MAGIC(transVolRec);

		/* remove this record from the list hanging off the transaction */
		listRemove( &(transVolRec->transList) );

		volRec = transVolRec->volRec;

		SM_ASSERT(LEVEL_1, ((volRec->volflags & 
			(VOL_OPEN|VOL_QUIESCE|VOL_INIT|VOL_FORMAT|VOL_INVALID)) == VOL_OPEN) );
			
		/* 
		 *	If the volume is not used by any other transaction or
		 *	link (and is not the log volume) then unmount it
		 */
		if (LIST_EMPTY( &(volRec->openList) ) &&
				LIST_EMPTY( &(volRec->activeTransList) ) ) { 

			TRPRINT(TR_MSG, TR_LEVEL_2, ("last user of vol"));

			bf_DismountVolume(volRec, TRUE);
		}

		/*
		 *	put the structure back in the pool
		 */
		poolEnq( &TransVolRecPool, &(transVolRec->volumeList) );
	}
}
