/*
 *   $RCSfile: initTransRec.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "util_funcs.h"
#include "trans_intfuncs.h"


 void
initTransRec (
	register TRANSREC   *transRec 
)
{
	int i;

	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	initialize the list components
	 */
	initializeList( &(transRec->serverList) );
	initializeList( &(transRec->threadList) );
	initializeList( &(transRec->quiesceList) );
	initializeList( &(transRec->volumeList) );
	initializeList( &(transRec->clientDirtyList) );
	initializeList( &(transRec->fileDeallocList) );
	initializeList( &(transRec->pageDeallocList) );
	initializeList( &(transRec->pendingBitmapList) );
	initializeListElement( &(transRec->distrTransList), (char *) transRec );
	initializeListElement( &(transRec->tidList), (char *) transRec );
	initializeListElement( &(transRec->linkList), (char *) transRec );
	initializeListElement( &(transRec->logActiveList), (char *) transRec );
	initializeListElement( &(transRec->activeTransList), (char *) transRec );
	INIT_TRANSREC_MAGIC(transRec);

	/*
	 *	initialize the state to inactive
	 */
	transRec->transState = T_INACTIVE;

	/*
	 *	initialize the lock information
	 */
	initializeList( &(transRec->lockGrantedList) );
	initializeList( &(transRec->lockWaitList) );
	initializeList( &(transRec->lockUpgradeList) );
	transRec->numLocks = 0;

	/*
	 *	initialize the lock class information
	 */
	for (i = 0; i < NUM_LOCK_CLASSES; i++)	{

		initializeList( &(transRec->lockClassList[i]) );
	}

	/*
	 *	Null out the lock class mask
	 */
	transRec->lockClassMap = 0;

	initializeSemaphore( &(transRec->logSemaphore), FALSE);
}
