/*
 *   $RCSfile: internalAbort.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bitvec.h"
#include "logrecs.h"
#include "openlog.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "thread_funcs.h"
#include "lm_extfuncs.h"
#include "log_extfuncs.h"
#include "msg_funcs.h"
#include "thread_globals.h"
#include "trans_globals.h"

/*
 *	This function is to be called when a transaction is
 *	active and needs to be aborted.  abortTrans() should be
 *	called if the transaction is in any other state.
 */

 void
internalAbort (

	TRANSREC	*transRec,
	int			reason
)
{
	FOUR	args[2];

	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", GETTID(transRec)));

	/*
	 *	take action based on what state
	 *	the transaction is currently in
	 */
	switch (transRec->transState)	{

		case T_ACTIVE:

			/*
			 *	abort will be pending
			 */
			transRec->transState = T_LATENT;
			transRec->intent = T_ABORT;

			/*
			 *	check to see if there are threads
			 */
			if (transRec->threadCount > 0)	{

				/*
				 *	release lock waiters
				 */
				releaseLockBlocked(transRec, esmTRANSABORTED);

				/*
				 *	release log waiters
				 */
				releaseLogBlocked(transRec, esmTRANSABORTED);
			}
			
			/* Store the reason for the abort on this link */
			transRec->masterLink->linkTransError = reason;

			/*
			 *	fork off an abort
			 */
			args[0] = (FOUR) (transRec);
			args[1] = (FOUR) (reason);
			threadFork(T_PARENT, (PFI) asyncAbort, 2, args);
			break;

		case T_QUIESCE:

			/*
			 *	mark the intent to abort
			 */
			transRec->intent = T_ABORT;
			break;

		case T_LATENT:

			/*
			 *	mark the intent to abort
			 */
			transRec->intent = T_ABORT;
			break;

		case T_ABORT:

			/*
			 *	Some other thread should be aborting the transaction
			 */
			SM_ASSERT(LEVEL_3, transRec->masterThread != NULL);
			break;

		case T_COMMIT:
		case T_PREPARED:
		case T_END:
			/* too late to abort */
			break;

		default:

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
			break;
	}
}
