/*
 *   $RCSfile: internalBegin.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "openlog.h"
#include "log_globals.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "trans_globals.h"
#include "threadstate.h"


 TRANSREC
*internalBegin ()

{

	TRANSREC	*transRec;
	register OPENLOG    *openLog;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	check the state of transaction processing
	 */
	if (TransInfo.transState != T_ACTIVE)	{

		SM_ERROR(TYPE_USER, esmTRANSDISABLED);
		return(NULL);
	}

    /*
     *  get a register pointer to the openlog structure
     */
    openLog = &OpenLog;

    /*
     *  check the open log magic number
     */
    CHECK_OPENLOG_MAGIC(openLog);

	/*
	 *	Get a transaction record
	 */
	if ((transRec = allocTrans()) == NULL)	{

		return(NULL);
	}

    /*
     *  Get a transactions id.  The log must be active to get
     *  a usable id
     */
    if (TransInfo.logState != L_ACTIVE) {

        /*
         *  check to see if recovery is happening
         */
        if (TransInfo.logState == L_RECOVER)    {

            /*
             *  queue the request pending recovery completion
             */
            if (waitList( &(openLog->recoverList) , THREAD_LOG_RECOVERY_WAIT)) {

                return(NULL);
            }

        } else {

            /*
             *  reject the request
             */
            SM_ERROR(TYPE_LOG, esmLOGDISABLED);
            return(NULL);
        }
    }

	/*
	 *	plug the tid into the transrec
	 */
	transRec->tid = TransInfo.tidSequence++;
	TRPRINT(TR_TRANS, TR_LEVEL_2, ("tid:%d", GETTID(transRec)));

	/*
	 *	return the transrec	
	 */
	return(transRec);
}
