/*
 *   $RCSfile: listDirtyTransPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "threadstate.h"
#include "util_funcs.h"
#include "trans_extfuncs.h"
#include "bf_extfuncs.h"
#include "bf_globals.h"
#include "thread_globals.h"


 int
listDirtyTransPages (

	TRANSREC			*transRec,
	DIRTYPAGELOGINFO	*dirtyInfo 
)
{
	register PAGEHASH		*pageHash;
	register int			pageCount;

	TRACE(TR_TRANS, TR_LEVEL_1);


	/*
	 *	Add info on each page to the array of dirty pages to log,
	 *	and remove the page from the clientDirtyList at the same time.
	 */
	pageCount = 0;
	while ( (pageHash = (PAGEHASH*) listDeq(&(transRec->clientDirtyList))) != NULL)	{

		/*
		 *	check the pagehash magic number
		 */
		CHECK_PAGEHASH_MAGIC(pageHash);

		/*
		 *	check to see if the page is dirty
		 */
		if ((!PAGE_IS_LOG(pageHash)) && PAGE_IS_DIRTY(pageHash))	{

			/*
			 *	record the information
			 */
			dirtyInfo->pid =	   pageHash->hashList.hashPid;
			dirtyInfo->firstLSN =	   pageHash->lsn;
			dirtyInfo->firstLRC =	   pageHash->lrc;
			TRPRINT(TR_TRANS, TR_LEVEL_2, ("pid:%d lsn:%d:%d, lrc:%d:%d",
					dirtyInfo->pid.page, dirtyInfo->firstLSN.wrapCount,
					dirtyInfo->firstLSN.offset, dirtyInfo->firstLRC.wrapCount,
					dirtyInfo->firstLRC.count));

			/*
			 *	increment the count and the pointer
			 */
			dirtyInfo++;
			pageCount++;

#ifdef JUNK
			/*
			 *	If logging is turned off for this transaction,
			 *	these dirty pages should be written to disk before
			 *	commit.
			 */
			if (!transRec->loggingOn) {
				bf_ForcePage(pageHash, BF_SEM);
			}
#endif /* JUNK */

		} else {

			SM_ERROR(TYPE_FATAL, esmINTERNAL);
		}

	}

	/*
	 *	return the count of pages
	 */
	return(pageCount);
}
