/*
 *   $RCSfile: notifyTrans.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "thread_globals.h"
#include "trans_globals.h"


 int
notifyTrans (

	register TRANSREC	*transRec,
	register LIST		*list,
	int					error 
)
{

	register LIST		*element;
	register LIST		*next;
	register TCB		*tcb;
	register int		count;


	TRPRINT(TR_TRANS, TR_LEVEL_1, ("tid:%x", GETTID(transRec)));

	/*
	 *	initialize count to zero
	 */
	count = 0;

	/*
	 *	get a pointer to the first list element
	 */
	element = list->succ;

	/*
	 *	run through the list
	 */
	while (element != list)	{

		/*
		 *	get a pointer to the tcb
		 */
		tcb = (TCB *) element->item;
		CHECK_TCB_MAGIC(tcb);

		/*
		 *	check to see if the transaction matches
		 */
		if (((TRANSREC *) tcb->transRec == transRec))	{

			/*
			 *	save the next pointer
			 */
			next = element->succ;

			/*
			 *	store the error code
			 */
			tcb->error = esmFAILURE;
			tcb->errno = error;
	
			/*
			 *	move the element onto the ready list
			 */
			listMoveEnq( &ReadyList, &(tcb->controlList) );
			TRPRINT(TR_TRANS, TR_LEVEL_2, ("tcb:%d", tcb->id));

			/*
			 *	increment the count
			 */
			 count++;

			/*
			 *	get the next element
			 */
			element = next;

		} else {

			/*
			 *	get the next tcb
			 */
			 element = element->succ;
		}
	}

	/*
	 *	return the number found
	 */
	return(count);
}
