/*
 *   $RCSfile: quiesceTrans.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "logrecs.h"
#include "openlog.h"
#include "threadstate.h"
#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "log_extfuncs.h"
#include "lm_extfuncs.h"
#include "thread_globals.h"
#include "trans_globals.h"


 void
quiesceTrans (

	register TRANSREC	*transRec,
	int					error 
)
{

	TRPRINT(TR_TRANS, TR_LEVEL_1, ("threadCount:%d transState:%d",
			transRec->threadCount, transRec->transState));

	/*
	 *	mark this thread as the master
	 */
	transRec->masterThread = Active;

	/*
	 *	check to see if there are any threads that
	 *	are executing on behalf the transaction
	 */
	if (transRec->threadCount > 0)	{

		/*
		 *	assert the transaction state
		 */
		SM_ASSERT(LEVEL_1, ((transRec->transState == T_ACTIVE) ||
							(transRec->transState == T_LATENT)));

		/*
		 *	check to see if the transaction is active
		 */
		if (transRec->transState == T_ACTIVE)	{

			/*
			 *	release lock waiters
			 */
			releaseLockBlocked(transRec, error);

			/*
			 *	release log waiters if the transaction is
			 *	to be aborted.  If it is being committed, then
			 *	they should not be releases since they may be
			 *	writing important log records.
			if ((transRec->intent != T_COMMIT) && (transRec->intent != T_PREPARED)) {
				releaseLogBlocked(transRec, error);
			}
			 */
		}

		/*
		 *	place the transaction in quiesce state
		 */
		transRec->transState = T_QUIESCE;

		/*
		 *	wait for all other tasks to finish
		 */
		if (waitList( &(transRec->quiesceList), THREAD_TRANS_QUIESCE_WAIT ))	{

			SM_ERROR(TYPE_FATAL, Active->errno);
		}

		/*
		 *	there should be no other active tasks
		 */
		SM_ASSERT(LEVEL_3, transRec->threadCount == 0);

	} else {

		/*
		 *	just place the transaction in the quiesce state
		 */
		transRec->transState = T_QUIESCE;
	}
}
