#ifndef __RECOVERYDIRYTPAGETABLE_C__
#define __RECOVERYDIRYTPAGETABLE_C__
/*
 *   $RCSfile: recoveryDirtyPageTable.C,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "HashTable.h"
#include "ess.h"
#include "sysdefs.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "pool.h"
#include "resources.h"
#include "class_alias_macros.h"
#include "log.h"
#include "logrecs.h"
#include "openlog.h"

#include "trans_intfuncs.h"
#include "trans_extfuncs.h"
#include "bf_intfuncs.h"
#include "trans_globals.h"
#include "bf_globals.h"
#include "log_extfuncs.h"

BOOL 
DIRTYPAGEINFO:: Dump(FILE *f) {
	fprintf(f, 
		"DIRTYPAGEINFO pid %d.%d flags 0x%x\n", 
			this->pid.page, this->pid.volid, this->flags);
	fprintf(f, "\t lrc %d.%d lsn %d.%d, tid %d\n",
		this->lrc.wrapCount, this->lrc.count, 
		this->lsn.wrapCount, this->lsn.offset, 
		this->tid);
	return FALSE;
}; 
PID *
DIRTYPAGEINFO:: keylocation (int unique, PID **unused) { 
	switch(unique) {
	case DIRTYPAGETABLE:
		return &(this->pid);
	default:
		TRPRINT(TR_BF, TR_LEVEL_1, ("bad unique value: %d\n", unique));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		/* to prevent the compiler from complaining about no return value: */
		return &(this->pid);
	}
};
LISTELEMENT *
DIRTYPAGEINFO:: listlocation (int unique) { 
	switch(unique) {
	case DIRTYPAGETABLE:
		return &(hashList); 
	default:
		TRPRINT(TR_BF, TR_LEVEL_1, ("bad unique value: %d\n", unique));
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
		/* to prevent the compiler from 
			complaining about no return value: */
		return &(hashList); 
	}
};
void 
DIRTYPAGEINFO:: ReInit() {
	this->flags = H_FREE;
	this->lrc = LrcNone;
	this->lsn.wrapCount = 0;
	this->lsn.offset = 0;
	this->pid = PidNone;
	this->tid = NULL_TID;
};
void 
DIRTYPAGEINFO:: Init() {
	initializeListElement( this->listlocation(DIRTYPAGETABLE), this);
	this->ReInit();
};

HashTable<PID,DIRTYPAGEINFO>   	*recoveryDirtyPageTable;

void
initializeRecoveryDirtyPageTable()
{
    recoveryDirtyPageTable = 
		new HashTable<PID,DIRTYPAGEINFO>( "RecoveryDirtyPages",
			DIRTYPAGETABLE, PageHashTableSize);
	/*
	recoveryDirtyPageTable->Stats(stderr);
	recoveryDirtyPageTable->Dump(stderr, __LINE__,__FILE__);
	*/
}

 DIRTYPAGEINFO *
insertDirtyPageTable (
	PID			*pid 
)
{
	DIRTYPAGEINFO				*dirtyInfo;

	TRPRINT(TR_TRANS, TR_LEVEL_1, ("inserting volid:%d page:%d", pid->volid, pid->page));

	/* TODO: replace with a Pool */
	if ((dirtyInfo = (DIRTYPAGEINFO *) listDeq( &(DirtyPageFreeList) )) == NULL)	{
        /*
         * Malloc another DIRTYPAGEINFO struct.
         * TODO: This code should be replace with a pool 
         */
        dirtyInfo = (DIRTYPAGEINFO *) malloc(sizeof(DIRTYPAGEINFO));
        if (dirtyInfo == NULL) {
            SM_ERROR(TYPE_SYS, esmMALLOCFAILED);
            return (NULL);
        }
        initializeListElement( &(dirtyInfo->hashList), (char *) dirtyInfo);
        PRINT_PROGRESS(("fix code in insertDirtyPageTable() to better alloc new DIRTYPAGEINFO structs\n"));
	}

	dirtyInfo->ReInit();

	dirtyInfo->pid = *pid;
	recoveryDirtyPageTable->Insert(dirtyInfo);
	return dirtyInfo;
}

/*
 *	This function has two purposes.  First, it removed pages that
 *	were deallocated by a transaction from the dirty page table.
 *	Second any page which needs its firstLSN/LRC adjusted (because
 *	of an aborted transaction) has the adjustment made.
 */

static BOOL
helperfunc1(
	DIRTYPAGEINFO	*dirtyInfo,
	TRANSREC		*transRec 
)
{
	SM_ASSERT(LEVEL_1, (dirtyInfo != NULL));

	TRPRINT(TR_TRANS, TR_LEVEL_2, ("examining volid:%d page:%d",
			dirtyInfo->pid.volid, dirtyInfo->pid.page));

	/* Did this transaction dirty or deallocate the page */
	if (dirtyInfo->tid == transRec->tid ) {
	 
		/*
		 *	Was the page deallocated by this transaction
		 */
		if ( (dirtyInfo->flags & DPT_DEALLOC) ) {
			recoveryDirtyPageTable->Remove(dirtyInfo);

			/* TODO: replace with a Pool */
			listEnq(&DirtyPageFreeList, &(dirtyInfo->hashList));	

		} else {

			/* 
			 *	If this page needs its firstLSN/LRC fixed, set
			 *	it to the beginning of the transaction.
			 */
			if ( (dirtyInfo->flags & DPT_FIXFIRSTLRC) &&
				 (compareLSN(&(transRec->firstLSN), &(dirtyInfo->lsn)) < 0)) {
				dirtyInfo->lsn = transRec->firstLSN;
				SM_ASSERT(LEVEL_3, (dirtyInfo->lrc.wrapCount == 0) && (dirtyInfo->lrc.count == 1));
				/* the firstLSN for the page is now fixed */
				dirtyInfo->flags &= (~DPT_FIXFIRSTLRC);
				PRINT_PROGRESS(("Adjusting firstLSN/LRC for page %d\n", dirtyInfo->pid.page));

			}
		}
	}
	return FALSE;
} /* helperfunc1 */

 void
removeDeallocedDirtyPages (
	TRANSREC		*transRec 
)
{
	TRPRINT(TR_TRANS, TR_LEVEL_1, ("searching for tid:%d", transRec->tid));

#ifdef __GNUC__
	recoveryDirtyPageTable->ForEach(2,helperfunc1,transRec);
#else __GNUC__
	recoveryDirtyPageTable->ForEach(2, (FOREACHFUNC)helperfunc1,transRec);
#endif __GNUC__
}

/*
 *	This function finds the oldest page in the dirty page table
 *	after analysis.  As a side effect it also adjusts the firstLSN
 *	for any page which was added by an aborted transaction.  This 
 *	must be done to guarantee that redo begins at a safe point for
 *	the page.
 */


static BOOL
helperfunc2(
	DIRTYPAGEINFO	*dirtyInfo,
	DIRTYPAGEINFO	**oldestPage
)
{
	TRANSREC		*transRec;

	if (*oldestPage == NULL) *oldestPage = dirtyInfo;

	/*
	 *	See if we need to adjust the firstLSN
	 */
	if (dirtyInfo->flags & DPT_FIXFIRSTLRC) {

		/*
		 *	If the transaction is "still around", then it
		 *	it was aborted and we need to adjust the firstLSN.
		 */
		transRec = findTransRec(dirtyInfo->tid);
		if (transRec != NULL) {

			/*
			 *	If the firstLSN of the transaction is < than
			 *	the page's firstLSN, then substitute the 
			 *	transaction's firstLSN.  Also, set the firstLRC
			 *	for the page to be one.
			 */
			if (compareLSN(&(transRec->firstLSN), &(dirtyInfo->lsn)) < 0) {
				dirtyInfo->lsn = transRec->firstLSN;
				dirtyInfo->lrc.wrapCount = 0;
				dirtyInfo->lrc.count = 1;
				/* the firstLSN for the page is now fixed */
				dirtyInfo->flags &= (~DPT_FIXFIRSTLRC);
				PRINT_PROGRESS(("Adjusting firstLSN/LRC for page %d\n", dirtyInfo->pid.page));
			}
		}

		/*
		 *	Remember that it has been adjusted, just in case 
		 *	we are called again
		 */
		dirtyInfo->tid = NULL_TID;
	}

	/*
	 *	See if the is page has an older lsn, if so, mark it the oldest
	 */
	if (compareLSN(&(dirtyInfo->lsn), &((*oldestPage)->lsn)) < 0)	{
		*oldestPage = dirtyInfo;
	}
	return FALSE;
} /* helperfunc2 */

 DIRTYPAGEINFO
*findOldestDirtyPage ()

{
	DIRTYPAGEINFO	*oldestPage = NULL;

	TRPRINT(TR_TRANS, TR_LEVEL_1, (""));
#ifdef __GNUC__
	recoveryDirtyPageTable->ForEach(2,helperfunc2,&oldestPage);
#else __GNUC__
	recoveryDirtyPageTable->ForEach(2, (FOREACHFUNC)helperfunc2,&oldestPage);
#endif __GNUC__
	return(oldestPage);
}

 DIRTYPAGEINFO
*searchDirtyPageTable (
	PID			*pid 
)
{
	TRPRINT(TR_TRANS, TR_LEVEL_1, ("searchDirtyPageTable pid %d,%d", 
		pid->page, pid->volid));
	return (DIRTYPAGEINFO *)recoveryDirtyPageTable->Find(pid);
}

static BOOL
helperfunc3( 
	DIRTYPAGEINFO	*dirtyInfo,
	VOLID			volid
)
{
	if (dirtyInfo->pid.volid == volid)    {

		TRPRINT(TR_LOG, TR_LEVEL_2, ("lastLSN:%d:%d, lastLRC:%d:%d",
				dirtyInfo->lsn.wrapCount, dirtyInfo->lsn.offset,
				dirtyInfo->lrc.wrapCount, dirtyInfo->lrc.count));
		/*
		 * 	Remove the page from the list and return it to the pool
		 */
		listMoveEnq(&DirtyPageFreeList, &(dirtyInfo->hashList));
	}
	return FALSE;
} /* helperfunc3 */

void
removeDirtyPagesForVolume (
	VOLID volid
)
{
	TRPRINT(TR_TRANS, TR_LEVEL_1, ("removeDirtyPagesForVolume %d", volid));
#ifdef __GNUC__
	recoveryDirtyPageTable->ForEach(2,helperfunc3,volid);
#else __GNUC__
	recoveryDirtyPageTable->ForEach(2, (FOREACHFUNC)helperfunc3,volid);
#endif __GNUC__
}
#endif __RECOVERYDIRYTPAGETABLE_C__
