/*
 *   $RCSfile: transDeallocFilePages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "bf_macro.h"
#include "volume.h"
#include "openlog.h"
#include "trans.h"
#include "consist.h"
#include "file.h"
#include "deallocinfo.h"
#include "fi_extfuncs.h"
#include "trans_extfuncs.h"
#include "io_globals.h"
#include "bf_globals.h"


/*
 *	This function traverses the list of files to deallocate (attached
 *	to the transRec) and adds all the files pages to the list
 *	of pages to be deallocated.
 */

 int
transDeallocFilePages (

	TRANSREC		*transRec 
)
{

	FILEDEALLOCINFO *currFile;


	TRACE(TR_TRANS, TR_LEVEL_1);

	/*
	 *	See if there is any work to be done;
	 */
	currFile = (FILEDEALLOCINFO*) FIRST_LIST_ELEMENT(&(transRec->fileDeallocList));
	if (currFile == NULL) {
		return(esmNOERROR);
	}

	/*
	 *	Process each file of pages
	 */
	while ((currFile = (FILEDEALLOCINFO*) listDeq(&(transRec->fileDeallocList))) != NULL) {
	
		/*
		 *	log the destruction of the file and fake the
		 *	page deallocations
		 */
		CHECK_FILEDEALLOCINFO_MAGIC(currFile);
		if (fi_DestroyFile(UserBufGroup, &(currFile->fid.pid), &(transRec->pageDeallocList), FALSE, TRUE) != esmNOERROR) {
			return(esmFAILURE);
		}

		/*
		 *	Return to the pool
		 */
		poolEnq(&FileDeallocInfoPool, &(currFile->list));

	}

	return(esmNOERROR);
}
