/*
 *   $RCSfile: trans_FlushDirtyPages.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:02 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "latch.h"
#include "bf.h"
#include "volume.h"
#include "trans.h"
#include "bf_intfuncs.h"
#include "bf_extfuncs.h"
#include "trans_extfuncs.h"

 int 
trans_FlushDirtyPages (
	TRANSREC		*transRec
)
{
	register 	LOCKENTRY	*lockEntry;
	register 	PID		*pid;
	register	PAGEHASH *pageHash;


	TRPRINT(TR_BF, TR_LEVEL_1, ("flushing dirty pages for tid:%d", transRec->tid));

	lockEntry = (LOCKENTRY *) FIRST_LIST_ELEMENT( &(transRec->lockGrantedList) );
	for ( lockEntry = (LOCKENTRY *) 
					FIRST_LIST_ELEMENT( &(transRec->lockGrantedList) );
		lockEntry != NULL;
		lockEntry = (LOCKENTRY *) 
					NEXT_LIST_ELEMENT( &(lockEntry->transList) )
		) {

		CHECK_LOCKENTRY_MAGIC(lockEntry);

		if(lockEntry->lockHeader->hashList.lockid.page.type == PAGE_LOCK)  {

			TRPRINT(TR_BF, TR_LEVEL_1, 
				("tid %d has lock for page %d, mode %x", 
				transRec->tid,
				lockEntry->lockHeader->hashList.lockid.page.pid.page,
				lockEntry->lockMode ));

			if( (lockEntry->lockMode  == EX) ||
				(lockEntry->lockMode  == SIX) ||
				(lockEntry->lockMode  == IX) ) {

				pid = &(lockEntry->lockHeader->hashList.lockid.page.pid);

				TRPRINT(TR_BF, TR_LEVEL_1,  ("exclusive lock"));

				if ((pageHash = bf_FindPage(pid, NOFLAGS)) != NULL) {
					/* waits for the semaphore here */
					bf_ForcePage(pageHash, BF_SEM);
				} else {
					TRPRINT(TR_BF, TR_LEVEL_1, ("page not present"));
				}
			}
		}
	}
	return(esmNOERROR);
}
