/*
 *   $RCSfile: initMissingUpdatePools.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:03 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/

#include "log.h"
#ifndef INIT_LRC_IS_LSN

#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "pool.h"
#include "logrecs.h"
#include "bf.h"
#include "undo.h"
#include "undo_extfuncs.h"
#include "undo_globals.h"
#include "bf_globals.h"


 void
initMissingUpdatePools()

{
	
	MISSINGUPDATEINFO	*missingUpdateInfo;
	MISSINGUPDATE		*missingUpdate;
	int					tableCount = 4;			/* for testing 	*/
	int					tableEntryCount = 100;	/* for testing	*/
	int					i,j;
	
	TRPRINT(TR_RECOVER, TR_LEVEL_1, (""));

	/*
	 *	Fill a pool of missing update info tables.
	 *	1. initialize the pool 
	 *	2. create the pool elements.
	 *	3. For each element, initialize it and add it to the pool
	 */
	initializePool(&MissingUpdateInfoPool, NULL, tableCount);
	missingUpdateInfo = (MISSINGUPDATEINFO*) 
						malloc(tableCount * sizeof(MISSINGUPDATEINFO));
	if (missingUpdateInfo == NULL) {
		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}
	for (i = 0; i < tableCount; i++) {
		initializeListElement(&(missingUpdateInfo[i].list), 
							  (char*) &(missingUpdateInfo[i]) );
		INIT_MISSINGUPDATEINFO_MAGIC(&(missingUpdateInfo[i]));
		missingUpdateInfo[i].table = (LIST*) malloc(PageHashTableSize * sizeof(LIST));

		/*
		 *	Initialize each element of the table
		 */
		for (j = 0; j < PageHashTableSize; j++) {
			initializeList(&(missingUpdateInfo[i].table[j]));
		}

		listEnq(&(MissingUpdateInfoPool.freeList), &(missingUpdateInfo[i].list));
	}

	/*
	 *	Fill a pool of missing update info table entries.
	 *	1. initialize the pool 
	 *	2. create the pool elements.
	 *	3. For each element, initialize it and add it to the pool
	 */
	initializePool(&MissingUpdatePool, NULL, tableEntryCount);
	missingUpdate = (MISSINGUPDATE*) 
						malloc(tableEntryCount * sizeof(MISSINGUPDATE));
	if (missingUpdate == NULL) {
		SM_ERROR(TYPE_FATAL, esmMALLOCFAILED);
	}
	for (i = 0; i < tableEntryCount; i++) {
		initializeListElement(&(missingUpdate[i].list), (char*) &(missingUpdate[i]) );
		listEnq(&(MissingUpdatePool.freeList), &(missingUpdate[i].list));
	}

	/*
	 *	Initialize the is of tables which are in use
	 */
	initializeList(&(UsedMissingUpdateTables));

	return;
}
#endif INIT_LRC_IS_LSN
