/*
 *   $RCSfile: undoFileCreateDone.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:04 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "pool.h"
#include "tid.h"
#include "io.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "thread.h"
#include "latch.h"
#include "semaphore.h"
#include "link.h"
#include "lsn.h"
#include "bf.h"
#include "log.h"
#include "volume.h"
#include "logrecs.h"
#include "openlog.h"
#include "trans.h"
#include "bitmap.h"
#include "file.h"
#include "consist.h"
#include "logaction.h"
#include "undo.h"
#include "io_extfuncs.h"
#include "bf_extfuncs.h"
#include "bm_extfuncs.h"
#include "fi_extfuncs.h"
#include "log_extfuncs.h"
#include "undo_extfuncs.h"
#include "util_funcs.h"
#include "thread_globals.h"
#include "bf_globals.h"


 void
undoFileCreateDone (
	LOGRECORDHDR		*recordHeader 
)
{
	FID					*fid;


	TRPRINT(TR_IO, TR_LEVEL_1, ("lsn:%d", recordHeader->recordLSN.offset));

	/*
	 *	get a pointer to the file id to destroy
	 */
	fid = (FID *) GET_LOG_IMAGE(recordHeader, 0);
	TRPRINT(TR_IO, TR_LEVEL_2, ("fid:%d", fid->pid.page));

	/*
	 *	destroy the file
	 */
	if (fi_RecordFileForDestruction(fid) ) {
		SM_ERROR(TYPE_FATAL, esmINTERNAL);
	}

/*
 *	Don't log anything now, although it might be nice to 
 *	log something for debugging.
 */
#ifdef JUNK
	/*
	 *	initialize the log information
	 */
	recordInfo.type			= LOG_REC_TYPE_COMPENSATION;
	recordInfo.action		= LOG_ACTION_LOGICAL_FILE_DESTROY;
	recordInfo.imageCount	= 1;
	recordInfo.actionPid	= NULL;
    recordInfo.actionLRC    = NULL;
	recordInfo.imageSize[0] = sizeof(FID);
	recordInfo.imageData[0] = (VOID *) fid;
	recordInfo.flags		= NOFLAGS;
	recordInfo.nextUndoLSN	= who knows recordHeader->previousLSN;

	/*
	 *	write the record to the log
	 */
	if ((forceMark = writeLogRecord((TRANSREC *) Active->transRec, &recordInfo,
							  &lsn, NOFLAGS)) < 0)	{
		
		SM_ERROR(TYPE_FATAL, Active->errno);
	}
#endif

}
