/*
 *   $RCSfile: SM_Error.c,v $  
 *   $Revision: 1.1.1.1 $  
 *   $Date: 1996/05/04 21:56:03 $      
 */ 
/**********************************************************************
* EXODUS Database Toolkit Software
* Copyright (c) 1991 Computer Sciences Department, University of
*                    Wisconsin -- Madison
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE COMPUTER SCIENCES DEPARTMENT OF THE UNIVERSITY OF WISCONSIN --
* MADISON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION.  
* THE DEPARTMENT DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The EXODUS Project Group requests users of this software to return 
* any improvements or extensions that they make to:
*
*   EXODUS Project Group 
*     c/o David J. DeWitt and Michael J. Carey
*   Computer Sciences Department
*   University of Wisconsin -- Madison
*   Madison, WI 53706
*
*	 or exodus@cs.wisc.edu
*
* In addition, the EXODUS Project Group requests that users grant the 
* Computer Sciences Department rights to redistribute these changes.
**********************************************************************/


#include "sysdefs.h"
#include "ess.h"
#include "checking.h"
#include "trace.h"
#include "error.h"
#include "list.h"
#include "tid.h"
#include "pool.h"
#include "io.h"
#include "bitvec.h"
#include "lock.h"
#include "object.h"
#include "msgdefs.h"
#include "adminmsg.h"
#include "disk.h"
#include "thread.h"
#include "semaphore.h"
#include "latch.h"
#include "bf.h"
#include "link.h"
#include "volume.h"
#include "trace.h"
#include "msgvector.h"
#include "admin_funcs.h"
#include "thread_globals.h"
#include "disk_funcs.h"

/*
 * NB: PrintUser is used by the options processing
 * stuff to determine when to print error messages.
 * It is set to TRUE during io pre-initialization (handling
 * config options) so that errors are printed.  Then its
 * initial value is restored after the io module is done with the
 * server's config files. 
 * When input from the server's terminal interface invokes
 * addvolume() or removevolume() (io module), PrintUser is again
 * temporarily set to TRUE so that errors in those commands will
 * be printed (PrintUser's initial value is then restored).
 * When the client issues addvolume() and removevolume() requests,
 * the original value of PrintUser determines whether or not
 * error messages are printed.
 *
 * The gist of all this is that the value of PrintUser must
 * not be changed during pre-initialization (while ioIsInitializing
 * is true - a static BOOL in the io module), or while an
 * addvolume() or removevolume() is going on (neither one gives up
 * the cpu).
 */
BOOL	PrintUser = FALSE; /* SEE COMMENT ABOVE */

 void
SM_Error (

	int		type,
	int		eno,
	char	*file,
	int		line 
)
{
	BOOL printmessage = FALSE;

	if (type == TYPE_USER) {
		printmessage = PrintUser;
	} else {
		printmessage = (type != TYPE_QUIET);
	}
	if (printmessage) {
		fprintf(sm_ErrorStream, "SERVER file: %-22s line:%d errno: %4d error id:%s -- %s\n", file, line, eno, errorIdString(eno), errorCodeString(eno));
		fflush(sm_ErrorStream);
	}

	/*
	 *	record the error in the tcb
	 */
	if(Active) {
		Active->error = esmFAILURE;
		Active->errno = eno;
	}

	if (type == TYPE_CRASH)	{
		/* CRASH is only for unimplemented things.  We want an error
		 * message that's different from that for FATAL (and don't need
		 * a core dump) so that it's easy to tell when the problem is
		 * a known problem.
		 */
		fprintf(sm_ErrorStream, 
		"SERVER %d aborting at file: %-22s line:%d: UNIMPLEMENTED CASE\n", 
				getpid(), file, line);
		admin_ShutServer(SHUT_ABNORMAL_EXIT);
	}
	if (type == TYPE_FATAL || type == TYPE_CRASH)	{
		/* 
		 * bad news; we don't know about this bug!
		 * dump core 
		 */
		fprintf(sm_ErrorStream, "SERVER %d aborting; creating core file...\n",
			getpid());
		admin_ShutServer(SHUT_DUMP_CORE);
	}
	if ((type == TYPE_STOP) || (type == TYPE_QUIET)) {
		fprintf(sm_ErrorStream, "SERVER %d exiting.\n", getpid());
		admin_ShutServer(SHUT_ABNORMAL_EXIT);
	}
}
